/****************************************************************************
Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/


#ifndef DISPLAY_WIN_H
#define	DISPLAY_WIN_H
#include "list_rec.h"
#include "geometry.h"
#include "rect.h"
#include "intensity_rec.h"
#include "pixel_desc.h"
#include "slice_roi_rec.h"
#include "str_def.h"
#include <xview/xview.h>
#include <xview/font.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/scrollbar.h>
#include <xview/cursor.h>

/*
#include <pixrect/pixrect.h>
*/

#define SLICE_AUTO_ON 1
#define SLICE_SHIFT_ON 2
struct select_slice_win
   {
    struct display_win_rec *ancestor;
    Frame frame;
    Panel panel;
    unsigned long *auto_on;
    Panel_item auto_item,auto_offset_item,shift_on_item,which_order;
    Panel_item slice_num_slider,scan_num_slider;
   };

struct select_slice_control_rec
   {
    unsigned long cur_slice;
    char slice_offset_action_str[2];
    int slice_offset;
   };

struct trans_roi_rec
   {
#define TRANS_ACCEPT_NEXT 1
#define TRANS_PARAMS_INITIALIZED 2
#define TRANS_MOVE 4
#define TRANS_RESIZE 8
#define TRANS_ROTATE 16
#define TRANS_WAIT_FOR_NEXT 32
#define TRANS_PERFORMED 0x0020
#define TRANS_DRAG 0x0040
    unsigned short status;
    struct int_bound_rec roi_bound;
    int initx,inity,last_curx,last_cury,curx,cury;
    struct matrix_2d_rec matrix,mult_matrix;
   };

struct win_slice_roi_rec
   {
    struct matrix_2d_rec *tolocal,*localto;
    struct slice_roi_win_rec *sroi_win;
   };

/*
possible values for which_cursor
  0 = default
*/
#define CURSOR_DEFAULT 0
#define CURSOR_PEN 1
#define CURSOR_CUT 2
#define CURSOR_ROTATE 3
#define CURSOR_ROTATE_VERTICAL 4
#define CURSOR_ROTATE_HORIZONTAL 5
#define CURSOR_0 6
#define CURSOR_90 7
#define CURSOR_180 8
#define CURSOR_270 9
#define CURSOR_R90 10
#define CURSOR_R180 11
#define CURSOR_R270 12

#define CURSOR_X_REFLECTION 13
#define CURSOR_Y_REFLECTION 14
#define CURSOR_YX_REFLECTION 15
#define CURSOR_Y_X_REFLECTION 16
#define CURSOR_NO_REFLECTION 17
#define CURSOR_CLOCKWISE 18
#define CURSOR_ANTICLOCKWISE 19
#define CURSOR_ANGLE_CUT 20
#define CURSOR_DELETE_ROI 21
#define CURSOR_MOVE_ROI 22
#define CURSOR_RESIZE_ROI 23
#define CURSOR_ROTATE_ROI 24
#define CURSOR_MAGNIFY_PLUS 25
#define CURSOR_MAGNIFY_MINUS 26
#define CURSOR_PT_SCAN 27
#define CURSOR_LOCATE_PLANE 28
#define CURSOR_MOVE_SUPERIMPOSE 29
#define CURSOR_3D_PT1 30
#define CURSOR_3D_PT2 31
#define CURSOR_3D_PT3 32
#define CURSOR_SAME_ANGLE_CUT 33
#define TOTAL_CURSORS 34

/*
  possible values for the status
*/
#define DRAW_ENABLED 0x0001
#define EDGE_DETECTED 0x0002
#define DRAW_UPDATE 0x0004
#define DELETE_CURVE_MODE_BY_MOUSE 0x0008
#define ACTION_ON_ALL_SLICES 0x0010
#define ACTION_ON_ALL_ROIS 0x0020
#define ACTION_ON_CUR_ROI 0x0040

#define ACTION_ON_ROI 0x0060

#define ACTION_LABEL_ON 0x0070
#define _3D_DRAW_MODE 0x4000


/*
  possible values for the display options
*/
#define DISPLAY_HIDE_SHOW_ALL_ROIS 0x0001         
#define DISPLAY_FILLED_ROI 0x0010
#define DISPLAY_ENABLED_LABEL 0x0020
#define DISPLAY_ROW_COL_LABEL 0x0040
#define SET_ALLOWABLE_PEN_TYPE 0x0080
#define SET_MAX_INPUT_PTS 0x0100
#define DISPLAY_ACTION_LABEL 0x0200

#define DISPLAY_BREAKER 0x0400
#define DISPLAY_CLIPBOARD 0x0800
#define DISPLAY_CLIPBOARD_FIRST 0x1000
#define DISPLAY_HIDE_ALL_ROIS_BUT_CUR 0x2000

struct draw_setting_rec
   {
    unsigned short status;
    unsigned long which_cursor;
    unsigned long last_which_cursor;
    unsigned short pen_type;
    unsigned short allowable_pen_type;
    unsigned short display_options;
    unsigned short delete_roi;

#define EXPECT_FIRST_PT 0x0001
#define FIRST_PT 0x0002
#define EXPECT_SECOND_PT 0x0004
#define DELETE_PREVIOUS_PT 0x0008
#define EXPECT_LAST_PT 0x0010
#define PREVIOUS_PT_DELETED 0x0020
#define ANGLE_PT 0x0040
#define DONT_DELETE_PREVIOUS_PT 0x00f7
#define EXPECT_PT 0x0017

    unsigned char first_pt;
    int must_display;
    unsigned long num_display_cur_pts;
    unsigned long max_input_pts;

#define CTRL_COMMAND_KEY 1
#define SHIFT_COMMAND_KEY 2
#define META_COMMAND_KEY 3
    unsigned long last_command_key;
    unsigned long lastx,lasty;
    unsigned long last_cursorx,last_cursory;
    int (*proc_upon_input_done)();
    void *proc_arg;
    int (*proc_upon_cursor_change)();
    void *proc_upon_cursor_change_arg;

    struct win_slice_roi_rec sroi;
    struct trans_roi_rec trans;
   };

#define BACKGROUND_PIXEL_TYPE_STR "BACKGROUND"
#define MIDLINE "MIDLINE"

/* paint_status... */
#define PAINT_NO 0x0000
#define PAINT_SUPERIMPOSE 0x0001
#define PAINT_ERASE 0x0002
#define PAINTED 0x0004
#define PAINT_MUST_NOT_ERASE 0x0008
struct draw_mask_rec
   {
    struct mask_rec mask;
    unsigned short paint_status;
    unsigned long pixel_value;
   };

#define TOTAL_PROGRAM_COLORS 7
#define TOTAL_DRAW_MASKS 4
#define INIT_TOTAL_DRAW_MASKS 3
#define DRAW_MASK_CUR_CURVE 0
#define DRAW_MASK_CUR_ROI   1
#define DRAW_MASK_OTHER_ROIS 2
#define DRAW_MASK_PIXEL_SELECTION 3

/*
#define DRAW_MASK_NON_MEMBER_ROIS 3
*/
#define PIXEL_BUF_NON_EMPTY 0x0001
/*
status
*/
#define COLOR_DATA_DISABLED 1
#define COLOR_DATA_DONT_DELETE 2
#define COLOR_DATA_ID_VALID 4
#define COLOR_DATA_DEFAULT 8
#define COLOR_DATA_POS_ACTIVATION 0x00000010
#define COLOR_DATA_NEG_ACTIVATION 0x00000020
#define COLOR_NO_LOCAL_COLOR_INDXS 0x00000040

#define COLOR_DATA_POS_NEG_ACTIVATION 0x00000030
#define COLOR_DATA_DEFAULT_AND_ID_VALID 0x0000000c

/*
id value
*/
#define PIXEL_TYPE_ID 2
#define PIXEL_SELECTION_ID 3
#define SUPERIMPOSE_ID 3000
#define SUPERIMPOSE_3D_ID 3001
#define SUPERIMPOSE_3D_ID_ROIS 3002
struct color_data_rec
   {
    unsigned long status4,gid,member_id;
    struct buf_2d_rec _2d;
    struct color_scale_rec scale;
    unsigned long total_color_indxs,num_color_indxs;
    unsigned long *color_indxs;
   };

#define SCREEN_TRANSLATED 1
#define SCREEN_ROTATED 2
#define SCREEN_SCALED 4
#define SCREEN_TRANFORMED 7
struct screen_data_rec
   {
    unsigned short status;
    struct pixel_desc_rec pixel;
    struct int_region_rect def_dimension;
    struct int_region_rect dimension;
    struct pixel_desc_rec last_pixel;
    struct int_region_rect last_dimension;
    int def_mag_fac;
    struct buf_2d_rec _2d;
    XImage *im;
   };

struct shared_buf_2d_rec
   {int in_use;
    struct buf_2d_rec _2d;
   };

#define PIXEL_SELECTION_LB_UB_SET 0x0001
#define PIXEL_SELECTION_LB_UB_EXIST 0x0002

#define PIXEL_SELECTION_WHOLE_SLICE 0x0004
#define PIXEL_SELECTION_CUR_CURVE 0x0008
#define PIXEL_SELECTION_CUR_ROI 0x0010
#define PIXEL_SELECTION_ALL_ROIS 0x0020
#define PIXEL_SELECTION_GET_OPTION 0x003c

#define PIXEL_SELECTION_MASK_READY 0x0080
#define PIXEL_SELECTION_SETUP_OK 0x0100
#define PIXEL_SELECTION_PARAMS_VALID 0x0200
#define PIXEL_SELECTION_CLEAR_OPTIONS 0xffe3
struct pixel_selection_rec
   {
    unsigned short status;
    struct slice_roi_win_rec *selected_pixel_of_this_slice;
    struct buf_2d_rec selected_pixel;
    struct shared_buf_2d_rec *share;
    unsigned long lb,ub;
   };

#define TOTAL_COLOR_2DS 4
struct color_screen_rec
   {
    struct int_bound_rec bound;
    unsigned long total_colors,num_colors;
    struct color_data_rec **colors;
    struct pixel_selection_rec selection;
    char *def_pos_scale_label,*def_neg_scale_label,*scale_label;
    unsigned long cur_gid,cur_member_id;
    unsigned long cur_color_data,total_color_datas,num_color_datas;
    struct color_data_rec **color_datas;

    unsigned long num_draw_masks;
    struct draw_mask_rec draw_masks[TOTAL_DRAW_MASKS];
   };

struct frame_canvas_rec
   {
    Canvas canvas;
    Scrollbar hbar,vbar;
    unsigned long hbar_width,vbar_height;
    unsigned long height,width,pixel_depth;
   };

/*
 POSSIBLE VALUES FOR display_win_rec.status2
    A new window created from a master window of the same type can be either a
      slave or a master, or it can be only a master window when created
      from a slave window.
    When a master window dies, its slave window becomes a master window
      automatically.
    The MASTER_OR_SLAVE_MOUSE_XY works only between a pair of
         master-slave windows.
*/

#define MASTER_WINDOW 0x0100
#define INIT_WINDOW 0x1000
#define INIT_WINDOW_CHECK_ON 0x2000
#define MOVE_WINDOW 0x0001
#define WINDOW_WIN_RECT_VALID 0x0002

/*
  for display status...
*/
#define WIN_SHOW_SCREEN_BOUND 0x0001
#define WIN_NOT_YET_PAINTED 0x1000
#define WIN_ROI_LABEL_PAINTED 0x2000
#define WIN_ROW_COL_PAINTED 0x4000
#define WIN_RIGHT_ANGLE_LABEL_PAINTED 0x8000
#define WIN_ACTION_LABEL_PAINTED 0x0100

#define VALUE_SHOW_ROI_PIXEL_TYPE 0x0200
#define VALUE_SHOW_ALL_ROIS_PIXEL_TYPE 0x0400
#define VALUE_SHOW_SLICE_PIXEL_TYPE 0x0800
#define DISPLAY_PIXEL_TYPE 0x0e00

struct display_rec
   {
    Xv_Server server;
    Display *dp;
    int screen_num;
   };

/* analysis_status */
/*
#define PIXEL_TYPE_2_ECHO_ANALYSIS 0x0001
#define PIXEL_TYPE_1_ECHO_ANALYSIS 0x0002
*/
#define PIXEL_TYPE_PARTITION_ANALYSIS 0x0001
#define PIXEL_TYPE_GROW_REGION_ANALYSIS 0x0002
#define PIXEL_COUNT_VALUE_ANALYSIS 0x0004
#define ALL_PIXEL_ANALYSIS 0x0007
struct display_win_rec
   {
    struct image_display_window *owner;
    struct display_rec dp;
    unsigned short status2,display_status,analysis_status,resize_status;
    Frame ancestor;
    Frame frame;
    char *frame_label;
    Rect win_rect;
    int last_frame_width,last_frame_height;
    Panel panel,horiz_scroll,verti_scroll;
    Panel_item anal_button,horiz_scroll_item,verti_scroll_item;
    Menu menu;
    Xv_Font font;
    Xv_Cursor default_cursor;

    struct int_rect_rec right_angle_label_area; 
    struct int_rect_rec action_label_area; 
    struct int_rect_rec row_col_area; 
    struct int_rect_rec label_area; 
    struct frame_canvas_rec frame_canvas;
    struct select_slice_win select_slice;
    struct screen_data_rec screen;
    struct display_win_rec *master,*slave;
    struct int_bound_rec show_screen_bound;
  };

/*
    struct image_display_window *first_partner,*last_partner,*next_partner;
 is the link list used to connect all the image_display_window
 whose data need to be updated
 together.  For example, drawing on the PROTON window also causes the
 T2 window and all other sub-windows to be updated;  "next slice" applies
 to all the windows in the linked list.   But, a window which may display
 the same slice not on that list is not updated.
*/

/*
 possible value for status4
*/
#define VIEW_ROI_FILTER_ON 0x00000001
#define HIDE_ALL_ROIS 0x00000002
#define WIN_ROI_MARK_DELETE 0x00000004
#define DISPLAY_SLICE_TISSUE_TYPE 0x00000008
#define DISPLAY_CUR_CURVE_TISSUE_TYPE 0x00000010
#define DISPLAY_CUR_ROI_TISSUE_TYPE 0x00000020
#define DISPLAY_ALL_ROIS_TISSUE_TYPE 0x00000040
#define UNSELECTED_TISSUE_UNAFFECTED 0x00000080
#define DONT_APPLY_SAME_TISSUE_TO_STACK_WINS 0x00000100
#define DISPLAY_VIEW_FILTER_ROI_TISSUE_TYPE 0x00000200
#define RECOMPUTE_PIXEL_TYPES 0x00000400
#define RECOMPUTE_OVERLAY 0x00000800

#define STACK_WIN_COSECUTIVE 0x00001000
#define STACK_WIN_SCAN 0x00002000
#define WIN_COLOR_DATA_UNCHANGED 0x00004000
#define HIDE_ALL_ROIS_BUT_CUR 0x00008000
#define WIN_POS_ACTIVATION 0x00010000
#define WIN_NEG_ACTIVATION 0x00020000

#define WIN_POS_NEG_ACTIVATION 0x00030000
#define RECOMPUTE_BEFORE_NEXT_SLICE 0x00000c00
#define DISPLAY_TISSUE_TYPE 0x00000278
#define FILTER_ON (DISPLAY_TISSUE_TYPE | VIEW_ROI_FILTER_ON)


#define STACK_WIN_NEXT_SLICE_METHOD 0x00000600
#define WIN_HIDE_ROIS 0x00008002


/*
possible values for win->type2
*/
#define DISPLAY_WINDOW 0x0001
#define HISTOGRAM_1D_WINDOW 0x0002
#define HISTOGRAM_MULTI_D_WINDOW 0x0004
#define HISTOGRAM_1D_WINDOW_DISPLAY_ONLY 0x0008
#define DISPLAY_WINDOW_3D 0x0010
#define HISTO_1D_WINDOW 0x0020
#define HISTO_2D_WINDOW 0x0040

#define EDITING_WINDOW 0x0077
#define EDIT_NO_3D_WINDOW 0x0067
#define HISTOGRAM_WINDOW 0x006e
#define HISTOGRAM_1D_WINDOWS 0x0022
#define HISTOGRAM_2D_WINDOWS 0x0044
#define ALL_WINDOWS 0x00ff
#define IMAGE_WINDOWS 0x0011
#define MANY_SLICES_WINDOWS 0x0071

/*
possible value for process_status
*/

#define WIN_PROCESSED 0x00000001
#define WIN_ADVANCE_SLICE_PROCESSED 0x00000002
#define WIN_COLOR_DATA_PROCESSED 0x00000004
#define WIN_SCREEN_DATA_PROCESSED 0x00000008
#define WIN_CANVAS_DATA_PROCESSED 0x00000010
#define WIN_OVERLAY_DATA_PROCESSED 0x00000020
#define WIN_SCREEN_SETUP_PROCESSED 0x00000040

#define WIN_PROCESSED_ERROR 0x00010000
#define WIN_ADVANCE_SLICE_ERROR 0x00020000
#define WIN_COLOR_DATA_ERROR 0x00040000
#define WIN_SCREEN_DATA_ERROR 0x00080000
#define WIN_CANVAS_DATA_ERROR 0x00100000
#define WIN_OVERLAY_DATA_ERROR 0x00200000
#define WIN_SCREEN_SETUP_ERROR 0x00400000

#define WIN_STARTUP_3D_APPROX 0x10000000

#define WIN_ALL_PROCESSED_STATUS 0x007f007f
#define WIN_PROCESSED_STATUS 0x0000007f
#define WIN_PROCESSED_ERROR_STATUS 0x007f0000
struct image_display_window
   {
    unsigned long status4,process_status4;
    unsigned short type2;
    struct win_rec *owner;
#define WIN_PARTNER 1
#define WIN_LINK 2
#define WIN_LIST 4
#define WIN_PROC_AFTER_DP 8
#define WIN_OVERLAY 16
#define WIN_DO_NOT_PROCESS_CUR_WIN 32
#define WIN_DRAW_LIST 64
    struct list_rec partner,link,list,proc_after_dp,overlay,draw_list;
    Frame ancestor;
    unsigned long num_display_wins;
    struct display_win_rec **dwins;
    struct display_win_rec *dwin;  /* this points to a "display_win_rec" in
                                       dwins;  it is first initialize to NULL
                                       and any subsequent
                                       access to it involves letting it
                                       point to some "display_win_rec" */
    struct intensity_rec *intensity;
    struct draw_setting_rec control;
    struct roi_rec image_view_roi;
    struct buf_2d_rec image_view_2d;
    struct strs_rec data_filter_types; /* filter pixel types
                                          used when DISPLAY_TISSUE_TYPE */
    struct color_screen_rec data;
   };

struct template_rec
   {void *ptrs[5];
   };

struct show_menu_rec
   {
    struct display_win_rec *win;
    Xv_Window window;
    Menu menu;
    Event event;
    int num,arg1;
   };

#define OVERLAY_MUST_COMPUTE 1
#define OVERLAY_IMG_CAN_BE_DELETED 2
struct screen_data_overlay_rec
   {
    unsigned short status;
    struct input_file_rec *file;
    struct img_buf_rec *img;
    struct hdr_scan_entry_rec *host_slice,*guest_slice;
    struct buf_2d_rec mask;
    struct color_data_rec color_data;
   };
#endif DISPLAY_WIN_H

