/****************************************************************************
Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

#ifndef MARK_HDR_H
#define	MARK_HDR_H

struct mark_hdr_key
   {int size;
    char data_type[10];
    char name[18];
    int extents;
    short int session_error;
    char regular;
    char hkey_un0;
   };

#define _1_ECHO_IMAGE 0x0001
#define _2_ECHO_IMAGE 0x0002
#define _2_ECHO_INDIVIDUAL_MIN_MAX 0x1000
struct mark_hdr_dimension
   {
    unsigned short int num_dims;
    unsigned short int dimx,dimy;
    unsigned short max_slices;
    unsigned short int num_vols;
    unsigned short int histo_dimx,histo_dimy;

    unsigned short status;
    unsigned short unused_ints[8];

    short int pixel_depth_bits;
    short int unused;

    float unused_f;
    float xpixel_size,ypixel_size;
    float pixel_thickness;
    float spacing;
    short plane_type;  /* change */
          /* -1 = unknown,0 = axial, 1 = sagittal, 2 = coronal, 3 = oblique */
    short unused2;   /* old = pulse_sequence */

    unsigned long var_use_status;     /* old = orientation &  position */
#define MARK_USED2_USED 0x00000004
#define MARK_USED3_USED 0x00000008
#define MARK_USED4_USED 0x00000010
#define MARK_USED5_USED 0x00000020
#define MARK_USED6_USED 0x00000040
#define MARK_USED7_USED 0x00000080
#define MARK_UNUSED_FLOATS 0x00000100

    short num_echoes;
    short unused3; /* old = flip_angle */
    short unused4; /* old = scan_dimx */
    short unused5; /* old = scan_dimy */

    float unused6;    /* old = field_of_view */
    float unused7;    /* old = num_excitations */
    float tr;    /* repetition/Recovery time */
    char te[12];    /* echo delay */
    float unused_floats[1];

    int glmax,glmin;
   };

struct mark_patient_rec
    {char name[63];
     char ssnum[12];
     char patient_id[12];  /* medical record# */
     char study_num[8];    /* study # */
     char patient_num[8];  /* patient # */
     char sex;
     int age;
     int height; /* in mm */
     int weight; /* in grams */
    };

struct mark_hdr_history 
   {struct mark_patient_rec patient;
    char date_of_study[12];
    char time_of_study[12];
    char comment[60];
   };

struct mark_hdr
   {struct mark_hdr_key key;
    struct mark_hdr_dimension dimen;
    struct mark_hdr_history history;
   };

struct segmentation
   {	int SEG_STATUS; /* bit 0 for CSF, bit 1 = BRAIN, and bit 2 = LESION */
                                    /* 1d index to image array  STAT*/
	int r1c, r2c, r3c,r12c,r13c,r23c,r123c; /* coordinates */
	float r1a,r2a,r3a,r12a,r13a,r23a; /* slope for gradient  */
	int th1,th2;	/* coordinates for PD & T2 respectively */
	struct cluster_stats 
	   {	float x,y,slope,intercept; } RR[3];
   } ;

struct volume_data
   {	char	VERSION[8],	/* "v3.0" */	
		filename[16],
		special_text[48];
	struct  segmentation SEG;
	float	pixsize,spacing,voxel_size;
	int	num_slices; 
	int	tot_csf,tot_rcsf,tot_lcsf,
		tot_brain,tot_rbrain,tot_lbrain,
		tot_vent,tot_rvent,tot_lvent,
		tot_abn,tot_rabn,tot_labn,
		tot_v3;
	int	dummy[16];
   };

struct slice_data
  {	unsigned char	/* 3V=0x16 B=0x08 V=0x04 F=0x02 M=0x01 */
		B_bound_saved,
		V_bound_saved,
		F_bound_saved,
		M_bound_saved,
		V3_saved,
		boundary_saved_code,
		fill_byte_7,
		fill_byte_8;
	int	num_bp,num_vp,num_fp,num_mp;
	int	r_bseed,l_bseed,
		r_vseed,l_vseed,
		r_fseed,l_fseed;  /* seed points for fill */
	int	csf,rcsf,lcsf,
		brain,rbrain,lbrain,
		vent,rvent,lvent,
		abn,rabn,labn,
		v3;
	int	RECALC;	/* new data -> recalc this slice */
	int	dummy[10];	/* for future use */
   };

  /* make sure MRI_MAX_BOUND_PTS = MRI_LIST_ELEMS = 6000 = MRI_MAX_DRAWN_PTS*/
struct slice_bound
   {unsigned short *bp_list,	/*List of B_BOUNDARY points */
		   *vp_list,	/*List of V_BOUNDARY points */
		   *fp_list,	/*List of F_BOUNDARY(pofo)*/
		   *mp_list;	/*List of M_BOUNDARY(midline)*/
   };


#endif MARK_HDR_H
