/****************************************************************************
Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/


#ifndef MRI_FORMAT_EXTERN_H
#define	MRI_FORMAT_EXTERN_H

#include <stdio.h>
#include <math.h>
#include "typedef.h"
#include "common.h"
#include "error_constants.h"
#include "special_procs.h"
#include "common_constants.h"
#include "command_keys.h"
#include "list_rec.h"
#include "menu.h"

#include "rect.h"
#include "str_def.h"
#include "slice_roi_rec.h"
#include "text_format.h"
#include "count_analysis_rec.h"
#include "output.h"
#include "histo.h"

#include "list_rec.h"

#include "ia_hdr.h"
#include "pixel_desc.h"
#include "coord_rec.h"
#include "input.h"

#include "ria_extern_constants.h"
#include "ria_extern_define_constants.h"
#define EXTERN_CONSTANTS
/*
#include "ria_define_constants.h"

*/
#include "display_win.h"
#include "win.h"
#include "analysis.h"


struct input_rec
   {
    struct prog_rec *prog;
    unsigned int total_files,num_files;
    struct input_file_rec **files;
   };

#define PROG_ID_V1 "P1.0"
#define HDR_ID_V1 "H1.0"
#define DAT_ID_V1 "D1.0"
#define DAT_ID_V2 "D2.0"
#define DAT_ID_V3 "D3.0"
#define PIXMAP_V1 "PIXMAP1.0"
struct defaults_fn_rec
   {
    char *color_fn,*roi_labels_fn,*pixel_types_fn;
    char *txt_format_fn,*batch_prog,*batch_status_fn;
    char *hdr_dir,*hdr_fn,*dat_dir,*dat_fn,*rater,*startup_pixel_mag_factor;
    char *bin_ratio_row_col,*sec_roi_labels_fn,*sec_pixel_types_fn;
    char *pixel_type_priority_fn,*auto_save_text,*txt_format_fn_1_echo;
    char *txt_format_fn_count_value,*cv_roi_labels_fn,*cv_sec_roi_labels_fn;
   };

#define TOTAL_DEFAULT_FNS 30
#define NUM_DEFAULT_FNS 21
#define DEFAULT_COLOR_FN "color.dat"
#define DEFAULT_ROI_LABELS_FN "pixel_type_roi_labels.main"
#define DEFAULT_PIXEL_TYPES_FN "pixel_types.main"
#define DEFAULT_SEC_ROI_LABELS_FN "pixel_type_roi_labels.sub"
#define DEFAULT_SEC_PIXEL_TYPES_FN "pixel_types.sub"
#define DEFAULT_TXT_FORMAT_FN "text_format.fmt"
#define DEFAULT_BATCH_PROG_FN "batch_main"
#define DEFAULT_BATCH_STATUS_FN "batch.status"
#define DEFAULT_HDR_FN "*.nhdr"
#define DEFAULT_DAT_FN "*.ndat"
#define DEFAULT_BIN_RATIO_ROW_COL ".5/3/3"
#define DEFAULT_PIXEL_TYPE_PRIORITY_FN "pixel_types.order"
#define DEFAULT_AUTO_SAVE_TEXT "0"
#define DEFAULT_TXT_FORMAT_FN_1_ECHO "text_format_1_echo.fmt"
#define DEFAULT_TXT_FORMAT_FN_COUNT_VALUE "text_format_count_value.fmt"
#define DEFAULT_CV_ROI_LABELS_FN "count_value_roi_labels.main"
#define DEFAULT_CV_SEC_ROI_LABELS_FN "count_value_roi_labels.sub"

#define VALUE_DEFAULT_HDR_DIR 6
#define VALUE_DEFAULT_DAT_DIR 8
#define VALUE_DEFAULT_RATER 10
#define VALUE_MAG_FACTOR 11

/*
# identifier       function            status              default value
#      0          color fn          read-write file          color.dat
#      1        roi labels fn       read-write file          pixel_type_roi_labels.main
#          for the pixel type analysis
#      2        pixel types fn      read-write file          pixel_types.main
#      3        text output
#                  format fn        read-write file          text_format.fmt
#      4        batch program       execute-only file        batch_main
#      5        batch log file      write-only file          batch.status
#      6        .hdr directory      directory specification  user directory
#                                                            from where the
#                                                            program is run
#      7        .hdr fns            fn specification         *.hdr
#      8        .dat directory      directory specification  user directory
#                                                            from where the
#                                                            program is run
#      9        .dat fns            fn specification         *.dat
#     10        rater's name        text                     login user
#     11        default image       1 <= factor <= 20        1
#               magnifying factor
#     12        bin image_ratio/num_rows/num_cols
#                            .25 <= ratio <= 1.0            .5/3/3
#     13        secondary roi labels fn       
#                                   read-write file          pixel_type_roi_labels.sub
#     14        secondary pixel types fn      
#                                   read-write file          pixel_types.sub
#     15        pixel types priority fn      
#                                   read-write file          pixel_types.order
#     16        save txt automatically      
#                                   0 or non-zero            0
#     17        text output format fn
#                 for 1 echo        read-write file          text_format_1_echo.fmt
#     18        text output format fn for count
#               value analysis     read-write file          text_format_1_count_value.fmt

#     19        roi labels fn       read-write file          count_value_roi_labels.main
#          for the count value analysis
#     20        secondary roi labels fn       
#          for the count value analysis
#                                   read-write file          count_value_roi_labels.sub

*/
union defaults_rec
   {
    char *fns[TOTAL_DEFAULT_FNS];
    struct defaults_fn_rec whch;
   };

struct prog_rec
   {
    char *prog_id,*hdr_id,*dat_id;
    char *prog_name;
    char *defaults_fn;
    union defaults_rec defaults;
    struct slice_roi_rec cut_slice_roi,delete_slice_roi;
    struct input_rec input;
    struct all_win_rec win;
   };

#endif MRI_FORMAT_EXTERN_H
