/****************************************************************************
Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

/*
   On the output, the old content of the output file will be entirely
   overwritten by the corresponding fields from the input file.
*/

#ifndef OUTPUT_H
#define	OUTPUT_H

struct slice_num_and_roi_rec
   {long slice_num;
    struct slice_roi_rec *slice_roi;
   };

#define OUTPUT_HDR_INFO_REC_SETUP 1
struct output_hdr_info_rec
   {
    unsigned short status;
    char *dirfn,*fn;         /* directory fn & fn  for the .hdr & .img files */
    char *hdr_fn_suffix,*img_fn_suffix;
    unsigned short int dimx;
    unsigned short int dimy;
    unsigned short max_slices;
    unsigned short int pixel_depth_bits;
   };

struct output_field_offset_rec
   {
    long info_start,info_size;
    long slice_roi_start,slice_roi_size;
    long histo_params_start,histo_params_size;
    long pixel_type_params_start,pixel_type_params_size;
    long text_format_start,text_format_size;
    long count_start,count_size;
   };

#define NDAT_READ_SLICE_ROIS 0x00000001
#define NDAT_READ_HISTO_PARAMS 0x00000002
#define NDAT_READ_PIXEL_TYPES 0x00000004
#define NDAT_READ_PIXEL_TYPE_TEXT_FORMAT 0x00000008
#define NDAT_READ_ALL_FIELDS 0x000F

struct output_hdr_rec
   {
    long size;
    char *id,*rater;
    struct output_field_offset_rec offset;
    struct output_hdr_info_rec info;

/* the following array of slice_nums[0..num_slice_rois - 1] contains the
      slice #s of the corresponding slice_rois of "struct hdr_scan_entry_rec"
      in input.h that are saved and whose num_rois > 0.
      Therefore 0 <= num_slice_rois <= total image slices excluding echoes.
      if (slice_roi_rec is empty == 0) the corresponding slice_nums will not
      be saved.
*/
    unsigned long actual_total_slice_rois,num_slice_rois;
    unsigned long *slice_nums;
   };

/* used in wr_dat_opr.c */
#define FILE_TO_BE_SAVED 0x0001
#define FILE_CHANGED 0x0002
#define FILE_TO_BE_SETUP 0x0004
#define FILE_READY 0x0008
#define FILE_READ 0x0010
#define FILE_AUTO_SAVE_TEXT 0x0020
#define FILE_ASK_BEFORE_SAVE 0x0040

 /* 
 possible values for status
    FILE_TO_BE_SETUP --- need to setup the output_file_rec.
    FILE_TO_BE_SAVED  --- user wants to save file.
    FILE_CHANGED --- the file has been changed.
    FILE_AUTO_SAVE_TEXT --- save the text output automatically.
*/
struct output_file_rec
   {
    struct input_file_rec *file;
    unsigned short status;
    char *dirfn,*fn;         /* directory fn & fn */
    char *dat_fn_suffix,*bin_fn_suffix;
    struct output_hdr_rec hdr;
   };


#endif OUTPUT_H
 
