/****************************************************************************
Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/


#ifndef TEXT_FORMAT_H
#define	TEXT_FORMAT_H

#include "str_def.h"
#include "common.h"

#define PIXEL_TYPE_CNTS_COMPUTED 0x0001
#define PIXEL_TYPE_CNTS_CORRECTED 0x0002
struct pixel_type_cnt_rec
   {
    unsigned short status;
    unsigned long num_pixel_type_cnts;
    unsigned long *pixel_type_cnts;
   };

struct collection_rec
   {
    unsigned char max_digit_chs;
    unsigned long num_fields;
    unsigned long num_slices;
    unsigned long total_buf_size,buf_size,cur_indx;
    unsigned char *buf;
   };

struct page_rec
   {
    unsigned long num_collects;
    struct collection_rec **collects;
   };

#define FIRST_COL_SPACES 7
struct all_collection_rec
   {
    unsigned long num_line_label_spaces,total_slices;

    unsigned long cur_collect,num_collects;
    struct collection_rec *collects;
    char *text_temp_fn;
    unsigned long num_pages;
    struct page_rec *pages;
   };

struct text_output_analysis_group_rec
   {
    unsigned long num_rois;
    char **roi_labels;
    unsigned long total_rois;
    unsigned long *roi_nums; /* [i] = 0 = roi not defined
                                    1 <= [i] <= num_rois exist */
    unsigned long num_pixel_types;
    char **pixel_type_labels;
    unsigned long total_pixel_types;
    unsigned long *pixel_type_nums; /* [i] = 0 = pixel type not exist
                                    1 <= [i] <= num_pixel_types exist */
   };

struct output_field_rec
   {unsigned long pixel_type_indx,num_roi_indxs;
    unsigned long *roi_indxs;
   };

#define COUNT_ROI_DEFINED 1
#define COUNT_ROI_NOT_COMPUTED 2
#define COUNT_PIXEL_TYPE_NOT_DEFINED 4
struct output_count_field_value_rec
   {unsigned char valid;
    unsigned long value;
   };

struct output_count_field_rec
   {unsigned long num_fields;
    struct output_count_field_value_rec *fields;
   };

struct output_count_group_rec
   {
    unsigned long num_groups;
    struct output_count_field_rec *groups;
   };

#define OUT_FORMAT_DEFINED 1
#define OUT_FORMAT_CHECK_FORMAT 2
#define OUT_FORMAT_COUNT_DEFINED 4
#define OUT_FORMAT_COUNT_NEG 8
#define OUT_FORMAT_LT_PERCENT_MEAN 0x0010
#define OUT_FORMAT_COUNT_ALL_ECHOES 0x0020
#define OUT_FORMAT_POS_NEG_ECHOES 0x0040
#define OUT_FORMAT_COUNT_CUR_ECHO_ONLY 0x0080
#define OUT_FORMAT_LT_PERCENT_ROI_MEAN 0x0100
#define OUT_FORMAT_GT_PERCENT_MEAN 0x0200
#define OUT_FORMAT_GT_PERCENT_ROI_MEAN 0x0400
#define OUT_FORMAT_POS_NEG_ACTIVATION_ECHOES 0x0800
#define OUT_FORMAT_NEG_VALUES 0x1000
#define OUT_FORMAT_UNIT_2D_VALID 0x2000
#define OUT_FORMAT_USE_DEFAULT_IMAGE_VIEW 0x4000

#define OUT_FORMAT_COUNT_OPTIONS 0x0ff8
#define OUT_FORMAT_NEG_OPTION 0x1008
#define OUT_FORMAT_COUNT_ONCE 0x08a0
#define OUT_FORMAT_COUNT_ECHO_METHOD 0x08e0
#define OUT_FORMAT_COUNT_ECHO_AND_REJECTION_METHOD 0x0ff0
#define OUT_FORMAT_COUNT_LT_GT_PERCENT_OPTIONS 0x0710
#define OUT_FORMAT_NEG_AND_LT_GT_PERCENT_OPTIONS 0x1718
#define OUT_FORMAT_COUNT_ROI_LT_GT_PERCENT_OPTIONS 0x0500
#define OUT_FORMAT_COUNT_SLICE_LT_GT_PERCENT_OPTIONS 0x0210
struct output_text_format_rec
   {
    unsigned short status;
    struct strs_rec roi_labels;
    unsigned long *roi_nums;
    struct strs_rec pixel_type_labels;
    unsigned long *pixel_type_nums;

    unsigned long num_groups;
    struct output_field_rec *groups;
    unsigned long num_slices;
    struct output_count_group_rec *counts;

    unsigned long page_num_rows,page_row_size;
    unsigned long num_field_spaces,num_collect_spaces;

    float xpixel_size,ypixel_size,pixel_thickness,voxel_size;
    float lt_percent,gt_percent,roi_lt_percent,roi_gt_percent;
    struct unit_2d_rec img_unit;
   };

struct pixel_type_order_rec
   {
    unsigned long order;
    char *pixel_type_label;
   };
#define NUM_FILED_SPACES 1
#define NUM_COLLECT_SPACES 3
#define PAGE_ROW_SIZE 80
#define PAGE_NUM_ROWS 60

/*
 assuming that every file has a "struct text_output_rec".
 possible values for status
*/
#define TEXT_OUTPUT_PARAMS_SETUP 0x0001
#define TEXT_OUTPUT_FORMAT_SETUP 0x0002
#define TEXT_OUTPUT_IN_PROGRESS 0x0004
#define TEXT_OUTPUT_COMPUTED 0x0008
#define TEXT_OUTPUT_DELETED 0xfff3

#define TEXT_OUTPUT_TO_FILE 0x0010
#define TEXT_OUTPUT_TO_TEMP_FILE 0x0020
#define TEXT_OUTPUT_TO_WINDOW 0x0040
#define TEXT_OUTPUT_SAVE_FORMAT 0x0080

#define TEXT_OUTPUT_PIXEL_TYPE_ANALYSIS 0x0100
#define TEXT_OUTPUT_PIXEL_COUNT_ANALYSIS 0x0200
#define TEXT_OUTPUT_PIXEL_COUNT_ALL_ANALYSIS 0x0400

#define FLUSH_TEXT_OUTPUT_STATUS 0xff23
struct text_output_rec
   {
    unsigned short status;
    struct input_file_rec *file;
    char *text_printing_time;

    unsigned long num_groups;
    struct text_output_analysis_group_rec *groups;

    struct output_text_format_rec format;
    struct all_collection_rec analysis;
   };


#endif TEXT_FORMAT_H
