/****************************************************************************
2dtran.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: 2dtran.c,v 1.6 1999/07/07 18:01:05 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <math.h>
#include <stdarg.h>
#include <stdio.h>

#include <sys/types.h>

#include "geometry.h"
#include "error_constants.h"
#include "typedef.h"
#include "nria_lib.h"

/*
#define TRACE(x) x; fflush(stdout);
*/
#define TRACE(x)



/*-------------------------------------------------------------------------
identity_2d_matrix - ???
-------------------------------------------------------------------------*/

void 
identity_2d_matrix(matrix)
 struct matrix_2d_rec *matrix;
{
 int row,col;

 for (row = 0; row < 3; row++)
    for (col = 0; col < 3; col++)
       if (row == col) matrix->buf[row][col] = 1.0;
         else matrix->buf[row][col] = .0;
 matrix->status = MATRIX_INITIALIZED;
}


/*-------------------------------------------------------------------------
setup_2d_matrix - ???
-------------------------------------------------------------------------*/

void
setup_2d_matrix(matrix,e00,e01,e02,e10,e11,e12,e20,e21,e22)
 struct matrix_2d_rec *matrix;
 float e00,e01,e02,e10,e11,e12,e20,e21,e22;
{
 matrix->buf[0][0] = e00;
 matrix->buf[0][1] = e01;
 matrix->buf[0][2] = e02;
 matrix->buf[1][0] = e10;
 matrix->buf[1][1] = e11;
 matrix->buf[1][2] = e12;
 matrix->buf[2][0] = e20;
 matrix->buf[2][1] = e21;
 matrix->buf[2][2] = e22;

 matrix->status = MATRIX_INITIALIZED;
}


/*-------------------------------------------------------------------------
mult_2d_matrix - ???
-------------------------------------------------------------------------*/

void
mult_2d_matrix(dest,src1,src2)
 struct matrix_2d_rec *dest,*src1,*src2;
{
 unsigned long row,col;
 float temp[3][3];

 for (row = 0; row < 3; row++)
    for (col = 0; col < 3; col++)
       temp[row][col] = src1->buf[row][0] * src2->buf[0][col] +
                        src1->buf[row][1] * src2->buf[1][col] +
                        src1->buf[row][2] * src2->buf[2][col];
 for (row = 0; row < 3; row++)
    for (col = 0; col < 3; col++)
       dest->buf[row][col] = temp[row][col];
 dest->status |= src1->status | src2->status;
}


/*-------------------------------------------------------------------------
check_and_mult_2d_matrix - ???
-------------------------------------------------------------------------*/

void
check_and_mult_2d_matrix(dest,src1,src2)
 struct matrix_2d_rec *dest,*src1,*src2;
{
 if (src2 == NULL) 
    {if (src1 == NULL) identity_2d_matrix(dest);
       else *dest = *src1;}
   else if (src1 == NULL)
    *dest = *src2;
   else mult_2d_matrix(dest,src1,src2);
}


/*-------------------------------------------------------------------------
assign_2d_translation - ???
-------------------------------------------------------------------------*/

void
assign_2d_translation(matrix,tx,ty)
 struct matrix_2d_rec *matrix;
 float tx,ty;
{
 matrix->buf[0][2] = tx;
 matrix->buf[1][2] = ty;
 matrix->status = MATRIX_TRANSLATED;
}



/*-------------------------------------------------------------------------
setup_translate_2d_matrix - ???
-------------------------------------------------------------------------*/

void
setup_translate_2d_matrix(matrix,tx,ty)
 struct matrix_2d_rec *matrix;
 float tx,ty;
{
 identity_2d_matrix(matrix);
 assign_2d_translation(matrix,tx,ty);
}


/*-------------------------------------------------------------------------
translate_2d_matrix - ???
-------------------------------------------------------------------------*/

void
translate_2d_matrix(matrix,tx,ty)
 struct matrix_2d_rec *matrix;
 float tx,ty;
{
 struct matrix_2d_rec temp;

 setup_translate_2d_matrix(&temp,tx,ty);
 mult_2d_matrix(matrix,&temp,matrix);
}


/*-------------------------------------------------------------------------
drop_2d_matrix_translation - ???
-------------------------------------------------------------------------*/

void
drop_2d_matrix_translation(matrix)
 struct matrix_2d_rec *matrix;
{
 matrix->buf[0][2] = .0;
 matrix->buf[1][2] = .0;
}


/*-------------------------------------------------------------------------
setup_scale_2d_matrix - ???
-------------------------------------------------------------------------*/

void
setup_scale_2d_matrix(matrix,sx,sy)
 struct matrix_2d_rec *matrix;
 float sx,sy;
{
 identity_2d_matrix(matrix);
 matrix->buf[0][0] = sx;
 matrix->buf[1][1] = sy;
}


/*-------------------------------------------------------------------------
scale_2d_matrix - ???
-------------------------------------------------------------------------*/

void
scale_2d_matrix(matrix,sx,sy)
 struct matrix_2d_rec *matrix;
 float sx,sy;
{
 struct matrix_2d_rec temp;

 setup_scale_2d_matrix(&temp,sx,sy);
 mult_2d_matrix(matrix,&temp,matrix);
}


/*-------------------------------------------------------------------------
setup_scale_2d_matrix_from_xy - ???
-------------------------------------------------------------------------*/

void
setup_scale_2d_matrix_from_xy(matrix,sx,sy,fx,fy)
 struct matrix_2d_rec *matrix;
 float sx,sy,fx,fy;
{
 struct matrix_2d_rec temp;

 identity_2d_matrix(matrix);
 setup_translate_2d_matrix(matrix,-fx,-fy);
 setup_scale_2d_matrix(&temp,sx,sy);
 mult_2d_matrix(matrix,&temp,matrix);
 translate_2d_matrix(matrix,fx,fy);
 matrix->status |= MATRIX_SCALED | MATRIX_TRANSLATED;
}


/*-------------------------------------------------------------------------
scale_2d_matrix_from_xy - ???
-------------------------------------------------------------------------*/

void
scale_2d_matrix_from_xy(matrix,sx,sy,fx,fy)
 struct matrix_2d_rec *matrix;
 float sx,sy,fx,fy;
{
 struct matrix_2d_rec temp;

 scale_2d_matrix_from_xy(&temp,sx,sy,fx,fy);
 mult_2d_matrix(matrix,&temp,matrix);
}


/*-------------------------------------------------------------------------
setup_rotate_2d_matrix - ???
-------------------------------------------------------------------------*/

void
setup_rotate_2d_matrix(matrix,radian_angle)
 struct matrix_2d_rec *matrix;
 float radian_angle;
{
 identity_2d_matrix(matrix);
 matrix->buf[0][0] = cos(radian_angle);
 matrix->buf[0][1] = sin(radian_angle);
 matrix->buf[1][0] = -matrix->buf[0][1];
 matrix->buf[1][1] = matrix->buf[0][0];
 matrix->status |= MATRIX_ROTATED;
}


/*-------------------------------------------------------------------------
rotate_2d_matrix - ???
-------------------------------------------------------------------------*/

void
rotate_2d_matrix(matrix,radian_angle)
 struct matrix_2d_rec *matrix;
 float radian_angle;
{
 struct matrix_2d_rec temp;

 setup_rotate_2d_matrix(&temp,radian_angle);
 mult_2d_matrix(matrix,&temp,matrix);
}


/*-------------------------------------------------------------------------
setup_rotate_2d_matrix_from_xy - ???
-------------------------------------------------------------------------*/

void
setup_rotate_2d_matrix_from_xy(matrix,radian_angle,fx,fy)
 struct matrix_2d_rec *matrix;
 float radian_angle,fx,fy;
{
 struct matrix_2d_rec temp;

 setup_translate_2d_matrix(matrix,-fx,-fy);
 setup_rotate_2d_matrix(&temp,radian_angle);
 mult_2d_matrix(matrix,&temp,matrix);
 translate_2d_matrix(matrix,fx,fy);
 matrix->status |= MATRIX_TRANSLATED;
}


/*-------------------------------------------------------------------------
rotate_2d_matrix_from_xy - ???
-------------------------------------------------------------------------*/

void
rotate_2d_matrix_from_xy(matrix,radian_angle,fx,fy)
 struct matrix_2d_rec *matrix;
 float radian_angle,fx,fy;
{
 struct matrix_2d_rec temp;

 setup_rotate_2d_matrix_from_xy(&temp,radian_angle,fx,fy);
 mult_2d_matrix(matrix,&temp,matrix);
}


/*-------------------------------------------------------------------------
combine_2d_matrices - ???

combine_matrices(dest, mat1, ..., matn, 0);
-------------------------------------------------------------------------*/

int
combine_2d_matrices(
    struct matrix_2d_rec *dest,
    ...)
    
{
 va_list args;
 struct matrix_2d_rec *matrix;

 va_start(args, dest);
 for (;(matrix = va_arg(args,struct matrix_2d_rec *)) != 0;)
    mult_2d_matrix(dest,matrix,dest);
 va_end(args);
 return NO_ERROR;
}

#if 0       /* Old varargs version */
int
combine_2d_matrices(va_alist)
 va_dcl
{
 va_list args;
 struct matrix_2d_rec *matrix,*dest;

 va_start(args);
 dest = va_arg(args,struct matrix_2d_rec *);
 for (;(matrix = va_arg(args,struct matrix_2d_rec *)) != 0;)
    mult_2d_matrix(dest,matrix,dest);
 va_end(args);
 return NO_ERROR;
}
#endif


/*-------------------------------------------------------------------------
det_2d_matrix - ???
-------------------------------------------------------------------------*/

float 
det_2d_matrix(matrix)
 struct matrix_2d_rec *matrix;
{
 struct matrix_3d_rec _3d_matrix;
 int row,col;
 float det;

 identity_3d_matrix(&_3d_matrix);
 for (row = 0; row < 3; row++)
    for (col = 0; col < 3; col++)
       _3d_matrix.buf[row][col] = matrix->buf[row][col];
 det = det_3d_matrix(&_3d_matrix);
 return det;
}


/*-------------------------------------------------------------------------
inverse_2d_matrix - ???
-------------------------------------------------------------------------*/

int
inverse_2d_matrix(inv,matrix)
 struct matrix_2d_rec *inv,*matrix;
{
 float det;
 int row,col,nextrow,nextcol,nextrow1,nextcol1;
 double temp1,temp2;
/*
 struct matrix_2d_rec m;
*/
 det = det_2d_matrix(matrix);
 /*printf("det = %f\n",det);*/
 if (is_float_0(det)) return 1;
 for (row = 0; row < 3; row++)
    {nextrow = (row + 1) % 3;
     nextrow1 = (nextrow + 1) % 3;
     for (col = 0; col < 3; col++)
        {nextcol = (col + 1) % 3;
         nextcol1 = (nextcol + 1) % 3;
         temp1 = matrix->buf[nextcol][nextrow] * 
             matrix->buf[nextcol1][nextrow1];
         temp2 = matrix->buf[nextcol1][nextrow] * 
             matrix->buf[nextcol][nextrow1];

         temp1 = (temp1 - temp2) / (double) det;
         inv->buf[row][col] = (float) temp1;
        }
    }
 inv->status = MATRIX_INITIALIZED;
/*
 mult_2d_matrix(&m,matrix,inv);
 pr_2d_matrix(&m);
*/
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
setup_xreflect_2d_matrix - ???
-------------------------------------------------------------------------*/

void
setup_xreflect_2d_matrix(matrix)
 struct matrix_2d_rec *matrix;
{
 identity_2d_matrix(matrix);
 matrix->buf[1][1] = -1.0;
}


/*-------------------------------------------------------------------------
setup_yreflect_2d_matrix - ???
-------------------------------------------------------------------------*/

void
setup_yreflect_2d_matrix(matrix)
 struct matrix_2d_rec *matrix;
{
 identity_2d_matrix(matrix);
 matrix->buf[0][0] = -1.0;
}


/*-------------------------------------------------------------------------
setup_xyreflect_2d_matrix - ???
-------------------------------------------------------------------------*/

void
setup_xyreflect_2d_matrix(matrix)
 struct matrix_2d_rec *matrix;
{
 identity_2d_matrix(matrix);
 matrix->buf[0][0] = -1.0;
 matrix->buf[1][1] = -1.0;
}


/*-------------------------------------------------------------------------
setup_xshear_2d_matrix - ???
-------------------------------------------------------------------------*/

void
setup_xshear_2d_matrix(matrix,shear_factor)
 struct matrix_2d_rec *matrix;
 float shear_factor;
{
 identity_2d_matrix(matrix);
 matrix->buf[1][0] = shear_factor;
}


/*-------------------------------------------------------------------------
setup_yshear_2d_matrix - ???
-------------------------------------------------------------------------*/

void
setup_yshear_2d_matrix(matrix,shear_factor)
 struct matrix_2d_rec *matrix;
 float shear_factor;
{
 identity_2d_matrix(matrix);
 matrix->buf[0][1] = shear_factor;
}


/*-------------------------------------------------------------------------
pr_2d_matrix - ???
-------------------------------------------------------------------------*/

void
pr_2d_matrix(matrix)
 struct matrix_2d_rec *matrix;
{
 int row,col;

 printf("pr_2d_matrix status = %lx\n", matrix->status);
 for (row = 0; row < 3; row++)
    {for (col  = 0; col < 3; col++)
        printf("%d/%d) %f   ",row,col,matrix->buf[row][col]);
     printf("\n");}
 printf("\n");
}


/*-------------------------------------------------------------------------
trans_2d_pt - ???
-------------------------------------------------------------------------*/

void
trans_2d_pt(matrix,x,y,newx,newy,is_result_rounded)
 struct matrix_2d_rec *matrix;
 float x,y,*newx,*newy;
 int is_result_rounded;
{
 float x1,y1;

 x1 = matrix->buf[0][0] * x + matrix->buf[0][1] * y +
         matrix->buf[0][2];
 y1 = matrix->buf[1][0] * x + matrix->buf[1][1] * y +
         matrix->buf[1][2];
 if (is_result_rounded)
    {*newx = (float) floor((double) (x1 + .5));
     *newy = (float) floor((double) (y1 + .5));
    }
   else
    {*newx = x1;
     *newy = y1;}
}


/*-------------------------------------------------------------------------
side_and_dir_indx_of_center - ???
-------------------------------------------------------------------------*/

#define TL 1
#define TR 2
#define BL 3
#define BR 4

void
side_and_dir_indx_of_center(centerx,centery,firstx,firsty,nextx,nexty,
                            which_side,dir_indx)
 float centerx,centery,firstx,firsty,nextx,nexty;
 int *which_side,*dir_indx;
{
 static char TR_dirs[9] = {2,1,0,5,4,3,8,7,6};
 float x,y;

 if (firstx < centerx)
    {if (firsty <= centery) *which_side = TL;
       else *which_side = BL;
    }
   else if (firstx > centerx)
    {if (firsty < centery) *which_side = TR;
       else *which_side = BR;
    }
   else
    {if (firsty < centery) *which_side = TL;
       else *which_side = BR;
    }
 x = nextx - firstx;
 y = nexty - firsty;
 if (x > 0) *dir_indx = 6;
   else if (x == 0) *dir_indx = 3;
   else *dir_indx = 0;
 if (y > 0) *dir_indx += 2;
   else if (y == 0) *dir_indx += 1;
 if (*which_side == TL) *dir_indx = 8 - *dir_indx;
   else if (*which_side == TR) *dir_indx = TR_dirs[*dir_indx];
   else if (*which_side == BL) *dir_indx = 8 - (int) TR_dirs[*dir_indx];
}


/*-------------------------------------------------------------------------
scale_xy_ratio - ???
-------------------------------------------------------------------------*/

int
scale_xy_ratio(centerx,centery,firstx,firsty,nextx,nexty,width,height,sx,sy)
 float centerx,centery,firstx,firsty,nextx,nexty,width,height,*sx,*sy;
{
 int which_side,dir;
 float x,y;

 side_and_dir_indx_of_center(centerx,centery,firstx,firsty,nextx,nexty,
               &which_side,&dir);

 x = nextx - firstx;
 y = nexty - firsty;
 TRACE(printf("w = %f h = %f s = %d x = %f  y = %f dir = %d  ",
     width,height,which_side,x,y,dir))
 if (x < .0) x = -x;
 if (y < .0) y = -y;
 if (dir > 5) *sx = 1.0 + 2.0 * x / width;
   else if (dir > 2) *sx = 1.0;
   else
    {*sx = 1.0 - 2.0 * x / width;
     if (*sx < .0) *sx = -(*sx);}
 if (*sx == .0) *sx = 1.0;

 if ((dir % 3) == 0)
    {*sy = 1.0 - 2.0 * y / height;
     if (*sy < .0) *sy = -(*sy);}
   else if ((dir % 3) == 1) *sy = 1.0;
   else *sy = 1.0 + 2.0 * y / height;
 if (*sy == .0) *sy = 1.0;
 TRACE(printf("sx = %f  sy = %f\n",*sx,*sy))
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
rotate_2d_rad - ???
-------------------------------------------------------------------------*/

int
rotate_2d_rad(centerx,centery,firstx,firsty,nextx,nexty,rad)
 float centerx,centery,firstx,firsty,nextx,nexty,*rad;
{
 int which_side,dir,dir1;
 static char TL_signs[9] = {1,-1,-1,1,0,-1,1,1,-1};
 static char TR_dirs[9] =  {1,1,1,-1,0,1,-1,-1,-1};
 struct vector_3d_rec vector,vector1,vector2;
 float dot;
 float x,y;

 side_and_dir_indx_of_center(centerx,centery,firstx,firsty,nextx,nexty,
               &which_side,&dir);
 x = nextx - firstx;
 y = nexty - firsty;

 dir1 = dir;
 if ((which_side == TL) || (which_side == BR)) dir = TL_signs[dir];
   else dir = TR_dirs[dir];

 setup_vector_3d_rec(&vector,centerx,centery,.0);
 setup_vector_3d_rec(&vector1,firstx,firsty,.0);
 setup_vector_3d_rec(&vector2,nextx,nexty,.0);
 sub_vector_3d(&vector1,&vector);
 sub_vector_3d(&vector2,&vector);
 dot = degree_angle_vectors(&vector1,&vector2);
 dot = (float) radian(dot);
 if (dot < 0) dot = -dot;
 if (dir > 0) *rad = dot;
   else if (dir < 0) *rad = -dot;
   else *rad = .0;

 TRACE(printf("which_side = %d x = %f  y = %f indx = %d dir = %d  rad = %f\n",
     which_side,x,y,dir1,dir,*rad))
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
is_same_matrix_2d_rec - ???
-------------------------------------------------------------------------*/

int
is_same_matrix_2d_rec(mat,mat1)
 struct matrix_2d_rec *mat,*mat1;
{
 int row,col;

 for (row = 0; row < 3; row++)
    for (col = 0; col < 3; col++)
       if (mat->buf[row][col] != mat1->buf[row][col])
          return 0;
 return 1;
}
