/****************************************************************************
geometry.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: geometry.c,v 1.5 1999/07/07 18:01:06 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <math.h>
#include <stdio.h>

#include "error_constants.h"
#include "geometry.h"
#include "rect.h"
#include "nria_lib.h"

/*
#define PRINT(x) x; fflush(stdout);
#define TRACE(x) x; fflush(stdout);
*/
#define PRINT(x)
#define TRACE(x)


double distance();


/*------------------------------------------------------------------------
line_gradient - ???
-------------------------------------------------------------------------*/

void
line_gradient(line,gradient,is_infinity)
 struct line_rec *line;
 float *gradient;
 int *is_infinity;
{
 if (line->gradient_x == 0)
    *is_infinity = 1;
   else
    {*is_infinity = 0;
     /* y = mx + c */
     *gradient = line->gradient_y / line->gradient_x;
    }
}


/*------------------------------------------------------------------------
pr_line_rec - ???
-------------------------------------------------------------------------*/

void
pr_line_rec(line)
 struct line_rec *line;
{
 printf("pr_line_rec gy = %f  gx = %f  g = %f  c = %f\n",
      line->gradient_y,line->gradient_x,line->gradient_y / line->gradient_x,
      line->intercept);

}


/*------------------------------------------------------------------------
setup_line_rec - ???
-------------------------------------------------------------------------*/

void
setup_line_rec(line,x1,y1,x2,y2)
 struct line_rec *line;
 float x1,y1,x2,y2;
{
 float gradient;

 line->gradient_y = y1 - y2;
 line->gradient_x = x1 - x2;
 if (line->gradient_x == 0)
    line->intercept = x1; /* x = line->intercept */
   else if (line->gradient_y == 0)
    line->intercept = y1; /* y = line->intercept */
   else
    {   /* y = mx + c */
     gradient = line->gradient_y / line->gradient_x;
     line->intercept = y1 - gradient * x1;
    }
}


/*------------------------------------------------------------------------
setup_line_rec_by_gradient - ???
-------------------------------------------------------------------------*/

void
setup_line_rec_by_gradient(line,x1,y1,gradient)
 struct line_rec *line;
 float x1,y1,gradient;
{
 if (gradient == 0)
    {line->gradient_y = 1;
     line->gradient_x = 0;
     line->intercept = x1;}
 else
   {line->intercept = y1 - gradient * x1;
    line->gradient_y = gradient;
    line->gradient_x = 1.0;
   }
}



/*------------------------------------------------------------------------
apply_line_equation - ???
-------------------------------------------------------------------------*/

int
apply_line_equation(line,x,y)
 struct line_rec *line;
 float x,y;
{
 float gradient,ans;

 if (line->gradient_x == 0)
    {if (line->intercept == x) return 0;
       else if (line->intercept < x) return 1;
       else return -1;
    }
   else if (line->gradient_y == 0)
    {if (line->intercept == y) return 0;
       else if (line->intercept < y) return 1;
       else return -1;
    }
   else
    {gradient = line->gradient_y / line->gradient_x;
     ans = gradient * x + line->intercept - y;
     if (ans == 0) return 0;
       else if (ans > 0) return 1;
       else return -1;
    }
}


/*------------------------------------------------------------------------
get_tangent_line - ???
-------------------------------------------------------------------------*/

void
get_tangent_line(tangent,x,y,line)
 struct line_rec *tangent,*line;
 float x,y;
{
 float gradient;

 if (line->gradient_x == 0)
    {tangent->gradient_y = 0;
     tangent->gradient_x = 1; /* set to 1 to indicate y = tangent->intercept */
     tangent->intercept = y;
    }
   else if (line->gradient_y == 0)
    {tangent->gradient_x = 0;
     tangent->gradient_y = 1; /* set to 1 to indicate x = tangent->intercept */
     tangent->intercept = x;
    }
   else
    { /* gradient = - (1 / m) */
     tangent->gradient_y = -line->gradient_x;
     tangent->gradient_x = line->gradient_y;
     gradient = tangent->gradient_y / tangent->gradient_x;
     TRACE(printf("-1/m gradient = %f g = %f\n",gradient,
                (float) line->gradient_y/line->gradient_x))
     tangent->intercept = y - gradient * x;
     TRACE(printf("x = %f y = %f c = %f\n",x,y,tangent->intercept))
    }

}


/*------------------------------------------------------------------------
get_xy_intercept - ???
-------------------------------------------------------------------------*/

void
get_xy_intercept(line,is_x_valid,xintercept,is_y_valid,yintercept)
 struct line_rec *line;
 int *is_x_valid,*is_y_valid;
 float *xintercept,*yintercept;
{
 float gradient;

 if (line->gradient_x == 0)
    {*is_x_valid = 1;
     *xintercept = line->intercept;
     *is_y_valid = 0;
    }
   else if (line->gradient_y == 0)
    {*is_y_valid = 1;
     *yintercept = line->intercept;
     *is_x_valid = 0;
    }
   else
    {*is_y_valid = 1;
     *yintercept = line->intercept;
     *is_x_valid = 1;
     gradient = line->gradient_y / line->gradient_x;
     *xintercept = -line->intercept / gradient;
    }
     
}


/*------------------------------------------------------------------------
lines_parallel - ???

  return LINE_X_EQUAL_K_SAME 
     if line1 == line2  and both are in the form of x = k
  return LINE_X_EQUAL_K_PARALLEL
     if line1 is parallel to line2 and both are in the form of x = k
  return LINE_Y_EQUAL_C_SAME
     if line1 == line2  and both are in the form of  y = c
  return LINE_Y_EQUAL_C_PARALLEL
     if line1 is parallel to line2 and both are in the form of y = c
  return LINE_PARALLEL if line1 is parallel to line2
  return LINE_NO_PARALLEL otherwise
-------------------------------------------------------------------------*/

int
lines_parallel(line1,line2)
 struct line_rec *line1,*line2;
{
 float gradient1,gradient2;

 if ((line1->gradient_x == 0) && (line2->gradient_x == 0))
    {if (line1->intercept == line2->intercept)
        return LINE_X_EQUAL_K_SAME;
       else return LINE_X_EQUAL_K_PARALLEL;
    }
   else if ((line1->gradient_y == 0) && (line2->gradient_y == 0))
    {if (line1->intercept == line2->intercept)
        return LINE_Y_EQUAL_C_SAME;
       else return LINE_Y_EQUAL_C_PARALLEL;
    }
   else
    {
     gradient1 = line1->gradient_y / line1->gradient_x;
     gradient2 = line2->gradient_y / line2->gradient_x;
     if (gradient1 == gradient2) return LINE_PARALLEL;
     return LINE_NO_PARALLEL;
    }
}


/*------------------------------------------------------------------------
get_lines_intersection - ???
-------------------------------------------------------------------------*/

void
get_lines_intersection(line1,line2,xintersect,yintersect,is_line_parallel)
 struct line_rec *line1,*line2;
 float *xintersect,*yintersect;
 int *is_line_parallel;
{
 float gradient1,gradient2;
 int is_infinity1,is_infinity2;

 *is_line_parallel = lines_parallel(line1,line2);
 if (*is_line_parallel != 0) return;
 line_gradient(line1,&gradient1,&is_infinity1);
 line_gradient(line2,&gradient2,&is_infinity2);
 if (is_infinity1)
    {*xintersect = line1->intercept;
     *yintersect = gradient2 * *xintersect + line2->intercept;
    }
   else if (is_infinity2)
    {*xintersect = line2->intercept;
     *yintersect = gradient1 * *xintersect + line1->intercept;
    }
   else
    {gradient1 = line1->gradient_y / line1->gradient_x;
     gradient2 = line2->gradient_y / line2->gradient_x;
     *xintersect = (line2->intercept - line1->intercept) /
                   (gradient1 - gradient2);
     *yintersect = gradient1 * *xintersect + line1->intercept;
    }

}


/*------------------------------------------------------------------------
pt_distance_fr_line - ???
-------------------------------------------------------------------------*/

void
pt_distance_fr_line(line,x,y,dist)
 struct line_rec *line;
 float x,y,*dist;
{
 struct line_rec tangent;
 float xintersect,yintersect;
 int is_line_parallel;

 get_tangent_line(&tangent,x,y,line);
 get_lines_intersection(&tangent,line,&xintersect,
          &yintersect,&is_line_parallel);
 *dist = (float) distance(x, y,xintersect,yintersect);

}


/*------------------------------------------------------------------------
angle_of_lines_intersection - ???

 angle in radian is return
-------------------------------------------------------------------------*/

void
angle_of_lines_intersection(line1,line2,angle,is_line_parallel)
 struct line_rec *line1,*line2;
 float *angle;
 int *is_line_parallel;
{
 float gradient1,gradient2;
 int is_infinity1,is_infinity2;

 *is_line_parallel = lines_parallel(line1,line2);
 if (*is_line_parallel != 0) return;

 if (line1->gradient_x == 0)
    {printf("line2->gradient_x = %f line2->gradient_y = %f = %f\n",
           line2->gradient_x,line2->gradient_y,
           line2->gradient_x / line2->gradient_y);
     if (line2->gradient_y == 0) *angle = PI_RADIAN / 2;
       else
        {printf("atan = %f\n",
               atan((double) (line2->gradient_x / line2->gradient_y)));
         *angle = (float) atan((double) (line2->gradient_x / line2->gradient_y));
        }
    }
   else if (line1->gradient_y == 0)
    {if (line2->gradient_x == 0) *angle = PI_RADIAN / 2;
       else
        *angle = (float) atan((double) (line2->gradient_y / line2->gradient_x));
    }
   else if (line2->gradient_x == 0)
    {if (line1->gradient_y == 0) *angle = PI_RADIAN / 2;
       else
        *angle = (float) atan((double) (line1->gradient_x / line1->gradient_y));
    }
   else if (line2->gradient_y == 0)
    {if (line1->gradient_x == 0) *angle = PI_RADIAN / 2;
       else
        *angle = (float) atan((double) (line1->gradient_y / line1->gradient_x));
    }
   else
    {line_gradient(line1,&gradient1,&is_infinity1);
     line_gradient(line2,&gradient2,&is_infinity2);
     gradient1 = (gradient2 - gradient1) / (1 + gradient1 * gradient2);
     *angle = (float) atan((double) gradient1);
    }
}


/*------------------------------------------------------------------------
angle_between_line_and_y_axis - ???

angle in radian is return
-------------------------------------------------------------------------*/

void
angle_between_line_and_y_axis(line,angle,is_line_parallel)
 struct line_rec *line;
 float *angle;
 int *is_line_parallel;
{
 struct line_rec line1;

 setup_line_rec(&line1,0.0,0.0,0.0,1.0);
 TRACE(printf("line1->gx = %f gy = %f c = %f\n",
  line1.gradient_x,line1.gradient_y,line1.intercept))
 angle_of_lines_intersection(&line1,line,angle,is_line_parallel);
}


/*------------------------------------------------------------------------
setup_line_segment_rec - ???
-------------------------------------------------------------------------*/

void
setup_line_segment_rec(segment,x1,y1,x2,y2)
 struct line_segment_rec *segment;
 int x1,y1,x2,y2;
{
 setup_line_rec(&(segment->eq),(float) x1,(float) y1,(float) x2, (float) y2);
 segment->start.x = x1;
 segment->start.y = y1;
 segment->end.x = x2;
 segment->end.y = y2;
}



/*------------------------------------------------------------------------
are_pts_divided - ???
-------------------------------------------------------------------------*/

int
are_pts_divided(line,x1,y1,x2,y2)
 struct line_rec *line;
 float x1,y1,x2,y2;
{
 int sign1,sign2;

 sign1 = apply_line_equation(line,x1,y1);
 sign2 = apply_line_equation(line,x2,y2);
 if ((sign1 == 0) || (sign2 == 0)) return 1;
    else if (sign1 != sign2) return 2;
 return 0;
}



/*****************************
#define LINE_SEGMENT_NOT_INTERSECTING 0
#define LINE_SEGMENT_EQUAL 1
#define LINE_SEGMENT_INTERSECTS 2
#define LINE_SEGMENT_PARALLEL 3

    xintersect,yintersect are only defined 
         if is_intersect = LINE_SEGMENT_INTERSECTS
*******************************/

/*
to be modified
*/

/****************************
line_segments_intersection(segment1,segment2,xintersect,yintersect,is_intersect)
 struct line_segment_rec *segment1,*segment2;
 float *xintersect,*yintersect;
 int *is_intersect;
{
 struct int_rect_rec rr1,rr2;
 int overlap,sign1,sign2,is_line_parallel;

 make_int_rect_rec(&rr1,segment1->start.x,segment1->start.y,
                       segment1->end.x,segment1->end.y);
 make_int_rect_rec(&rr2,segment2->start.x,segment2->start.y,
                       segment2->end.x,segment2->end.y);
 overlap = overlapping_rects(&rr1,&rr2);
 if ((overlap = overlapping_rects(&rr1,&rr2)) == RECT_NO_OVERLAP)
    *is_intersect = LINE_SEGMENT_NOT_INTERSECTING;
   else if (overlap == RECT_EQUAL)
    *is_intersect = LINE_SEGMENT_EQUAL;
   else if ((overlap >= RECT_OVERLAPS_AT_TL_CORNER) &&
            (overlap <= RECT_OVERLAPS_AT_BL_CORNER))
    {if (overlap == RECT_OVERLAPS_AT_TL_CORNER)
        {*xintersect = rr1.left;
         *yintersect = rr1.top;}
       else if (overlap == RECT_OVERLAPS_AT_TR_CORNER)
        {*xintersect = rr1.left + rr1.width - 1;
         *yintersect = rr1.top;}
       else if (overlap == RECT_OVERLAPS_AT_BR_CORNER)
        {*xintersect = rr1.left + rr1.width - 1;
         *yintersect = rr1.top + rr1.height - 1;}
       else
        {*xintersect = rr1.left;
         *yintersect = rr1.top + rr1.height - 1;}

     *is_intersect = LINE_SEGMENT_INTERSECTS;
    }
   else
    {*is_intersect = LINE_SEGMENT_NOT_INTERSECTING;
     if (are_pts_divided(&(segment2->eq),
                   (float) segment1->start.x,(float) segment1->start.y,
                   (float) segment1->end.x,(float) segment1->end.y))
        if (are_pts_divided(&(segment1->eq),
                   (float) segment2->start.x,(float) segment2->start.y,
                   (float) segment2->end.x,(float) segment2->end.y))
           {printf("get_lines_intersection...\n");
            get_lines_intersection(&(segment1->eq),&(segment2->eq),
                    xintersect,yintersect,&is_line_parallel);
            if (!is_line_parallel) *is_intersect = LINE_SEGMENT_INTERSECTS;
              else *is_intersect = LINE_SEGMENT_PARALLEL;
            return;
           }
     get_lines_intersection(&(segment1->eq),&(segment2->eq),
                    xintersect,yintersect,&is_line_parallel);
     if (is_line_parallel) *is_intersect = LINE_SEGMENT_PARALLEL;
    }

}

*****************************/
