/****************************************************************************
hdr_opr.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: hdr_opr.c,v 1.7 1999/07/07 18:01:06 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdarg.h>
#include <stdio.h>

#include <sys/types.h>

#include "typedef.h"
#include "ia_hdr.h"
#include "error_constants.h"
#include "common.h"
#include "nria_lib.h"



#if 0
#  define PRT(x) x; fflush(stdout)
#else
#  define PRT(x)
#endif




/*-----------------------------------------------------------------------
init_color_control_rec - ???
-----------------------------------------------------------------------*/

void
init_color_control_rec(color_control)
     struct color_control_rec *color_control;
{
  color_control->status = 0;
}



/*-----------------------------------------------------------------------
init_extra_file_info_rec - ???
-----------------------------------------------------------------------*/

void
init_extra_file_info_rec(extra_info)
     struct extra_file_info_rec *extra_info;
{
  extra_info->color_bundle_label = NULL;
  init_color_control_rec(&(extra_info->color_control));
  extra_info->last_edit_begin = NULL;
  extra_info->last_edit_end = NULL;
}



/*-----------------------------------------------------------------------
init_hdr_scan_entry_rec - ???
-----------------------------------------------------------------------*/

void
init_hdr_scan_entry_rec(slice)
     struct hdr_scan_entry_rec *slice;
{
  slice->status = 0;
  slice->img_label = NULL;
  slice->img_addr = slice->num = 0;
  slice->slice_num = slice->scan_num = 0;
  slice->prior_slice = slice->partner_slice = slice->next_slice = NULL;
  slice->slice = NULL;
}



/*-----------------------------------------------------------------------
free_hdr_scan_entry_rec - ???
-----------------------------------------------------------------------*/

void
free_hdr_scan_entry_rec(slice)
     struct hdr_scan_entry_rec *slice;
{
  c_free(slice->img_label);
}



/*-----------------------------------------------------------------------
init_patient_rec - ???
-----------------------------------------------------------------------*/

void
init_patient_rec(patient)
     struct patient_rec *patient;
{
  patient->name = NULL;
  patient->ssnum = NULL;
  patient->patient_num = NULL;
  patient->patient_id = NULL;
  patient->study_num = NULL;
  patient->sex = ' ';
  patient->age = 0;
  patient->height = patient->weight = 0;
}



/*-----------------------------------------------------------------------
free_patient_rec - ???
-----------------------------------------------------------------------*/

void
free_patient_rec(patient)
     struct patient_rec *patient;
{
  c_free(patient->name);
  c_free(patient->ssnum);
  c_free(patient->patient_id);
  c_free(patient->study_num);
  c_free(patient->patient_num);
}



/*-----------------------------------------------------------------------
init_data_info_rec - ???
-----------------------------------------------------------------------*/

void
init_data_info_rec(info)
     struct data_info_rec *info;
{
  static float trans_matrix[4][4] = {
    {1.0,.0,.0,.0},
    {.0,1.0,.0,.0},
    {.0,.0,1.0,.0},
    {.0,.0,.0,1.0}
  };
  info->image_id = NULL;
  info->default_datafn = NULL;
  info->dimx = info->dimy = info->max_scans = 0;
  info->min_pixel = info->max_pixel = 0;
  info->pixel_depth_bits = 0;
  info->xpixel_size = info->ypixel_size = .0;
  info->spacing = info->pixel_thickness = .0;
  info->plane_type = -1;
  memcpy(info->trans_matrix,trans_matrix,(size_t) sizeof(info->trans_matrix));
  info->var_use_status = 0;
  info->unused1 = 0;
  info->num_scans = 0;
  info->unused2 = info->unused3 = info->unused4 = info->unused5 = 0;
  info->unused6 = info->unused7 = .0;
  info->num_echoes = 0;
  info->tr = 0;
  info->te = info->date_of_study = info->time_of_study = NULL;
  init_patient_rec(&(info->patient));
  info->refered_by = info->description = info->hosp_name = NULL;
}



/*-----------------------------------------------------------------------
free_data_info_rec - ???
-----------------------------------------------------------------------*/

void
free_data_info_rec(info)
     struct data_info_rec *info;
{
  c_free(info->image_id);
  c_free(info->default_datafn);
  c_free(info->te);
  c_free(info->date_of_study);
  c_free(info->time_of_study);
  free_patient_rec(&(info->patient));
  c_free(info->refered_by);
  c_free(info->description);
  c_free(info->hosp_name);
  init_data_info_rec(info);
}



/*-----------------------------------------------------------------------
init_analysis_group_rec - ???
-----------------------------------------------------------------------*/

void
init_analysis_group_rec(group)
     struct analysis_group_rec *group;
{
  group->num_members = 0;
  group->member_img_label_indx = 0;
}



/*-----------------------------------------------------------------------
init_hdr_analysis_rec - ???
-----------------------------------------------------------------------*/

void
init_hdr_analysis_rec(analysis)
     struct hdr_analysis_rec *analysis;
{
  analysis->num_img_labels = 0;
  analysis->img_labels = NULL;
  analysis->num_analysis_groups = 0;
  analysis->groups = NULL;
}



/*-----------------------------------------------------------------------
init_hdr_rec - ???

initialize the pointers & the counters
-----------------------------------------------------------------------*/

void
init_hdr_rec(hdr)
     struct hdr_rec *hdr;
{
  hdr->size = 0;
  hdr->id = NULL;
  hdr->offset.info_start = hdr->offset.info_size = 0;
  hdr->offset.extra_info_start = hdr->offset.extra_info_size = 0;
  hdr->offset.comment_start = hdr->offset.comment_size = 0;
  hdr->offset.user_defined_start = hdr->offset.user_defined_size = 0;

  init_data_info_rec(&(hdr->data_info));
  init_hdr_analysis_rec(&(hdr->analysis));

  hdr->num_scans = 0;
  hdr->scans = NULL;
  init_extra_file_info_rec(&(hdr->extra_info));
  hdr->comment_size = 0;
  hdr->comment = NULL;
  hdr->user_defined_size = 0;
  hdr->user_defined = NULL;
}



/*-----------------------------------------------------------------------
free_analysis_group_rec - ???
-----------------------------------------------------------------------*/

void
free_analysis_group_rec(groups,num_analysis_groups)
     struct analysis_group_rec *groups;
     unsigned int num_analysis_groups;
{
  unsigned int lcv;

  for (lcv = 0; lcv < num_analysis_groups; lcv++)
    c_free(groups[lcv].member_img_label_indx);
}



/*-----------------------------------------------------------------------
free_hdr_analysis_rec - ???
-----------------------------------------------------------------------*/

void
free_hdr_analysis_rec(analysis)
     struct hdr_analysis_rec *analysis;
{
  free_handle_memory(analysis->img_labels,analysis->num_img_labels);
  free_analysis_group_rec(analysis->groups,analysis->num_analysis_groups);
  c_free(analysis->groups);
  init_hdr_analysis_rec(analysis);
}



/*-----------------------------------------------------------------------
setup_hdr_analysis_rec - ???

setup_hdr_analysis_rec(analysis,num_img_labels[,labe1,...,labeln,
            [num_analysis_groups[,num_members,g1_num1,...,g1_numn,
                num_members,g2_num1,...,g2_numn,...]]])
-----------------------------------------------------------------------*/

int
setup_hdr_analysis_rec(
    struct hdr_analysis_rec *analysis,
    unsigned int num_img_labels,
    ...)
{
  va_list args;

  unsigned int lcv,cnt;
  unsigned int num_analysis_groups,num_members;
  struct analysis_group_rec *group;
  int err;
  char *str;

  va_start(args, num_img_labels);

  if (num_img_labels <= 0) return CANCEL;
  if ((err = get_ptr(&(analysis->img_labels),
		     &(analysis->num_img_labels),num_img_labels,
		     (size_t) sizeof(*(analysis->img_labels)))) != NO_ERROR)
    return err;

  for (lcv = 0; lcv < analysis->num_img_labels; lcv++)
    analysis->img_labels[lcv] = NULL;

  for (lcv = 0; lcv < analysis->num_img_labels; lcv++) {
    if ((str = va_arg(args, char *)) != NULL)
      if ((err = cp_str(&(analysis->img_labels[lcv]),str)) != NO_ERROR)
	break;
  }

  if (err == NO_ERROR)
    {num_analysis_groups = va_arg(args, unsigned int);
     if (num_analysis_groups > 0)
       {if ((err = get_ptr(&(analysis->groups),
			   &(analysis->num_analysis_groups),num_analysis_groups,
			   (size_t) sizeof(*(analysis->groups)))) == NO_ERROR)
	  {for (lcv = 0; lcv < analysis->num_analysis_groups; lcv++)
	     init_analysis_group_rec(&(analysis->groups[lcv]));
	   for (lcv = 0; lcv < analysis->num_analysis_groups; lcv++)
	     {num_members = va_arg(args, unsigned int);
	      group = &(analysis->groups[lcv]);
	      if ((err = get_ptr(&(group->member_img_label_indx),
				 &(group->num_members),num_members,
				 (size_t) sizeof(*(group->member_img_label_indx)))) != NO_ERROR)
		break;
	      for (cnt = 0; cnt < num_members; cnt++)
		group->member_img_label_indx[cnt] = va_arg(args, unsigned int);
	    }
	 }
      }
   }

  va_end(args);

  if (err != NO_ERROR)
    free_hdr_analysis_rec(analysis);

  return err;
}


#if 0     /* Old varargs version */
int
setup_hdr_analysis_rec(va_alist)
     va_dcl
{
  va_list args;

  unsigned int lcv,cnt;
  unsigned int num_img_labels,num_analysis_groups,num_members;
  struct hdr_analysis_rec *analysis;
  struct analysis_group_rec *group;
  int err;
  char *str;



  va_start(args);

  analysis = va_arg(args, struct hdr_analysis_rec *);
  num_img_labels = va_arg(args, unsigned int);
  if (num_img_labels <= 0) return CANCEL;
  if ((err = get_ptr(&(analysis->img_labels),
		     &(analysis->num_img_labels),num_img_labels,
		     (size_t) sizeof(*(analysis->img_labels)))) != NO_ERROR)
    return err;

  for (lcv = 0; lcv < analysis->num_img_labels; lcv++)
    analysis->img_labels[lcv] = NULL;

  for (lcv = 0; lcv < analysis->num_img_labels; lcv++) {
    if ((str = va_arg(args, char *)) != NULL)
      if ((err = cp_str(&(analysis->img_labels[lcv]),str)) != NO_ERROR)
	break;
  }

  if (err == NO_ERROR)
    {num_analysis_groups = va_arg(args, unsigned int);
     if (num_analysis_groups > 0)
       {if ((err = get_ptr(&(analysis->groups),
			   &(analysis->num_analysis_groups),num_analysis_groups,
			   (size_t) sizeof(*(analysis->groups)))) == NO_ERROR)
	  {for (lcv = 0; lcv < analysis->num_analysis_groups; lcv++)
	     init_analysis_group_rec(&(analysis->groups[lcv]));
	   for (lcv = 0; lcv < analysis->num_analysis_groups; lcv++)
	     {num_members = va_arg(args, unsigned int);
	      group = &(analysis->groups[lcv]);
	      if ((err = get_ptr(&(group->member_img_label_indx),
				 &(group->num_members),num_members,
				 (size_t) sizeof(*(group->member_img_label_indx)))) != NO_ERROR)
		break;
	      for (cnt = 0; cnt < num_members; cnt++)
		group->member_img_label_indx[cnt] = va_arg(args, unsigned int);
	    }
	 }
      }
   }

  va_end(args);

  if (err != NO_ERROR)
    free_hdr_analysis_rec(analysis);

  return err;
}
#endif


/*-----------------------------------------------------------------------
free_hdr_scan_entry_recs - ???
-----------------------------------------------------------------------*/

void
free_hdr_scan_entry_recs(scans,num_scans)
     struct hdr_scan_entry_rec *scans;
     unsigned int num_scans;
{
  unsigned int lcv;

  for (lcv = 0; lcv < num_scans; lcv++)
    free_hdr_scan_entry_rec(&(scans[lcv]));
}



/*-----------------------------------------------------------------------
free_hdr_rec - ???
-----------------------------------------------------------------------*/

void
free_hdr_rec(hdr)
     struct hdr_rec *hdr;
{
  c_free(hdr->id);
  free_data_info_rec(&(hdr->data_info));
  free_hdr_analysis_rec(&(hdr->analysis));
  free_hdr_scan_entry_recs(hdr->scans,hdr->num_scans);
  c_free(hdr->scans);
  free_extra_file_info_rec(&(hdr->extra_info));
  if (hdr->comment_size > 0) c_free(hdr->comment);
  if (hdr->user_defined_size > 0) c_free(hdr->user_defined);
  init_hdr_rec(hdr);
}



/*-----------------------------------------------------------------------
free_extra_file_info_rec - ???
-----------------------------------------------------------------------*/

void
free_extra_file_info_rec(extra_info)
     struct extra_file_info_rec *extra_info;
{
  c_free(extra_info->color_bundle_label);
  c_free(extra_info->last_edit_begin);
  c_free(extra_info->last_edit_end);
  init_extra_file_info_rec(extra_info);
}



/*-----------------------------------------------------------------------
get_plane_type_indicator - ???
-----------------------------------------------------------------------*/

char 
get_plane_type_indicator(plane_type)
     int plane_type;
{
  switch (plane_type) {
  case 0 : return 'a';
  case 1 : return 's';
  case 2 : return 'c';
  case 3 : return 'o';
  default : return 'u';
  }
}



/*-----------------------------------------------------------------------
get_plane_type - ???
-----------------------------------------------------------------------*/

char *
get_plane_type(plane_type)
     int plane_type;
{
  switch (plane_type) {
  case 0 : return "AXIAL";
  case 1 : return "SAGITTAL";
  case 2 : return "CORONAL";
  case 3 : return "OBLIQUE";
  default : return "UNKNOWN";
  }
}



/*-----------------------------------------------------------------------
get_patient_id - ???
-----------------------------------------------------------------------*/

char *
get_patient_id(hdr)
     struct hdr_rec *hdr;
{
  if ((hdr->data_info.patient.patient_num != NULL) &&
      (strlen(hdr->data_info.patient.patient_num) != 0))
    return hdr->data_info.patient.patient_num;
  return hdr->data_info.patient.study_num;
}

  

/*-----------------------------------------------------------------------
build_default_datafn - ???
-----------------------------------------------------------------------*/
  
int
build_default_datafn(hdr)
     struct hdr_rec *hdr;
{
  char *str,*str1,str2[2];
  int err;

  if ((err = cp_str(&str1,hdr->data_info.image_id)) != NO_ERROR)
    return err;
  str2[0] = get_plane_type_indicator(hdr->data_info.plane_type);
  str2[1] = '\0';

  if ((err = concat_strs(&str,5,str1,get_patient_id(hdr),str2,".","hdr"))
      != NO_ERROR)
    {c_free(hdr->data_info.default_datafn);
     hdr->data_info.default_datafn = str;
   }
  c_free(str1);
  return err;

}



/*-----------------------------------------------------------------------
slice_entry_index_to_addr - ???
-----------------------------------------------------------------------*/

void
slice_entry_index_to_addr(scans,num_scans)
     struct hdr_scan_entry_rec *scans;
     unsigned int num_scans;
{
  unsigned int lcv;

  for (lcv = 0; lcv < num_scans; lcv += 1)
    {
      scans[lcv].prior_slice = &(scans[(int) scans[lcv].prior_slice]);
      scans[lcv].next_slice = &(scans[(int) scans[lcv].next_slice]);
      scans[lcv].partner_slice = &(scans[(int) scans[lcv].partner_slice]);
    }

}



/*-----------------------------------------------------------------------
slice_entry_addr_to_index - ???
-----------------------------------------------------------------------*/

void
slice_entry_addr_to_index(scans,num_scans)
     struct hdr_scan_entry_rec *scans;
     unsigned int num_scans;
{
  unsigned int lcv;

  for (lcv = 0; lcv < num_scans; lcv += 1)
    {scans[lcv].prior_slice = (struct hdr_scan_entry_rec *)
       (scans[lcv].prior_slice - 
	&(scans[0]));
     scans[lcv].next_slice =  (struct hdr_scan_entry_rec *)
       (scans[lcv].next_slice - 
	&(scans[0]));
     scans[lcv].partner_slice = (struct hdr_scan_entry_rec *)
       (scans[lcv].partner_slice -
	&(scans[0]));
   }

}



/*-----------------------------------------------------------------------
wr_extra_file_info_rec - ???
-----------------------------------------------------------------------*/

int
wr_extra_file_info_rec(fid,extra_info)
     FILE *fid;
     struct extra_file_info_rec *extra_info;
{
  int err;

  PRT(printf("wr_extra_file_info_rec color_control = %d\n",
	     sizeof(extra_info->color_control)));
  PRT(printf("color_bundle_label = %s\n",extra_info->color_bundle_label));
  PRT(printf("last_edit_begin = %s\n",extra_info->last_edit_begin));
  PRT(printf("last_edit_end = %s\n",extra_info->last_edit_end));
  err = WRITE_ERROR;
  if (((err = wr_file_strs(fid,&(extra_info->color_bundle_label),1)) 
       != NO_ERROR) ||
      (fwrite(&(extra_info->color_control),
	      sizeof(extra_info->color_control),1,fid) != 1) ||
      ((err = wr_file_strs(fid,&(extra_info->last_edit_begin),1)) != NO_ERROR) ||
      ((err = wr_file_strs(fid,&(extra_info->last_edit_end),1)) != NO_ERROR))
    return err;
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
wr_hdr - ???
-----------------------------------------------------------------------*/

int
wr_hdr(fid,hdr)
     FILE *fid;
     struct hdr_rec *hdr;
{
  size_t amt,offset;
  int err;

  if (str_cmp(hdr->id,HDR_ID_V2) != 0)
    {c_free(hdr->id);
     if ((err = cp_str(&(hdr->id),HDR_ID_V2)) != NO_ERROR)
       return err;
   }

  fseek(fid,0L,0);
  err = WRITE_ERROR;
  PRT(printf("wr_hdr...hdr->id = %s\n",hdr->id));
  if ((fwrite(&(hdr->size),sizeof(hdr->size),1,fid) != 1) ||
      ((err = wr_file_strs(fid,&(hdr->id),1)) != NO_ERROR))
    {if (err == NO_ERROR) err = WRITE_ERROR;
     return err;}
  fflush(fid);
  offset = ftell(fid);
  PRT(printf("wr_hdr...offset = %d\n",offset));
  if (fwrite(&(hdr->offset),sizeof(hdr->offset),1,fid) != 1)
    return WRITE_ERROR;
  fflush(fid);
  hdr->offset.info_start = ftell(fid);

  err = WRITE_ERROR;
  if ((fwrite(&(hdr->status),sizeof(hdr->status),1,fid) != 1) ||
      ((err = WrDataInfoRec(fid,&(hdr->data_info))) != NO_ERROR) ||
      (wr_analysis_rec(fid,&(hdr->analysis)) != 0) ||
      (fwrite(&(hdr->num_scans),sizeof(hdr->num_scans),1,fid) != 1) ||
      (wr_slice_entries(fid,hdr->scans,hdr->num_scans) != 0))
    {if (err == NO_ERROR) err = WRITE_ERROR;
     return err;}
  fflush(fid);
  hdr->offset.info_size = hdr->offset.extra_info_start = ftell(fid);
  hdr->offset.info_size -= hdr->offset.info_start;

  PRT(printf("info start = %d size = %d\n",hdr->offset.info_start,hdr->offset.info_size));

  if ((err = wr_extra_file_info_rec(fid,&(hdr->extra_info))) != NO_ERROR)
    return err;
  fflush(fid);
  hdr->offset.extra_info_size = hdr->offset.comment_start = ftell(fid);
  hdr->offset.extra_info_size -= hdr->offset.extra_info_start;
  PRT(printf("extra_info start = %d size = %d\n",hdr->offset.extra_info_start,hdr->offset.extra_info_size));

  err = WRITE_ERROR;
  PRT(printf("hdr->comment_size = %d\n",hdr->comment_size));
  if ((fwrite(&(hdr->comment_size),sizeof(hdr->comment_size),1,fid) != 1) ||
      ((err = fwr(hdr->comment,hdr->comment_size,1,fid,&amt))
       != NO_ERROR))
    {if (err == NO_ERROR) err = WRITE_ERROR;
     return err;
   }
  fflush(fid);
  hdr->offset.user_defined_start = hdr->offset.comment_size = ftell(fid);
  hdr->offset.comment_size -= hdr->offset.comment_start;

  PRT(printf("hdr->user_defined_size = %d\n",hdr->user_defined_size));
  if ((fwrite(&(hdr->user_defined_size),sizeof(hdr->user_defined_size),1,
	      fid) != 1) ||
      ((err = fwr(hdr->user_defined,hdr->user_defined_size,1,fid,&amt))
       != NO_ERROR))
    {if (err == NO_ERROR) err = WRITE_ERROR;
     return err;}
  fflush(fid);
  hdr->offset.user_defined_size = ftell(fid);
  hdr->offset.user_defined_size -= hdr->offset.user_defined_start;

  hdr->size = ftell(fid);
  fseek(fid,0L,0);
  PRT(printf("hdr->size = %d\n",(int) hdr->size));
  if (fwrite(&(hdr->size),sizeof(hdr->size),1,fid) != 1)
    return WRITE_ERROR;

  fseek(fid,offset,0);
  if (fwrite(&(hdr->offset),sizeof(hdr->offset),1,fid) != 1)
    return WRITE_ERROR;

  return NO_ERROR;
}



/*-----------------------------------------------------------------------
wr_the_hdr - ???
-----------------------------------------------------------------------*/

int
wr_the_hdr(fn,hdr)
     char *fn;
     struct hdr_rec *hdr;
{
  FILE *fid;
  int err;


  if (hdr == NULL) {
    PRT(printf("wr_the_hdr: NULL hdr.\n" ); );
    return CANCEL;
  }

  PRT(printf("wr_the_hdr...hdr = %u\n",hdr));

  if ((fid = fopen(fn,"w+")) == NULL)
    {PRT(printf("\nCould not open output file: %s\n",fn));
     return FILE_ACCESS_ERROR;}

  err = wr_hdr(fid,hdr);
  PRT(printf("wr_the_hdr...err = %d\n",err));

  fclose(fid);
  return err;
}



/*-----------------------------------------------------------------------
wr_patient_rec - ???
-----------------------------------------------------------------------*/

int
wr_patient_rec(fid,patient)
     FILE *fid;
     struct patient_rec *patient;
{
  int err;

  err = WRITE_ERROR;
  if (((err = wr_file_strs(fid,&(patient->name),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = wr_file_strs(fid,&(patient->ssnum),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = wr_file_strs(fid,&(patient->patient_id),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = wr_file_strs(fid,&(patient->study_num),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = wr_file_strs(fid,&(patient->patient_num),(unsigned int) 1))
       != NO_ERROR) ||
      (fwrite(&(patient->sex),sizeof(patient->sex),1,fid) != 1) ||
      (fwrite(&(patient->age),sizeof(patient->age),1,fid) != 1) ||
      (fwrite(&(patient->height),sizeof(patient->height),1,fid) != 1) ||
      (fwrite(&(patient->weight),sizeof(patient->weight),1,fid) != 1))
    return err;

  return NO_ERROR;
}



/*-----------------------------------------------------------------------
WrDataInfoRec - ???
-----------------------------------------------------------------------*/

int
WrDataInfoRec(fid,info)
     FILE *fid;
     struct data_info_rec *info;
{
  int err;

  PRT(printf("WrDataInfoRec...\n"));
  err = WRITE_ERROR;
  if (((err = wr_file_strs(fid,&(info->image_id),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = wr_file_strs(fid,&(info->default_datafn),(unsigned int) 1))
       != NO_ERROR) ||
      (fwrite(&(info->num_scans),sizeof(info->num_scans),1,fid) != 1) ||
      (fwrite(&(info->dimx),sizeof(info->dimx),1,fid) != 1) ||
      (fwrite(&(info->dimy),sizeof(info->dimy),1,fid) != 1) ||
      (fwrite(&(info->max_scans),sizeof(info->max_scans),1,fid) != 1) ||
      (fwrite(&(info->min_pixel),sizeof(info->min_pixel),1,fid) != 1) ||
      (fwrite(&(info->max_pixel),sizeof(info->max_pixel),1,fid) != 1) ||
      (fwrite(&(info->unused1),sizeof(info->unused1),1,fid) != 1) ||
      (fwrite(&(info->pixel_depth_bits),sizeof(info->pixel_depth_bits),1,fid)
       != 1) ||
      (fwrite(&(info->xpixel_size),sizeof(info->xpixel_size),1,fid) != 1) ||
      (fwrite(&(info->ypixel_size),sizeof(info->ypixel_size),1,fid) != 1) ||
      (fwrite(&(info->pixel_thickness),sizeof(info->pixel_thickness),1,fid)
       != 1) ||
      (fwrite(&(info->spacing),sizeof(info->spacing),1,fid) != 1) ||
      (fwrite(&(info->plane_type),sizeof(info->plane_type),1,fid) != 1) ||
      (fwrite(info->trans_matrix,sizeof(info->trans_matrix),1,fid) != 1) ||
      (fwrite(&(info->unused2),sizeof(info->unused2),1,fid) != 1) ||
      (fwrite(&(info->var_use_status),sizeof(info->var_use_status),1,fid) != 1) ||
      (fwrite(&(info->num_echoes),sizeof(info->num_echoes),1,fid) != 1) ||
      (fwrite(&(info->unused3),sizeof(info->unused3),1,fid) != 1) ||
      (fwrite(&(info->unused4),sizeof(info->unused4),1,fid) != 1) ||
      (fwrite(&(info->unused5),sizeof(info->unused5),1,fid) != 1) ||
      (fwrite(&(info->unused6),sizeof(info->unused6),1,fid) != 1) ||
      (fwrite(&(info->unused7),sizeof(info->unused7),1,fid) != 1) ||
      (fwrite(&(info->tr),sizeof(info->tr),1,fid) != 1) ||
      ((err = wr_file_strs(fid,&(info->te),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = wr_file_strs(fid,&(info->date_of_study),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = wr_file_strs(fid,&(info->time_of_study),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = wr_patient_rec(fid,&(info->patient))) != NO_ERROR) ||
      ((err = wr_file_strs(fid,&(info->refered_by),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = wr_file_strs(fid,&(info->description),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = wr_file_strs(fid,&(info->hosp_name),(unsigned int) 1))
       != NO_ERROR))
    return err;
  PRT(printf("WrDataInfoRec...num_scans = %d\n",info->num_scans));

  return NO_ERROR;
}



/*-----------------------------------------------------------------------
wr_analysis_group - ???
-----------------------------------------------------------------------*/

int
wr_analysis_group(fid,group,num_groups)
     FILE *fid;
     struct analysis_group_rec *group;
     unsigned int num_groups;
{
  unsigned int lcv;

  for (lcv = 0; lcv < num_groups; lcv++)
    if ((fwrite(&(group[lcv].num_members),sizeof(group[lcv].num_members),1,
		fid) != 1) ||
        (wr_file_ints(fid,group[lcv].member_img_label_indx,
		      group[lcv].num_members) != NO_ERROR))
      return WRITE_ERROR;
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
wr_analysis_rec - ???
-----------------------------------------------------------------------*/

int
wr_analysis_rec(fid,analysis)
     FILE *fid;
     struct hdr_analysis_rec *analysis;
{
  if ((fwrite(&(analysis->num_img_labels),sizeof(analysis->num_img_labels),1,fid)
       != 1) ||
      (wr_file_strs(fid,analysis->img_labels,analysis->num_img_labels) != 0) ||
      (fwrite(&(analysis->num_analysis_groups),
	      sizeof(analysis->num_analysis_groups),1,fid) != 1) ||
      (wr_analysis_group(fid,analysis->groups,analysis->num_analysis_groups)
       != 0))
    return WRITE_ERROR;
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
wr_slice_entries - ???
-----------------------------------------------------------------------*/

int
wr_slice_entries(fid,scans,num_scans)
     FILE *fid;
     struct hdr_scan_entry_rec *scans;
     unsigned int num_scans;
{
  unsigned int lcv;

  PRT(printf("wr_slice_entries...\n"));
  slice_entry_addr_to_index(scans,num_scans);
  for (lcv = 0; lcv < num_scans; lcv++)
    {if ((fwrite(&(scans[lcv].status),sizeof(scans[lcv].status),1,
		 fid) != 1) ||
         (wr_file_strs(fid,&(scans[lcv].img_label),(unsigned int) 1) != 
	  NO_ERROR) ||
         (fwrite(&(scans[lcv].num),sizeof(scans[lcv].num),1,fid) != 1) ||
         (fwrite(&(scans[lcv].img_addr),
                 sizeof(scans[lcv].img_addr),1,fid) != 1) ||
         (fwrite(&(scans[lcv].plane.vector),sizeof(scans[lcv].plane.vector),
		 1,fid) != 1) ||
         (fwrite(&(scans[lcv].plane.constant),
		 sizeof(scans[lcv].plane.constant),1,fid) != 1) ||
         (fwrite(&(scans[lcv].prior_slice),sizeof(scans[lcv].prior_slice),
		 1,fid) != 1) ||
         (fwrite(&(scans[lcv].next_slice),sizeof(scans[lcv].next_slice),
		 1,fid) != 1) ||
         (fwrite(&(scans[lcv].partner_slice),sizeof(scans[lcv].partner_slice),
		 1,fid) != 1))
       {slice_entry_index_to_addr(scans,num_scans);
        return WRITE_ERROR;}
   }
  slice_entry_index_to_addr(scans,num_scans);
  return NO_ERROR;

}



/*-----------------------------------------------------------------------
rd_extra_file_info_rec - ???
-----------------------------------------------------------------------*/

int
rd_extra_file_info_rec(fid,extra_info)
     FILE *fid;
     struct extra_file_info_rec *extra_info;
{
  int err;

  PRT(printf("rd_extra_file_info_rec...\n"));
  err = READ_ERROR;
  if (((err = rd_file_strs(fid,&(extra_info->color_bundle_label),1))
       != NO_ERROR) ||
      (fread(&(extra_info->color_control),
	     sizeof(extra_info->color_control),1,fid) != 1) ||
      ((err = rd_file_strs(fid,&(extra_info->last_edit_begin),1)) != NO_ERROR) ||
      ((err = rd_file_strs(fid,&(extra_info->last_edit_end),1)) != NO_ERROR))
    return err;
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
rd_patient_rec - ???
-----------------------------------------------------------------------*/

int
rd_patient_rec(fid,patient)
     FILE *fid;
     struct patient_rec *patient;
{
  int err;

  err = READ_ERROR;
  if (((err = rd_file_strs(fid,&(patient->name),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = rd_file_strs(fid,&(patient->ssnum),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = rd_file_strs(fid,&(patient->patient_id),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = rd_file_strs(fid,&(patient->study_num),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = rd_file_strs(fid,&(patient->patient_num),(unsigned int) 1))
       != NO_ERROR) ||
      (fread(&(patient->sex),sizeof(patient->sex),1,fid) != 1) ||
      (fread(&(patient->age),sizeof(patient->age),1,fid) != 1) ||
      (fread(&(patient->height),sizeof(patient->height),1,fid) != 1) ||
      (fread(&(patient->weight),sizeof(patient->weight),1,fid) != 1))
    return err;
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
RdDataInfoRec - ???
-----------------------------------------------------------------------*/

int
RdDataInfoRec(fid,info,verid)
     FILE *fid;
     struct data_info_rec *info;
     char *verid;
{
  int err;
  unsigned long min,max,temp;
  
  PRT(printf("RdDataInfoRec...verid = %s\n",verid));
  err = READ_ERROR;
  if (((err = rd_file_strs(fid,&(info->image_id),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = rd_file_strs(fid,&(info->default_datafn),(unsigned int) 1))
       != NO_ERROR) ||
      (fread(&(info->num_scans),sizeof(info->num_scans),1,fid) != 1) ||
      (fread(&(info->dimx),sizeof(info->dimx),1,fid) != 1) ||
      (fread(&(info->dimy),sizeof(info->dimy),1,fid) != 1) ||
      (fread(&(info->max_scans),sizeof(info->max_scans),1,fid) != 1))
    return err;
  
  if (str_cmp(verid,HDR_ID_V2) == 0)
    {if ((fread(&(info->min_pixel),sizeof(info->min_pixel),1,fid) != 1) ||
	 (fread(&(info->max_pixel),sizeof(info->max_pixel),1,fid) != 1) ||
	 (fread(&(info->unused1),sizeof(info->unused1),1,fid) != 1))
       return err;
     }
  else
    {PRT(printf("RdDataInfoRec  old\n"));
     if ((fread(&max,sizeof(max),1,fid) != 1) ||
         (fread(&min,sizeof(min),1,fid) != 1) ||
         (fread(&temp,sizeof(temp),1,fid) != 1) ||
         (fread(&temp,sizeof(temp),1,fid) != 1) ||
         (fread(&temp,sizeof(temp),1,fid) != 1))
       return err;
     info->min_pixel = (double) min;
     info->max_pixel = (double) max;}
  PRT(printf("RdDataInfoRec  1\n"));
  if ((fread(&(info->pixel_depth_bits),sizeof(info->pixel_depth_bits),1,fid)
       != 1) ||
      (fread(&(info->xpixel_size),sizeof(info->xpixel_size),1,fid) != 1) ||
      (fread(&(info->ypixel_size),sizeof(info->ypixel_size),1,fid) != 1) ||
      (fread(&(info->pixel_thickness),sizeof(info->pixel_thickness),1,fid)
       != 1) ||
      (fread(&(info->spacing),sizeof(info->spacing),1,fid) != 1) ||
      (fread(&(info->plane_type),sizeof(info->plane_type),1,fid) != 1) ||
      (fread(info->trans_matrix,sizeof(info->trans_matrix),1,fid) != 1) ||
      (fread(&(info->unused2),sizeof(info->unused2),1,fid) != 1) ||
      (fread(&(info->var_use_status),sizeof(info->var_use_status),1,fid) != 1) ||
      (fread(&(info->num_echoes),sizeof(info->num_echoes),1,fid) != 1) ||
      (fread(&(info->unused3),sizeof(info->unused3),1,fid) != 1) ||
      (fread(&(info->unused4),sizeof(info->unused4),1,fid) != 1) ||
      (fread(&(info->unused5),sizeof(info->unused5),1,fid) != 1) ||
      (fread(&(info->unused6),sizeof(info->unused6),1,fid) != 1) ||
      (fread(&(info->unused7),sizeof(info->unused7),1,fid) != 1) ||
      (fread(&(info->tr),sizeof(info->tr),1,fid) != 1) ||
      ((err = rd_file_strs(fid,&(info->te),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = rd_file_strs(fid,&(info->date_of_study),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = rd_file_strs(fid,&(info->time_of_study),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = rd_patient_rec(fid,&(info->patient))) != NO_ERROR) ||
      ((err = rd_file_strs(fid,&(info->refered_by),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = rd_file_strs(fid,&(info->description),(unsigned int) 1))
       != NO_ERROR) ||
      ((err = rd_file_strs(fid,&(info->hosp_name),(unsigned int) 1))
       != NO_ERROR))
    return err;
  PRT(printf("RdDataInfoRec...num_scans = %d\n",info->num_scans));
  PRT(printf("RdDataInfoRec...end\n"));
  
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
rd_analysis_group - ???
-----------------------------------------------------------------------*/

int
rd_analysis_group(fid,group,num_groups)
     FILE *fid;
     struct analysis_group_rec **group;
     unsigned int num_groups;
{
  unsigned int lcv;
  int err;
  
  if ((err = get_ptr(group,NULL,(size_t) num_groups,
		     (size_t) sizeof(**group))) != NO_ERROR)
    return err;

  for (lcv = 0; lcv < num_groups; lcv++)
    {if (fread(&((*group)[lcv].num_members),sizeof((*group)[lcv].num_members),1,
	       fid) != 1)
       return READ_ERROR;

     if ((err = get_ptr(&((*group)[lcv].member_img_label_indx),NULL,
			(size_t) (*group)[lcv].num_members,
			(size_t) sizeof(*((*group)[lcv].member_img_label_indx)))) 
	 != NO_ERROR)
       return err;

     if (rd_file_ints(fid,(*group)[lcv].member_img_label_indx,
		      (*group)[lcv].num_members) != NO_ERROR)
       return READ_ERROR;
   }
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
rd_analysis_rec - ???
-----------------------------------------------------------------------*/

int
rd_analysis_rec(fid,analysis)
     FILE *fid;
     struct hdr_analysis_rec *analysis;
{
  if ((fread(&(analysis->num_img_labels),sizeof(analysis->num_img_labels),1,fid)
       != 1) ||
      (rd_file_labels(fid,&(analysis->img_labels),
		      analysis->num_img_labels) != NO_ERROR) ||
      (fread(&(analysis->num_analysis_groups),
	     sizeof(analysis->num_analysis_groups),1,fid) != 1) ||
      (rd_analysis_group(fid,&(analysis->groups),analysis->num_analysis_groups)
       != NO_ERROR))
    return READ_ERROR;
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
RdSliceEntries - ???
-----------------------------------------------------------------------*/

int
RdSliceEntries(fid,scans,num_scans,hdr)
     FILE *fid;
     struct hdr_scan_entry_rec **scans;
     unsigned int *num_scans;
     struct hdr_rec *hdr;
{
  unsigned int lcv,num_cuts,max_scans;
  struct hdr_scan_entry_rec scan;
  int err;

  num_cuts = NumberOfSliceCuts(&(hdr->data_info));
  max_scans = NumberOfSliceScans(&(hdr->data_info));
  hdr->data_info.max_scans = num_cuts * hdr->data_info.num_echoes * max_scans;    
  if ((err = get_ptr(scans,NULL,(size_t) hdr->data_info.max_scans,
		     (size_t) sizeof(struct hdr_scan_entry_rec))) != NO_ERROR)
    {*num_scans = 0;
     return NOT_ENOUGH_MEMORY;}
  for (lcv = 0; lcv < hdr->data_info.max_scans; lcv++)
    init_hdr_scan_entry_rec(&((*scans)[lcv]));

  for (lcv = 0; lcv < *num_scans; lcv++)
    {init_hdr_scan_entry_rec(&scan);
     if ((fread(&(scan.status),sizeof(scan.status),1,fid) != 1) ||
         (rd_file_strs(fid,&(scan.img_label),(unsigned int) 1) != NO_ERROR) ||
         (fread(&(scan.num),sizeof(scan.num),1,fid) != 1) ||
         (fread(&(scan.img_addr),sizeof(scan.img_addr),1,fid) != 1) ||
         (fread(&(scan.plane.vector),
		sizeof(scan.plane.vector),1,fid) != 1) ||
         (fread(&(scan.plane.constant),sizeof(
					      scan.plane.constant),1,fid) != 1) ||
         (fread(&(scan.prior_slice),
		sizeof(scan.prior_slice),1,fid) != 1) ||
         (fread(&(scan.next_slice),sizeof(scan.next_slice),1,fid) != 1) ||
         (fread(&(scan.partner_slice),sizeof(scan.partner_slice),
		1,fid) != 1))
       {free_hdr_scan_entry_rec(&scan);
	return READ_ERROR;}
     if (lcv < hdr->data_info.max_scans)
       (*scans)[lcv] = scan;
     else free_hdr_scan_entry_rec(&scan);
   }
  *num_scans = hdr->data_info.max_scans;
  slice_entry_index_to_addr(*scans,*num_scans);
  return NO_ERROR;

}



/*-----------------------------------------------------------------------
rd_hdr - ???
-----------------------------------------------------------------------*/

int
rd_hdr(fid,hdr)
     FILE *fid;
     struct hdr_rec *hdr;
{
  size_t amt;
  int err;

  fseek(fid,0L,0);
  err = READ_ERROR;
  if ((fread(&(hdr->size),sizeof(hdr->size),1,fid) != 1) ||
      ((err = rd_file_strs(fid,&(hdr->id),1)) != NO_ERROR) ||
      (fread(&(hdr->offset),sizeof(hdr->offset),1,fid) != 1))
    {if (err == NO_ERROR) err = READ_ERROR;
     return err;}

  fseek(fid,hdr->offset.info_start,0);
  PRT(printf("rd..hdr->offset.info_start = %d hdr->offset.info_size = %d\n",hdr->offset.info_start,hdr->offset.info_size));
  err = READ_ERROR;

  if (fread(&(hdr->status),sizeof(hdr->status),1,fid) != 1)
    {
      if (err == NO_ERROR) err = READ_ERROR;
      return err;}
  if (((err = RdDataInfoRec(fid,&(hdr->data_info),hdr->id)) != NO_ERROR) ||
      ((err = rd_analysis_rec(fid,&(hdr->analysis))) != NO_ERROR) ||
      (fread(&(hdr->num_scans),sizeof(hdr->num_scans),1,fid) != 1) ||
      ((err = RdSliceEntries(fid,&(hdr->scans),&(hdr->num_scans),hdr))
       != NO_ERROR))
    {
      if (err == NO_ERROR) err = READ_ERROR;
      return err;}


  fseek(fid,hdr->offset.extra_info_start,0);
  PRT(printf("hdr->offset.extra_info_start = %d hdr->offset.extra_info_size = %d\n",hdr->offset.extra_info_start,hdr->offset.extra_info_size));
  if (hdr->offset.extra_info_size > 0)
    if ((err = rd_extra_file_info_rec(fid,&(hdr->extra_info))) != NO_ERROR)
      return err;

  fseek(fid,hdr->offset.comment_start,0);
  err = READ_ERROR;
  if (fread(&(hdr->comment_size),sizeof(hdr->comment_size),1,fid) != 1)
    {if (err == NO_ERROR) err = READ_ERROR;
     return err;}
  if (hdr->comment_size > 0)
    {
      if ((err = get_ptr(&(hdr->comment),NULL,(size_t) hdr->comment_size,
			 (size_t) sizeof(char))) != NO_ERROR)
        return err;
      if ((err = frd(hdr->comment,hdr->comment_size,1,fid,&amt)) != NO_ERROR)
        {if (err == NO_ERROR) err = READ_ERROR;
         return err;
       }
    }

  PRT(printf("hdr->comment_size = %u hdr->offset.user_defined_start = %u\n",hdr->comment_size,hdr->offset.user_defined_start));

  fseek(fid,hdr->offset.user_defined_start,0);
  if (fread(&(hdr->user_defined_size),sizeof(hdr->user_defined_size),1,fid) != 1)
    {if (err == NO_ERROR) err = READ_ERROR;
     return err;
   }
  PRT(printf("rd user_defined...size = %u\n",hdr->user_defined_size));
  if (hdr->user_defined_size > 0)
    {
      if ((err = get_ptr(&(hdr->user_defined),NULL,
			 (size_t) hdr->user_defined_size,(size_t) sizeof(char))) != NO_ERROR)
        return err;
      if ((err = frd(hdr->user_defined,hdr->user_defined_size,1,fid,&amt))
	  != NO_ERROR)
        {if (err == NO_ERROR) err = READ_ERROR;
         return err;
       }
    }
  if (hdr->data_info.min_pixel >= hdr->data_info.max_pixel)
    hdr->status |= GLOBAL_MIN_MAX_UNDEFINED;

#if 0
  if (str_cmp(hdr->id,HDR_ID_V2) != 0)
    {c_free(hdr->id);
     if ((err = cp_str(&(hdr->id),HDR_ID_V2)) != NO_ERROR)
       return err;
   }
#endif

  PRT(printf("rd hdr end...\n"));
  return NO_ERROR;

}



/*-----------------------------------------------------------------------
rd_the_hdr - ???
-----------------------------------------------------------------------*/

int
rd_the_hdr(fn,hdr)
     char *fn;
     struct hdr_rec *hdr;
{
  FILE *fid;
  int err;

  if ((fid = fopen(fn,"r")) == NULL)
    {PRT(printf("\nCould not open output file: %s\n",fn));
     return FILE_ACCESS_ERROR;}
  if ((err = rd_hdr(fid,hdr)) == NO_ERROR)
    ComputeScanEntryExtraInfo(hdr);

  PRT(printf("rd_the_hdr err = %d\n",err));
  fclose(fid);
  return err;
}



/*-----------------------------------------------------------------------
rd_ria_hdr_params - ???
-----------------------------------------------------------------------*/

int
rd_ria_hdr_params(fn,dimx,dimy,num_slices,num_echoes,
                  xpixel_size,ypixel_size,zpixel_size,
                  data_type,pixel_depth,min_pixel,max_pixel)
     char *fn;
     unsigned long *dimx,*dimy,*num_slices,*num_echoes,*data_type,*pixel_depth;
     float *xpixel_size,*ypixel_size,*zpixel_size;
     double *min_pixel,*max_pixel;
{
  struct hdr_rec hdr;
  int err;

  init_hdr_rec(&hdr);
  if ((err = rd_the_hdr(fn,&hdr)) != NO_ERROR)
    {free_hdr_rec(&hdr);
     return err;}
  *dimx = hdr.data_info.dimx;
  *dimy = hdr.data_info.dimy;
  *num_slices = hdr.data_info.max_scans;
  *num_echoes = hdr.data_info.num_echoes;
  *xpixel_size = hdr.data_info.xpixel_size;
  *ypixel_size = hdr.data_info.ypixel_size;
  *zpixel_size = hdr.data_info.pixel_thickness;
  if (hdr.data_info.var_use_status & INPUT_DATA_SIGNED)
    *data_type = 1;
  else if (hdr.data_info.var_use_status & INPUT_DATA_FLOAT)
    *data_type = 2;
  else if (hdr.data_info.var_use_status & INPUT_DATA_DOUBLE)
    *data_type = 3;
  else *data_type = 0;
  *pixel_depth = hdr.data_info.pixel_depth_bits;
  *min_pixel = hdr.data_info.min_pixel;
  *max_pixel = hdr.data_info.max_pixel;
  free_hdr_rec(&hdr);

  return NO_ERROR;
}



/*-----------------------------------------------------------------------
NumberOfSliceScans - ???
-----------------------------------------------------------------------*/

unsigned long 
NumberOfSliceScans(info)
     struct data_info_rec *info;
{
  if (info->var_use_status & NUM_SCANS_USED)
    return info->num_scans;
  else return 1;
}


/*-----------------------------------------------------------------------
NumberOfSliceCuts - ???

 PWH 19980430: Modified the code below to interpret 0/0 as 0 
   rather than throw a SIGFPE.  I don't know if this will break
   anything else.
-----------------------------------------------------------------------*/

unsigned long 
NumberOfSliceCuts(info)
     struct data_info_rec *info;
{
  if (info->max_scans == 0) 
    return 0;
  else if (info->var_use_status & NUM_SCANS_USED)
    return info->max_scans / (info->num_echoes *
			      info->num_scans);
  else return info->max_scans / info->num_echoes;
}



/*-----------------------------------------------------------------------
ValidateHdrParams - ???
-----------------------------------------------------------------------*/

int
ValidateHdrParams(hdr,maxdimx,maxdimy,maxscans,maxechoes)
     struct hdr_rec *hdr;
     unsigned long maxdimx,maxdimy,maxscans,maxechoes;
{
  struct data_info_rec *info;

  info = &(hdr->data_info);
  PRT(printf("ValidateHdrParams...info->pixel_depth_bits = %d\n",
	     info->pixel_depth_bits));
  if ((info->dimx == 0) || (info->dimy == 0) ||
      (info->max_scans == 0) || (info->pixel_depth_bits == 0) ||
      (info->num_echoes == 0))
    return 1;
  if ((info->dimx > maxdimx) || (info->dimy > maxdimy) ||
      (info->max_scans > maxscans) || (info->pixel_depth_bits > 32) ||
      (info->num_echoes > maxechoes))
    return 2;
  if (hdr->num_scans != info->max_scans)
    return 3;
  if (hdr->analysis.num_img_labels != info->num_echoes)
    return 4;
  return NO_ERROR;
}


#if 0   /******************/
CurSliceHeadLast(hdr,head,last,cur_slice)
     struct hdr_rec *hdr;
     struct hdr_scan_entry_rec *head,*last,*cur_slice;
{
  unsigned long num_scans;

  num_scans = NumberOfSliceScans(&(hdr->data_info));
}


ScanEchoNum(hdr,scan,echo_num)
     struct hdr_rec *hdr;
     struct hdr_scan_entry_rec *scan;
     unsigned long *echo_num;
{
  unsigned long indx;

  *echo_num = 0;
  if (hdr->scans > scan) return ~NO_ERROR;
  indx = scan - hdr->scans;
  printf("ScanEchoNum indx = %d scan = %u hdr->scans = %u\n",indx,scan,hdr->scans);
  if (indx >= hdr->data_info.max_scans) return ~NO_ERROR;
  *echo_num = indx % hdr->data_info.num_echoes;
  return NO_ERROR;
}
#endif  /*************************/



/*-----------------------------------------------------------------------
LocateScanEntry - ???
-----------------------------------------------------------------------*/

int
LocateScanEntry(hdr,slice_num,scan_num,echo_num,scan_entry)
     struct hdr_rec *hdr;
     unsigned long slice_num,scan_num,echo_num;
     struct hdr_scan_entry_rec **scan_entry;
{
  unsigned long num_scans,num_slices,indx;

  PRT(printf("\nslice_num = %d scan_num = %d echo_num = %d\n",
	     slice_num,scan_num,echo_num));
  if (echo_num >= hdr->data_info.num_echoes) return ~NO_ERROR;
  num_scans = NumberOfSliceScans(&(hdr->data_info));
  num_slices = NumberOfSliceCuts(&(hdr->data_info));
  if (scan_num >= num_scans) return ~NO_ERROR;
  if (slice_num >= num_slices) return ~NO_ERROR;
  indx = (scan_num * num_slices + slice_num) *
    hdr->data_info.num_echoes + echo_num;
  PRT(printf("indx = %d num_scans = %d\n",
	     indx,hdr->num_scans));
  if (indx >= hdr->num_scans) return ~NO_ERROR;
  *scan_entry = &(hdr->scans[indx]);
  PRT(printf("final addr = %u num = %d snum = %d scan = %d echo = %d\n\n",
	     (*scan_entry)->img_addr,
	     (*scan_entry)->num,(*scan_entry)->slice_num,
	     (*scan_entry)->scan_num,(unsigned long)(*scan_entry)->echo_num));
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
FileHdrScan - ???
-----------------------------------------------------------------------*/

int
FileHdrScan(hdr,scan,indx)
     struct hdr_rec *hdr;
     struct hdr_scan_entry_rec *scan;
     unsigned long *indx;
{
  if (hdr->scans > scan) return ~NO_ERROR;
  *indx = scan - hdr->scans;
  if (*indx >= hdr->data_info.max_scans) return ~NO_ERROR;
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
ComputeScanEntryExtraInfo - ???
-----------------------------------------------------------------------*/

void
ComputeScanEntryExtraInfo(hdr)
     struct hdr_rec *hdr;
{
  unsigned long num_slices,cnt,num;

  PRT(printf("ComputeScanEntryExtraInfo...\n"));
  num_slices = NumberOfSliceCuts(&(hdr->data_info));
  PRT(printf("ComputeScanEntryExtraInfo...num_slices = %d num_scans = %d\n",
	     num_slices,hdr->num_scans));
  PRT(printf("ComputeScanEntryExtraInfo...num_echoes = %d\n",
	     hdr->data_info.num_echoes));
  for (cnt = 0; cnt < hdr->num_scans; cnt++)
    {num = cnt / hdr->data_info.num_echoes;
     hdr->scans[cnt].echo_num = cnt % hdr->data_info.num_echoes;
     hdr->scans[cnt].slice_num = num % num_slices;
     hdr->scans[cnt].scan_num = num / num_slices;
     PRT(printf("%d) echo_num = %d  slice_num = %d  scan_num = %d\n",
		cnt,(unsigned long) hdr->scans[cnt].echo_num,
		hdr->scans[cnt].slice_num,hdr->scans[cnt].scan_num));

   }
}



/*-----------------------------------------------------------------------
RedefineHdrNumBySliceNum - ???
-----------------------------------------------------------------------*/

int
RedefineHdrNumBySliceNum(hdr)
     struct hdr_rec *hdr;
{
  unsigned long cnt;

  for (cnt = 0; cnt < hdr->num_scans; cnt++)
    hdr->scans[cnt].num = hdr->scans[cnt].slice_num + 1;
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
ReverseHdrScans - ???
-----------------------------------------------------------------------*/

int
ReverseHdrScans(hdr)
     struct hdr_rec *hdr;
{
  unsigned long num_slices,num_scans,off,num_group_slices,lcv,
  cnt,start_last,start_next,last,next,indx,indx1;
  struct hdr_scan_entry_rec scan;

  if (hdr->num_scans == 1) return NO_ERROR;
  num_slices = NumberOfSliceCuts(&(hdr->data_info));
  num_scans = NumberOfSliceScans(&(hdr->data_info));
  if (num_slices % 2)
    {start_next = (num_slices + 1) / 2;
     start_last = start_next - 2;}
  else
    {start_next = num_slices / 2;
     start_last = start_next - 1;}
  off = 0;
  num_group_slices = num_slices * hdr->data_info.num_echoes;
  for (cnt = 0; cnt < num_scans; cnt++)
    {
      next = start_next;
      last = start_last;
      for (;;)
        {indx = next * hdr->data_info.num_echoes + off;
         indx1 = last * hdr->data_info.num_echoes + off;
         for (lcv = 0; lcv < hdr->data_info.num_echoes; lcv++)
	   {scan = hdr->scans[indx + lcv];
	    hdr->scans[indx + lcv] = hdr->scans[indx1 + lcv];
	    hdr->scans[indx1 + lcv] = scan;
	  }
         if (last == 0) break;
         last -= 1;
         next += 1;
       }
      off += num_group_slices;
    }
  for (off = 0,cnt = 0; cnt < num_scans; cnt++)
    {last = off + num_group_slices - hdr->data_info.num_echoes;
     indx = off;
     indx1 = indx + hdr->data_info.num_echoes;
     for (next = 0; next < num_slices; next++)
       {
         for (lcv = 0; lcv < hdr->data_info.num_echoes; lcv++)
	   {hdr->scans[indx + lcv].next_slice = &(hdr->scans[indx1 + lcv]);
	    hdr->scans[indx + lcv].prior_slice = &(hdr->scans[last + lcv]);
	    hdr->scans[indx + lcv].partner_slice = 
	      &(hdr->scans[indx + (lcv + 1) % hdr->data_info.num_echoes]);
	  }
         last = indx;
         indx = indx1;
         if (next == (num_slices - 2))
	   indx1 = off;
	 else indx1 += hdr->data_info.num_echoes;
       }
     off += num_group_slices;
   }
  PRT(for (cnt = 0; cnt < hdr->num_scans; cnt++)
      printf("%d) s# = %d e# = %d sc# = %d n# = %d  ns# = %d\n",cnt,
	     hdr->scans[cnt].slice_num,hdr->scans[cnt].echo_num,
	     hdr->scans[cnt].scan_num,hdr->scans[cnt].num,
	     hdr->scans[cnt].next_slice->slice_num););

  /*******
     for (cnt = 0; cnt < num_scans; cnt++)
     {for (lcv = 0; lcv < hdr->data_info.num_echoes; lcv++)
     {next = start_next;
     last = start_last;
     for (;;)
     {indx = next * hdr->data_info.num_echoes + lcv + off;
     scan = hdr->scans[indx];
     indx1 = last * hdr->data_info.num_echoes + lcv + off;
     hdr->scans[indx] = hdr->scans[indx1];
     hdr->scans[indx1] = scan;
     if (last == 0) break;
     last -= 1;
     next += 1;
     }
     }
     off += num_group_slices;
     }
  *******/
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
SetHdrScanEntryStatus - ???

   set_select = 1 then scan_entry->status = status
   set_select = 2 then scan_entry->status |= status
   set_select = 3 then scan_entry->status &= ~status
   else no operation
-----------------------------------------------------------------------*/

unsigned long 
SetHdrScanEntryStatus(hdr,fsnum,lsnum,fecho,
				    lecho,fscan,lscan,set_select,status)
     struct hdr_rec *hdr;
     unsigned long fsnum,lsnum,fecho,lecho,fscan,lscan;
     int set_select;
     unsigned short status;
{
  unsigned long lcv,cnt,num,total;
  struct hdr_scan_entry_rec *scan_entry;

  total = 0;
  for (lcv = fsnum; lcv <= lsnum; lcv++)
    for (cnt = fecho; cnt <= lecho; cnt++)
      for (num = fscan; num <= lscan; num++)
	if (LocateScanEntry(hdr,lcv,num,cnt,&scan_entry) == NO_ERROR)
	  {total += 1;
	   if (set_select == 1) scan_entry->status = status;
	   else if (set_select == 2) scan_entry->status |= status;
	   else if (set_select == 3) scan_entry->status &= ~status;
	 }
  return total;
}



/*-----------------------------------------------------------------------
cp_patient_rec - ???
-----------------------------------------------------------------------*/

int
cp_patient_rec(dest,src,is_dest_new)
     struct patient_rec *dest,*src;
     int is_dest_new;
{
  struct patient_rec temp,*pat;
  int err;

  PRT(printf("cp_patient_rec 0\n"));
  if (!is_dest_new)
    {init_patient_rec(&temp);
     pat = &temp;}
  else pat = dest;
  pat->sex = src->sex;
  pat->age = src->age;
  pat->height = src->height;
  pat->weight = src->weight;
  if (((err = cp_str(&(pat->name),src->name)) != NO_ERROR) ||
      ((err = cp_str(&(pat->ssnum),src->ssnum)) != NO_ERROR) ||
      ((err = cp_str(&(pat->patient_id),src->patient_id)) != NO_ERROR) ||
      ((err = cp_str(&(pat->study_num),src->study_num)) != NO_ERROR) ||
      ((err = cp_str(&(pat->patient_num),src->patient_num)) != NO_ERROR))
    {if (!is_dest_new) free_patient_rec(&temp);
     return err;}
  if (!is_dest_new)
    {free_patient_rec(dest);
     *dest = temp;}
  PRT(printf("cp_patient_rec end\n"));
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
cp_data_info_rec - ???
-----------------------------------------------------------------------*/

int
cp_data_info_rec(dest,src,is_dest_new)
     struct data_info_rec *dest,*src;
     int is_dest_new;
{
  struct data_info_rec temp,*info;
  int err;

  PRT(printf("cp_data_info_rec 0\n"));
  if (!is_dest_new)
    {init_data_info_rec(&temp);
     info = &temp;}
  else info = dest;
  *info = *src;
  info->image_id = info->default_datafn = info->te = NULL;
  info->date_of_study = info->time_of_study = NULL;
  info->refered_by = info->description = info->hosp_name = NULL;
  init_patient_rec(&(info->patient));
  if (((err = cp_str(&(info->image_id),src->image_id)) != NO_ERROR) ||
      ((err = cp_str(&(info->default_datafn),src->default_datafn)) != NO_ERROR) ||
      ((err = cp_str(&(info->te),src->te)) != NO_ERROR) ||
      ((err = cp_str(&(info->date_of_study),src->date_of_study)) != NO_ERROR) ||
      ((err = cp_str(&(info->time_of_study),src->time_of_study)) != NO_ERROR) ||
      ((err = cp_patient_rec(&(info->patient),&(src->patient),is_dest_new)) != NO_ERROR) ||
      ((err = cp_str(&(info->refered_by),src->refered_by)) != NO_ERROR) ||
      ((err = cp_str(&(info->description),src->description)) != NO_ERROR) ||
      ((err = cp_str(&(info->hosp_name),src->hosp_name)) != NO_ERROR))
    {if (!is_dest_new) free_data_info_rec(&temp);
     return err;}
  if (!is_dest_new)
    {free_data_info_rec(dest);
     *dest = temp;}
  PRT(printf("cp_data_info_rec end\n"));
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
cp_analysis_group_rec - ???
-----------------------------------------------------------------------*/

int
cp_analysis_group_rec(dest,src,is_dest_new)
     struct analysis_group_rec *dest,*src;
     int is_dest_new;
{
  struct analysis_group_rec temp,*anal;
  int err;
  unsigned long lcv;

  PRT(printf("cp_analysis_group_rec 0\n"));
  if (!is_dest_new)
    {init_analysis_group_rec(&temp);
     anal = &temp;}
  else anal = dest;

  anal->num_members = 0;
  if ((err = get_ptr(&(anal->member_img_label_indx),&(anal->num_members),
		     src->num_members,(size_t) sizeof(*(anal->member_img_label_indx)))) != NO_ERROR)
    {return err;}
  for (lcv = 0; lcv < anal->num_members; lcv++)
    anal->member_img_label_indx[lcv] = src->member_img_label_indx[lcv];

  /* FIXME!  The argument 0 below is almost certainly not correct, although
     it should be safe except that it leaks memory.  -PWH 19990311 */
  if (!is_dest_new)
    {free_analysis_group_rec(dest,0);
     *dest = temp;}
  PRT(printf("cp_analysis_group_rec end\n"));
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
cp_hdr_analysis_rec - ???
-----------------------------------------------------------------------*/

int
cp_hdr_analysis_rec(dest,src,is_dest_new)
     struct hdr_analysis_rec *dest,*src;
     int is_dest_new;
{
  struct hdr_analysis_rec temp,*anal;
  int err;
  unsigned long lcv;

  PRT(printf("cp_hdr_analysis_rec 0\n"));
  if (!is_dest_new)
    {init_hdr_analysis_rec(&temp);
     anal = &temp;}
  else anal = dest;

  anal->num_img_labels = 0;
  anal->num_analysis_groups = 0;
  if (((err = copy_strs(&(anal->img_labels),&(anal->num_img_labels),
			src->img_labels,src->num_img_labels)) != NO_ERROR) ||
      ((err = get_ptr(&(anal->groups),&(anal->num_analysis_groups),
		      src->num_analysis_groups,(size_t) sizeof(*(anal->groups)))) != NO_ERROR))
    {if (!is_dest_new) free_hdr_analysis_rec(&temp);
     return err;}
  PRT(printf("cp_hdr_analysis_rec 1\n"));
  for (lcv = 0; lcv < anal->num_analysis_groups; lcv++)
    init_analysis_group_rec(&(anal->groups[lcv]));

  for (lcv = 0; lcv < anal->num_analysis_groups; lcv++)
    if ((err = cp_analysis_group_rec(&(anal->groups[lcv]),
				     &(src->groups[lcv]),is_dest_new)) != NO_ERROR)
      {if (!is_dest_new) free_hdr_analysis_rec(&temp);
       return err;}

  if (!is_dest_new)
    {free_hdr_analysis_rec(dest);
     *dest = temp;}
  PRT(printf("cp_hdr_analysis_rec end\n"));
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
cp_extra_file_info_rec - ???
-----------------------------------------------------------------------*/

int
cp_extra_file_info_rec(dest,src,is_dest_new)
     struct extra_file_info_rec *dest,*src;
     int is_dest_new;
{
  struct extra_file_info_rec temp,*ptr;
  int err;

  PRT(printf("cp_extra_file_info_rec 0\n"));
  if (!is_dest_new)
    {init_extra_file_info_rec(&temp);
     ptr = &temp;}
  else ptr = dest;

  if (((err = cp_str(&(ptr->color_bundle_label),src->color_bundle_label)) != NO_ERROR) ||
      ((err = cp_str(&(ptr->last_edit_begin),src->last_edit_begin)) != NO_ERROR) ||
      ((err = cp_str(&(ptr->last_edit_end),src->last_edit_end)) != NO_ERROR))
    {if (!is_dest_new) free_extra_file_info_rec(&temp);
     return err;}
  ptr->color_control = src->color_control;
  if (!is_dest_new)
    {free_extra_file_info_rec(dest);
     *dest = temp;}
  PRT(printf("cp_extra_file_info_rec end\n"));
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
cp_hdr_scan_entry_rec - ???
-----------------------------------------------------------------------*/

int
cp_hdr_scan_entry_rec(dest,src,is_dest_new)
     struct hdr_scan_entry_rec *dest,*src;
     int is_dest_new;
{
  struct hdr_scan_entry_rec temp,*ptr;
  int err;

  if (!is_dest_new)
    {init_hdr_scan_entry_rec(&temp);
     ptr = &temp;}
  else ptr = dest;
  *ptr = *src;
  ptr->img_label = NULL;
  if ((err = cp_str(&(ptr->img_label),src->img_label)) != NO_ERROR)
    {return err;}
  if (!is_dest_new)
    {free_hdr_scan_entry_rec(dest);
     *dest = temp;}
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
cp_hdr_rec - ???
-----------------------------------------------------------------------*/

int
cp_hdr_rec(dest,src,is_dest_new)
     struct hdr_rec *dest,*src;
     int is_dest_new;
{
  struct hdr_rec temp,*hdr;
  unsigned long lcv;
  int err;

  PRT(printf("cp_hdr_rec 0\n"));
  if (!is_dest_new)
    {init_hdr_rec(&temp);
     hdr = &temp;}
  else hdr = dest;
  hdr->size = src->size;
  hdr->status = src->status;
  hdr->offset = src->offset;
  hdr->num_scans = 0;
  hdr->comment_size = 0;
  hdr->user_defined_size = 0;
  if (((err = cp_str(&(hdr->id),src->id)) != NO_ERROR) ||
      ((err = cp_data_info_rec(&(hdr->data_info),&(src->data_info),
			       is_dest_new)) != NO_ERROR) ||
      ((err = cp_hdr_analysis_rec(&(hdr->analysis),&(src->analysis),
				  is_dest_new)) != NO_ERROR) ||
      ((err = get_ptr(&(hdr->scans),&(hdr->num_scans),
		      src->num_scans,(size_t) sizeof(*(hdr->scans)))) != NO_ERROR) ||
      ((err = cp_extra_file_info_rec(&(hdr->extra_info),
				     &(src->extra_info),is_dest_new)) != NO_ERROR) ||
      ((err = get_ptr(&(hdr->comment),&(hdr->comment_size),
		      src->comment_size,(size_t) sizeof(*(hdr->comment)))) != NO_ERROR) ||
      ((err = get_ptr(&(hdr->user_defined),&(hdr->user_defined_size),
		      src->user_defined_size,(size_t) sizeof(*(hdr->user_defined)))) != NO_ERROR))
    {if (!is_dest_new) free_hdr_rec(&temp);
     return err;}
  PRT(printf("cp_hdr_rec 1\n"));
  for (lcv = 0; lcv < hdr->num_scans; lcv++)
    init_hdr_scan_entry_rec(&(hdr->scans[lcv]));

  for (lcv = 0; lcv < hdr->num_scans; lcv++)
    if ((err = cp_hdr_scan_entry_rec(&(hdr->scans[lcv]),
				     &(src->scans[lcv]),is_dest_new)) != NO_ERROR)
      {if (is_dest_new) free_hdr_rec(&temp);
       return err;}
  memcpy(hdr->comment,src->comment,hdr->comment_size);
  memcpy(hdr->user_defined,src->user_defined,hdr->user_defined_size);
  if (!is_dest_new)
    {free_hdr_rec(dest);
     *dest = temp;}
  PRT(printf("cp_hdr_rec end\n"));
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
ImageFileSize - ???
-----------------------------------------------------------------------*/

unsigned long 
ImageFileSize(hdr)
     struct hdr_rec *hdr;
{
  unsigned long size;

  size = GetImageSize(hdr->data_info.dimx,hdr->data_info.dimy,
		      hdr->data_info.pixel_depth_bits);
  size *= hdr->data_info.max_scans;
  return size;
}



/*-----------------------------------------------------------------------
VarInputDataTypeToBufType - ???
-----------------------------------------------------------------------*/

unsigned long 
VarInputDataTypeToBufType(var_type)
     unsigned long var_type;
{
  if (var_type & INPUT_DATA_FLOAT)
    return BUF_VALUE_FLOAT;
  else if (var_type & INPUT_DATA_DOUBLE)
    return BUF_VALUE_DOUBLE;
  else if (var_type & INPUT_DATA_SIGNED)
    return BUF_VALUE_SIGNED;
  else return 0;
}



/*-----------------------------------------------------------------------
InputDataTypeToBufType - ???
-----------------------------------------------------------------------*/

unsigned long 
InputDataTypeToBufType(info)
     struct data_info_rec *info;
{
  if (info->var_use_status & INPUT_DATA_TYPE)
    {return VarInputDataTypeToBufType(info->var_use_status);
   }
  else return 0;
} 



/*-----------------------------------------------------------------------
BufTypeToInputDataType - ???
-----------------------------------------------------------------------*/

unsigned long 
BufTypeToInputDataType(buf_type4)
     unsigned long buf_type4;
{
  if (buf_type4 & BUF_VALUE_TYPE)
    {if (buf_type4 & BUF_VALUE_SIGNED)
       return INPUT_DATA_SIGNED;
    else if (buf_type4 & BUF_VALUE_FLOAT)
      return INPUT_DATA_FLOAT;
    else if (buf_type4 & BUF_VALUE_DOUBLE)
      return INPUT_DATA_DOUBLE;
    /* FIXME:  Guessed at following case.  -PWH 19990311 */
    else
	return 0;
     }
  else return 0;
}



/*-----------------------------------------------------------------------
BufTypeToDepth - ???
-----------------------------------------------------------------------*/

unsigned long 
BufTypeToDepth(buf_type4,pixel_depth)
     unsigned long buf_type4,pixel_depth;
{
  if (buf_type4 & BUF_VALUE_TYPE)
    {if (buf_type4 & BUF_VALUE_SIGNED)
	return pixel_depth;
    else if (buf_type4 & BUF_VALUE_FLOAT)
	return sizeof(float) * 8;
    else if (buf_type4 & BUF_VALUE_DOUBLE)
	return sizeof(double) * 8;
    /* FIXME:  Guessed at following case.  -PWH 19990311 */
    else
	return pixel_depth;
     }
  else 
      return pixel_depth;
}
