/****************************************************************************
ptr_and_dim.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.

FIXME:  It is likely that some of the pointer arguments used in this file
are generic pointers and should be declared as void * rather than some other
type.
****************************************************************************/

static const char rcsid[] = "$Id: ptr_and_dim.c,v 1.9 1999/07/07 18:01:07 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>

#include <memory.h>        /* FIXME:  Superseded? */

#include "error_constants.h"
#include "nria_lib.h"

/*
#define TRACE(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
#define PRT3(x) x; fflush(stdout);
*/
#define TRACE(x)
#define PRT(x)
#define PRT3(x)



/*------------------------------------------------------------------------
init_ptr_ary - ???
-------------------------------------------------------------------------*/

void
init_ptr_ary(ptr,num_elems,init_ptr_val)
 void **ptr;
 unsigned long num_elems;
 void *init_ptr_val;
{
 unsigned long cnt;

 for (cnt = 0; cnt < num_elems; cnt++)
   ptr[cnt] = init_ptr_val;

}


/*------------------------------------------------------------------------
deallocate_2d_ary - ???
-------------------------------------------------------------------------*/

void
deallocate_2d_ary(ary,elems)
 char **ary;
 unsigned long elems;
{
 int cnt;

 if (elems > 0)
    {for (cnt = 0; cnt < elems; cnt++) if (ary[cnt] != NULL) free(ary[cnt]);
     free(ary);}

}


/*------------------------------------------------------------------------
get_handle_memory - ???
-------------------------------------------------------------------------*/

unsigned long 
get_handle_memory(ptr,total_items,add_num_items,item_size,err)
 char ***ptr;
 unsigned long *total_items;
 unsigned long add_num_items;
 size_t item_size;
 int *err;
{
 unsigned long cnt,total,num_items;
 int is_new;

 if ((total_items == NULL) || (*total_items == 0))
    {is_new = 1;
     total = add_num_items;}
   else
    {is_new = 0;
     total = *total_items + add_num_items;}

 if (is_new)
    {if (((*ptr) = (char **) malloc((size_t) (total *
                  (size_t) sizeof(char *)) )) == NULL)
        {printf("Not enough memory...get_handle_memory...amt = %lu\n",
                total);
        *err = NOT_ENOUGH_MEMORY;
        return 0;}
     }
   else
    {
    if (((*ptr) = (char **) realloc((*ptr),
      (size_t) total * sizeof(char *))) == NULL)
        {printf("NULL = %d\n",(int) NULL);
         printf("Not enough memory...get_handle_memory.(realloc)..amt = %lu\n",
                total);
         *err = NOT_ENOUGH_MEMORY;
         return 0;}
    }
 if (total_items != NULL)
    {num_items = *total_items;
     cnt = *total_items;}
   else
    num_items = cnt = 0;

 for (; cnt < total; cnt++)
    {if (((*ptr)[cnt] = (char *)
         malloc((size_t) item_size)) == NULL)
        {printf("Not enough memory...get_handle_memory...\n");
         if (cnt > 0)
            for (cnt -= 1; cnt >= num_items; cnt--)
               {free((*ptr)[cnt]);
                (*ptr)[cnt] = NULL;
                if (cnt == 0) break;}
         *err = NOT_ENOUGH_MEMORY;
         return 0;}
    }
 if (total_items != NULL)
    {cnt = *total_items;
     *total_items = total;}
   else cnt = 0;
 *err = NO_ERROR;

 return cnt;
}


/*------------------------------------------------------------------------
get_handle - ???
-------------------------------------------------------------------------*/

int
get_handle(ptr,total_items,add_num_items,item_size)
 char ***ptr;
 unsigned long *total_items;
 unsigned long add_num_items;
 size_t item_size;
{
 int err;

 get_handle_memory(ptr,total_items,add_num_items,item_size,&err);
 return err;
}


/*----------------------------------------------------------------------
get_ptr_memory - Reallocate a dynamic array

This function reallocates an array originally allocated by malloc or
one of its friends.  The argument ptr is a pointer to a cell which points
to the beginning of the array (or NULL if no array has been allocated);
total_items is the count of items already allocated; add_num_items is
the number of items by which the array is to be enlarged (or shrunk);
item_size is the size of each item; and err will be assigned an error
code if necessary.  The returned value is the new item_count.

FIXME: If realloc fails, the pointer to the old array will be
overwritten.
----------------------------------------------------------------------*/

unsigned long 
get_ptr_memory(
    void **ptr,
    unsigned long *total_items,
    unsigned long add_num_items,
    size_t item_size,
    int *err)
{
 unsigned long cnt, total;
 int is_new;

 if ((total_items == NULL) || (*total_items == 0))
    {is_new = 1;
     total = add_num_items;}
 else
    {is_new = 0;
     total = *total_items + add_num_items;}

 if (is_new)
    {PRT3(printf("get_ptr_memory  total = %d item_size = %d\n",
                       total,item_size);)
     if (((*ptr) = malloc((size_t) (total * item_size))) == NULL)
        {printf("Not enough memory...get_ptr_memory requested = %lu\n",
                   total * item_size);
         *err = NOT_ENOUGH_MEMORY;
         return 0;}
     }
 else
    {PRT3(printf("get_ptr_memory realloc total = %d item_size = %d\n",
                       total,item_size);)
     if (((*ptr) = realloc((*ptr), (size_t) total * item_size)) == NULL)
        {printf("Not enough memory...get_ptr_memory-realloc requested = %lu\n",
                   total * item_size);
         *err = NOT_ENOUGH_MEMORY;
         return 0;}
    }
 PRT3(printf("get_ptr_memory done\n");)
 if (total_items != NULL)
    {cnt = *total_items;
     *total_items = total;}
 else cnt = 0;
 *err = NO_ERROR;

 return cnt;
}



/*------------------------------------------------------------------------
get_ptr - ???
-------------------------------------------------------------------------*/

int
get_ptr(ptr,total_items,add_num_items,item_size)
 void **ptr;
 unsigned long *total_items;
 unsigned long add_num_items;
 size_t item_size;
{
 int err;

 get_ptr_memory(ptr,total_items,add_num_items,item_size,&err);
 return err;
}



/*------------------------------------------------------------------------
free_handle_memory - ???
-------------------------------------------------------------------------*/

void
free_handle_memory(ptr,total_items)
 char **ptr;
 unsigned long total_items;
{
 unsigned long cnt;

 for (cnt = 0; cnt < total_items; cnt++)
    c_free(ptr[cnt]);
 c_free(ptr);
}


/*------------------------------------------------------------------------
free_handle_elements - ???
-------------------------------------------------------------------------*/

void
free_handle_elements(ptr,total_items)
 char **ptr;
 unsigned long total_items;
{
 unsigned long cnt;

 for (cnt = 0; cnt < total_items; cnt++)
    c_free(ptr[cnt]);
}


/*------------------------------------------------------------------------
add_handle_element - ???
-------------------------------------------------------------------------*/

int
add_handle_element(handle,num_elems,ptr)
 void ***handle;
 unsigned long *num_elems;
 void *ptr;
{
 if (*num_elems == 0)
    {*num_elems += 1;
     if ((*handle = (void **) malloc((size_t) (*num_elems *
                  (size_t) sizeof(char *)) )) == NULL)
        {printf("Not enough memory...add_handle_element\n");
         return NOT_ENOUGH_MEMORY;}
     (*handle)[0] = ptr;}
    else
     {
      if ((*handle = (void **) realloc(*handle,(size_t) (*num_elems + 1) *
                      sizeof(char *))) == NULL)
        {printf("Not enough memory...realloc...add_handle_element\n");
         return NOT_ENOUGH_MEMORY;}
       (*handle)[*num_elems] = ptr;
       *num_elems += 1;}
 return NO_ERROR;
}


/*------------------------------------------------------------------------
find_ptr_element1 - ???
-------------------------------------------------------------------------*/

unsigned long 
find_ptr_element1(ptr,num_elems,elem_size,element_ptr,err)
 char *ptr;
 unsigned long num_elems;
 size_t elem_size;
 char *element_ptr;
 int *err;
{
 unsigned long cnt;

 for (cnt = 0; cnt < num_elems; cnt++)
    {
     if (&(ptr[cnt * elem_size]) == element_ptr)
        {*err = NO_ERROR;
         return cnt;}
    }
 *err = NOT_FOUND;
 return 0;
}


/*------------------------------------------------------------------------
find_handle_element - ???
-------------------------------------------------------------------------*/

unsigned long 
find_handle_element(handle,num_elems,ptr,err)
 char **handle;
 unsigned long num_elems;
 char *ptr;
 int *err;
{
 unsigned long cnt;

 for (cnt = 0; cnt < num_elems; cnt++)
    if (handle[cnt] == ptr)
       {*err = NO_ERROR;
        return cnt;}

 *err = NOT_FOUND;
 return 0;
}


/*------------------------------------------------------------------------
FindPtrInHandleAry - ???
-------------------------------------------------------------------------*/

int
FindPtrInHandleAry(ary,num_elems,ptr,ptr_posi)
 void **ary;
 unsigned long num_elems;
 void *ptr;
 unsigned long *ptr_posi;
{
 unsigned long lcv;

 for (lcv = 0; lcv < num_elems; lcv++)
    if (ary[lcv] == ptr)
       {*ptr_posi = lcv;
        return NO_ERROR;}
 return NOT_FOUND;
}


/*------------------------------------------------------------------------
FindStrInHandleAryByContent - ???
-------------------------------------------------------------------------*/

unsigned long 
FindStrInHandleAryByContent(handle,num_elems,ptr,err)
 char **handle;
 unsigned long num_elems;
 char *ptr;
 int *err;
{
 unsigned long cnt;

 for (cnt = 0; cnt < num_elems; cnt++)
    if (no_case_str_cmp(handle[cnt],ptr) == 0)
       {*err = NO_ERROR;
        return cnt;}

 *err = NOT_FOUND;
 return 0;
}


/*------------------------------------------------------------------------
DropPtrAryPtr - ???
-------------------------------------------------------------------------*/

int
DropPtrAryPtr(root,num_elems,elem_size,ptr)
 char **root;
 unsigned int *num_elems;
 size_t elem_size;
 char *ptr;
{
 unsigned int cnt,lcv,indx;
 char *the_root;
 int err;

 if (*num_elems == 0) return NO_ERROR;
 cnt = find_ptr_element1(*root,*num_elems,elem_size,ptr,&err);
 if (err == NO_ERROR)
    {
     *num_elems -= 1;
     if (*num_elems == 0)
        {
         free(*root);
         *root = NULL;
         return NO_ERROR;}
     if ((the_root = (char *) malloc((size_t) (*num_elems * elem_size)
               )) == NULL)
        {printf("Not enough memory...DropPtrAryPtr\n");
         return NOT_ENOUGH_MEMORY;}
     for (indx = lcv = 0; lcv < cnt; lcv++,indx += elem_size)
        memcpy(&(the_root[indx]),&((*root)[indx]),elem_size);
     for (; cnt < *num_elems; cnt++,indx += elem_size)
        memcpy(&(the_root[indx]),&((*root)[indx + elem_size]),elem_size);
     free(*root);
     *root = the_root;
     }
 return err;
}


/*------------------------------------------------------------------------
DropHandleAryPtr - ???

 Result :
    if ptr is found and dropped
       1) ary is reallocated
       2) num_elems decreases by 1 
-------------------------------------------------------------------------*/

int
DropHandleAryPtr(ary,num_elems,ptr)
 char ***ary;
 unsigned long *num_elems;
 char *ptr;
{
 unsigned long cnt,lcv;
 char **the_handle;
 int err;

 if (*num_elems == 0) return NO_ERROR;
 if ((err = FindPtrInHandleAry(*ary,*num_elems,ptr,&cnt)) == NO_ERROR)
    {
     *num_elems -= 1;
     if (*num_elems == 0)
        {
         free(*ary);
         *ary = NULL;
         return NO_ERROR;}
     if ((the_handle = (char **) malloc((size_t) (*num_elems *
                  (size_t) sizeof(char *)) )) == NULL)
        {printf("Not enough memory...DropHandleAryPtr\n");
         *num_elems += 1;
         return NOT_ENOUGH_MEMORY;}
     for (lcv = 0; lcv < cnt; lcv++)
        the_handle[lcv] = (*ary)[lcv];
     for (; cnt < *num_elems; lcv++,cnt++)
        the_handle[lcv] = (*ary)[cnt+1];
     free(*ary);
     *ary = the_handle;
    }
 return err;
}


/*------------------------------------------------------------------------
DropPtrFromHandleAry - ???

 Result :
    1) ary unchanged
    2) num_elems decreases by 1 if ptr is found and dropped
-------------------------------------------------------------------------*/

int
DropPtrFromHandleAry(ary,num_elems,ptr)
 void **ary;
 unsigned long *num_elems;
 void *ptr;
{
 unsigned long ptr_posi,lcv;
 int err;

 if (*num_elems == 0) return NOT_FOUND;
 if ((err = FindPtrInHandleAry(ary,*num_elems,ptr,&ptr_posi))
           == NO_ERROR)
    {*num_elems -= 1;
     for (lcv = ptr_posi; lcv < *num_elems; lcv++)
        ary[lcv] = ary[lcv + 1];
    }
 return err;
}



/*------------------------------------------------------------------------
c_free - ???
-------------------------------------------------------------------------*/

void
c_free(
    void *ptr)
{
    if (ptr != NULL) free(ptr);
    return;
}



/*------------------------------------------------------------------------
free_ptr - ???
-------------------------------------------------------------------------*/

void
free_ptr(ptr)
 unsigned char **ptr;
{
 c_free(*ptr);
 *ptr = NULL;
}


/*------------------------------------------------------------------------
cp_ptrs - ???
-------------------------------------------------------------------------*/

int
cp_ptrs(new_handle,old_handle,num_ptrs)
 unsigned char ***new_handle,**old_handle;
 unsigned long num_ptrs;
{
 unsigned long total;
 int err;

 total = 0;
/* make sure each pointer are adjacent to each other;  note that
   get_handle_memory is not used here because what we really want here
   are the spaces for the pointers but not the spaces pointed to by
   the pointers */ 
 get_ptr_memory(new_handle,&total,num_ptrs,
                   (size_t) sizeof(unsigned char *),&err);
 if (err)
    {*new_handle = NULL;
     return err;}
 for (total = 0; total < num_ptrs; total++)
    (*new_handle)[total] = old_handle[total];
 return NO_ERROR;
}


/*------------------------------------------------------------------------
cp_unsigned_int_ary - ???
-------------------------------------------------------------------------*/

int
cp_unsigned_int_ary(new,old,num_elems)
 unsigned int **new,*old;
 unsigned long num_elems;
{
 unsigned long total;
 int err;

 total = 0;
 get_ptr_memory(new,&total,num_elems,(size_t) sizeof(unsigned long *),&err);
 if (err)
    {*new = NULL;
     return err;}
 for (total = 0; total < num_elems; total++)
    (*new)[total] = old[total];
 return NO_ERROR;
}


/*------------------------------------------------------------------------
GetPixelBytes - ???
-------------------------------------------------------------------------*/

unsigned long 
GetPixelBytes(pixel_depth)
 unsigned long pixel_depth;
{
 unsigned long pixel_bytes;

 pixel_bytes = pixel_depth / 8;
 if (pixel_depth % 8) pixel_bytes += 1;
 return pixel_bytes;
}


/*------------------------------------------------------------------------
GetRowBytes - ???
-------------------------------------------------------------------------*/

unsigned long 
GetRowBytes(dimx,pixel_depth)
 unsigned long dimx,pixel_depth;
{
 unsigned long row_bytes;

 row_bytes = dimx * pixel_depth;
 if (row_bytes % 8) row_bytes = row_bytes / 8 + 1;
   else row_bytes /= 8;
 return row_bytes;
}


/*------------------------------------------------------------------------
XFirstAndLastByteOffset - ???
-------------------------------------------------------------------------*/

int
XFirstAndLastByteOffset(xposi,pixel_depth,first,last)
 unsigned long xposi,pixel_depth,*first,*last;
{
 unsigned long row_bytes;

 row_bytes = xposi * pixel_depth;
 *first = row_bytes / 8;
 row_bytes = row_bytes + pixel_depth;
 *last = row_bytes / 8;
 if ((row_bytes % 8) == 0)
    *last -= 1;

 return NO_ERROR;
}


/*------------------------------------------------------------------------
GetImageSize - ???
-------------------------------------------------------------------------*/

unsigned long 
GetImageSize(dimx,dimy,pixel_depth)
 unsigned long dimx,dimy,pixel_depth;
{
 unsigned long size;

 size = dimy * GetRowBytes(dimx,pixel_depth);
 return size;
}



/*------------------------------------------------------------------------
Setup2dOn1dAry - ???

 for pixel_size = 0, dimx should be the # of row bytes, but not the
 size of dimension x since the size may not be divisible by 8.  For
 example, when the size = 123 bits, row_bytes = 16 bytes.  Also for
 pixel_size = 0, "leftx,rightx" specified in bits should be divisible
 by 8, or that they will be assumed to be.
 -------------------------------------------------------------------------*/

int
Setup2dOn1dAry(ary_1d,dimx,dimy,pixel_depth,
        leftx,lowery,rightx,uppery,ary_2d)
 unsigned char *ary_1d;
 unsigned long dimx,dimy,pixel_depth,leftx,lowery,rightx,uppery;
 unsigned char ***ary_2d;
{
 unsigned long cnt,indx,dimx_size,dimy_lines,leftx_byte;

 dimy_lines = uppery - lowery + 1;
 if (pixel_depth == 0)
    {TRACE(printf("Setup2dOn1dAry...pixel_depth = 0\n"))
     return CANCEL;}
 if (((*ary_2d) = (unsigned char **) malloc((size_t) (dimy_lines *
        (size_t) sizeof(char *)) )) == NULL)
    return NOT_ENOUGH_MEMORY;
 leftx_byte = leftx * pixel_depth / 8;
 dimx_size = GetRowBytes(dimx,pixel_depth);

 for (cnt = 0,indx = lowery * dimx_size + leftx_byte; cnt < dimy_lines; cnt++)
    {(*ary_2d)[cnt] = &(ary_1d[indx]);
     indx += dimx_size;}
 return NO_ERROR;
}

