/****************************************************************************
rect_extra.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: rect_extra.c,v 1.5 1999/07/07 18:01:07 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <math.h>

#include "rect.h"
#include "error_constants.h"
#include "nria_lib.h"


/*
same_rect(rr1,rr2)
can_fit_rect(assumed_bigger,assumed_smaller)
can_fit_rect_height(assumed_bigger,assumed_smaller)
can_fit_rect_width(assumed_bigger,assumed_smaller)
fit_rect(assumed_bigger,assumed_smaller)

centerx_rect(rr,centerx)
centery_rect(rr,centery)
centerxy_rect(rr,centerx,centery)
center_rect(center,rr)

place_adjacent_rect(limitrect,rr1,rr2)

rect_pts_in(rr1,rr2,in_rr1_pts,num_in_rr1_pts)
get_intersected_rect_1(rr1,rr2,intersected,in_rr2_pts)
get_intersected_rect(rr1,rr2,intersected)
make_int_rect_rec(rr,x1,y1,x2,y2)
combine_rects(rr1,rr2)

combine_int_bound_recs(rr1,rr2)
*/


/*------------------------------------------------------------------------
same_rect - ???

return 1 if the rr1 is the same rr2 
------------------------------------------------------------------------*/

int
same_rect(rr1,rr2)
 struct int_rect_rec *rr1,*rr2;
{
 if ((rr1->left == rr2->left) && (rr1->width == rr2->width) &&
     (rr1->top == rr2->top) && (rr1->height == rr2->height)) return 1;
 return 0;
}


/*------------------------------------------------------------------------
can_fit_rect - ???

 return 1 if the rect assumed_smaller fits completely within assumed_bigger 
------------------------------------------------------------------------*/

int
can_fit_rect(assumed_bigger,assumed_smaller)
 struct int_rect_rec *assumed_bigger,*assumed_smaller;
{
 if ((assumed_smaller->width <= assumed_bigger->width) &&
     (assumed_smaller->height <= assumed_bigger->height)) return 1;
 return 0;
}



/*------------------------------------------------------------------------
can_fit_rect_height - ???

 return 1 if the rect assumed_smaller has height <= assumed_bigger 
------------------------------------------------------------------------*/

int
can_fit_rect_height(assumed_bigger,assumed_smaller)
 struct int_rect_rec *assumed_bigger,*assumed_smaller;
{
 if (assumed_smaller->height <= assumed_bigger->height) return 1;
 return 0;
}


/*------------------------------------------------------------------------
can_fit_rect_width - ???

 return 1 if the rect assumed_smaller has width <= assumed_bigger 
------------------------------------------------------------------------*/

int
can_fit_rect_width(assumed_bigger,assumed_smaller)
 struct int_rect_rec *assumed_bigger,*assumed_smaller;
{
 if (assumed_smaller->width <= assumed_bigger->width) return 1;
 return 0;
}


/*------------------------------------------------------------------------
fit_rect - ???

 return 0 if assumed_smaller fits in assumed_bigger, with the assumption that
    the [left,top] of assumed_smaller may be changed for falling within
    assumed_bigger.
 return non-0 err code otherwise
------------------------------------------------------------------------*/

int
fit_rect(assumed_bigger,assumed_smaller)
 struct int_rect_rec *assumed_bigger,*assumed_smaller;
{
 int right1,right2,bottom1,bottom2;

 if (can_fit_rect(assumed_bigger,assumed_smaller))
    {if (assumed_smaller->left < assumed_bigger->left)
        assumed_smaller->left = assumed_bigger->left;
       else
        {right1 = assumed_smaller->left + assumed_smaller->width;
         right2 = assumed_bigger->left + assumed_bigger->width;
         if (right1 > right2)
            assumed_smaller->left = right2 - assumed_smaller->width;
        }
     if (assumed_smaller->top < assumed_bigger->top)
        assumed_smaller->top = assumed_bigger->top;
       else
        {bottom1 = assumed_smaller->top + assumed_smaller->height;
         bottom2 = assumed_bigger->top + assumed_bigger->height;
         if (bottom1 > bottom2)
            assumed_smaller->top = bottom2 - assumed_smaller->height;
        }
     return 0;
    }
   else return 1;
}


/*------------------------------------------------------------------------
centerx_rect - ???

 make centerx the center of rect in the x dimension
------------------------------------------------------------------------*/

void
centerx_rect(rr,centerx)
 struct int_rect_rec *rr;
 int centerx;
{
 rr->left = centerx - rr->width / 2;
}


/*------------------------------------------------------------------------
centery_rect - ???

 make centery the center of rect in the y dimension
------------------------------------------------------------------------*/

void
centery_rect(rr,centery)
 struct int_rect_rec *rr;
 int centery;
{
 rr->top = centery - rr->height / 2;
}

/*------------------------------------------------------------------------
centerxy_rect - ???
------------------------------------------------------------------------*/

void
centerxy_rect(rr,centerx,centery)
 struct int_rect_rec *rr;
 int centerx;
 int centery;
{
 rr->left = centerx - rr->width / 2;
 rr->top = centery - rr->height / 2;
}


/*------------------------------------------------------------------------
center_rect - ???

 place the rect "center"in  the center of rect rr
------------------------------------------------------------------------*/

void
center_rect(center,rr)
 struct int_rect_rec *center,*rr;
{
 center->left = rr->left + rr->width / 2 - center->width / 2;
 center->top = rr->top + rr->height / 2 - center->height / 2;
}


/*------------------------------------------------------------------------
place_adjacent_rect - ???

  try to place rr2 adjacent to rr1 under the following conditions :-
    1) limitrect is the rect containing rr1 & rr2
    2) starting from the top, attempts of placing rr2 are made in a clock 
       wise direction so that rr2 won't overlap with rr1 if possible.
    3) If rr2 is to be placed overlapping with rr1, place rr2 on top of
       rr1.
  return 0 if successfully fit without overlapping
  return 1 otherwise
------------------------------------------------------------------------*/

int
place_adjacent_rect(limitrect,rr1,rr2)
 struct int_rect_rec *limitrect,*rr1,*rr2;
{
 struct int_rect_rec fit;
 int right,top,centerx,centery;
 int status;

 status = 0;
 setup_int_rect_rec(&fit,limitrect->left,limitrect->top,limitrect->width,
                rr1->top - limitrect->top + 1);
 if (!can_fit_rect(&fit,rr2))
    {right = rr1->left + rr1->width;
     setup_int_rect_rec(&fit,right,limitrect->top,
          limitrect->left + limitrect->width - right + 1,limitrect->height);
     if (!can_fit_rect(&fit,rr2))
        {top = rr1->top + rr1->height;
         setup_int_rect_rec(&fit,limitrect->left,top,
                 limitrect->width,
                 limitrect->top + limitrect->height - top);
         if (!can_fit_rect(&fit,rr2))
            {setup_int_rect_rec(&fit,limitrect->left,limitrect->top,
                       rr1->left - limitrect->left,limitrect->height);
             if (!can_fit_rect(&fit,rr2))
                {setup_int_rect_rec(&fit,limitrect->left,limitrect->top,
                           limitrect->width,rr2->height);
                 centerx = rr1->left + rr1->width / 2;
                 centery = limitrect->top + rr2->height / 2;
                 status = 1;
                }
               else
                {centerx = rr1->left - rr2->width / 2;
                 centery = rr1->top + rr1->height / 2;}
            }
           else
            {centerx = rr1->left + rr1->width / 2;
             centery = rr1->top + rr1->height + rr2->height / 2;}
        }
       else
        {centerx = rr1->left + rr1->width + rr2->width / 2;
         centery = rr1->top + rr1->height / 2;}
    }
   else
    {centerx = rr1->left + rr1->width / 2;
     centery = rr1->top - rr2->height / 2;}

 centerxy_rect(rr2,centerx,centery);
 fit_rect(&fit,rr2);
 return status;
}

/*
 return RECT_NO_OVERLAP if they are disjoint
        RECT_EQUAL if rr1 is equal to rr2
        RECT_CONTAINED if rr1 is contained in rr2
        RECT_CONTAINS if rr2 is contained in rr1
        RECT_OVERLAPS_AT_TL_CORNER if they overlap on the top left corner pt
                   of rr1
        RECT_OVERLAPS_AT_TR_CORNER if they overlap on the top right corner pt
                   of rr1
        RECT_OVERLAPS_AT_BR_CORNER if they overlap on the bottom right corner pt
                   of rr1
        RECT_OVERLAPS_AT_BL_CORNER if they overlap on the bottom left corner pt
                   of rr1
        RECT_OVERLAP if they overlap
*/
/*************************
overlapping_rects(rr1,rr2)
 struct int_rect_rec *rr1,*rr2;
{
 int right1,bottom1,right2,bottom2;
 int in;

 if ((rr1->width == 0) || (rr2->width == 0)) return RECT_NO_OVERLAP;
 if (same_rect(rr1,rr2)) return RECT_EQUAL;
 right1 = rr1->left + rr1->width - 1;
 right2 = rr2->left + rr2->width - 1;
 bottom1 = rr1->top + rr1->height - 1;
 bottom2 = rr2->top + rr2->height - 1;
 in = 0;
 if ((rr1->left < rr2->left) || (rr1->left > right2))
    {if ((rr2->left < rr1->left) || (rr2->left > right1)) 
        return RECT_NO_OVERLAP;
       else 
        {
         if (pt_in_int_rect(rr1,rr2->left,rr2->top))
            {
             if ((rr2->left == right1) && (rr2->top == bottom1))
                return RECT_OVERLAPS_AT_BR_CORNER;
             if (pt_in_int_rect(rr1,right2,bottom2)) return RECT_CONTAINS;
            }
           else
            {if ((rr2->left == right1) && (bottom2 == rr1->top))
                return RECT_OVERLAPS_AT_TR_CORNER;
             if ((right2 == rr1->left) && (bottom2 == rr1->top))
                return RECT_OVERLAPS_AT_TL_CORNER;
            }
         if (pt_in_int_rect(rr2,rr1->left,rr1->top))
            if (pt_in_int_rect(rr2,right1,bottom1)) return RECT_CONTAINED;
        }
    }
   else 
    {
     if (pt_in_int_rect(rr2,rr1->left,rr1->top))
        {
         if ((rr1->left == right2) && (rr1->top == bottom2))
            return RECT_OVERLAPS_AT_TL_CORNER;
         if (pt_in_int_rect(rr2,right1,bottom1)) return RECT_CONTAINED;
        }
       else
        {
         if ((rr1->left == right2) && (bottom1 == rr2->top))
            return RECT_OVERLAPS_AT_BL_CORNER;
         if ((right1 == rr2->left) && (bottom1 == rr2->top))
            return RECT_OVERLAPS_AT_BR_CORNER;
        }
     if (pt_in_int_rect(rr1,rr2->left,rr2->top))
        if (pt_in_int_rect(rr1,right2,bottom2)) return RECT_CONTAINS;
    }
 return RECT_OVERLAPS;
}

intersected_rect(intersected,rr1,rr2)
 struct int_rect_rec *intersected,*rr1,*rr2;
{
 int right2,bottom2;
 int overlap;

 if ((overlap = overlapping_rects(rr1,rr2)) == RECT_NO_OVERLAP)
    setup_int_rect_rec(intersected,0,0,0,0);
   else if ((overlap == RECT_EQUAL) || (overlap == RECT_CONTAINED))
    *intersected = *rr1;
   else if (overlap == RECT_CONTAINS)
    *intersected = *rr2;
   else if (overlap == RECT_OVERLAPS_AT_TL_CORNER)
    setup_int_rect_rec(intersected,rr1->left,rr1->top,1,1);
   else if (overlap == RECT_OVERLAPS_AT_TR_CORNER)
    setup_int_rect_rec(intersected,rr1->left + rr1->width - 1,rr1->top,1,1);
   else if (overlap == RECT_OVERLAPS_AT_BR_CORNER)
    setup_int_rect_rec(intersected,rr1->left + rr1->width - 1,
                       rr1->top + rr1->height - 1,1,1);
   else if (overlap == RECT_OVERLAPS_AT_BL_CORNER)
    setup_int_rect_rec(intersected,rr1->left,
                       rr1->top + rr1->height - 1,1,1);
   else
    {right2 = rr2->left + rr2->width - 1;
     if ((rr1->left < rr2->left) || (rr1->left > right2))
        {intersected->left = rr2->left;
         intersected->width = rr1->left + rr1->width - rr2->left;}
       else
        {intersected->left = rr1->left;
         intersected->width = rr2->left + rr2->width - rr1->left;}
     bottom2 = rr2->top + rr2->height - 1;
     if ((rr1->top < rr2->top) || (rr1->top > bottom2))
        {intersected->top = rr2->top;
         intersected->height = rr1->top + rr1->height - rr2->top;}
       else
        {intersected->top = rr1->top;
         intersected->height = rr2->top + rr2->height - rr1->top;}
    }
}
***********************/


/*------------------------------------------------------------------------
rect_pts_in - ???
------------------------------------------------------------------------*/

void
rect_pts_in(rr1,rr2,in_rr1_pts,num_in_rr1_pts)
 struct int_rect_rec *rr1,*rr2;
 int *in_rr1_pts,*num_in_rr1_pts;
{
 int right,bottom;

 if (pt_in_int_rect(rr1,rr2->left,rr2->top))
    {*num_in_rr1_pts = 1;
     *in_rr1_pts = 1;}
   else
    {*in_rr1_pts = 0;
     *num_in_rr1_pts = 0;}

 right = rr2->left + rr2->width - 1;
 if (pt_in_int_rect(rr1,right,rr2->top))
    {*num_in_rr1_pts += 1;
     *in_rr1_pts |= 2;}

 bottom = rr2->top + rr2->height - 1;
 if (pt_in_int_rect(rr1,right,bottom))
    {*num_in_rr1_pts += 1;
     *in_rr1_pts |= 4;}
 if (pt_in_int_rect(rr1,rr2->left,bottom))
    {*num_in_rr1_pts += 1;
     *in_rr1_pts |= 8;}
}

/*
 return RECT_NO_OVERLAP if they are disjoint
        RECT_EQUAL if rr1 is equal to rr2
        RECT_CONTAINED if rr1 is contained in rr2
        RECT_CONTAINS if rr2 is contained in rr1
        RECT_OVERLAPS_AT_TL_CORNER_PT if they overlap on the top left corner pt
                   of rr1
        RECT_OVERLAPS_AT_TL_CORNER if they overlap on the top left corner
                   of rr1
        RECT_OVERLAPS_AT_TR_CORNER_PT if they overlap on the top right corner pt
                   of rr1
        RECT_OVERLAPS_AT_TR_CORNER if they overlap on the top right corner
                   of rr1
        RECT_OVERLAPS_AT_BR_CORNER_PT if they overlap on the bottom right 
                   corner pt of rr1
        RECT_OVERLAPS_AT_BR_CORNER if they overlap on the bottom right 
                   corner of rr1
        RECT_OVERLAPS_AT_BL_CORNER_PT if they overlap on the bottom left 
                   corner pt of rr1
        RECT_OVERLAPS_AT_BL_CORNER if they overlap on the bottom left 
                   corner of rr1
*/


/*------------------------------------------------------------------------
get_intersected_rect_1 - ???

FIXME:  Does either the comment below or the one above belong to this
function?
------------------------------------------------------------------------*/

/*
 return 0 when no pts in r1 is in r2 and intersected is undefined
*/

int
get_intersected_rect_1(rr1,rr2,intersected,in_rr2_pts)
 struct int_rect_rec *rr1,*rr2,*intersected;
 int in_rr2_pts;
{
 int right1,bottom1,right2,bottom2;

 if (in_rr2_pts == 0)
    {
     return 0;}
 if (in_rr2_pts == 15)
    {*intersected = *rr1;
     return RECT_CONTAINED;}
   else if (in_rr2_pts == 1)
    {bottom2 = rr2->top + rr2->height - 1;
     right2 = rr2->left + rr2->width - 1;
     setup_int_rect_rec(intersected,rr1->left,rr1->top,
          right2 - rr1->left + 1,bottom2 - rr1->top + 1);
     if (intersected->width == 1)
        {if (intersected->height == 1)
            return RECT_OVERLAPS_AT_TL_CORNER_PT;
           else return RECT_OVERLAPS_AT_LT_EDGE;
        }
       else if (intersected->height == 1)
        return RECT_OVERLAPS_AT_TL_EDGE;

     return RECT_OVERLAPS_AT_TL_CORNER;
    }
   else if (in_rr2_pts == 2)
    {right1 = rr1->left + rr1->width - 1;
     bottom2 = rr2->top + rr2->height - 1;
     setup_int_rect_rec(intersected,rr2->left,rr1->top,
          right1 - rr2->left + 1,bottom2 - rr1->top + 1);
     if (intersected->width == 1) 
        {if (intersected->height == 1)
            return RECT_OVERLAPS_AT_TR_CORNER_PT;
           else return RECT_OVERLAPS_AT_RT_EDGE;
        }
       else if (intersected->height == 1)
        return RECT_OVERLAPS_AT_TR_EDGE;
     return RECT_OVERLAPS_AT_TR_CORNER;
    }
   else if (in_rr2_pts == 4)
    {
     right1 = rr1->left + rr1->width - 1;
     bottom1 = rr1->top + rr1->height - 1;
     setup_int_rect_rec(intersected,rr2->left,rr2->top,
          right1 - rr2->left + 1,bottom1 - rr2->top + 1);
     if (intersected->width == 1)
        {if (intersected->height == 1)
            return RECT_OVERLAPS_AT_BR_CORNER_PT;
           else return RECT_OVERLAPS_AT_RB_EDGE;
        }
       else if (intersected->height == 1)
        return RECT_OVERLAPS_AT_BR_EDGE;

     return RECT_OVERLAPS_AT_BR_CORNER;
    }
   else if (in_rr2_pts == 8)
    {
     right2 = rr2->left + rr2->width - 1;
     bottom1 = rr1->top + rr1->height - 1;
     setup_int_rect_rec(intersected,rr1->left,rr2->top,
          right2 - rr1->left + 1,bottom1 - rr2->top + 1);
     if (intersected->width == 1)
        {if (intersected->height == 1)
            return RECT_OVERLAPS_AT_BL_CORNER_PT;
           else return RECT_OVERLAPS_AT_LB_EDGE;
        }
       else if (intersected->height == 1)
        return RECT_OVERLAPS_AT_BL_EDGE;
     return RECT_OVERLAPS_AT_BL_CORNER;
    }
   else if (in_rr2_pts == 3)
    {
     bottom2 = rr2->top + rr2->height - 1;
     setup_int_rect_rec(intersected,rr1->left,rr1->top,
          rr1->width,bottom2 - rr1->top + 1);
     if (intersected->height == 1)
        return RECT_OVERLAPS_AT_TOP_EDGE;
     return RECT_OVERLAPS_AT_TOP;
    }
   else if (in_rr2_pts == 6)
    {
     right1 = rr1->left + rr1->width - 1;
     setup_int_rect_rec(intersected,rr2->left,rr2->top,
          right1 - rr2->left + 1,rr1->height);
     if (intersected->width == 1)
        return RECT_OVERLAPS_AT_RIGHT_EDGE;
     return RECT_OVERLAPS_AT_RIGHT;
    }
   else if (in_rr2_pts == 12)
    {bottom1 = rr1->top + rr1->height - 1;
     setup_int_rect_rec(intersected,rr1->left,rr2->top,
          rr1->width,bottom1 - rr2->top + 1);
     if (intersected->height == 1)
        return RECT_OVERLAPS_AT_BOTTOM_EDGE;
     return RECT_OVERLAPS_AT_BOTTOM;
    }
   else if (in_rr2_pts == 9)
    {
     right2 = rr2->left + rr2->width - 1;
     setup_int_rect_rec(intersected,rr1->left,rr1->top,
          right2 - rr1->left + 1,rr1->height);
     if (intersected->width == 1)
        return RECT_OVERLAPS_AT_LEFT_EDGE;
     return RECT_OVERLAPS_AT_LEFT;
    }
 return 0;
}


/*------------------------------------------------------------------------
get_intersected_rect - ???
------------------------------------------------------------------------*/

int
get_intersected_rect(rr1,rr2,intersected)
 struct int_rect_rec *rr1,*rr2,*intersected;
{
 int right1,bottom1,right2,bottom2,in_rr1_pts,in_rr2_pts,
          num_rr1_pts,num_rr2_pts;
 int status;

 right1 = rr1->left + rr1->width - 1;
 right2 = rr2->left + rr2->width - 1;
 bottom1 = rr1->top + rr1->height - 1;
 bottom2 = rr2->top + rr2->height - 1;
 if ((rr1->left > right2) || (rr2->left > right1) || 
     (rr1->top > bottom2) || (rr2->top > bottom1))
    {setup_int_rect_rec(intersected,0,0,0,0);
     return RECT_NO_OVERLAP;}
 rect_pts_in(rr1,rr2,&in_rr1_pts,&num_rr1_pts);
 rect_pts_in(rr2,rr1,&in_rr2_pts,&num_rr2_pts);
 if ((in_rr2_pts == 15) && (in_rr1_pts == 15))
    {*intersected = *rr1;
     return RECT_EQUAL;}
 if (num_rr1_pts > num_rr2_pts)
    {status = get_intersected_rect_1(rr2,rr1,intersected,in_rr1_pts);
     status = inverse_rect_overlap_status[status];}
   else
    status = get_intersected_rect_1(rr1,rr2,intersected,in_rr2_pts);
 return status;
}


/*------------------------------------------------------------------------
make_int_rect_rec - ???
------------------------------------------------------------------------*/

void
make_int_rect_rec(rr,x1,y1,x2,y2)
 struct int_rect_rec *rr;
 int x1,y1,x2,y2;
{
 if (x1 <= x2)
    {rr->left = x1;
     rr->width = x2 - x1 + 1;}
   else
    {rr->left = x2;
     rr->width = x1 - x2 + 1;}

 if (y1 <= y2)
    {rr->top = y1;
     rr->height = y2 - y1 + 1;}
   else
    {rr->top = y2;
     rr->height = y1 - y2 + 1;}
}


/*------------------------------------------------------------------------
combine_rects - ???
------------------------------------------------------------------------*/

void
combine_rects(rr1,rr2)
 struct int_rect_rec *rr1,*rr2;
{
 int right1,right2,bottom1,bottom2;

 right1 = rr1->left + rr1->width - 1;
 right2 = rr2->left + rr2->width - 1;
 bottom1 = rr1->top + rr1->height - 1;
 bottom2 = rr2->top + rr2->height - 1;
 if (rr1->left > rr2->left) rr1->left = rr2->left;
 if (rr1->top > rr2->top) rr1->top = rr2->top;
 if (right1 > right2) rr1->width = right1 - rr1->left + 1;
   else rr1->width = right2 - rr1->left + 1;

 if (bottom1 > bottom2) rr1->height = bottom1 - rr1->top + 1;
   else rr1->height = bottom2 - rr1->top + 1;
}


/*------------------------------------------------------------------------
combine_int_bound_recs - ???
------------------------------------------------------------------------*/

void
combine_int_bound_recs(rr1,rr2)
 struct int_bound_rec *rr1,*rr2;
{

 if (rr1->left > rr2->left) rr1->left = rr2->left;
 if (rr1->top > rr2->top) rr1->top = rr2->top;
 if (rr1->right < rr2->right) rr1->right = rr2->right;
 if (rr1->bottom < rr2->bottom) rr1->bottom = rr2->bottom;

}

