/****************************************************************************
rect_opr.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: rect_opr.c,v 1.6 1999/07/07 18:01:07 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdio.h>
#include <math.h>

#include <sys/types.h>

#include "typedef.h"
#include "rect.h"
#include "error_constants.h"
#include "nria_lib.h"


/*-------------------------------------------------------------------------
init_float_bound_rec - ???
-------------------------------------------------------------------------*/

void
init_float_bound_rec(
    struct float_bound_rec *bound)
{
 bound->left = bound->top = 1;
 bound->right = bound->bottom = 0;
}



/*-------------------------------------------------------------------------
setup_int_rect_rec - ???
-------------------------------------------------------------------------*/

void
setup_int_rect_rec(
    struct int_rect_rec *rr,
    int left,
    int top,
    int width,
    int height)
{
 rr->left = left;
 rr->top = top;
 rr->width = width;
 rr->height = height;
}



/*-------------------------------------------------------------------------
setup_int_bound_rec - ???
-------------------------------------------------------------------------*/

void
setup_int_bound_rec(
    struct int_bound_rec *bound,
    int left,
    int top,
    int right,
    int bottom)
{
 bound->left = left;
 bound->top = top;
 bound->right = right;
 bound->bottom = bottom;
}



/*-------------------------------------------------------------------------
get_valid_value - ???
-------------------------------------------------------------------------*/

void
get_valid_value(
    int *result,
    int lb,
    int ub,
    int value)
{
 if (value < lb)
    {if (lb < 0) *result = 0;
       else *result = lb;}
   else if (value > ub) 
    {if (ub < 0) *result = 0;
       else *result = ub;}
   else *result = value;
}



/*-------------------------------------------------------------------------
valid_bound - ???
-------------------------------------------------------------------------*/

void
valid_bound(
    struct int_bound_rec *checked,
    struct int_bound_rec *valid,
    int *valid_left,
    int *valid_right,
    int *valid_top,
    int *valid_bottom)
{
 get_valid_value(valid_left,valid->left,valid->right,checked->left);
 get_valid_value(valid_right,valid->left,valid->right,checked->right);
 get_valid_value(valid_top,valid->top,valid->bottom,checked->top);
 get_valid_value(valid_bottom,valid->top,valid->bottom,checked->bottom);
}



/*-------------------------------------------------------------------------
move_int_bound_rec - ???
-------------------------------------------------------------------------*/

void
move_int_bound_rec(
    struct int_bound_rec *bound,
    int addx,
    int addy)
{
 bound->left += addx;
 bound->right += addx;
 bound->top += addy;
 bound->bottom += addy;
}



/*-------------------------------------------------------------------------
pt_on_int_bound_rec - ???
-------------------------------------------------------------------------*/

static int
pt_on_int_bound_rec(bound,x,y)
 struct int_bound_rec *bound;
 int x,y;
{
 if ((x == bound->left) || (x == bound->right) ||
     (y == bound->top) || (y == bound->bottom))
    return 1;
 return 0;
}



/*-------------------------------------------------------------------------
is_between - ???
-------------------------------------------------------------------------*/

int
is_between(
    int dot1,
    int dot2,
    int dot3)
{
 if ((dot1 >= dot2) && (dot1 <= dot3)) return 1;
 else if ((dot1 >= dot3) && (dot1 <= dot2)) return 1;
 else return 0;
}



/*-------------------------------------------------------------------------
is_float_between - ???
-------------------------------------------------------------------------*/

int
is_float_between(
    float dot1,
    float dot2,
    float dot3)
{
 if ((dot1 >= dot2) && (dot1 <= dot3)) return 1;
 else if ((dot1 >= dot3) && (dot1 <= dot2)) return 1;
 else return 0;
}



/*-------------------------------------------------------------------------
is_pt_in_area - ???
-------------------------------------------------------------------------*/

int
is_pt_in_area(
    int x1,
    int y1,
    int x2,
    int y2,
    int ptx,
    int pty)
{
 if (is_between(ptx,x1,x2) && is_between(pty,y1,y2)) return 1;
    else return 0;
}



/*-------------------------------------------------------------------------
is_long_between - ???
-------------------------------------------------------------------------*/

static int
is_long_between(dot1,dot2,dot3)
 long dot1,dot2,dot3;
{
 if ((dot1 >= dot2) && (dot1 <= dot3)) return 1;
   else if ((dot1 >= dot3) && (dot1 <= dot2)) return 1;
   else return 0;
}



/*-------------------------------------------------------------------------
is_long_pt_in_area - ???
-------------------------------------------------------------------------*/

static int
is_long_pt_in_area(x1,y1,x2,y2,ptx,pty)
 long x1,y1,x2,y2,ptx,pty;
{
 if (is_long_between(ptx,x1,x2) && is_long_between(pty,y1,y2)) return 1;
    else return 0;
}



/*-------------------------------------------------------------------------
move_area - ???
-------------------------------------------------------------------------*/

void
move_area(
    int *x1,
    int *y1, 
    int *x2,
    int *y2,
    int refx,
    int refy,
    int newx,
    int newy)
{
 *x1 = *x1 - refx + newx;
 *x2 = *x2 - refx + newx;
 *y1 = *y1 - refy + newy;
 *y2 = *y2 - refy + newy;
}



/*-------------------------------------------------------------------------
pt_in_int_rect - ???
-------------------------------------------------------------------------*/

int
pt_in_int_rect(
    struct int_rect_rec *rr,
    int x,
    int y)
{
 if ((x >= rr->left) && (x < rr->left + rr->width) &&
     (y >= rr->top) && (y < rr->top + rr->height))
    return 1;
   else return 0;
}



/*-------------------------------------------------------------------------
pt_in_2d_int_rect - ???
-------------------------------------------------------------------------*/

int
pt_in_2d_int_rect(
    struct int_rect_rec **rr,
    unsigned int num_rows,
    unsigned int num_cols,
    unsigned int total_rects,
    int x,
    int y,
    unsigned int *row_num,
    unsigned int *col_num,
    unsigned int *rect_num)
{
 for (*rect_num = 0,*row_num = 0; *row_num < num_rows; (*row_num)++)
    {for (*col_num = 0; *col_num < num_cols; (*col_num)++,(*rect_num)++)
        {
          if (*rect_num == total_rects)
             return 0;
           else if (pt_in_int_rect(&(rr[*row_num][*col_num]),x,y))
             return 1;
        }
    }
 return 0;
}



/*-------------------------------------------------------------------------
build_2d_int_rects - ???
-------------------------------------------------------------------------*/

int
build_2d_int_rects(
    struct int_rect_rec ***rr,
    unsigned int num_rows,
    unsigned int num_cols,
    unsigned int total_rects,
    int x,
    int y,
    int width,
    int height,
    int x_gap,
    int y_gap)
{
 unsigned int row,col,rr_width,rr_height,left,cnt_rect,total;
 struct int_rect_rec *ptr;
 int err;

 total = 0;
 *rr = NULL;
 get_ptr_memory(&ptr,&total,total_rects,(size_t) sizeof(struct int_rect_rec),
         &err);
 if (err != NO_ERROR) return err;
 total = 0;
 get_handle_memory(rr,&total,num_rows,
         (size_t) sizeof(struct int_rect_rec *),
         &err);
 if (err != NO_ERROR)
    {c_free(ptr);
     return err;}

 for (col = row = 0; row < num_rows; row++,col += num_cols)
    (*rr)[row] = &(ptr[col]);

 rr_width = width / num_cols;
 rr_height = height / num_rows;
 for (cnt_rect = 0,row = 0; row < num_rows; row++,y += rr_height)
    {for (left = x,col = 0; col < num_cols;
          col++,left += rr_width,cnt_rect++)
       if (cnt_rect == total_rects) break;
         else
          {(*rr)[row][col].left = left;
           (*rr)[row][col].top = y;
           (*rr)[row][col].width = rr_width - x_gap;
           (*rr)[row][col].height = rr_height - y_gap;
          }
    }
 return NO_ERROR;
}



/*-------------------------------------------------------------------------
move_2d_int_rects - ???
-------------------------------------------------------------------------*/

static void
move_2d_int_rects(rr,num_rows,num_cols,refx,refy,x,y)
 struct int_rect_rec **rr;
 unsigned int num_rows,num_cols;
 int refx,refy,x,y;
{
 unsigned int row,col;

 for (row = 0; row < num_rows; row++)
    for (col = 0; col < num_cols; col++)
       {rr[row][col].left = rr[row][col].left - refx + x;
        rr[row][col].left = rr[row][col].top - refy + y;}
}



/*-------------------------------------------------------------------------
short_to_float_bound_rec - ???
-------------------------------------------------------------------------*/

static void
short_to_float_bound_rec(fl_bound,sh_bound)
 struct float_bound_rec *fl_bound;
 struct short_bound_rec *sh_bound;
{
 fl_bound->left = (float) sh_bound->left;
 fl_bound->right = (float) sh_bound->right;
 fl_bound->top = (float) sh_bound->top;
 fl_bound->bottom = (float) sh_bound->bottom;
}



/*-------------------------------------------------------------------------
float_to_int_bound_rec - ???
-------------------------------------------------------------------------*/

void
float_to_int_bound_rec(
    struct int_bound_rec *int_bound,
    struct float_bound_rec *fl_bound)
{
 int_bound->left = (int) rint((double) fl_bound->left);
 int_bound->right = (int) rint((double) fl_bound->right);
 int_bound->top = (int) rint((double) fl_bound->top);
 int_bound->bottom = (int) rint((double) fl_bound->bottom);
}



/*-------------------------------------------------------------------------
pr_int_rect_rec - ???
-------------------------------------------------------------------------*/

void
pr_int_rect_rec(
    struct int_rect_rec *rr)
{
 printf(
    "left = %d top = %d right = %d bottom = %d width = %d height = %d\n",
     rr->left,rr->top,rr->left + rr->width - 1,
     rr->top + rr->height - 1,rr->width,rr->height);
}



/*-------------------------------------------------------------------------
pr_short_bound_rec - ???
-------------------------------------------------------------------------*/

static void
pr_short_bound_rec(bound)
 struct short_bound_rec *bound;
{
 printf(
    "left = %d top = %d right = %d bottom = %d\n",
     bound->left,bound->top,bound->right,bound->bottom);
}



/*-------------------------------------------------------------------------
pr_float_bound_rec - ???
-------------------------------------------------------------------------*/

void
pr_float_bound_rec(
    struct float_bound_rec *bound)
{
 printf(
    "left = %f top = %f right = %f bottom = %f\n",
     bound->left,bound->top,bound->right,bound->bottom);
}

