/****************************************************************************
wr_hdr_bin.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: wr_hdr_bin.c,v 1.8 1999/07/07 18:01:07 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <math.h>
#include <stdio.h>

#include <unistd.h>
#include <sys/types.h>

#include "typedef.h"
#include "mark_hdr.h"
#include "ia_hdr.h"
#include "error_constants.h"
#include "common_constants.h"
#include "mri_hdr_constants.h"
#include "nria_lib.h"


#ifdef DEBUG
#define TRACE(x) x; fflush(stdout);
#else
#define TRACE(x)
#endif



/*-----------------------------------------------------------------------
mri_hdr_constants - (Global) definition of ??? mri header format

If you modify this table, you need to keep it consistent with the contents
of mri_hdr_constants.h.

FIXME:  Why isn't this table defined in the same place as what it needs
to be consistent with?
------------------------------------------------------------------------*/

struct hdr_field_rec 
mri_hdr_fields[TOTAL_FIELDS] =
       {{INPUT_OUTPUT_FN,     RD_TEXT_INPUT,    "INPUT_OUTPUT_FN", 0},
        {NAME,                RD_TEXT_SEQUENCE, "NAME",            0},
        {SSNUM,               RD_TEXT_INPUT,    "SSNUM",           0},
        {SEX,                 RD_TEXT_INPUT,    "SEX",             0},
        {AGE,                 RD_INT_INPUT,     "AGE",             0},
        {HEIGHT,              RD_INT_INPUT,     "HEIGHT",          0},
        {WEIGHT,              RD_INT_INPUT,     "WEIGHT",          0},
        {DEFAULT_DATA_FN,     RD_TEXT_INPUT,    "DEFAULT_DATA_FN", 0},
        {DIMX,                RD_INT_INPUT,     "DIMX",            0},
        {DIMY,                RD_INT_INPUT,     "DIMY",            0},
        {PIXEL_DEPTH_BITS,    RD_INT_INPUT,     "DEPTH",           0},
        {MAX_SLICES,          RD_INT_INPUT,     "SLICES",          0},
        {NUM_SCANS,           RD_INT_INPUT,     "SCANS",           0},
        {MIN_PIXEL,           RD_REAL_INPUT,    "MIN",             0},
        {MAX_PIXEL,           RD_REAL_INPUT,    "MAX",             0},
        {XPIXEL_SIZE,         RD_REAL_INPUT,    "X_PSIZE",         0},
        {YPIXEL_SIZE,         RD_REAL_INPUT,    "Y_PSIZE",         0},
        {PIXEL_THICKNESS,     RD_REAL_INPUT,    "THICKNESS",       0},
        {TRANSFORMATION_MATRIX_4_BY_4,  RD_MATRIX_INPUT,  "",      0},
        {NUM_KIND_OF_IMAGES,  RD_NUM_KIND_OF_IMAGES,  "KIND",      0},
        {IMAGE_TYPE,          RD_TEXT_SEQUENCE, "IMAGE_TYPE",      0},
        {COMMENT,             RD_TEXT_SEQUENCE, "COMMENT",         0},
        {PLANE_TYPE,          RD_INT_INPUT,     "PLANE",           0},
        {DATE_OF_STUDY,       RD_TEXT_INPUT,    "DATE",            0},
        {TIME_OF_STUDY,       RD_TEXT_INPUT,    "TIME",            0},
        {PATIENT_ID,          RD_TEXT_INPUT,    "PID",             0},
        {STUDY_NUM,           RD_TEXT_INPUT,    "STUDY#",          0},
        {PATIENT_NUM,         RD_TEXT_INPUT,    "mhcrc",           0},
        {REFERED_BY,          RD_TEXT_SEQUENCE, "REFERED_BY",      0},
        {DESCRIPTION,         RD_TEXT_SEQUENCE, "DESCRIPTION",     0},
        {HOSP_NAME,           RD_TEXT_SEQUENCE, "HOSP_NAME",       0},
        {SPACING,             RD_REAL_INPUT,    "SPACING",         0},
        {TR,                  RD_REAL_INPUT,    "TR",              0},
        {TE,                  RD_TEXT_SEQUENCE, "TE",              0},
        {VAR_USE_STATUS,      RD_INT_INPUT,     "STATUS",          0},
        {UNUSED1,             RD_REAL_INPUT,    "UNUSED1",         0},
        {UNUSED2,             RD_INT_INPUT,     "UNUSED2",         0},
        {UNUSED3,             RD_INT_INPUT,     "UNUSED3",         0},
        {UNUSED4,             RD_INT_INPUT,     "UNUSED4",         0},
        {UNUSED5,             RD_INT_INPUT,     "UNUSED5",         0},
        {UNUSED6,             RD_REAL_INPUT,    "UNUSED6",         0},
        {UNUSED7,             RD_REAL_INPUT,    "UNUSED7",         0},
        {END_OF_INPUT,        0,                "END_OF_INPUT",    0},
       };



/*------------------------------------------------------------------------
assign_float_defaults - ???
-------------------------------------------------------------------------*/

void
assign_float_defaults(hdr,select,value)
 struct hdr_rec *hdr;
 int select;
 float value;
{
 if (select == XPIXEL_SIZE) hdr->data_info.xpixel_size = value;
   else if (select == YPIXEL_SIZE) hdr->data_info.ypixel_size = value;
   else if (select == PIXEL_THICKNESS) hdr->data_info.pixel_thickness = value;
   else if (select == SPACING) hdr->data_info.spacing = value;
   else if (select == TR) hdr->data_info.tr = value;
   else if (select == MIN_PIXEL) hdr->data_info.min_pixel = value;
   else if (select == MAX_PIXEL) hdr->data_info.max_pixel = value;
   else if (select == UNUSED1)
    {hdr->data_info.unused1 = value;
     hdr->data_info.var_use_status |= UNUSED1_USED;}
   else if (select == UNUSED6)
    {hdr->data_info.unused6 = value;
     hdr->data_info.var_use_status |= UNUSED6_USED;}
   else if (select == UNUSED7)
    {hdr->data_info.unused7 = value;
     hdr->data_info.var_use_status |= UNUSED7_USED;}
}


/*--------------------------------------------------------------------------
assign_int_defaults - ???
--------------------------------------------------------------------------*/

void
assign_int_defaults(hdr,select,value)
 struct hdr_rec *hdr;
 int select;
 int value;
{
 if (select == AGE) hdr->data_info.patient.age = value;
   else if (select == HEIGHT) hdr->data_info.patient.height = value;
   else if (select == WEIGHT) hdr->data_info.patient.weight = value;
   else if (select == DIMX) hdr->data_info.dimx = value;
   else if (select == DIMY) hdr->data_info.dimy = value;
   else if (select == PIXEL_DEPTH_BITS)
    hdr->data_info.pixel_depth_bits = value;
   else if (select == MAX_SLICES) hdr->data_info.max_scans = value;
   else if (select == PLANE_TYPE) hdr->data_info.plane_type = value;
   else if (select == NUM_SCANS)
    {hdr->data_info.num_scans = value;
     hdr->data_info.var_use_status |= NUM_SCANS_USED;}
   else if (select == VAR_USE_STATUS) hdr->data_info.var_use_status |= value;
   else if (select == UNUSED2) 
    {hdr->data_info.unused2 = value;
     hdr->data_info.var_use_status |= UNUSED2_USED;}
   else if (select == UNUSED3) 
    {hdr->data_info.unused3 = value;
     hdr->data_info.var_use_status |= UNUSED3_USED;}
   else if (select == UNUSED4) 
    {hdr->data_info.unused4 = value;
     hdr->data_info.var_use_status |= UNUSED4_USED;}
   else if (select == UNUSED5) 
    {hdr->data_info.unused5 = value;
     hdr->data_info.var_use_status |= UNUSED5_USED;}
}


/*--------------------------------------------------------------------------
assign_text_defaults - ???
--------------------------------------------------------------------------*/

int
assign_text_defaults(hdr,select,str)
 struct hdr_rec *hdr;
 int select;
 char *str;
{
 int err;
 char *str1;

 if ((err = cp_str(&str1,str)) != NO_ERROR) return err;
   else
    {trim_str_blanks(str1);
     if (select == NAME)
        {c_free(hdr->data_info.patient.name);
         hdr->data_info.patient.name = str1;
        }
       else if (select == SSNUM)
        {c_free(hdr->data_info.patient.ssnum);
        hdr->data_info.patient.ssnum = str1;}
       else if (select == DEFAULT_DATA_FN)
        {c_free(hdr->data_info.default_datafn);
         hdr->data_info.default_datafn = str1;}
       else if (select == COMMENT)
        {c_free(hdr->comment);
         hdr->comment = str1;
         hdr->comment_size = get_strlen(str1);
        }
       else if (select == SEX)
        hdr->data_info.patient.sex = str1[0];
       else if (select == IMAGE_TYPE)
        {c_free(hdr->data_info.image_id);
         hdr->data_info.image_id = str1;}
       else if (select == DATE_OF_STUDY)
        {c_free(hdr->data_info.date_of_study);
         hdr->data_info.date_of_study = str1;}
       else if (select == TIME_OF_STUDY)
        {c_free(hdr->data_info.time_of_study);
         hdr->data_info.time_of_study = str1;}
       else if (select == PATIENT_ID)
        {c_free(hdr->data_info.patient.patient_id);
         hdr->data_info.patient.patient_id = str1;}
       else if (select == STUDY_NUM)
        {c_free(hdr->data_info.patient.study_num);
         hdr->data_info.patient.study_num = str1;}
       else if (select == PATIENT_NUM)
        {c_free(hdr->data_info.patient.patient_num);
         hdr->data_info.patient.patient_num = str1;}
       else if (select == REFERED_BY)
        {c_free(hdr->data_info.refered_by);
         hdr->data_info.refered_by = str1;}
       else if (select == DESCRIPTION)
        {c_free(hdr->data_info.description);
         hdr->data_info.description = str1;}
       else if (select == HOSP_NAME)
        {c_free(hdr->data_info.hosp_name);
         hdr->data_info.hosp_name = str1;}
       else if (select == TE)
        {c_free(hdr->data_info.te);
         hdr->data_info.te = str1;}
    }
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
assign_text - ???
--------------------------------------------------------------------------*/

void
assign_text(dest,src,select)
 struct hdr_rec *dest,*src;
 int select;
{
 if (select == NAME)
    dest->data_info.patient.name = src->data_info.patient.name;
   else if (select == SSNUM)
    dest->data_info.patient.ssnum = src->data_info.patient.ssnum;
   else if (select == DEFAULT_DATA_FN)
    dest->data_info.default_datafn = src->data_info.default_datafn;
   else if (select == COMMENT)
    {
     dest->comment = src->comment;
     dest->comment_size = get_strlen(dest->comment);
    }
   else if (select == SEX)
    dest->data_info.patient.sex = src->data_info.patient.sex;
   else if (select == IMAGE_TYPE)
    dest->data_info.image_id = src->data_info.image_id;
   else if (select == DATE_OF_STUDY)
    dest->data_info.date_of_study = src->data_info.date_of_study;
   else if (select == TIME_OF_STUDY)
    dest->data_info.time_of_study = src->data_info.time_of_study;
   else if (select == PATIENT_ID)
    dest->data_info.patient.patient_id = src->data_info.patient.patient_id;
   else if (select == STUDY_NUM)
    dest->data_info.patient.study_num = src->data_info.patient.study_num;
   else if (select == PATIENT_NUM)
    dest->data_info.patient.patient_num = src->data_info.patient.patient_num;
   else if (select == REFERED_BY)
    dest->data_info.refered_by = src->data_info.refered_by;
   else if (select == DESCRIPTION)
     dest->data_info.description = src->data_info.description;
   else if (select == HOSP_NAME)
    dest->data_info.hosp_name = src->data_info.hosp_name;
   else if (select == TE)
    dest->data_info.te = src->data_info.te;
}


/*--------------------------------------------------------------------------
assign_int - ???
--------------------------------------------------------------------------*/

void
assign_int(dest,src,select)
 struct hdr_rec *dest,*src;
 int select;
{
 if (select == AGE)
    dest->data_info.patient.age = src->data_info.patient.age;
   else if (select == HEIGHT) 
    dest->data_info.patient.height = src->data_info.patient.height;
   else if (select == WEIGHT)
    dest->data_info.patient.weight = src->data_info.patient.weight;
   else if (select == DIMX)
    dest->data_info.dimx = src->data_info.dimx;
   else if (select == DIMY)
    dest->data_info.dimy = src->data_info.dimy;
   else if (select == PIXEL_DEPTH_BITS)
    dest->data_info.pixel_depth_bits = src->data_info.pixel_depth_bits;
   else if (select == MAX_SLICES) 
    dest->data_info.max_scans = src->data_info.max_scans;
   else if (select == PLANE_TYPE)
    dest->data_info.plane_type = src->data_info.plane_type;
   else if (select == NUM_SCANS)
    {dest->data_info.num_scans = src->data_info.num_scans;
     dest->data_info.var_use_status |= NUM_SCANS_USED;}
   else if (select == VAR_USE_STATUS) 
     dest->data_info.var_use_status |= src->data_info.var_use_status;
   else if (select == UNUSED2) 
    {dest->data_info.unused2 = src->data_info.unused2;
     dest->data_info.var_use_status |= UNUSED2_USED;}
   else if (select == UNUSED3) 
    {dest->data_info.unused3 = src->data_info.unused3;
     dest->data_info.var_use_status |= UNUSED3_USED;}
   else if (select == UNUSED4) 
    {dest->data_info.unused4 = src->data_info.unused4;
     dest->data_info.var_use_status |= UNUSED4_USED;}
   else if (select == UNUSED5) 
    {dest->data_info.unused5 = src->data_info.unused5;
     dest->data_info.var_use_status |= UNUSED5_USED;}
}


/*--------------------------------------------------------------------------
assign_float - ???
--------------------------------------------------------------------------*/

void
assign_float(dest,src,select)
 struct hdr_rec *dest,*src;
 int select;
{
 if (select == XPIXEL_SIZE)
    dest->data_info.xpixel_size = src->data_info.xpixel_size;
   else if (select == YPIXEL_SIZE) 
    dest->data_info.ypixel_size = src->data_info.ypixel_size;
   else if (select == PIXEL_THICKNESS)
    dest->data_info.pixel_thickness = src->data_info.pixel_thickness;
   else if (select == SPACING)
    dest->data_info.spacing = src->data_info.spacing;
   else if (select == TR) dest->data_info.tr = src->data_info.tr;
   else if (select == MIN_PIXEL)
    dest->data_info.min_pixel = src->data_info.min_pixel;
   else if (select == MAX_PIXEL)
    dest->data_info.max_pixel = src->data_info.max_pixel;
   else if (select == UNUSED1) 
    {dest->data_info.unused1 = src->data_info.unused1;
     dest->data_info.var_use_status |= UNUSED1_USED;}
}


/*--------------------------------------------------------------------------
assign_num_kind_of_images - ???
--------------------------------------------------------------------------*/

int
assign_num_kind_of_images(dest,src)
 struct hdr_rec *dest,*src;
{
 dest->analysis = src->analysis;
 dest->status = src->status;
 dest->data_info.num_echoes = src->data_info.num_echoes;

 return NO_ERROR;
}


/*--------------------------------------------------------------------------
assign_hdrs - ???
--------------------------------------------------------------------------*/

int
assign_hdrs(dest,src,fields)
 struct hdr_rec *dest,*src;
 struct hdr_field_rec fields[TOTAL_FIELDS];
{
 int lcv,select;

 for (lcv = 0; lcv < TOTAL_USER_DEFINED_FIELDS; lcv++)
    {if (fields[lcv].has_been_read == 0) continue;
     select = lcv;
     switch (fields[lcv].how_to_read)
        {case RD_TEXT_INPUT :
            if (select == INPUT_OUTPUT_FN) continue;
            assign_text(dest,src,select);
            break;
         case RD_INT_INPUT :
            assign_int(dest,src,select);
            break;
         case RD_REAL_INPUT :
            assign_float(dest,src,select);
            break;
         case RD_MATRIX_INPUT :
            memcpy(dest->data_info.trans_matrix,src->data_info.trans_matrix,
                    (size_t) sizeof(dest->data_info.trans_matrix));
            break;
         case RD_NUM_KIND_OF_IMAGES :
            assign_num_kind_of_images(dest,src);
            break;
        }
     }
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
rd_num_kind_of_images - ???
--------------------------------------------------------------------------*/

int
rd_num_kind_of_images(fid,hdr,is_new)
 FILE *fid;
 struct hdr_rec *hdr;
 int is_new;
{
 int err,value,lcv;
 unsigned int num;
 char str[255];

 num = 1;
 if ((err = RdText(fid,&value,&num,RD_INT,0)) != NO_ERROR) return err;
 if (num == 0) return READ_ERROR;

 if ((value != 2) && (value != 1))
    {printf("# of image kinds must be either 1 or 2...value read = %d\n",value);
     return 1;}
    
 hdr->analysis.num_img_labels = 0;
 get_ptr_memory(&(hdr->analysis.img_labels),&(hdr->analysis.num_img_labels),
        value,(size_t) sizeof(char *),&err);
 if (err != NO_ERROR) return err;
 for (lcv = 0; lcv < hdr->analysis.num_img_labels; lcv++)
    {num = sizeof(str);
     if ((err = RdText(fid,str,&num,RD_TEXT,0)) != NO_ERROR) return err;
     if (num == 0) return READ_ERROR;
     if ((err = cp_str(&(hdr->analysis.img_labels[lcv]),str)) != NO_ERROR)
        return err;
    }
 if (hdr->analysis.num_img_labels == 1)
    {hdr->status &= ~_2_ECHO_IMAGE; 
     hdr->status |= _1_ECHO_IMAGE;
     hdr->data_info.num_echoes = 1;
    }
   else
    {hdr->status &= ~_1_ECHO_IMAGE; 
     hdr->status |= _2_ECHO_IMAGE;
     hdr->data_info.num_echoes = 2;
    }
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
must_be_supplied - ???
--------------------------------------------------------------------------*/

int
must_be_supplied(hdr,fields)
 struct hdr_rec *hdr;
 struct hdr_field_rec fields[TOTAL_FIELDS];
{
 int err;

 err = NO_ERROR;
 if (!fields[DIMX].has_been_read)
    {err = 1;
     printf("DIMX must be supplied...\n");
    }
 if (!fields[DIMY].has_been_read)
    {err = 1;
     printf("DIMY must be supplied...\n");
    }
 if (!fields[PIXEL_DEPTH_BITS].has_been_read)
    {err = 1;
     printf("PIXEL_DEPTH_BITS must be supplied...\n");
    }
 if (!fields[MAX_SLICES].has_been_read)
    {err = 1;
     printf("MAX_SLICES must be supplied...\n");
    }
 if (!fields[XPIXEL_SIZE].has_been_read)
    {err = 1;
     printf("XPIXEL_SIZE must be supplied...\n");
    }
 if (!fields[YPIXEL_SIZE].has_been_read)
    {err = 1;
     printf("YPIXEL_SIZE must be supplied...\n");
    }
 if (!fields[NUM_KIND_OF_IMAGES].has_been_read)
    {err = 1;
     printf("NUM_KIND_OF_IMAGES must be supplied...\n");
    }
 if (err == NO_ERROR)
    {if (!fields[MAX_PIXEL].has_been_read)
        hdr->data_info.max_pixel = 
                pow((double) 2.0, (double) hdr->data_info.pixel_depth_bits);
    }

 return err;
}


/*--------------------------------------------------------------------------
build_hdr_from_text - ???
--------------------------------------------------------------------------*/

int
build_hdr_from_text(fid,hdr,is_new,fields)
 FILE *fid;
 struct hdr_rec *hdr;
 int is_new;
 struct hdr_field_rec fields[TOTAL_FIELDS];
{
 int err,select,value;
 unsigned int num,lcv;
 float fl;
 char str[512];

 if (is_new)
    for (lcv = 0; lcv < TOTAL_FIELDS; lcv++) 
       fields[lcv].has_been_read = 0;
   else 
    for (lcv = 0; lcv < TOTAL_FIELDS; lcv++) 
       fields[lcv].has_been_read = 1;

 while (1)
    {num = 1;
     TRACE(printf("before RD_INT\n"))
     if ((err = RdText(fid,&select,&num,RD_INT,0)) != NO_ERROR) return err;
     TRACE(printf("after RD_INT select = %d\n",select))
     if (num != 1) 
        {printf("no more input.....num = %d err = %d\n",num,err);
         break;}
     if ((select < 0) || (select >= TOTAL_USER_DEFINED_FIELDS))
        if (select == END_OF_INPUT)
           {
            break;}
          else continue;
     TRACE(printf("build_hdr_from_text select = %d how_to_read = %d\n",select,fields[select].how_to_read))

     switch (fields[select].how_to_read)
        {case RD_TEXT_INPUT :
         case RD_TEXT_SEQUENCE :
            num = sizeof(str);
            if ((err = RdText(fid,str,&num,RD_TEXT,0)) != NO_ERROR) return err;
            TRACE(printf("str = %s\n",str))
/*
            rd_text(fid,str,&num,&err,RD_TEXT);
            if (err != NO_ERROR) return READ_ERROR;
*/
            assign_text_defaults(hdr,select,str);
            TRACE(printf("after assign_text_defaults\n"))
            break;
         case RD_INT_INPUT :
            num = 1;
            if ((err = RdText(fid,&value,&num,RD_INT,0)) != NO_ERROR) return err;
            if (num == 0) return READ_ERROR;
            assign_int_defaults(hdr,select,value);
            break;
         case RD_REAL_INPUT :
            num = 1;
            if ((err = RdText(fid,&fl,&num,RD_FLOAT,0)) != NO_ERROR) return err;
            if (num == 0) return READ_ERROR;
            assign_float_defaults(hdr,select,fl);
            break;
         case RD_MATRIX_INPUT :
            num = 16;
            if ((err = RdText(fid,hdr->data_info.trans_matrix,&num,RD_FLOAT,0))
                       != NO_ERROR) 
               return err;
            if (num != 16) return READ_ERROR;
            break;
         case RD_NUM_KIND_OF_IMAGES :
            if ((err = rd_num_kind_of_images(fid,hdr,is_new)) != NO_ERROR) 
               return err;
            break;
        }
     fields[select].has_been_read = 1;
    }
 TRACE(printf("before must_be_supplied\n"))
 err = must_be_supplied(hdr,fields);
 TRACE(printf("after must_be_supplied\n"))
 return err;
}


/*--------------------------------------------------------------------------
rd_the_hdr_file - ???
--------------------------------------------------------------------------*/

int
rd_the_hdr_file(in_outfn,hdr,is_all_supplied)
 char *in_outfn;
 struct hdr_rec *hdr;
 int *is_all_supplied;
{
 int err;

 *is_all_supplied = 0;
 if ((err = access(in_outfn, W_OK | R_OK)) != 0)
    return NO_ERROR;
 if ((err = rd_the_hdr(in_outfn,hdr)) == NO_ERROR)
    *is_all_supplied = 1;

 return NO_ERROR;
}


/*--------------------------------------------------------------------------
construct_hdr_from_text - ???
--------------------------------------------------------------------------*/

int
construct_hdr_from_text(inp_textfn,out_hdrfn,inp_hdrfn)
 char *inp_textfn,*out_hdrfn,*inp_hdrfn;
{
 FILE *in_textfid;
 struct hdr_rec hdr;
 int err,is_new;
 char fn[250];
 struct mark_hdr mark;

 init_hdr_rec(&hdr);
 if (inp_hdrfn == NULL)
     {
      is_new = 1;
     }
    else
     {if ((err = rd_the_hdr(inp_hdrfn,&hdr)) != NO_ERROR) return err;
      is_new = 0;
     }

 if ((in_textfid = fopen(inp_textfn,"r")) == NULL)
    {printf("Error opening the input fn = %s\n",inp_textfn);
     return FILE_ACCESS_ERROR;}

 if (((err = build_hdr_from_text(in_textfid,&hdr,is_new,mri_hdr_fields)) == NO_ERROR) &&
     ((err = build_extra_file_info_rec(&(hdr.extra_info))) == NO_ERROR) &&
     ((err = BuildRiaHdr(&hdr,0)) == NO_ERROR))
    {
     sprintf(fn,"%s.nhdr",out_hdrfn);
     TRACE(printf("after pr_hdr_rec\n"))
     if ((err = wr_the_hdr(fn,&hdr)) == NO_ERROR)
        {printf(".... %s - RIA header file was generated...\n",fn);
         if ((err = assign_nhdr_to_hdr(&mark,&hdr)) == NO_ERROR)
            {sprintf(fn,"%s.hdr",out_hdrfn);
             if (wr_hdr_file(fn,&mark) == NO_ERROR)
                printf(".... %s - MRVBR header file was generated...\n",fn);
            }
         }
     TRACE(printf("after wr_the_hdr\n"))
    }
 fclose(in_textfid);
 return err;
}
