#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include "lib/mri_format.h"
#include "lib/segment.h"
/*
csh lgrow_region_batch.com
cc -c grow_region_batch.c
*/
/*
#define PRINT(x) x; fflush(stdout);
#define TIME(x) x; fflush(stdout);
#define TRACE(x) x; fflush(stdout);
*/
#define PRINT(x)
#define TIME(x)
#define TRACE(x)

/*
process_grow_region(fn,ndatfn,order)
prog_usage(argc,argv)
get_params(argc,argv,dirfn,fn_patt,ndatdir,prefix,order);
main(argc,argv)
*/
char *locate_dirfn();


process_grow_region(fn,ndatfn,order)
 char *fn,*ndatfn;
 struct strs_rec *order;
{
 struct hdr_rec hdr;
 struct output_hdr_rec ndathdr;
 int err;
 struct histo_params_rec histo;
 struct pixel_type_rec pixel_type;

 struct slice_roi_rec *slice_rois;
 long total_slice_rois;
 struct text_output_analysis_group_rec *groups;
 unsigned_int num_groups;
 struct output_text_format_rec format;
 struct count_analysis_rec count;

 FILE *imgfid;
 char *reason,*imgfn,*temp,*pixfn;

 TRACE(printf("process_grow_region...fn = %s\n",fn))
 init_hdr_rec(&hdr);
 temp = imgfn = NULL;
 if (((err = cp_str(&temp,fn)) != NO_ERROR) ||
     ((err = drop_suffix(temp)) != NO_ERROR) ||
     ((err = concat_strs(&imgfn,2,temp,".img")) != NO_ERROR))
    {printf("Error constructing fn = %s...err = %d\n",fn,err);
     c_free(temp);
     c_free(imgfn);
     return err;
    }
 c_free(temp);

 if ((err = rd_the_hdr(fn,&hdr)) != NO_ERROR)
    {printf("Error reading the hdr file = %s...err = %d\n",fn,err);
     c_free(imgfn);
     return err;
    }
 TRACE(printf("process_grow_region...imgfn = %s\n",imgfn))

 if ((imgfid = fopen(imgfn,"r")) == NULL)
    {file_access(imgfn,4,NULL,&reason);
     printf("Error opening the img file = %s...reason = %s\n",imgfn,reason);
     free_hdr_rec(&hdr);
     c_free(reason);
     c_free(imgfn);
     return FILE_ACCESS_ERROR;}
 TRACE(printf("process_grow_region...ndatfn = %s\n",ndatfn))


 TRACE(printf("process_grow_region...2\n"))
 if ((err = RdTheNdatFields(ndatfn,&ndathdr,&slice_rois,&total_slice_rois,
        &histo,&pixel_type,&groups,&num_groups,&format,&count)) != NO_ERROR)
    {if (err == FILE_ACCESS_ERROR)
        {file_access(ndatfn,14,NULL,&reason);
         printf("Error opening the ndat file = %s...reason = %s\n",ndatfn,reason);}
       else printf("Error reading the ndat file...ndatfn = %s err = %d\n",ndatfn,err);
    }
 TRACE(printf("process_grow_region...3\n"))

 if (err == NO_ERROR)
    {if (hdr.data_info.num_echoes == 2)
        err = GrowRegion2Echo(imgfid,&hdr,&histo,&pixel_type,order,-1,-1);
       else err = grow_region_1_echo(imgfid,&hdr,&histo,&pixel_type,order);
     if (err != NO_ERROR)
        printf("Error segmenting the slices...%s err = %d\n",fn,err);
       else
        {pixfn = NULL;
         if (((err = cp_str(&pixfn,ndatfn)) == NO_ERROR) &&
             ((err = drop_suffix(pixfn)) == NO_ERROR))
            {TRACE(printf("before wr_file_pixmap...\n"))
             err = wr_file_pixmap("",pixfn,PIXMAP_SUFFIX,&pixel_type,&reason);
             if (err != NO_ERROR)
                printf("Error writing the pixmap = %s%s...reason = %s\n",pixfn,
                            PIXMAP_SUFFIX,reason);
               else
                {TRACE(printf("before WrRiaNdatFile...\n"))
                 if (WrRiaNdatFile(ndatfn,
                        &ndathdr,slice_rois,total_slice_rois,
                        &histo,&pixel_type,
                        groups,num_groups,&format,&count) != NO_ERROR)
                    printf("Error saving %s !!!\n",ndatfn);
                   else printf("...%s updated!!!\n",ndatfn);
                }

            }
           else printf("Memory error...err = %d\n",err);
         c_free(pixfn);
        }
    }
 TRACE(printf("before free_ndat_fields...\n"))
 free_ndat_fields(&ndathdr,slice_rois,total_slice_rois,
              &histo,&pixel_type,groups,num_groups,&format,&count);

 TRACE(printf("process_grow_region...end...err = %d\n",err))
 free_hdr_rec(&hdr);
 c_free(imgfn);
 fclose(imgfid);
 return err;
}

prog_usage(argc,argv)
 int argc;
 char *argv[];
{
 if ((argc != 2) && (argc != 5))
    {printf("\nUsage : %s nhdr-fn ndat-dir ndat-prefix /exusr/mri/ria/pixel_types.order\n",argv[0]);
     printf("Example 1 : %s mri10257a.nhdr /omri/ndat abc /exusr/mri/ria/priority\n",argv[0]);
     printf("   Input : mri10257a.nhdr\n\n");
     printf("         : /omri/ndat/abcmri10257a.ndat\n\n");
     printf("         : /exusr/mri/ria/pixel_types.order\n\n");
     printf("   Output : /omri/ndat/abcmri10257a.pixmap\n\n");
     printf("\n");
     printf("Example 2 : %s \"/amri/*.nhdr\" /omri/ndat abc /exusr/mri/ria/pixel_types.order\n",
             argv[0]);
     printf("   Input : /amri/*.nhdr\n\n");
     printf("         : /omri/ndat/abc*.ndat\n\n");
     printf("         : /exusr/mri/ria/pixel_types.order\n\n");
     printf("   Output : /omri/ndat/abc*.pixmap\n\n");
     printf("\n");
     printf("Example 3 : %s \"/amri/*.nhdr\" /omri/ndat \"\" /exusr/mri/ria/pixel_types.order\n",
             argv[0]);
     printf("   Input : /amri/*.nhdr\n\n");
     printf("         : /omri/ndat/*.ndat\n\n");
     printf("         : /exusr/mri/ria/pixel_types.order\n\n");
     printf("   Output : /omri/ndat/*.pixmap\n\n");
     printf("\n");
     printf("Example 4 : %s grow_region_params.dat\n",argv[0]);
     printf("   Input : grow_region_params.dat\n\n");
     printf("   Output : as specified in grow_region_params.dat\n\n");
     printf("\n");

     printf("Note : if you use '*' in the file specification, please\n");
     printf("       enclose that file specification by double quotes.\n");
     exit(1);}
}

get_params(argc,argv,dirfn,fn_patt,ndatdir,prefix,order)
 int argc;
 char *argv[];
 char **dirfn,**fn_patt,**ndatdir,**prefix;
 struct strs_rec *order;
{
 int err;

 init_strs_rec(order);
 if (argc != 2)
    {*dirfn = (char *) locate_dirfn(argv[1]);
     if (*dirfn == NULL) 
        if ((err = cp_str(dirfn,".")) != NO_ERROR) return err;
     drop_dirfn(argv[1]);
     if (((err = cp_str(fn_patt,argv[1])) != NO_ERROR) ||
         ((err = cp_str(ndatdir,argv[2])) != NO_ERROR) ||
         ((err = cp_str(prefix,argv[3])) != NO_ERROR))
        return err;
     if ((err = rd_ascii_file_strs_to_strs_rec(argv[4],order,0)) != NO_ERROR)
        {printf("Error reading the pixel type order = %s...err = %d\n",argv[4],err);
         return err;}
     }
   else
    {exit(1);
    }
}

main(argc,argv)
 int argc;
 char *argv[];
{
 int rater;
 int err;
 char **fns,*dirfn,*fn_patt,*fn,*temp,*ndatfn,*ndatdir,*prefix;
 unsigned int num_fns;
 unsigned long lcv;
 struct strs_rec order,errlog;

 prog_usage(argc,argv);
 if ((err = get_params(argc,argv,&dirfn,&fn_patt,&ndatdir,&prefix,&order)) != NO_ERROR)
    {printf("Error getting parameters...err = %d\n",err);
     exit(err);}

 if ((err = get_fns(dirfn,NULL,0,0,NULL,&fns,&num_fns,1,fn_patt,
         NULL,0,0,NULL)) != NO_ERROR) exit(err);

 err = CANCEL;
 if (num_fns == 0)
    {printf("Sorry...No file names...\n");
    }
   else
    {
     printf("\n");
     init_strs_rec(&errlog);
     for (lcv = 0; lcv < num_fns; lcv++)
        {fn = temp = ndatfn = NULL;
         if (((err = concat_strs(&fn,3,dirfn,"/",fns[lcv])) == NO_ERROR) &&
             ((err = cp_str(&temp,fn)) == NO_ERROR) &&
             ((err = drop_suffix(temp)) == NO_ERROR) &&
             ((err = drop_dirfn(temp)) == NO_ERROR) &&
             ((err = concat_strs(&ndatfn,5,ndatdir,"/",prefix,temp,".ndat")) == 
                       NO_ERROR))
            {printf("%d) processing %s & %s\n",lcv,fn,ndatfn);
             if ((err = process_grow_region(fn,ndatfn,&order)) != NO_ERROR)
                {printf("...%d) ERROR processing %s & %s...err = %d\n",
                       lcv,fn,ndatfn,err);
                 add_str_to_strs_rec(&errlog,fn);
                 err = NO_ERROR;}
            }
         c_free(fn);
         c_free(temp);
         c_free(ndatfn);
         if (err != NO_ERROR)
            {
             printf("...%d) ERROR processing %s/%s\n",lcv,dirfn,fns[lcv]);
             printf("Quiting...\n");
             break;}
        }
     if (errlog.num_strs > 0)
        {printf("\nErrors occurred when processing the following files :-\n");
         for (lcv = 0; lcv < errlog.num_strs; lcv++)
            printf("%d) %s\n",lcv,errlog.strs[lcv]);
         printf("\n");}
     free_strs_rec(&errlog);
    }
 c_free(dirfn);
 c_free(fn_patt);
 c_free(prefix);
 c_free(ndatdir);
 free_strs_rec(&order);
 exit(err);
}
