/****************************************************************************
3d_co_register_menu.c - Handle menu for 3d image co-registration ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: 3d_co_register_menu.c,v 1.8 1999/07/07 17:41:13 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/wmgr.h>

#include "buf_3d_rec.h"
#include "geometry.h"
#include "mri_format.h"
#include "nria_procedures.h"



#if 0
#  define PRINT(x) printf(x); fflush(stdout);
#  define PRT2(x) x; fflush(stdout);
#  define PRT(x) x; fflush(stdout);
#else
#  define PRINT(x)
#  define PRT2(x)
#  define PRT(x)
#endif



/* ??? */
#define CLEAN_UP_3D_CO_REGISTER 0
#define SETUP_CO_REGISTER 1
#define GET_ALL_3D_CO_REGISTER_PTR 2



/*-----------------------------------------------------------------------
struct co_menu_rec & all_co_menu_recs - ???
-----------------------------------------------------------------------*/

struct co_menu_rec
   {
    unsigned short status;
    struct image_display_window *win;
    struct display_win_rec *dwin;
    Frame frame;
    Panel panel;
    Panel_item co_plane,lock_plane;
   };

struct all_co_menu_recs
   {
    unsigned long num_3ds;
    struct co_menu_rec **_3ds;
   };



/* Local functions */
void init_co_menu_rec(struct co_menu_rec *_3d);
void free_co_menu_rec(struct co_menu_rec *_3d);
void init_all_co_menu_recs(struct all_co_menu_recs *all_3ds);
void _3d_co_done_proc(Frame frame);
void co_register_proc(Panel_item item, char *wid, caddr_t client_data,
    Panel_list_op op, Event *event);
void co_adjust_mode_more_proc(Panel_item item, Event *event);
void co_more_proc(Panel_item item, Event *event);
int select_co_registered_items(struct co_menu_rec *_3d, Panel_item item);
int setup_co_menu_rec(struct co_menu_rec *_3d,
    struct image_display_window *win);
int update_co_register_menu(struct image_display_window *win);
struct co_menu_rec *find_co_menu_rec(struct all_co_menu_recs *all_3ds,
    struct image_display_window *win);
int enter_3d_co_menu(struct image_display_window *win, int status);



/*-----------------------------------------------------------------------
init_co_menu_rec - ???
-----------------------------------------------------------------------*/

void
init_co_menu_rec(_3d)
 struct co_menu_rec *_3d;
{
 _3d->status = 0;
 _3d->win = NULL;
 _3d->frame  = 0;
}



/*-----------------------------------------------------------------------
free_co_menu_rec - ???
-----------------------------------------------------------------------*/

void
free_co_menu_rec(_3d)
 struct co_menu_rec *_3d;
{
 struct all_co_menu_recs *all_3ds;

 PRT(printf("free_co_menu_rec...\n"))
 if (_3d->frame != 0)  xv_destroy_safe(_3d->frame);
 all_3ds = (struct all_co_menu_recs *) enter_3d_co_menu(NULL,
            GET_ALL_3D_CO_REGISTER_PTR);
 c_free(_3d);
 DropHandleAryPtr(&(all_3ds->_3ds),&(all_3ds->num_3ds),_3d);
 PRT(printf("free_co_menu_rec...end\n"))
}



/*-----------------------------------------------------------------------
init_all_co_menu_recs - ???
-----------------------------------------------------------------------*/

void
init_all_co_menu_recs(
    struct all_co_menu_recs *all_3ds)
{
 all_3ds->num_3ds = 0;
 all_3ds->_3ds = NULL;
}



/*-----------------------------------------------------------------------
_3d_co_done_proc - ???
-----------------------------------------------------------------------*/

void
_3d_co_done_proc(frame)
 Frame frame;
{
 struct co_menu_rec *_3d;

 _3d = (struct co_menu_rec *) xv_get(frame,WIN_CLIENT_DATA);
 free_co_menu_rec(_3d);
}



/*-----------------------------------------------------------------------
co_register_proc - ???
-----------------------------------------------------------------------*/

void
co_register_proc(item,wid,client_data,op,event)
 Panel_item item;
 char *wid;
 caddr_t client_data;
 Panel_list_op op;
 Event *event;
{
 struct co_menu_rec *_3d;
 
 _3d = (struct co_menu_rec *) xv_get(item,PANEL_CLIENT_DATA);
 PRT2(printf("co_register_proc op = %d\n",op))
 if (op) win_co_register_to_wid(_3d->win,wid);
   else WinUncoRegisterFromWid(_3d->win,wid,1);
}



/*-----------------------------------------------------------------------
co_adjust_mode_more_proc - ???
-----------------------------------------------------------------------*/

void
co_adjust_mode_more_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct co_menu_rec *_3d;

 _3d = (struct co_menu_rec *) xv_get(item,PANEL_CLIENT_DATA);
 notice_prompt(_3d->frame,(Event *) NULL,
            NOTICE_MESSAGE_STRINGS,
               "When the adjust-co-registered mode is set on for a WID, ",
               "a letter J appears in the label displayed on the ",
               "bottom-right corner of the WID window, and any subsequent",
               "change in the orientation of that WID plane will not",
               "cause the orientation of its corresponding co-WID to change",
               "automatically.  Instead, the new plane orientation of that WID",
               "is reco-registered with its co-WID, replacing the old parameters.",
               "",
               "J -- Set the adjust-co-registered mode & lock on for the main",
               "co-registered WID and all its coplane.",
               "Sh J -- Set the adjust-co-registered mode & lock off for the main",
               "co-registered WID and all its coplane.",
               "Ctrl J -- Set the adjust-co-registered mode on for the ",
               "current WID only",
               "Ctrl Sh J -- Set the adjust-co-registered mode off for the ",
               "current WID only",
               "",
               "<> Ins -- Adjust and reco-register using the last plane",
              0,
            NOTICE_BUTTON_YES," OK !",
            0);
}



/*-----------------------------------------------------------------------
co_more_proc - ???
-----------------------------------------------------------------------*/

void
co_more_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct co_menu_rec *_3d;

 _3d = (struct co_menu_rec *) xv_get(item,PANEL_CLIENT_DATA);
 notice_prompt(_3d->frame,(Event *) NULL,
            NOTICE_MESSAGE_STRINGS,
               "When a WID P is co-registered with another WID Q,",
               " P is called the co-WID of Q, and vice-versa.",
               "If any WID R from the same group of P is co-registered with ",
               "another WID S from the same group of Q based on the",
               "co-registered parameters of P & Q, R is called",
               "the co-plane of P, S the co-plane of Q.",
               "",
               "A -- Same co-plane orientation as WID & superimpose the two",
               "Sh A -- Same co-plane orientation as WID",
               "<> A -- Superimpose the 2 co-registered WIDs",
               "",
               "L -- Lock all co-planes of WID",
               "Sh L -- Unlocked all co-planes of WID",
               "Ctrl L -- Locked to WID indicated by the cur roi",
               "Sh Ctrl L -- Unlocked from WID indicated by the cur roi",
               "SH <> L -- Unlocked WID from other WIDS in the same group",
               "Sh O -- Unlocked all WIDs group in the same group",
               "Ctrl O -- Unlocked every WID from WIDs",
               "",
               "K -- Co-register with WID",
               "Sh K -- Unco-register every WID from any group",
               "Ctrl K -- Unco-register with WID",
               "<> K -- Reco-register all registered group WIDs",
               "     -- based on the cur WID & its co-reg WID",
               "Sh I -- Unco-register all group WIDs with other groups",
               "Ctrl I -- Unco-register cur WID from any group",
               "",
               "Ctrl Sh O -- Unlock & unco-register every WID from every group",
               "",
               "<> F9 -- display absolute or co-reg xyz cordinate",
               "Ctrl Sh Mleft -- locate the cur xyz on the co-reg WID",
              0,
            NOTICE_BUTTON_YES," OK !",
            0);
}



/*-----------------------------------------------------------------------
select_co_registered_items - ???
-----------------------------------------------------------------------*/

int
select_co_registered_items(_3d,item)
 struct co_menu_rec *_3d;
 Panel_item item;
{
 struct img_buf_rec *img;
 struct view_rec *view;
 struct strs_rec wids;
 int err;

 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return CANCEL;
 view = img->cur_slice->slice->view;
 init_strs_rec(&wids);
 if ((err = co_registered_wids(view,&wids,0,1,3)) == NO_ERROR)
    select_list_items(&wids,item);
 free_strs_rec_ptrs(&wids);

 return err;
}



/*-----------------------------------------------------------------------
setup_co_menu_rec - ???
-----------------------------------------------------------------------*/

int
setup_co_menu_rec(_3d,win)
 struct co_menu_rec *_3d;
 struct image_display_window *win;
{
 int row,offset;
 struct img_buf_rec *img;
 struct view_rec *view;
 char *str;
 Rect *rr;

 PRT2(printf("setup_co_menu_rec...\n"))
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 view = img->cur_slice->slice->view;

 _3d->frame = xv_create(win->dwin->frame, FRAME,
              FRAME_INHERIT_COLORS,TRUE,
              WIN_CLIENT_DATA,_3d,
              0);
 if (_3d->frame == 0)
    {free_co_menu_rec(_3d);
     return ~NO_ERROR;}
 _3d->win = win;
 _3d->dwin = win->dwin;
 str = win_rec_win_id_title(win->owner,_3D_CO_REGISTRATION);
 xv_set(_3d->frame,
              FRAME_SHOW_LABEL, TRUE,
              XV_LABEL,str,
              FRAME_NO_CONFIRM,TRUE,
              FRAME_DONE_PROC,_3d_co_done_proc,
              0);
 c_free(str);
 xv_set(_3d->frame,FRAME_CLOSED,FALSE,0);
 PRT2(printf("setup_co_menu_rec...1\n"))

 _3d->panel = xv_create(_3d->frame,PANEL,
               FRAME_INHERIT_COLORS,TRUE,
               0);

 if (_3d->panel == 0)
    {free_co_menu_rec(_3d);
     return ~NO_ERROR;}
 row = 5;
 offset = xv_rows(_3d->panel,1) - xv_rows(_3d->panel,0) - 10;
 xv_create(_3d->panel,PANEL_MESSAGE,
        PANEL_LABEL_STRING,"Planes from the same group of wids can be",
        XV_X,5,XV_Y,row,0);
 row += offset - 5;
 xv_create(_3d->panel,PANEL_MESSAGE,
        PANEL_LABEL_STRING,"locked, and co-registered among groups.",
        XV_X,5,XV_Y,row,0);
 row += offset;
 _3d->co_plane = xv_create(_3d->panel,PANEL_LIST,
	         PANEL_LABEL_STRING,"CO-REGISTER",
                 PANEL_CHOOSE_ONE,FALSE,
                 PANEL_LIST_DISPLAY_ROWS,4,
                 PANEL_LIST_WIDTH,100,
                 PANEL_LAYOUT,PANEL_VERTICAL,
	          PANEL_NOTIFY_PROC,co_register_proc,
                  PANEL_CLIENT_DATA,_3d,
                  XV_X,5,XV_Y,row,
                 0);
 setup_view_window_list(img,_3d->co_plane,0,2,1,0,view->group_num);
 PRT2(printf("setup_co_menu_rec...5\n"))
 if ((view->status & PLANE_CO_REGISTERED) && (view->num_cos > 0))
    select_co_registered_items(_3d,_3d->co_plane);
 rr = (Rect *) xv_get(_3d->co_plane,XV_RECT);
 _3d->lock_plane = xv_create(_3d->panel,PANEL_LIST,
	         PANEL_LABEL_STRING,"LOCK PLANE",
                 PANEL_CHOOSE_ONE,FALSE,
                 PANEL_LIST_DISPLAY_ROWS,4,
                 PANEL_LIST_WIDTH,100,
                 PANEL_LAYOUT,PANEL_VERTICAL,
	          PANEL_NOTIFY_PROC,co_register_proc,
                  PANEL_CLIENT_DATA,_3d,
                  XV_X,rr->r_left + rr->r_width + 2,XV_Y,row,
                 0);
 setup_view_window_list(img,_3d->lock_plane,0,0,0,0,0);
 PRT2(printf("setup_co_menu_rec...5\n"))
 if ((view->status & PLANE_CO_REGISTERED) && (view->num_cos > 0))
    select_co_registered_items(_3d,_3d->lock_plane);
 row = rr->r_top + rr->r_height + 2;
 xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"DEFINITIOIN/COMMAND HELP...",
	PANEL_NOTIFY_PROC,co_more_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,
	0);
 row += offset;
 xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"ADJUST-CO-REGISTERED MODE...",
	PANEL_NOTIFY_PROC,co_adjust_mode_more_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,
	0);

 window_fit(_3d->panel);
 rr = (Rect *) xv_get(_3d->panel,XV_RECT);
 xv_set(_3d->panel,XV_WIDTH,rr->r_width - 20,
           XV_HEIGHT,rr->r_height - 10,0);

 PRT2(printf("setup_co_menu_rec...11\n"))
 window_fit(_3d->frame);
 make_2_wins_adjacent(_3d->frame,win->dwin->frame,0);
 win_set_mouse_at_center(_3d->frame);

 return NO_ERROR;
}



/*-----------------------------------------------------------------------
update_co_register_menu - ???
-----------------------------------------------------------------------*/

int
update_co_register_menu(win)
 struct image_display_window *win;
{
 struct all_co_menu_recs *all_3ds;
 struct co_menu_rec *_3d;
 int is_clear;
 struct img_buf_rec *img;
 struct view_rec *view;

 if (((all_3ds = (struct all_co_menu_recs *) 
       enter_3d_co_menu(win,GET_ALL_3D_CO_REGISTER_PTR)) != NULL) &&
     ((_3d = find_co_menu_rec(all_3ds,win)) != NULL))
    {deselect_all_list_items(_3d->co_plane);
     deselect_all_list_items(_3d->lock_plane);
     select_co_registered_items(_3d,_3d->co_plane);
     select_co_registered_items(_3d,_3d->lock_plane);
    }
 is_clear = 1;
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 view = img->cur_slice->slice->view;
 win_execute_proc_to_multiple_wins(win,win_draw_enabled_label,&is_clear,
				   0,0,0,0);

 return NO_ERROR;
}



/*-----------------------------------------------------------------------
find_co_menu_rec - ???
-----------------------------------------------------------------------*/

struct co_menu_rec *
find_co_menu_rec(
    struct all_co_menu_recs *all_3ds,
    struct image_display_window *win)
{
 int lcv;

 for (lcv = 0; lcv < all_3ds->num_3ds; lcv++)
    if (all_3ds->_3ds[lcv]->win == win)
       return all_3ds->_3ds[lcv];
 return NULL;
}



/*-----------------------------------------------------------------------
enter_3d_co_menu - ???
-----------------------------------------------------------------------*/

int
enter_3d_co_menu(win,status)
     struct image_display_window *win;
     int status;
{
  static struct all_co_menu_recs all_3ds;
  static char first_time = 1;
  int err;
  struct co_menu_rec *_3d;

  err = NO_ERROR;

  if (status == GET_ALL_3D_CO_REGISTER_PTR)
  {
    if (first_time)
      return 0;
    else
      return (unsigned long) &all_3ds;     /* FIXME:  ptr as long */
  }

  if (status == CLEAN_UP_3D_CO_REGISTER)
  {
    if (!first_time)
      if ((_3d = find_co_menu_rec(&all_3ds,win)) != NULL)
	free_co_menu_rec(_3d);
    return NO_ERROR;
  }

  if (first_time)
  {
    init_all_co_menu_recs(&all_3ds);
    first_time = 0;
  }
  else
  { 
    if ((_3d = find_co_menu_rec(&all_3ds,win)) != NULL)
    {
      xv_set( _3d->frame,
	      XV_SHOW, TRUE,
	      FRAME_CLOSED, FALSE,
	      NULL );
      win_alarm( _3d->frame, 1 );
      update_3d_menu(win);
      win_set_mouse_at_center(_3d->frame);
      wmgr_top(_3d->frame);
      return NO_ERROR;
    }
  }
  get_handle_memory(&(all_3ds._3ds),&(all_3ds.num_3ds),1,
		    (size_t) sizeof(struct co_menu_rec),&err);
  if (err != NO_ERROR)
    return err;
  _3d = all_3ds._3ds[all_3ds.num_3ds - 1];
  init_co_menu_rec(_3d);
  if ((err = setup_co_menu_rec(_3d,win)) == NO_ERROR)
    xv_set(_3d->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
  return err;
}
