/****************************************************************************
3d_file_menu.c - Handle menu for operations on 3d image files???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: 3d_file_menu.c,v 1.8 1999/07/07 17:41:13 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/wmgr.h>

#include "buf_3d_rec.h"
#include "geometry.h"
#include "mri_format.h"
#include "nria_procedures.h"


#if 0
#   define TRACE(x) x; fflush(stdout);
#   define PRT2(x)  x; fflush(stdout);
#   define PRT(x)   x; fflush(stdout);
#   define PRT7(x)  x; fflush(stdout);
#   define PRT8(x)  x; fflush(stdout);
#else
#   define TRACE(x)
#   define PRT(x)
#   define PRT2(x)
#   define PRT7(x)
#   define PRT8(x)
#endif



/* ??? */
#define CLEAN_UP_3D_MENU 0
#define SETUP_3D_MENU 1
#define GET_ALL_3D_MENU_PTR 2


/* Local structures */
struct file_menu_3d_rec
   {
    unsigned short status;
    struct image_display_window *win;
    struct display_win_rec *dwin;
    Frame frame;
    Panel panel;
    Panel_item order_msg;
/*    Panel_item group,apply_to,order_msg,origin; */
   };


struct all_file_menu_3d_recs
   {
    int num_3ds;
    struct file_menu_3d_rec **_3ds;
   };


/* Local functions */
void init_file_menu_3d_rec(struct file_menu_3d_rec *_3d);
void free_file_menu_3d_rec(struct file_menu_3d_rec *_3d);
void init_all_file_menu_3d_recs(struct all_file_menu_3d_recs *all_3ds);
void _3d_file_done_proc(Frame frame);
int open_view_win(struct view_rec *view);
int close_view_win(struct view_rec *view);
int win_reverse_image_order(struct image_display_window *win);
int reverse_image_order_proc(Panel_item item, Event *event);
int win_save_cur_image_order(struct image_display_window *win);
int save_cur_image_order_proc(Panel_item item, Event *event);
int win_cur_default_image_order(struct image_display_window *win);
int cur_default_image_order_proc(Panel_item item, Event *event);
int win_default_image_order(struct image_display_window *win);
int default_image_order_proc(Panel_item item, Event *event);
int setup_file_menu_3d_rec(struct file_menu_3d_rec *_3d,
    struct image_display_window *win);
void cur_3d_order_msg_proc(struct image_display_window *win);
struct file_menu_3d_rec *find_file_menu_3d_rec(
    struct all_file_menu_3d_recs *all_3ds, struct image_display_window *win);
int enter_3d_file_menu(struct image_display_window *win, int status);



/*------------------------------------------------------------------------
init_file_menu_3d_rec - ???
------------------------------------------------------------------------*/

void
init_file_menu_3d_rec(_3d)
 struct file_menu_3d_rec *_3d;
{
 _3d->status = 0;
 _3d->win = NULL;
 _3d->frame = 0;
}



/*------------------------------------------------------------------------
free_file_menu_3d_rec - ???
------------------------------------------------------------------------*/

void
free_file_menu_3d_rec(_3d)
 struct file_menu_3d_rec *_3d;
{
 struct all_file_menu_3d_recs *all_3ds;

 PRT7(printf("free_file_menu_3d_rec...\n"))
 if (_3d->frame != 0)  xv_destroy_safe(_3d->frame);
 all_3ds = (struct all_file_menu_3d_recs *) 
          enter_3d_file_menu(NULL,GET_ALL_3D_MENU_PTR);
 c_free(_3d);
 DropHandleAryPtr(&(all_3ds->_3ds),&(all_3ds->num_3ds),_3d);
 PRT7(printf("free_file_menu_3d_rec...end\n"))
}



/*------------------------------------------------------------------------
init_all_file_menu_3d_recs - ???
------------------------------------------------------------------------*/

void
init_all_file_menu_3d_recs(all_3ds)
 struct all_file_menu_3d_recs *all_3ds;
{
 all_3ds->num_3ds = 0;
 all_3ds->_3ds = NULL;
}



/*------------------------------------------------------------------------
_3d_file_done_proc - ???
------------------------------------------------------------------------*/

void
_3d_file_done_proc(frame)
 Frame frame;
{
 struct file_menu_3d_rec *_3d;

 _3d = (struct file_menu_3d_rec *)
                     xv_get(frame,WIN_CLIENT_DATA);
 free_file_menu_3d_rec(_3d);
}



/*------------------------------------------------------------------------
open_view_win - ???
------------------------------------------------------------------------*/

int
open_view_win(view)
 struct view_rec *view;
{
 if ((view->img != NULL) && (view->img->win != NULL) &&
     (view->img->win->image_display != NULL))
    win_show_multiple_windows(view->img->win->image_display);
 return NO_ERROR;
}



/*------------------------------------------------------------------------
close_view_win - ???
------------------------------------------------------------------------*/

int
close_view_win(view)
 struct view_rec *view;
{
 if ((view->img != NULL) && (view->img->win != NULL) &&
     (view->img->win->image_display != NULL))
    win_close_multiple_windows(view->img->win->image_display);
 return NO_ERROR;
}


/*************  Massive chop-out starts here ************/

#if 0

win_file_view_group_nums(win,group_num,total,num)
 struct image_display_window *win;
 int **group_num;
 long *total,*num;
{
 struct img_buf_rec *img;
 struct slice_info_rec *slice_info;
 long lcv,cnt;
 int err;

 *total = *num = 0;
 *group_num = NULL;
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 slice_info = &(img->file->slice_info);
 if (slice_info->num_views == 0) return NO_ERROR;
 if ((err = get_ptr(group_num,total,slice_info->num_views,
       (size_t) sizeof(**group_num))) != NO_ERROR)
    return err;
 for (lcv = 0; lcv < *total; lcv++) 
    (*group_num)[lcv] = slice_info->views[lcv]->group_num;

 qsort(*group_num,*total,
        (size_t) sizeof(**group_num),sort_asc_int_proc);
 for (cnt = 0,lcv = 1; lcv < *total; lcv++)
    if ((*group_num)[cnt] != (*group_num)[lcv])
       {cnt += 1;
        if (cnt != lcv) (*group_num)[cnt] = (*group_num)[lcv];
       }
 *num = cnt + 1;
 return NO_ERROR;
}


setup_groups(_3d)
 struct file_menu_3d_rec *_3d;
{
 int *group_num,err;
 long total,num;

 err = win_file_view_group_nums(_3d->win,&group_num,&total,&num);
 if (err == NO_ERROR)
    setup_int_items(group_num,num,_3d->group,0);
 if (total > 0) c_free(group_num);
 return err;
}


select_group(_3d,view)
 struct file_menu_3d_rec *_3d;
 struct view_rec *view;
{
 char str[40];

 sprintf(str,"%d",view->group_num);
 select_list_str_item(str,_3d->group,1);
}


add_group_list(_3d,view)
 struct file_menu_3d_rec *_3d;
 struct view_rec *view;
{
 char str[40];
 int num_rows;

 sprintf(str,"%d",view->group_num);
 num_rows = (int) xv_get(_3d->group,PANEL_LIST_NROWS);
 xv_set(_3d->group,PANEL_LIST_STRING,num_rows,str,
        PANEL_LIST_SELECT,num_rows,0,0);

}


change_group_proc(item,str,client_data,op,event)
 Panel_item item;
 char *str;
 caddr_t client_data;
 Panel_list_op op;
 Event *event;
{
 struct file_menu_3d_rec *_3d;
 struct img_buf_rec *img;
 int group_num;
 struct view_rec *view;

 _3d = (struct file_menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);

 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return CANCEL;
 sscanf(str,"%d",&group_num);
 TRACE(printf("%s...group_num = %d\n",str,group_num))
 if ((view = view_with_group_num(img->file,group_num)) != NULL)
    {delete_all_list_items(_3d->apply_to);
     setup_view_window_list(view->img,_3d->apply_to,0,0,0,0,0);
    }
}


delete_3d_group_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct file_menu_3d_rec *_3d;
 char *str;
 int val,group_num;
 struct view_rec *view;
 struct img_buf_rec *img;

 _3d = (struct file_menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return CANCEL;
 if (next_selected_item(_3d->group,0,&val) != NO_ERROR) return CANCEL;
 if ((str = (char *) xv_get(_3d->group,PANEL_LIST_STRING,val)) != NULL)
    {sscanf(str,"%d",&group_num);
     TRACE(printf("delete_3d_group_proc group_num = %d\n",group_num))
     if ((view = view_with_group_num(img->file,group_num)) != NULL)
        {}
    }
}


open_close_group_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct file_menu_3d_rec *_3d;
 char *str;
 int val,group_num;
 struct view_rec *view;
 struct img_buf_rec *img;

 _3d = (struct file_menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return CANCEL;
 if (next_selected_item(_3d->group,0,&val) != NO_ERROR) return CANCEL;

 if ((str = (char *) xv_get(_3d->group,PANEL_LIST_STRING,val)) != NULL)
    {sscanf(str,"%d",&group_num);
     TRACE(printf("open_close_group_proc group_num = %d\n",group_num))
     if ((view = view_with_group_num(img->file,group_num)) != NULL)
        if (item == _3d->open)
           run_next_view_proc(view,open_view_win);
          else run_next_view_proc(view,close_view_win);
    }
}


setup_origin_choices(_3d)
 struct file_menu_3d_rec *_3d;
{
 struct vector_3d_rec vertices[8];
 struct img_buf_rec *img;
 int lcv;
 char str[50];
 int select;

 PRT(printf("setup_origin_choices...1\n"))
 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return CANCEL;
 xv_set(_3d->origin,PANEL_CHOICE_STRINGS,"DEFAULT",
           "REVERSE 3D IMAGE ORDER - <> R",NULL,NULL);
 PRT(printf("setup_origin_choices...img->file = %u\n",img->file))
 if (img->file->slice_info.select_3d_gravity == _3D_SLICE_DEFAULT_ORDER)
    xv_set(_3d->origin,PANEL_VALUE,0,0);
   else if (img->file->slice_info.select_3d_gravity == _3D_SLICE_BOTTOM_TO_TOP)
    xv_set(_3d->origin,PANEL_VALUE,1,0);
}

#endif

/************ Massive chop-out ends here *************/



/*------------------------------------------------------------------------
win_reverse_image_order - ???
------------------------------------------------------------------------*/

int
win_reverse_image_order(win)
 struct image_display_window *win;
{
 struct img_buf_rec *img;
 struct view_rec *view;
 long lcv;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 if (img->file->slice_info.select_3d_gravity == _3D_SLICE_DEFAULT_ORDER)
    {
     img->file->slice_info.select_3d_gravity = _3D_SLICE_BOTTOM_TO_TOP;
    }
   else if (img->file->slice_info.select_3d_gravity == _3D_SLICE_BOTTOM_TO_TOP)
    {
     img->file->slice_info.select_3d_gravity = _3D_SLICE_DEFAULT_ORDER;
    }
   else return NO_ERROR;
 cur_3d_order_msg_proc(win);
 set_win_busy(win);
 for (lcv = 0; lcv < img->file->slice_info.num_3ds; lcv++)
    reverse_3d_slice_order(img->file->slice_info._3ds[lcv]);

 for (lcv = 0; lcv < img->file->slice_info.num_views; lcv++)
    {view = img->file->slice_info.views[lcv];
     BuildAndDisplayPlaneData(view);
    }
 set_win_not_busy(win);
 return NO_ERROR;    /* FIXME:  Added but not checked.  -PWH 19990319 */
}



/*------------------------------------------------------------------------
reverse_image_order_proc - ???
------------------------------------------------------------------------*/

int
reverse_image_order_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct file_menu_3d_rec *_3d;

 _3d = (struct file_menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 return win_reverse_image_order(_3d->win);
}



/*------------------------------------------------------------------------
win_save_cur_image_order - ???
------------------------------------------------------------------------*/

int
win_save_cur_image_order(win)
 struct image_display_window *win;
{
 struct img_buf_rec *img;
 int err;
 FILE *fid;
 char *the_fn;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 img->file->hdr.data_info.var_use_status |= _3D_IMAGE_ORDER_SET;
 if (img->file->slice_info.select_3d_gravity == _3D_SLICE_BOTTOM_TO_TOP)
    img->file->hdr.data_info.var_use_status |= _3D_IMAGE_BOTTOM_TO_TOP;
   else img->file->hdr.data_info.var_use_status &= ~_3D_IMAGE_BOTTOM_TO_TOP;
 if ((err = open_output_file(img->file->dirfn,img->file->fn,
            img->file->hdr_fn_suffix,&the_fn,&fid,win->dwin->frame,1)) == NO_ERROR)
    {if ((err = wr_hdr(fid,&(img->file->hdr))) != NO_ERROR)
        {
         ShowMsgStrs(win->dwin->frame,0,NO_ERROR,0,2,
               "Error updating the nhdr file !!!",the_fn,
                NULL,0);
        }
     fclose(fid);
    }
 c_free(the_fn);
 return err;
}



/*------------------------------------------------------------------------
save_cur_image_order_proc - ???
------------------------------------------------------------------------*/

int
save_cur_image_order_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct file_menu_3d_rec *_3d;

 _3d = (struct file_menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 return win_save_cur_image_order(_3d->win);
}



/*------------------------------------------------------------------------
win_cur_default_image_order - ???
------------------------------------------------------------------------*/

int
win_cur_default_image_order(win)
 struct image_display_window *win;
{
 struct img_buf_rec *img;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 if (img->file->hdr.data_info.var_use_status & _3D_IMAGE_BOTTOM_TO_TOP)
    {if (img->file->slice_info.select_3d_gravity == _3D_SLICE_BOTTOM_TO_TOP)
        return NO_ERROR;}
   else 
    {if (img->file->slice_info.select_3d_gravity == _3D_SLICE_DEFAULT_ORDER)
        return NO_ERROR;}
 win_reverse_image_order(win);
 return NO_ERROR;   /* FIXME:  Added but not checked.  -PWH 19990319 */
}



/*------------------------------------------------------------------------
cur_default_image_order_proc - ???
------------------------------------------------------------------------*/

int
cur_default_image_order_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct file_menu_3d_rec *_3d;

 _3d = (struct file_menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 return win_cur_default_image_order(_3d->win);
}



/*------------------------------------------------------------------------
win_default_image_order - ???
------------------------------------------------------------------------*/

int
win_default_image_order(win)
 struct image_display_window *win;
{
 struct img_buf_rec *img;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 if (img->file->slice_info.select_3d_gravity == _3D_SLICE_DEFAULT_ORDER)
    {ShowMsgStrs(win->dwin->frame,1,NO_ERROR,1,1,
                "Already in default image order !!!",NULL,0);
     return NO_ERROR;}
 win_reverse_image_order(win);
 return NO_ERROR;    /* FIXME:  Added but not checked. -PWH 19990319 */
}



/*------------------------------------------------------------------------
default_image_order_proc - ???
------------------------------------------------------------------------*/

int
default_image_order_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct file_menu_3d_rec *_3d;

 _3d = (struct file_menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 return win_default_image_order(_3d->win);
}



/*------------------------------------------------------------------------
setup_file_menu_3d_rec - ???
------------------------------------------------------------------------*/

int
setup_file_menu_3d_rec(_3d,win)
 struct file_menu_3d_rec *_3d;
 struct image_display_window *win;
{
 Cms cms;
 int row,offset;
 struct img_buf_rec *img;
 struct view_rec *view;
 char *str;
 Rect *rr;

 PRT(printf("setup_file_menu_3d_rec...1\n"))
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 view = img->cur_slice->slice->view;
 cms = (Cms) xv_get(win->dwin->frame,WIN_CMS);
 _3d->frame = xv_create(win->dwin->frame, FRAME,
			/*WIN_DYNAMIC_VISUAL,TRUE,*/
			FRAME_INHERIT_COLORS,TRUE,
			WIN_CLIENT_DATA,_3d,
			0);
 if (_3d->frame == 0)
    {free_file_menu_3d_rec(_3d);
     return ~NO_ERROR;}
 xv_set(_3d->frame,		/* DRW 19970811  See display_init_free.c */
	WIN_CMS,cms,
	NULL);
 _3d->win = win;
 _3d->dwin = win->dwin;
 PRT(printf("setup_file_menu_3d_rec...2\n"))
 str = win_rec_win_id_title(win->owner,_3D_IMAGE_OPTIONS);
 xv_set(_3d->frame,
              FRAME_SHOW_LABEL, TRUE,
              XV_LABEL,str,
              FRAME_NO_CONFIRM,TRUE,
              FRAME_DONE_PROC,_3d_file_done_proc,
              0);
 c_free(str);
 xv_set(_3d->frame,FRAME_CLOSED,FALSE,0);

 _3d->panel = xv_create(_3d->frame,PANEL,
               FRAME_INHERIT_COLORS,TRUE,
               0);
 if (_3d->panel == 0)
    {free_file_menu_3d_rec(_3d);
     return ~NO_ERROR;}

 row = 5;
 offset = xv_rows(_3d->panel,1) - xv_rows(_3d->panel,0) - 10;
 PRT(printf("setup_file_menu_3d_rec...9\n"))
 _3d->order_msg = xv_create(_3d->panel,PANEL_MESSAGE,
                  XV_X,5,XV_Y,row,
                  PANEL_LABEL_STRING, "",
                   0);
 cur_3d_order_msg_proc(win);

 row += offset - 4;
 xv_create(_3d->panel,PANEL_BUTTON,
                  XV_X,5,XV_Y,row,
                  PANEL_LABEL_STRING, "REVERSE 3D IMAGE ORDER - <> R",
                  PANEL_CLIENT_DATA,_3d,
	          PANEL_NOTIFY_PROC,reverse_image_order_proc,
                   0);
 row += offset;
 xv_create(_3d->panel,PANEL_BUTTON,
                  XV_X,5,XV_Y,row,
                  PANEL_LABEL_STRING,"SAVE CURRENT IMAGE ORDER - Sh <> R",
                  PANEL_CLIENT_DATA,_3d,
	          PANEL_NOTIFY_PROC,save_cur_image_order_proc,
                  0);
 PRT(printf("setup_file_menu_3d_rec...10\n"))
 row += offset;
 xv_create(_3d->panel,PANEL_BUTTON,
                  XV_X,5,XV_Y,row,
                  PANEL_LABEL_STRING, "Current Saved 3D IMAGE ORDER - <> T",
                  PANEL_CLIENT_DATA,_3d,
	          PANEL_NOTIFY_PROC,cur_default_image_order_proc,
                   0);

 row += offset;
 xv_create(_3d->panel,PANEL_BUTTON,
                  XV_X,5,XV_Y,row,
                  PANEL_LABEL_STRING, "DEFAULT 3D IMAGE ORDER - Sh <> T",
                  PANEL_CLIENT_DATA,_3d,
	          PANEL_NOTIFY_PROC,default_image_order_proc,
                   0);

 window_fit(_3d->panel);
 rr = (Rect *) xv_get(_3d->panel,XV_RECT);
 xv_set(_3d->panel,XV_HEIGHT,rr->r_height - 10,XV_WIDTH,rr->r_width - 10,0);
 PRT(printf("setup_file_menu_3d_rec...11\n"))
           
 window_fit(_3d->frame);
 make_2_wins_adjacent(_3d->frame,win->dwin->frame,0);
 win_set_mouse_at_center(_3d->frame);

 return NO_ERROR;
}



/*------------------------------------------------------------------------
cur_3d_order_msg_proc - ???
------------------------------------------------------------------------*/

void
cur_3d_order_msg_proc(win)
 struct image_display_window *win;
{
 struct all_file_menu_3d_recs *all_3ds;
 struct file_menu_3d_rec *_3d;
 struct img_buf_rec *img;

 if ((all_3ds = (struct all_file_menu_3d_recs *) 
          enter_3d_file_menu(NULL,GET_ALL_3D_MENU_PTR)) != NULL)
    {if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return;
     if ((_3d = find_file_menu_3d_rec(all_3ds,win)) != NULL)
        {if (img->file->slice_info.select_3d_gravity == _3D_SLICE_BOTTOM_TO_TOP)
           xv_set(_3d->order_msg,PANEL_LABEL_STRING,"3D IMAGE ORDER : BOTTOM TO TOP",0);
          else xv_set(_3d->order_msg,PANEL_LABEL_STRING,"3D IMAGE ORDER : TOP TO BOTTOM",0);
         }
    }
}



/*------------------------------------------------------------------------
find_file_menu_3d_rec - ???
------------------------------------------------------------------------*/

struct file_menu_3d_rec *
find_file_menu_3d_rec(all_3ds,win)
 struct all_file_menu_3d_recs *all_3ds;
 struct image_display_window *win;
{
 int lcv;
 struct input_file_rec *f1;
 struct img_buf_rec *img;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return NULL;
 f1 = img->file;
 for (lcv = 0; lcv < all_3ds->num_3ds; lcv++)
    if ((img = win_rec_cur_img_buf(all_3ds->_3ds[lcv]->win->owner)) != NULL)
       if (img->file == f1)
          return all_3ds->_3ds[lcv];
 return NULL;
}



/*------------------------------------------------------------------------
enter_3d_file_menu - ???
------------------------------------------------------------------------*/

int
enter_3d_file_menu(win,status)
 struct image_display_window *win;
 int status;
{
 static struct all_file_menu_3d_recs all_3ds;
 static char first_time = 1;
 int err;
 struct file_menu_3d_rec *_3d;

 PRT(printf("enter_3d_file_menu...\n"))
 err = NO_ERROR;
 if (status == GET_ALL_3D_MENU_PTR)
    {if (first_time) return 0;
     return (unsigned long) &all_3ds;}     /* FIXME:  ptr as long */
  else if (status == CLEAN_UP_3D_MENU)
   {if (!first_time)
       if ((_3d = find_file_menu_3d_rec(&all_3ds,win)) != NULL)
          free_file_menu_3d_rec(_3d);
    return NO_ERROR;}

 PRT(printf("enter_3d_file_menu...1\n"))
 if (first_time)
    {init_all_file_menu_3d_recs(&all_3ds);
     first_time = 0;}
   else
    {if ((_3d = find_file_menu_3d_rec(&all_3ds,win)) != NULL)
        if (_3d->win == win)
           {xv_set(_3d->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,WIN_ALARM,0);
            win_set_mouse_at_center(_3d->frame);
            wmgr_top(_3d->frame);
            return NO_ERROR;}
          else free_file_menu_3d_rec(_3d);
    }
 PRT(printf("enter_3d_file_menu...2\n"))
 get_handle_memory(&(all_3ds._3ds),&(all_3ds.num_3ds),1,
     (size_t) sizeof(struct file_menu_3d_rec),&err);
 if (err != NO_ERROR) return err;
 _3d = all_3ds._3ds[all_3ds.num_3ds - 1];
 init_file_menu_3d_rec(_3d);
 if ((err = setup_file_menu_3d_rec(_3d,win)) == NO_ERROR)
    xv_set(_3d->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
 PRT(printf("enter_3d_file_menu...3\n"))
 return err;
}
