/****************************************************************************
3d_menu.c - Handle menu for 3d image operations???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: 3d_menu.c,v 1.9 1999/07/07 17:41:13 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <xview/xview.h>
#include <xview/canvas.h>
/* #include <xview/cms.h> */
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/wmgr.h>

#include "buf_3d_rec.h"
#include "geometry.h"
#include "mri_format.h"
#include "nria_procedures.h"


#if 0
#   define PRINT(x) printf(x); fflush(stdout);
#   define TRACE(x) x; fflush(stdout);
#   define PRT1(x) x; fflush(stdout);
#   define PRT2(x) x; fflush(stdout);
#   define PRT3(x) x; fflush(stdout);
#   define PRT4(x) x; fflush(stdout);
#   define PRT5(x) x; fflush(stdout);
#   define PRT(x) x; fflush(stdout);
#else
#   define PRT1(x)
#   define PRT2(x)
#   define PRT3(x)
#   define PRT4(x)
#   define PRINT(x)
#   define TRACE(x)
#   define PRT5(x)
#   define PRT(x)
#endif


/* External functions */
/***
extern unsigned long enter_3d_plane();
extern int FormPlaneAndAdjust();
extern int _3d_last_plane();
extern int win_new_view();
extern unsigned long ULongRound();
extern float degree_angle_vectors();
extern struct img_buf_rec *win_rec_cur_img_buf();
extern char *win_rec_win_id_title();
extern double degree();
extern double radian();
extern double acos_to_degree();
extern double cos_degree();
extern struct curve_rec *get_cur_curve();
extern struct curve_rec *locate_curve_of_type();
extern struct roi_rec *get_cur_roi();
extern struct roi_rec *next_cur_roi();
extern struct roi_rec *get_roi();
extern long max_plane_dims();
extern struct view_rec *find_view_rec();
extern struct view_rec *find_file_view_rec();
extern struct view_rec *find_any_view_rec();
extern struct list_rec *head_list();
extern struct co_register_rec *find_co_register_rec();
***/


/* ??? */
#define CLEAN_UP_3D_MENU 0
#define SETUP_3D_MENU 1
#define GET_ALL_3D_MENU_PTR 2

#define CUT_PARAMS_DEFINED 2


/* Local structures */
struct menu_3d_rec
   {
    unsigned short status;
    struct image_display_window *win;
    struct display_win_rec *dwin;
    Frame frame;
    Panel panel;
    Panel_item /* auto_rot,auto_sec,cur_plane_msg,
                 cur_plane_param,cut_plane_param,*/
               rotate,obs_rel,about,at,
               rot_angle,gplane_angle,window,same_view,
               cut_mode,set_degree_betw,cur_degree_betw,after_cut_deg_betw,
               pl_num;
    struct strs_rec window_strs;

    unsigned char is_auto_rotating;
    struct plane_3d_rec gplane;
    float centerx,centery,rad_angle;
    struct vector_3d_rec pt1,pt2;
   };

struct all_menu_3d_recs
   {
    int num_3ds;
    struct menu_3d_rec **_3ds;
   };


/* Local functions */
void init_menu_3d_rec(struct menu_3d_rec *_3d);
void free_menu_3d_rec(struct menu_3d_rec *_3d);
void init_all_menu_3d_recs(struct all_menu_3d_recs *all_3ds);
int about_proc(Panel_item item, Event *event);
int rotate_proc(Panel_item item, Event *event);
void auto_rotate_proc(Panel_item item, int value, Event *event);
void last_plane_proc(Panel_item item, Event *event);
int win_same_plane_view(struct image_display_window *win, char *wid,
    int is_same_view);
void win_same_orientation(struct image_display_window *win, char *wid);
void win_same_plane(struct image_display_window *win, char *wid);
void same_plane_proc(Panel_item item, Event *event);
int win_reverse_plane(struct image_display_window *win);
void reverse_plane_proc(Panel_item item, Event *event);
void cut_mode_proc(Panel_item item, int value, Event *event);
void set_degree_betw_proc(Panel_item item, int value, Event *event);
void degree_cut_proc(Panel_item item, Event *event);
void plane_more_proc(Panel_item item, Event *event);
void keypad_mode_proc(Panel_item item, Event *event);
int setup_3d_window_labels(struct img_buf_rec *img, Panel_item item,
    struct strs_rec *labels);
void setup_any_view_wids(struct img_buf_rec *img, Panel_item item,
    char *cur_str, int *cur_indx);
int line_and_2d_bounds_intersection(struct plane_3d_rec *gplane,
    double centerx, double centery, double rad_angle,
    struct vector_3d_rec *and1, struct vector_3d_rec *and2);
void define_default_cut_params(struct menu_3d_rec *_3d, struct view_rec *view);
int set_cut_plane_line(struct menu_3d_rec *_3d, struct view_rec *view,
    char *win_str, double rad_angle, int set_angle);
void show_cut_plane_params(struct menu_3d_rec *_3d);
int set_3d_plane_num_proc(Panel_item item, int value, Event *event);
int set_3d_window_angle_proc(Panel_item item, int value, Event *event);
int cur_roi_win_cut_view(struct menu_3d_rec *_3d);
void current_window_proc(Panel_item item, int value, Event *event);
void _3d_done_proc(Frame frame);
int show_2_plane_angle(struct menu_3d_rec *_3d, int select);
int setup_menu_3d_rec(struct menu_3d_rec *_3d,
    struct image_display_window *win);
struct menu_3d_rec *find_menu_3d_rec(struct all_menu_3d_recs *all_3ds,
    struct image_display_window *win);
struct menu_3d_rec *find_menu_3d_rec_win(struct all_menu_3d_recs *all_3ds,
    struct image_display_window *win);
void update_3d_menu(struct image_display_window *win);
int get_3d_plane_rot_angle(struct image_display_window *win);
int get_3d_plane_cut_angle(struct image_display_window *win);
void update_3d_cut_mode(struct image_display_window *win);
int bring_up_cur_win(struct image_display_window *win);
int add_view_wid(struct view_rec *view, Panel_item item);
int add_plane_window_list(struct view_rec *view);
unsigned long enter_3d_plane(struct image_display_window *win, int status);



/*----------------------------------------------------------------------
init_menu_3d_rec - ???
----------------------------------------------------------------------*/

void
init_menu_3d_rec(_3d)
 struct menu_3d_rec *_3d;
{
 _3d->status = 0;
 _3d->win = NULL;
 _3d->frame  = 0;
 _3d->is_auto_rotating = 0;
 init_strs_rec(&(_3d->window_strs));
}


/*----------------------------------------------------------------------
free_menu_3d_rec - ???
----------------------------------------------------------------------*/

void
free_menu_3d_rec(_3d)
 struct menu_3d_rec *_3d;
{
 struct all_menu_3d_recs *all_3ds;

 PRT(printf("free_menu_3d_rec...\n"))
/*
 if (xv_get(_3d->auto_rot,PANEL_VALUE))
    {
     notify_set_itimer_func(_3d->rotate,NOTIFY_FUNC_NULL,ITIMER_REAL,
             (struct itimerval *) 0,(struct itimerval *) 0);}
*/
 PRT(printf("free_menu_3d_rec...1\n"))
 free_strs_rec(&(_3d->window_strs));

 if (_3d->frame != 0) xv_destroy_safe(_3d->frame);
 all_3ds = (struct all_menu_3d_recs *) enter_3d_plane(NULL,
            GET_ALL_3D_MENU_PTR);
 c_free(_3d);
 DropHandleAryPtr(&(all_3ds->_3ds),&(all_3ds->num_3ds),_3d);
 PRT(printf("free_menu_3d_rec...end\n"))
}


/*----------------------------------------------------------------------
init_all_menu_3d_recs - ???
----------------------------------------------------------------------*/

void
init_all_menu_3d_recs(all_3ds)
 struct all_menu_3d_recs *all_3ds;
{
 all_3ds->num_3ds = 0;
 all_3ds->_3ds = NULL;
}


/*----------------------------------------------------------------------
about_proc - ???
----------------------------------------------------------------------*/

int
about_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct menu_3d_rec *_3d;
 int value,about;
 struct view_rec *view;
 struct img_buf_rec *img;

 _3d = (struct menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return CANCEL;
 view = img->cur_slice->slice->view;
 about = (int) xv_get(_3d->about,PANEL_VALUE);
 value = (int) xv_get(_3d->set_degree_betw,PANEL_VALUE);
 PRT4(printf("about_proc...about = %d value = %d",about,value);)
 if (about == 0)
    return _3d_axis_rotate_plane(view,value);
   else
    return _3d_axis_rotate_plane(view,-value);
}


/************************
at_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct menu_3d_rec *_3d;
 int about,at;

 _3d = (struct menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 at = (int) xv_get(item,PANEL_VALUE);
 if (at == 2)
    {about = (int) xv_get(_3d->about,PANEL_VALUE);
     if ((about != 5) && (about != 6))
        xv_set(_3d->about,PANEL_VALUE,5,0);}
 if (at != 0)
    {if ((int) xv_get(_3d->obs_rel,PANEL_VALUE) == 0)
        xv_set(_3d->obs_rel,PANEL_VALUE,1,0);
    }
}

obs_rel_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct menu_3d_rec *_3d;
 int about,at;

}
*****************/


/*----------------------------------------------------------------------
rotate_proc - ???
----------------------------------------------------------------------*/

int
rotate_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct menu_3d_rec *_3d;
 struct view_rec *view;
 struct img_buf_rec *img;
 int err,deg;
 int is_rel;

 _3d = (struct menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 xv_set(_3d->frame,FRAME_BUSY,TRUE,0);
 set_win_busy(_3d->win);
 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return CANCEL;
 view = img->cur_slice->slice->view;
 is_rel = (int) xv_get(_3d->obs_rel,PANEL_VALUE);
 deg = (int) xv_get(_3d->rot_angle,PANEL_VALUE);
 err = RotatePlaneFromCenter(view,(float) deg,is_rel);
 set_win_not_busy(_3d->win);
 xv_set(_3d->frame,FRAME_BUSY,FALSE,0);
 set_3d_window_angle_proc(_3d->gplane_angle,0,NULL);
 return err;
}


/*----------------------------------------------------------------------
auto_rotate_proc - ???
----------------------------------------------------------------------*/

void
auto_rotate_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct menu_3d_rec *_3d;

 _3d = (struct menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if (_3d->is_auto_rotating && (value == 0))
    {
     notify_set_itimer_func(_3d->rotate,NOTIFY_FUNC_NULL,ITIMER_REAL,
             (struct itimerval *) 0,(struct itimerval *) 0);}
}


/*----------------------------------------------------------------------
last_plane_proc - ???
----------------------------------------------------------------------*/

void
last_plane_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct menu_3d_rec *_3d;

 _3d = (struct menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 ShowMsgStrs(_3d->win->dwin->frame,1,NO_ERROR,1,1,
               "Last plane !!!",
                _3d_last_plane,2,_3d->win,0);
/*
 (void) notify_set_itimer_func(_3d->win,
             last_plane_it_proc,ITIMER_REAL,
             NOTIFY_POLLING_ITIMER,(struct itimerval *) 0);
*/
}


/*----------------------------------------------------------------------
win_same_plane_view - ???
----------------------------------------------------------------------*/

int
win_same_plane_view(win,wid,is_same_view)
 struct image_display_window *win;
 char *wid;
 int is_same_view;
{
 struct img_buf_rec *img;
 struct view_rec *view,*view1;
 static struct vector_3d_rec base;
 int same_file,err;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 view = img->cur_slice->slice->view;
 err = NO_ERROR;
 if (str_cmp(view->wid,wid) != 0)
    {if ((view1 = find_file_view_rec(img->file,wid)) != NULL)
        same_file = 1;
       else if ((view1 = find_any_view_rec(img->file,wid)) != NULL)
        same_file = 0;

     if (view1 != NULL)
        {view->last_gplane = view->gplane;
         set_win_busy(win);
         if ((is_same_view == 1) && !same_file) is_same_view = 2;

         if (is_same_view == 1)
            {
             ShowMsgStrs(win->dwin->frame,1,0,0,1,
               "Same Plane...",win_new_view,2,win,&(view1->gplane));
            }
           else
            {GlobalPt(&(view->gplane),&(view1->gplane.base),&base);
             ShowMsgStrs(win->dwin->frame,1,0,0,1,
               "Same orientation...",FormPlaneAndAdjust,7,
                    view,&base,&(view1->gplane.normal),NULL,NULL,1,0);
            }
         set_win_not_busy(win);
        }
    if (view1 == NULL)
       {notice_prompt(win->dwin->frame,(Event *) NULL,
               NOTICE_MESSAGE_STRINGS,"Sorry...Cannot find the window !!! ",
                            "It might have been deleted !!!",0,
                            NOTICE_BUTTON_YES,"OK !",0);
        err = CANCEL;}
   }
 return err;
}


/*----------------------------------------------------------------------
win_same_orientation - ???
----------------------------------------------------------------------*/

void
win_same_orientation(win,wid)
 struct image_display_window *win;
 char *wid;
{
 if (wid != NULL)
    {set_win_busy(win);
     win_same_plane_view(win,wid,0);
     set_win_not_busy(win);}
}


/*----------------------------------------------------------------------
win_same_plane - ???
----------------------------------------------------------------------*/

void
win_same_plane(win,wid)
 struct image_display_window *win;
 char *wid;
{
 if (wid != NULL)
    {set_win_busy(win);
     win_same_plane_view(win,wid,1);
     set_win_not_busy(win);}
}


/*----------------------------------------------------------------------
same_plane_proc - ???
----------------------------------------------------------------------*/

void
same_plane_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct menu_3d_rec *_3d;
 char *wid;
 int value,is_same_view;

 _3d = (struct menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 value = (int) xv_get(_3d->same_view,PANEL_VALUE);
 is_same_view = (int) xv_get(item,XV_KEY_DATA,1);
 xv_set(_3d->frame,FRAME_BUSY,TRUE,0);
 if ((wid = (char *) xv_get(_3d->same_view,PANEL_CHOICE_STRING,value)) != NULL)
    win_same_plane_view(_3d->win,wid,is_same_view);
 xv_set(_3d->frame,FRAME_BUSY,FALSE,0);
}


/*----------------------------------------------------------------------
win_reverse_plane - ???
----------------------------------------------------------------------*/

int
win_reverse_plane(win)
 struct image_display_window *win;
{
 struct img_buf_rec *img;
 struct view_rec *view;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 view = img->cur_slice->slice->view;
 reverse_plane(&(view->gplane));
 reflect_view_slice_num(view);
 change_slice_image_label(&(view->sl.slice),
      (char *) get_plane_img_label(&(view->gplane)));
 win_multiple_set_frame_label(win);
 update_cur_slice_num(win);
 update_3d_menu(win);
 return NO_ERROR;
}


/*----------------------------------------------------------------------
reverse_plane_proc - ???
----------------------------------------------------------------------*/

void
reverse_plane_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct menu_3d_rec *_3d;
 
 _3d = (struct menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 ShowMsgStrs(_3d->win->dwin->frame,1,NO_ERROR,2,1,
               "Reverse the plane sequence !!!",
                win_reverse_plane,1,_3d->win);
}


/*----------------------------------------------------------------------
cut_mode_proc - ???
----------------------------------------------------------------------*/

void
cut_mode_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct menu_3d_rec *_3d;
 int mode;

 _3d = (struct menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 mode = (int) xv_get(item,PANEL_VALUE);
 if (mode == 0)
    {if (_3d->win->control.which_cursor == CURSOR_ANGLE_CUT)
        win_select_cursor(_3d->win,CURSOR_CUT);
    }
   else 
    {if (_3d->win->control.which_cursor == CURSOR_CUT)
        win_select_cursor(_3d->win,CURSOR_ANGLE_CUT);
    }
 show_cut_plane_params(_3d);
}


/*----------------------------------------------------------------------
set_degree_betw_proc - ???
----------------------------------------------------------------------*/

void
set_degree_betw_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct menu_3d_rec *_3d;

 _3d = (struct menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if (_3d->win->control.which_cursor == CURSOR_CUT)
    win_select_cursor(_3d->win,CURSOR_ANGLE_CUT);
 xv_set(_3d->cut_mode,PANEL_VALUE,1,0);
 show_cut_plane_params(_3d);
}


/*----------------------------------------------------------------------
degree_cut_proc - ???
----------------------------------------------------------------------*/

void
degree_cut_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct menu_3d_rec *_3d;
 struct img_buf_rec *img;

 _3d = (struct menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return;
 ChangeCutPlane(img->cur_slice->slice->view,0);
 cur_roi_win_cut_view(_3d);
}


/*----------------------------------------------------------------------
plane_more_proc - ???
----------------------------------------------------------------------*/

void
plane_more_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct menu_3d_rec *_3d;

 _3d = (struct menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 notice_prompt(_3d->frame,(Event *) NULL,
            NOTICE_MESSAGE_STRINGS,
               "T -- get a new window where its",
               "     normal = center_of_wid - center_of_cur_wid",
               "Sh T -- get a new window where its",
               "     normal is 90 degree to both normals of wid cur_wid",
               "",
               "Ctrl Mleft -- locate the cur xyz on wid indicated by cur roi",
               "           -- the new newxyz may be above, on, or below wid",
               "Ctrl Sh Mleft -- locate the cur xyz on the co-reg wid",
               "",
               "Sh Ins -- last plane for the whole group",
               "",
               "Rotate Plane",
               "Ctrl [<>] <digit> - rotate in the anti-clockwise direction",
               "...............by <digit> degrees",
               "Ctrl Sh [<>] <digit> - rotate clockwisely",
               "..<> - if <> exists, multiply the degree by .1",
              0,
            NOTICE_BUTTON_YES," OK !",
            0);

}


/*----------------------------------------------------------------------
keypad_mode_proc - ???
----------------------------------------------------------------------*/

void
keypad_mode_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct menu_3d_rec *_3d;

 _3d = (struct menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 notice_prompt(_3d->frame,(Event *) NULL,
            NOTICE_MESSAGE_STRINGS,
               "Notations...",
               "...KP = Keypad keys..RA = ROTATION AXIS",
               "...E = East, S = SOUTH, W = WEST, N = NORTH",
               "",
               "KPs are defined for 3 different modes, activated by",
               "pressing the F11 key, and the WID at the SE corner",
               "of the window is preceded by either FR or FA or O...",
               "",
               "1) FR = Flip the plane relatively from the RA DRAWN",
               "...Both FR & FA divide the plane regions into 9 partitions.",
               "...FR differs from FA that the 9 partitions are now defined",
               "...relatively from the RA drawn, and the RA is just like",
               "...the vertical axis through the center in FA.",
               "",
               "2) FA = Flip the plane absolutely from predefined RA",
               "...KP1 -- 1 degree down at SW corner, RA = NW to SE",
               "...KP2 -- 1 degree down at S corner, RA = W to E",
               "...KP3 -- 1 degree down at SE corner, RA = NE to SW",
               "...KP4 -- 1 degree down at W corner, RA = N to S",
               "...KP6 -- 1 degree down at E corner, RA = N to S",
               "...KP7 -- 1 degree down at NW corner, RA = NE to SW",
               "...KP8 -- 1 degree down at N corner, RA = E to W",
               "...KP9 -- 1 degree down at NE corner, RA = NW to SE",
               "",
               "3) O = Plane orientation...",
               "...KP1 -- center | relecftion where x = -x",
               "...KP2 -- 270 degree rotation within plane",
               "...KP3 -- center -- relecftion where y = -y",
               "...KP4 -- 180 degree rotation within plane",
               "...KP5 -- Default orientation",
               "...KP6 -- 0 degree rotation within plane",
               "...KP7 -- center \\ relecftion where x = -y",
               "...KP8 -- 90 degree rotation within plane",
               "...KP9 -- center / relecftion where x = y",
              0,
            NOTICE_BUTTON_YES," OK !",
            0);

}



/*----------------------------------------------------------------------
setup_3d_window_labels - ???
----------------------------------------------------------------------*/

int
setup_3d_window_labels(img,item,labels)
 struct img_buf_rec *img;
 Panel_item item;
 struct strs_rec *labels;
{
 struct list_rec *cur,*next;
 struct view_rec *view1,*view;
 struct slice_roi_rec *slice_roi;
 struct roi_rec *roi;
 int cur_win;
 long cnt;

 view = img->cur_slice->slice->view;
 slice_roi = &(view->sl.slice_roi_win.slice_roi);
 roi = get_cur_roi(slice_roi);
 next = cur = &(view->_3d_list);
 cur_win = cnt = 0;
 do 
    {view1 = next->item;
     if ((view1 != NULL) && (view1 != view) &&  
         (view1->img != NULL) && (view1->img->win != NULL))
        {xv_set(item,PANEL_CHOICE_STRING,cnt,view1->wid,0);
         if (labels != NULL) add_str_to_strs_rec(labels,view1->wid);
             
         if (roi != NULL)
            if (str_cmp(roi->label,view1->wid) == 0)
               {
                cur_win = cnt;
                xv_set(item,PANEL_VALUE,cnt,0);}
         cnt++;
        }
     next = next->next;
    }
 while (cur != next);
 xv_set(item,XV_KEY_DATA,1,cnt,0);

 return cur_win;
}


/*----------------------------------------------------------------------
setup_any_view_wids - ???
----------------------------------------------------------------------*/

void
setup_any_view_wids(img,item,cur_str,cur_indx)
 struct img_buf_rec *img;
 Panel_item item;
 char *cur_str;
 int *cur_indx;
{
 struct view_rec *view;
 int yet;
 struct slice_info_rec *slice_info;
 long lcv,indx,cnt;

 view = img->cur_slice->slice->view;
 cnt = 0;
 if (cur_str == NULL) yet = 1;
   else yet = 0;
 if (cur_indx != NULL) *cur_indx = -1;
   else yet = 1;

 slice_info = &(img->file->slice_info);
 PRT2(printf("cnt = %d...yet = %d slice_info->num_views = %d\n",cnt,yet,slice_info->num_views);)
 for (lcv = 0; lcv < slice_info->num_views; lcv++)
    {if (!yet && (cur_str != NULL))
        {PRT2(printf("cur_str = %s wid = %s\n",
                       cur_str,slice_info->views[lcv]->wid))
         if (str_cmp(cur_str,slice_info->views[lcv]->wid) == 0)
            {yet = 1;
             *cur_indx = cnt;}
        }
     xv_set(item,PANEL_CHOICE_STRING,cnt,slice_info->views[lcv]->wid,0);
     cnt++;
    }
 PRT1(printf("setup_any_view_wids...\n"))
 for (indx = 0; indx < img->file->prog->input.num_files; indx++)
    if (img->file != img->file->prog->input.files[indx])
       {slice_info = &(img->file->prog->input.files[indx]->slice_info);
        PRT2(printf("%d) num_views = %d file = %u   file1 = %u\n",indx,
            slice_info->num_views,img->file,img->file->prog->input.files[indx]))
        for (lcv = 0; lcv < slice_info->num_views; lcv++)
           {if (!yet && (cur_str != NULL))
               {PRT2(printf("***cur_str = %s wid = %s\n",
                       cur_str,slice_info->views[lcv]->wid))
                if (str_cmp(cur_str,slice_info->views[lcv]->wid) == 0)
                   {yet = 1;
                    *cur_indx = cnt;}
               }
            xv_set(item,PANEL_CHOICE_STRING,cnt,slice_info->views[lcv]->wid,0);
            cnt++;}
       }
 xv_set(item,XV_KEY_DATA,1,cnt,0);
 PRT4(printf("final...yet = %d\n",yet);)

}


/*----------------------------------------------------------------------
line_and_2d_bounds_intersection - ???
----------------------------------------------------------------------*/

int
line_and_2d_bounds_intersection(gplane,centerx,centery,rad_angle,and1,and2)
 struct plane_3d_rec *gplane;
 float centerx,centery,rad_angle;
 struct vector_3d_rec *and1,*and2;
{
 struct vector_3d_rec center,bounds[4],dire;

 setup_vector_3d_rec(&center,centerx,centery,.0);
 setup_vector_3d_rec(&dire,-(float) cos((double) rad_angle),
           (float) sin((double) rad_angle),.0);

 bounds[0].x = (float) ((long) gplane->proj_buf_dimx -
                        (long) gplane->trans_proj_dimx) / 2.0;
 bounds[0].y = (float) ((long) gplane->proj_buf_dimy -
                        (long) gplane->trans_proj_dimy) / 2.0;
 bounds[1].x = bounds[0].x + (float) (gplane->trans_proj_dimx - 1);
 bounds[1].y = bounds[0].y;
 bounds[2].x = bounds[1].x;
 bounds[2].y = bounds[0].y + (float) (gplane->trans_proj_dimy - 1);
 bounds[3].x = bounds[0].x;
 bounds[3].y = bounds[2].y;
 bounds[0].z = bounds[1].z = bounds[2].z = bounds[3].z = .0;
 return plane_line_and_bound_intersection(bounds,&center,&dire,
                                   and1,and2,0);
}


/*----------------------------------------------------------------------
define_default_cut_params - ???
----------------------------------------------------------------------*/

void
define_default_cut_params(_3d,view)
 struct menu_3d_rec *_3d;
 struct view_rec *view;
{
 float tempz;

 if (!(_3d->status & CUT_PARAMS_DEFINED))
    {
     _3d->rad_angle = 0.0;
     LocalXYZToPlaneXYZ(&(view->gplane),_3d->gplane.base.x,_3d->gplane.base.y,
                     _3d->gplane.base.z,&(_3d->centerx),&(_3d->centery),&tempz);
     _3d->status |= CUT_PARAMS_DEFINED;}
}


/*----------------------------------------------------------------------
set_cut_plane_line - ???
----------------------------------------------------------------------*/

int
set_cut_plane_line(_3d,view,win_str,rad_angle,set_angle)
 struct menu_3d_rec *_3d;
 struct view_rec *view;
 char *win_str;
 float rad_angle;
 int set_angle;
{
 struct slice_roi_rec *slice_roi;
 struct roi_rec *cur_roi;
 struct curve_rec *cut_curve,*cur_curve;
 long x1,y1,x2,y2;
 float deg;
 int err;

 PRT3(printf("set_cut_plane_line...0\n"))
 slice_roi = &(view->sl.slice_roi_win.slice_roi);
 if ((cur_roi = get_cur_roi(slice_roi)) == NULL)
    {if ((err = setup_more_rois(slice_roi,win_str,1)) != NO_ERROR)
        return err;
     PRT3(printf("set_cut_plane_line...1\n"))
     define_default_cut_params(_3d,view);
     if (set_angle) _3d->rad_angle = rad_angle;
    }
   else
    {
     PRT3(printf("set_cut_plane_line...2\n"))
     if ((cur_curve = get_cur_curve(slice_roi)) != NULL)
        {
         delete_curve_all_pts(cur_curve,&(slice_roi->status));}

     if (!(_3d->status & CUT_PARAMS_DEFINED))
        {if (((cut_curve = locate_curve_of_type(slice_roi,
                  CURVE_NEVER_CURRENT,0)) == NULL) ||
              (LongCurvePtByIndx(cut_curve,1,&x2,&y2,NULL) != NO_ERROR) ||
              (LongCurvePtByIndx(cut_curve,0,&x1,&y1,NULL) != NO_ERROR))
            {
             define_default_cut_params(_3d,view);}
           else
           {TRACE(printf("x1 = %d y1 = %d  x2 = %d y2 = %d\n",x1,y1,x2,y2))
            if (set_angle) _3d->rad_angle = rad_angle;
              else if (x2 == x1) _3d->rad_angle = RADIAN / 2;
              else
               {_3d->rad_angle = (float) atan2((double) (y2 - y1),
                     (double) (x2 - x1));
                TRACE(printf("compute angle\n"))}
            TRACE(printf("angle = %f\n",_3d->rad_angle))
            _3d->centerx = (float) (x2 + x1) / 2.0;
            _3d->centery = (float) (y2 + y1) / 2.0;
            _3d->status |= CUT_PARAMS_DEFINED;
            _3d->status |= CUT_PARAMS_DEFINED;
            TRACE(printf("define CUT_PARAMS_DEFINED...\n"))
           }
        }
       else
        {TRACE(printf("set_angle...\n"))
         if (set_angle) _3d->rad_angle = rad_angle;}
    }
 PRT3(printf("set_cut_plane_line...3\n"))
 if (line_and_2d_bounds_intersection(&(view->gplane),
             _3d->centerx,_3d->centery,_3d->rad_angle,&(_3d->pt1),&(_3d->pt2)))
    {
     PRT3(printf("set_cut_plane_line...4\n"))
     x1 = (int) ULongRound((double) _3d->pt1.x);
     y1 = (int) ULongRound((double) _3d->pt1.y);
     x2 = (int) ULongRound((double) _3d->pt2.x);
     y2 = (int) ULongRound((double) _3d->pt2.y);
     PRT3(printf("..line_and_2d_bounds_intersection..x1 = %d  y1 = %d  x2 = %d y2= %d\n",
                   x1,y1,x2,y2))
     PRT3(printf("....x1 = %f  y1 = %f  x2 = %f y2= %f\n",
                   _3d->pt1.x,_3d->pt1.y,_3d->pt2.x,_3d->pt2.y))
     err = EnterPtsIntoSliceRoiRec(slice_roi,NULL,win_str,
                  (unsigned long) DRAW_LINE,0,2,(float) x1,(float) y1,
                  (float) x2,(float) y2);}
   else err = CANCEL;
 PRT3(printf("set_cut_plane_line...5.._3d->win = %u\n",_3d->win))
 if (err == NO_ERROR)
    {if (_3d->win != NULL)
        PRT3(printf("set_cut_plane_line.....dwin = %u\n",_3d->win->dwin))
     win_redisplay_all_curves(_3d->win);
    }
 deg = (float) degree(_3d->rad_angle);
 PRT3(printf("set_cut_plane_line...6\n"))
 xv_set(_3d->gplane_angle,PANEL_VALUE,(int) ULongRound((double) deg),0);
 show_cut_plane_params(_3d);
 PRT3(printf("set_cut_plane_line...7\n"))

 return err;
}


/*----------------------------------------------------------------------
show_cut_plane_params - ???
----------------------------------------------------------------------*/

void
show_cut_plane_params(_3d)
 struct menu_3d_rec *_3d;
{
 struct img_buf_rec *img;
 struct view_rec *pl,*pl1;
 struct plane_3d_rec *gplane;
 int val,err;
 char *str,str1[60],str2[60];
 struct vector_3d_rec base,normal;
        
 PRT3(printf("show_cut_plane_params...0\n"))

 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return;
 pl = img->cur_slice->slice->view;
 val = (int) xv_get(_3d->window,PANEL_VALUE);
 str1[0] = '\0';
 str2[0] = '\0';
 err = NO_ERROR;
 if ((str = (char *) xv_get(_3d->window,PANEL_CHOICE_STRING,val)) != NULL)
    if ((pl1 = find_view_rec(pl,str)) != NULL)
       {sprintf(str1,"CUR: %s",pl1->sl.slice.img_label);
        if (xv_get(_3d->cut_mode,PANEL_VALUE) == 0) val = 90;
          else val = (int) xv_get(_3d->set_degree_betw,PANEL_VALUE);
        err = base_normal_after_angle_cut(&(pl->gplane),_3d->pt1.x,_3d->pt1.y,
                 _3d->pt2.x,_3d->pt2.y,(float) val,&base,&normal);
        PRT3(printf("show_cut_plane_params...1\n"))
        if (err == NO_ERROR)
           { 
            gplane = &(pl1->gplane);
            GlobalPt(&(pl->gplane),&base,&base);
            LocalPt(gplane,&base,&base);
            PRT3(printf("show_cut_plane_params...2\n"))
            _3d->gplane = *gplane;

            ChangePlane(&(_3d->gplane),base.x,base.y,base.z,
             normal.x,normal.y,normal.z,NULL,NULL,
             (int) (gplane->status & PLANE_ADJUST_TO_CENTER),NULL);
            PRT3(printf("hello...gplane_num = %d\n",_3d->gplane.plane_num))

            if (_3d->gplane.total_planes > 0)
               xv_set(_3d->pl_num,PANEL_MIN_VALUE,1,
                      PANEL_VALUE,_3d->gplane.plane_num + 1,
                      PANEL_MAX_VALUE,_3d->gplane.total_planes,0);
              else  xv_set(_3d->pl_num,PANEL_MIN_VALUE,0,
                      PANEL_VALUE,0,PANEL_MAX_VALUE,0,0);
            show_2_plane_angle(_3d,3);
           }
       }
 PRT3(printf("hello...5\n"))
/*
 xv_set(_3d->cur_plane_param,PANEL_LABEL_STRING,str1,0);
 xv_set(_3d->cut_plane_param,PANEL_LABEL_STRING,str2,0);
*/
}


/*----------------------------------------------------------------------
set_3d_plane_num_proc - ???
----------------------------------------------------------------------*/

int
set_3d_plane_num_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct menu_3d_rec *_3d;
 struct view_rec *view;
 struct img_buf_rec *img;
 struct slice_roi_rec *slice_roi;
 struct curve_rec *cur_curve;
 long x1,y1,x2,y2;
 char *win_str;
 int val,err;
 float tempz;

 _3d = (struct menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return CANCEL;
 view = img->cur_slice->slice->view;
 slice_roi = &(view->sl.slice_roi_win.slice_roi);
 if ((cur_curve = get_cur_curve(slice_roi)) != NULL)
    {delete_curve_all_pts(cur_curve,&(slice_roi->status));}

 val = (int) xv_get(_3d->window,PANEL_VALUE);
 win_str = (char *) xv_get(_3d->window,PANEL_CHOICE_STRING,val);

 value -= 1;
 TRACE(printf("\nset_3d_plane_num_proc  plane_num = %d\n",value))
 TRACE(printf("before locate_plane..."))
 TRACE(pr_vector_3d_rec(&(_3d->gplane.base)))
 TRACE(printf("_3d->centerx = %f  _3d->centery = %f\n",
           _3d->centerx,_3d->centery))

 locate_plane(&(_3d->gplane),(long) value,0);
 TRACE(printf("after locate_plane..."))
 TRACE(pr_vector_3d_rec(&(_3d->gplane.base)))
 TRACE(printf("\nset_3d_plane_num_proc  plane_num = %d\n",value))
 LocalXYZToPlaneXYZ(&(view->gplane),_3d->gplane.base.x,_3d->gplane.base.y,
                     _3d->gplane.base.z,&(_3d->centerx),&(_3d->centery),&tempz);
 TRACE(printf("_3d->centerx = %f  _3d->centery = %f\n",
           _3d->centerx,_3d->centery))

 if (line_and_2d_bounds_intersection(&(view->gplane),
             _3d->centerx,_3d->centery,_3d->rad_angle,&(_3d->pt1),&(_3d->pt2)))
    {x1 = (int) ULongRound((double) _3d->pt1.x);
     y1 = (int) ULongRound((double) _3d->pt1.y);
     x2 = (int) ULongRound((double) _3d->pt2.x);
     y2 = (int) ULongRound((double) _3d->pt2.y);
     TRACE(printf("..line_and_2d_bounds_intersection..x1 = %d  y1 = %d  x2 = %d y2= %d\n",
                   x1,y1,x2,y2))
     TRACE(printf("....x1 = %f  y1 = %f  x2 = %f y2= %f\n",
                   _3d->pt1.x,_3d->pt1.y,_3d->pt2.x,_3d->pt2.y))
     err = EnterPtsIntoSliceRoiRec(slice_roi,NULL,win_str,
                  (unsigned int) DRAW_LINE,0,2,(float) x1,(float) y1,
                  (float) x2,(float) y2);}
   else err = CANCEL;
 if (err == NO_ERROR)
    {win_redisplay_all_curves(_3d->win);

     if (_3d->gplane.total_planes > 0)
        xv_set(_3d->pl_num,PANEL_MIN_VALUE,1,
                      PANEL_VALUE,_3d->gplane.plane_num + 1,
                      PANEL_MAX_VALUE,_3d->gplane.total_planes,0);
       else  xv_set(_3d->pl_num,PANEL_MIN_VALUE,0,
                      PANEL_VALUE,0,PANEL_MAX_VALUE,0,0);
     TRACE(printf("set...plane_num\n"))
    }
 return err;
}


/*----------------------------------------------------------------------
set_3d_window_angle_proc - ???
----------------------------------------------------------------------*/

int
set_3d_window_angle_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct menu_3d_rec *_3d;
 int val;
 char *str;
 float rad_angle;
 long deg;
 struct img_buf_rec *img;
 struct view_rec *view;

 _3d = (struct menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return CANCEL;
 view = img->cur_slice->slice->view;
 val = (int) xv_get(_3d->window,PANEL_VALUE);
 str = (char *) xv_get(_3d->window,PANEL_CHOICE_STRING,val);
 deg = (int) xv_get(_3d->gplane_angle,PANEL_VALUE);
 rad_angle = (float) radian((float) deg);
 return set_cut_plane_line(_3d,view,str,rad_angle,1);
}


/*----------------------------------------------------------------------
cur_roi_win_cut_view - ???
----------------------------------------------------------------------*/

int
cur_roi_win_cut_view(_3d)
 struct menu_3d_rec *_3d;
{
 int val;
 char *str;
 struct roi_rec *cur_roi;
 struct curve_rec *cur_curve;
 struct img_buf_rec *img;
 struct view_rec *view;
 struct slice_roi_rec *slice_roi;

 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return CANCEL;
 view = img->cur_slice->slice->view;
 slice_roi = &(view->sl.slice_roi_win.slice_roi);
 val = (int) xv_get(_3d->window,PANEL_VALUE);
 str = (char *) xv_get(_3d->window,PANEL_CHOICE_STRING,val);
 PRT3(printf("cur_roi_win_cut_view...\n"))
 if ((cur_roi = get_cur_roi(slice_roi)) != NULL)
    if (str_cmp(str,cur_roi->label) != 0)
       if ((cur_curve = get_cur_curve(slice_roi)) != NULL)
          {PRT3(printf("cur_roi_win_cut_view...1\n"))
           delete_curve_all_pts(cur_curve,&(slice_roi->status));}
 PRT3(printf("cur_roi_win_cut_view...2\n"))
 if ((cur_roi = next_cur_roi(slice_roi,str)) == NULL)
    return NO_ERROR;
 _3d->status &= ~CUT_PARAMS_DEFINED;
 PRT3(printf("cur_roi_win_cut_view...3\n"))
 return set_cut_plane_line(_3d,view,str,.0,0);
}


/*----------------------------------------------------------------------
current_window_proc - ???
----------------------------------------------------------------------*/

void
current_window_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct menu_3d_rec *_3d;

 _3d = (struct menu_3d_rec *) xv_get(item,PANEL_CLIENT_DATA);
 cur_roi_win_cut_view(_3d);
}


/*----------------------------------------------------------------------
_3d_done_proc - ???
----------------------------------------------------------------------*/

void
_3d_done_proc(frame)
 Frame frame;
{
 struct menu_3d_rec *_3d;

 _3d = (struct menu_3d_rec *)
                     xv_get(frame,WIN_CLIENT_DATA);
 free_menu_3d_rec(_3d);
}


/*----------------------------------------------------------------------
show_2_plane_angle - ???

 select == 0 ....find cur_degree_betw 
                      ...after_cut_deg_betw = cur_degree_betw
 select == 1 ....find cur_degree_betw 
 select == 2 ....find after_cut_deg_betw
 select == 3 ....find cur_degree_betw & after_cut_deg_betw
----------------------------------------------------------------------*/

int
show_2_plane_angle(_3d,select)
 struct menu_3d_rec *_3d;
 int select;
{
 struct img_buf_rec *img;
 struct view_rec *view,*view1;
 int val,err;
 char *str,str1[100];
 static char *cur_label = "Cur deg betw =";
 static char *after_label = "After Cut =";
 float deg;

 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL)
    err = CANCEL;
   else
    {view = img->cur_slice->slice->view;
     val = (int) xv_get(_3d->window,PANEL_VALUE);
     err = NO_ERROR;}
 if ((err == NO_ERROR) &&
     ((str = (char *) xv_get(_3d->window,PANEL_CHOICE_STRING,val)) != NULL) &&
     ((view1 = find_view_rec(view,str)) != NULL))
    {
     if ((select == 0) || (select & 1))
        {deg = degree_angle_vectors(&(view->gplane.normal_cosine),
                &(view1->gplane.normal_cosine));
         sprintf(str1,"%s %.1f",cur_label,deg);
         xv_set(_3d->cur_degree_betw,PANEL_LABEL_STRING,str1,0);}

     if ((select == 0) || (select & 2))
        {
         if (select & 2)
            deg = degree_angle_vectors(&(view->gplane.normal_cosine),
                    &(_3d->gplane.normal_cosine));
         sprintf(str1,"%s %.1f",after_label,deg);
         xv_set(_3d->after_cut_deg_betw,PANEL_LABEL_STRING,str1,0);
        }
    }
   else err = CANCEL;
  if (err != NO_ERROR)
    {
     if ((select == 0) || (select & 2))
         xv_set(_3d->after_cut_deg_betw,PANEL_LABEL_STRING,after_label);
     if ((select & 1))
        xv_set(_3d->cur_degree_betw,PANEL_LABEL_STRING,cur_label,0);
   }
 return NO_ERROR;
}


/************************
show_cur_plane_params(_3d)
 struct menu_3d_rec *_3d;
{
 char str[100];
 struct img_buf_rec *img;
 struct view_rec *view;

 if ((img = win_rec_cur_img_buf(_3d->win->owner)) == NULL) return CANCEL;
 view = img->cur_slice->slice->view;
 PRT1(printf("show_cur_plane_params...\n"))
 PRT1(printf("show_cur_plane_params...img_label = %s\n",
         view->sl.slice.img_label))
 sprintf(str,"PLANE: %s",view->sl.slice.img_label);
 xv_set(_3d->cur_plane_msg,PANEL_LABEL_STRING,str,0);
}
****************/



/*----------------------------------------------------------------------
setup_menu_3d_rec - ???
----------------------------------------------------------------------*/

int
setup_menu_3d_rec(_3d,win)
 struct menu_3d_rec *_3d;
 struct image_display_window *win;
{
 Panel_item item;
 int row,offset;
 struct img_buf_rec *img;
 struct view_rec *view;
 char *str;
 float angle;
 long the_angle;
 Rect *rr;

 PRT1(printf("setup_menu_3d_rec...\n"))
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 view = img->cur_slice->slice->view;

 _3d->frame = xv_create(win->dwin->frame, FRAME,
              FRAME_INHERIT_COLORS,TRUE,
              WIN_CLIENT_DATA,_3d,
              0);
 if (_3d->frame == 0)
    {free_menu_3d_rec(_3d);
     return ~NO_ERROR;}
 _3d->win = win;
 _3d->dwin = win->dwin;
 str = win_rec_win_id_title(win->owner,_3D_MENU);
 xv_set(_3d->frame,
              FRAME_SHOW_LABEL, TRUE,
              XV_LABEL,str,
              FRAME_NO_CONFIRM,TRUE,
              FRAME_DONE_PROC,_3d_done_proc,
              0);
 c_free(str);
 xv_set(_3d->frame,FRAME_CLOSED,FALSE,0);
 PRT1(printf("setup_menu_3d_rec...1\n"))

 _3d->panel = xv_create(_3d->frame,PANEL,
               FRAME_INHERIT_COLORS,TRUE,
               0);

 if (_3d->panel == 0)
    {free_menu_3d_rec(_3d);
     return ~NO_ERROR;}

 row = 5;
 offset = xv_rows(_3d->panel,1) - xv_rows(_3d->panel,0) - 10;
/*
  _3d->cur_plane_msg = xv_create(_3d->panel,PANEL_MESSAGE,
                      XV_X,5,XV_Y,row,0);
 PRT1(printf("setup_menu_3d_rec...2\n"))
 show_cur_plane_params(_3d);
           
 PRT1(printf("setup_menu_3d_rec...3\n"))
 row += offset - 10;
*/

 item = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"LAST PLANE-[Sh] Ins",
	PANEL_NOTIFY_PROC,last_plane_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
 xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"REVERSE PLANE-B",
	PANEL_NOTIFY_PROC,reverse_plane_proc,
        PANEL_CLIENT_DATA,_3d,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
	0);

 row += offset;
 item = xv_create(_3d->panel,PANEL_MESSAGE,PANEL_LABEL_STRING,"SAME",
        XV_X,5,XV_Y,row,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
 item = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"PLANE AS-G",
	PANEL_NOTIFY_PROC,same_plane_proc,
        PANEL_CLIENT_DATA,_3d,
        XV_KEY_DATA,1,1,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
 xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"ORIENTATION AS-H",
	PANEL_NOTIFY_PROC,same_plane_proc,
        PANEL_CLIENT_DATA,_3d,
        XV_KEY_DATA,1,0,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
	0);

 _3d->same_view = xv_create(_3d->panel,PANEL_CHOICE_STACK,
        XV_Y,row,
	0);
 setup_any_view_wids(img,_3d->same_view,NULL,NULL);
 PRT1(printf("setup_menu_3d_rec...4\n"))


 PRT2(printf("setup_menu_3d_rec...5...2\n"))
 row += offset;
 item = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"Keypad mode-F11...",
	PANEL_NOTIFY_PROC,keypad_mode_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
 item = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"MORE...",
	PANEL_NOTIFY_PROC,plane_more_proc,
        XV_X,rr->r_left + rr->r_width + 3,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,
	0);
 row += offset;

/*
 _3d->cur_plane_param = xv_create(_3d->panel,PANEL_MESSAGE,
        XV_X,10,XV_Y,row,
	0);
 row += offset-10;
 _3d->cut_plane_param = xv_create(_3d->panel,PANEL_MESSAGE,
        XV_X,10,XV_Y,row,
	0);
 row += offset-10;
*/
 _3d->cur_degree_betw = xv_create(_3d->panel,PANEL_MESSAGE,
        XV_X,15,XV_Y,row,0);
 _3d->after_cut_deg_betw = xv_create(_3d->panel,PANEL_MESSAGE,
        XV_X,170,XV_Y,row,0);
 row += offset - 5;

 item = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"NEW PLANE",
	PANEL_NOTIFY_PROC,degree_cut_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
 _3d->window = xv_create(_3d->panel,PANEL_CHOICE_STACK,
	PANEL_LABEL_STRING,"TO",
        PANEL_CLIENT_DATA,_3d,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
        PANEL_NOTIFY_PROC,current_window_proc,
        0);
 setup_3d_window_labels(img,_3d->window,&(_3d->window_strs));
 row += offset;
 _3d->cut_mode = xv_create(_3d->panel,PANEL_CHOICE_STACK,
	PANEL_LABEL_STRING,"BET. CUR & NEW PLANES",
        PANEL_CHOICE_STRINGS,"90 CUT","DEGREE CUT",NULL,
        PANEL_NOTIFY_PROC,cut_mode_proc,
        PANEL_CLIENT_DATA,_3d,
        PANEL_VALUE,0,
        XV_X,15,XV_Y,row,
        0);
 if (_3d->win->control.which_cursor == CURSOR_ANGLE_CUT)
    xv_set(_3d->cut_mode,PANEL_VALUE,1,0);

 PRT1(printf("setup_menu_3d_rec...6\n"))
 PRT1(printf("setup_menu_3d_rec...7\n"))

 row += offset;
 _3d->set_degree_betw = xv_create(_3d->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"DEGREE CUT",
        PANEL_VALUE_DISPLAY_LENGTH,3,
        PANEL_VALUE_STORED_LENGTH,3,
	PANEL_VALUE,2,
	PANEL_MIN_VALUE,0,
	PANEL_MAX_VALUE,180,
	PANEL_SLIDER_WIDTH,60,
        XV_X,20,XV_Y,row,
	PANEL_NOTIFY_PROC,set_degree_betw_proc,
        PANEL_CLIENT_DATA,_3d,
	0);
 row += offset;

 item = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"ROT. ABOUT ROT. AXIS",
        PANEL_NOTIFY_PROC,about_proc,
        XV_X,20,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
 _3d->about = xv_create(_3d->panel,PANEL_CHOICE_STACK,
        PANEL_CHOICE_STRINGS,
               "ANTI-CLOCKWISE \"+\"","CLOCKWISE \"-\"",NULL,
        PANEL_VALUE,0,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
        0);
 TRACE(printf("...7...\n"))
 row += offset;
 _3d->gplane_angle = xv_create(_3d->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"ROT. AXIS DEG.",
	PANEL_VALUE,0,
	PANEL_MIN_VALUE,0,
	PANEL_MAX_VALUE,180,
	PANEL_SLIDER_WIDTH,60,
        PANEL_CLIENT_DATA,_3d,
        XV_X,15,XV_Y,row,
	PANEL_NOTIFY_PROC,set_3d_window_angle_proc,
	0);
 row += offset - 2;
/*
 item = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"PLANE#",
	PANEL_NOTIFY_PROC,locate_plane_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
*/
 _3d->pl_num = xv_create(_3d->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"NEW PLANE#",
	PANEL_SLIDER_WIDTH,90,
        PANEL_CLIENT_DATA,_3d,
        XV_X,20,XV_Y,row,
	PANEL_NOTIFY_PROC,set_3d_plane_num_proc,
	0);
 PRT1(printf("setup_menu_3d_rec...8\n"))
 cur_roi_win_cut_view(_3d);
 PRT1(printf("setup_menu_3d_rec...9\n"))
 row += offset + 5;
 _3d->rotate = xv_create(_3d->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"ROTATE WITHIN PLANE",
	PANEL_NOTIFY_PROC,rotate_proc,
        XV_X,15,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,
	0);
 rr = (Rect *) xv_get(_3d->rotate,XV_RECT);
 _3d->obs_rel = xv_create(_3d->panel,PANEL_CHOICE_STACK,
        PANEL_CHOICE_STRINGS,"ABSOLUTELY","RELATIVELY",NULL,
        PANEL_VALUE,0,
/*
        PANEL_NOTIFY_PROC,obs_rel_proc,
*/
        PANEL_CLIENT_DATA,_3d,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
        0);
 row += offset;

 angle = (float) degree(view->gplane.rotate.z);
 if (angle < .0) the_angle = 360 + (long) angle;
   else the_angle = (long) angle;

 _3d->rot_angle = xv_create(_3d->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"BY DEG.",
	PANEL_VALUE,the_angle,
	PANEL_MIN_VALUE,-179,
	PANEL_VALUE,2,
	PANEL_MAX_VALUE,180,
	PANEL_SLIDER_WIDTH,90,
        XV_X,20,XV_Y,row,
        PANEL_CLIENT_DATA,_3d,
	0);
 TRACE(printf("...8...\n"))
/*
 PRT1(printf("setup_menu_3d_rec...10\n"))
 row += offset - 5;
 _3d->auto_rot = xv_create(_3d->panel,PANEL_CHECK_BOX,
                   XV_X,15,XV_Y,row,
 	           PANEL_LABEL_STRING, "AUTO",PANEL_VALUE,0,
                   PANEL_CLIENT_DATA,_3d,
	           PANEL_NOTIFY_PROC,auto_rotate_proc,
                   0);
 rr = (Rect *) xv_get(_3d->auto_rot,XV_RECT);
 _3d->auto_sec = xv_create(_3d->panel,PANEL_NUMERIC_TEXT,
                   XV_X,rr->r_left + rr->r_width,XV_Y,row,
                   PANEL_VALUE_DISPLAY_LENGTH,5,
 	           PANEL_LABEL_STRING, "By sec.",PANEL_VALUE,0,
                   PANEL_MIN_VALUE,0,PANEL_MAX_VALUE,18000,
                   0);
*/
 window_fit(_3d->panel);
 rr = (Rect *) xv_get(_3d->panel,XV_RECT);
 xv_set(_3d->panel,XV_WIDTH,rr->r_width - 40,
           XV_HEIGHT,rr->r_height - 10,0);

 PRT1(printf("setup_menu_3d_rec...11\n"))
 window_fit(_3d->frame);
 make_2_wins_adjacent(_3d->frame,win->dwin->frame,0);
 win_set_mouse_at_center(_3d->frame);

 return NO_ERROR;
}


/*----------------------------------------------------------------------
find_menu_3d_rec - ???
----------------------------------------------------------------------*/

struct menu_3d_rec *
find_menu_3d_rec(all_3ds,win)
 struct all_menu_3d_recs *all_3ds;
 struct image_display_window *win;
{
 int lcv;

 for (lcv = 0; lcv < all_3ds->num_3ds; lcv++)
    if (all_3ds->_3ds[lcv]->win == win)
       return all_3ds->_3ds[lcv];
 return NULL;
}


/*----------------------------------------------------------------------
find_menu_3d_rec_win - ???
----------------------------------------------------------------------*/

struct menu_3d_rec *
find_menu_3d_rec_win(all_3ds,win)
 struct all_menu_3d_recs *all_3ds;
 struct image_display_window *win;
{
 int lcv;

 for (lcv = 0; lcv < all_3ds->num_3ds; lcv++)
    if (all_3ds->_3ds[lcv]->win == win)
       return all_3ds->_3ds[lcv];
 return NULL;
}



/*----------------------------------------------------------------------
update_3d_menu - ???
----------------------------------------------------------------------*/

void
update_3d_menu(win)
 struct image_display_window *win;
{
 struct all_menu_3d_recs *all_3ds;
 struct menu_3d_rec *_3d;
 int  val;
 long lcv;
 char *str;
 struct img_buf_rec *img;
 struct view_rec *view;

 PRT3(printf("....update_3d_menu..0\n");)
 if ((all_3ds = (struct all_menu_3d_recs *) enter_3d_plane(win,
              GET_ALL_3D_MENU_PTR)) != NULL)
    {
     if ((img = win_rec_cur_img_buf(win->owner)) == NULL)
        return;
     if ((view = img->cur_slice->slice->view) == NULL) return;
     for (lcv = 0; lcv < all_3ds->num_3ds; lcv++)
        {_3d = all_3ds->_3ds[lcv];
         if (_3d->win == win)
            {_3d->win->dwin = _3d->dwin;
             PRT3(printf("....update_3d_menu.._3d->dwin = %u\n",_3d->dwin);)
             /* show_cur_plane_params(_3d); */
             PRT3(printf("....update_3d_menu..2\n");)
             cur_roi_win_cut_view(_3d);
             PRT3(printf("....update_3d_menu..3\n");)
            }
          else
           {val = (int) xv_get(_3d->window,PANEL_VALUE);
            str = (char *) xv_get(_3d->window,PANEL_CHOICE_STRING,val);
             PRT3(printf("....update_3d_menu..4\n");)
            if (str_cmp(str,view->wid) == 0)
               cur_roi_win_cut_view(_3d);
             PRT3(printf("....update_3d_menu..5\n");)
           }
        }
    }
 PRT3(printf("....update_3d_menu..6\n");)
}


/*----------------------------------------------------------------------
get_3d_plane_rot_angle - ???
----------------------------------------------------------------------*/

int
get_3d_plane_rot_angle(win)
 struct image_display_window *win;
{
 struct all_menu_3d_recs *all_3ds;
 struct menu_3d_rec *_3d;

 if (((all_3ds = (struct all_menu_3d_recs *) 
       enter_3d_plane(win,GET_ALL_3D_MENU_PTR)) != NULL) &&
     ((_3d = find_menu_3d_rec_win(all_3ds,win)) != NULL))
    return (int) xv_get(_3d->set_degree_betw,PANEL_VALUE);
   else return 2;
}


/*----------------------------------------------------------------------
get_3d_plane_cut_angle - ???
----------------------------------------------------------------------*/

int
get_3d_plane_cut_angle(win)
 struct image_display_window *win;
{
 struct all_menu_3d_recs *all_3ds;
 struct menu_3d_rec *_3d;

 if (win->control.which_cursor == CURSOR_CUT) return 90;
 if (((all_3ds = (struct all_menu_3d_recs *) 
       enter_3d_plane(win,GET_ALL_3D_MENU_PTR)) != NULL) &&
     ((_3d = find_menu_3d_rec_win(all_3ds,win)) != NULL))
    {if (xv_get(_3d->cut_mode,PANEL_VALUE))
        return (int) xv_get(_3d->set_degree_betw,PANEL_VALUE);
    }
 return 90;
}


/*----------------------------------------------------------------------
update_3d_cut_mode - ???
----------------------------------------------------------------------*/

void
update_3d_cut_mode(win)
 struct image_display_window *win;
{
 struct all_menu_3d_recs *all_3ds;
 struct menu_3d_rec *_3d;

 if (((all_3ds = (struct all_menu_3d_recs *) 
       enter_3d_plane(win,GET_ALL_3D_MENU_PTR)) != NULL) &&
     ((_3d = find_menu_3d_rec_win(all_3ds,win)) != NULL))
    {if (xv_get(_3d->cut_mode,PANEL_VALUE))
        {if (win->control.which_cursor == CURSOR_CUT)
            {xv_set(_3d->cut_mode,PANEL_VALUE,0,0);
             show_cut_plane_params(_3d);}
        }
       else if (win->control.which_cursor == CURSOR_ANGLE_CUT)
        {xv_set(_3d->cut_mode,PANEL_VALUE,1,0);
         show_cut_plane_params(_3d);}
    }
}


/*----------------------------------------------------------------------
bring_up_cur_win - ???
----------------------------------------------------------------------*/

int
bring_up_cur_win(win)
 struct image_display_window *win;
{
 struct all_menu_3d_recs *all_3ds;
 struct menu_3d_rec *_3d;

 struct slice_roi_rec *slice_roi;
 char *cur_roi_label;
 unsigned long *indx;

 if (((all_3ds = (struct all_menu_3d_recs *) 
       enter_3d_plane(win,GET_ALL_3D_MENU_PTR)) != NULL) &&
     ((_3d = find_menu_3d_rec_win(all_3ds,win)) != NULL))
    {
     if (win_get_cur_slice_roi_rec(win,&slice_roi,&cur_roi_label) == NO_ERROR)
        {if (find_str_in_strs_rec(&(_3d->window_strs),cur_roi_label,&indx) == NO_ERROR)
            {
             xv_set(_3d->window,PANEL_VALUE,indx,0);
             return cur_roi_win_cut_view(_3d);}
        }
    }

 return NO_ERROR;   /* FIXME:  Added but not checked.  -PWH 19990319 */
}


/*----------------------------------------------------------------------
add_view_wid - ???
----------------------------------------------------------------------*/

int
add_view_wid(view,item)
 struct view_rec *view;
 Panel_item item;
{
 long cnt;

 if (view != NULL)
    {cnt = xv_get(item,XV_KEY_DATA,1);
     xv_set(item,PANEL_CHOICE_STRING,cnt,view->wid,0);
     cnt++;
     xv_set(item,XV_KEY_DATA,1,cnt,0);}
 return NO_ERROR;
}


/*----------------------------------------------------------------------
add_plane_window_list - ???
----------------------------------------------------------------------*/

int
add_plane_window_list(view)
 struct view_rec *view;
{
 struct all_menu_3d_recs *all_3ds;
 struct img_buf_rec *img;
 struct list_rec *head1,*head2;
 long lcv;

 if (view->img->win == NULL) return NO_ERROR;
 if ((all_3ds = (struct all_menu_3d_recs *) 
       enter_3d_plane(view->img->win->image_display,GET_ALL_3D_MENU_PTR)) != NULL)
    {
     for (lcv = 0; lcv < all_3ds->num_3ds; lcv++)
        {add_view_wid(view,all_3ds->_3ds[lcv]->same_view);
         if ((img = win_rec_cur_img_buf(all_3ds->_3ds[lcv]->win->owner)) != NULL) 
            if ((view->file == img->file) && (img->cur_slice->slice->view != NULL))
               {head1 = head_list(&(view->_3d_list));
                head2 = head_list(&(img->cur_slice->slice->view->_3d_list));
                if (head1 == head2)
                   add_view_wid(view,all_3ds->_3ds[lcv]->window);
               }
        }
    }
 return NO_ERROR;
}


/*----------------------------------------------------------------------
enter_3d_plane - ???
----------------------------------------------------------------------*/

unsigned long 
enter_3d_plane(win,status)
 struct image_display_window *win;
 int status;
{
 static struct all_menu_3d_recs all_3ds;
 static char first_time = 1;
 int err;
 struct menu_3d_rec *_3d;

 err = NO_ERROR;
 if (status == GET_ALL_3D_MENU_PTR)
    {if (first_time) return 0;
       else return (unsigned long) &all_3ds;}     /* FIXME:  ptr as long */
  else if (status == CLEAN_UP_3D_MENU)
   {if (!first_time)
       if ((_3d = find_menu_3d_rec(&all_3ds,win)) != NULL)
          free_menu_3d_rec(_3d);
    return NO_ERROR;}

 if (first_time)
    {init_all_menu_3d_recs(&all_3ds);
     first_time = 0;}
   else
    { 
     if ((_3d = find_menu_3d_rec_win(&all_3ds,win)) != NULL)
        {
         xv_set(_3d->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,WIN_ALARM,0);
         update_3d_menu(win);
         win_set_mouse_at_center(_3d->frame);
         wmgr_top(_3d->frame);
         return NO_ERROR;}
    }
 get_handle_memory(&(all_3ds._3ds),&(all_3ds.num_3ds),1,
     (size_t) sizeof(struct menu_3d_rec),&err);
 if (err != NO_ERROR) return err;
 _3d = all_3ds._3ds[all_3ds.num_3ds - 1];
 init_menu_3d_rec(_3d);
 if ((err = setup_menu_3d_rec(_3d,win)) == NO_ERROR)
    xv_set(_3d->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
 return err;
}
