/****************************************************************************
analysis_opr.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: analysis_opr.c,v 1.13 1999/07/07 17:41:14 hughett Exp $ Release $Name: nria_1_0_6 $";

/* FIXME:  Not needed?
#include <time.h>
#include <sys/timeb.h>
*/

#include <xview/frame.h>
#include <xview/notify.h>
#include <xview/panel.h>

#include "mri_format.h"
#include "debug.h"
#include "nria_procedures.h"


struct dp_analysis_rec
   {
    struct dp_analysis_rec **anal_addr;
    unsigned short status;
    struct analysis_rec param;
    Frame frame;
    Panel panel;
    Panel_item total_processed_slices,slice_num,num_rois,roi_label;
   };



/* Functions defined in this file */
void analysis_err_msgs(Frame anal_frame, int *err, int select);
int win_delete_all_slices_pixel_type_cnt_rec(struct image_display_window *win);
void init_dp_analysis_rec(struct dp_analysis_rec *anal);
void free_dp_analysis_rec(struct dp_analysis_rec *anal);
void free_dp_analysis_rec_it(struct dp_analysis_rec *anal, int which);
int win_analyze_pixel_types(struct dp_analysis_rec *anal, int which);
void cancel_anal_proc(Panel_item item, Event *event);
void anal_done_proc(Frame frame);
int setup_dp_analysis_rec(struct dp_analysis_rec *anal,
    struct dp_analysis_rec **anal_addr, struct image_display_window *win,
    unsigned int num_slices, int anal_status);
int win_anal_precondition(struct image_display_window *win,
    unsigned int num_slices, int anal_status);
int win_pixel_type_analysis(struct dp_analysis_rec **anal,
    struct image_display_window *win, unsigned int num_slices,
    int anal_status);
int win_analysis(struct image_display_window *win, unsigned int num_slices,
    int anal_status);



/*--------------------------------------------------------------------------
analysis_err_msgs - ???
--------------------------------------------------------------------------*/

void
analysis_err_msgs(anal_frame,err,select)
     Frame anal_frame;
     int *err,select;
{
  Frame frame;

  if (*err == ANALYSIS_ERR_SHOWN)
    return;
  if (anal_frame != 0)
    frame = xv_get(anal_frame,WIN_OWNER);
  else
    frame = 0;

  if (frame == 0)
    frame = *(main_display(2,NULL,0,0));

  switch (select)
  {
  case ANALYSIS_ERR :
    ShowMsgStrs(frame,0,*err,0,2,
		"An error occured when analyzing the pixel type counts...",
		"Operation aborted !!!",NULL,0);
    *err = ANALYSIS_ERR_SHOWN;
    break;
  case ANALYSIS_OUTPUT_FORMAT_ERR :
    ShowMsgStrs(frame,0,*err,0,2,
		"An error occured when setting up the text output format...",
		"Please check your text output format !!!",NULL,0);
    *err = ANALYSIS_ERR_SHOWN;
    break;
  case ANALYSIS_WINDOW_SETUP_ERR :
    ShowMsgStrs(frame,0,*err,0,2,
		"An error occured when setting up the analysis window...",
		"Probably out of memory !!!",NULL,0);
    *err = ANALYSIS_ERR_SHOWN;
    break;
  }
}


/*--------------------------------------------------------------------------
win_delete_all_slices_pixel_type_cnt_rec - ???
--------------------------------------------------------------------------*/

int
win_delete_all_slices_pixel_type_cnt_rec(win)
 struct image_display_window *win;
{
 struct img_buf_rec *img;
 unsigned int lcv,cnt;
 struct slice_roi_rec *slice_roi;

 TRACE(printf("win_delete_all_slices_pixel_type_cnt_rec...begin\n"))
 img = win_rec_cur_img_buf(win->owner);
 if (img == NULL) return CANCEL;

 for (lcv = 0; lcv < img->file->slice_info.num_sroi_wins; lcv++)
    {slice_roi = &(img->file->slice_info.sroi_wins[lcv]->slice_roi);
     for (cnt = 0; cnt < slice_roi->num_rois; cnt++)
        {if (slice_roi->rois[cnt]->user.ptr != NULL)
            slice_roi->rois[cnt]->user.exit(&(slice_roi->rois[cnt]->user.ptr));
        }
    }
 TRACE(printf("win_delete_all_slices_pixel_type_cnt_rec...end\n"))
 return NO_ERROR;

}


/*--------------------------------------------------------------------------
init_dp_analysis_rec - ???
--------------------------------------------------------------------------*/

void
init_dp_analysis_rec(anal)
 struct dp_analysis_rec *anal;
{
 init_analysis_rec(&(anal->param));
 anal->status = 0;
 anal->frame = 0;
}


/*--------------------------------------------------------------------------
free_dp_analysis_rec - ???
--------------------------------------------------------------------------*/

void
free_dp_analysis_rec(anal)
 struct dp_analysis_rec *anal;
{
  TRACE(printf("free_dp_analysis_rec()\n"));
  TRACE(printf("	anal = %p\n",(void *) anal));

  if (anal->frame != 0)
  {
    TRACE(printf("	destroying frame %p\n", (void *) anal->frame));
    xv_destroy_safe(anal->frame);
  }
  *(anal->anal_addr) = NULL;
  free_analysis_rec(&(anal->param));
  c_free(anal);
  TRACE(printf("free_dp_analysis_rec() end.\n"));
}


/*--------------------------------------------------------------------------
free_dp_analysis_rec_it - ???
--------------------------------------------------------------------------*/

void
free_dp_analysis_rec_it(anal,which)
 struct dp_analysis_rec *anal;
 int which;
{
 (void) notify_set_itimer_func((Notify_client) anal,
             NOTIFY_FUNC_NULL,ITIMER_REAL,
             (struct itimerval *) 0,(struct itimerval *) 0);
 free_dp_analysis_rec(anal);
}



/*--------------------------------------------------------------------------
win_analyze_pixel_types - ???
--------------------------------------------------------------------------*/

int
win_analyze_pixel_types(anal,which)
     struct dp_analysis_rec *anal;
     int which;
{
  struct int_bound_rec affected;
  unsigned int cnt,roi_indx;
  struct slice_roi_rec *slice_roi;
  struct pixel_type_cnt_rec *ptr;
  char slice_num_str[10];
  struct input_file_rec *file;
  struct output_text_format_rec *format;
  int err1;
  struct roi_rec *roi;
#ifdef DEBUG
  unsigned int indx, lcv;       /* FIXME:  Needed for DEBUG only */
#endif

  TRACE(printf("win_analyze_pixel_types()\n"));

  file = anal->param.file;
  slice_roi = &(anal->param.cur_slice->slice->slice_roi_win->slice_roi);
  if (anal->frame != 0)
  {
    xv_set(anal->total_processed_slices,PANEL_VALUE,
	   anal->param.cnt_slice + 1,0);
    sprintf(slice_num_str,"%ld",anal->param.cur_slice->num);
    xv_set(anal->slice_num,PANEL_VALUE,slice_num_str,0);
  }
  TRACE(printf("\nanal->param.cur_slice->num = %d\n",
	       anal->param.cur_slice->num));
  if (slice_roi->num_rois == 0)
  {
    if (anal->frame != 0)
    {
      xv_set(anal->num_rois,PANEL_MAX_VALUE,0,0);
      xv_set(anal->roi_label,PANEL_VALUE,"",0);
    }
  }
  else
  {
    /* # ROI's > 0 */
    init_common_pixel_rec_matrix(&(anal->param.common),0);
    format = &(anal->param.file->text.format);
    if (anal->frame != 0)
      xv_set(anal->num_rois,PANEL_MAX_VALUE,slice_roi->num_rois,0);

    anal->param._2d_ptr = NULL;
    /*  Analyze each ROI  */
    for (cnt = 0; cnt < format->roi_labels.num_strs; cnt++)
    {
      if (anal->frame != 0)
      {
	if (anal->num_rois)
	  xv_set(anal->num_rois,PANEL_VALUE,cnt + 1,0);
	if (anal->roi_label != 0)
	{
	  xv_set(anal->roi_label,PANEL_VALUE,
		 format->roi_labels.strs[cnt],0);
	  win_flush(anal->frame);
	}
      }
      roi_indx = format->roi_nums[cnt];
      if (roi_indx == 0)
	continue;
      if ((roi = get_roi(slice_roi,format->roi_labels.strs[cnt])) == NULL)
	continue;

#if 0
      /*  DEBUGGING DRW 960830  */
      printf("Slice #%d, ROI Label = \"%s\", ROI Index = %d.\n",
	     anal->param.next_slice_num + 1,
	     roi->label,
	     roi_indx);
#endif

      if (roi->user.ptr == NULL)
      {
	roi->user.exit = exit_pixel_type_cnt_rec;
	if ((anal->param.err =
	     setup_pixel_type_cnt_rec(&(roi->user.ptr),
				      &(file->pixel_type))) != NO_ERROR)
	  break;
      }
      NOPRT(printf("...***\n"));
      ptr = (struct pixel_type_cnt_rec *) roi->user.ptr;
      if (anal->param._2d_ptr == NULL)
      {
	TRACE(printf("...setup_cur_slice_map\n"))
	  /*  see pixel_type.c: it seems to be getting the pixel types
	   *    from file 'file' (which ought to be a pixmap) into _2d.
	   *    DRW 960906
	   */
	  if ((anal->param.err =
	       setup_cur_slice_map(&(anal->param._2d),&(anal->param._2d_ptr),
				   file, anal->param.cur_slice)) != NO_ERROR)
	    break;

#if 0
	/*  DEBUGGING  DRW 960830  This shows the whole pixmap counts
	    New procedure name, nlib/2d_buf_opr.c;
	    old was cnt_val_buf_2d_rec()  */
	for (lcv = 0; lcv <= file->pixel_type.order.num_strs + 1; lcv++)
	{
	  printf("    %d) cnt = %d\n",lcv, CntValBuf2dRec(anal->param._2d_ptr,
							  (double) lcv));
	}
#endif

      }
      TRACE(printf("...anal->param._2d_ptr = %u num_pixel_type_cnts = %d\n",
		   anal->param._2d_ptr,
		   ptr->num_pixel_type_cnts));
      if (((anal->param.err =
	    GetFilledRoiMask(roi,NULL,
			     &(anal->param.mask),&affected)) != NO_ERROR) ||
	  ((anal->param.err = 
	    cnt_pixel_types_by_mask(anal->param._2d_ptr,
				    &(anal->param.mask.mask._2d),&affected,
				    ptr->pixel_type_cnts,
				    ptr->num_pixel_type_cnts))
	   != NO_ERROR) ||
	  ((anal->param.err =
	    add_global_pixel_rec_matrix_by_mask(
						&(anal->param.common),
						&(anal->param.mask.mask._2d),
						&affected,
						anal->param._2d_ptr,roi_indx)) 
	   != NO_ERROR))
	break;

      ptr->status |= PIXEL_TYPE_CNTS_COMPUTED;
    } /* Done analyzing each ROI */


    TRACE(printf("set_common_pixel_rec_matrix...err = %d\n",anal->param.err));

    if (anal->param.err == NO_ERROR)
    {
      /*
       *  If desired, place the debugging routine here to see values
       *  before the call to correct_slice_counts().
       */
      anal->param.err = correct_slice_counts(&(anal->param),format,slice_roi);
      NOPRT(printf("after correct_slice_counts...err = %d\n",
		   anal->param.err));
    }

    /*
     *  Check for errors again after correct_slice_counts()
     */
    if (anal->param.err != NO_ERROR)
      unset_slice_counts(slice_roi,format,cnt);
    else 
    {

#ifdef OLD_DEBUG
      /*
       *  Old debugging routine
       *  DEBUGGING  DRW 960830
       */
      printf("Slice #%d, after\n", anal->param.next_slice_num + 1);
      for (cnt = 0; cnt < format->roi_labels.num_strs; cnt++)
      {
	if (format->roi_nums[cnt] == 0)
	  continue;
	if ((roi = get_roi(slice_roi,format->roi_labels.strs[cnt])) == NULL)
	  continue;
	    
	ptr = (struct pixel_type_cnt_rec *) roi->user.ptr;
	if (ptr == NULL)
	  continue;
	printf("        \"%s\" [%d]", roi->label, format->roi_nums[cnt]);
	for (lcv = 1; lcv < ptr->num_pixel_type_cnts; lcv++)
	  printf("; %d", ptr->pixel_type_cnts[lcv]);
	printf("\n");
      }
#endif

#ifdef DEBUG
      /*
       *  DEBUGGING: I discovered that the previous debugging routine only
       *    showed TRUE region pixel counts, not what was placed in the value
       *    field, as shown below.  This debugging printout occurs after
       *    the call to correct_slice_counts().
       *    DRW 960906
       */
      PRT3(
	   cnt = anal->param.next_slice_num;
	   for (indx = 0; indx < format->num_groups; indx++)
	   {
	     for (lcv = 0;
		  lcv < format->counts[cnt].groups[indx].num_fields;
		  lcv++)
	     {
	       if (format->counts[cnt].groups[indx].fields[lcv].valid == 1)
		 printf("Sl %d, Gr %d, Fl %d) valid = %x val = %lu\n",
			cnt+1,
			indx+1,
			lcv+1,
			format->counts[cnt].groups[indx].fields[lcv].valid,
			format->counts[cnt].groups[indx].fields[lcv].value);
	     }
	   }
	   );
#endif
    }
    TRACE(printf("correct_slice_counts...\n"));
  }

  if ((anal->param.err != NO_ERROR) ||
      (anal->param.cnt_slice < anal->param.num_slices))
  {
    anal->param.cnt_slice += 1;
    anal->param.next_slice_num += 1;
    if (anal->param.next_slice_num >= anal->param.total_slices)
      anal->param.next_slice_num = 0;
    anal->param.cur_slice = anal->param.cur_slice->next_slice;
  }
  check_analysis_status(&(anal->param));
  if (anal->param.err != NO_ERROR) 
    analysis_err_msgs(anal->frame,&(anal->param.err),ANALYSIS_ERR);

  TRACE(printf("*****anal->param.cnt_slice = %d anal->param.num_slices = %d err = %d\n",
	       anal->param.cnt_slice,
	       anal->param.num_slices,anal->param.err));

  if ((anal->param.err != NO_ERROR) ||
      (anal->param.cnt_slice == anal->param.num_slices))
  {
    (void) notify_set_itimer_func((Notify_client) anal,
				  NOTIFY_FUNC_NULL,ITIMER_REAL,
				  (struct itimerval *) 0,
				  (struct itimerval *) 0);
    err1 = anal->param.err;
    notify_set_itimer_func((Notify_client) anal,
			   (Notify_func) free_dp_analysis_rec_it,ITIMER_REAL,
			   &NOTIFY_POLLING_ITIMER,(struct itimerval *) 0);
    return err1;
  }
  else
    return anal->param.err;

}  /*  win_analyze_pixel_types()  */



/*--------------------------------------------------------------------------
cancel_anal_proc - ???
--------------------------------------------------------------------------*/

void
cancel_anal_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct dp_analysis_rec *anal;

 anal = (struct dp_analysis_rec *) panel_get(item,PANEL_CLIENT_DATA);
 (void) notify_set_itimer_func((Notify_client) anal,
             NOTIFY_FUNC_NULL,ITIMER_REAL,
             (struct itimerval *) 0,(struct itimerval *) 0);
 notify_set_itimer_func((Notify_client) anal,
             (Notify_func) free_dp_analysis_rec_it,ITIMER_REAL,
             &NOTIFY_POLLING_ITIMER,(struct itimerval *) 0);
}


/*--------------------------------------------------------------------------
anal_done_proc - ???
--------------------------------------------------------------------------*/

void
anal_done_proc(frame)
 Frame frame;
{
 struct dp_analysis_rec *anal;

 TRACE(printf("anal_done_proc()\n	(Frame *) = %p", (void *) frame););
 anal = (struct dp_analysis_rec *) xv_get(frame,WIN_CLIENT_DATA);
 TRACE(printf("	anal = %p", (void *) anal););
 (void) notify_set_itimer_func((Notify_client) anal,NOTIFY_FUNC_NULL,
             ITIMER_REAL,
             (struct itimerval *) 0,(struct itimerval *) 0);
 free_dp_analysis_rec(anal);
 TRACE(printf("anal_done_proc() end\n"););
}


/****************************
define_pixel_type_indxs(pixel_type_indxs,format,pixel_types)
 unsigned int *pixel_type_indxs;
 struct output_text_format_rec *format;
 struct strs_rec *pixel_types;
{
 unsigned int lcv,indx;
 int err;
 char *str;

 for (lcv = 0; lcv < format->num_pixel_types; lcv++)
    {
     if ((err = cp_str(&str,pixel_types->strs[lcv])) != NO_ERROR)
        return err;
     drop_suffix(str);
     if ((err = find_str_in_strs_rec(&(format->pixel_type_labels),str,&indx)) == 
                    NO_ERROR)
        pixel_type_indxs[lcv] = indx + 1;
       else
        {pixel_type_indxs[lcv] = 0;
        }
     printf("define_pixel_type %d) str = %s  pixel_type_indxs = %d\n",lcv,str,pixel_type_indxs[lcv]);
     c_free(str);
    }
 return NO_ERROR;
}
*******************/



/*--------------------------------------------------------------------------
setup_dp_analysis_rec - ???
--------------------------------------------------------------------------*/

int
setup_dp_analysis_rec(anal,anal_addr,win,num_slices,anal_status)
 struct dp_analysis_rec *anal,**anal_addr;
 struct image_display_window *win;
 unsigned int num_slices;
 unsigned short anal_status;
{
  struct img_buf_rec *img;
  char *str;
  Rect *rr;
  int err;
  struct output_text_format_rec *format;
  int row,offset;

  anal->anal_addr = anal_addr;
  anal->param.num_slices = num_slices;
  anal->status = anal_status;
  img = win_rec_cur_img_buf(win->owner);
  if (img == NULL) return CANCEL;
  anal->param.cur_slice = img->cur_slice;
  anal->param.cnt_slice = 0;
  anal->param.file = img->file;
  anal->param.next_slice_num = anal->param.cur_slice->slice_num;

  anal->param.total_slices = win_total_slice_cuts(win->owner);
  TRACE(printf("setup_dp_analysis_rec...0\n"));
  if ((err = setup_output_format(&(anal->param.file->text))) != NO_ERROR)
  {
    analysis_err_msgs(anal->frame,&(anal->param.err),
		      ANALYSIS_OUTPUT_FORMAT_ERR);
    return err;
  }
  TRACE(printf("setup_dp_analysis_rec...1\n"));
  format = &(anal->param.file->text.format);
  if ((err = setup_analysis_params(&(anal->param),
				   format,&(anal->param.file->pixel_type)))
      != NO_ERROR)
    return err;
  TRACE(printf("num_str = %d \n", format->roi_labels.num_strs));

  if (((err = get_draw_mask_rec(&(anal->param.mask),
				img->_2d.dimx,img->_2d.dimy)) != NO_ERROR) ||
      ((err = SetupBuf2dRec(&(anal->param.common),
			    img->_2d.dimx,img->_2d.dimy,
			    8 * (size_t) sizeof(struct common_pixel_rec),0))
       != NO_ERROR))
  {
    return err;
  }
  init_common_pixel_rec_matrix(&(anal->param.common),1);


  /*  Create analysis dialog, which keeps user up-to-date of status  */
  anal->frame =
    xv_create(win->dwin->frame,FRAME,
	      FRAME_INHERIT_COLORS,TRUE,
	      FRAME_NO_CONFIRM,TRUE,
	      0);
  if (anal->frame == 0)
    return NO_ERROR;

  /*  Set dialog title bar text, and routine to call upon completion  */
  if (img->file->pixel_type.status & PIXMAP_METHOD_DEFINED)
    str = PIXEL_TYPE_GROW_REGION_ANALYSIS_STR;
  else
    str = PIXEL_TYPE_PARTITION_ANALYSIS_STR;
  str = win_rec_win_id_title(win->owner,str);
  xv_set(anal->frame,
	 FRAME_SHOW_LABEL, TRUE,
	 XV_LABEL,str,
	 FRAME_DONE_PROC,anal_done_proc,
	 WIN_CLIENT_DATA,anal,
	 0);
  c_free(str);

  anal->panel =
    xv_create(anal->frame,PANEL,
	      FRAME_INHERIT_COLORS,TRUE,
	      0);
  if (anal->panel == 0)
  {
    xv_destroy_safe(anal->frame);
    anal->frame = 0;
    return NO_ERROR;
  }

  row = 5;
  offset = xv_rows(anal->panel,1) - xv_rows(anal->panel,0) - 10;

  anal->total_processed_slices =
    xv_create(anal->panel,PANEL_SLIDER,
	      PANEL_LABEL_STRING,"# slices processed: ",
	      PANEL_VALUE,0,PANEL_MIN_VALUE,0,
	      PANEL_MAX_VALUE,num_slices,
	      PANEL_SLIDER_WIDTH,50,
	      XV_X,5,XV_Y,row,
	      0);
  row += offset;

  anal->slice_num =
    xv_create(anal->panel,PANEL_TEXT,
	      PANEL_LABEL_STRING,"Current slice #: ",
	      PANEL_VALUE,"",
	      PANEL_VALUE_STORED_LENGTH,10,
	      PANEL_VALUE_DISPLAY_LENGTH,10,
	      XV_X,5,XV_Y,row,
	      0);

  str = "CANCEL";
  xv_create(anal->panel,PANEL_BUTTON,
	    PANEL_LABEL_STRING, str,
	    PANEL_NOTIFY_PROC,cancel_anal_proc,
	    PANEL_CLIENT_DATA,anal,
	    XV_Y,row,
	    0);
  row += offset;

  anal->num_rois =
    xv_create(anal->panel,PANEL_SLIDER,
	      PANEL_LABEL_STRING,"# of ROIs processed: ",
	      PANEL_VALUE,0,PANEL_MIN_VALUE,0,
	      PANEL_MAX_VALUE,0,
	      PANEL_SLIDER_WIDTH,50,
	      XV_X,5,XV_Y,row,
	      0);
  row += offset;

  anal->roi_label =
    xv_create(anal->panel,PANEL_TEXT,
	      PANEL_LABEL_STRING,"Current ROI label: ",
	      PANEL_VALUE,"",
	      PANEL_VALUE_STORED_LENGTH,60,
	      PANEL_VALUE_DISPLAY_LENGTH,25,
	      XV_X,5,XV_Y,row,
	      0);
  rr = (Rect *) xv_get(anal->roi_label,XV_RECT);
  xv_set(anal->panel,
	 XV_WIDTH,rr->r_left + rr->r_width + 5,
	 XV_HEIGHT,rr->r_top + rr->r_height + 5,
	 0);
  window_fit(anal->frame);

  make_2_wins_adjacent(anal->frame,win->dwin->frame,0);
  xv_set(anal->frame,
	 XV_SHOW,TRUE,
	 FRAME_CLOSED,FALSE,
	 0);
  xv_set(anal->frame,
	 WIN_CLIENT_DATA,anal,
	 0);
  xv_set(anal->frame,
	 XV_KEY_DATA,1,anal,
	 XV_KEY_DATA,2,win_analyze_pixel_types,
	 XV_KEY_DATA,3,1,
	 0);
  notify_interpose_event_func((Notify_client) anal->frame,
			      (Notify_func) next_func,NOTIFY_SAFE);
  
  return NO_ERROR;
}



/*--------------------------------------------------------------------------
win_anal_precondition - ???
--------------------------------------------------------------------------*/

int
win_anal_precondition( win, num_slices, anal_status )
     struct image_display_window *win;
     unsigned int num_slices;
     unsigned short anal_status;
{
  struct img_buf_rec *img;
  struct hdr_scan_entry_rec *cur_slice;
  long lcv,cnt,sl_num;
  struct pixmap_hdr_rec *hdr;

  /*  Added for more descriptive error messages.  DRW 960808  */
  char addlErrInfo[256], tmpString[256];


  TRACE(printf("win_anal_precondition()\n"););

  if ((img = win_rec_cur_img_buf(win->owner)) == NULL)
    return CANCEL;
  addlErrInfo[0] = 0;


  /*  Count Value has no prerequisites.  */
  if (anal_status & PIXEL_COUNT_VALUE_ANALYSIS)
  {

    TRACE(printf("	Checking Count Value prerequisites...\n"););

    return NO_ERROR;
  }

  /*  2D Partition needs thresholds to define pixel types.  */
  else if (anal_status & PIXEL_TYPE_PARTITION_ANALYSIS)
  {

    TRACE(printf("	Checking Partition prerequisites...\n"););

    if (!(img->file->pixel_type.status & PIXEL_TYPE_2D_PARTITION))
    {
      ShowMsgStrs(win->dwin->frame,0,NO_ERROR,0,2,
		  "Please define the pixel types by thresholding first.",
		  OPERATION_ABORTED,NULL,0);
      return CANCEL;
    }
    return NO_ERROR;
  }

  /*  Grow Region needs a pixel type map (pixmap).  */
  else if (anal_status & PIXEL_TYPE_GROW_REGION_ANALYSIS)
  {

    TRACE(printf("	Checking Grow Region prerequisites...\n"););

    hdr = &(img->file->pixel_type.pixmap_hdr);
    cur_slice = img->cur_slice;
    for (cnt = 0,lcv = 0; lcv < num_slices; lcv++)
    {
      sl_num = cur_slice->num - 1;
      TRACE(printf("\r	Checking slice #%d... ",cur_slice->num););

      if ((sl_num >= 0) &&
	  (sl_num < img->file->pixel_type.num_slice_maps) &&
	  (img->file->pixel_type.slice_maps[sl_num] != NULL) &&
	  (img->file->pixel_type.slice_maps[sl_num]->status4 & 
	   BUF_DATA_VALID))
      {
	cur_slice = cur_slice->next_slice;
	cnt += 1;
	TRACE(printf("OK.\n"););
      }
      else
      {
	if ((img->file->pixel_type.status & PIXEL_TYPE_PIXMAP_HDR_VALID) &&
	    (sl_num < hdr->max_slices) &&
	    (hdr->pixmap_offsets[sl_num] != 0))
	{
	  cur_slice = cur_slice->next_slice;
	  cnt += 1;
	}
	else
	{
	  TRACE(printf("error!\n"););
	  if (~img->file->pixel_type.status & PIXEL_TYPE_PIXMAP_HDR_VALID)
	  {
	    strcpy(addlErrInfo, "Pixel type pixmap hdr invalid.");
	    strcat(addlErrInfo, "\n(Perhaps pixmap file non-existent?)  ");
	  }
	  else if (sl_num >= hdr->max_slices)
	  {
	    sprintf(tmpString, "Slice # too large (%ld >= %ld).",
		    sl_num, hdr->max_slices);
	    strcpy(addlErrInfo, tmpString);
	  }
	  else if (hdr->pixmap_offsets[sl_num] == 0)
	  {
	    sprintf(tmpString, "Pixmap offset for slice %ld is 0.", sl_num);
	    strcpy(addlErrInfo, tmpString);
	  }
	  else
	    strcpy(addlErrInfo, "Unable to determine cause.");
	  break;
	}
      }

    } /* for (# slices) */

    if (lcv == num_slices)
    {
      TRACE(printf("\n	win_anal_precondition() done, no errors.\n"););
      return NO_ERROR;
    }

    TRACE(printf("\n	Error message:\n%s\n", addlErrInfo););
    TRACE(printf("	Compiled error cause; now displaying.\n"););

    /*  No slices have defined pixel types.  */
    if ((cnt == 0) && (num_slices == img->file->slice_info.num_sroi_wins))
    {
      ShowMsgStrs(win->dwin->frame,0,NO_ERROR,0,4,
		  "Please define the pixel types first",
		  "for all slices to be analyzed.\n",
		  addlErrInfo,
		  OPERATION_ABORTED,
		  NULL,0);
    }

    /*  The only slice to be analyzed does not have defined pixel types.  */
    else if (num_slices == 1)
    {
      ShowMsgStrs(win->dwin->frame,0,NO_ERROR,0,6,
		  "The pixel types for the current slice have",
		  "not yet been defined.",
		  "Please define the pixel types first",
		  "for all slices to be analyzed.\n",
		  addlErrInfo,
		  OPERATION_ABORTED,
		  NULL,0);
    }

    /*  We're analyzing > 1 slice, and some slices do not have
	defined pixel types.  */
    else
    {
      ShowMsgStrs(win->dwin->frame,0,NO_ERROR,0,6,
		  "The pixel types for some slices may",
		  "not have been defined yet.",
		  "Please define the pixel types first",
		  "for all slices to be analyzed.\n",
		  addlErrInfo,
		  OPERATION_ABORTED,
		  NULL,0);
    }
  }

  /*  There are no other analysis method options, currently.  */
  else
  {
    ShowMsgStrs(win->dwin->frame,0,NO_ERROR,0,2,
		"Method of analysis is unknown (this should not happen!)",
		OPERATION_ABORTED,
		NULL,0);
  }

  TRACE(printf("	win_anal_precondition() end.\n"););
  return CANCEL;
}



/*--------------------------------------------------------------------------
win_pixel_type_analysis - ???
--------------------------------------------------------------------------*/

int
win_pixel_type_analysis(anal,win,num_slices,anal_status)
     struct dp_analysis_rec **anal;
     struct image_display_window *win;
     unsigned int num_slices;
     unsigned short anal_status;
{
  int err;

  TRACE(printf("win_pixel_type_analysis()\n"););
  if ((err = get_ptr(anal,NULL,(unsigned int) 1,
		     (size_t) sizeof(struct dp_analysis_rec))) != NO_ERROR)
    return err;
  init_dp_analysis_rec(*anal);
  TRACE(printf("	dp_analysis_rec struct initialized: %p\n",
	       (void *) *anal););

  if ((err = setup_dp_analysis_rec(*anal,anal,win,num_slices, anal_status))
      != NO_ERROR)
  {
    TRACE(printf("	problem with setup; aborting analysis.\n"););
    analysis_err_msgs((*anal)->frame,&((*anal)->param.err),
		      ANALYSIS_WINDOW_SETUP_ERR);
    TRACE(printf("	cleaning up dp_analysis_rec struct.\n"););
    free_dp_analysis_rec(*anal);
  }
  else
  {
    TRACE(printf("	analysis setup OK; timer proceeding.\n"););
  }
  TRACE(printf("win_pixel_type_analysis() end.\n"););
  return err;
}
  

/*--------------------------------------------------------------------------
win_analysis - ???

This gets called by analyze_all_slices_proc() in anal_menu.c,
when shift-A is pressed inside an editing window
(win->type2 & EDITING_WINDOW  ==  1)
--------------------------------------------------------------------------*/

int
win_analysis(win,num_slices,anal_status)
     struct image_display_window *win;
     unsigned int num_slices;
     unsigned short anal_status;
{
  struct img_buf_rec *img;
  static void *anal = NULL;
  int err;

  TRACE(printf("win_anal num_slices = %d anal = %u\n",num_slices,anal));
  if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
  TRACE(pr_strs_rec(&(img->file->pixel_type.order)));
  if (anal != NULL) return NO_ERROR;
  if ((err = win_anal_precondition(win,num_slices,anal_status)) != 
      NO_ERROR)
  {
    return err;
  }
  if (anal_status & PIXEL_COUNT_VALUE_ANALYSIS)
      /* FIXME!!  In the line below, anal is used as a pointer to a
	 private struct defined only within pixel_count_anal.c; I
	 really suspect a serious bug lurking here.  In any case,
	 the mismatch prevents us from defining a global prototype for
	 win_pixel_count_analysis.  -PWH 19990527 */
      return win_pixel_count_analysis(&anal,win,num_slices,anal_status);
  else
      return win_pixel_type_analysis(&anal,win,num_slices,anal_status);
}

