/****************************************************************************
co_register_file.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: co_register_file.c,v 1.9 1999/07/07 17:41:14 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/wmgr.h>

#include "geometry.h"
#include "mri_format.h"
#include "nria_procedures.h"

/*
#define PRINT(x) printf(x); fflush(stdout);
#define PRT(x) x; fflush(stdout);
#define PRT2(x)
#define PRT2(x) x; fflush(stdout);
#define PRT9(x) x; fflush(stdout);
#define PRT8(x) x; fflush(stdout);
*/
#define PRINT(x)
#define PRT(x)
#define PRT2(x)
#define PRT9(x)
#define PRT8(x)


#define NPCO_2d ".npco-2d"


#define CLEAN_UP_DP 0
#define SETUP_CO_REGISTER 1
#define GET_ALL_DP_PTR 2

struct co_file_rec
   {
    unsigned short status;
    struct image_display_window *win;
    struct display_win_rec *dwin;
    Frame frame;
    Panel panel;
    Panel_item wid,use_option,open_fn,open_dir,
               save_fn,save_dir;
   };

struct all_co_file_recs
   {
    unsigned long numdps;
    struct co_file_rec **dps;
   };


struct plane_param_rec
   {
    struct vector_3d_rec base,normal,center,coord_ratio,rotate;
    int which_format,adjust_to_center;
    struct matrix_3d_rec xyztoplane;
   };

struct co_param_rec
   {
    struct plane_param_rec p1,p2;
   };


/* Functions defined in this file */
void init_co_file_rec(struct co_file_rec *dp);
void free_co_file_rec(struct co_file_rec *dp);
void init_all_co_file_recs(struct all_co_file_recs *alldps);
void dp_co_file_done_proc(Frame frame);
int win_update_co_file_menu(struct image_display_window *win, char *wid);
int win_co_register_with_wid_file(struct image_display_window *win, char *wid);
int WinCoRegFiles(struct image_display_window *win, struct strs_rec *fns);
int win_unco_register_with_wid_file(struct image_display_window *win,
    char *wid);
int WinCoRegSnum(struct image_display_window *win, int *is_snum, long snum);
int WinSetupFileListItem(struct image_display_window *win, Panel_item item,
    int select);
char *co_file_unique_save_fn(char *dir, char *fn, char *suffix);
int co_file_def_save_fn(struct input_file_rec *file,
    struct input_file_rec *regfile, char *dir, char *suffix, char **fn);
int co_file_next_save_fn(struct input_file_rec *file);
int SaveNpco2d(char *dir, char *fn1, char *suffix, Frame frame,
    int is_replace, struct file_register_rec *fco,
    struct input_file_rec *file);
int win_save_file_npco_2d(struct image_display_window *win,
    struct input_file_rec *file, struct input_file_rec *regfile);
int win_save_npco_2d(struct image_display_window *win, char *fn);
int co_file_open_param(struct image_display_window *win, char *wid,
    struct co_param_rec *pa);
int WinAskOtherFileWid(struct image_display_window *win, char *title,
    int (*action_proc)(), void *arg);
int rd_npco_2d_params(FILE *fid, struct image_display_window *win,
    Xv_Window window);
int win_open_npco_2d(struct image_display_window *win, char *fn, char *dir);
int win_load_npco_2d(struct image_display_window *win);
int win_reverse_npco_2d_params(struct image_display_window *win, char *fn);
void co_file_proc(Panel_item item, char *fn, caddr_t client_data,
    Panel_list_op op, Event *event);
void co_file_fns_proc(Panel_item item, Event *event);
void co_file_use_option_proc(Panel_item item, int value, Event *event);
void co_file_more_proc(Panel_item item, Event *event);
void co_file_def_open_npco_fn_proc(Panel_item item, Event *event);
int co_file_open_npco_proc(Panel_item item, Event *event);
void co_file_def_save_npco_fn_proc(Panel_item item, Event *event);
void co_file_save_npco_proc(Panel_item item, Event *event);
int setup_co_file_rec(struct co_file_rec *dp,
    struct image_display_window *win);
struct co_file_rec *find_co_file_rec(struct all_co_file_recs *alldps,
    struct image_display_window *win);
struct co_file_rec *find_co_file_rec_by_file(struct all_co_file_recs *alldps,
    struct input_file_rec *file);
int enter_co_register_file(struct image_display_window *win, int status);



/*-------------------------------------------------------------------------
init_co_file_rec - ???
-------------------------------------------------------------------------*/

void
init_co_file_rec(dp)
 struct co_file_rec *dp;
{
 dp->status = 0;
 dp->win = NULL;
 dp->frame = 0;
}


/*-------------------------------------------------------------------------
free_co_file_rec - ???
-------------------------------------------------------------------------*/

void
free_co_file_rec(dp)
 struct co_file_rec *dp;
{
 struct all_co_file_recs *alldps;

 PRT(printf("free_co_file_rec...\n"))
 if (dp->frame != 0) xv_destroy_safe(dp->frame);
 alldps = (struct all_co_file_recs *) enter_co_register_file(NULL,
            GET_ALL_DP_PTR);
 c_free(dp);
 DropHandleAryPtr(&(alldps->dps),&(alldps->numdps),dp);
 PRT(printf("free_co_file_rec...end\n"))
}


/*-------------------------------------------------------------------------
init_all_co_file_recs - ???
-------------------------------------------------------------------------*/

void
init_all_co_file_recs(alldps)
 struct all_co_file_recs *alldps;
{
 alldps->numdps = 0;
 alldps->dps = NULL;
}


/*-------------------------------------------------------------------------
dp_co_file_done_proc - ???
-------------------------------------------------------------------------*/

void
dp_co_file_done_proc(frame)
 Frame frame;
{
 struct co_file_rec *dp;

 dp = (struct co_file_rec *) xv_get(frame,WIN_CLIENT_DATA);
 free_co_file_rec(dp);
}


/*-------------------------------------------------------------------------
win_update_co_file_menu - ???
-------------------------------------------------------------------------*/

int
win_update_co_file_menu(win,wid)
 struct image_display_window *win;
 char *wid;
{
 struct all_co_file_recs *alldps;
 struct co_file_rec *dp;
 struct win_rec *rwin;
 struct img_buf_rec *img,*img1;
 struct file_register_rec *fco;

 PRT2(printf("win_update_co_file_menu...wid = %s\n",wid))
 if (((img = win_rec_cur_img_buf(win->owner)) == NULL) ||
     ((alldps = (struct all_co_file_recs *) enter_co_register_file(NULL,
            GET_ALL_DP_PTR)) == NULL) ||
     ((dp = find_co_file_rec(alldps,win)) == NULL) ||
     ((rwin = get_win_rec_by_wid_str(win->owner,wid)) == NULL) ||
     ((img1 = win_rec_cur_img_buf(rwin)) == NULL))
    return CANCEL;

 PRT2(printf("win_update_co_file_menu...1\n"))
 if ((fco = find_file_register_rec(img->file,img1->file,NULL,2)) != NULL)
    xv_set(dp->use_option,PANEL_VALUE,1,0);
   else xv_set(dp->use_option,PANEL_VALUE,0,0);

 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_co_register_with_wid_file - ???
-------------------------------------------------------------------------*/

int
win_co_register_with_wid_file(win,wid)
 struct image_display_window *win;
 char *wid;
{
 struct img_buf_rec *img,*img1;
 struct win_rec *rwin;
 struct image_display_window *win1;
 int err,which;

 if ((wid == NULL) ||
     ((img = win_rec_cur_img_buf(win->owner)) == NULL))
    return CANCEL;
 PRT8(printf("win_co_register_with_wid_file wid =%s\n",wid))
 err = CANCEL;
 if (((rwin = get_win_rec_by_wid_str(win->owner,wid)) != NULL) &&
     ((win1 = rwin->image_display) != NULL) &&
     ((img1 = win_rec_cur_img_buf(rwin)) != NULL) &&
     (img1->file != img->file))
    {PRT8(printf("win_co_register_with_wid_file 1\n"))
     which = 1;
     if ((err = SetupImgRegParams(img,img1,0x0000000d,NULL,NULL)) != NO_ERROR)
        return err;
     PRT8(printf("win_co_register_with_wid_file which = %d\n",which))
     SetFileRegisterRecInUse(img->file,img1->file,which);
     SetFileRegisterRecInUse(img1->file,img->file,which);

     win_run_proc_to_all_file_wins(win,WinSetDrawLinkCoRegParams,1,0,0,0);
     win_update_co_file_menu(win,wid);
     win_run_proc_to_all_file_wins(win1,WinSetDrawLinkCoRegParams,1,0,0,0);
     win_update_co_file_menu(win1,win->owner->wid);
    }
 PRT8(printf("win_co_register_with_wid_file err = %d\n",err))
 return err;
}


/*-------------------------------------------------------------------------
WinCoRegFiles - ???
-------------------------------------------------------------------------*/

int
WinCoRegFiles(win,fns)
 struct image_display_window *win;
 struct strs_rec *fns;
{
 struct img_buf_rec *img;
 struct strs_rec fns1;
 struct input_file_rec *file;
 struct file_register_rec *fco;
 unsigned long lcv;
 int err;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 init_strs_rec(&fns1);
 if ((err = AllFiles(img->file,&fns1,1,0)) == NO_ERROR)
    {for (lcv = 0; lcv < fns1.num_strs; lcv++)
        if (((file = find_file(img->file,fns1.strs[lcv],NULL)) != NULL) &&
            ((fco = find_file_register_rec(img->file,file,NULL,2)) != NULL))
           {if ((err = add_str_to_strs_rec(fns,fns1.strs[lcv])) != NO_ERROR)
               break;
           }
    }
 free_strs_rec(&fns1);
 return err;
}


/*-------------------------------------------------------------------------
win_unco_register_with_wid_file - ???
-------------------------------------------------------------------------*/

int
win_unco_register_with_wid_file(win,wid)
 struct image_display_window *win;
 char *wid;
{
 struct img_buf_rec *img,*img1;
 struct win_rec *rwin;
 int err,is_setup;

 PRT8(printf("win_unco_register_with_wid_file wid =%s\n",wid))
 if ((wid == NULL) ||
     ((img = win_rec_cur_img_buf(win->owner)) == NULL) ||
     ((rwin = get_win_rec_by_wid_str(win->owner,wid)) == NULL) ||
     ((img1 = win_rec_cur_img_buf(rwin)) == NULL))
    return CANCEL;
 is_setup = 0;
 if ((SetFileRegisterRecInUse(img->file,img1->file,1) != NO_ERROR) ||
     (SetFileRegisterRecInUse(img1->file,img->file,1) != NO_ERROR))
    is_setup = 1;
   else is_setup = 0;

 if (is_setup)
    {PRT8(printf("win_unco_register_with_wid_file is_setup...\n"))
     if ((err = SetupImgRegParams(img,img1,0x00000007,NULL,NULL)) != NO_ERROR)
        return err;
    }
 win_run_proc_to_all_file_wins(win,WinSetDrawLinkCoRegParams,1,0,0,0);
 win_update_co_file_menu(win,wid);
 PRT8(printf("win_co_register_with_wid_file 1\n"))
 if (rwin->image_display != NULL)
    {
     win_run_proc_to_all_file_wins(rwin->image_display,
				   WinSetDrawLinkCoRegParams,1,0,0,0);
     win_update_co_file_menu(rwin->image_display,win->owner->wid);}
 PRT8(printf("win_co_register_with_wid_file 2\n"))
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
WinCoRegSnum - ???
-------------------------------------------------------------------------*/

int
WinCoRegSnum(win,is_snum,snum)
 struct image_display_window *win;
 int *is_snum;
 long snum;
{
 *is_snum = 0;
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
WinSetupFileListItem - ???
-------------------------------------------------------------------------*/

int
WinSetupFileListItem(win,item,select)
 struct image_display_window *win;
 Panel_item item;
 int select;
{
 struct img_buf_rec *img;
 int err;
 struct strs_rec fns;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 PRT8(printf("WinSetupFileListItem\n"))
 init_strs_rec(&fns);
 if ((err = AllFiles(img->file,&fns,1,select)) == NO_ERROR)
    {setup_list_items(&fns,item,0);
    }
 free_strs_rec(&fns);
 return err;
}


/*-------------------------------------------------------------------------
co_file_unique_save_fn - ???
-------------------------------------------------------------------------*/

char *
co_file_unique_save_fn(dir,fn,suffix)
 char *dir,*fn,*suffix;
{
 static char *last_fn = NULL,*cur_fn = NULL;
 char *fn1,*fn2;

 PRT8(printf("co_file_unique_save_fn fn = %s\n",fn))
 PRT8(printf("last_fn = %s cur_fn = %s\n",last_fn,cur_fn))
 if (str_cmp(cur_fn,fn) == 0)
    {if (last_fn != NULL) fn2 = last_fn;}
   else fn2 = fn;

 if ((fn1 = UniqueFn(dir,fn2,suffix)) != NULL)
    {c_free(cur_fn);
     c_free(last_fn);
     cp_str(&last_fn,fn2);
     cp_str(&cur_fn,fn1);
     drop_dirfn(cur_fn);
     drop_suffix(cur_fn);
     PRT8(printf("after last_fn = %s cur_fn = %s\n",last_fn,cur_fn))
    }
 return fn1;
}


/*-------------------------------------------------------------------------
co_file_def_save_fn - ???
-------------------------------------------------------------------------*/

int
co_file_def_save_fn(file,regfile,dir,suffix,fn)
 struct input_file_rec *file,*regfile;
 char *dir,*suffix,**fn;
{
 char *fn3,*fn1,*fn2;
 int err;

 *fn = NULL;
 fn2 = fn3 = fn1 = NULL;
 if (((err = FileName(file,&fn3)) == NO_ERROR) &&
     ((err = FileName(regfile,&fn1)) == NO_ERROR) &&
     ((err = drop_suffix(fn3)) == NO_ERROR) &&
     ((err = drop_dirfn(fn3)) == NO_ERROR) &&
     ((err = drop_suffix(fn1)) == NO_ERROR) &&
     ((err = drop_dirfn(fn1)) == NO_ERROR) &&
     ((err = concat_strs(&fn2,3,fn3,"-",fn1)) == NO_ERROR))
     if ((*fn = co_file_unique_save_fn(dir,fn2,suffix)) == NULL)
        err = CANCEL;

 c_free(fn2);
 c_free(fn3);
 c_free(fn1);
 PRT8(printf("co_file_def_save_fn *fn = %s err = %d\n",*fn,err))
 return err;
}


/*-------------------------------------------------------------------------
co_file_next_save_fn - ???
-------------------------------------------------------------------------*/

int
co_file_next_save_fn(file)
 struct input_file_rec *file;
{
 struct all_co_file_recs *alldps;
 struct co_file_rec *dp;
 char *dir,*fn,*fn1,*fn2,*suffix;
 int err;

 if (((alldps = (struct all_co_file_recs *) enter_co_register_file(NULL,
                 GET_ALL_DP_PTR)) != NULL) &&
     ((dp = find_co_file_rec_by_file(alldps,file)) != NULL))
    {dir = (char *) xv_get(dp->save_dir,PANEL_VALUE);
     fn = (char *) xv_get(dp->save_fn,PANEL_VALUE);
     if ((err = cp_str(&fn1,fn)) != NO_ERROR) return err;
     suffix = (char *) locate_suffix(fn1);
     drop_suffix(fn1);
     PRT8(printf("co_file_next_save_fn fn1 = %s suffix = %s\n",fn1,suffix))
     if ((fn2 = co_file_unique_save_fn(dir,fn1,suffix)) == NULL)
        err = CANCEL;
       else
        {drop_dirfn(fn2);
         xv_set(dp->save_fn,PANEL_VALUE,fn2,0);}
     c_free(suffix);
     c_free(fn1);
     c_free(fn2);
     return err;
    }
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
SaveNpco2d - ???
-------------------------------------------------------------------------*/

int
SaveNpco2d(dir,fn1,suffix,frame,is_replace,fco,file)
 char *dir,*fn1,*suffix;
 Frame frame;
 int is_replace;
 struct file_register_rec *fco;
 struct input_file_rec *file;
{
 int err;
 FILE *fid;
 char *thefn;
 struct vector_3d_rec base,normal,coord_ratio;
 struct matrix_3d_rec *mat;

 PRT8(printf("SaveNpco2d...dir = %s  fn = %s suffix = %s\n",dir,fn1,suffix))
 PRT8(printf("frame = %u  is_replace = %d fco = %u\n",frame,is_replace,fco))
 PRT8(printf("fco status = %x\n",(unsigned long) fco->co.status))
 if ((err = open_output_file(dir,fn1,suffix,
        &thefn,&fid,frame,is_replace)) == NO_ERROR)
    {PRT8(printf("SaveNpco2d...thefn = %s\n",thefn))
     mat = &(fco->co.reg.xyztoplane);
     PRT8(pr_3d_matrix(mat))
     setup_vector_3d_rec(&base,mat->buf[0][3],mat->buf[1][3],mat->buf[2][3]);
     setup_vector_3d_rec(&normal,mat->buf[0][2],mat->buf[1][2],mat->buf[2][2]);
     setup_vector_3d_rec(&coord_ratio,mat->buf[0][0],mat->buf[1][1],mat->buf[2][2]);
     if ((err = WrPlaneParams(fid,&base,&normal,&base,&coord_ratio,
          &(fco->co.reg.rotation),7,1,mat)) == NO_ERROR)
        {PRT(printf("SaveNpco2d...2\n"))
         mat = &(fco->co.coreg.xyztoplane);
         PRT8(pr_3d_matrix(mat))
         setup_vector_3d_rec(&base,mat->buf[0][3],mat->buf[1][3],mat->buf[2][3]);
         setup_vector_3d_rec(&normal,mat->buf[0][2],mat->buf[1][2],mat->buf[2][2]);
         setup_vector_3d_rec(&coord_ratio,mat->buf[0][0],mat->buf[1][1],mat->buf[2][2]);
         err = WrPlaneParams(fid,&base,&normal,&base,&coord_ratio,
                &(fco->co.reg.rotation),7,1,mat);
        }
     PRT8(printf("SaveNpco2d...3\n"))
     fclose(fid);
     if (err != NO_ERROR) 
        {notice_prompt(frame,(Event *) NULL,
         NOTICE_MESSAGE_STRINGS,"File WRITE Error !!!",
                "Unable to save the .npar file !!!",0,
         NOTICE_BUTTON_YES,FAILED,0);
         delete_file(thefn);
        }
       else
        {
         ShowMsgStrs(frame,1,NO_ERROR,2,2,thefn,"SAVED !!!",NULL,0);
         co_file_next_save_fn(file);
        }            
    }
 PRT8(printf("SaveNpco2d...err = %d\n",err))
 c_free(thefn);
 return err;
}


/*-------------------------------------------------------------------------
win_save_file_npco_2d - ???
-------------------------------------------------------------------------*/

int
win_save_file_npco_2d(win,file,regfile)
 struct image_display_window *win;
 struct input_file_rec *file,*regfile;
{
 struct file_register_rec *fco;
 struct all_co_file_recs *alldps;
 struct co_file_rec *dp;
 char *fn,*fn1,*dir,*str;
 static char *fn2 = NULL;
 int err;

 PRT8(printf("win_save_file_npco_2d 1\n"))
 if ((fco = find_file_register_rec(file,regfile,NULL,0)) == NULL)
    return CANCEL;
 PRT8(printf("win_save_file_npco_2d 2\n"))
 c_free(fn2);
 fn2 = NULL;
 if (fco->co.status & CO_REG_PARAM_DEFAULT)
    str = "Saving 2D Default Co-registered paramters !!!";
   else str = "Saving 2D Co-registered paramters !!!";

 if (((alldps = (struct all_co_file_recs *) enter_co_register_file(NULL,
            GET_ALL_DP_PTR)) == NULL) ||
     ((dp = find_co_file_rec_by_file(alldps,file)) == NULL))
    {dir = file->output.dirfn;
     if ((err = co_file_def_save_fn(file,regfile,dir,NPCO_2d,&fn1)) ==
           NO_ERROR)
        {drop_dirfn(fn1);
         fn = fn1;}
    }
   else
    {fn1 = NULL;
     err = NO_ERROR;
     dir = (char *) xv_get(dp->save_dir,PANEL_VALUE);
     fn = (char *) xv_get(dp->save_fn,PANEL_VALUE);
    }
 PRT8(printf("win_save_file_npco_2d 2 err = %d\n",err))
 if (err == NO_ERROR)
    if ((err = cp_str(&fn2,fn)) == NO_ERROR)
       ShowMsgStrs(win->dwin->frame,1,NO_ERROR,2,1,
                 str,SaveNpco2d,7,dir,fn2,"",win->dwin->frame,0,fco,file);
 c_free(fn1);
 PRT8(printf("win_save_file_npco_2d err = %d\n",err))
 return err;
}


/*-------------------------------------------------------------------------
win_save_npco_2d - ???
-------------------------------------------------------------------------*/

int
win_save_npco_2d(win,fn)
 struct image_display_window *win;
 char *fn;
{
 struct img_buf_rec *img,*img1;
 struct image_display_window *win1;
 struct file_register_rec *fco;
 struct input_file_rec *file;
 int err;

 PRT8(printf("win_save_npco_2d fn = %s\n",fn))
 if ((fn != NULL) &&
     ((img = win_rec_cur_img_buf(win->owner)) != NULL) &&
     ((file = find_file(img->file,fn,NULL)) != NULL) &&
     (file != img->file) &&
     ((win1 = find_file_first_window(file,NULL,1)) != NULL) &&
     ((img1 = win_rec_cur_img_buf(win1->owner)) != NULL))
    {err = NO_ERROR;
     PRT8(printf("win_save_npco_2d 1\n"))
     if ((fco = find_file_register_rec(img->file,file,NULL,0)) == NULL)
        if ((err = SetupImgRegParams(img,img1,0x00000007,NULL,NULL)) == NO_ERROR)
           if ((fco = find_file_register_rec(img->file,file,NULL,0)) == NULL)
              err = CANCEL;
     if (err == NO_ERROR)
        err = win_save_file_npco_2d(win,img->file,file);
    }
 PRT8(printf("win_save_npco_2d err = %d\n",err))
 if (err != NO_ERROR)
    ShowMsgStrs(win->dwin->frame,0,err,2,1,
           "Sorry...Error saving the .npco-2d file !!",NULL,0);

 return err;
}


/*-------------------------------------------------------------------------
co_file_open_param - ???
-------------------------------------------------------------------------*/

int
co_file_open_param(win,wid,pa)
 struct image_display_window *win;
 char *wid;
 struct co_param_rec *pa;
{
 struct img_buf_rec *img,*img1;
 struct image_display_window *win1;
 struct win_rec *rwin;
 struct file_register_rec co1,co2;
 int err;
 struct matrix_3d_rec matrix;

 PRT8(printf("co_file_open_param...\n"))
 if ((wid == NULL) ||
     ((img = win_rec_cur_img_buf(win->owner)) == NULL) ||
     ((rwin = get_win_rec_by_wid_str(win->owner,wid)) == NULL) ||
     ((win1 = rwin->image_display) == NULL) ||
     ((img1 = win_rec_cur_img_buf(rwin)) == NULL))
    return CANCEL;
 co1.co.reg.xyztoplane = pa->p1.xyztoplane;
 setup_vector_3d_rec(&(co1.co.reg.rotation),.0,.0,.0);
 inverse_4d_matrix(&(co1.co.reg.planetoxyz),&(co1.co.reg.xyztoplane));

 co2.co.reg.xyztoplane = pa->p2.xyztoplane;
 setup_vector_3d_rec(&(co2.co.reg.rotation),.0,.0,.0);
 inverse_4d_matrix(&(co2.co.reg.planetoxyz),&(co2.co.reg.xyztoplane));

 co1.co.coreg = co2.co.reg;
 co2.co.coreg = co1.co.reg;

 mult_3d_matrix(&matrix,&(co1.co.reg.planetoxyz),&(co1.co.coreg.xyztoplane));
 matrix_3d_to_2d(&(co1.coregxyztoregxyz),&matrix);
 mult_3d_matrix(&matrix,&(co2.co.reg.planetoxyz),&(co2.co.coreg.xyztoplane));
 matrix_3d_to_2d(&(co2.coregxyztoregxyz),&matrix);

 if (((err = add_file_register_rec(img->file,img1->file,
                   &co1,NULL,2)) != NO_ERROR) ||
     ((err = add_file_register_rec(img1->file,img->file,&co2,NULL,2)) 
                  != NO_ERROR))
    return err;
 SetFileRegisterRecInUse(img->file,img1->file,2);
 SetFileRegisterRecInUse(img1->file,img->file,2);
 win_run_proc_to_all_file_wins(win,WinSetDrawLinkCoRegParams,1,0,0,0);
 win_update_co_file_menu(win,wid);
 win_run_proc_to_all_file_wins(win1,WinSetDrawLinkCoRegParams,1,0,0,0);
 win_update_co_file_menu(win1,win->owner->wid);
 return err;
}


/*-------------------------------------------------------------------------
WinAskOtherFileWid - ???
-------------------------------------------------------------------------*/

int
WinAskOtherFileWid(win,title,action_proc,arg)
 struct image_display_window *win;
 char *title;
 int (*action_proc)();
 void *arg;
{
 struct strs_rec *wids;
 int err;

 if (((err = setup_strs_rec(&wids,0,0)) == NO_ERROR) &&
     ((err = AllWids(win->owner,wids,1,2,DISPLAY_WINDOW)) == NO_ERROR))
    {if (wids->num_strs == 0)
        {ShowMsgStrs(win->dwin->frame,1,NO_ERROR,2,2,
                        "SORRY !!!",
                        "NO OTHER WIDS OF OTHER FILES",NULL,0);
         c_free(wids);
        }
       else
        {
         err = ShowMenuStrsProc(win->dwin->frame,NULL,title,
                    wids,USER_MENU_FREE_STR_PTRS_AND_PTR,
                    action_proc,win,1,arg,0);
        }
    }
   else
    {if (wids != NULL) free_strs_rec(wids);
     c_free(wids);
     win_alarm(win->dwin->frame,2);}
 return err;
}


/*-------------------------------------------------------------------------
rd_npco_2d_params - ???
-------------------------------------------------------------------------*/

int
rd_npco_2d_params(fid,win,window)
 FILE *fid;
 struct image_display_window *win;
 Xv_Window window;
{
 int err;
 static struct co_param_rec pa;

 PRT8(printf("rd_npco_2d_params...\n");)
 if ((err = RdPlaneParams(fid,&(pa.p1.base),&(pa.p1.normal),
          &(pa.p1.center),&(pa.p1.coord_ratio),
          &(pa.p1.rotate),&(pa.p1.which_format),
          &(pa.p1.adjust_to_center),&(pa.p1.xyztoplane))) == NO_ERROR)
    if (pa.p1.which_format & 4)
       err = RdPlaneParams(fid,&(pa.p2.base),&(pa.p2.normal),
          &(pa.p2.center),&(pa.p2.coord_ratio),
          &(pa.p2.rotate),&(pa.p2.which_format),
          &(pa.p2.adjust_to_center),&(pa.p2.xyztoplane));
 if (err != NO_ERROR)
    notice_prompt(window,(Event *) NULL,
         NOTICE_MESSAGE_STRINGS,"File READ Error !!!",0,
         NOTICE_BUTTON_YES,FAILED,0);
   else if (pa.p1.which_format & 4)
    {
     PRT8(printf("rd_npco_2d_params...1\n");)
     err = WinAskOtherFileWid(win,LOAD_CO_FILE,co_file_open_param,&pa);
    }
   else
    {err = CANCEL;
     notice_prompt(window,(Event *) NULL,
         NOTICE_MESSAGE_STRINGS,"Sorry...Invalid .npco file !!!",
         "It does not have 2 co-reg parameter sets...",0,
         NOTICE_BUTTON_YES,FAILED,0);
    }
 return err;
}


/*-------------------------------------------------------------------------
win_open_npco_2d - ???
-------------------------------------------------------------------------*/

int
win_open_npco_2d(win,fn,dir)
 struct image_display_window *win;
 char *fn,*dir;
{
 FILE *fid;
 char *the_fn,*fn1,*suffix;
 struct img_buf_rec *img;
 int err;

 PRT8(printf("\nwin_open_npco_2d dir = %s  fn = %s\n",dir,fn))
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL)
    return CANCEL;
 cp_str(&fn1,fn);
 suffix = (char *) locate_suffix(fn1);
 drop_suffix(fn1);
 PRT8(printf(" dir = %s  fn1 = %s  suffix = %s\n",dir,fn1,suffix))
 if ((err = open_input_file(dir,fn1,suffix,&the_fn,&fid,win->dwin->frame))
        == NO_ERROR)
    {PRT8(printf("the_fn = %s\n",the_fn))
     err = rd_npco_2d_params(fid,win,win->dwin->frame);
     fclose(fid);
    }
 c_free(the_fn);
 c_free(fn1);
 c_free(suffix);
 return err;
}


/*-------------------------------------------------------------------------
win_load_npco_2d - ???
-------------------------------------------------------------------------*/

int
win_load_npco_2d(win)
 struct image_display_window *win;
{
 struct img_buf_rec *img;
 struct all_co_file_recs *alldps;
 struct co_file_rec *dp;
 char *dir,*fn;

 PRT8(printf("win_load_npco_2d \n"))
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 if (((alldps = (struct all_co_file_recs *) enter_co_register_file(NULL,
            GET_ALL_DP_PTR)) == NULL) ||
     ((dp = find_co_file_rec(alldps,win)) == NULL))
    {dir = img->file->output.dirfn;
     fn = "*.npco-2d";
    }
   else
    {dir = (char *) xv_get(dp->open_dir,PANEL_VALUE);
     fn = (char *) xv_get(dp->open_fn,PANEL_VALUE);}
 PRT8(printf("win_load_npco_2d 1\n"))
 return win_show_param_fns(win,dir,fn,win_open_npco_2d);
}


/*-------------------------------------------------------------------------
win_reverse_npco_2d_params - ???
-------------------------------------------------------------------------*/

int
win_reverse_npco_2d_params(win,fn)
 struct image_display_window *win;
 char *fn;
{
 struct img_buf_rec *img,*img1;
 struct file_register_rec *fco,*fco1;
 struct image_display_window *win1;
 struct input_file_rec *file;
 int err;

 PRT9(printf("win_reverse_npco_2d_params wid = %s\n",fn))
 if ((fn != NULL) &&
     ((img = win_rec_cur_img_buf(win->owner)) != NULL) &&
     ((file = find_file(img->file,fn,NULL)) != NULL) &&
     (file != img->file) &&
     ((win1 = find_file_first_window(file,NULL,1)) != NULL) &&
     ((img1 = win_rec_cur_img_buf(win1->owner)) != NULL))
    {err = NO_ERROR;
     PRT9(printf("win_save_npco_2d 1\n"))
     if (((fco = find_file_register_rec(img->file,file,NULL,0)) == NULL) ||
         ((fco1 = find_file_register_rec(file,img->file,NULL,0)) == NULL))
        {PRT9(printf("win_save_npco_2d SetupImgRegParams\n"))
         if ((err = SetupImgRegParams(img,img1,0x00000007,NULL,NULL)) == NO_ERROR)
            if (((fco = find_file_register_rec(img->file,file,NULL,0)) == NULL) ||
                ((fco1 = find_file_register_rec(file,img->file,NULL,0)) == NULL))
               err = CANCEL;
        }
     if (err == NO_ERROR)
        {fco->co.reg.planetoxyz.buf[2][2] = -fco->co.reg.planetoxyz.buf[2][2];
         inverse_4d_matrix(&(fco->co.reg.xyztoplane),&(fco->co.reg.planetoxyz));
         fco1->co.coreg.planetoxyz.buf[2][2] = -fco1->co.coreg.planetoxyz.buf[2][2];
         inverse_4d_matrix(&(fco1->co.coreg.xyztoplane),&(fco1->co.coreg.planetoxyz));
        }
    }
   else err = CANCEL;
 PRT9(printf("win_reverse_npco_2d_params err = %d\n",err))
 if (err != NO_ERROR)
    ShowMsgStrs(win->dwin->frame,0,err,2,1,
           "Sorry...Error reversing the .npco-2d params !!",NULL,0);

 return err;
}


/*-------------------------------------------------------------------------
co_file_proc - ???
-------------------------------------------------------------------------*/

void
co_file_proc(item,fn,client_data,op,event)
 Panel_item item;
 char *fn;
 caddr_t client_data;
 Panel_list_op op;
 Event *event;
{
 struct all_co_file_recs *alldps;
 struct co_file_rec *dp,*dp1;
 struct file_register_rec *fco;
 unsigned long indx;
 struct input_file_rec *file;
 struct img_buf_rec *img;
 int is_del;

 dp = (struct co_file_rec *) xv_get(item,PANEL_CLIENT_DATA);
 PRT8(printf("co_file_proc...fn = %s\n",fn))
 if ((img = win_rec_cur_img_buf(dp->win->owner)) == NULL)
     return /* CANCEL */;
 is_del = 0;
 PRT8(printf("co_file_proc...1\n"))
 while (1)
    {if ((file = find_file(img->file,fn,NULL)) == NULL)
        {is_del = 1;
         PRT8(printf("co_file_proc...delete_list_item_by_str\n"))
         delete_list_item_by_str(fn,dp->wid);
         if (next_selected_item(dp->wid,0,&indx) != NO_ERROR)
            {win_alarm(dp->frame,2);
	    return /* CANCEL */;}
         fn = (char *) xv_get(dp->wid,PANEL_LIST_STRING,indx);
         PRT8(printf("co_file_proc...next fn = %s\n",fn))
        }
       else break;
    }
 PRT8(printf("co_file_proc...is_del = %d  file = %u\n",is_del,file))
 if (is_del) win_alarm(dp->frame,2);
 if (((fco = find_file_register_rec(img->file,file,NULL,0)) != NULL))
    {PRT8(printf("co_file_proc...\n"))
     if (fco->co.status & CO_REG_PARAM_REG)
        xv_set(dp->use_option,PANEL_VALUE,1,0);
       else xv_set(dp->use_option,PANEL_VALUE,0,0);
    }
   else xv_set(dp->use_option,PANEL_VALUE,0,0);

 PRT8(printf("co_file_proc...2\n"))
 if (((alldps = (struct all_co_file_recs *) enter_co_register_file(NULL,
            GET_ALL_DP_PTR)) == NULL) ||
     ((dp1 = find_co_file_rec_by_file(alldps,file)) == NULL))
     return /* CANCEL*/;
 PRT8(printf("co_file_proc...3\n"))
 if ((fco = find_file_register_rec(file,img->file,NULL,0)) == NULL)
    xv_set(dp1->use_option,PANEL_VALUE,0,0);
   else if (fco->co.status & CO_REG_PARAM_REG)
    xv_set(dp1->use_option,PANEL_VALUE,1,0);
   else xv_set(dp1->use_option,PANEL_VALUE,0,0);
 PRT8(printf("co_file_proc...4\n"))
}


/*-------------------------------------------------------------------------
co_file_fns_proc - ???
-------------------------------------------------------------------------*/

void
co_file_fns_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct co_file_rec *dp;
 struct strs_rec fns;

 dp = (struct co_file_rec *) xv_get(item,PANEL_CLIENT_DATA);
 PRT8(printf("co_file_fns_proc \n"))
 init_strs_rec(&fns);
 selected_list_items(&fns,dp->wid);
 delete_all_list_items(dp->wid);
 WinSetupFileListItem(dp->win,dp->wid,0);
 select_list_items(&fns,dp->wid);
 free_strs_rec(&fns);
}


/*-------------------------------------------------------------------------
co_file_use_option_proc - ???
-------------------------------------------------------------------------*/

void
co_file_use_option_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct co_file_rec *dp;
 struct file_register_rec *fco;
 struct img_buf_rec *img;
 struct input_file_rec *file;
 unsigned long indx;
 char *fn;
 int which1,which2;

 dp = (struct co_file_rec *) xv_get(item,PANEL_CLIENT_DATA);
 PRT8(printf("co_file_use_option_proc...dp = %u\n",dp))
 if (((img = win_rec_cur_img_buf(dp->win->owner)) == NULL) ||
     (next_selected_item(dp->wid,0,&indx) != NO_ERROR) ||
     ((fn = (char *) xv_get(dp->wid,PANEL_LIST_STRING,indx)) == NULL))
     return /* CANCEL */;
 PRT8(printf("co_file_use_option_proc...fn = %s\n",fn))
     if ((file = find_file(img->file,fn,NULL)) == NULL) {
	 co_file_proc(item,fn,NULL,0,NULL);
	 return; }
 PRT8(printf("co_file_use_option_proc...1\n"))

 if (value == 0)
    {which1 = 1;
     which2 = 2;}
   else
    {which1 = 2;
     which2 = 1;}
 PRT8(printf("co_file_use_option_proc...which1 = %d  which2 = %d\n",which1,which2))
 if ((fco = find_file_register_rec(img->file,file,NULL,which1)) != NULL)
    {PRT8(printf("co_file_use_option_proc *** | which1 = %d\n",which1))
     fco->co.status |= CO_REG_PARAM_IN_USE;
     if ((fco = find_file_register_rec(img->file,file,NULL,which2)) != NULL)
       {PRT8(printf("co_file_use_option_proc *** &~which2 = %d\n",which2))
        fco->co.status &= ~CO_REG_PARAM_IN_USE;}
    }

 if ((fco = find_file_register_rec(file,img->file,NULL,which1)) != NULL)
    {PRT8(printf("co_file_use_option_proc  === | which1 = %d\n",which1))
     fco->co.status |= CO_REG_PARAM_IN_USE;
     if ((fco = find_file_register_rec(file,img->file,NULL,which2)) != NULL)
        {PRT8(printf("co_file_use_option_proc === &~which2 = %d\n",which2))
         fco->co.status &= ~CO_REG_PARAM_IN_USE;}
    }
 PRT8(printf("co_file_use_option_proc...2\n"))
 co_file_proc(item,fn,NULL,0,NULL);
 return;

}


/*-------------------------------------------------------------------------
co_file_more_proc - ???
-------------------------------------------------------------------------*/

void
co_file_more_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct co_file_rec *dp;

 dp = (struct co_file_rec *) xv_get(item,PANEL_CLIENT_DATA);
 notice_prompt(dp->frame,(Event *) NULL,
            NOTICE_MESSAGE_STRINGS,
               "Co-register files in X & Y dimensions and",
               "the Z dimension is preassumed to be registered",
              0,
            NOTICE_BUTTON_YES," OK !",
            0);
}


/*-------------------------------------------------------------------------
co_file_def_open_npco_fn_proc - ???
-------------------------------------------------------------------------*/

void
co_file_def_open_npco_fn_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct co_file_rec *dp;

 dp = (struct co_file_rec *) xv_get(item,PANEL_CLIENT_DATA);
 xv_set(dp->open_fn,PANEL_VALUE,"*.npco-2d",0);
}


/*-------------------------------------------------------------------------
co_file_open_npco_proc - ???
-------------------------------------------------------------------------*/

int
co_file_open_npco_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct co_file_rec *dp;
 char *dir,*fn;

 dp = (struct co_file_rec *) xv_get(item,PANEL_CLIENT_DATA);
 dir = (char *) xv_get(dp->open_dir,PANEL_VALUE);
 fn = (char *) xv_get(dp->open_fn,PANEL_VALUE);
 return win_show_param_fns(dp->win,dir,fn,win_open_npco_2d);
}


/*-------------------------------------------------------------------------
co_file_def_save_npco_fn_proc - ???
-------------------------------------------------------------------------*/

void
co_file_def_save_npco_fn_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct co_file_rec *dp;
 struct img_buf_rec *img;
 struct input_file_rec *file;
 unsigned long indx;
 char *dir,*fn;

 dp = (struct co_file_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if (((img = win_rec_cur_img_buf(dp->win->owner)) == NULL) ||
     (next_selected_item(dp->wid,0,&indx) != NO_ERROR) ||
     ((fn = (char *) xv_get(dp->wid,PANEL_LIST_STRING,indx)) == NULL) ||
     ((file = find_file(img->file,fn,NULL)) == NULL))
    {if (event != NULL) win_alarm(dp->frame,2);
     return;}
 dir = (char *) xv_get(dp->save_dir,PANEL_VALUE);
 if (co_file_def_save_fn(img->file,file,dir,NPCO_2d,&fn) == NO_ERROR)
    {drop_dirfn(fn);
     xv_set(dp->save_fn,PANEL_VALUE,fn,0);
     c_free(fn);}
   else win_alarm(dp->frame,2);
}


/*-------------------------------------------------------------------------
co_file_save_npco_proc - ???
-------------------------------------------------------------------------*/

void
co_file_save_npco_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct co_file_rec *dp;
 unsigned long indx;
 struct img_buf_rec *img;
 char *fn;

 dp = (struct co_file_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if (((img = win_rec_cur_img_buf(dp->win->owner)) != NULL) &&
     (next_selected_item(dp->wid,0,&indx) == NO_ERROR) &&
     ((fn = (char *) xv_get(dp->wid,PANEL_LIST_STRING,indx)) != NULL))
    win_save_npco_2d(dp->win,fn);
}


/*-------------------------------------------------------------------------
setup_co_file_rec - ???
-------------------------------------------------------------------------*/

int
setup_co_file_rec(dp,win)
 struct co_file_rec *dp;
 struct image_display_window *win;
{
 Panel_item item;
 int row,offset;
 unsigned long indx;
 struct img_buf_rec *img;
 char *str;
 Rect *rr,*rr1;

 PRT2(printf("setup_co_file_rec...\n"))
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;

 dp->frame = xv_create(win->dwin->frame, FRAME,
              FRAME_INHERIT_COLORS,TRUE,
              WIN_CLIENT_DATA,dp,
              0);
 if (dp->frame == 0)
    {free_co_file_rec(dp);
     return ~NO_ERROR;}
 dp->win = win;
 dp->dwin = win->dwin;
 str = win_rec_win_id_title(win->owner,CO_REGISTER_FILE);
 xv_set(dp->frame,
              FRAME_SHOW_LABEL, TRUE,
              XV_LABEL,str,
              FRAME_NO_CONFIRM,TRUE,
              FRAME_DONE_PROC,dp_co_file_done_proc,
              0);
 c_free(str);
 xv_set(dp->frame,FRAME_CLOSED,FALSE,0);
 PRT2(printf("setup_co_file_rec...1\n"))

 dp->panel = xv_create(dp->frame,PANEL,
               FRAME_INHERIT_COLORS,TRUE,
               0);

 if (dp->panel == 0)
    {free_co_file_rec(dp);
     return ~NO_ERROR;}
 row = 5;
 offset = xv_rows(dp->panel,1) - xv_rows(dp->panel,0) - 10;
 item = xv_create(dp->panel,PANEL_MESSAGE,
	         PANEL_LABEL_STRING,"Use key J to define the co-registered",
                  XV_X,5,XV_Y,row,
                 0);
 rr1 = (Rect *) xv_get(item,XV_RECT);
 xv_create(dp->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"MORE...",
	PANEL_NOTIFY_PROC,co_file_more_proc,
        XV_X,rr1->r_left + rr1->r_width + 3,XV_Y,row,
        PANEL_CLIENT_DATA,dp,
	0);

 row += offset - 5;
 xv_create(dp->panel,PANEL_MESSAGE,
	         PANEL_LABEL_STRING,"coordinates for any 2 wids of different files",
                  XV_X,5,XV_Y,row,
                 0);
 row += offset - 5;
 item = xv_create(dp->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"FILEs",
	PANEL_NOTIFY_PROC,co_file_fns_proc,
        PANEL_CLIENT_DATA,dp,
        XV_X,5,XV_Y,row,
	0);
 rr1 = (Rect *) xv_get(item,XV_RECT);
 dp->use_option = xv_create(dp->panel,PANEL_CHOICE_STACK,
	PANEL_LABEL_STRING,"CO-REG PARAMS =",
        PANEL_LAYOUT,PANEL_HORIZONTAL,
        PANEL_CHOICE_STRINGS,"DEFAULT",
                           "REGISTERED FROM 2 WIDS",NULL,
	PANEL_NOTIFY_PROC,co_file_use_option_proc,
        PANEL_CLIENT_DATA,dp,
        XV_X,rr1->r_left + rr1->r_width,XV_Y,row,
	0);

 row += offset;

 dp->wid = xv_create(dp->panel,PANEL_LIST,
                 PANEL_LIST_DISPLAY_ROWS,4,
                 PANEL_LAYOUT,PANEL_VERTICAL,
	          PANEL_NOTIFY_PROC,co_file_proc,
                  PANEL_CLIENT_DATA,dp,
                  XV_X,5,XV_Y,row,
                 0);
 WinSetupFileListItem(dp->win,dp->wid,0);
 rr = (Rect *) xv_get(dp->wid,XV_RECT);

 PRT2(printf("setup_co_file_rec...1\n"))
 row = rr->r_top + rr->r_height + 2;

 item = xv_create(dp->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"LOAD .npco-2d - Ctrl J",
	PANEL_NOTIFY_PROC,co_file_open_npco_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,dp,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
 xv_create(dp->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"DEFAULT FN",
	PANEL_NOTIFY_PROC,co_file_def_open_npco_fn_proc,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
        PANEL_CLIENT_DATA,dp,
	0);

 row += offset;
 str = "LOAD DIR :";
 dp->open_dir = xv_create(dp->panel,PANEL_TEXT,
        XV_X,15,XV_Y,row,
        PANEL_LABEL_STRING, str,
        PANEL_VALUE_DISPLAY_LENGTH,20,
        PANEL_VALUE_STORED_LENGTH,100,
        PANEL_VALUE,img->file->output.dirfn,
        0);

 row += offset;
 dp->open_fn = xv_create(dp->panel,PANEL_TEXT,
        XV_X,15,XV_Y,row,
	PANEL_LABEL_STRING, "FN",
        PANEL_VALUE_DISPLAY_LENGTH,40,
        PANEL_VALUE_STORED_LENGTH,100,
        PANEL_VALUE,"",
        0);
 co_file_def_open_npco_fn_proc(dp->wid,NULL);

 PRT2(printf("setup_co_file_rec...2\n"))
 row += offset;
 item = xv_create(dp->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"SAVE .npco-2d - Ctrl Sh J",
	PANEL_NOTIFY_PROC,co_file_save_npco_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,dp,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
 xv_create(dp->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"DEFAULT FN",
	PANEL_NOTIFY_PROC,co_file_def_save_npco_fn_proc,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
        PANEL_CLIENT_DATA,dp,
	0);
 PRT2(printf("setup_co_file_rec...6\n"))
 row += offset;
 str = "SAVE DIR :";
 dp->save_dir = xv_create(dp->panel,PANEL_TEXT,
        XV_X,15,XV_Y,row,
        PANEL_LABEL_STRING, str,
        PANEL_VALUE_DISPLAY_LENGTH,20,
        PANEL_VALUE_STORED_LENGTH,100,
        PANEL_VALUE,img->file->output.dirfn,
        0);

 row += offset;
 dp->save_fn = xv_create(dp->panel,PANEL_TEXT,
        XV_X,15,XV_Y,row,
	PANEL_LABEL_STRING,"FN",
        PANEL_VALUE_DISPLAY_LENGTH,40,
        PANEL_VALUE_STORED_LENGTH,100,
        PANEL_VALUE,"",
        0);
 PRT2(printf("setup_co_file_rec...7\n"))
 co_file_def_save_npco_fn_proc(dp->wid,NULL);
 PRT2(printf("setup_co_file_rec...7\n"))

 if (next_selected_item(dp->wid,0,&indx) == NO_ERROR)
    {str = (char *) xv_get(dp->wid,PANEL_LIST_STRING,indx);
     co_file_proc(dp->wid,str,NULL,0,NULL);}

 window_fit(dp->panel);
 rr = (Rect *) xv_get(dp->panel,XV_RECT);
 xv_set(dp->panel,XV_WIDTH,rr->r_width - 20,
           XV_HEIGHT,rr->r_height - 10,0);

 PRT2(printf("setup_co_file_rec...11\n"))
 window_fit(dp->frame);
 make_2_wins_adjacent(dp->frame,win->dwin->frame,0);
 win_set_mouse_at_center(dp->frame);

 return NO_ERROR;
}


/*-------------------------------------------------------------------------
find_co_file_rec - ???
-------------------------------------------------------------------------*/

struct co_file_rec *
find_co_file_rec(alldps,win)
 struct all_co_file_recs *alldps;
 struct image_display_window *win;
{
 int lcv;
 struct img_buf_rec *img,*img1;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return NULL;
 for (lcv = 0; lcv < alldps->numdps; lcv++)
    if ((img1 = win_rec_cur_img_buf(alldps->dps[lcv]->win->owner)) != NULL)
       if (img1->file == img->file)
          return alldps->dps[lcv];
 return NULL;
}


/*-------------------------------------------------------------------------
find_co_file_rec_by_file - ???
-------------------------------------------------------------------------*/

struct co_file_rec *
find_co_file_rec_by_file(alldps,file)
 struct all_co_file_recs *alldps;
 struct input_file_rec *file;
{
 int lcv;
 struct img_buf_rec *img1;

 for (lcv = 0; lcv < alldps->numdps; lcv++)
    if ((img1 = win_rec_cur_img_buf(alldps->dps[lcv]->win->owner)) != NULL)
       if (img1->file == file)
          return alldps->dps[lcv];
 return NULL;
}


/*-------------------------------------------------------------------------
enter_co_register_file - ???
-------------------------------------------------------------------------*/

int
enter_co_register_file(win,status)
 struct image_display_window *win;
 int status;
{
 static struct all_co_file_recs alldps;
 static char first_time = 1;
 int err;
 struct co_file_rec *dp;

 err = NO_ERROR;
 if (status == GET_ALL_DP_PTR)
    {if (first_time) return 0;
       else return (unsigned long) &alldps;}     /* FIXME:  ptr as long */
  else if (status == CLEAN_UP_DP)
   {if (!first_time)
       if ((dp = find_co_file_rec(&alldps,win)) != NULL)
          free_co_file_rec(dp);
    return NO_ERROR;}
 if (first_time)
    {init_all_co_file_recs(&alldps);
     first_time = 0;}
   else
    { 
     if ((dp = find_co_file_rec(&alldps,win)) != NULL)
        {
         xv_set(dp->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,WIN_ALARM,0);
         win_set_mouse_at_center(dp->frame);
         wmgr_top(dp->frame);
         return NO_ERROR;}
    }
 get_handle_memory(&(alldps.dps),&(alldps.numdps),1,
     (size_t) sizeof(struct co_file_rec),&err);
 if (err != NO_ERROR) return err;
 dp = alldps.dps[alldps.numdps - 1];
 init_co_file_rec(dp);
 if ((err = setup_co_file_rec(dp,win)) == NO_ERROR)
    xv_set(dp->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
 return err;
}
