/****************************************************************************
color_editor.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: color_editor.c,v 1.12 1999/07/07 17:41:14 hughett Exp $ Release $Name: nria_1_0_6 $";


/* FIXME:  Not needed?
#include <time.h>
#include <sys/timeb.h>
*/

#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/xv_xrect.h>

#include "mri_format.h"
#include "nria_procedures.h"

/*
#define TRACE(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
*/
#define TRACE(x)
#define PRT(x)



#define PALLETTE_FN "paletteXYZ"
#define AFTER_FIRST_TIME 1

struct color_editor_rec
   {
    unsigned short status;
    struct image_display_window *win;
    struct display_win_rec *dwin;
    struct pallette_rec def_color;
    struct intensity_rec *intensity;
    struct color_rec *color;
    Frame frame;
    Panel panel;
    Panel_item fn,label,color_num,end_num,to_color_num,red,green,blue,pallette,
               first,last,reserve_color_start,which_win,scale_name;
    Canvas canvas;
    Scrollbar vscrollbar;
    Pixmap pm;

    long total_rects;
    long cur_rect;
    long num_row_rects;
    long num_col_rects;
    struct int_rect_rec **rr;
   };



/* Functions defined in this file */
void init_color_editor_rec(struct color_editor_rec *edit);
void free_color_editor_memory(struct color_editor_rec *edit);
void free_color_editor_rec(struct color_editor_rec *edit);
int color_editor_save_color(struct color_editor_rec *edit, char *dir,
    char *fn);
int color_editor_rd_color(struct color_editor_rec *edit, char *dir, char *fn);
void load_color_fn_proc(Panel_item item, Event *event);
char *get_color_label(struct color_editor_rec *edit);
void save_color_fn_proc(Panel_item item, Event *event);
void save_same_color_fn_proc(Panel_item item, Event *event);
void color_editor_done_proc(Frame frame);
void done_color_editor_proc(Panel_item item, Event *event);
void free_color_editor_rec_it(struct color_editor_rec *edit, int which);
void cancel_color_editor_proc(Panel_item item, Event *event);
void color_editor_set_params(struct color_editor_rec *edit);
void win_update_color_related(struct image_display_window *win);
void next_pallette_proc(Panel_item item, int value, Event *event);
void setup_pallette_labels(struct color_editor_rec *edit);
void next_color_scale_proc(Panel_item item, int value, Event *event);
void setup_color_labels(struct color_editor_rec *edit);
void new_scale_proc(Panel_item item, Event *event);
void delete_scale_proc(Panel_item item, Event *event);
void help_color_editor_proc(Panel_item item, Event *event);
void color_editor_event_proc(Xv_Window canvas, Event *event);
void color_editor_repaint_proc(Canvas canvas, Xv_Window xvwin,
    Display *dp, Window xwin, Xv_xrectlist *xrects);
void exchange_proc(Panel_item item, Event *event);
int apply_to_proc(Panel_item item, Event *event);
void color_num_proc(Panel_item item, int value, Event *event);
void to_end_proc(Panel_item item, int value, Event *event);
void set_cur_rgb(struct color_editor_rec *edit);
void set_cur_color_entry(struct color_editor_rec *edit,
    int red, int green, int blue);
void color_edit_proc(Panel_item item, int value, Event *event);
void change_color_to_proc(Panel_item item, Event *event);
void first_proc(Panel_item item, int value, Event *event);
void last_color_proc(Panel_item item, int value, Event *event);
void reserve_color_proc(Panel_item item, int value, Event *event);
int define_color_editor_rects(struct color_editor_rec *edit,
    int width, int height);
int draw_color_editor_rects(struct color_editor_rec *edit);
int setup_color_editor_rec(struct color_editor_rec *edit,
    struct image_display_window *win);
int enter_color_editor(struct image_display_window *win, int status);



/*---------------------------------------------------------------------------
init_color_editor_rec - ???
---------------------------------------------------------------------------*/

void
init_color_editor_rec(edit)
 struct color_editor_rec *edit;
{
 edit->status = 0;
 edit->win = NULL;
 edit->frame = 0;
 edit->vscrollbar = 0;
 edit->num_row_rects = 0;
 edit->num_col_rects = 0;
 edit->total_rects = 0;
 edit->intensity = NULL;
 edit->pm = 0;
 init_pallette_rec(&(edit->def_color));
}


/*---------------------------------------------------------------------------
free_color_editor_memory - ???
---------------------------------------------------------------------------*/

void
free_color_editor_memory(edit)
 struct color_editor_rec *edit;
{
 if (edit->num_col_rects > 0)
    {free_handle_memory(edit->rr,edit->num_row_rects);
     edit->num_row_rects = edit->num_col_rects = 0;
    }
}


/*---------------------------------------------------------------------------
free_color_editor_rec - ???
---------------------------------------------------------------------------*/

void
free_color_editor_rec(edit)
 struct color_editor_rec *edit;
{
 free_color_editor_memory(edit);
 free_pallette_rec(&(edit->def_color));
 if (edit->pm != 0)
    {
     win_free_pixmap(edit->canvas,edit->pm);
    }
 if (edit->frame != 0) xv_destroy_safe(edit->frame);
 init_color_editor_rec(edit); 
}


/*---------------------------------------------------------------------------
color_editor_save_color - ???
---------------------------------------------------------------------------*/

int
color_editor_save_color(edit,dir,fn)
 struct color_editor_rec *edit;
 char *dir,*fn;
{
 int err;
 char *the_fn,*reason;

 printf("dir = %s  fn = %s\n",dir,fn);
 if ((err = build_fn(&the_fn,dir,fn,"")) != NO_ERROR) 
    {ShowMsgStrs(edit->frame,0,err,0,1,
             "Error setting up the color data fn  !!!",NULL,0);
     return err;}
 if ((err = file_access(the_fn,4,NULL,&reason)) != NO_ERROR)
    {ShowMsgStrs(edit->frame,0,err,0,1,reason,NULL,0);
     c_free(the_fn);
     c_free(reason);
     return err;
    }
 if ((err = wr_pallette_rec(the_fn,&(edit->def_color))) != NO_ERROR)
    ShowMsgStrs(edit->frame,0,err,0,2,
               "An error occured when saving the color file...",
               "Operation aborted !!!",NULL,0);
 c_free(the_fn);
 return NO_ERROR;
}


/*---------------------------------------------------------------------------
color_editor_rd_color - ???
---------------------------------------------------------------------------*/

int
color_editor_rd_color(edit,dir,fn)
 struct color_editor_rec *edit;
 char *dir,*fn;
{
 char *str,*label;
 int err;
 struct pallette_rec def_color;
 char *the_fn,*reason;

 if ((err = build_fn(&the_fn,dir,fn,"")) != NO_ERROR) 
    {ShowMsgStrs(edit->frame,0,err,0,1,
             "Error setting up the color data fn  !!!",NULL,0);
     return err;}
 if ((err = file_access(the_fn,4,NULL,&reason)) != NO_ERROR)
    {ShowMsgStrs(edit->frame,0,err,0,1,reason,NULL,0);
     c_free(the_fn);
     c_free(reason);
     return err;
    }

 init_pallette_rec(&def_color);
 if ((err = rd_pallette_rec(the_fn,&def_color)) != NO_ERROR)
    {free_pallette_rec(&def_color);
     ShowMsgStrs(edit->frame,0,err,0,2,
               "An error occured when reading the color file...",
               "Operation aborted !!!",NULL,0);
     c_free(the_fn);
     return err;}
   else
    {
     TRACE(printf("color_editor_rd_color....1\n"))
     free_pallette_rec(&(edit->def_color));
     edit->def_color = def_color;
     edit->intensity = edit->def_color.pallettes[edit->def_color.cur_pallette];
     TRACE(printf("color_editor_rd_color....3\n"))

     edit->color = &(edit->intensity->color);
     label = edit->intensity->scales[edit->intensity->cur_scale].label;
     SetColorSegment(edit->frame,PALLETTE_FN,edit->color,1);
     color_editor_repaint_proc(edit->canvas,NULL,NULL,NULL,NULL);

     TRACE(printf("color_editor_rd_color....4\n"))
     concat_strs(&str,2,"CUR COLOR FN = ",edit->def_color.color_fn);
     xv_set(edit->fn,PANEL_LABEL_STRING,str,0);
     c_free(str);
     setup_pallette_labels(edit);
     setup_color_labels(edit);
     
     TRACE(printf("color_editor_rd_color....5\n"))

     color_editor_set_params(edit);
    }
 c_free(the_fn);
 return NO_ERROR;
}


/*---------------------------------------------------------------------------
load_color_fn_proc - ???
---------------------------------------------------------------------------*/

void
load_color_fn_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct color_editor_rec *edit;
 char *dir,*str;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 dir = get_prog_dir(edit->win->owner->root->prog);
 str = win_rec_win_id_title(edit->win->owner,"READ *.DAT COLOR FILE");
 ask_files(1,edit->frame,str,
       NULL,1,0,color_editor_rd_color,edit,
       NULL,NULL,0,"*.dat",0,0,5,285,NULL,NULL,1,
       dir,NULL,NULL,
       "FN",NULL,NULL,NULL
      );
 c_free(dir);
 c_free(str);
}


/*---------------------------------------------------------------------------
get_color_label - ???
---------------------------------------------------------------------------*/

char *
get_color_label(edit)
 struct color_editor_rec *edit;
{
 char *label;
 int val;

 val = (int) xv_get(edit->label,PANEL_VALUE);
 if (cp_str(&label,(char *) xv_get(edit->label,PANEL_CHOICE_STRING,val)) != NO_ERROR)
    return NULL;
 trim_str_blanks(label);
 TRACE(printf("get_color_label = %s\n",label))
 return label;
}


/*---------------------------------------------------------------------------
save_color_fn_proc - ???
---------------------------------------------------------------------------*/

void
save_color_fn_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct color_editor_rec *edit;
 char *dir,*str,*fn;
 int err;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((err = cp_str(&fn,edit->def_color.color_fn)) != NO_ERROR)
     return /*err*/;
 str = win_rec_win_id_title(edit->win->owner,"SAVE COLOR SCALE AS...");
 if ((dir = locate_dirfn(fn)) == NULL)
    cp_str(&dir,".");
 drop_dirfn(fn);
 ask_files(1,edit->frame,str,
       NULL,1,0,color_editor_save_color,edit,
       NULL,NULL,1,"*.dat",0,0,5,285,NULL,NULL,1,
       dir,NULL,NULL,
       "FN",fn,NULL,NULL
      );
 c_free(fn);
 c_free(dir);
 c_free(str);

}


/*---------------------------------------------------------------------------
save_same_color_fn_proc - ???
---------------------------------------------------------------------------*/

void
save_same_color_fn_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct color_editor_rec *edit;
 int err;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((err = wr_pallette_rec(edit->def_color.color_fn,
                &(edit->def_color))) != NO_ERROR)
    ShowMsgStrs(edit->frame,0,err,0,2,
               "An error occured when saving the color file...",
               "Operation aborted !!!",NULL,0);

}


/*---------------------------------------------------------------------------
color_editor_done_proc - ???
---------------------------------------------------------------------------*/

void
color_editor_done_proc(frame)
 Frame frame;
{
 struct color_editor_rec *edit;

 edit = (struct color_editor_rec *) xv_get(frame,WIN_CLIENT_DATA);
 free_color_editor_rec(edit);
}


/*---------------------------------------------------------------------------
done_color_editor_proc - ???
---------------------------------------------------------------------------*/

void
done_color_editor_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct color_editor_rec *edit;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 color_editor_done_proc(edit->frame);
}


/*---------------------------------------------------------------------------
free_color_editor_rec_it - ???
---------------------------------------------------------------------------*/

void
free_color_editor_rec_it(edit,which)
 struct color_editor_rec *edit;
 int which;
{
 (void) notify_set_itimer_func((Notify_client) edit,
             NOTIFY_FUNC_NULL,ITIMER_REAL,
             (struct itimerval *) 0,(struct itimerval *) 0);
 free_color_editor_rec(edit);
}


/*---------------------------------------------------------------------------
cancel_color_editor_proc - ???
---------------------------------------------------------------------------*/

void
cancel_color_editor_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct color_editor_rec *edit;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 free_color_editor_rec(edit);
/*
 (void) notify_set_itimer_func((Notify_client) edit,
             (Notify_func) free_color_editor_rec_it,ITIMER_REAL,
             &NOTIFY_POLLING_ITIMER,(struct itimerval *) 0);
*/
}


/*---------------------------------------------------------------------------
color_editor_set_params - ???
---------------------------------------------------------------------------*/

void
color_editor_set_params(edit)
 struct color_editor_rec *edit;
{
 long first,last,num_colors;

 first = edit->intensity->scales[edit->intensity->cur_scale].first;
 num_colors = edit->intensity->scales[edit->intensity->cur_scale].num_colors;
 last = num_colors + first - 1;
 TRACE(printf("color_editor_set_params cur_scale = %d first = %d num_colors = %d reserve_color_start = %d\n",edit->intensity->cur_scale,first,num_colors,edit->color->reserve_color_start);)
 xv_set(edit->first,
     PANEL_MIN_VALUE,0,
     PANEL_MAX_VALUE,(int) edit->color->total_colors - NUM_RESERVED_COLORS,
     PANEL_VALUE,first,0);

 xv_set(edit->last,
     PANEL_MIN_VALUE,0,
     PANEL_MAX_VALUE,(int) edit->color->total_colors - NUM_RESERVED_COLORS,
     PANEL_VALUE,last,0);

 xv_set(edit->reserve_color_start,
     PANEL_MIN_VALUE,0,
     PANEL_MAX_VALUE,(int) (edit->color->total_colors - 1),
     PANEL_VALUE,edit->color->reserve_color_start,0);

 xv_set(edit->color_num,
     PANEL_MIN_VALUE,0,
     PANEL_MAX_VALUE,(int) (edit->color->total_colors - 1),
     PANEL_VALUE,first,0);
 xv_set(edit->end_num,
     PANEL_MIN_VALUE,0,
     PANEL_MAX_VALUE,(int) (edit->color->total_colors - 1),
     PANEL_VALUE,first,0);
 xv_set(edit->to_color_num,
     PANEL_MIN_VALUE,0,
     PANEL_MAX_VALUE,(int) (edit->color->total_colors - 1),
     PANEL_VALUE,first,0);
 xv_set(edit->red,
     PANEL_MIN_VALUE,0,
     PANEL_MAX_VALUE,255,
     PANEL_VALUE,(int) edit->color->ori_red[first],0);
 xv_set(edit->green,
     PANEL_MIN_VALUE,0,
     PANEL_MAX_VALUE,255,
     PANEL_VALUE,(int)edit->color->ori_green[first],0);
 
 xv_set(edit->blue,
     PANEL_MIN_VALUE,0,
     PANEL_MAX_VALUE,255,
     PANEL_VALUE,(int) edit->color->ori_blue[first],0);
 TRACE(printf("color_editor_set_params end...\n");)

}


/*---------------------------------------------------------------------------
win_update_color_related - ???
---------------------------------------------------------------------------*/

void
win_update_color_related(win)
 struct image_display_window *win;
{
 all_win_rec_get_default_color_values(win->owner->root);
 win_execute_proc_to_related_wins(win,init_draw_mask_value,0,0);
 WinNewColorScreenForSliceRoiWins(win);
}


/*---------------------------------------------------------------------------
next_pallette_proc - ???
---------------------------------------------------------------------------*/

void
next_pallette_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct color_editor_rec *edit;
 char *label;
 struct intensity_rec *intensity;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((label = (char *) xv_get(item,PANEL_CHOICE_STRING,value)) != NULL)
    {printf("next_pallette_proc = %s\n",label);
     if ((intensity = find_intensity_rec(&(edit->def_color),label,1)) != NULL)
        {printf("next_pallette_proc = %s  found\n",label);
        }
    }
}


/*---------------------------------------------------------------------------
setup_pallette_labels - ???
---------------------------------------------------------------------------*/

void
setup_pallette_labels(edit)
 struct color_editor_rec *edit;
{
 long lcv;
 int x,y,num_choices;

 num_choices = (int) xv_get(edit->pallette,PANEL_NCHOICES);
 if (num_choices > edit->def_color.num_pallettes)
    {
     x = (int) xv_get(edit->pallette,XV_X);
     y = (int) xv_get(edit->pallette,XV_Y);
     xv_destroy(edit->pallette);
     edit->pallette = xv_create(edit->panel,PANEL_CHOICE_STACK,
                  PANEL_LABEL_STRING,"PALLETTE",
                  XV_X,x,XV_Y,y,
                  PANEL_NOTIFY_PROC,next_pallette_proc,
                  PANEL_CLIENT_DATA,edit,
                  0);
    }
 for (lcv = 0; lcv < edit->def_color.num_pallettes; lcv++)
    {xv_set(edit->pallette,PANEL_CHOICE_STRING,lcv,
          edit->def_color.pallettes[lcv]->label,0);}

 xv_set(edit->pallette,PANEL_VALUE,edit->def_color.cur_pallette,0);
}


/*---------------------------------------------------------------------------
next_color_scale_proc - ???
---------------------------------------------------------------------------*/

void
next_color_scale_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct color_editor_rec *edit;
 char *label;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((label = (char *) xv_get(item,PANEL_CHOICE_STRING,value)) != NULL)
    {printf("next_color_scale_proc = %s\n",label);
     if (find_scale(edit->intensity,label,1) != NULL)
        {color_editor_set_params(edit);
        }
    }
}


/*---------------------------------------------------------------------------
setup_color_labels - ???
---------------------------------------------------------------------------*/

void
setup_color_labels(edit)
 struct color_editor_rec *edit;
{
 long lcv;
 int x,y,num_choices;

 num_choices = (int) xv_get(edit->label,PANEL_NCHOICES);
 PRT(printf("setup_color_labels num_choices = %d\n",num_choices);)
 if (num_choices > edit->intensity->num_scales)
    {printf("rebuilt\n");
     x = (int) xv_get(edit->label,XV_X);
     y = (int) xv_get(edit->label,XV_Y);
     xv_destroy(edit->label);
     edit->label = xv_create(edit->panel,PANEL_CHOICE_STACK,
        PANEL_LABEL_STRING,"SCALE",
        XV_X,x,XV_Y,y,
        PANEL_NOTIFY_PROC,next_color_scale_proc,
        PANEL_CLIENT_DATA,edit,
        0);
    }
 for (lcv = 0; lcv < edit->intensity->num_scales; lcv++)
    xv_set(edit->label,PANEL_CHOICE_STRING,lcv,edit->intensity->scales[lcv].label,0); 
 xv_set(edit->label,PANEL_VALUE,edit->intensity->cur_scale,0);
}


/*---------------------------------------------------------------------------
new_scale_proc - ???
---------------------------------------------------------------------------*/

void
new_scale_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct color_editor_rec *edit;
 char *scale,*str;
 int err;
 long lcv;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 scale = (char *) xv_get(edit->scale_name,PANEL_VALUE);
 if ((err = cp_str(&str,scale)) != NO_ERROR)
    xv_set(edit->frame,WIN_ALARM,0);
   else
    {trim_str_blanks(str);
     if (get_strlen(str) == 0)
        xv_set(edit->frame,WIN_ALARM,0);
       else
        {for (lcv = 0; lcv < edit->intensity->num_scales; lcv++)
            if (str_cmp(edit->intensity->scales[lcv].label,str) == 0)
               {edit->intensity->cur_scale = lcv;
                color_editor_set_params(edit);
                break;}
         if (lcv == edit->intensity->num_scales)
            {if ((err = get_ptr(&(edit->intensity->scales),&(edit->intensity->total_scales),1,
                  (size_t) sizeof(*(edit->intensity->scales)))) == NO_ERROR)
                {edit->intensity->num_scales += 1;
                 edit->intensity->cur_scale = lcv;
                 init_color_scale_rec(&(edit->intensity->scales[lcv]));
                 edit->intensity->scales[lcv].label = str;
                 str = NULL;
                 edit->intensity->scales[lcv].first = (int) xv_get(edit->first,PANEL_VALUE);
                 edit->intensity->scales[lcv].num_colors = (int) xv_get(edit->last,PANEL_VALUE) - 
                            edit->intensity->scales[lcv].first + 1;
                 def_color_scale_rec_control(&(edit->intensity->scales[lcv]));
                 setup_color_labels(edit);
                 color_editor_set_params(edit);
                }
               else xv_set(edit->frame,WIN_ALARM,0);
            }
        }
     c_free(str);
    }
}


/*---------------------------------------------------------------------------
delete_scale_proc - ???
---------------------------------------------------------------------------*/

void
delete_scale_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct color_editor_rec *edit;
 char *label;
 unsigned long lcv;
 int val;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if (edit->intensity->num_scales == 1)
    xv_set(edit->frame,WIN_ALARM,0);
   else
    {
     val = (int) xv_get(edit->label,PANEL_VALUE);
     if ((label = (char *) xv_get(edit->label,PANEL_CHOICE_STRING,val)) != NULL)
        if (find_scale(edit->intensity,label,1) != NULL)
           {free_color_scale_rec(&(edit->intensity->scales[edit->intensity->cur_scale]));
            edit->intensity->num_scales -= 1;
            for (lcv = edit->intensity->cur_scale; lcv < edit->intensity->num_scales; lcv++)
               edit->intensity->scales[lcv] = edit->intensity->scales[lcv+1];
            if (edit->intensity->cur_scale == edit->intensity->num_scales)
               edit->intensity->cur_scale = edit->intensity->num_scales - 1;
            setup_color_labels(edit);
            color_editor_set_params(edit);
           }
    }
}


/*---------------------------------------------------------------------------
help_color_editor_proc - ???
---------------------------------------------------------------------------*/

void
help_color_editor_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct color_editor_rec *edit;
 char *fn;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 fn = (char *) get_prog_def_fn(edit->win->owner->root->prog,
              COLOR_EDITOR_HELP_FN);
 enter_display_text_file(edit->frame,1,NULL,fn);
 c_free(fn);
}


/*---------------------------------------------------------------------------
color_editor_event_proc - ???
---------------------------------------------------------------------------*/

void
color_editor_event_proc(canvas,event)
 Xv_Window canvas;
 Event *event;
{
 struct color_editor_rec *edit;
 int x,y,value,min1,max1,temp_value;
 static Panel_item item,last_item;
 unsigned int row_num,col_num,rect_num,first,temp_rect_num;
 int color_num;
 static int last_color_num;
 Event event1;

 if (event_is_up(event)) return /*NO_ERROR*/;
 edit = (struct color_editor_rec *) xv_get(canvas,WIN_CLIENT_DATA);
 color_num = (int) xv_get(edit->color_num,PANEL_VALUE);
 first  = 1;
 last_item = 0;
 do
    {
     x = event_x(event);
     y = event_y(event);
     if (!pt_in_2d_int_rect(edit->rr,edit->num_row_rects,
                       edit->num_col_rects,edit->total_rects,x,y,
                       &row_num,&col_num,&rect_num))
        break;
     if (first || (temp_rect_num != rect_num))
        {
         temp_rect_num = rect_num;
         xv_set(edit->color_num,PANEL_VALUE,rect_num,0);
         color_num_proc(edit->color_num,rect_num,NULL);
        }
     if (event_action(event) == ACTION_SELECT) item = edit->red;
       else if (event_action(event) == ACTION_ADJUST) item = edit->green;
       else if (event_action(event) == ACTION_MENU) item = edit->blue;
       else if (event_action(event) == LOC_DRAG)
        {if (last_item == 0)
            {if (event_left_is_down(event)) item = edit->red;
               else if (event_middle_is_down(event)) item = edit->green;
               else if (event_right_is_down(event)) item = edit->blue;
               else break;
            }
           else item = last_item;
        }
       else break;
     /*printf("first = %d  last_color_num = %d color_num = %d rect_num = %d\n",first,last_color_num,color_num,rect_num);*/
     if (last_item != item)
        {value = (int) xv_get(item,PANEL_VALUE);
         min1 = (int) xv_get(item,PANEL_MIN_VALUE);
         max1 = (int) xv_get(item,PANEL_MAX_VALUE);}
     if (first) first = 0;
       else if (event_shift_is_down(event)) value -= 1;
       else value += 1;
     if (value < min1) value = min1;
       else if (value > max1) value = max1;
     if ((last_item != item) || (temp_value != value))
        {xv_set(item,PANEL_VALUE,value,0);
         color_edit_proc(item,value,NULL);
         temp_value = value;
        }
     last_item = item;
     event1 = *event;
     if ((color_num != rect_num) || !(edit->status & AFTER_FIRST_TIME) || 
         (last_color_num != rect_num))
        {edit->status |= AFTER_FIRST_TIME;
         last_color_num = rect_num;
         while (xv_input_readevent(canvas,event,FALSE,FALSE,NULL) != 0)
            {printf("again...\n");
            }
         break;}
     if (xv_input_readevent(canvas,event,FALSE,FALSE,NULL) == 0)
        {printf("waiting...\n");
         *event = event1;}
       else if (event_is_up(event)) break;
       else if (event_action(event) != LOC_DRAG)
        {
         *event = event1;}
    }
 while (1);
}


/*---------------------------------------------------------------------------
color_editor_repaint_proc - ???
---------------------------------------------------------------------------*/

void
color_editor_repaint_proc(canvas,xvwin,dp,xwin,xrects)
 Canvas canvas;
 Xv_Window xvwin;
 Display *dp;
 Window xwin;
 Xv_xrectlist *xrects;
{
 struct color_editor_rec *edit;
 int x,y,height,width;

 edit = (struct color_editor_rec *) xv_get(canvas,WIN_CLIENT_DATA);

 if (xrects == NULL)
    {
     height = (int) xv_get(edit->canvas,CANVAS_HEIGHT);
     width = (int) xv_get(edit->canvas,CANVAS_WIDTH);
     win_copy_from_and_to_pixmap(edit->canvas,edit->pm,0,0,GXcopy,0,0,
        width,height,1);
    }
   else
    {
     for (xrects->count--; xrects->count >= 0; xrects->count--)
        {
         x = xrects->rect_array[xrects->count].x;
         y = xrects->rect_array[xrects->count].y;
         width = xrects->rect_array[xrects->count].width;
         height = xrects->rect_array[xrects->count].height;
         win_copy_from_and_to_pixmap(edit->canvas,edit->pm,x,y,GXcopy,x,y,
                                     width,height,1);
        }
    }
}


/*---------------------------------------------------------------------------
exchange_proc - ???
---------------------------------------------------------------------------*/

void
exchange_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct color_editor_rec *edit;
 int temp,lcv,first,last,opr;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 opr = (int) xv_get(item,XV_KEY_DATA,1);
 first = edit->cur_rect;
 last = (int) xv_get(edit->end_num,PANEL_VALUE);
 printf("opr = %d first = %d last = %d\n",opr,first,last);
 if (opr == 1)
    for (lcv = first; lcv <= last; lcv++)
       {temp = edit->color->ori_red[lcv];
        edit->color->ori_red[lcv] = edit->color->ori_green[lcv];
        edit->color->ori_green[lcv] = temp;
       }
   else if (opr == 2)
    for (lcv = first; lcv <= last; lcv++)
       {temp = edit->color->ori_red[lcv];
        edit->color->ori_red[lcv] = edit->color->ori_blue[lcv];
        edit->color->ori_blue[lcv] = temp;
       }
   else
    for (lcv = first; lcv <= last; lcv++)
       {temp = edit->color->ori_green[lcv];
        edit->color->ori_green[lcv] = edit->color->ori_blue[lcv];
        edit->color->ori_blue[lcv] = temp;
       }
 ChangeColorSegment(edit->frame,PALLETTE_FN,edit->color,1);
 set_cur_rgb(edit);
}



/*---------------------------------------------------------------------------
apply_to_proc - ???
---------------------------------------------------------------------------*/

int
apply_to_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct color_editor_rec *edit;
 int err,is_to_all_window;
 Menu menu,parent,menu1;
 Menu_item item1;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 is_to_all_window = (int) xv_get(edit->which_win,PANEL_VALUE);
 TRACE(printf("apply_to_proc...is_to_all_window = %d\n",is_to_all_window);)
 if ((err = win_set_color_by_new_intensity_rec(edit->win,edit->intensity,1,
              is_to_all_window)) == NO_ERROR)
    {
     if (((menu = (Menu) setup_color_pallette_menu(2,NULL,
                        edit->frame)) != 0) &&
         ((parent = (Menu) xv_get(menu,MENU_PARENT)) != 0) &&
         ((item1 = (Menu) xv_get(parent,MENU_PARENT)) != 0))
        {TRACE(PRT("apply_to_proc...ori menu = %u\n",menu))
         xv_set(parent,MENU_PULLRIGHT,NULL,0);
         menu1 = (Menu) setup_color_pallette_menu(3,NULL,edit->frame);
         PRT(printf("...after destroy menu1 = %u\n",menu1))
         if ((menu = (Menu) setup_color_pallette_menu(1,
                   &(edit->win->owner->root->def_color),
                   edit->frame)) != 0)
            {PRT(printf("apply_to_proc...menu = %u item1 = %u parent = %u\n",menu,
                     item1,parent))
             xv_set(parent,MENU_PULLRIGHT,menu,0);}
        }
    }
 return NO_ERROR;
}


/*---------------------------------------------------------------------------
color_num_proc - ???
---------------------------------------------------------------------------*/

void
color_num_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct color_editor_rec *edit;
 int end;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 end = (int) xv_get(edit->end_num,PANEL_VALUE);
 if (value > end) xv_set(edit->end_num,PANEL_VALUE,value,0);
 edit->cur_rect = value;
 xv_set(edit->red,PANEL_VALUE,(int) edit->color->ori_red[value],0);
 xv_set(edit->green,PANEL_VALUE,(int) edit->color->ori_green[value],0);
 xv_set(edit->blue,PANEL_VALUE,(int) edit->color->ori_blue[value],0);
}


/*---------------------------------------------------------------------------
to_end_proc - ???
---------------------------------------------------------------------------*/

void
to_end_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct color_editor_rec *edit;
 int color_num;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 color_num = (int) xv_get(edit->color_num,PANEL_VALUE);
 if (value < color_num) xv_set(item,PANEL_VALUE,color_num,0);
}


/*---------------------------------------------------------------------------
set_cur_rgb - ???
---------------------------------------------------------------------------*/

void
set_cur_rgb(edit)
 struct color_editor_rec *edit;
{
 xv_set(edit->red,PANEL_VALUE,(int) edit->color->ori_red[edit->cur_rect],0);
 xv_set(edit->green,PANEL_VALUE,(int) edit->color->ori_green[edit->cur_rect],0);
 xv_set(edit->blue,PANEL_VALUE,(int) edit->color->ori_blue[edit->cur_rect],0);
}


/*---------------------------------------------------------------------------
set_cur_color_entry - ???
---------------------------------------------------------------------------*/

void
set_cur_color_entry(edit,red,green,blue)
 struct color_editor_rec *edit;
 int red,green,blue;
{

 TRACE(printf("set_cur_color_entry...edit->cur_rect = %d\n",edit->cur_rect))
 edit->color->ori_red[edit->cur_rect] = red;
 edit->color->ori_green[edit->cur_rect] = green;
 edit->color->ori_blue[edit->cur_rect] = blue;
 ChangeColorSegment(edit->frame,PALLETTE_FN,edit->color,1);
 TRACE(printf("set_cur_color_entry...end\n"))
}


/*---------------------------------------------------------------------------
color_edit_proc - ???
---------------------------------------------------------------------------*/

void
color_edit_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct color_editor_rec *edit;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 set_cur_color_entry(edit,(int) xv_get(edit->red,PANEL_VALUE),
                     (int) xv_get(edit->green,PANEL_VALUE),
                     (int) xv_get(edit->blue,PANEL_VALUE));
}


/*---------------------------------------------------------------------------
change_color_to_proc - ???
---------------------------------------------------------------------------*/

void
change_color_to_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct color_editor_rec *edit;
 int opr,to_color_num,last;
 int next,lcv,cnt,temp;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 opr = (int) xv_get(item,XV_KEY_DATA,1);
 to_color_num = (int) xv_get(edit->to_color_num,PANEL_VALUE);
 last = (int) xv_get(edit->end_num,PANEL_VALUE);
 if (edit->cur_rect == to_color_num) return /*NO_ERROR*/;
 if (opr == 1)
    {
     for (next = 0,lcv = edit->cur_rect; lcv <= last; lcv++,next++)
        {edit->color->red[next] = edit->color->ori_red[lcv];
         edit->color->green[next] = edit->color->ori_green[lcv];
         edit->color->blue[next] = edit->color->ori_blue[lcv];}
     if (edit->cur_rect > to_color_num)
        {
         for (next = last,lcv = edit->cur_rect - 1; lcv >= to_color_num; 
                   lcv--,next--)
            {edit->color->ori_red[next] = edit->color->ori_red[lcv];
             edit->color->ori_green[next] = edit->color->ori_green[lcv];
             edit->color->ori_blue[next] = edit->color->ori_blue[lcv];}
        }
       else
        {cnt = last - edit->cur_rect + 1 + to_color_num;
         for (next = edit->cur_rect,lcv = last + 1; lcv < cnt;lcv++,next++)
            {edit->color->ori_red[next] = edit->color->ori_red[lcv];
             edit->color->ori_green[next] = edit->color->ori_green[lcv];
             edit->color->ori_blue[next] = edit->color->ori_blue[lcv];}
        }
     cnt = last - edit->cur_rect + 1;
     for (next = 0,lcv = to_color_num; next < cnt; lcv++,next++)
        {edit->color->ori_red[lcv] = edit->color->red[next];
         edit->color->ori_green[lcv] = edit->color->green[next];
         edit->color->ori_blue[lcv] = edit->color->blue[next];}
    }
   else if (opr == 2)
    {
     for (next = to_color_num,lcv = edit->cur_rect; lcv <= last; lcv++,next++)
        {temp = edit->color->ori_red[next];
         edit->color->ori_red[next] = edit->color->ori_red[lcv];
         edit->color->ori_red[lcv] = temp;

         temp = edit->color->ori_green[next];
         edit->color->ori_green[next] = edit->color->ori_green[lcv];
         edit->color->ori_green[lcv] = temp;

         temp = edit->color->ori_blue[next];
         edit->color->ori_blue[next] = edit->color->ori_blue[lcv];
         edit->color->ori_blue[lcv] = temp;
        }
    }
   else
    {
     for (next = to_color_num,lcv = edit->cur_rect; lcv <= last; lcv++,next++)
        {edit->color->ori_red[lcv] = edit->color->ori_red[next];
         edit->color->ori_green[lcv] = edit->color->ori_green[next];
         edit->color->ori_blue[lcv] = edit->color->ori_blue[next];}
    }
 ChangeColorSegment(edit->frame,PALLETTE_FN,edit->color,1);
 set_cur_rgb(edit);
}


/*---------------------------------------------------------------------------
first_proc - ???
---------------------------------------------------------------------------*/

void
first_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct color_editor_rec *edit;
 long first,last;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 TRACE(printf("first_proc edit = %u \n",edit);)
 TRACE(printf("first_proc edit->color = %u\n",edit->color);)
 first = edit->intensity->scales[edit->intensity->cur_scale].first;
 last = (int) xv_get(edit->last,PANEL_VALUE);
 if ((value + 16) > edit->color->total_colors)
    {value = edit->color->total_colors - 16;
     xv_set(item,PANEL_VALUE,value,0);
    }
 if ((value + 15) > last)
    xv_set(edit->last,PANEL_VALUE,value + 15,0);

 edit->intensity->scales[edit->intensity->cur_scale].first = value;
}


/*---------------------------------------------------------------------------
last_color_proc - ???
---------------------------------------------------------------------------*/

void
last_color_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct color_editor_rec *edit;
 long first;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 first = (int) xv_get(edit->first,PANEL_VALUE);
 printf("first = %ld value = %d\n",first,value);
 if ((first + 16) > value)
    {value = first + 16;
     xv_set(item,PANEL_VALUE,value,0);
    }
 edit->intensity->scales[edit->intensity->cur_scale].num_colors = value - first + 1;
}


/*---------------------------------------------------------------------------
reserve_color_proc - ???
---------------------------------------------------------------------------*/

void
reserve_color_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct color_editor_rec *edit;

 edit = (struct color_editor_rec *) xv_get(item,PANEL_CLIENT_DATA);
 edit->color->reserve_color_start = value;
}


/*---------------------------------------------------------------------------
define_color_editor_rects - ???
---------------------------------------------------------------------------*/

int
define_color_editor_rects(edit,width,height)
 struct color_editor_rec *edit;
 int width,height;
{
 int row_height,col_width,len;
 char str[10];
 unsigned int total;
 int err;

 sprintf(str,"%ld",edit->color->total_colors);
 free_color_editor_memory(edit);
 len = strlen(str);
 col_width = (int) xv_get(edit->canvas,WIN_COLUMN_WIDTH) +
             (int) xv_get(edit->canvas,WIN_COLUMN_GAP);
 col_width *= len;
 if (col_width < 20) col_width = 20;

#define COL_GAP 15
 edit->num_col_rects = (width - 10) / (col_width + COL_GAP);
 col_width = (col_width + COL_GAP) * edit->num_col_rects;
#undef COL_GAP
 if (edit->num_col_rects > edit->color->total_colors)
    {edit->num_col_rects = edit->color->total_colors;
     edit->num_row_rects = 1;}
   else
    {edit->num_row_rects = edit->color->total_colors / edit->num_col_rects;
     if (edit->num_col_rects % edit->color->total_colors)
        edit->num_row_rects += 1;
    }

 row_height = (int) xv_get(edit->canvas,WIN_ROW_HEIGHT) +
              (int) xv_get(edit->canvas,WIN_ROW_GAP) + 20;
 row_height = row_height * edit->num_row_rects;

 if (edit->vscrollbar != 0) xv_destroy_safe(edit->vscrollbar);
 xv_set(edit->canvas,CANVAS_HEIGHT,row_height + 10,
        CANVAS_WIDTH,col_width + 10,XV_WIDTH,col_width + 25,
        CANVAS_AUTO_SHRINK,FALSE,CANVAS_AUTO_EXPAND,FALSE,0);
 if (row_height + 10 > height)
    {
     xv_set(edit->canvas,XV_HEIGHT,height + 10,0);
     xv_set(edit->canvas,
	WIN_VERTICAL_SCROLLBAR,xv_create(edit->canvas,SCROLLBAR,0),0);
     edit->vscrollbar = xv_get(edit->canvas,WIN_VERTICAL_SCROLLBAR);
     }
   else xv_set(edit->canvas,XV_HEIGHT,row_height + 10,0);
 TRACE(printf("canvas_height = %d canvas_width = %d xv_height = %d xv_width = %d\n",
          xv_get(edit->canvas,CANVAS_HEIGHT),
          xv_get(edit->canvas,CANVAS_WIDTH),
          xv_get(edit->canvas,XV_HEIGHT),
          xv_get(edit->canvas,XV_WIDTH)))
 total = 0;
 edit->cur_rect = edit->intensity->scales[edit->intensity->cur_scale].first;
 edit->total_rects = edit->color->total_colors;
 err = build_2d_int_rects(&(edit->rr),edit->num_row_rects,
             edit->num_col_rects,edit->total_rects,5,5,
             col_width,row_height,3,3);
 if (err != NO_ERROR) return err;
 if ((edit->vscrollbar != 0) && (edit->num_row_rects > 0))
    xv_set(edit->vscrollbar,SCROLLBAR_PIXELS_PER_UNIT,edit->rr[0][0].height,0);
 return NO_ERROR;           

}


/*---------------------------------------------------------------------------
draw_color_editor_rects - ???
---------------------------------------------------------------------------*/

int
draw_color_editor_rects(edit)
 struct color_editor_rec *edit;
{
 int row,col;
 int forecolor,bkgcolor,the_textcolor,textcolor;
 char str[10];
 int err;
 unsigned int cnt_rect;

 if ((err = win_setup_pixmap(edit->canvas,&(edit->pm))) != NO_ERROR)
    {
     ShowMsgStrs(edit->frame,0,err,0,3,
               "An error occured when setting up the pixmap",
               "for the color scale window...",
               "Color editor will be aborted !!!",NULL,0);
     free_color_editor_rec(edit);
     return err;}
 win_clear(edit->canvas,edit->pm,0);

 textcolor = forecolor = xv_get(edit->canvas,WIN_FOREGROUND_COLOR);
 for (cnt_rect = 0,bkgcolor = 0,row = 0; row < edit->num_row_rects; row++)
    for (col = 0; col < edit->num_col_rects; col++,bkgcolor++,cnt_rect++)
       {
        if (cnt_rect == edit->total_rects) break;
        sprintf(str,"%d",bkgcolor);
        if (bkgcolor == textcolor) 
           {if (bkgcolor != 0) the_textcolor = 0;
              else the_textcolor =  bkgcolor - 1;}
          else the_textcolor = textcolor;
        win_center_text_in_int_rect(edit->canvas,edit->pm,0,
          &(edit->rr[row][col]),forecolor,bkgcolor,
          NULL,str,the_textcolor,GXcopy,1,0,0);
       }
 return NO_ERROR;
}


/*---------------------------------------------------------------------------
setup_color_editor_rec - ???
---------------------------------------------------------------------------*/

int
setup_color_editor_rec(edit,win)
 struct color_editor_rec *edit;
 struct image_display_window *win;
{
 Rect *rr,rr1;
 Panel_item item;
 int err;
 char *str,*label;
 Cms cms;
 int row,offset;

 TRACE(printf("setup_color_editor_rec...win->owner = %u\n",win->owner))
 TRACE(printf("setup_color_editor_rec...win->owner->root = %u\n",win->owner->root))
 if ((err = copy_pallette_rec(&(edit->def_color),&(win->owner->root->def_color),
             (unsigned short) RGB_FREE_BUFFERS)) != NO_ERROR)
    return err;

 TRACE(printf("setup_color_editor_rec...1..cur_pallette = %d\n",edit->def_color.cur_pallette))
 edit->intensity = edit->def_color.pallettes[edit->def_color.cur_pallette];
 edit->color = &(edit->intensity->color);
 unmap_all_scale_rgb(edit->color);
 /*pr_color_rec(edit->color);*/
 edit->frame = xv_create(win->ancestor,FRAME,
                         /*WIN_DYNAMIC_VISUAL,TRUE,*/0);

 if (edit->frame == 0)
    {free_color_editor_rec(edit);
     return CANCEL;}
 edit->win = win;
 edit->dwin = win->dwin;
 cms = xv_get(edit->frame,WIN_CMS);
 TRACE(printf("setup_color_editor_rec...cms = %u\n",cms))
 SetColorSegment(edit->frame,PALLETTE_FN,edit->color,1);
 cms = xv_get(edit->frame,WIN_CMS);
 str = (char *) win_rec_win_id_title(win->owner,COLOR_EDITOR);
 TRACE(printf("setup_color_editor_rec...edit->frame = %u cms = %u\n",edit->frame,cms))
 xv_set(edit->frame,
              WIN_BACKGROUND_COLOR,1 + edit->color->num_system_colors,
              WIN_FOREGROUND_COLOR,2 + edit->color->num_system_colors,
              FRAME_SHOW_LABEL, TRUE,
              XV_LABEL,str,
              FRAME_NO_CONFIRM,TRUE,
              FRAME_DONE_PROC,color_editor_done_proc,
              WIN_CLIENT_DATA,edit,
              0);
 c_free(str);
 TRACE(printf("setup_color_editor_rec...3..1\n"))

 edit->panel = xv_create(edit->frame,PANEL,WIN_CMS,cms,
                         FRAME_INHERIT_COLORS,TRUE,
                         0);
 edit->canvas = xv_create(edit->frame,CANVAS,WIN_CMS,cms,
              WIN_BACKGROUND_COLOR,1 + edit->color->num_system_colors,
              WIN_FOREGROUND_COLOR,2 + edit->color->num_system_colors,
                         /*WIN_DYNAMIC_VISUAL,TRUE,*/
                         FRAME_INHERIT_COLORS,TRUE,CANVAS_RETAINED,FALSE,0);
 if ((edit->panel == 0) || (edit->canvas == 0))
    {free_color_editor_rec(edit);
     return CANCEL;}

 row = 5;
 offset = xv_rows(edit->panel,1) - xv_rows(edit->panel,0) - 10;
 concat_strs(&str,2,"CUR COLOR FN = ",edit->def_color.color_fn);
 edit->fn = xv_create(edit->panel,PANEL_MESSAGE,
              PANEL_LABEL_STRING,str,XV_Y,row,
              XV_X,5,XV_Y,row,
              0);
 c_free(str);
 row += offset - 7;
 str = "LOAD COLOR FN";
 item = xv_create(edit->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING, str,
                   PANEL_NOTIFY_PROC,load_color_fn_proc,
                   PANEL_CLIENT_DATA,edit,
                   XV_X,5,XV_Y,row,
                   0);
 rr = (Rect *) xv_get(item,XV_RECT);
 str = "SAVE";
 item = xv_create(edit->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING, str, 	           
                   PANEL_NOTIFY_PROC,save_color_fn_proc,
                   PANEL_CLIENT_DATA,edit,
                   XV_X,rr->r_left + rr->r_width,XV_Y,row,
                   0);
 str = "SAVE SAME";
 rr = (Rect *) xv_get(item,XV_RECT);
 xv_create(edit->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING, str, 	           
                   PANEL_NOTIFY_PROC,save_same_color_fn_proc,
                   PANEL_CLIENT_DATA,edit,
                   XV_X,rr->r_left + rr->r_width,XV_Y,row,
                   0);

 str = "NEW SCALE";
 item = xv_create(edit->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING, str, 	           
                   PANEL_NOTIFY_PROC,new_scale_proc,
                   PANEL_CLIENT_DATA,edit,
                   XV_Y,row,
                   0);
 rr = (Rect *) xv_get(item,XV_RECT);
 edit->scale_name = xv_create(edit->panel,PANEL_TEXT,
                   PANEL_CLIENT_DATA,edit,
                   PANEL_VALUE_DISPLAY_LENGTH,20,
                   PANEL_VALUE_STORED_LENGTH,30,
                   PANEL_VALUE,"",
                   XV_X,rr->r_left + rr->r_width,XV_Y,row,
                   0);
 TRACE(printf("setup_color_editor_rec...4\n"))

 row += offset - 2;
 str = "PALLETTE";
 edit->pallette = xv_create(edit->panel,PANEL_CHOICE_STACK,
                  PANEL_LABEL_STRING,str,
                  XV_X,5,XV_Y,row,
                  PANEL_NOTIFY_PROC,next_pallette_proc,
                  PANEL_CLIENT_DATA,edit,
                  0);
 setup_pallette_labels(edit);
 rr = (Rect *) xv_get(edit->pallette,XV_RECT);
 str = "DELETE SCALE";
 item = xv_create(edit->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING, str, 	           
                   PANEL_NOTIFY_PROC,delete_scale_proc,
                   PANEL_CLIENT_DATA,edit,
                   XV_X,rr->r_left + rr->r_width + 20,XV_Y,row,
                   0);

 str = "SCALE";
 rr = (Rect *) xv_get(item,XV_RECT);
 label = edit->intensity->scales[edit->intensity->cur_scale].label;
 edit->label = xv_create(edit->panel,PANEL_CHOICE_STACK,
        PANEL_LABEL_STRING,str,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
        PANEL_NOTIFY_PROC,next_color_scale_proc,
        PANEL_CLIENT_DATA,edit,
        0);

 setup_color_labels(edit);
 row += offset;
 edit->color_num = xv_create(edit->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"FROM COLOR# :",
	PANEL_SLIDER_WIDTH,50,
	PANEL_NOTIFY_PROC,color_num_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,edit,
	0);
 edit->end_num = xv_create(edit->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"TO END#:",
	PANEL_SLIDER_WIDTH,50,
	PANEL_NOTIFY_PROC,to_end_proc,
        XV_Y,row,
        PANEL_CLIENT_DATA,edit,
	0);

 row += offset - 4;
 str = "MOVE TO";
 item = xv_create(edit->panel,PANEL_BUTTON,
        XV_X,25,XV_Y,row,
 	PANEL_LABEL_STRING, str,
	PANEL_NOTIFY_PROC,change_color_to_proc,
        XV_KEY_DATA,1,1,PANEL_CLIENT_DATA,edit,
	0);
 str = "SWAP";
 rr = (Rect *) xv_get(item,XV_RECT);
 item = xv_create(edit->panel,PANEL_BUTTON,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
 	PANEL_LABEL_STRING, str,
	PANEL_NOTIFY_PROC,change_color_to_proc,
        XV_KEY_DATA,1,2,PANEL_CLIENT_DATA,edit,
	0);
 str = "SAME AS";
 rr = (Rect *) xv_get(item,XV_RECT);
 item = xv_create(edit->panel,PANEL_BUTTON,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
 	PANEL_LABEL_STRING, str,
	PANEL_NOTIFY_PROC,change_color_to_proc,
        XV_KEY_DATA,1,3,PANEL_CLIENT_DATA,edit,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
 edit->to_color_num = xv_create(edit->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"TO START# :",
	PANEL_SLIDER_WIDTH,50,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
	0);
 row += offset;
 edit->red = xv_create(edit->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"RED     :",
	PANEL_SLIDER_WIDTH,80,
	PANEL_NOTIFY_PROC,color_edit_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,edit,
	0);
 str = "RED<>GREEN";
 rr = (Rect *) xv_get(edit->red,XV_RECT);
 item = xv_create(edit->panel,PANEL_BUTTON,
        PANEL_LABEL_STRING, str,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
        XV_KEY_DATA,1,1,
	PANEL_NOTIFY_PROC,exchange_proc,
        PANEL_CLIENT_DATA,edit,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
 str = "RED<>BLUE";
 item = xv_create(edit->panel,PANEL_BUTTON,
        PANEL_LABEL_STRING, str,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
        XV_KEY_DATA,1,2,
	PANEL_NOTIFY_PROC,exchange_proc,
        PANEL_CLIENT_DATA,edit,
	0);
 rr = (Rect *) xv_get(item,XV_RECT);
 str = "GREEN<>BLUE";
 item = xv_create(edit->panel,PANEL_BUTTON,
        PANEL_LABEL_STRING, str,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,
        XV_KEY_DATA,1,3,
	PANEL_NOTIFY_PROC,exchange_proc,
        PANEL_CLIENT_DATA,edit,
	0);

 row += offset;
 edit->green = xv_create(edit->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"GREEN   :",
	PANEL_SLIDER_WIDTH,80,
	PANEL_NOTIFY_PROC,color_edit_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,edit,
	0);
 str = "APPLY TO";
 item = xv_create(edit->panel,PANEL_BUTTON,
        PANEL_LABEL_STRING, str,        
	PANEL_NOTIFY_PROC,apply_to_proc,
        PANEL_CLIENT_DATA,edit,XV_Y,row,
	0);

 rr = (Rect *) xv_get(item,XV_RECT);
 edit->which_win = xv_create(edit->panel,PANEL_CHOICE_STACK,
        PANEL_CHOICE_STRINGS,"WINDOWS OF SAME OLD SCALE","ALL WINDOWS",NULL,
        XV_X,rr->r_left + rr->r_width,XV_Y,row,0);

 row += offset;
 edit->blue = xv_create(edit->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"BLUE    :",
	PANEL_SLIDER_WIDTH,80,
	PANEL_NOTIFY_PROC,color_edit_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,edit,
	0);
 str = "HELP";
 xv_create(edit->panel,PANEL_BUTTON,
        PANEL_LABEL_STRING, str,        
	PANEL_NOTIFY_PROC,help_color_editor_proc,
        PANEL_CLIENT_DATA,edit,XV_Y,row,
	0);

 str = "DONE";
 xv_create(edit->panel,PANEL_BUTTON,
        PANEL_LABEL_STRING, str,                
	PANEL_NOTIFY_PROC,done_color_editor_proc,
        PANEL_CLIENT_DATA,edit,XV_Y,row,
	0);
 str = "CANCEL";
 xv_create(edit->panel,PANEL_BUTTON,
        PANEL_LABEL_STRING, str,                
	PANEL_NOTIFY_PROC,cancel_color_editor_proc,
        PANEL_CLIENT_DATA,edit,XV_Y,row,
	0);

 row += offset + 3;
 edit->first = xv_create(edit->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"1st color#   :",
	PANEL_SLIDER_WIDTH,60,
	PANEL_NOTIFY_PROC,first_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,edit,
	0);

 edit->last = xv_create(edit->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"Last :",
	PANEL_SLIDER_WIDTH,60,
	PANEL_NOTIFY_PROC,last_color_proc,
        XV_Y,row,
        PANEL_CLIENT_DATA,edit,
	0);

 row += offset;
 edit->reserve_color_start = xv_create(edit->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"Reserve Color Start",
	PANEL_SLIDER_WIDTH,100,
	PANEL_NOTIFY_PROC,reserve_color_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,edit,
	0);
 color_editor_set_params(edit);

#define SET_CANVAS_HEIGHT 500
#define SET_CANVAS_WIDTH 610

 xv_set(edit->canvas,
        WIN_CLIENT_DATA,edit,
        CANVAS_FIXED_IMAGE,TRUE,
        CANVAS_AUTO_SHRINK,FALSE,
        CANVAS_AUTO_EXPAND,FALSE,
        CANVAS_X_PAINT_WINDOW,TRUE,
        CANVAS_REPAINT_PROC,color_editor_repaint_proc,
	0);
 xv_set(canvas_paint_window(edit->canvas),
	WIN_EVENT_PROC,color_editor_event_proc,
	WIN_CONSUME_EVENTS,
		MS_LEFT,MS_RIGHT,MS_MIDDLE,LOC_DRAG,
		0,
        WIN_CLIENT_DATA,edit,
	0);


 if (define_color_editor_rects(edit,SET_CANVAS_WIDTH,SET_CANVAS_HEIGHT) != 
          NO_ERROR)
    {free_color_editor_rec(edit);
     return CANCEL;}
 window_fit(edit->panel);
 rr = (Rect *) xv_get(edit->panel,XV_RECT);
 xv_set(edit->panel,XV_HEIGHT,rr->r_height - 15,0);
 rr = (Rect *) xv_get(edit->canvas,XV_RECT);
 xv_set(edit->panel,XV_WIDTH,rr->r_width,0);
 xv_set(edit->canvas,WIN_BELOW,edit->panel,0);

 window_fit(edit->frame);
 /* FIXME:  do not remove this redundant call or else the window will not be
     fit correctly since it is followed by center_frame;  this may well be
     a bug in xview.... */
 frame_get_rect(edit->frame,&rr1);

 center_frame(edit->frame,0);
 win_set_mouse_at_center(edit->frame);
 return NO_ERROR;
}


/*---------------------------------------------------------------------------
enter_color_editor - ???
---------------------------------------------------------------------------*/

#define CLEAN_UP_COLOR_EDITOR 0
#define SETUP_COLOR_EDITOR 1

int
enter_color_editor(win,status)
 struct image_display_window *win;
 int status;
{
 static struct color_editor_rec edit;
 static char first_time = 1;
 int err;

 if (status == CLEAN_UP_COLOR_EDITOR)
    {if ((first_time != 1) && (edit.win == win) &&
         (edit.dwin == win->dwin))
        free_color_editor_rec(&edit);
     return NO_ERROR;}

 if (first_time)
    {init_color_editor_rec(&edit);
     first_time = 0;}
   else if (edit.frame != 0)
    {
     if ((edit.win != win) || (edit.dwin != win->dwin))
        free_color_editor_rec(&edit);
       else if (!xv_get(edit.frame,FRAME_CLOSED) &&
                xv_get(edit.frame,XV_SHOW))
           {color_editor_done_proc(edit.frame);
            return NO_ERROR;}
    }

 if (edit.frame == 0) 
    {if ((err = setup_color_editor_rec(&edit,win)) != NO_ERROR)
        return err;
       else
        {draw_color_editor_rects(&edit);}
    }
 xv_set(edit.frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
 return NO_ERROR;

}


