/****************************************************************************
color_screen_rec.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: color_screen_rec.c,v 1.6 1999/07/07 17:41:14 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <X11/Xlib.h>
#include <X11/Xutil.h>         /* For XDestroyImage */

#include <xview/cms.h>

#include "mri_format.h"
#include "debug.h"
#include "nria_procedures.h"



/* Functions defined in this file */
void init_color_data_rec(struct color_data_rec *color_data);
void free_color_data_rec(struct color_data_rec *color_data);
int cp_color_data_rec(struct color_data_rec *dst, struct color_data_rec *src);
void init_shared_buf_2d_rec(struct shared_buf_2d_rec *share);
void free_shared_buf_2d_rec(struct shared_buf_2d_rec **share);
int cp_shared_buf_2d_rec(struct shared_buf_2d_rec *dst,
    struct shared_buf_2d_rec *src);
void init_pixel_selection_rec(struct pixel_selection_rec *selection);
void free_pixel_selection_rec(struct pixel_selection_rec *selection);
int cp_pixel_selection_rec(struct pixel_selection_rec *dst,
    struct pixel_selection_rec *src);
void init_color_screen_rec(struct color_screen_rec *display);
int free_color_screen_rec(struct color_screen_rec *display);
int cp_color_screen_rec(struct color_screen_rec *dst,
    struct color_screen_rec *src, int is_cp_hidden, int is_cp_pixel_selection,
    int is_cp_mask);
int win_free_pixel_selection_rec(struct image_display_window *win);
int ValidateLocalColorIndxs(struct color_data_rec *color_data,
    struct color_scale_rec *scale);
int SetupLocalColorIndxsAry(struct color_data_rec *color_data,
    unsigned long num_colors);
int SetupLocalScale(struct color_data_rec *color_data,
    struct color_scale_rec *scale, unsigned long *color_indxs,
    unsigned long num_color_indxs);
int SetupLocalColorScale(struct intensity_rec *intensity,
    struct color_data_rec *color_data);
int SetupColorScreenRecLocalScales(struct intensity_rec *intensity,
    struct color_screen_rec *dp);
int TurnOffColorScreenRecNonLocalScales(struct color_screen_rec *dp);
int WinDefineCurColorData(struct image_display_window *win);
struct color_data_rec *WinCurColorData(struct image_display_window *win);
int WinColorDataDims(struct image_display_window *win, unsigned long *dimx,
    unsigned long *dimy, unsigned long *pixel_depth);
int WinSetupColorDatas(struct image_display_window *win);
int WinEnableDisableColorData(struct image_display_window *win,
    unsigned long gid, int is_disable);
int WinSetColorDataStatus(struct image_display_window *win, unsigned long gid,
    unsigned long andstatus, unsigned long orstatus);
int WinFindColorDataRec1(struct image_display_window *win, unsigned long gid,
    unsigned long member_id, struct color_data_rec **color_data,
    unsigned long *indx);
int WinFindPosNegColorDataRec(struct image_display_window *win,
    struct color_data_rec **color_data, unsigned long *indx, int is_pos);
int SortColorDataRecIdProc(const void *color1, const void *color2);
int WinFindAvailColorDataRecId(struct image_display_window *win,
    unsigned long *gid);
int WinSetupColorDataRec2(struct image_display_window *win, int is_new,
    unsigned long gid, unsigned long member_id, int use_dimxy,
    unsigned long dimx, unsigned long dimy, unsigned long pixel_depth,
    int is_init, struct color_data_rec **new_color_data,
    unsigned long color_data_type);
int WinAddColorDataRec(struct image_display_window *win,
    struct color_data_rec *color_data, int is_new_gid);
int WinDropColorDataRec1(struct image_display_window *win, unsigned long gid,
    int drop_whole_group, unsigned long member_id,
    struct color_data_rec *color_data, int is_deallocate);
int WinGetColorDataVal(struct image_display_window *win,
    unsigned long x, unsigned long y, double *val);
int win_setup_mask_bufs(struct image_display_window *win,
    long startoffx, long startoffy, long dimx, long dimy);
int init_draw_mask_value(struct image_display_window *win);
int WinInitScreenBuffer(struct image_display_window *win, int screen_depth);
int setup_color_data_rec(struct image_display_window *win);



/*------------------------------------------------------------------------
init_color_data_rec - ???
------------------------------------------------------------------------*/

void
init_color_data_rec(color_data)
 struct color_data_rec *color_data;
{
 color_data->status4 = 0;
 color_data->gid = 0;
 color_data->member_id = 0;
 init_buf_2d_rec(&(color_data->_2d));
 init_color_scale_rec(&(color_data->scale));
 color_data->total_color_indxs = 0;
 color_data->num_color_indxs = 0;
 color_data->color_indxs = NULL;
}


/*------------------------------------------------------------------------
free_color_data_rec - ???
------------------------------------------------------------------------*/

void
free_color_data_rec(color_data)
 struct color_data_rec *color_data;
{

 color_data->status4 = 0;
 color_data->gid = 0;
 color_data->member_id = 0;
 free_buf_2d_rec(&(color_data->_2d));
 free_color_scale_rec(&(color_data->scale));
 init_color_scale_rec(&(color_data->scale));
 if (color_data->total_color_indxs > 0)
    {color_data->total_color_indxs = 0;
     color_data->num_color_indxs = 0;
     c_free(color_data->color_indxs);
     color_data->color_indxs = NULL;
    }
}


/*------------------------------------------------------------------------
cp_color_data_rec - ???
------------------------------------------------------------------------*/

int
cp_color_data_rec(dst,src)
 struct color_data_rec *dst,*src;
{
 struct color_data_rec tmp;
 unsigned long lcv;
 int err;

 PRT2(printf("cp_color_data_rec...1\n"))
 init_color_data_rec(&tmp);
 if (((err = cp_color_scale_rec(&(tmp.scale),&(src->scale))) == NO_ERROR) &&
     ((err = copy_buf_2d_rec(&(tmp._2d),&(src->_2d))) == NO_ERROR))
    {
     PRT2(printf("cp_color_data_rec...2\n"))
     if (src->num_color_indxs > 0)
        {if ((err = get_ptr(&(tmp.color_indxs),&(tmp.total_color_indxs),
          src->num_color_indxs,(size_t) sizeof(*(tmp.color_indxs)))) == NO_ERROR)
            {tmp.num_color_indxs = tmp.total_color_indxs;
             for (lcv = 0; lcv < tmp.num_color_indxs; lcv++)
                tmp.color_indxs[lcv] = src->color_indxs[lcv];
            }
        }
    }
 if (err == NO_ERROR) 
    {
     tmp.status4 = src->status4;
     tmp.gid = src->gid;
     tmp.member_id = src->member_id;
     free_color_data_rec(dst);
     *dst = tmp;}
   else free_color_data_rec(&tmp);
 PRT2(printf("cp_color_data_rec...err = %d\n",err))
 return err;
}


/*------------------------------------------------------------------------
init_shared_buf_2d_rec - ???
------------------------------------------------------------------------*/

void
init_shared_buf_2d_rec(share)
 struct shared_buf_2d_rec *share;
{
 if (share == NULL) return;
 share->in_use = 0;
 init_buf_2d_rec(&(share->_2d));
}


/*------------------------------------------------------------------------
free_shared_buf_2d_rec - ???
------------------------------------------------------------------------*/

void
free_shared_buf_2d_rec(share)
 struct shared_buf_2d_rec **share;
{
 if (*share == NULL) return;
 (*share)->in_use -= 1;
 if ((*share)->in_use <= 0)
    {
     free_buf_2d_rec(&((*share)->_2d));
     c_free(*share);
    }
 *share = NULL;
}


/*------------------------------------------------------------------------
cp_shared_buf_2d_rec - ???
------------------------------------------------------------------------*/

int
cp_shared_buf_2d_rec(dst,src)
 struct shared_buf_2d_rec *dst,*src;
{
 int err;
 struct buf_2d_rec tmp;

 init_buf_2d_rec(&tmp);
 if ((err = copy_buf_2d_rec(&tmp,&(src->_2d))) == NO_ERROR)
    {free_buf_2d_rec(&(dst->_2d));
     dst->in_use = src->in_use;
     dst->_2d = tmp;}
   else free_buf_2d_rec(&tmp);
 return err;
}


/*------------------------------------------------------------------------
init_pixel_selection_rec - ???
------------------------------------------------------------------------*/

void
init_pixel_selection_rec(selection)
 struct pixel_selection_rec *selection;
{
 selection->status = 0;
 selection->selected_pixel_of_this_slice = NULL;
 init_buf_2d_rec(&(selection->selected_pixel));
 selection->share = NULL;
}


/*------------------------------------------------------------------------
free_pixel_selection_rec - ???
------------------------------------------------------------------------*/

void
free_pixel_selection_rec(selection)
 struct pixel_selection_rec *selection;
{
 free_shared_buf_2d_rec(&(selection->share));
 free_buf_2d_rec(&(selection->selected_pixel));
 init_pixel_selection_rec(selection);
}


/*------------------------------------------------------------------------
cp_pixel_selection_rec - ???
------------------------------------------------------------------------*/

int
cp_pixel_selection_rec(dst,src)
 struct pixel_selection_rec *dst,*src;
{
 struct pixel_selection_rec pix;
 int err;

 PRT2(printf("cp_pixel_selection_rec...1\n"))
 init_pixel_selection_rec(&pix);
 pix.status = src->status;
 pix.selected_pixel_of_this_slice = src->selected_pixel_of_this_slice;
 pix.lb = src->lb;
 pix.ub = src->ub;
 if ((err = copy_buf_2d_rec(&(pix.selected_pixel),&(src->selected_pixel))) == 
          NO_ERROR)
    {PRT2(printf("cp_pixel_selection_rec...2\n"))
     if (src->share != NULL)
        if ((err = get_ptr(&(pix.share),NULL,1,(size_t) sizeof(*(pix.share))))
               == NO_ERROR)
           err = cp_shared_buf_2d_rec(pix.share,src->share);
     PRT2(printf("cp_pixel_selection_rec...3..err = %d\n",err))
    }
 if (err == NO_ERROR)
    {free_pixel_selection_rec(dst);
     *dst = pix;
    }
   else free_pixel_selection_rec(&pix);
 PRT2(printf("cp_pixel_selection_rec...end..err = %d\n",err))
 return err;
}


/*------------------------------------------------------------------------
init_color_screen_rec - ???
------------------------------------------------------------------------*/

void
init_color_screen_rec(display)
 struct color_screen_rec *display;
{
 unsigned long lcv;

 setup_int_bound_rec(&(display->bound),0,0,0,0);
 display->total_colors = display->num_colors = 0;
 display->colors = NULL;
 display->def_pos_scale_label = display->def_neg_scale_label = NULL;
 display->scale_label = NULL;
 display->cur_gid = 0;
 display->cur_member_id = 0;
 display->cur_color_data = 0;
 display->total_color_datas = display->num_color_datas = 0;
 display->color_datas = NULL;
 init_pixel_selection_rec(&(display->selection));
 for (lcv = 0; lcv < TOTAL_DRAW_MASKS; lcv++)
    init_draw_mask_rec(&(display->draw_masks[lcv]));
 display->num_draw_masks = 0;
}


/*------------------------------------------------------------------------
free_color_screen_rec - ???
------------------------------------------------------------------------*/

int
free_color_screen_rec(display)
 struct color_screen_rec *display;
{
 unsigned long lcv;

 if (display->total_colors > 0)
    {for (lcv = 0; lcv < display->num_colors; lcv++)
        if ((display->colors[lcv] != NULL) &&
            !(display->colors[lcv]->status4 & COLOR_DATA_DONT_DELETE))
           {free_color_data_rec(display->colors[lcv]);
            c_free(display->colors[lcv]);}
     c_free(display->colors);}
 c_free(display->def_pos_scale_label);
 c_free(display->def_neg_scale_label);
 c_free(display->scale_label);
 if (display->total_color_datas > 0)
    c_free(display->color_datas);
 free_pixel_selection_rec(&(display->selection));
 for (lcv = 0; lcv < TOTAL_DRAW_MASKS; lcv++)
    free_draw_mask_rec(&(display->draw_masks[lcv]));

 init_color_screen_rec(display);
 return NO_ERROR;
}


/*------------------------------------------------------------------------
cp_color_screen_rec - ???
------------------------------------------------------------------------*/

int
cp_color_screen_rec(dst,src,is_cp_hidden,is_cp_pixel_selection,is_cp_mask)
 struct color_screen_rec *dst,*src;
{
 struct color_screen_rec tmp;
 unsigned long lcv,num,cnt;
 int err;

 PRT2(printf("cp_color_screen_rec...\n"))
 init_color_screen_rec(&tmp);
 tmp.bound = src->bound;
 tmp.cur_gid = src->cur_gid;
 tmp.cur_member_id = src->cur_member_id;
 if (((err = cp_str(&(tmp.def_pos_scale_label),&(src->def_pos_scale_label))) ==
           NO_ERROR) &&
     ((err = cp_str(&(tmp.def_neg_scale_label),&(src->def_neg_scale_label))) ==
           NO_ERROR) &&
     ((err = cp_str(&(tmp.scale_label),&(src->scale_label))) ==
           NO_ERROR))
    {if (is_cp_hidden) num = src->num_colors;
       else num = src->num_color_datas;
     PRT2(printf("cp_color_screen_rec...num = %d\n",num))
     if ((num > 0) &&
         ((err = get_handle(&(tmp.colors),&(tmp.total_colors),num,
           (size_t) sizeof(**(tmp.colors)))) == NO_ERROR))
        {tmp.num_colors = tmp.total_colors;
         PRT2(printf("cp_color_screen_rec...2\n"))
         for (lcv = 0; lcv < tmp.total_colors; lcv++)
            init_color_data_rec(tmp.colors[lcv]);
         for (lcv = 0; (lcv < tmp.total_colors) && (err == NO_ERROR); lcv++)
            if (is_cp_hidden)
               err = cp_color_data_rec(tmp.colors[lcv],src->colors[lcv]);
              else err = cp_color_data_rec(tmp.colors[lcv],src->color_datas[lcv]);

         PRT2(printf("cp_color_screen_rec...3...err = %d\n",err))
         if ((err == NO_ERROR) && (src->num_color_datas > 0))
            {if ((err = get_ptr(&(tmp.color_datas),&(tmp.total_color_datas),
                  src->num_color_datas,
                   (size_t) sizeof(*(tmp.color_datas)))) == NO_ERROR)
               {tmp.num_color_datas = tmp.total_color_datas;
                for (lcv = 0; lcv < tmp.num_color_datas; lcv++)
                   {for (cnt = 0; cnt < tmp.num_colors; cnt++)
                       if ((tmp.colors[cnt]->gid == src->color_datas[lcv]->gid) &&
                           (tmp.colors[cnt]->member_id == 
                            src->color_datas[lcv]->member_id))
                          {tmp.color_datas[lcv] = tmp.colors[cnt];
                           PRT2(printf("%d) gid = %d member_id = %d\n",cnt,
                              tmp.colors[cnt]->gid,tmp.colors[cnt]->member_id))
                           break;}
                   }
               }
            }
        }
    }
 PRT2(printf("cp_color_screen_rec...4 err = %d\n",err))
 if (err == NO_ERROR)
    if (is_cp_pixel_selection)
       err = cp_pixel_selection_rec(&(tmp.selection),&(src->selection));

 if (err == NO_ERROR)
    if (is_cp_mask)
       for (lcv = 0; lcv < src->num_draw_masks; lcv++)
          if ((err = cp_draw_mask_rec(&(tmp.draw_masks[lcv]),
                &(src->draw_masks[lcv]))) != NO_ERROR)
             break;

 PRT2(printf("cp_color_screen_rec...5 err = %d\n",err))
 if (err == NO_ERROR)
    {free_color_screen_rec(dst);
     *dst = tmp;}
   else free_color_screen_rec(&tmp);
 return err;
}


/*------------------------------------------------------------------------
win_free_pixel_selection_rec - ???
------------------------------------------------------------------------*/

int
win_free_pixel_selection_rec(win)
 struct image_display_window *win;
{
 free_pixel_selection_rec(&(win->data.selection));

 return NO_ERROR;
}


/*------------------------------------------------------------------------
ValidateLocalColorIndxs - ???
------------------------------------------------------------------------*/

int
ValidateLocalColorIndxs(color_data,scale)
 struct color_data_rec *color_data;
 struct color_scale_rec *scale;
{
 PRT1(printf("ValidateLocalColorIndxs num = %d total_color_indxs = %d\n",
       color_data->num_color_indxs,color_data->total_color_indxs))

 if (color_data->num_color_indxs == 0) return CANCEL;
 if ((scale->num_colors != color_data->num_color_indxs) ||
     (scale->num_colors > color_data->total_color_indxs) ||
     (scale->first != color_data->scale.first) ||
     (str_cmp(scale->label,color_data->scale.label) != 0))
    {
     PRT1(printf("ValidateLocalColorIndxs..CANCEL\n"))
     return CANCEL;}
 PRT1(printf("ValidateLocalColorIndxs..NO_ERROR\n"))
 return NO_ERROR;
}


/*------------------------------------------------------------------------
SetupLocalColorIndxsAry - ???
------------------------------------------------------------------------*/

int
SetupLocalColorIndxsAry(color_data,num_colors)
 struct color_data_rec *color_data;
 unsigned long num_colors;
{
 unsigned long lcv;
 int err;

 if (num_colors > color_data->total_color_indxs)
    {color_data->num_color_indxs = 0;
     lcv = num_colors - color_data->total_color_indxs;
     if ((err = get_ptr(&(color_data->color_indxs),
         &(color_data->total_color_indxs),lcv,
         (size_t) sizeof(*(color_data->color_indxs))) ) != NO_ERROR)
        return err;
    }
 color_data->num_color_indxs = num_colors;
 return NO_ERROR;
}


/*------------------------------------------------------------------------
SetupLocalScale - ???
------------------------------------------------------------------------*/

int
SetupLocalScale(color_data,scale,color_indxs,num_color_indxs)
 struct color_data_rec *color_data;
 struct color_scale_rec *scale;
 unsigned long *color_indxs,num_color_indxs;
{
 int err;
 unsigned long lcv;

 if ((scale != &(color_data->scale)) &&
     ((err = cp_color_scale_rec(&(color_data->scale),scale)) != NO_ERROR))
    return err;

 if ((err = SetupLocalColorIndxsAry(color_data,num_color_indxs)) != NO_ERROR)
    return err;
 color_data->scale.status |= COLOR_CONTROL_VALID;
 for (lcv = 0; lcv < num_color_indxs; lcv++)
    {color_data->color_indxs[lcv] = color_indxs[lcv];
     PRT(printf("%d) indx = %d\n",lcv,color_data->color_indxs[lcv]);)}

 return err;
}


/*------------------------------------------------------------------------
SetupLocalColorScale - ???
------------------------------------------------------------------------*/

int
SetupLocalColorScale(intensity,color_data)
 struct intensity_rec *intensity;
 struct color_data_rec *color_data;
{
 unsigned long *color_indxs;
 struct color_scale_rec *scale;

 if (color_data->scale.status & COLOR_LOCAL)
    {PRT(printf("SetupLocalColorScale...COLOR_LOCAL\n"))
     color_indxs = &(intensity->color.color_indxs[color_data->scale.first]);}
   else 
    {PRT(printf("SetupLocalColorScale...cur_scale = %d\n",intensity->cur_scale))
     PRT(printf("...color_data->scale.label = %s\n",color_data->scale.label))
     if ((color_data->scale.label == NULL) ||
         ((scale = find_scale(intensity,color_data->scale.label,0)) == NULL))
        {scale = &(intensity->scales[intensity->cur_scale]);
         PRT(printf("SetupLocalColorScale def scale\n"))}
       else
        {PRT(printf("SetupLocalColorScale  scale found\n"))}
     color_indxs = &(intensity->color.color_indxs[scale->first]);}
 PRT(printf("SetupLocalColorScale...scale->label = %s\n",scale->label))
 return SetupLocalScale(color_data,&(color_data->scale),
                   color_indxs,color_data->scale.num_colors);
}


/*------------------------------------------------------------------------
SetupColorScreenRecLocalScales - ???
------------------------------------------------------------------------*/

int
SetupColorScreenRecLocalScales(intensity,dp)
 struct intensity_rec *intensity;
 struct color_screen_rec *dp;
{
 unsigned long lcv;
 struct color_data_rec *color;
 int err;

 err = NO_ERROR;
 for (lcv = 0; lcv < dp->num_color_datas; lcv++)
    {color = dp->color_datas[lcv];
     PRT(printf("%d) LOCAL num_color_indxs = %d %d %d\n",
         lcv,color->num_color_indxs,color->total_color_indxs,
         color->scale.num_colors))
     if ((color->num_color_indxs != color->scale.num_colors) ||
         (color->total_color_indxs < color->scale.num_colors))
        if ((err = SetupLocalColorScale(intensity,color)) != NO_ERROR)
           break;
    }
 PRT(printf("SetupColorScreenRecLocalScales num_color_datas = %d err = %d\n",
       dp->num_color_datas,err))
 return err;
}


/*------------------------------------------------------------------------
TurnOffColorScreenRecNonLocalScales - ???
------------------------------------------------------------------------*/

int
TurnOffColorScreenRecNonLocalScales(dp)
 struct color_screen_rec *dp;
{
 unsigned long lcv;

 for (lcv = 0; lcv < dp->num_color_datas; lcv++)
    if (!(dp->color_datas[lcv]->scale.status & COLOR_LOCAL))
       {dp->color_datas[lcv]->num_color_indxs = 0;
        PRT(printf("%d) TurnOff LOCAL num_color_indxs = %d %d %d\n",
            lcv,dp->color_datas[lcv]->num_color_indxs,
            dp->color_datas[lcv]->total_color_indxs,
            dp->color_datas[lcv]->scale.num_colors))}
 return NO_ERROR;
}



/*------------------------------------------------------------------------
WinDefineCurColorData - ???
------------------------------------------------------------------------*/

int
WinDefineCurColorData( win )
     struct image_display_window *win;
{
  unsigned long lcv = 0;
  int found = 0;

  if (win->data.num_color_datas == 0)
  {
    if (win->data.num_colors == 0)
      return NOT_FOUND;
    win->data.cur_color_data = 0;
    win->data.cur_gid = win->data.colors[0]->gid;
    win->data.cur_member_id = win->data.colors[0]->member_id;
    cp_str(&(win->data.scale_label),win->data.colors[0]->scale.label);

    PRT1(printf("WinDefineCurColorData NOT_FOUND\n"));
    return NO_ERROR;
  }
  if (win->data.scale_label == NULL)
  {
    win->data.cur_color_data = 0;
    win->data.cur_gid = win->data.color_datas[0]->gid;
    win->data.cur_member_id = win->data.color_datas[0]->member_id;
    cp_str(&(win->data.scale_label),win->data.color_datas[0]->scale.label);
  }
  else
  {
    lcv = win->data.cur_color_data;
    if ((win->data.cur_color_data >= win->data.num_color_datas) ||
	(win->data.color_datas[lcv]->gid != win->data.cur_gid) ||
	(win->data.color_datas[lcv]->member_id != win->data.cur_member_id) ||
	(str_cmp(win->data.scale_label,win->data.color_datas[lcv]->scale.label) != 0))
    {
      for (found = 0,lcv = 0; lcv < win->data.num_color_datas; lcv++)
      {
	if ((win->data.color_datas[lcv]->gid == win->data.cur_gid) &&
	    (win->data.color_datas[lcv]->member_id == win->data.cur_member_id) &&
	    (win->data.color_datas[lcv]->status4 & COLOR_DATA_DEFAULT) &&
	    (str_cmp(win->data.scale_label,
		     win->data.color_datas[lcv]->scale.label) == 0))
	{
	  found = 1;
	}
      }

      if (!found)
      {
	win->data.cur_color_data = 0;
	win->data.cur_gid = win->data.color_datas[0]->gid;
	win->data.cur_member_id = win->data.color_datas[0]->member_id;
	cp_str(&(win->data.scale_label),win->data.color_datas[0]->scale.label);
      }
    }
  }
  PRT1(printf("WinDefineCurColorData cur = %d cur_gid = %d cur_member_id = %d label = %s\n",
	       win->data.cur_color_data,win->data.cur_gid,
	       win->data.cur_member_id,win->data.scale_label));
  return NO_ERROR;
}



/*------------------------------------------------------------------------
WinCurColorData - ???
------------------------------------------------------------------------*/

struct color_data_rec *
WinCurColorData( win )
     struct image_display_window *win;
{

  if (win->data.cur_color_data >= win->data.num_color_datas)
    WinDefineCurColorData( win );

  if (win->data.num_color_datas == 0)
  {
    if (win->data.num_colors == 0)
      return NULL;
    return win->data.colors[0];
  }
  return win->data.color_datas[win->data.cur_color_data];
}



/*------------------------------------------------------------------------
WinColorDataDims - ???
------------------------------------------------------------------------*/

int
WinColorDataDims( win, dimx, dimy, pixel_depth )
     struct image_display_window *win;
     unsigned long *dimx, *dimy, *pixel_depth;
{
  struct color_data_rec *color_data = NULL;

  if ((color_data = WinCurColorData( win )) == NULL)
    return CANCEL;

#if 0
  if (dimx != NULL)
    *dimx = color_data->_2d.dimx;
  if (dimy != NULL)
    *dimy = color_data->_2d.dimy;
#endif

  if (dimx != NULL)
    *dimx = win->data.bound.right - win->data.bound.left + 1;
  if (dimy != NULL) 
    *dimy = win->data.bound.bottom - win->data.bound.top + 1;
  if (pixel_depth != NULL)
    *pixel_depth = color_data->_2d.pixel_depth;

  return NO_ERROR;
}


/*************
WinDefaultColorDataReady(win)
 struct image_display_window *win;
{
 unsigned long lcv;
 int found;

 PRT1(printf("WinDefaultColorDataReady num_color_datas = %d\n",
      win->data.num_color_datas))
 for (found = 0,lcv = 0; lcv < win->data.num_color_datas; lcv++)
    {PRT1(printf("%d) status4 = %x\n",
          lcv,win->data.color_datas[lcv]->status4))
     if ((win->data.color_datas[lcv]->status4 & COLOR_DATA_DEFAULT_AND_ID_VALID)
             == COLOR_DATA_DEFAULT_AND_ID_VALID)
        {found = 1;
         if (!(win->data.color_datas[lcv]->_2d.status4 & BUF_NON_EMPTY) ||
             !(win->data.color_datas[lcv]->_2d.status4 & BUF_DATA_VALID))
            {PRT2(printf("WinDefaultColorDataReady gid = %d=== not ready\n",
                   win->data.color_datas[lcv]->gid))
             return CANCEL;}
        }
    }
 if (found) return NO_ERROR;
 PRT1(printf("WinDefaultColorDataReady not ready\n"))
 return CANCEL;
}
**************/


/*------------------------------------------------------------------------
WinSetupColorDatas - ???
------------------------------------------------------------------------*/

int
WinSetupColorDatas(win)
 struct image_display_window *win;
{
 unsigned long lcv,cnt;
 int err;

 PRT1(printf("WinSetupColorDatas...num_colors = %d total_colors = %d\n",
       win->data.num_colors,win->data.total_colors))
 if (win->data.num_colors == 0) return CANCEL;
 for (cnt = 0,lcv = 0; lcv < win->data.num_colors; lcv++)
    if (!(win->data.colors[lcv]->status4 & COLOR_DATA_DISABLED)) cnt++;

 if (cnt > win->data.total_color_datas)
   {lcv = win->data.total_color_datas;
    if ((err = get_ptr(&(win->data.color_datas),&(win->data.total_color_datas),
          cnt + 5,(size_t) sizeof(*(win->data.color_datas)))) != NO_ERROR)
       return err;
    for (; lcv < win->data.total_color_datas; lcv++)
       win->data.color_datas[lcv] = NULL;}

 PRT1(printf("num_colors = %d...cnt = %d total_color_datas = %d\n",
      win->data.num_colors,cnt,win->data.total_color_datas))
 for (cnt = 0,lcv = 0; lcv < win->data.num_colors; lcv++)
    {
     PRT1(printf("%d) status4 = %x\n",lcv,win->data.colors[lcv]->status4))
     if (!(win->data.colors[lcv]->status4 & COLOR_DATA_DISABLED)) 
        win->data.color_datas[cnt++] = win->data.colors[lcv];
    }
 win->data.num_color_datas = cnt;
 WinDefineCurColorData(win);

 PRT1(printf("WinSetupColorDatas cnt  num_color_datas = %d\n",cnt))
 return NO_ERROR;
}


/*------------------------------------------------------------------------
WinEnableDisableColorData - ???
------------------------------------------------------------------------*/

int
WinEnableDisableColorData(win,gid,is_disable)
 struct image_display_window *win;
 unsigned long gid;
 int is_disable;
{
 struct color_data_rec *color_data;
 unsigned long lcv;
 int err;

 err = NOT_FOUND;
 for (lcv = 0; lcv < win->data.num_colors; lcv++)
    if (win->data.colors[lcv]->gid == gid)
       {color_data = win->data.colors[lcv];
        if (is_disable) color_data->status4 |= COLOR_DATA_DISABLED;
          else color_data->status4 &= ~COLOR_DATA_DISABLED;
        err = NO_ERROR;}
 if (err == NO_ERROR) return WinSetupColorDatas(win);
 return err;
}


/*------------------------------------------------------------------------
WinSetColorDataStatus - ???
------------------------------------------------------------------------*/

int
WinSetColorDataStatus(win,gid,andstatus,orstatus)
 struct image_display_window *win;
 unsigned long gid,andstatus,orstatus;
{
 struct color_data_rec *color_data;
 unsigned long lcv;
 int err;

 err = NOT_FOUND;
 for (lcv = 0; lcv < win->data.num_colors; lcv++)
    if (win->data.colors[lcv]->gid == gid)
       {color_data = win->data.colors[lcv];
        color_data->status4 &= andstatus;
        color_data->status4 |= orstatus;
        PRT1(printf("WinSetColorDataStatus gid = %d member_id = %d\n",
              color_data->gid,color_data->member_id))
        err = NO_ERROR;}
 if (err == NO_ERROR) return WinSetupColorDatas(win);
 return err;
}


/*------------------------------------------------------------------------
WinFindColorDataRec1 - ???

gid == 0 then return the default color_data_rec win->data.colors[0]
gid == 1 then return the pixel type buf if win->data.colors[lcv].id == id
gid == 2 then return the overlay buf if win->data.colors[lcv].id == id
gid == 3 then return the pixel selection buf if win->data.colors[lcv].id == id
------------------------------------------------------------------------*/

int
WinFindColorDataRec1(win,gid,member_id,color_data,indx)
 struct image_display_window *win;
 unsigned long gid,member_id;
 struct color_data_rec **color_data;
 unsigned long *indx;
{
 unsigned long lcv;

 *color_data = NULL;
 PRT1(printf("WinFindColorDataRec1 wid = %s gid = %d member_id = %d win->data.num_colors = %d\n",
      win->owner->wid,gid,member_id,win->data.num_colors))
 for (lcv = 0; lcv < win->data.num_colors; lcv++)
    if ((win->data.colors[lcv]->gid == gid) &&
        (win->data.colors[lcv]->member_id == member_id))
       {*color_data = win->data.colors[lcv];
        PRT1(printf("WinFindColorDataRec1 FOUND\n"))
        if (indx != NULL) *indx = lcv;
        return NO_ERROR;}
 PRT1(printf("WinFindColorDataRec1 NOT_FOUND\n"))
 return NOT_FOUND;
}


/*------------------------------------------------------------------------
WinFindPosNegColorDataRec - ???
------------------------------------------------------------------------*/

int
WinFindPosNegColorDataRec(win,color_data,indx,is_pos)
 struct image_display_window *win;
 struct color_data_rec **color_data;
 unsigned long *indx;
 int is_pos;
{
 unsigned long lcv;

 *color_data = NULL;
 PRT3(printf("WinFindPosNegColorDataRec wid = %s num_color_datas = %d\n",
      win->owner->wid,win->data.num_color_datas))
 for (lcv = 0; lcv < win->data.num_color_datas; lcv++)
     if ((win->data.color_datas[lcv]->status4 & COLOR_DATA_DEFAULT_AND_ID_VALID)
             == COLOR_DATA_DEFAULT_AND_ID_VALID)
       {if (is_pos) 
           {if (win->data.color_datas[lcv]->status4 & COLOR_DATA_NEG_ACTIVATION)
               continue;
           }
          else if (!(win->data.color_datas[lcv]->status4 & COLOR_DATA_NEG_ACTIVATION))
           continue;
        PRT3(printf("WinFindPosNegColorDataRec FOUND\n"))
        *color_data = win->data.color_datas[lcv];
        if (indx != NULL) *indx = lcv;
        return NO_ERROR;}
 PRT3(printf("WinFindPosNegColorDataRec NOT_FOUND\n"))
 return NOT_FOUND;
}

/*********************
int SortColorDataRecIdProc(color1,color2)
 struct color_data_rec **color1,**color2;
{
 PRT1(printf("SortColorDataRecIdProc...gid1 = %d gid2 = %d\n",
         (*color1)->gid,(*color2)->gid))
 if ((*color1)->gid < (*color2)->gid) return -1;
 if ((*color1)->gid > (*color2)->gid) return 1;
 if ((*color1)->member_id < (*color2)->member_id) return -1;
 if ((*color1)->member_id > (*color2)->member_id) return 1;
 return 0;
}
**************************/


/*------------------------------------------------------------------------
SortColorDataRecIdProc - ???
------------------------------------------------------------------------*/

int 
SortColorDataRecIdProc(color1,color2)
 const void *color1,*color2;
{
 PRT1(printf("SortColorDataRecIdProc...gid1 = %d gid2 = %d\n",
         (*((struct color_data_rec **) color1))->gid,
         (*((struct color_data_rec **) color2))->gid))
 if ((*((struct color_data_rec **) color1))->gid < 
     (*((struct color_data_rec **) color2))->gid) return -1;
 if ((*((struct color_data_rec **) color1))->gid > 
     (*((struct color_data_rec **) color2))->gid) return 1;

 if ((*((struct color_data_rec **) color1))->member_id < 
     (*((struct color_data_rec **) color2))->member_id) return -1;
 if ((*((struct color_data_rec **) color1))->member_id > 
     (*((struct color_data_rec **) color2))->member_id) return 1;
 return 0;
}


/*------------------------------------------------------------------------
WinFindAvailColorDataRecId - ???
------------------------------------------------------------------------*/

int
WinFindAvailColorDataRecId(win,gid)
 struct image_display_window *win;
 unsigned long *gid;
{
 unsigned long lcv,gid1;

 if (win->data.num_colors == 0) return CANCEL;
 for (gid1 = 4,lcv = 0; lcv < win->data.num_colors; lcv++)
    {if (win->data.colors[lcv]->gid < 4) continue;
     if (win->data.colors[lcv]->gid == gid1) gid1++;
       else if (win->data.colors[lcv]->gid > gid1) break;
    }
 *gid = gid1;
 return NO_ERROR;
}


/*------------------------------------------------------------------------
WinSetupColorDataRec2 - ???

if (is_new) gid = WinFindAvailColorDataRecId(win,&gid)
if (gid == 0) add default color_data_rec
if (gid == OVERLAY_ID) add overlay
if (gid == PIXEL_TYPE_ID) add pixel_type_overlay
if (gid == PIXEL_SELECTION_ID) add selection.selected_pixel
  else add color_data_rec of gid
------------------------------------------------------------------------*/

int
WinSetupColorDataRec2(win,is_new,gid,member_id,use_dimxy,dimx,dimy,pixel_depth,
                     is_init,new_color_data,color_data_type)
 struct image_display_window *win;
 int is_new,use_dimxy,is_init;
 unsigned long gid,member_id,dimx,dimy,pixel_depth;
 struct color_data_rec **new_color_data;
 unsigned long color_data_type;
{
 struct color_data_rec *color;
 struct buf_2d_rec *_2d;
 int err;
 unsigned long gid1,member_id1,dimx1,dimy1,lcv;
 struct img_buf_rec *img;

 PRT1(printf("WinSetupColorDataRec2...is_new  = %d gid = %d member_id = %d\n",
          is_new,gid,member_id);
       printf("use_dimxy = %d dimx = %d dimy = %d pixel_depth = %d\n",
          use_dimxy,dimx,dimy,pixel_depth))
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 if (use_dimxy)
    {dimx1 = dimx;
     dimy1 = dimy;}
   else
    {dimx1 = img->_2d.dimx;
     dimy1 = img->_2d.dimy;}
 if (is_new)
    {if ((err = WinFindAvailColorDataRecId(win,&gid1)) != NO_ERROR)
        return err;
     member_id1 = 0;
     PRT1(printf("is new creating new_color_data...is_init = %d gid1 = %d member_id1 = %d\n",
        is_init,gid1,member_id1))
    }
   else
    {gid1 = gid;
     member_id1 = member_id;
     if ((err = WinFindColorDataRec1(win,gid1,member_id1,
                            new_color_data,NULL)) == NO_ERROR)
        {_2d = &((*new_color_data)->_2d);
         if ((err = SetupBuf2dRec(_2d,dimx1,dimy1,
                      pixel_depth,is_init)) != NO_ERROR)
            return err;
         (*new_color_data)->status4 |= color_data_type;
         (*new_color_data)->status4 &= ~COLOR_DATA_DISABLED;
         PRT1(printf("using old_color_data...is_init = %d gid1 = %d member_id1 = %d\n",
               is_init,gid1,member_id1))
         PRT1(pr_buf_2d_rec(_2d))
         return WinSetupColorDatas(win);}
     PRT1(printf("creating new_color_data...is_init = %d gid1 = %d member_id1 = %d\n",
        is_init,gid1,member_id1))
    }
 PRT1(printf("WinSetupColorDataRec2...1\n"))
 if ((err = get_ptr(&color,NULL,1,(size_t) sizeof(*color))) != NO_ERROR)
    return err;
 init_color_data_rec(color);
 _2d = &(color->_2d);
 if ((err = SetupBuf2dRec(_2d,dimx1,dimy1,
                      pixel_depth,is_init)) != NO_ERROR)
    {c_free(color);
     return err;}

 PRT1(printf("WinSetupColorDataRec2...num_colors = %d total_colors = %d\n",
            win->data.num_colors,win->data.total_colors))
 if (win->data.num_colors >= win->data.total_colors)
    {lcv = win->data.total_colors;
     if ((err = get_ptr(&(win->data.colors),&(win->data.total_colors),5,
              (size_t) sizeof(*(win->data.colors)))) != NO_ERROR)
        {free_buf_2d_rec(_2d);
         c_free(color);
         return err;
        }
     for (; lcv < win->data.total_colors; lcv++)
        win->data.colors[lcv] = NULL;
    }
 color->status4 |= COLOR_DATA_ID_VALID | color_data_type;
 color->gid = gid1;
 color->member_id = member_id1;
 if (win->intensity != NULL)
    {PRT(printf("WinSetupColorDataRec2...%u cur_scale = %d status = %x\n",
           &(win->intensity->scales[win->intensity->cur_scale]),
           win->intensity->cur_scale,
           (unsigned long) win->intensity->scales[win->intensity->cur_scale].status))
     if ((err = WinDefaultColorScale(win,color)) != NO_ERROR)
        {free_buf_2d_rec(_2d);
         free_color_data_rec(color);
         c_free(color);
         return err;
        }
    }
 win->data.colors[win->data.num_colors] = color;
 win->data.num_colors += 1;

 if (win->data.num_colors > 1)
    {qsort(win->data.colors,win->data.num_colors,
        (size_t) sizeof(*(win->data.colors)),SortColorDataRecIdProc);
     PRT1(for (lcv = 0; lcv < win->data.num_colors; lcv++)
              printf("%d) gid = %d\n",lcv,win->data.colors[lcv]->gid);)
    }
 *new_color_data = color;
 PRT1(printf("WinSetupColorDataRec2...NO_ERROR\n"))
 return WinSetupColorDatas(win);
}


/*------------------------------------------------------------------------
WinAddColorDataRec - ???
------------------------------------------------------------------------*/

int
WinAddColorDataRec(win,color_data,is_new_gid)
 struct image_display_window *win;
 struct color_data_rec *color_data;
 int is_new_gid;
{
 unsigned long lcv,indx;
 struct color_data_rec *color_data1;
 int err;

 PRT1(printf("WinAddColorDataRec...is_new_gid = %d\n",is_new_gid))
 if (is_new_gid)
    {if ((err = WinFindAvailColorDataRecId(win,&(color_data->gid))) != NO_ERROR)
        return err;}
   else
    for (lcv = 0; lcv < win->data.num_colors; lcv++)
       if (WinFindColorDataRec1(win,color_data->gid,color_data->member_id,
             &color_data1,&indx) == NO_ERROR)
          {if (color_data1 != color_data)
              {win->data.colors[indx] = color_data;
               if (!(color_data1->status4 & COLOR_DATA_DONT_DELETE))
                  {free_color_data_rec(color_data1);
                   c_free(color_data1);}
              }
           return NO_ERROR;}
  
 if (win->data.num_colors >= win->data.total_colors)
   {if ((err = get_ptr(&(win->data.colors),&(win->data.total_colors),5,
              (size_t) sizeof(*(win->data.colors)))) != NO_ERROR)
       return err;
    }
 PRT1(printf("WinAddColorDataRec...\n"))
 color_data->status4 |= COLOR_DATA_ID_VALID;
 win->data.colors[win->data.num_colors] = color_data;
 win->data.num_colors += 1;
 if ((err = WinSetupColorDatas(win)) != NO_ERROR)
    {win->data.num_colors -= 1;
     win->data.colors[win->data.num_colors] = NULL;
     color_data->status4 &= ~COLOR_DATA_ID_VALID;}

 return err;
}


/*------------------------------------------------------------------------
WinDropColorDataRec1 - ???
------------------------------------------------------------------------*/

int
WinDropColorDataRec1(win,gid,drop_whole_group,member_id,color_data,is_deallocate)
 struct image_display_window *win;
 int drop_whole_group;
 unsigned long gid,member_id;
 struct color_data_rec *color_data;
 int is_deallocate;
{
 unsigned long lcv,cnt;
 struct color_data_rec *color_data1;
 int err;

 PRT1(printf("WinDropColorDataRec1...gid = %d member_id = %d is_deallocate = %d\n",
        gid,member_id,is_deallocate))
 err = NOT_FOUND;
 for (lcv = 0; lcv < win->data.num_colors; lcv++)
    if (win->data.colors[lcv]->gid == gid)
       {color_data1 = win->data.colors[lcv];
        if (!drop_whole_group && (color_data1->member_id != member_id))
           continue;
        if ((color_data == NULL) || (color_data == color_data1))
           {
            PRT1(printf("WinDropColorDataRec NO_ERROR\n"))
            if (is_deallocate)
               {free_color_data_rec(color_data1);
                c_free(color_data1);}
            win->data.num_colors -= 1;
            for (cnt = lcv; cnt < win->data.num_colors; cnt++)
               win->data.colors[cnt] = win->data.colors[cnt + 1];
            win->data.colors[cnt] = NULL;
            err = NO_ERROR;
            lcv--;}
        if (!drop_whole_group || (color_data != NULL)) break;
       }
 PRT1(printf("WinDropColorDataRec1 NOT_FOUND\n"))
 if (err == NO_ERROR) return WinSetupColorDatas(win);
 return NOT_FOUND;
}


/*------------------------------------------------------------------------
WinGetColorDataVal - ???
------------------------------------------------------------------------*/

int
WinGetColorDataVal(win,x,y,val)
 struct image_display_window *win;
 unsigned long x,y;
 double *val;
{
 unsigned long lcv;

 if (win->data.num_color_datas == 0) return CANCEL;
 for (lcv = win->data.num_color_datas  - 1; ; )
    {if ((x < win->data.color_datas[lcv]->_2d.dimx) &&
         (y < win->data.color_datas[lcv]->_2d.dimy))
        {*val = win->data.color_datas[lcv]->_2d.GetVal(
                     &(win->data.color_datas[lcv]->_2d),y,x);
         if (*val != .0) return NO_ERROR;
         if (lcv == 0) return NO_ERROR;
        }
     if (lcv == 0) return CANCEL;
     lcv--;}

 /* FIXME:  Added the following line to make compiler happy, but
    am not sure it's right.  -PWH 19990401 */
 return CANCEL;     
}


/*------------------------------------------------------------------------
win_setup_mask_bufs - ???
------------------------------------------------------------------------*/

int
win_setup_mask_bufs(win,startoffx,startoffy,dimx,dimy)
 struct image_display_window *win;
 long startoffx,startoffy,dimx,dimy;
{
 int err;
 unsigned long lcv,cnt,total;
 struct draw_mask_rec *draw_masks;
 struct buf_2d_rec *_2ds;

 draw_masks = win->data.draw_masks;
 for (cnt = 0,lcv = 0; lcv < win->data.num_draw_masks; lcv++)
    {if (!(draw_masks[lcv].mask._2d.status4 & BUF_NON_EMPTY) ||
          (draw_masks[lcv].mask._2d.dimx != dimx) ||
          (draw_masks[lcv].mask._2d.dimy != dimy))
        cnt += 1;
    }
 if (cnt == 0) return NO_ERROR;
 total = 0;
 if ((err = get_ptr(&_2ds,&total,cnt,(size_t) sizeof(*_2ds))) != NO_ERROR)
    return err;
 for (lcv = 0; lcv < total; lcv++)
    {init_buf_2d_rec(&(_2ds[lcv]));
     _2ds[lcv].startoffx = startoffx;
     _2ds[lcv].startoffy = startoffy;}
 for (lcv = 0; lcv < total; lcv++)
    if ((err = SetupBuf2dRec(&(_2ds[lcv]),dimx,dimy,1,0)) != NO_ERROR)
       {for (cnt = 0; cnt < lcv; cnt++)
           free_buf_2d_rec(&(_2ds[cnt]));
        c_free(_2ds);
        return err;}
 for (lcv = 0; lcv < total; lcv++)
    InitBuf2dRecData(&(_2ds[lcv]),(double) .0);

 TRACE(printf("win_setup_mask_bufs(win)\n"))
 for (lcv = 0; lcv < win->data.num_draw_masks; lcv++)
    {if (!(draw_masks[lcv].mask._2d.status4 & BUF_NON_EMPTY) ||
          (draw_masks[lcv].mask._2d.dimx != dimx) ||
          (draw_masks[lcv].mask._2d.dimy != dimy))
        {free_buf_2d_rec(&(draw_masks[lcv].mask._2d));
         draw_masks[lcv].mask._2d = _2ds[lcv];
         draw_masks[lcv].paint_status &= ~PAINTED;}
    }
 c_free(_2ds);
 return NO_ERROR;
}


/*------------------------------------------------------------------------
init_draw_mask_value - ???
------------------------------------------------------------------------*/

int
init_draw_mask_value(win)
 struct image_display_window *win;
{
 if (win->intensity->color.total_colors >= CUR_CURVE_COLOR)
    win->data.draw_masks[DRAW_MASK_CUR_CURVE].pixel_value = 
           win->intensity->color.total_colors - CUR_CURVE_COLOR;
   else win->data.draw_masks[DRAW_MASK_CUR_CURVE].pixel_value = 0;

 if (win->intensity->color.total_colors > CUR_ROI_COLOR)
    win->data.draw_masks[DRAW_MASK_CUR_ROI].pixel_value = 
           win->intensity->color.total_colors - CUR_ROI_COLOR;
   else win->data.draw_masks[DRAW_MASK_CUR_ROI].pixel_value = 0;

 if (win->intensity->color.total_colors > OTHER_ROIS_COLOR)
    win->data.draw_masks[DRAW_MASK_OTHER_ROIS].pixel_value = 
           win->intensity->color.total_colors - OTHER_ROIS_COLOR;
   else win->data.draw_masks[DRAW_MASK_OTHER_ROIS].pixel_value = 0;
/*
 if (win->intensity->color.total_colors >= NON_MEMBER_ROIS_COLOR)
    win->data.draw_masks[DRAW_MASK_NON_MEMBER_ROIS].pixel_value = 
           win->intensity->color.total_colors - NON_MEMBER_ROIS_COLOR;
   else win->data.draw_masks[DRAW_MASK_NON_MEMBER_ROIS].pixel_value = 0;
*/
 if (win->intensity->color.total_colors >= PIXEL_SELECTION_COLOR)
    win->data.draw_masks[DRAW_MASK_PIXEL_SELECTION].pixel_value = 
           win->intensity->color.total_colors - PIXEL_SELECTION_COLOR;
   else win->data.draw_masks[DRAW_MASK_PIXEL_SELECTION].pixel_value = 0;

 return NO_ERROR;
}


/*------------------------------------------------------------------------
WinInitScreenBuffer - ???
------------------------------------------------------------------------*/

int
WinInitScreenBuffer(win,screen_depth)
 struct image_display_window *win;
 int screen_depth;
{
 struct screen_data_rec *screen;
 unsigned long mag_total_size;
 unsigned long mag_dimx,mag_dimy,dimx,dimy;
 int err;
 struct buf_2d_rec *_2d;
 struct color_data_rec *color_data;

 PRT3(printf("WinInitScreenBuffer...screen_depth = %d\n",screen_depth))
 screen = &(win->dwin->screen);
/*
 if ((color_data = WinCurColorData(win)) == NULL) ||
     (WinDefaultColorDataReady(win) != NO_ERROR))
*/
 if ((color_data = WinCurColorData(win)) == NULL)
    {PRT2(printf("WinInitScreenBuffer....BAD_DATA\n"))
     return BAD_DATA;}
 PRT3(printf("color_data gid = %d member id = %d\n",
         color_data->gid,color_data->member_id))
 _2d = &(color_data->_2d);
 dimx = screen->dimension.bound.right - screen->dimension.bound.left + 1;
 dimy = screen->dimension.bound.bottom - screen->dimension.bound.top + 1;
 PRT3(printf("num_draw_masks = %d _2d...dimx = %d dimy = %d\n",
      win->data.num_draw_masks,dimx,dimy))
 PRT3(printf("dimension.bound...\n"))
 PRT3(pr_int_bound_rec(&(screen->dimension.bound)))
 PRT3(pr_buf_2d_rec(_2d))
 if (win->data.num_draw_masks == 0)
    {
     win->data.num_draw_masks = INIT_TOTAL_DRAW_MASKS;
     if ((err = win_setup_mask_bufs(win,0,0,color_data->_2d.dimx,
                           color_data->_2d.dimy)) != NO_ERROR)
        {win->process_status4 |= WIN_SCREEN_SETUP_ERROR | WIN_SCREEN_SETUP_PROCESSED;
         return err;}

     init_draw_mask_value(win);
    }
 get_mag_image_size1(&mag_total_size,&mag_dimx,&mag_dimy,
                    dimx,dimy,screen_depth,
                    &(screen->pixel));

 PRT3(printf("total_bytes = %d,total_size = %d mag_dimx = %d mag_dimy = %d\n",
      screen->_2d.total_bytes,mag_total_size,mag_dimx,mag_dimy))
 PRT3(printf("WinInitScreenBuffer...mag_total_size = %d\n",mag_total_size))
 PRT3(printf("pixel_depth = %d screen_depth = %d\n",
      screen->_2d.pixel_depth,screen_depth))
 if (!(screen->_2d.status4 & BUF_NON_EMPTY ) ||
      (screen->_2d.total_bytes != mag_total_size) ||
      (screen->_2d.dimx != mag_dimx) ||
      (screen->_2d.dimy != mag_dimy) ||
      (screen->_2d.pixel_depth != screen_depth))
    {PRT3(printf("WinInitScreenBuffer ...screen->im = %u\n",screen->im))
     if (screen->im != 0)
        {screen->im->data = NULL;
         XDestroyImage(screen->im);
         screen->im = NULL;}
     PRT3(printf("WinInitScreenBuffer mag_dimx = %d mag_dimy = %d screen_depth = %d\n",
         mag_dimx,mag_dimy,screen_depth))
     if ((err = SetupBuf2dRec(&(screen->_2d),mag_dimx,mag_dimy,
               screen_depth,0)) != NO_ERROR)
        {
         PRT3(printf("WinInitScreenBuffer ...done..err = %d\n",err))
         win->process_status4 |= WIN_SCREEN_SETUP_ERROR | WIN_SCREEN_SETUP_PROCESSED;
         return err;}
     PRT3(printf("WinInitScreenBuffer ...done..err = %d\n",err))
    }
 win->process_status4 |= WIN_SCREEN_SETUP_PROCESSED;
 return NO_ERROR;
}



/*------------------------------------------------------------------------
setup_color_data_rec - ???
------------------------------------------------------------------------*/

int
setup_color_data_rec(win)
 struct image_display_window *win;
{
 int err;
 unsigned long pixel_depth;
 struct buf_2d_rec *cur_2d;
 struct color_data_rec *new_color_data;
 unsigned long lcv;

 PRT2(printf("setup_color_data_rec...wid = %s\n",win->owner->wid))
 if (win->intensity->color.total_colors <= 256) pixel_depth = 8;
   else if (win->intensity->color.total_colors <= 65535) pixel_depth = 16;
   else pixel_depth = 32;
 cur_2d = win_rec_cur_data_buf(win->owner,win->type2);
 PRT1(printf("num_color_datas = %d cur_2d = %u\n",
     win->data.num_color_datas,cur_2d))
 err = NO_ERROR;
 if (win->data.num_color_datas == 0)
    {setup_int_bound_rec(&(win->data.bound),0,0,cur_2d->dimx - 1,cur_2d->dimy - 1);
     if (win->status4 & WIN_POS_NEG_ACTIVATION)
        {if (win->status4 & WIN_POS_ACTIVATION)
            {PRT2(printf("WIN_POS_ACTIVATION...\n"))
             err = WinSetupColorDataRec2(win,0,0,0,0,0,0,
                    pixel_depth,0,&new_color_data,
                    COLOR_DATA_DEFAULT | COLOR_DATA_POS_ACTIVATION);}
         if ((err == NO_ERROR) && (win->status4 & WIN_NEG_ACTIVATION))
            {PRT2(printf("WIN_NEG_ACTIVATION...\n"))
             err = WinSetupColorDataRec2(win,0,0,1,0,0,0,
                    pixel_depth,0,&new_color_data,
                    COLOR_DATA_DEFAULT | COLOR_DATA_NEG_ACTIVATION);
            }
        }
       else err = WinSetupColorDataRec2(win,0,0,0,0,0,0,
             pixel_depth,0,&new_color_data,COLOR_DATA_DEFAULT);
    }
   else
    {PRT2(printf("setup_color_data_rec...num_color_datas = %d\n",
                    win->data.num_color_datas))
     for (lcv = 0; lcv < win->data.num_color_datas; lcv++)
        {if (win->data.color_datas[lcv]->status4 & COLOR_DATA_DEFAULT_AND_ID_VALID)
            if (!(win->data.color_datas[lcv]->_2d.status4 & BUF_NON_EMPTY) ||
                 (win->data.color_datas[lcv]->_2d.dimx != cur_2d->dimx) ||
                 (win->data.color_datas[lcv]->_2d.dimy != cur_2d->dimy) ||
                 (win->data.color_datas[lcv]->_2d.pixel_depth != pixel_depth))
            {
             PRT2(printf("gid = %d member_id = %d ...dimx = %d dimy = %d pixel_depth = %d\n",
                  win->data.color_datas[lcv]->gid,
                  win->data.color_datas[lcv]->member_id,
                  cur_2d->dimx,cur_2d->dimy,pixel_depth))
             if ((err = WinSetupColorDataRec2(win,
                   0,win->data.color_datas[lcv]->gid,
                   win->data.color_datas[lcv]->member_id,0,0,0,
                   pixel_depth,0,&new_color_data,0)) != NO_ERROR)
                break;}
         }
    }

 PRT2(printf("num_color_datas = %d...end err = %d\n",win->data.num_color_datas,err))
 return err;
}

