/****************************************************************************
display_canvas.c - Handle drawing code within each image canvas

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.


Handle drawing code within canvas of each image sub-window, and
event code: resize, repaint, button clicks, pass keys along to
event handler, scrolling.
****************************************************************************/

static const char rcsid[] = "$Id: display_canvas.c,v 1.9 1999/07/07 17:41:15 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/scrollbar.h>
#include <xview/xv_xrect.h>
#include <xview/cursor.h>
#include <xview/win_input.h>

#include "debug.h"
#include "mri_format.h"
#include "nria_procedures.h"


/* Functions defined in this file */
void init_frame_canvas_rec(struct frame_canvas_rec *frame_canvas);
int create_image(XImage **im, Display *dp, int screen_num,
    struct buf_2d_rec *_2d, unsigned long screen_depth);
int dshow_buf(struct display_win_rec *win1, int x, int y,
    int width, int height);
int resize_it_proc(Xv_Window canvas, int which);
int dp_resize_proc(Canvas canvas, int width, int height, Event *event);
int display_repaint_proc(Canvas canvas, Xv_Window xvwin, Display *dp,
    Window xwin, Xv_xrectlist *xrects);
int SelectCurve(struct image_display_window *win, Event *event,
    int is_select_curve, int is_beep, int *is_selected);
int process_cursor_proc(struct image_display_window *win,
    Xv_Window canvas_win, Event *event);
void process_event_proc(Xv_Window canvas_win, Event *event);
void set_dwin_init_frame_size_proc(Frame frame, int which);
Notify_value frame_interposer(Xv_Window frame, Event *event, Notify_arg arg,
    Notify_event_type type);
void update_after_kbd_done(Xv_Window canvas_win);
void update_after_loc_winexit(Xv_Window canvas_win);
int KeyEvent(Event *event);
int WinCanvas(struct image_display_window *win, int is_show_canvas);
int display_data_to_canvas1(struct image_display_window *win,
    int is_show_canvas);
int dwin_set_scrollbar(struct display_win_rec *win);
int horiz_scroll_proc(Panel_item item, int value, Event *event);
int vertical_scroll_proc(Panel_item item, int value, Event *event);
int dsetup_win_canvas_scrollbar(struct display_win_rec *win);
int check_dwin_frame_size(struct display_win_rec *dwin);
void check_dwin_frame_size_proc(struct display_win_rec *dwin, int which);
int dsetup_frame_canvas_rec(struct display_win_rec *win, int is_show_canvas);



/*-------------------------------------------------------------------------
init_frame_canvas_rec - ???
-------------------------------------------------------------------------*/

void
init_frame_canvas_rec(frame_canvas)
 struct frame_canvas_rec *frame_canvas;
{
 frame_canvas->canvas= 0;
 frame_canvas->height = 0;
 frame_canvas->width = 0;
 frame_canvas->pixel_depth = 8;  /* 1 byte */
 frame_canvas->hbar = 0;
 frame_canvas->vbar = 0;
}


/*-------------------------------------------------------------------------
create_image - ???
-------------------------------------------------------------------------*/

int
create_image(im,dp,screen_num,_2d,screen_depth)
 XImage **im;
 Display *dp;
 int screen_num;
 struct buf_2d_rec *_2d;
 unsigned long screen_depth;
{
 int bytes;
 Visual *visual;

 bytes = GetRowBytes(_2d->dimx,_2d->pixel_depth);
 PRT1(printf("XCreateImage..1.depth = %d bytes = %d\n",
            _2d->pixel_depth,bytes))
 visual = DefaultVisual(dp,screen_num);
 PRT1(printf("create_image..1.depth = %d bytes = %d screen_depth = %d\n",
            _2d->pixel_depth,bytes,screen_depth))
 if ((*im = XCreateImage(dp,visual,_2d->pixel_depth,
            ZPixmap,0,(char *)_2d->buf[0],_2d->dimx,
            _2d->dimy,screen_depth,bytes)) == NULL)
    return ~NO_ERROR;
 (*im)->bitmap_unit = screen_depth;
 (*im)->bits_per_pixel = screen_depth;
 PRT1(printf("XC w = %d h = %d dep = %d bu = %d pad = %d\n",
            (*im)->width,(*im)->height,(*im)->depth,
            (*im)->bitmap_unit,(*im)->bitmap_pad);
      printf("off = %d bpl = %d bpp = %d\n",
            (*im)->xoffset,
           (*im)->bytes_per_line,(*im)->bits_per_pixel))

 return NO_ERROR;
}


/*-------------------------------------------------------------------------
dshow_buf - ???
-------------------------------------------------------------------------*/

int
dshow_buf(win1,x,y,width,height)
 struct display_win_rec *win1;
 int x,y,width,height;
{
 struct image_display_window *win;
 static int is_clear;
 int right,bottom;

 if (win1->screen.im == NULL)
    {
     create_image(&(win1->screen.im),win1->dp.dp,win1->dp.screen_num,
           &(win1->screen._2d),win1->screen._2d.pixel_depth);
    }
 if (win1->screen.im != NULL)
    {PRT2(printf("dshow_buf...x = %d y = %d w  = %d h = %d\n",x,y,width,height))
     user_write(win1->frame_canvas.canvas,x,y,width,height,GXcopy,
              win1->screen.im,x,y);
     win = win1->owner;
     if ((win->control.sroi.sroi_win->slice_roi.status & 
                                     SLICE_ROI_TO_BE_SAVED) &&
        !(win->control.sroi.sroi_win->slice_roi.status &
                       SLICE_ROI_SHOW_EDITED))
        win_execute_proc_to_related_wins(win,win_show_edited,0,0);
     right = x + width - 1;
     bottom = y + height - 1;
     if (is_pt_in_area(x,y,right,bottom,
            win1->label_area.left,win1->label_area.top) ||
         is_pt_in_area(x,y,right,bottom,
            win1->label_area.left + win1->label_area.width - 1,
            win1->label_area.top + win1->label_area.height - 1))
        {
         is_clear = 1;
         win_draw_enabled_label(win,&is_clear);}
     if ((win->control.display_options & DISPLAY_ACTION_LABEL) &&
         (win->control.status & ACTION_LABEL_ON))
        {
         if (is_pt_in_area(x,y,right,bottom,
             win1->action_label_area.left,win1->action_label_area.top) ||
            is_pt_in_area(x,y,right,bottom,
             win1->action_label_area.left + win1->action_label_area.width - 1,
             win1->action_label_area.top + win1->action_label_area.height - 1))
            win_action_label(win);
        }
     return NO_ERROR;}
   else return ~NO_ERROR;
}


/*-------------------------------------------------------------------------
resize_it_proc - ???
-------------------------------------------------------------------------*/

int
resize_it_proc(canvas,which)
 Xv_Window canvas;
 int which;
{
 struct display_win_rec *win1;
 struct image_display_window *win;
 int err;

 (void) notify_set_itimer_func((Notify_client) canvas,
             NOTIFY_FUNC_NULL,ITIMER_REAL,
             (struct itimerval *) 0,(struct itimerval *) 0);
 win1 = (struct display_win_rec *) xv_get(canvas_paint_window(canvas),
                              WIN_CLIENT_DATA);
 win = win1->owner;
 win->dwin = win1;

 err = WinColorCanvas(win);
 return err;
}


/*-------------------------------------------------------------------------
dp_resize_proc - ???
-------------------------------------------------------------------------*/

int
dp_resize_proc(canvas,width,height,event)
 Canvas canvas;
 int width,height;
 Event *event;
{ 
 struct display_win_rec *win1;
 struct image_display_window *win;
 struct int_region_rect last;
 int modifier,right,bottom;
 int fixed_bound_opr;

 win1 = (struct display_win_rec *) xv_get(canvas_paint_window(canvas),
                              WIN_CLIENT_DATA);
 win = win1->owner;
 win->dwin = win1;
 modifier = 0;
 last = win1->screen.dimension;
 right = win1->screen.dimension.limitboundrect.left + 
         win1->screen.dimension.limitboundrect.width - 1;
 bottom = win1->screen.dimension.limitboundrect.top + 
         win1->screen.dimension.limitboundrect.height - 1;

 PRT3(printf("...dp_resize_proc....rr.width = %d h = %d  width = %d h = %d\n",
              win1->screen.dimension.rr.width,
             win1->screen.dimension.rr.height,width,height))
 height -= xv_get(win1->frame_canvas.canvas,XV_Y);
 width -= xv_get(win1->frame_canvas.canvas,XV_X);
 PRT3(printf("...after....width = %d h = %d\n",
             width,height))
 PRT3(printf("canvas x = %d y = %d w = %d h = %d\n",
        xv_get(win1->frame_canvas.canvas,XV_X),
        xv_get(win1->frame_canvas.canvas,XV_Y),
        xv_get(canvas_paint_window(win1->frame_canvas.canvas),XV_WIDTH),
        xv_get(canvas_paint_window(win1->frame_canvas.canvas),XV_HEIGHT)))

 if ((win1->screen.dimension.rr.width <= width) ||
     (win1->screen.dimension.rr.height <= height))
    fixed_bound_opr = 0;
   else
    {PRT3(printf("dp_resize_proc...DIM_SHRINK = %d\n",DIM_SHRINK))
     fixed_bound_opr = DIM_SHRINK;}

 win1->screen.dimension.rr.width = width;
 win1->screen.dimension.rr.height = height;

 if ((win1->screen.dimension.bound.left ==    
      win1->screen.dimension.limitboundrect.left) &&
     (win1->screen.dimension.bound.right == right) &&
     (win1->screen.dimension.bound.top == 
      win1->screen.dimension.limitboundrect.top) &&
     (win1->screen.dimension.bound.bottom == bottom))
    {PRT3(printf("dp_resize_proc...DIM_FIXED_BOUND\n"))
     setup_screen_dimension(&(win->dwin->screen.dimension),
            &(win->dwin->screen.last_dimension),
            &(win->dwin->screen.pixel),&(win->dwin->screen.last_pixel),
             DIM_FIXED_BOUND,4 | fixed_bound_opr,0,0);
    }
   else
    {PRT3(printf("dp_resize_proc...DIM_FIXED_MAG\n"))
     if (((win1->screen.dimension.bound.left ==    
           win1->screen.dimension.limitboundrect.left) &&
         (win1->screen.dimension.bound.right == right)) ||
        ((win1->screen.dimension.bound.top == 
           win1->screen.dimension.limitboundrect.top) &&
         (win1->screen.dimension.bound.bottom == bottom)))
       {PRT3(printf("dp_resize_proc...3\n"))
        if (win->dwin->screen.dimension.magrect.left < MAX_MAGNIFY_PIXEL_DIM)
           win->dwin->screen.dimension.magrect.left += 1;
        if (win->dwin->screen.dimension.magrect.top < MAX_MAGNIFY_PIXEL_DIM)
           win->dwin->screen.dimension.magrect.top += 1;
       }
     setup_screen_dimension(&(win->dwin->screen.dimension),
            &(win->dwin->screen.last_dimension),
            &(win->dwin->screen.pixel),&(win->dwin->screen.last_pixel),
             DIM_FIXED_MAG,DIM_CENTER_AT_POINT | DIM_OPR_SAME_DIM_MAG_FACTOR,
             bound_centerx(&(win1->screen.dimension.bound)),
             bound_centery(&(win1->screen.dimension.bound)));
    }
 win1->screen.last_dimension = last;
 win1->resize_status += 1;
 PRT3(pr_int_region_rect(&(win->dwin->screen.dimension));)
 (void) notify_set_itimer_func((Notify_client) win1->frame_canvas.canvas,
             (Notify_func) resize_it_proc,ITIMER_REAL,
             &NOTIFY_POLLING_ITIMER,(struct itimerval *) 0);
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
display_repaint_proc - ???
-------------------------------------------------------------------------*/

int
display_repaint_proc(canvas,xvwin,dp,xwin,xrects)
 Canvas canvas;
 Xv_Window xvwin;
 Display *dp;
 Window xwin;
 Xv_xrectlist *xrects;
{
 struct display_win_rec *win1;
 struct image_display_window *win;
 int x,y,width,height;
 static int is_clear;

 win1 = (struct display_win_rec *) xv_get(canvas_paint_window(canvas),
                              WIN_CLIENT_DATA);
 win = win1->owner;
 win->dwin = win1;

 if (win1->screen.im == NULL)
    {
     create_image(&(win1->screen.im),win1->dp.dp,win1->dp.screen_num,
           &(win1->screen._2d),win1->screen._2d.pixel_depth);
    }
 if (win1->screen.im != NULL)
    {
     if ((xrects == NULL) || (win1->display_status & WIN_NOT_YET_PAINTED))
        {win1->display_status &= ~WIN_NOT_YET_PAINTED;
         user_write(win1->frame_canvas.canvas,0,0,
           win1->screen._2d.dimx,win1->screen._2d.dimy,GXcopy,
                 win1->screen.im,0,0);
        }
       else
        {
         for (xrects->count--; xrects->count >= 0; xrects->count--)
            {
             x = xrects->rect_array[xrects->count].x;
             y = xrects->rect_array[xrects->count].y;
             width = xrects->rect_array[xrects->count].width;
             height = xrects->rect_array[xrects->count].height;
             user_write(win1->frame_canvas.canvas,x,y,width,height,GXcopy,
                 win1->screen.im,x,y);
            }
        }
    }
   else 
    {
     return ~NO_ERROR;}
 is_clear = 1;
 win_draw_enabled_label(win,&is_clear);
 return NO_ERROR;
}

/******************
 if (eve_action == ACTION_AGAIN)
    {
     if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return;
     wr_2d("_2d.dat",&(img->_2d));
     return;
    }
 if (eve_action == ACTION_UNDO)
    {
     if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return;
     rd_2d("_2d.dat",&(img->_2d));
     WinNewColorMultiCanvas(win);
     return;
    }
********************/


/*-------------------------------------------------------------------------
SelectCurve - ???
-------------------------------------------------------------------------*/

int
SelectCurve(win,event,is_select_curve,is_beep,is_selected)
 struct image_display_window *win;
 Event *event;
 int is_select_curve,*is_selected;
{
 int err,use_this_xybound;
 struct int_bound_rec xybound;
 double minx,maxx,miny,maxy;
 long x,y;

 if ((win->control.which_cursor == CURSOR_DEFAULT) ||
     is_select_curve ||
     (!event_meta_is_down(event) &&
      event_shift_is_down(event)))
    {
     if (event_left_is_down(event))
        {/*printf("select_curve event_left_is_down\n");*/
         if ((err = win_get_cursor_xy_and_bound(win,event,&x,&y,
               &use_this_xybound,&xybound)) != NO_ERROR)
            return err;
         if (event_ctrl_is_down(event))
            FindSliceRoiMinMax(&(win->control.sroi.sroi_win->slice_roi),
                   win->control.sroi.localto,&minx,&maxx,&miny,&maxy,0,0,1);

         if (!win_search_enclosing_curve(win,x,y,use_this_xybound,&xybound))
            {if (!is_select_curve || is_beep)
                xv_set(win->dwin->frame,WIN_ALARM,0);
             *is_selected = ~NO_ERROR;}
           else *is_selected = NO_ERROR;
         win_display_row_col(win,x,y,1,1);}
        else *is_selected = ~NO_ERROR;
     return NO_ERROR;
    }
 return CANCEL;
}



/*-------------------------------------------------------------------------
process_cursor_proc - ???
-------------------------------------------------------------------------*/

int
process_cursor_proc( win, canvas_win, event )
     struct image_display_window *win;
     Xv_Window canvas_win;
     Event *event;
{
  long x,y;
  int err;

  if (event_is_button(event))
  {
    err = NO_ERROR;

    switch (win->control.which_cursor) {

    case CURSOR_DELETE_ROI:
      if (event_is_up(event)) 
	delete_curve_mode( NULL, NULL, win->dwin, canvas_win, event );
      break;

    case CURSOR_MAGNIFY_PLUS:
      if (event_is_up(event)) 
	magnify_pixel_option_proc(0,0,win->dwin,
				  canvas_win,event,MAGNIFY_WIN);
      break;

    case CURSOR_MAGNIFY_MINUS:
      if (event_is_up(event)) 
	magnify_pixel_option_proc(0,0,win->dwin,
				  canvas_win,event,MAGNIFY_SHRINK_WIN);
      break;

    case CURSOR_PT_SCAN:
      if (event_is_up(event)) 
	pt_to_region_proc(0,0,win->dwin,canvas_win,event,1);
      break;

    case CURSOR_LOCATE_PLANE:
      if (event_is_up(event)) 
	win_3d_locate_plane(win,event);
      break;

    default:
      err = CANCEL;
      break;
    }

    if (err == NO_ERROR)
    {
      if ((err = dwin_get_coord_xy_from_event(win->dwin,event,&x,&y))
	  != NO_ERROR)
	return err;
      win_display_row_col(win,x,y,1,1);
      return NO_ERROR;
    }
  }


  if ((win->control.which_cursor == CURSOR_MOVE_ROI) ||
      (win->control.which_cursor == CURSOR_RESIZE_ROI) ||
      (win->control.which_cursor == CURSOR_ROTATE_ROI) ||
      (win->control.which_cursor == CURSOR_MOVE_SUPERIMPOSE))
  {
    if ((err = trans_roi_proc(NULL,NULL,win->dwin,canvas_win,event))
	!= NO_ERROR)
      return err;
    if ((err = dwin_get_coord_xy_from_event(win->dwin,event,&x,&y))
	!= NO_ERROR)
      return err;
    win_display_row_col(win,x,y,1,1);
    return NO_ERROR;
  }

  /*printf("process_cursor_proc...CANCEL\n");fflush(stdout);*/
  return CANCEL;
}



/*-------------------------------------------------------------------------
process_event_proc - ???

Process Canvas events in image sub-windows
-------------------------------------------------------------------------*/

void
process_event_proc(canvas_win,event)
     Xv_Window canvas_win;
     Event *event;
{
  struct display_win_rec *win1;
  struct image_display_window *win;
  int eve_action;
  long x,y;
  int err,is_selected;

  TRACE(printf("\nIn process_event_proc(): "););
  win1 = (struct display_win_rec *) xv_get(canvas_win,WIN_CLIENT_DATA);
  TRACE(printf(" win = %p, ",(void *) win1););
  win = win1->owner;
  win->dwin = win1;

  eve_action = event_action(event);
  TRACE(printf(" eve_action = %04X: ",eve_action));


  if (eve_action == LOC_MOVE)
  {
    TRACE(printf("localizer moved\n"););
    process_loc_move(canvas_win,event,1);
    return /*NO_ERROR*/;
  }

  if (event_is_button(event))
  {
    if (event_ctrl_is_down(event))
    {
      TRACE(printf(" CTRL "););

      if (!event_meta_is_down(event))
      {
	if (win->type2 & DISPLAY_WINDOW_3D)
	{
	  if (event_left_is_down(event) && event_is_up(event))
	  {
	    TRACE(printf(" LMB_down, up event (??) "););
	    /* FIXME!  Removed a final argument "0" from the call below
	       without being really sure that it's right.  -PWH 19990607 */
	    _3d_move_mouse_to_wid(NULL,NULL,win->dwin,canvas_win,event/*,0*/);
	  }
	  TRACE(printf(" end.\n"););
	  return /*NO_ERROR*/;
	}
      }
      else if (event_shift_is_down(event))
      {
	TRACE(printf(" SHIFT-META "););

	if ((err = contrast_mouse_proc(win,event)) == NO_ERROR)
	{
	  TRACE(printf(" end.\n"););
	  return /*NO_ERROR*/;
	}
      }
    } /* ctrl_is_down ? */
    else if (event_meta_is_down(event) && !event_shift_is_down(event))
    {
      TRACE(printf(" META "););	
      if (event_left_is_down(event)) 
      {
	TRACE(printf(" LMB_down "););

	/* FIXME!!  The following call is not at all consistent with
	   the definition of slave_win_proc, but I don't have time
	   to figure out the bug right now.  -PWH 19990607 */
	if ((err = slave_win_proc(NULL,NULL,win->dwin,canvas_win,
				  event,0))  == NO_ERROR)
	{
	  TRACE(printf(" end.\n"););
	  return /*NO_ERROR*/;
	}
      }
      else
      {
	TRACE(printf(" LMB_up "););
	if (event_is_up(event))
	{
	  TRACE(printf(" up_event "););
	  TRACE(printf(" process_event_proc() about to end...\n"););
	  TRACE(printf("  (after calling magnify_pixel_option_proc())\n"););
	  /* Added last argument below as a dummy.  -PWH 19990226 */
	  magnify_pixel_option_proc(0,0,win->dwin, canvas_win, event, 0);
	  return;
	}
	TRACE(printf(" end.\n"););
	return /*NO_ERROR*/;
      }
    }
  } /* is button? */

  if (win->control.which_cursor &&
      (win->control.which_cursor != CURSOR_PEN))
  {
    if (process_cursor_proc(win,canvas_win,event) == NO_ERROR)
    {
      TRACE(printf(" end.\n"););
      return /*NO_ERROR*/;
    }
  }

  if (event_is_button(event))
  {
    if (event_middle_is_down(event))
    {
      TRACE(printf(" MMB_down "););

      if (event_meta_is_down(event))
      {
	TRACE(printf(" META  end.\n"););
	return /*NO_ERROR*/;
      }

      if ((win->control.which_cursor == CURSOR_PEN) &&
	  !event_shift_is_down(event))
      {
	PRT2(printf("\n	win_show_sub_list_roi_labels ?\n"));
	/*  DRW 961115: changed up to down; easier to select region
	 *    from list of labels by dragging into menu this way.  */
	if (event_is_down(event))
	{
	  TRACE(printf(" yes: event_is_down. "););
	  win_show_sub_list_roi_labels(win,0);
	}
	else
	{
	  PRT2(printf("	no: event_is_up. "));
	}
	TRACE(printf(" end.\n"););
	return /*NO_ERROR*/;
      }

      if ((err = dwin_get_coord_xy_from_event(win->dwin,event,&x,&y))
	  != NO_ERROR)
      {
	TRACE(printf("	process_event_proc() end, with error.\n"););
	return /*err*/;
      }

      if (!event_shift_is_down(event) && !KeyEvent(event) &&
	  !event_meta_is_down(event) && !event_ctrl_is_down(event)) 
      {
	PRT(printf(" --> win_show_pixel_value() "));
	if (event_is_up(event))
	{
	  win_show_pixel_value(win,event,x,y);
	}
      }
      else
	win_display_row_col(win,x,y,1,1);

      TRACE(printf(" end.\n"););
      return /*NO_ERROR*/;
    }

    if (win->type2 & EDITING_WINDOW)
    {
      if (SelectCurve(win,event,0,1,&is_selected) != CANCEL)
      {
	TRACE(printf(" end.\n"););
	return /*NO_ERROR*/;
      }
    }

    if (win->control.which_cursor == CURSOR_DEFAULT)
    {
      TRACE(printf(" end.\n"););
      return /*NO_ERROR*/;
    }
  }

  if (KeyEvent(event) || event_meta_is_down(event) ||
      (eve_action == ACTION_INSERT) ||
      (eve_action == ACTION_ERASE_CHAR_BACKWARD) ||
      (event_ctrl_is_down(event) && !event_is_button(event)))
  {
    if (event_is_up(event))
    {
      TRACE(printf(" end.\n"););
      return;
    }

    if ((err = process_ascii_event(canvas_win,event)) == NO_ERROR)
    {
      TRACE(printf(" end.\n"););
      return /*NO_ERROR*/;
    }
  }

  if (!(win->type2 & EDITING_WINDOW))
  {
    TRACE(printf(" end.\n"););
    return /*NO_ERROR*/;
  }
  else if (!(win->control.status & DRAW_ENABLED))
  {
    TRACE(printf(" end.\n"););
    return /*NO_ERROR*/;
  }

  /*printf("process_draw_proc...\n");fflush(stdout);*/
  TRACE(printf("process_event_proc() about to end.\n"););
  TRACE(printf("  (after calling process_draw_proc())\n"););
  process_draw_proc(canvas_win,event);
  return;
}



/*-------------------------------------------------------------------------
set_dwin_init_frame_size_proc - ???
-------------------------------------------------------------------------*/

void
set_dwin_init_frame_size_proc(frame,which)
 Frame frame;
 int which;
{
 struct display_win_rec *win;
 int width,height;

 PRT3(printf("set_dwin_init_frame_size_proc...1\n"))
 (void) notify_set_itimer_func((Notify_client) frame,
             NOTIFY_FUNC_NULL,ITIMER_REAL,
             (struct itimerval *) 0,(struct itimerval *) 0);
 win = (struct display_win_rec *) xv_get(frame,WIN_CLIENT_DATA);
             width = (int) xv_get(frame,XV_WIDTH);
             height = (int) xv_get(frame,XV_HEIGHT);
 PRT3(printf("frame = %u w = %d  h = %d last_width = %d last_height = %d\n",frame,
                 width,height,win->last_frame_width,win->last_frame_height);)
 xv_set(win->frame,XV_HEIGHT,win->last_frame_height,
                                XV_WIDTH,win->last_frame_width,0);
 win->status2 &= ~INIT_WINDOW_CHECK_ON;
 win->status2 &= ~INIT_WINDOW;
 PRT3(printf("set_dwin_init_frame_size_proc...end\n"))
}


/*
set_dwin_resize_status_proc(frame,which)
 Frame frame;
 int which;
{
 struct display_win_rec *win;

 PRT3(printf("set_dwin_resize_status_proc...1\n"))
 (void) notify_set_itimer_func((Notify_client) frame,
             NOTIFY_FUNC_NULL,ITIMER_REAL,
             (struct itimerval *) 0,(struct itimerval *) 0);
 win = (struct display_win_rec *) xv_get(frame,WIN_CLIENT_DATA);
 win->resize_status = 0;
}
*/
/*
                 if (win->status2 & INIT_WINDOW)
                    {PRT3(printf("INIT_WINDOW...win->status = %x\n",
                                  (unsigned long) win->status))
                     if (!(win->status2 & INIT_WINDOW_CHECK_ON))
                        {win->status2 |= INIT_WINDOW_CHECK_ON;
                         (void) notify_set_itimer_func((Notify_client) frame,
                             (Notify_func) set_dwin_init_frame_size_proc,
                                ITIMER_REAL,&NOTIFY_POLLING_ITIMER,
                               (struct itimerval *) 0);
                        }
                       PRT3(else printf("INIT_WINDOW...set\n"))
                    }
                   else
                    {PRT3(printf("dp_resize_proc...\n"))
                     dp_resize_proc(win->frame_canvas.canvas,width,height,event);
                    }
*/




/*-------------------------------------------------------------------------
frame_interposer - ???
-------------------------------------------------------------------------*/

Notify_value
frame_interposer( frame, event, arg, type )
     Xv_Window frame;
     Event *event;
     Notify_arg arg;
     Notify_event_type type;
{
  struct display_win_rec *win;
  int width,height;
  Rect *rr,*rr1;


#if 0
  PRT(printf("...frame_interposer = %u  eve = %d\n",
	     frame,event_action(event)););
#endif

/*
 PRT3(if (event_shift_is_down(event)) printf("event_shift_is_down...\n");)
 PRT3(if (event_shiftmask(event) & SHIFTMASK) printf("SHIFTMASK...\n");)
 if (event_shiftmask(event) & SHIFTMASK) printf("SHIFTMASK...\n");
 if (event_shiftmask(event) & CTRLMASK) printf("CTRLMASK...\n");
 if (event_shiftmask(event) & ALTMASK) printf("ALTMASK...\n");
*/

  if (event_action(event) == WIN_RESIZE)
  {
    win = (struct display_win_rec *) xv_get(frame,WIN_CLIENT_DATA);
#if 0
    PRT(if (event_is_down(event))
	  printf("...frame_interposer..event_is_down\n");
	else
	  printf("...frame_interposer..event_is_up\n"));
#endif

    if (win != NULL)
    {
      if (win->resize_status)
      {
	/* PRT3(printf("resize_status = %d\n",win->resize_status)); */
	win->resize_status -= 1;
      }
      else if (xv_get(frame,WIN_MAP) && (win->frame_canvas.canvas != 0))
      {
	if (win->status2 & INIT_WINDOW)
	{
	  frame_get_rect(frame,&(win->win_rect));
	  win->status2 &= ~INIT_WINDOW;
	  win->status2 |= WINDOW_WIN_RECT_VALID;
	  /* PRT3(printf("frame_interposer =INIT_WINDOW l = %d t = %d",
	     win->win_rect.r_left,win->win_rect.r_top );
	     printf(" w = %d h = %d\n",
	     win->win_rect.r_width,win->win_rect.r_height);); */
	}
	else
	{
	  /* PRT3( frame_get_rect(frame,&rr2);
	     printf("frame_interposer =WIN_MAP l = %d t = %d w = %d h = %d\n",
	     rr2.r_left,rr2.r_top,rr2.r_width,rr2.r_height);); */
	}

	width = (int) xv_get(frame,XV_WIDTH);
	height = (int) xv_get(frame,XV_HEIGHT);
	/* PRT3(printf("frame=%u w=%d h=%d last_width=%d last_height=%d\n",
	   frame,
	   width,height,win->last_frame_width,win->last_frame_height););*/

	if ((width != win->last_frame_width) || 
	    (height != win->last_frame_height))
	{
	  dp_resize_proc(win->frame_canvas.canvas,width,height,event);
	}
	else
	{
	  /* PRT3(frame_get_rect(frame,&rr2);
	     printf("frame_interposer =MOVE_WINDOW l=%d t=%d w=%d h=%d\n",
	     rr2.r_left,rr2.r_top,rr2.r_width,rr2.r_height);); */

	  win->status2 |= MOVE_WINDOW;
	}
      }
      else
      {
	/* PRT(frame_get_rect(frame,&rr2);
	   printf("frame_interposer =NOT WIN_MAP l=%d t=%d w=%d h=%d\n",
	   rr2.r_left,rr2.r_top,rr2.r_width,rr2.r_height);); */
	rr = (Rect *) xv_get(frame,WIN_MOUSE_XY);
	rr1 = (Rect *) xv_get(frame,XV_RECT);
	/* PRT(printf("frame l = %d t = %d\n",rr->r_left,rr->r_top);
	   printf("frame l = %d t = %d r = %d b = %d\n",
	   rr1->r_left,rr1->r_top,rr1->r_left + rr1->r_width - 1,
	   rr1->r_top + rr1->r_height - 1);); */
	if (pt_in_Rect(rr1,(int) rr->r_left,(int) rr->r_top))
	{
#if 0
	  /*  DRW 970225  BUG: Grabbing focus when resizing frame
	      screws up Window Manager big time (Solaris 2.x).  */
	  grap_keyboard(canvas_paint_window(win->frame_canvas.canvas));
#endif
	}
	/* PRT(printf("frame_interposer end\n\n");); */
      }
    }

    return NOTIFY_DONE;

  }  /*  End of Event is resize.  */

  /*  Event is NOT resize.  */
  else if (event_action(event) == LOC_WINENTER)
  {

    /* PRT(printf("frame_interposer grab keyboard focus...2\n")); */
    if (((win = (struct display_win_rec *) 
	  xv_get(frame,WIN_CLIENT_DATA)) != NULL) &&
	(win->frame_canvas.canvas != 0))
    {
#if 0
      /*  DRW 970225  BUG: Grabbing focus when entering frame
	  screws up Window Manager big time (Solaris 2.x).  */
      grap_keyboard(canvas_paint_window(win->frame_canvas.canvas));
#endif
    }
    return NOTIFY_DONE;
  }

  /* PRT(printf("frame_interposer---1 end\n\n");); */
  return notify_next_event_func((Notify_client) frame,
				(Notify_event) event,arg,type);
}


/*-------------------------------------------------------------------------
update_after_kbd_done - ???
-------------------------------------------------------------------------*/

void
update_after_kbd_done(canvas_win)
 Xv_Window canvas_win;
{
 struct display_win_rec *dwin;
 struct image_display_window *win;

 dwin = (struct display_win_rec *) xv_get(canvas_win,WIN_CLIENT_DATA);
 win = dwin->owner;
 win->dwin = dwin;

 win_update_trans_roi_rec(win);
}


/*-------------------------------------------------------------------------
update_after_loc_winexit - ???
-------------------------------------------------------------------------*/

void
update_after_loc_winexit(canvas_win)
 Xv_Window canvas_win;
{
 struct display_win_rec *dwin;
 struct image_display_window *win;

 dwin = (struct display_win_rec *) xv_get(canvas_win,WIN_CLIENT_DATA);
 win = dwin->owner;
 win->dwin = dwin;
 win_update_trans_roi_rec(win);
}


/*-------------------------------------------------------------------------
KeyEvent - ???
-------------------------------------------------------------------------*/

int
KeyEvent(event)
 Event *event;
{
 if (event_is_ascii(event) || event_is_iso(event) ||
     event_is_key_left(event) || event_is_key_right(event) ||
     event_is_key_top(event) ||  event_is_key_bottom(event) ||
     event_is_key_top(event))
    return 1;
   else return 0;
}



/*-------------------------------------------------------------------------
canvas_interposer - ???
-------------------------------------------------------------------------*/

static Notify_value
canvas_interposer( canvas_win, event, arg, type )
     Xv_Window canvas_win;
     Event *event;
     Notify_arg arg;
     Notify_event_type type;
{
  struct display_win_rec *win;

  /*  DRW 970226: Bug(?) fix(?): call base event handler on KBD_USE and
   *  LOC_WINENTER */
  Notify_value next_value;

#if 0  
  PRT(if (event_action(event) != LOC_MOVE)
      printf("canvas_interposer():\n	event_action = %04X\n",
	     event_action(event)); );
#endif

  if (canvas_win == 0)
  {
    TRACE(printf("	canvas_win is NULL. "););
    TRACE(printf("canvas_interposer() end.\n"););
    return NOTIFY_DONE;
  }

  win = (struct display_win_rec *)
    xv_get(canvas_win,WIN_CLIENT_DATA);

  if (win->status2 & MOVE_WINDOW)
  {
    /* PRT(printf("\ncanvas_interposer =MOVE_WINDOW = %x\n",
	 (unsigned long) win->status2);); */
    if (event_shift_is_down(event) && 
	(win->owner->list.next != &(win->owner->list)))
    {
      win->owner->dwin = win;
      WinRelativeMoveListWindows(win->owner,WIN_LIST);
      TRACE(printf("canvas_interposer() end.\n"););
      return NOTIFY_DONE;
    }
    else
    {
      frame_get_rect(win->frame,&(win->win_rect));
      win->status2 |= WINDOW_WIN_RECT_VALID;
      win->status2 &= ~MOVE_WINDOW;
      /* PRT(printf("canvas_interposer =MOVE_WINDOW continue\n")); */
    }
  }

#if 0
  if (event_is_button(event))
  {
    TRACE(printf("	Button Event:\n"););
    if (event_is_down(event))
    {
      TRACE(printf("		Down, "););
    }
    else
    {
      TRACE(printf("		Not Down, "););
    }

    if (event_is_up(event))
    {
      TRACE(printf("Up, "););
    }
    else
    {
      TRACE(printf("Not Up, "););
    }

    if ((event_action(event) == ACTION_MENU) && event_is_down(event))
    {
      TRACE(printf("RMB_down, "););
    }
    else if (event_action(event) == ACTION_MENU)
    {
      TRACE(printf("RMB_up, "););
    }

    if (event_meta_is_down(event))
    {
      TRACE(printf("META_down, "););
    }
    else
    {
      TRACE(printf("META_up, "););
    }
  }
  else if (KeyEvent(event))
  {
    TRACE(printf("	KeyEvent."););
  }
  else
  {
    TRACE(printf("	Neither KeyEvent nor ButtonEvent"););
  }
  TRACE(printf("\n"););
#endif

  /*  DRW 961115:  Bug workaround (fix?): replaced event_right_is_down with
   *    ((event_action(event) == ACTION_MENU).
   *    event_right_is_down does not catch a RMB down event, for
   *    some reason.  Also replaced "!KeyEvent" with "event_is_button" */
  if ((event_action(event) == ACTION_MENU) &&
      !event_meta_is_down(event) &&
      event_is_button(event))
  {
    TRACE(printf("\n Action_Menu, META_up, Button_Event\n"););
    if ((win->owner->control.which_cursor != CURSOR_DEFAULT) &&
	(win->owner->control.which_cursor != CURSOR_PEN))
    {
      TRACE(printf(" Ignoring (--> notify_next_event_func())\n"););
#if 0
      TRACE(printf("canvas_interposer() end.\n"););
#endif
      return notify_next_event_func( (Notify_client) canvas_win,
				     (Notify_event) event,
				     arg,
				     type );
    }

    if (win != NULL)
    {

      if (event_ctrl_is_down(event))
      {
	TRACE(printf("	CTRL_down\n"););
	if (win->owner->type2 & DISPLAY_WINDOW)
	  change_window_channel_proc(win,canvas_win,event);
      }

      else
      {
	TRACE(printf("	CTRL_up\n"););
	win->owner->dwin = win;
	PRT2(printf("	win_show_sub_list_roi_labels ?\n"));
	/*  DRW 961115: changed up to down; easier to select region
	 *    from list of labels by dragging into menu this way.  */
	if (event_is_down(event)) 
	  win_show_sub_list_roi_labels(win->owner,1);
	else
	{
	  PRT2(printf("	no, because event_is_up\n"));
	}
      }

    }
#if 0
      else if (win->menu != NULL)
      {
	TRACE(printf("canvas_interposer...analyze_image_proc..\n"));
	show_roi_menu_proc(NULL,NULL,win,canvas_win,event);
	TRACE(printf("...analyze_image_proc..1\n"))
      }
#endif
#if 0
    TRACE(printf("canvas_interposer() end.\n"););
#endif
    return NOTIFY_DONE;
  }

  else
  {
    /*      PRT(printf("canvas_interposer...***\n")); */
    if ((event_action(event) == KBD_USE) ||
	(event_action(event) == LOC_WINENTER))
    {
      TRACE(printf(" Keyboard_Use / Loc_WinEnter "););
#if 0
      grap_keyboard(canvas_win);
#endif
#if 0
      TRACE(printf("canvas_interposer() end.\n"););
#endif

      /*  DRW 970226: Bug(?) fix(?): call base event handler on KBD_USE and
       *  LOC_WINENTER */
      next_value = notify_next_event_func( (Notify_client) canvas_win,
					   (Notify_event) event,
					   arg,
					   type );
      return next_value;
    }
    else if (event_action(event) == KBD_DONE)
    {
      TRACE(printf(" Keyboard_Done "););
      update_after_kbd_done(canvas_win);
#if 0
      TRACE(printf("canvas_interposer() end.\n"););
#endif
      return NOTIFY_DONE;
    }
    else if (event_action(event) == LOC_WINEXIT)
    {
      TRACE(printf(" Loc_WinExit "););
      update_after_loc_winexit(canvas_win);
#if 0
      TRACE(printf("canvas_interposer() end.\n"););
#endif
      return NOTIFY_DONE;
    }
    if (event_action(event) == LOC_MOVE)
    {
      TRACE(printf(" Loc_Move "););
      process_loc_move(canvas_win,event,1);
    }
    else
    {
      TRACE(printf(" --> notify_next_event_func() \n"););
      notify_next_event_func((Notify_client) canvas_win,
			     (Notify_event) event,arg,type);
    }
  }

#if 0
  TRACE(printf("canvas_interposer() end.\n"););
#endif

  return NOTIFY_DONE;
}




/*-------------------------------------------------------------------------
WinCanvas - ???
-------------------------------------------------------------------------*/

int
WinCanvas( win, is_show_canvas )
     struct image_display_window *win;
{
  int err;

  err = CANCEL;
  PRT3(printf("\nWinCanvas() entered.\n"));
  if ((win->dwin->screen._2d.status4 & BUF_NON_EMPTY) &&
      ((err = dsetup_frame_canvas_rec(win->dwin,is_show_canvas)) == NO_ERROR))
  {
    if (!((int) xv_get(win->dwin->frame,FRAME_CLOSED)))
    {
      PRT3(printf("...WinCanvas...dshow_buf\n"));
      dshow_buf(win->dwin,0,0,win->dwin->screen._2d.dimx,
		win->dwin->screen._2d.dimy);
    }
  }
  else
    win->process_status4 |= WIN_CANVAS_DATA_ERROR;
  win->process_status4 |= WIN_CANVAS_DATA_PROCESSED;
  PRT(printf("WinCanvas() end win->dwin->frame_canvas.canvas = %p, err = %d\n",
	     (void *)win->dwin->frame_canvas.canvas,err));
  return err;
}




/*-------------------------------------------------------------------------
display_data_to_canvas1 - ???
-------------------------------------------------------------------------*/

int
display_data_to_canvas1(win,is_show_canvas)
 struct image_display_window *win;
{
 int err;
 Rect rr,rr1;

 TRACE(printf("display_data_to_canvas1...canvas = %u\n",
     win->dwin->frame_canvas.canvas))
 err = CANCEL;
 if ((win->dwin->screen._2d.status4 & BUF_NON_EMPTY) &&
     ((err = dsetup_frame_canvas_rec(win->dwin,is_show_canvas)) == NO_ERROR))
    {
     int_rect_rec_to_Rect(&(win->dwin->screen.dimension.rr),&rr);
     frame_get_rect(win->dwin->frame,&rr1);
     frame_get_rect(win->dwin->frame,&rr1);
     rr.r_width = rr1.r_width;
     rr.r_height = rr1.r_height;
     frame_set_rect(win->dwin->frame,&rr);
     TRACE(printf("display_data_to_canvas...dshow_buf\n"))
     dshow_buf(win->dwin,0,0,win->dwin->screen._2d.dimx,
                      win->dwin->screen._2d.dimy);
    }
   else win->process_status4 |= WIN_CANVAS_DATA_ERROR;
 win->process_status4 |= WIN_CANVAS_DATA_PROCESSED;

 TRACE(printf("...display_data_to_canvas1...end\n"))
 return err;
}


/*-------------------------------------------------------------------------
dwin_set_scrollbar - ???
-------------------------------------------------------------------------*/

int
dwin_set_scrollbar(win)
 struct display_win_rec *win;
{
 unsigned long dimx,dimy;
 int err;

 if ((err = WinColorDataDims(win->owner,&dimx,&dimy,NULL)) != NO_ERROR)
    return err;
 PRT(printf("\nBegin ddefine_win_scrollbar...win = %ld\n",win))
 if ((win->horiz_scroll != 0) && (win->horiz_scroll_item != 0) &&
          !xv_get(win->horiz_scroll_item,PANEL_INACTIVE))
    {xv_set(win->horiz_scroll_item,
                  PANEL_VALUE,win->screen.dimension.bound.left,0);
    }
 if ((win->verti_scroll != 0) && (win->verti_scroll_item != 0) &&
          !xv_get(win->verti_scroll_item,PANEL_INACTIVE))
    {xv_set(win->verti_scroll_item,
        PANEL_VALUE,dimy - 1 - win->screen.dimension.bound.top,0);
    }
 PRT(printf("define SCROLL end....\n\n"))
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
horiz_scroll_proc - ???
-------------------------------------------------------------------------*/

int
horiz_scroll_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct display_win_rec *win1;
 unsigned long dimx,dimy;
 int view_len;
 int err;

 PRT(printf("horiz_scroll_proc value = %d\n",value))
 win1 = (struct display_win_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((err = WinColorDataDims(win1->owner,&dimx,&dimy,NULL)) != NO_ERROR)
    return err;
 view_len = win1->screen.dimension.bound.right -
            win1->screen.dimension.bound.left + 1;
 if ((value + view_len) >= dimx)
    win1->screen.dimension.bound.left = dimx - view_len;
   else win1->screen.dimension.bound.left = value;
 win1->screen.dimension.bound.right = win1->screen.dimension.bound.left + view_len - 1;
 return WinColorCanvas(win1->owner);
}


/*-------------------------------------------------------------------------
vertical_scroll_proc - ???
-------------------------------------------------------------------------*/

int
vertical_scroll_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct display_win_rec *win1;
 unsigned long dimx,dimy;
 int view_len;
 int err;

 PRT(printf("vertical_scroll_proc value = %d\n",value))
 win1 = (struct display_win_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((err = WinColorDataDims(win1->owner,&dimx,&dimy,NULL)) != NO_ERROR)
    return err;
 value = dimy - 1 - value;
 view_len = win1->screen.dimension.bound.bottom -
            win1->screen.dimension.bound.top + 1;
 if ((value + view_len) >= dimy)
    win1->screen.dimension.bound.top = dimy - view_len;
   else win1->screen.dimension.bound.top = value;
 win1->screen.dimension.bound.bottom = win1->screen.dimension.bound.top + view_len - 1;
 return WinColorCanvas(win1->owner);
}


/*-------------------------------------------------------------------------
dsetup_win_canvas_scrollbar - ???
-------------------------------------------------------------------------*/

int
dsetup_win_canvas_scrollbar(win)
 struct display_win_rec *win;
{
 Rect *rr;
 int width,height,define_frame_size;
 unsigned long dimx,dimy;
 int err;

 if ((err = WinColorDataDims(win->owner,&dimx,&dimy,NULL)) != NO_ERROR)
    return err;
/* PRT2(printf("dsetup_win_canvas_scrollbar..dimx = %d dimy = %d\n",
    dimx,dimy)); */
/* PRT2(pr_int_region_rect(&(win->screen.dimension))); */
 define_frame_size = 0;
 if ((win->screen.dimension.bound.left != 0) ||
     (win->screen.dimension.bound.right != dimx - 1))
    {if (win->horiz_scroll == 0)
        {win->horiz_scroll = xv_create(win->frame,PANEL,FRAME_CLOSED,FALSE,
                                WIN_CLIENT_DATA,win,0);
         if (win->horiz_scroll != 0)
            {win->horiz_scroll_item = xv_create(win->horiz_scroll,PANEL_SLIDER,
                  XV_X,0,XV_Y,0,
                  PANEL_DIRECTION,PANEL_HORIZONTAL,
                  PANEL_SHOW_VALUE,FALSE,
                  PANEL_SHOW_RANGE,FALSE,
                  PANEL_MIN_VALUE,0,
                  PANEL_MAX_VALUE,dimx - 1,
                  PANEL_NOTIFY_PROC,horiz_scroll_proc,
                  0);
            }
       }
/*     PRT2(printf("dsetup_win_canvas_scrollbar...horiz_scroll..2\n")); */
     if ((win->horiz_scroll != 0) && (win->horiz_scroll_item != 0))
        {xv_set(win->horiz_scroll,XV_SHOW,FALSE,0);
         width = (int) xv_get(win->frame_canvas.canvas,CANVAS_WIDTH) - 7;
         xv_set(win->horiz_scroll_item,PANEL_SLIDER_WIDTH,width,XV_SHOW,TRUE,
                  PANEL_VALUE,win->screen.dimension.bound.left,
                  PANEL_CLIENT_DATA,win,PANEL_INACTIVE,FALSE,0);
         height = (int) xv_get(win->frame_canvas.canvas,XV_Y) +
                  (int) xv_get(win->frame_canvas.canvas,XV_HEIGHT) + 1;
         rr = (Rect *) xv_get(win->horiz_scroll_item,XV_RECT);

         xv_set(win->horiz_scroll,XV_SHOW,TRUE,XV_HEIGHT,rr->r_height - 2,
                      XV_WIDTH,rr->r_width + 6,XV_X,0,XV_Y,height,0);
         rr = (Rect *) xv_get(win->horiz_scroll,XV_RECT);

         win->last_frame_height = rr->r_top + rr->r_height - 5;
/*         PRT(printf("=== 1 h = %d  height = %d\n",win->last_frame_height,height);); */
         define_frame_size = 1;
        }
    }
   else if ((win->horiz_scroll != 0) && (win->horiz_scroll_item != 0))
    {xv_set(win->horiz_scroll_item,PANEL_INACTIVE,TRUE,XV_SHOW,FALSE,0);
/*
     rr = (Rect *) xv_get(canvas_paint_window(win->frame_canvas.canvas),XV_RECT);
     win->last_frame_height = rr->r_top + rr->r_height + 1;
*/
     win->last_frame_height = xv_get(win->frame_canvas.canvas,XV_HEIGHT) +
                      xv_get(win->panel,XV_HEIGHT) + 1;
/*     PRT(printf("=== 2 h = %d  height = %d\n",win->last_frame_height,height);); */
     define_frame_size = 1;
    }

 if ((win->screen.dimension.bound.top != 0) ||
     (win->screen.dimension.bound.bottom != dimy - 1))
    {if (win->verti_scroll == 0)
        {win->verti_scroll = xv_create(win->frame,PANEL,FRAME_CLOSED,FALSE,
                                WIN_CLIENT_DATA,win,0);
         if (win->verti_scroll != 0)
            {win->verti_scroll_item = xv_create(win->verti_scroll,PANEL_SLIDER,
                  XV_X,0,
                  PANEL_DIRECTION,PANEL_VERTICAL,
                  PANEL_SHOW_VALUE,FALSE,
                  PANEL_SHOW_RANGE,FALSE,
                  PANEL_MIN_VALUE,0,
                  PANEL_MAX_VALUE,dimy - 1,
                  PANEL_NOTIFY_PROC,vertical_scroll_proc,
                  0);
            }
       }
     if ((win->verti_scroll != 0) && (win->verti_scroll_item != 0))
        {height = (int) xv_get(win->frame_canvas.canvas,CANVAS_HEIGHT) - 7;
         xv_set(win->verti_scroll_item,PANEL_SLIDER_WIDTH,height,XV_SHOW,TRUE,
                  XV_Y,xv_get(win->frame_canvas.canvas,XV_Y),
                  PANEL_VALUE,dimy - 1 -
                          win->screen.dimension.bound.top,
                  PANEL_CLIENT_DATA,win,PANEL_INACTIVE,FALSE,0);
         width = (int) xv_get(win->frame_canvas.canvas,XV_WIDTH);
         rr = (Rect *) xv_get(win->verti_scroll_item,XV_RECT);

         xv_set(win->verti_scroll,XV_SHOW,TRUE,XV_HEIGHT,rr->r_height + 9,
                      XV_WIDTH,rr->r_width - 2,
                      XV_X,width + 2,
                      XV_Y,0,0);

         rr = (Rect *) xv_get(win->verti_scroll,XV_RECT);

         win->last_frame_width = rr->r_left + rr->r_width - 2;
/*         PRT(printf("===3 w = %d  width = %d\n",win->last_frame_width,width);); */
         define_frame_size = 1;
        }
    }
   else if ((win->verti_scroll != 0) && (win->verti_scroll_item != 0))
    {xv_set(win->verti_scroll,XV_SHOW,FALSE,0);
     xv_set(win->verti_scroll_item,PANEL_INACTIVE,TRUE,XV_SHOW,FALSE,0);
/*
     rr = (Rect *) xv_get(canvas_paint_window(win->frame_canvas.canvas),XV_RECT);
     win->last_frame_width = rr->r_left + rr->r_width + 1;
*/
     win->last_frame_width = xv_get(win->frame_canvas.canvas,XV_WIDTH) + 1;
/*     PRT(printf("===4 w = %d  width = %d\n",win->last_frame_width,width);); */
     define_frame_size = 1;
    }
/* PRT(printf("dsetup_win_canvas_scrollbar..verti_scroll.define_frame_size = %d\n",define_frame_size)); */


 if (define_frame_size)
    {xv_set(win->frame,XV_WIDTH,win->last_frame_width,
             XV_HEIGHT,win->last_frame_height,0);
     win->resize_status += 1;
    }
/* PRT(printf("f == h = %d  w = %d\n",win->last_frame_height,win->last_frame_width);); */

 return NO_ERROR;
}


/*-------------------------------------------------------------------------
check_dwin_frame_size - ???
-------------------------------------------------------------------------*/

int
check_dwin_frame_size(dwin)
 struct display_win_rec *dwin;
{
 int width,height;

 height = (int) xv_get(dwin->frame,XV_HEIGHT);
 width = (int) xv_get(dwin->frame,XV_WIDTH);
 PRT3(printf("check_dwin_frame_size...dwin = %u\n",dwin))
 PRT3(printf("w = %d...h = %d last w = %d h = %d\n",width,height,
         dwin->last_frame_width,dwin->last_frame_height);)
 if (dwin->last_frame_height != height)
    {xv_set(dwin->frame,XV_HEIGHT,dwin->last_frame_height,0);
     PRT3(printf("set XV_HEIGHT\n"))}

 if (dwin->last_frame_width != width)
    {xv_set(dwin->frame,XV_WIDTH,dwin->last_frame_width,0);
     PRT3(printf("set XV_WIDTH\n"))}
 PRT3(printf("..last == w = %d h = %d  \n",
         dwin->last_frame_width,dwin->last_frame_height);)
 return NO_ERROR;
}
  

/*-------------------------------------------------------------------------
check_dwin_frame_size_proc - ???
-------------------------------------------------------------------------*/

void
check_dwin_frame_size_proc(dwin,which)
 struct display_win_rec *dwin;
 int which;
{
 (void) notify_set_itimer_func((Notify_client) dwin,
             NOTIFY_FUNC_NULL,ITIMER_REAL,
             (struct itimerval *) 0,(struct itimerval *) 0);
 check_dwin_frame_size(dwin);
 dwin->status2 &= ~INIT_WINDOW;
}



/*-------------------------------------------------------------------------
dsetup_frame_canvas_rec - ???
-------------------------------------------------------------------------*/

int
dsetup_frame_canvas_rec( win, is_show_canvas )
     struct display_win_rec *win;
     int is_show_canvas;
{
  Cms cms;
  Rect *rr;
  
  if ((win->frame_canvas.canvas == 0) ||
      ((win->screen._2d.dimx + 1) != win->frame_canvas.width) ||
      ((win->screen._2d.dimy + 1) != win->frame_canvas.height) ||
      (win->screen._2d.pixel_depth != win->frame_canvas.pixel_depth))
  {
    if (win->frame_canvas.canvas != 0)
    {
      xv_destroy(canvas_paint_window(win->frame_canvas.canvas));
      xv_destroy(win->frame_canvas.canvas);
    }
  }
  else
    return dsetup_win_canvas_scrollbar(win);


  PRT3(printf("dsetup_frame_canvas_rec(): screen: [ %d x %d ], depth = %d\n",
	      win->screen._2d.dimx,
	      win->screen._2d.dimy,
	      win->screen._2d.pixel_depth));
  PRT(printf("	canvas %p: [ %d x %d ], depth = %d\n",
	     (void *) win->frame_canvas.canvas,
	     win->frame_canvas.width,
	     win->frame_canvas.height,
	     win->frame_canvas.pixel_depth));

  win->frame_canvas.width = win->screen._2d.dimx + 1;
  win->frame_canvas.height = win->screen._2d.dimy + 1;
  win->frame_canvas.pixel_depth = win->screen._2d.pixel_depth;
  rr = (Rect *) xv_get(win->panel,XV_RECT);

  cms = (Cms) xv_get( win->frame, WIN_CMS );
  PRT(printf("	frame = %p\n", (void *) win->frame ););
  PRT(printf("	frame CMS = %p\n", (void *) cms ););

  win->frame_canvas.canvas =
    xv_create( win->frame, CANVAS,
	       WIN_CLIENT_DATA, win,
	       /* WIN_DYNAMIC_VISUAL, TRUE, */
	       XV_X, 0,
	       XV_Y, rr->r_top + rr->r_height + 1,
	       FRAME_INHERIT_COLORS, TRUE,
	       CANVAS_WIDTH, win->frame_canvas.width,
	       CANVAS_HEIGHT, win->frame_canvas.height,
	       XV_WIDTH, win->frame_canvas.width + 1,
	       XV_HEIGHT, win->frame_canvas.height + 1,
	       CANVAS_X_PAINT_WINDOW, TRUE,
	       CANVAS_REPAINT_PROC, display_repaint_proc,
	       CANVAS_AUTO_SHRINK, TRUE,
	       CANVAS_AUTO_EXPAND, TRUE,
	       WIN_RETAINED, FALSE,
	       CANVAS_RETAINED, FALSE,
	       0);
  if (win->frame_canvas.canvas == 0)
    return WINDOW_ERROR;

  if (is_show_canvas)
    xv_set(win->frame_canvas.canvas,XV_SHOW,TRUE,0);
  else
    xv_set(win->frame_canvas.canvas,XV_SHOW,FALSE,0);

  win->default_cursor = (Xv_Cursor)
    xv_get( canvas_paint_window( win->frame_canvas.canvas ), WIN_CURSOR );
  set_win_back_and_foreground( win->frame_canvas.canvas,
			       1,
			       (unsigned long) (1 + win->owner->intensity->color.num_system_colors),
			       1,
			       (unsigned long) (2 + win->owner->intensity->color.num_system_colors));

  xv_set(canvas_paint_window(win->frame_canvas.canvas),
	 WIN_CLIENT_DATA,win,
	 WIN_EVENT_PROC,process_event_proc,
	 WIN_CONSUME_EVENTS,LOC_DRAG,ACTION_ERASE_CHAR_BACKWARD,
	   LOC_WINENTER,LOC_WINEXIT,LOC_MOVE,WIN_LEFT_KEYS,ACTION_AGAIN,
	   ACTION_STOP,WIN_KEY_FIND,ACTION_CAPS_LOCK,
	   WIN_ASCII_EVENTS,WIN_MOUSE_BUTTONS,WIN_RIGHT_KEYS,WIN_TOP_KEYS,
	   NULL,
	 WIN_IGNORE_EVENTS,WIN_UP_ASCII_EVENTS,NULL,
	 0);

  PRT(printf("	End: frame = %p, canvas = %p, win = %p\n",
	     (void *)win->frame,
	     (void *)win->frame_canvas.canvas,
	     (void *)win));
  xv_set(win->panel,XV_WIDTH,xv_get(win->frame_canvas.canvas,XV_WIDTH),0);

  win->last_frame_height =
    xv_get(win->frame_canvas.canvas,XV_HEIGHT) +
    xv_get(win->panel,XV_HEIGHT) + 1;
  win->last_frame_width = xv_get(win->frame_canvas.canvas,XV_WIDTH) + 1;
  PRT3(printf("	height: %d, width: %d\n",
	      win->last_frame_height,win->last_frame_width););

  dsetup_win_canvas_scrollbar(win);
  xv_set(win->frame,XV_HEIGHT,win->last_frame_height,
	 XV_WIDTH,win->last_frame_width,0);
  PRT3(printf("grab keyboard focus...INIT_WINDOW\n"));
  grap_keyboard(canvas_paint_window(win->frame_canvas.canvas));
  
  notify_interpose_event_func(canvas_paint_window(win->frame_canvas.canvas),
                              canvas_interposer,
			      NOTIFY_SAFE);
  xv_set(canvas_paint_window(win->frame_canvas.canvas),
	 WIN_CONSUME_EVENTS,WIN_RESIZE,0,0);
  dwin_default_cursor(win);
  PRT(printf("....dwin_default_cursor(win)\n"));
  win->resize_status = 0;
  win->status2 |= INIT_WINDOW;
  win->status2 &= ~WINDOW_WIN_RECT_VALID;
  (void) notify_set_itimer_func((Notify_client) win,
				(Notify_func) check_dwin_frame_size_proc,
				ITIMER_REAL,
				&NOTIFY_POLLING_ITIMER,(struct itimerval *) 0);
  return NO_ERROR;
}
  


#if 0
Event *last_event(event,is_set)
 Event *event;
 int is_set;
{
 static Event event1;
 static int first = 1;

 if (is_set)
    {first = 0;
     event1 = *event;
     return &event1;}
   else if (first)
    return NULL;
   else return &event1;
}
#endif
