/****************************************************************************
display_general.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: display_general.c,v 1.8 1999/07/07 17:41:15 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/scrollbar.h>
#include <xview/notify.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "mri_format.h"
#include "debug.h"
#include "nria_procedures.h"


/* Functions defined in this file */
void pr_XVisualInfo(XVisualInfo *visualList, int visualsMatched);
int ServerDefaultVisual(Xv_Server server, int screen_num,
    XVisualInfo *visualinfo);
int get_system_colors(Xv_Server server, int screen_num,
    Xv_singlecolor **system_colors, unsigned int *num_system_colors);
int map_color_format(Xv_singlecolor **colors,
    unsigned short *red, unsigned short *green, unsigned short *blue,
    unsigned int num_colors);
int AssignXColors(XColor *colors, unsigned short *red, unsigned short *green,
    unsigned short *blue, unsigned int num_colors, int is_new_pixel);
Cms setup_color_segment(Xv_Screen screen, char *segment_name,
    struct color_rec *color);
int set_win_cms(Xv_Window win, Cms cms, void *old_cms, long canvas_bg_color,
    long canvas_fg_color);
int SetColorSegment(Xv_Window win, char *segment_name, struct color_rec *color,
    int use_original);
int ChangeColorSegment(Xv_Window win, char *segment_name,
    struct color_rec *color, int use_original);
int ChangeExistingColorSegment(Xv_Window win, char *segment_name,
    struct color_rec *color, int use_original, int *redisplay);



/*-----------------------------------------------------------------------
pr_XVisualInfo - ???
-----------------------------------------------------------------------*/

void
pr_XVisualInfo(visualList,visualsMatched)
     XVisualInfo *visualList;
     int visualsMatched;
{
  int lcv;

  printf("pr_XVisualInfo(): # of visualsMatched = %d\n", visualsMatched);
  for (lcv = 0; lcv < visualsMatched; lcv++)
  {
    printf("	vaddr: %p  ", (void *) visualList[lcv].visual);
    if (visualList[lcv].class == PseudoColor)
      printf(" PseudoColor  ");
    else if (visualList[lcv].class == DirectColor)
      printf(" DirectColor  ");
    else if (visualList[lcv].class == TrueColor)
      printf(" TrueColor    ");
    else if (visualList[lcv].class == GrayScale)
      printf(" GrayScale    ");
    else if (visualList[lcv].class == StaticGray)
      printf(" StaticGray   ");
    else if (visualList[lcv].class == StaticColor)
      printf(" StaticColor  ");
    else
      printf(" Unrecognized (%d):   ",visualList[lcv].class);

    printf("%d-bit depth\n		RGB = [%02lx:%02lx:%02lx], size = %d, bits/RGB = %d\n",
	   visualList[lcv].depth,
	   visualList[lcv].red_mask,
	   visualList[lcv].green_mask,
	   visualList[lcv].blue_mask,
	   visualList[lcv].colormap_size,
	   visualList[lcv].bits_per_rgb);
  }
}



/*-----------------------------------------------------------------------
ServerDefaultVisual - ???
-----------------------------------------------------------------------*/

int
ServerDefaultVisual(server,screen_num,visualinfo)
     Xv_Server server;
     int screen_num;
     XVisualInfo *visualinfo;
{
  Display *display;
  XVisualInfo *visualList,vTemplate;
  Visual *visual;
  int visualsMatched;
  char *name;
  long lcv;


  NOPRT(printf("ServerDefaultVisual()...\n"));

  name = (char *) xv_get(server,XV_NAME);
  PRT(printf("	X Server Name: \"%s\"\n",name));

  display = XOpenDisplay(name);
  if (display == NULL)
    return ~NO_ERROR;
  NOPRT(printf("ServerDefaultVisual...display = %p\n",(void *)display));

  visualsMatched = 0;
  visualList = NULL;
  vTemplate.screen = screen_num;

  /*
    vTemplate.depth = 24;
    vTemplate.class = PseudoColor;
    vTemplate.class = DirectColor;
    |VisualClassMask|VisualDepthMask
    */

  visualList = XGetVisualInfo(display,VisualScreenMask,
			      &vTemplate, &visualsMatched);
  if (visualsMatched == 0)
    return ~NO_ERROR;

  NOPRT(pr_XVisualInfo(visualList,visualsMatched););
  visual = DefaultVisual(display,screen_num);

  NOPRT(printf("visualList = %p, visual = %p\n",
	       (void *) visualList,
	       (void *) visual););
  for (lcv = 0; lcv < visualsMatched; lcv++)
    if (visualList[lcv].visual == visual)
    {
      *visualinfo = visualList[lcv];
      break;
    }
  if (lcv == visualsMatched)
    return ~NO_ERROR;

  NOPRT(printf("ServerDefaultVisual...end\n"););
  return NO_ERROR;
}



/*-----------------------------------------------------------------------
get_system_colors - ???
-----------------------------------------------------------------------*/

int
get_system_colors( server, screen_num, system_colors, num_system_colors )
     Xv_Server server;
     int screen_num;
     Xv_singlecolor **system_colors;
     unsigned int *num_system_colors;
{
  int err;
  unsigned int size;
  Cms cms;
  XVisualInfo visualinfo;


  PRT(printf("get_system_colors...server = %u  screen_num = %d\n",
	     server,screen_num));

  if ((err = ServerDefaultVisual(server,screen_num,&visualinfo)) != NO_ERROR)
    return err;

  PRT(printf("get_system_colors...1...\n"));
  cms = xv_create( 0, CMS,
		   CMS_CONTROL_CMS, TRUE,
		   XV_VISUAL, visualinfo.visual,
		   NULL );
  PRT(printf("get_system_colors...2\n"));

  *num_system_colors = 0;
  PRT(printf("get_system_colors...cms = %u\n",cms));

  if (cms != 0)
  {
    size = xv_get(cms,CMS_SIZE);
    PRT(printf("size of control cms: %d\n",size););
    get_ptr_memory( system_colors, num_system_colors,
		    (unsigned int) size,
		    (size_t) sizeof(Xv_singlecolor), &err );
    if (err != NO_ERROR)
      return err;
    xv_get( cms, CMS_COLORS, (*system_colors) );
    PRT(printf("get_system_colors...2\n",cms));

    xv_destroy_safe(cms);
  }
  PRT(printf("get_system_colors...end\n"));
  return NO_ERROR;
}


/*-----------------------------------------------------------------------
map_color_format - ???
-----------------------------------------------------------------------*/

int
map_color_format(colors,red,green,blue,num_colors)
 Xv_Singlecolor **colors;
 unsigned short *red,*green,*blue;
 unsigned int num_colors;
{
 int err;
 unsigned int total,lcv;

 total = 0;
 if ((err = get_ptr(colors,&total,num_colors,(size_t) sizeof(Xv_Singlecolor)))
      != NO_ERROR) 
    return err;

 for (lcv = 0; lcv < num_colors; lcv++)
    {(*colors)[lcv].red = red[lcv];
     (*colors)[lcv].green = green[lcv];
     (*colors)[lcv].blue = blue[lcv];
    }
 return NO_ERROR;
}



/*-----------------------------------------------------------------------
AssignXColors - ???
-----------------------------------------------------------------------*/

int
AssignXColors( colors, red, green, blue, num_colors, is_new_pixel )
     XColor *colors;
     unsigned short *red,*green,*blue;
     unsigned int num_colors;
     int is_new_pixel;
{
  unsigned int lcv;
  char doflag;

  doflag = DoRed | DoGreen | DoBlue;
  if (is_new_pixel)
    for (lcv = 0; lcv < num_colors; lcv++)
      colors[lcv].pixel = 0;

  for (lcv = 0; lcv < num_colors; lcv++)
  {
    colors[lcv].red = ((unsigned long) red[lcv]) << 8;
    colors[lcv].green = ((unsigned long) green[lcv]) << 8;
    colors[lcv].blue = ((unsigned long) blue[lcv]) << 8;
    colors[lcv].flags = doflag;
    colors[lcv].pad = 0;
  }
  return NO_ERROR;
}




/*-----------------------------------------------------------------------
 - ???

Create colormap segment (XView-specific), labelled e.g. "GRAY SCALE",
and store the colors into a generic color structure.
-----------------------------------------------------------------------*/

Cms
setup_color_segment( screen, segment_name, color )
     Xv_Screen screen;
     char *segment_name;
     struct color_rec *color;
{
  Cms cms;
  XColor *colors;
  unsigned long *old_pixels;
  int err,is_new_pixel;
  unsigned long size,lcv,total;
  Xv_Server server;
  int screen_num;
  XVisualInfo visualinfo;
  

  PRT(printf("setup_color_segment()\n");
      printf("	segment_name = \"%s\", color = %p\n", segment_name,
	     (void *) color););

  if (screen == 0)
    return 0;
  server = (Xv_Server) xv_get(screen,SCREEN_SERVER);
  screen_num = (int) xv_get(screen,SCREEN_NUMBER);

  if ((err = ServerDefaultVisual(server,screen_num,&visualinfo)) != NO_ERROR)
    return err;

  total = 0;
  PRT(printf("	total_colors = %d #_system_colors = %d\n",
	     color->total_colors,color->num_system_colors));

  if ((err = get_ptr( &colors, &total,
		      color->total_colors - color->num_system_colors,
		      (size_t) sizeof(XColor))) != NO_ERROR)
    return err;
  NOPRT(printf("setup_color_segment 2\n"));

  cms = (Cms) xv_find( screen, CMS,
		       CMS_NAME, segment_name,
		       XV_AUTO_CREATE, FALSE, /* If we don't find the
						 named segment, don't
						 create a new one just yet.
						 */
		       NULL );
  NOPRT(printf("setup_color_segment 3\n"));
  if (cms != 0)
  {
    size = (int) xv_get(cms,CMS_SIZE);
    PRT(printf("	found cms, initial size = %d\n",size););
    if (total > size)
      total = size;
    old_pixels = (unsigned long *) xv_get(cms,CMS_INDEX_TABLE);
    for (lcv = 0; lcv < total; lcv++)
      colors[lcv].pixel = old_pixels[lcv];
    is_new_pixel = 0;
  }
  else
  {
    PRT(printf("	did not find cms\n"););
    is_new_pixel = 1;
  }

  AssignXColors( colors,
		 &(color->red[color->num_system_colors]),
		 &(color->green[color->num_system_colors]),
		 &(color->blue[color->num_system_colors]),
		 total,
		 is_new_pixel );
  PRT(printf("	cms = %p, total = %d, colors = %p\n",
	     (void *) cms, total, (void *) colors));

  if (cms == 0)
  {
    if ((visualinfo.class == PseudoColor) ||
	(visualinfo.class == DirectColor))
    {
      cms = (Cms) xv_create(0,CMS,
			    CMS_NAME, segment_name,
			    CMS_CONTROL_CMS, TRUE, /* TRUE 970305 */
			    XV_VISUAL, visualinfo.visual,
			    CMS_TYPE, XV_DYNAMIC_CMS,
			    CMS_SIZE, color->total_colors,
			    CMS_X_COLORS, colors,
			    NULL);
      PRT(printf("	CREATED NEW DYNAMIC CMS, %p\n",(void *) cms););
    }
    else
    {
      cms = (Cms) xv_create(0,CMS,
			    CMS_NAME, segment_name,
			    CMS_CONTROL_CMS, TRUE, /* TRUE 970305 */
			    XV_VISUAL, visualinfo.visual,
			    CMS_SIZE, color->total_colors,
			    CMS_X_COLORS, colors,
			    NULL);
      PRT(printf("	CREATED NEW STATIC CMS, %p\n",(void *) cms););
    }

  }
  else
    xv_set( cms,
	    CMS_X_COLORS, colors,
	    NULL );
  c_free(colors);

  if (cms == 0)
  {
    perror("	Sorry...unable to set up the color segment.\n");
    exit(0);
  }

  size = (int) xv_get(cms,CMS_SIZE);
  PRT(printf("	CMS size: %d; #Total-#System = %d; #Total = %d;",
	     size, total, color->total_colors);
      printf(" depth = %d\n",(int) xv_get(cms,XV_DEPTH)););


  return cms;
}



/*-----------------------------------------------------------------------
set_win_cms - ???
-----------------------------------------------------------------------*/

int
set_win_cms(win,cms,old_cms,canvas_bg_color,canvas_fg_color)
 Xv_Window win;
 Cms cms;
 void *old_cms;
 long canvas_bg_color,canvas_fg_color;
{
 void *ori_cms;

 PRT(printf(" set_win_cms...begin\n");)
 ori_cms = (void *) xv_get(win,WIN_CMS);

 PRT(printf(" set_win_cms win = %u ori_cms = %u old_cms = %u cms = %u\n",win,ori_cms,old_cms,cms);)
 PRT(printf(" set_win_cms...-1\n");)

 if ((ori_cms != old_cms) && (old_cms != NULL))
    {PRT(printf("not same set_win_cms ori_cms = %u old_cms = %u\n",ori_cms,old_cms);)
     return NO_ERROR;
    }
 PRT(printf(" set_win_cms...0\n");)
 xv_set(win,WIN_CMS,cms,0);
 PRT(printf(" set_win_cms...1\n");)
 if ((Xv_pkg *) xv_get(win,XV_TYPE) == CANVAS)
    {xv_set(win,WIN_BACKGROUND_COLOR,canvas_bg_color,
              WIN_FOREGROUND_COLOR,canvas_fg_color,0);
    }
 PRT(printf(" set_win_cms...end\n");)
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
SetColorSegment - ???
-----------------------------------------------------------------------*/

int
SetColorSegment(win,segment_name,color,use_original)
 Xv_Window win;
 char *segment_name;
 struct color_rec *color;
 int use_original;
{
 Cms cms,old_cms;
 Xv_Screen screen;
 Xv_Server server;
 int screen_num;
 XVisualInfo visualinfo;
 Xv_Singlecolor *colors;
 int err;
 unsigned int num_colors;
 char *fn;

 screen = (Xv_Screen) xv_get(win,XV_SCREEN);
 if (screen == 0) return ~NO_ERROR;
 old_cms = cms = (Cms) xv_find(screen,CMS,
                CMS_NAME,segment_name,XV_AUTO_CREATE,FALSE,
                NULL);
 TRACE(printf("SetColorSegment segment_name = %s  cms = %u\n",
           segment_name,cms))
 if (use_original)
    {if ((err = map_color_format(&colors,
            &(color->ori_red[color->num_system_colors]),
            &(color->ori_green[color->num_system_colors]),
            &(color->ori_blue[color->num_system_colors]),
            color->total_colors - color->num_system_colors)) != NO_ERROR)
        return err;}
   else
    {if ((err = map_color_format(&colors,
            &(color->red[color->num_system_colors]),
            &(color->green[color->num_system_colors]),
            &(color->blue[color->num_system_colors]),
            color->total_colors - color->num_system_colors)) != NO_ERROR)
        return err;}

 if (cms != 0)
    {num_colors = (int) xv_get(cms,CMS_SIZE);
     if (num_colors != color->total_colors)
        cms = 0;
       else if (((fn = (char *) xv_get(cms,CMS_NAME)) != NULL) &&
                (str_cmp(fn,segment_name) != 0))
        cms = 0;
     TRACE(printf("cms = %u fn = %s segment_name = %s\n",cms,fn,segment_name))
    }


 if (cms == 0)
    {
     if ((server = (Xv_Server) xv_get(screen,SCREEN_SERVER)) == 0)
        {c_free(colors);
         return ~NO_ERROR;
        }
     screen_num = (int) xv_get(screen,SCREEN_NUMBER);
     if ((err = ServerDefaultVisual(server,screen_num,&visualinfo)) != NO_ERROR)
        {c_free(colors);
         return ~NO_ERROR;
        }
     if (old_cms != 0) xv_destroy(old_cms);

     TRACE(printf("***cms not found...segment_name = %s color->total_colors = %d color->num_system_colors = %d\n", segment_name,           
            color->total_colors,color->num_system_colors);)
     if ((visualinfo.class == PseudoColor) ||
         (visualinfo.class == DirectColor))
        cms = (Cms) xv_create(0,CMS,CMS_NAME,segment_name,
              CMS_CONTROL_CMS,TRUE,
              CMS_TYPE,XV_DYNAMIC_CMS,
	       XV_VISUAL,visualinfo.visual,
              CMS_SIZE,color->total_colors,CMS_COLORS,colors,NULL);
       else cms = (Cms) xv_create(0,CMS,CMS_NAME,segment_name,
              CMS_CONTROL_CMS,TRUE,
	       XV_VISUAL,visualinfo.visual,
              CMS_SIZE,color->total_colors,CMS_COLORS,colors,NULL);
    }
   else 
    xv_set(cms,CMS_COLORS,colors,NULL);
 c_free(colors);

 TRACE(printf("SetColorSegment cms = %u\n",cms))

 if (cms != 0)
    apply_proc_to_all_tree_window(win,set_win_cms,cms,old_cms,
              color->num_system_colors + 1,color->num_system_colors + 2,0);

 TRACE(printf("SetColorSegment end\n"))
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
ChangeColorSegment - ???
-----------------------------------------------------------------------*/

int
ChangeColorSegment(win,segment_name,color,use_original)
 Xv_Window win;
 char *segment_name;
 struct color_rec *color;
 int use_original;
{
 Cms cms,old_cms;
 Xv_Singlecolor *colors;
 int err;
 unsigned int num_colors;
 char *fn;
 Xv_Screen screen;
 Xv_Server server;
 int screen_num;
 XVisualInfo visualinfo;

 old_cms = cms = (Cms) xv_get(win,WIN_CMS);
 TRACE(printf("ChangeColorSegment segment_name = %s cms = %u use_original = %d\n\n",segment_name,cms,use_original))
 if (use_original)
    {if ((err = map_color_format(&colors,
            &(color->ori_red[color->num_system_colors]),
            &(color->ori_green[color->num_system_colors]),
            &(color->ori_blue[color->num_system_colors]),
            color->total_colors - color->num_system_colors)) != NO_ERROR)
        return err;}
   else
    {if ((err = map_color_format(&colors,
            &(color->red[color->num_system_colors]),
            &(color->green[color->num_system_colors]),
            &(color->blue[color->num_system_colors]),
            color->total_colors - color->num_system_colors)) != NO_ERROR)
        return err;}
 if (cms != 0)
    {num_colors = (int) xv_get(cms,CMS_SIZE);
     TRACE(printf("num_colors = %d total_colors = %d\n",
                    num_colors,color->total_colors))
     if (num_colors != color->total_colors)
         cms = 0;
       else if (((fn = (char *) xv_get(cms,CMS_NAME)) != NULL) &&
                (str_cmp(fn,segment_name) != 0))
         cms = 0;
     PRT(printf("==> old_cms = %u cms = %u fn = %s segment_name = %s\n",old_cms,cms,fn,segment_name))
    }

 if (cms == 0)
    {
     if (((screen = (Xv_Screen) xv_get(win,XV_SCREEN)) == 0) ||
         ((server = (Xv_Server) xv_get(screen,SCREEN_SERVER)) == 0))
        {c_free(colors);
         return ~NO_ERROR;
        }
     screen_num = (int) xv_get(screen,SCREEN_NUMBER);
     if ((err = ServerDefaultVisual(server,screen_num,&visualinfo)) != NO_ERROR)
        {c_free(colors);
         return ~NO_ERROR;
        }
     if (old_cms != 0) xv_destroy(old_cms);

     PRT(printf("***cms not found...segment_name = %s color->total_colors = %d color->num_system_colors = %d\n", segment_name,           
            color->total_colors,color->num_system_colors))
     if ((visualinfo.class == PseudoColor) ||
         (visualinfo.class == DirectColor))
        cms = (Cms) xv_create(0,CMS,CMS_NAME,segment_name,
              CMS_CONTROL_CMS,TRUE,
              CMS_TYPE,XV_DYNAMIC_CMS,
	       XV_VISUAL,visualinfo.visual,
              CMS_SIZE,color->total_colors,CMS_COLORS,colors,NULL);
       else cms = (Cms) xv_create(0,CMS,CMS_NAME,segment_name,
              CMS_CONTROL_CMS,TRUE,
	       XV_VISUAL,visualinfo.visual,
              CMS_SIZE,color->total_colors,CMS_COLORS,colors,NULL);
     if (cms != 0)
        {PRT(printf("cms = %u CMS_NAME = %s\n",cms,
                     (char *) xv_get(cms,CMS_NAME)))
         apply_proc_to_all_tree_window(win,set_win_cms,cms,old_cms,
              color->num_system_colors + 1,color->num_system_colors + 2,0);}
    }
   else
    {PRT(printf("ChangeColorSegment xv_set(cms,CMS_COLORS...\n"))
     xv_set(cms,CMS_COLORS,colors,NULL);}
 c_free(colors);
 PRT(printf("ChangeColorSegment end...\n\n"))
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
ChangeExistingColorSegment - ???
-----------------------------------------------------------------------*/

int
ChangeExistingColorSegment(win,segment_name,color,use_original,redisplay)
 Xv_Window win;
 char *segment_name;
 struct color_rec *color;
 int use_original,*redisplay;
{
 Cms cms,old_cms;
 Xv_Screen screen;
 Xv_Server server;
 int screen_num;
 XVisualInfo visualinfo;
 unsigned long *old_pixels;
 XColor *colors;
 int err,class;
 unsigned long size,lcv,total;

 screen = (Xv_Screen) xv_get(win,XV_SCREEN);
 if (screen == 0) return ~NO_ERROR;
 cms = (Cms) xv_find(screen,CMS,
                CMS_NAME,segment_name,XV_AUTO_CREATE,FALSE,
                NULL);
 PRT1(printf("ChangeExistingColorSegment name = %s  cms = %u use_original = %d\n",segment_name,cms,use_original))
 *redisplay = 0;
 if (cms != 0)
    {

     total = 0;
     if ((err = get_ptr(&colors,&total,
           color->total_colors - color->num_system_colors,
           (size_t) sizeof(XColor))) != NO_ERROR) 
       return err;
     size = (int) xv_get(cms,CMS_SIZE);
     if (total > size) total = size;
     PRT1(printf("change_ last size = %d total = %d tc = %d dep = %d\n",size,
           total,color->total_colors,(int) xv_get(cms,XV_DEPTH));)

     old_pixels = (unsigned long *) xv_get(cms,CMS_INDEX_TABLE);
     for (lcv = 0; lcv < total; lcv++)
        colors[lcv].pixel = old_pixels[lcv];
     if (use_original)
        AssignXColors(colors,
            &(color->ori_red[color->num_system_colors]),
            &(color->ori_green[color->num_system_colors]),
            &(color->ori_blue[color->num_system_colors]),
            total,0);
       else AssignXColors(colors,
            &(color->red[color->num_system_colors]),
            &(color->green[color->num_system_colors]),
            &(color->blue[color->num_system_colors]),
            total,0);
     class = (int) xv_get(cms,XV_VISUAL_CLASS);
     if ((class != PseudoColor) &&
         (class != DirectColor))
        {PRT1(printf("===> reinit...\n"))
         old_cms = cms;
         if ((server = (Xv_Server) xv_get(screen,SCREEN_SERVER)) == 0)
            {c_free(colors);
             return ~NO_ERROR;
            }
         screen_num = (int) xv_get(screen,SCREEN_NUMBER);
         if ((err = ServerDefaultVisual(server,screen_num,&visualinfo)) != NO_ERROR)
            {c_free(colors);
             return ~NO_ERROR;
            }
         if (old_cms != 0) xv_destroy(old_cms);
         cms = (Cms) xv_create(0,CMS,CMS_NAME,segment_name,
              CMS_CONTROL_CMS,TRUE,
	       XV_VISUAL,visualinfo.visual,
              CMS_SIZE,color->total_colors,CMS_X_COLORS,colors,NULL);
         if (cms != 0)
            {PRT(printf("reinit cms = %u CMS_NAME = %s\n",cms,
                     (char *) xv_get(cms,CMS_NAME)))
             apply_proc_to_all_tree_window(win,set_win_cms,cms,old_cms,
                 color->num_system_colors + 1,color->num_system_colors + 2,0);
             *redisplay = 1;}
        }
       else
        {PRT1(printf("===> xv_set(cms,CMS_X_COLORS...\n"))
         xv_set(cms,CMS_X_COLORS,colors,NULL);}
     c_free(colors);}

 PRT(printf("ChangeExistingColorSegment end\n"))
 return NO_ERROR;
}


