/****************************************************************************
display_main_extra.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.


This file depends on XView.

HISTORY since 19961111 and prior to CVS:

DRW 19961111 Added canvaspw_event_proc() to add menu to canvas paint
window, since menu was not working in canvas area.

DRW 19970228 Cleaned up Notifier-related code & algorithm and
debugged (Solaris 2.x upgrade).

****************************************************************************/

static const char rcsid[] = "$Id: display_main_extra.c,v 1.13 1999/07/07 17:19:17 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/notify.h>
#include <xview/cms.h>
#include <xview/font.h>

#include "ask_file.h"
#include "debug.h"
#include "mri_format.h"
#include "nria_procedures.h"

/* Change with each version released */
#define NRIA_VERSION_STRING "Version 1.0.6"

#define STRING(a) #a

char version[] = NRIA_VERSION_STRING " (Compiled " __DATE__
#ifdef HOST
" on " HOST
#endif
 ")";



/* FIXME: These #define's are for use with main_display() routine.  They
 *    should be in some general header file.  */

#define SETUP_MAIN_DISPLAY 1
#define GET_MAIN_DISPLAY_FRAME 2
#define SHOW_MAIN_DISPLAY 3
#define CLEAN_UP_MAIN_DISPLAY 4
#define MAIN_DISPLAY_FONT 5
#define MAIN_DISPLAY_CANVAS 6


struct key_proc_rec
{
  int key;
  void (*proc)();
  unsigned short win_type;
};


/* Functions defined in this file */
void setup_main_canvas_message(Canvas canvas);
void main_canvas_repaint_proc(Canvas canvas, Xv_Window paint_window,
    Rectlist *repaint_area);
void about_ria_proc(Menu menu, Menu_item item);
void beginner_help_proc(Menu menu, Menu_item item);
void program_exit_proc(Menu menu, Menu_item item);
static void select_fns_menu_proc(Menu menu, Menu_item item);
void main_event_proc(Frame frame, Event *event);
void canvaspw_event_proc(Xv_Window canvas_pw, Event *event);
Notify_value prog_event_proc(Notify_client frame, Event *event,
    Notify_arg arg, Notify_event_type type);
Notify_value main_destroy_func(Xv_window win, Destroy_status status);
Frame *main_display(int select, struct prog_rec *prog, Xv_Server server,
    int screen_num);
int cleanup_prog(struct prog_rec *prog, int must_quit);



#define PROG_EVENT_PROC_NUM_ACTIONS 5
static struct key_proc_rec actions[PROG_EVENT_PROC_NUM_ACTIONS] =
{
  { 'O',		select_fns_menu_proc, EDITING_WINDOW },
  { 'o',		select_fns_menu_proc, EDITING_WINDOW },
  { 'q',		program_exit_proc, EDITING_WINDOW },
  { 'Q',		program_exit_proc, EDITING_WINDOW },
  { KEY_TOP(12),	import_image_file_from_main_proc, ALL_WINDOWS },
};


#define PROG_EVENT_PROC_NUM_TOP_ACTIONS 4

static struct key_proc_rec top_actions[PROG_EVENT_PROC_NUM_TOP_ACTIONS] =
{
  { KEY_TOP(10),	about_ria_proc, EDITING_WINDOW },
  { KEY_TOP(11),	beginner_help_proc, EDITING_WINDOW },
};


/*  Sets an itimer to occur once, and then automatically turn off.
 *
 *  From the setitimer(2) man page:
 *
 *     A timer value is defined by  the  itimerval  structure  (see
 *     gettimeofday(3C)  for  the  definition  of  timeval),  which
 *     includes the following members:
 * 
 *             struct timeval    it_interval;
 *             struct timeval    it_value;
 * 
 *     it_value indicates the time to the  next  timer  expiration.
 *     it_interval  specifies  a  value  to  be  used  in reloading
 *     it_value when the timer expires.  Setting it_value  to  zero
 *     disables  a  timer,  regardless of the value of it_interval.
 *     Setting it_interval to zero disables a timer after its  next
 *     expiration (assuming it_value is non-zero).
 */

struct itimerval ITIMER_ONCE =
{
  {0,0},			/* it_interval of zero: once only */
  
  {0,1}				/* it_value = one microsecond,
				 *  rounded up to process granularity  */
};

#define notify_cancel_itimer( client ) \
	notify_set_itimer_func( (Notify_client) (client), \
				NOTIFY_FUNC_NULL, ITIMER_REAL, NULL, NULL );


/*------------------------------------------------------------------------
setup_main_canvas_message - ???
-------------------------------------------------------------------------*/

void
setup_main_canvas_message( canvas )
     Canvas	canvas;
{
  int lcv, y, height, max_width, line_height;

#define SETUP_MAIN_CANVAS_MESSAGE_NUM_STRS 8

  static char *strs[SETUP_MAIN_CANVAS_MESSAGE_NUM_STRS] = {
    "New Regional Image Analysis (NRIA)",
    version,
    "Developed at the University of Pennsylvania",
    "   Based on MRVBR (on SunView) by Mark Kohn M.D. (Dept. Radiology), 1989",
    "   Written By Teong Thew (Dept. Psychiatry), 1989 - 1995",
    "   Solaris 2.x port and Maintenance By Daniel Widyono (Dept. Psychiatry), 1995 - 1998",
    "   Currently maintained by Paul Hughett, PhD (Brain Behavior Lab) since 1998"
  };

  get_strs_dim( (Xv_Window) canvas,
		strs, SETUP_MAIN_CANVAS_MESSAGE_NUM_STRS,
		&max_width, &height, &line_height );

  y = 20 + line_height / 2;
  for (lcv = 0; lcv < SETUP_MAIN_CANVAS_MESSAGE_NUM_STRS; lcv++)
  {
    writeTextToXY( (Xv_Window) canvas, (Pixmap)NULL, 1, 20, y, strs[lcv] );
    y += line_height;
  }

  return;
}


/*------------------------------------------------------------------------
main_canvas_repaint_proc - ???
-------------------------------------------------------------------------*/

void
main_canvas_repaint_proc( canvas, paint_window, repaint_area )
     Canvas	canvas;
     Xv_Window	paint_window;	/* (Pixwin *)? */
     Rectlist	*repaint_area;
{
  setup_main_canvas_message( canvas );

  return;
}


/*------------------------------------------------------------------------
about_ria_proc - ???
-------------------------------------------------------------------------*/

void
about_ria_proc( 
     Menu menu,
     Menu_item item)
{
  Frame *frame;
  struct prog_rec *prog;
  char *fn;

  frame = (Frame *) main_display(2,NULL,0,0);
  prog = (struct prog_rec *) xv_get(*frame,WIN_CLIENT_DATA);
  fn = (char *) get_prog_def_fn(prog,ABOUT_RIA_FN);
  enter_display_text_file(*frame,1,NULL,fn);
  c_free(fn);
}


/*------------------------------------------------------------------------
beginner_help_proc - ???
-------------------------------------------------------------------------*/

void
beginner_help_proc( 
     Menu menu,
     Menu_item item)
{
  Frame *frame;
  struct prog_rec *prog;
  char *fn;

  frame = (Frame *) main_display(2,NULL,0,0);
  prog = (struct prog_rec *) xv_get(*frame,WIN_CLIENT_DATA);
  fn = (char *) get_prog_def_fn(prog,BEGINNER_HELP_FN);
  enter_display_text_file(*frame,1,NULL,fn);
  c_free(fn);
}


/*------------------------------------------------------------------------
program_exit_proc - ???
-------------------------------------------------------------------------*/

void
program_exit_proc(
     Menu menu,
     Menu_item item)
{
  main_display( 4, NULL, 0, 0 );
  return;
}


/*------------------------------------------------------------------------
select_fns_menu_proc - ???
-------------------------------------------------------------------------*/

static void
select_fns_menu_proc( menu, item )
     Menu menu;
     Menu_item item;
{
  struct prog_rec	*prog;
  Frame			*frame;

  if ((frame = (Frame *) main_display(2,NULL,0,0)) != NULL)
  {
    prog = (struct prog_rec *) xv_get(*frame,WIN_CLIENT_DATA);
    ask_files(1,*frame,"GET *.nhdr/*.ndat/*.img FILES",
	      NULL,1,1,open_fn_proc,prog,NULL,NULL,1,
	      prog->defaults.whch.hdr_fn,1,1,10,0,show_all_file_proc,prog,
	      2,
	      NULL,NULL,NULL,
	      "*.nhdr FN",prog->defaults.whch.hdr_fn,NULL,NULL,
	      prog->defaults.whch.dat_dir,NULL,NULL,
	      "*.ndat FN",prog->defaults.whch.dat_fn,NULL,NULL
	      );
  }

  return;
}


/*------------------------------------------------------------------------
main_event_proc - ???
-------------------------------------------------------------------------*/

void
main_event_proc( frame, event )
     Frame frame;
     Event *event;
{
  Menu menu;

  /*  DRW 19961111:  Keep ACTION_ADJUST and ACTION_SELECT here, since
   *    there are menus associated with the LMB and MMB, in subwindows.  */
  if (event_action(event) == ACTION_MENU ||
      event_action(event) == ACTION_ADJUST ||
      event_action(event) == ACTION_SELECT)
  {
    if ( (menu = xv_get(frame, WIN_MENU)) )
    {
      if (event_is_down(event))
	menu_show( menu, frame, event, NULL );
    }
  }

  return;
}


/*------------------------------------------------------------------------
canvaspw_event_proc - ???

DRW 19961111:  Added menu event handler to canvas paint window
-------------------------------------------------------------------------*/

void
canvaspw_event_proc( canvas_pw, event )
     Xv_Window canvas_pw;	/* Canvas Paint Window, which actually
				 * gets the event (as opposed to the
				 * canvas window) */
     Event *event;
{
  Menu menu;
  int i, value;

#if 0
  PRT( printf( "canvaspw_event_proc()\n" ); );
  PRT( if (event_action(event) == ACTION_MENU)
         printf( "Menu button event.\n" );
       if (event_is_down(event))
         printf( "Down event.\n" ); );
#endif      


  if ((event_action(event) == ACTION_MENU) && /* Only Menu button */
      (event_is_down(event)) &&	/* Only when depressed, not released */
      (menu = xv_get( canvas_pw, WIN_CLIENT_DATA ))) /* Menu exists? */
  {
    menu_show( menu, canvas_pw, event, NULL );
  }

  else {
    value = event_action(event);

    if (event_meta_is_down(event))
    {
      for (i = 0; i < PROG_EVENT_PROC_NUM_ACTIONS; i++)
      {
	if (actions[i].key == value)
	{
	  actions[i].proc(NULL,NULL,NULL, canvas_pw, event);
	  break;
	}
      }
    }
  }
  return;
}


/*------------------------------------------------------------------------
prog_event_proc - Handle Notifier dispatches in main frame.

DRW 19970228: Cleaned up Notifier-related code & algorithm.
-------------------------------------------------------------------------*/

Notify_value
prog_event_proc( frame, event, arg, type )
     Notify_client	frame;
     Event		*event;	/* Should be Notify_event, but bug in
				   Xview headers causes all sorts of
				   problems using event_action(), etc. */
     Notify_arg		arg;
     Notify_event_type	type;
{
  Menu menu;
  int lcv, value;



  value = event_action(event);

  /*
   * DRW 19970228: Bug fix.  If event is KBD_USE, no need to grab
   *    keyboard focus (that's redundant).  Separate idea of KBD_USE
   *    and LOC_WINENTER.  I believe the original intention was to make
   *    sure we always got focus when our window was entered, whatever
   *    the setting of the Window Manager Properties.
   *
   * FIXME:
   * To properly implement this, the code will have to be able to determine
   *   in what mode the WM is currently, and whether or not we have moved
   *   from one of our own windows, or from that of another application.
   *   If the latter, we should not do anything (let the WM decide what is
   *   right).  If the former, i.e. we move from an NRIA image window to the
   *   NRIA main frame, AND the WM is *NOT* in focus-follows-mouse mode,
   *   then we need to actively grab the focus.  This allows us to work 
   *   under the Motif Window Manager, coincidentally.
   */

  if (event_action(event) == KBD_USE) {
    return notify_next_event_func( frame, (Notify_event) event, arg, type );
  }

  if (event_action(event) == LOC_WINENTER)
  {
    grap_keyboard(frame);
    return notify_next_event_func( frame, (Notify_event) event, arg, type );
  }


  /*  DRW 19961111:  Removed ACTION_SELECT and ACTION_ADJUST.  Only
   *    display menu when ACTION_MENU invoked.  */
  if (value == ACTION_MENU)
  {
    if ( (menu = (Menu) xv_get( (Frame)frame, XV_KEY_DATA, 1) ) )
    {
      if (event_is_down(event))
	menu_show( menu, (Frame) frame, event, NULL );
      return NOTIFY_DONE;
    }
  }

  if (event_shift_is_down(event))
  {

    if (event_ctrl_is_down(event))
    {
      for (lcv = 0; lcv < PROG_EVENT_PROC_NUM_TOP_ACTIONS; lcv++)
      {
	if (top_actions[lcv].key == value)
	{
	  top_actions[lcv].proc(NULL,NULL,NULL,frame,event);
	  break;
	}
      }

      return NOTIFY_DONE;
    }

    if (event_action(event) == KEY_TOP(11))
    {
      main_display(4,NULL,0,0);
      return NOTIFY_DONE;
    }

    else if ((value == 'o') || (value == 'O'))
    {
      select_fns_menu_proc(0, 0);
    }

    return NOTIFY_DONE;
  }

  if (event_meta_is_down(event))
  {
    PRT1(printf("prog_event_proc...3...value = %d num_actions = %d\n",
		value,PROG_EVENT_PROC_NUM_ACTIONS));
    for (lcv = 0; lcv < PROG_EVENT_PROC_NUM_ACTIONS; lcv++)
    {
      PRT1(printf("%d)...key = %d\n",lcv,actions[lcv].key));
      if (actions[lcv].key == value)
      {
	actions[lcv].proc(NULL,NULL,NULL,frame,event);
	break;
      }
    }
  }

#if 0
  if ((value == 'q') || (value == 'Q'))
    program_exit_proc( NULL, NULL );
#endif

  return NOTIFY_DONE;
}


/*------------------------------------------------------------------------
main_destroy_func - ???
-------------------------------------------------------------------------*/

Notify_value
main_destroy_func( win, status )
     Xv_window win;
     Destroy_status status;
{
  int result;
  struct prog_rec *prog;

  PRT1(printf("main_destroy_func, status: ");
       if (status & DESTROY_CHECKING)
       {
	 printf("checking ");
       }
       if (status & DESTROY_CLEANUP)
       {
	 printf("cleanup ");
       }
       if (status & DESTROY_SAVE_YOURSELF)
       {
	 printf("save_yourself ");
       }
       if (status & DESTROY_PROCESS_DEATH)
       {
	 printf("dying ");
       }
       printf("\n");
       );
	 

  /*  User-called exit; make sure we have saved everything.  This should
   *    perform an actual check, so that if no images are opened, we will
   *    not be asked this question.  */
  if (status == DESTROY_CHECKING)
  {
    PRT1(printf("main_destroy_func..DESTROY_CHECKING\n"));

    (void) xv_create( win, NOTICE,
		      NOTICE_MESSAGE_STRINGS,
		      "Really Quit ?",
		      NULL,
		      NOTICE_BUTTON_YES, "Yes",
		      NOTICE_BUTTON_NO, "Cancel",
		      NOTICE_STATUS, &result,
		      XV_SHOW, TRUE,
		      NULL );

    if (result == NOTICE_NO)
    {
      notify_veto_destroy( win );
      return NOTIFY_DONE;
    }

    /*  Yes, really quit, so let's clean up.  */
    prog = (struct prog_rec *) xv_get( win, WIN_CLIENT_DATA );
    if (prog != NULL)
    {

      /*  Try to clean up.  If we get an error, give the user a chance
       *    to back out of quitting.  */

      if (cleanup_prog( prog, 0 ) != NO_ERROR)
      {

	(void) xv_create( win, NOTICE,
			  NOTICE_MESSAGE_STRINGS,
			  "Ignore the error... Quit anyway ?",
			  NULL,
			  NOTICE_BUTTON_YES, "Yes",
			  NOTICE_BUTTON_NO, "No",
			  NOTICE_STATUS, &result,
			  XV_SHOW, TRUE,
			  NULL );

	if (result == NOTICE_NO)
	{
	  notify_veto_destroy( win );
	  return NOTIFY_DONE;
	}

	/* Force cleaning up; ignore all errors (can't close files, etc.) */

	cleanup_prog( prog, 2 );

      }	/* Problem cleaning up prog structure */

    } /* Had to clean up prog structure */

    PRT1(printf("main_destroy_func..done\n"));

    return NOTIFY_DONE;
  }

  else if (status == DESTROY_CLEANUP)
  {
    PRT1(printf("main_destroy_func..DESTROY_CLEANUP\n"));
    return notify_next_destroy_func(win,status);
  }

  /* Supposed to save command-line arguments, etc.  This is called
   *    when Window Manager tells all clients to save their current
   *    state to be recalled when re-started.  For instance, we could
   *    keep track of which images were open (not recommended).  */
  else if (status == DESTROY_SAVE_YOURSELF)
  {
    PRT1(printf("main_destroy_func..DESTROY_SAVE_YOURSELF\n"));
  }

  else
  {
    PRT1(printf("DESTROY_CLEANUP..1.\n"));
    prog = (struct prog_rec *) xv_get(win,WIN_CLIENT_DATA);
    if (prog != NULL)
      cleanup_prog(prog,0);
  }
  
  return NOTIFY_DONE;
}


/*-----------------------------------------------------------------------
main_display - ???

FIXME:  This function is an ugly conglomerate of distinct operations
selected by the value of the select argument; they really ought to be
separated or at least documented.  The server and screen_num arguments
are ignored unless select == SETUP_MAIN_DISPLAY.
------------------------------------------------------------------------*/

Frame *
main_display(
     int select,
     struct prog_rec *prog,
     Xv_Server server,
     int screen_num)
{
  static Frame frame = 0;
  Menu menu;
  Rect *rr;
  static Xv_Font xvfont = (Xv_Font)NULL;
  static Canvas canvas = (Canvas)NULL;
  char *str,*label;
  Xv_Screen screen;
  Xv_Window root;
  Cms cms;
  /* NEVER USED?   Xv_Singlecolor bg,fg;  */
  struct intensity_rec *intensity;


  switch (select)
  {

  case MAIN_DISPLAY_CANVAS :
    TRACE( printf( "main_display( MAIN_DISPLAY_CANVAS )\n" ); );
    return (Frame *) &canvas;

  case GET_MAIN_DISPLAY_FRAME :
    TRACE( printf( "main_display( GET_MAIN_DISPLAY_FRAME )\n" ); );
    return (Frame *) &frame;

  case MAIN_DISPLAY_FONT :
    TRACE( printf( "main_display( MAIN_DISPLAY_FONT )\n" ); );
    if (xvfont == (Xv_Font)NULL)
    {
      xvfont = (Xv_Font) xv_get(frame,XV_FONT);
    }
    return (Frame *) &xvfont;

  case SETUP_MAIN_DISPLAY :
    TRACE( printf( "main_display( SETUP_MAIN_DISPLAY )\n" ); );
    if (frame != 0)
      break;
    canvas = (Canvas)NULL;
    screen = (Xv_Screen) xv_get(server,SERVER_NTH_SCREEN,screen_num);
    root = (Xv_Window) xv_get(screen,XV_ROOT);
    intensity =
      prog->win.def_color.pallettes[prog->win.def_color.cur_pallette];
    label = intensity->label;

    /*  Initialize default ColorMap Segment (cms)  */
    PRT(printf("	calling setup_color_segment(), label = \"%s\"\n",
	       label););
    cms = (Cms) setup_color_segment(screen,label,&(intensity->color));
    PRT(printf("	CMS = %p\n", (void *) cms););
    /*  Initialize MAIN FRAME (background window without text)  */
    PRT(printf("	returned from setup_color_segment()\n"););

    frame = xv_create( root, FRAME,
		       WIN_CLIENT_DATA, prog,

		       /* FRAME_INHERIT_COLORS, TRUE, */
		       /*  DRW 19961111:  Commented out, to use system colors
			   instead of yellow on blue -- easier to read  */
		       /* WIN_BACKGROUND_COLOR,
			  1 + intensity->color.num_system_colors, */
		       /* WIN_FOREGROUND_COLOR,
			  2 + intensity->color.num_system_colors, */
		       /* WIN_DYNAMIC_VISUAL, TRUE, */

		       NULL );

    if (frame == 0)
    {
      printf("	Failed to create main frame.  Aborting."); fflush(stdout);
      exit(0);
    }
    set_win_foreground_to_default_gc(frame);
    set_win_background_to_default_gc(frame);
    get_ria_icon(frame);
    set_icon_label(frame,"NRIA");
    xv_set(frame,

	   WIN_CONSUME_EVENTS,
	   WIN_ASCII_EVENTS, WIN_TOP_KEYS, MS_LEFT, MS_MIDDLE, MS_RIGHT,
	   LOC_WINENTER, /* LOC_WINEXIT, */
	   NULL,

	   WIN_IGNORE_EVENTS,
	   WIN_UP_ASCII_EVENTS,
	   NULL,

	   NULL );
    notify_interpose_event_func((Notify_client) frame,
				(Notify_func) prog_event_proc,
				NOTIFY_SAFE);

#if 0
    xvfont = (Xv_Font) xv_find(frame,FONT,FONT_NAME,"courier",0);
    if (xvfont != NULL)
    {
      TRACE(printf("set courier...\n"));
      xv_set(frame,XV_FONT,xvfont,0);
    }
#endif

    win_clear(frame, (Pixmap)NULL, 1);
    if (concat_strs(&str,5,IMAGE_ANALYSIS_PROGRAM,"-",prog->prog_id,
		    " : DEFAULT FILE = ",prog->defaults_fn) != NO_ERROR)
      cp_str(&str,IMAGE_ANALYSIS_PROGRAM);

    xv_set( frame,
	    FRAME_LABEL,str,
	    FRAME_NO_CONFIRM,TRUE,
	    WIN_CMS, cms,
	    XV_X, 0,
	    XV_Y, 0,
	    NULL );
    c_free(str);
    TRACE(printf("Created main frame %p, depth = %d, cms = %p\n",(void *)frame,
		 (int) xv_get( frame, WIN_DEPTH ),
		 (void *) xv_get( frame, WIN_CMS )); );

    /*  Initialize main window background pop-up menu (RMB)  */
    menu = xv_create((Xv_object)NULL,MENU,
		     MENU_TITLE_ITEM,"MAIN MENU",

		     MENU_ITEM,
		     MENU_STRING,SELECT_FNS_MENU,
		     MENU_NOTIFY_PROC, select_fns_menu_proc, NULL,

		     MENU_ITEM,
		     MENU_STRING,IMPORT_IMG,
		     MENU_NOTIFY_PROC, import_image_file_from_main_proc, NULL,

		     MENU_ITEM,
		     MENU_STRING,ABOUT_RIA,
		     MENU_NOTIFY_PROC, about_ria_proc, NULL,

		     MENU_ITEM,
		     MENU_STRING,BEGINNER_HELP,
		     MENU_NOTIFY_PROC, beginner_help_proc, NULL,

		     MENU_ITEM,
		     MENU_STRING,EXIT_PROGRAM,
		     MENU_NOTIFY_PROC, program_exit_proc, NULL,

		     NULL);
    TRACE(printf("main...1..1\n"));

    canvas = xv_create( frame, CANVAS,
			CANVAS_RETAINED, FALSE,
			XV_SHOW, FALSE,
			WIN_CMS, cms,
			XV_X, 0,
			XV_Y, 0,
			FRAME_INHERIT_COLORS, TRUE,
			/*  DRW 19961111:  Commented out, to use system colors
			    instead of yellow on blue -- easier to read  */
			/* WIN_BACKGROUND_COLOR,
			   1 + intensity->color.num_system_colors, */
			/* WIN_FOREGROUND_COLOR,
			   2 + intensity->color.num_system_colors, */
			NULL );

    if (canvas != (Canvas)NULL)
    {
#if 0
      set_win_foreground_to_default_gc( canvas );
      set_win_background_to_default_gc( canvas );
#endif
      /*  DRW 19961111  Add menu event handler to canvas paint window  */
      xv_set( canvas_paint_window(canvas),
	      WIN_CONSUME_EVENTS, WIN_ASCII_EVENTS, WIN_MOUSE_BUTTONS, NULL,
	      WIN_EVENT_PROC, canvaspw_event_proc,
	      NULL );
      xv_set( canvas, CANVAS_FIXED_IMAGE, TRUE,
	      CANVAS_REPAINT_PROC, main_canvas_repaint_proc,
	      CANVAS_AUTO_SHRINK, FALSE,
	      CANVAS_AUTO_EXPAND, FALSE,
	      NULL );
      setup_main_canvas_message(canvas);
      PRT(printf("Created main canvas %p\n", (void *)canvas););
    }

    /* When debugging, let's keep the background window small and out
	of the way.  */
    if (!debugFlag) {
      set_big_win( frame );
      rr = (Rect *) xv_get( frame, XV_RECT );
      xv_set( frame,
	      XV_WIDTH, rr->r_width - 80,
	      XV_HEIGHT, rr->r_height - 80,
	      NULL );
    }

    if (menu != 0) {
      xv_set(frame,XV_KEY_DATA,1,menu,0);
      /*  DRW 19961111  Added menu event handler to canvas paint window  */
      if (canvas)
	xv_set( canvas_paint_window(canvas),
		WIN_CLIENT_DATA, menu,
		NULL );
    }

    TRACE(printf("main...out\n"));
    break;


  case SHOW_MAIN_DISPLAY :
    TRACE( printf( "main_display( SHOW_MAIN_DISPLAY )\n" ); );

    if (frame == 0)
      break;

    xv_set( canvas,
	    XV_SHOW, TRUE,
	    NULL );

    /*  Pop-up file menu when ready  */
    (void) notify_set_itimer_func( (Notify_client) frame,
				   (Notify_func) get_fns_menu,
				   ITIMER_REAL, &ITIMER_ONCE, NULL );

    /*  Specialized destroy function allows us to prevent spurious quits  */
    notify_interpose_destroy_func((Notify_client) frame,
				  (Notify_func) main_destroy_func);

    /*  Turn control over to event loop of Notifier  */
    window_main_loop(frame);
    break;


  case CLEAN_UP_MAIN_DISPLAY :
    TRACE( printf( "main_display( CLEAN_UP_MAIN_DISPLAY )\n" ); );

    if (frame == 0)
      return &frame;

    xv_destroy_safe(frame);
    break;
  }

  return &frame;
}


/*------------------------------------------------------------------------
cleanup_proc - Clean up prior to NRIA exit or abort???
-------------------------------------------------------------------------*/

int
cleanup_prog( prog, must_quit )
     struct prog_rec *prog;
     int must_quit;
{
  int err;

  TRACE(printf("before wr_all_dat_output total wins = %d...\n",
	       prog->win.total_wins));
  TRACE(printf("...cleanup_prog...must_quit = %d\n",must_quit));


  err = close_all_files(&(prog->input),must_quit);
  if ((err != NO_ERROR) && !must_quit)
    return err;

  TRACE(printf("close_all_files...end\n"));
  free_prog_rec(prog);
  exit_handler(2,NULL,NULL);
  TRACE(printf("after free_prog_rec\n"));

  return NO_ERROR;
}

