/****************************************************************************
display_select_slice.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: display_select_slice.c,v 1.10 1999/07/07 17:41:15 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/cms.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/notify.h>
#include <xview/wmgr.h>

#include "mri_format.h"
#include "geometry.h"

#include "nria_procedures.h"


/*
#define TRACE(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
#define PRT2(x) x; fflush(stdout);
#define PRT4(x) x; fflush(stdout);
#define PRT5(x) x; fflush(stdout);
#define PRT6(x) x; fflush(stdout);
#define PRT7(x) x; fflush(stdout);
#define PRT9(x) x; fflush(stdout);
#define PRT3(x) x; fflush(stdout);
#define PRT8(x) x; fflush(stdout);
#define PRT1(x) x; fflush(stdout);
*/
#define TRACE(x)
#define PRT(x)
#define PRT1(x)
#define PRT2(x)
#define PRT3(x)
#define PRT4(x)
#define PRT5(x)
#define PRT6(x)
#define PRT7(x)
#define PRT8(x)
#define PRT9(x)




#define AUTO_STR "AUTO Sh-Esc"
#define STOP_AUTO_STR "STOP AUTO=Sh-Esc"

struct auto_slice_rec
   {
    unsigned long num_args;
    unsigned long **args;
   };



/* Functions defined in this file */
static void init_auto_slice_rec(struct auto_slice_rec *autom);
static unsigned long *find_auto_arg(struct auto_slice_rec *autom,
    unsigned long *arg);
static struct auto_slice_rec *get_auto_ptr(void);
int auto_on_proc(struct display_win_rec *win, int which);
static int set_auto_on(unsigned long *arg);
static int stop_auto(unsigned long *arg);
static int set_auto_on_proc(struct image_display_window *win,
    unsigned long *arg);
int win_auto_arg(struct image_display_window *win, unsigned long *auto_on);
void auto_slice_proc(Panel_item item, Event event, void *arg, Xv_Window window,
    Event *last_event, int dummy);
int update_cur_slice_num(struct image_display_window *win);
int win_release_slice_roi_win(struct image_display_window *win);
int win_get_slice_roi_win(struct image_display_window *win);
int win_before_enter_next_slice(struct image_display_window *win);
static int WinAdvanceSlice(struct image_display_window *win, int slice_offset,
    int is_locate_scan, int is_rel);
int WinSliceProcessed(struct image_display_window *win,
    unsigned long which_processed4);
static int WinClearSliceProcessedStatus(struct image_display_window *win,
    unsigned long which_processed4);
static int WinMultiEchoClearSliceProcessedStatus(
   struct image_display_window *win, unsigned long which_processed4);
static int WinClearDrawListProcessedStatus(struct image_display_window *win,
    unsigned long which_processed4);
static int WinAdvanceDrawListSlice(struct image_display_window *win,
    struct image_display_window *headwin, void *dummy1, void *dummy2,
    void *dummy3, void *dummy4, void *dummy5, struct list_rec *win_list);
static int WinAdvanceStack(struct image_display_window *win,
    struct hdr_scan_entry_rec *last_slice, struct list_rec *list, int is_list);
static int WinNextSlice(struct image_display_window *win, int slice_offset,
    int is_locate_scan, int is_rel);
static int WinSetStackSliceOrScan(struct image_display_window *win);
int WinStackSliceOrScan(struct image_display_window *win);
static int WinNewMultiEchoSliceColorData(struct image_display_window *win,
    int is_processed_draw_list);
static int WinNewMultiEchoMultiScreenCanvas(struct image_display_window *win);
static int WinDisplaySlice(struct image_display_window *win);
void next_group_slice_proc(Panel_item item, Event *event, void *arg,
    Xv_Window window, Event *last_event, int val);
static void more_select_slice_proc(Panel_item item, Event *event);
static void select_slice_item_proc(Panel_item item, Event *event);
static int is_scan_order(struct select_slice_win *swin, Panel_item item,
    Event *event);
void select_slice_slider_proc(Panel_item item, int value, Event *last_event,
    void *arg, Xv_Window window);
void select_slice_order_proc(Menu menu, Menu_item item, void *arg,
    Xv_Window window, Event *last_event);
static void select_slice_done_proc(Frame frame);
int setup_select_slice_win(struct select_slice_win *win,
    struct display_win_rec *dwin);
static void free_select_slice_win(struct select_slice_win *win);
void copy_select_slice_win(struct select_slice_win *dst,
    struct select_slice_win *src, struct display_win_rec *dwin);
void select_slice_proc(Menu menu, Menu_item item, void *arg, Xv_Window window,
    Event *last_event);



/*------------------------------------------------------------------
init_auto_slice_rec - ???
------------------------------------------------------------------*/

static void
init_auto_slice_rec(autom)
 struct auto_slice_rec *autom;
{
 autom->num_args = 0;
 autom->args = NULL;
}


/*------------------------------------------------------------------
find_auto_arg - ???
------------------------------------------------------------------*/

static unsigned long *
find_auto_arg(autom,arg)
 struct auto_slice_rec *autom;
 unsigned long *arg;
{
 unsigned long lcv;

 for (lcv = 0; lcv < autom->num_args; lcv++)
    if (autom->args[lcv] == arg)
       return autom->args[lcv];
 return NULL;
}


/*------------------------------------------------------------------
get_auto_ptr - ???
------------------------------------------------------------------*/

static struct auto_slice_rec *
get_auto_ptr()
{
 static struct auto_slice_rec autom;
 static int first = 1;

 if (first)
    {first = 0;
     init_auto_slice_rec(&autom);
    }
 return &autom;
}


/*------------------------------------------------------------------
auto_on_proc - ???
------------------------------------------------------------------*/

int
auto_on_proc(win,which)
 struct display_win_rec *win;
 int which;
{
 int value;

 value = (int) xv_get(win->select_slice.auto_offset_item,PANEL_VALUE);
 TRACE(printf("auto_on_proc...auto_on = %d\n",win->select_slice.auto_on);)
 select_slice_slider_proc(0,value,NULL,win,
                              win->frame_canvas.canvas);
 return NO_ERROR;
}


/*------------------------------------------------------------------
set_auto_on - ???
------------------------------------------------------------------*/

static int
set_auto_on(arg)
 unsigned long *arg;
{
 unsigned long *the_arg;
 struct auto_slice_rec *autom;
 int err;

 if ((autom = get_auto_ptr()) == NULL) return CANCEL;
 if ((the_arg = find_auto_arg(autom,arg)) != NULL) return NO_ERROR;
 if ((err = get_ptr(&(autom->args),&(autom->num_args),1,
      (size_t) sizeof(*(autom->args)))) != NO_ERROR)
    return err;
 autom->args[autom->num_args - 1] = arg;
 TRACE(printf("set_auto_on...autom->num_args = %u\n",autom->num_args);)
 (void) notify_set_itimer_func((Notify_client) arg,
             (Notify_func) auto_on_proc,ITIMER_REAL,
             &NOTIFY_POLLING_ITIMER,(struct itimerval *) 0);

 return NO_ERROR;
}


/*------------------------------------------------------------------
stop_auto - ???
------------------------------------------------------------------*/

static int 
stop_auto(arg)
 unsigned long *arg;
{
 int err;
 struct auto_slice_rec *autom;

 if ((autom = get_auto_ptr()) == NULL) return CANCEL;
 TRACE(printf("stop_auto...autom->num_args = %u\n",autom->num_args);)
 if ((err = DropHandleAryPtr(&(autom->args),&(autom->num_args),arg))
         == NO_ERROR)
    {
     TRACE(printf("stop_auto..yes.autom->num_args = %u\n",autom->num_args);)
     (void) notify_set_itimer_func((Notify_client) arg,
             NOTIFY_FUNC_NULL,ITIMER_REAL,
             (struct itimerval *) 0,(struct itimerval *) 0);
    }
 return err;
}


/*------------------------------------------------------------------
set_auto_on_proc - ???
------------------------------------------------------------------*/

static int 
set_auto_on_proc(win,arg)
 struct image_display_window *win;
 unsigned long *arg;
{
 TRACE(printf("set_auto_on_proc win = %u arg = %u\n",win,arg))
 win->dwin->select_slice.auto_on = arg;
 if (arg)
    {if (win->dwin->select_slice.auto_item != 0)
        xv_set(win->dwin->select_slice.auto_item,
             PANEL_LABEL_STRING,STOP_AUTO_STR,0);
    }
   else if (win->dwin->select_slice.auto_item != 0)
    xv_set(win->dwin->select_slice.auto_item,PANEL_LABEL_STRING,AUTO_STR,0);
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
pr_item - ???

FIXME:  The function pr_hdr_scan_entry_rec called below appears not to
exist within NRIA.  Commented out for now.  -PWH 19990405
-----------------------------------------------------------------------*/

/****************
void
pr_item(win)
 struct image_display_window *win;
{
 struct img_buf_rec *img;
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return NO_ERROR;
 pr_hdr_scan_entry_rec(img->cur_slice);
}
*****************/


/*------------------------------------------------------------------
win_auto_arg - ???

FIXME:  It seems rather strange that this function should take a
pointer to an unsigned long as its second argument, when only a
boolean value would seem necessary.  There might be a bug lurking
here.  -PWH 19980608
------------------------------------------------------------------*/

int 
win_auto_arg(win,auto_on)
 struct image_display_window *win;
 unsigned long *auto_on;
{
 struct image_display_window *nextwin;
 struct list_rec *list,*head;
 int err;

 PRT(printf("\nwin_auto_arg...auto_on = %u\n",auto_on))
 if (auto_on) err = set_auto_on(auto_on);
   else err = stop_auto(win->dwin->select_slice.auto_on);
 if (err != NO_ERROR) return err;
 list = win->list.next;
 head = (struct list_rec *) head_list(list);
 list = head;
 do
    {
     nextwin = (struct image_display_window *) list->item;
     win_apply_partner_linked_wins(nextwin,set_auto_on_proc,auto_on,0);
     list = list->next;
    }
 while (list != head);
 PRT(printf("win_auto_arg...end\n"))
 return err;
}


/*------------------------------------------------------------------
auto_slice_proc - ???

FIXME:  There are a couple of potentially nasty bugs lurking here.

1.  This function used to be inplicitly declared type int.  Two of
its return statements returned ints; one returned nothing.  As near
as I can tell, neither of its callers expects a return value, so I
have made it type void and commented out the return values.

2.  This function is called with 6 arguments from process_key.c, and
with two arguments from display_select_slice.c, apparently to reuse
the same code to respond to a key press, or a menu button press
(which use different arguments).  The value of the argument item
distinguishes the two cases.  I have left this as is, but things
could get really nasty on some architectures.     -PWH 19980608
------------------------------------------------------------------*/

void 
auto_slice_proc(item,event,arg,window,last_event,dummy)
 Panel_item item;
 Event event;
 void *arg;
 Xv_Window window;
 Event *last_event;
 int dummy;
{
 struct display_win_rec *win1;
 struct image_display_window *win;
 Panel panel;

 if (item == 0)
    win1 = (struct display_win_rec *) arg;
 else {
    panel = (Panel) xv_get(item, XV_OWNER);
    if (panel == 0) return /* CANCEL */;
    win1 = (struct display_win_rec *) xv_get(panel, WIN_CLIENT_DATA);  }

 win = win1->owner;
 win->dwin = win1;
 if (!(win->type2 & MANY_SLICES_WINDOWS)) return;

 /*** FIXME:  Added a cast to (void *) in the code just below to
      avoid a compiler warning.  This code doesn't entirely make
      sense, so there might be a bug lurking.  -PWH 19980608
 ***/
 if (win1->select_slice.auto_on != 0)
    win_auto_arg(win,0);
 else
    win_auto_arg(win,(void *)win->dwin);

 return /* NO_ERROR */;
}


/*------------------------------------------------------------------
update_cur_slice_num - ???
------------------------------------------------------------------*/

int             /* FIXME: void? */
update_cur_slice_num(win)
 struct image_display_window *win;
{
 unsigned int slice_num;
 struct img_buf_rec *img;
 long total,max;

 if (win->dwin->select_slice.slice_num_slider != 0)
    {
     if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
     slice_num = img->cur_slice->slice_num + 1;
     xv_set(win->dwin->select_slice.slice_num_slider,PANEL_VALUE,slice_num,0);
     total = total_slice_cuts(img);
     max = (long) xv_get(win->dwin->select_slice.slice_num_slider,
                     PANEL_MAX_VALUE);
     if (total != max) 
        xv_set(win->dwin->select_slice.slice_num_slider,
                     PANEL_MAX_VALUE,total,0);
    }
 return NO_ERROR;
}


/*------------------------------------------------------------------
win_release_slice_roi_win - ???
------------------------------------------------------------------*/

int 
win_release_slice_roi_win(win)
 struct image_display_window *win;
{
 if (win->control.sroi.sroi_win != NULL)
    {PRT1(printf("win_release_slice_roi_win sroi_win = %u\n",
                win->control.sroi.sroi_win))
     if (win->control.sroi.sroi_win->num_wins == 0)
        {
         LoopCurCurve(&(win->control.sroi.sroi_win->slice_roi));
        }
     DisassociateWinWithSliceRoiWinRec(win->control.sroi.sroi_win,win);
     win->control.sroi.sroi_win = NULL;}
 return NO_ERROR;
}


/*------------------------------------------------------------------
win_get_slice_roi_win - ???
------------------------------------------------------------------*/

int 
win_get_slice_roi_win(win)
 struct image_display_window *win;
{
 struct img_buf_rec *img;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;

 PRT1(printf("win_get_slice_roi_win...sizeof(slice_roi) = %u\n",
       sizeof(struct slice_roi_rec)))
 if (win->type2 & IMAGE_WINDOWS)
    {win->control.sroi.sroi_win = img->cur_slice->slice->slice_roi_win;
     PRT1(printf("num_wins = %d  num = %d  %u\n",
                win->control.sroi.sroi_win->num_wins,
            (int) img->cur_slice->num,win->control.sroi.sroi_win))
     AssociateWinWithSliceRoiWinRec(win->control.sroi.sroi_win,win);}
   else if (win->type2 & HISTOGRAM_WINDOW)
    {if (win->type2 & HISTOGRAM_1D_WINDOWS)
        win->control.sroi.sroi_win = &(img->file->histo_params.slice_roi_win_1d);
       else win->control.sroi.sroi_win = &(img->file->histo_params.slice_roi_win_2d);
     AssociateWinWithSliceRoiWinRec(win->control.sroi.sroi_win,win);
     PRT1(printf("win_get_slice_roi_win...HISTOGRAM_WINDOW\n"))
    }
   else
    {win->control.sroi.sroi_win = NULL;
     return CANCEL;}
 PRT1(printf("win_get_slice_roi_win...3\n"))

 init_input_pt_state(&(win->control));
 win_set_control_pen_type(win,0);
 win_set_pt_status(win);

 return NO_ERROR;
}


/*------------------------------------------------------------------
win_before_enter_next_slice - ???
------------------------------------------------------------------*/

int 
win_before_enter_next_slice(win)
 struct image_display_window *win;
{
 struct roi_rec *roi;
 struct color_data_rec *color_data;

 PRT1(printf("win_before_enter_next_slice win->status4 = %x\n",(unsigned long) win->status4));
 PRT1(printf(" win->type2 = %x\n",(unsigned long) win->type2));
 win->process_status4 |= WIN_ADVANCE_SLICE_PROCESSED;
 if (win->type2 & HISTOGRAM_WINDOW)
    {if (win->control.sroi.sroi_win == NULL) win_get_slice_roi_win(win);
    }
   else
    {WinDeleteMarkRoi(win,0);
     if (win->control.sroi.sroi_win != NULL)
        roi = get_cur_roi(&(win->control.sroi.sroi_win->slice_roi));
       else roi = NULL;
     PRT1(printf("before _release_slice_roi_win sroi_win = %u\n",
           win->control.sroi.sroi_win))
     win_release_slice_roi_win(win);
     win_get_slice_roi_win(win);
     PRT1(printf("1 sroi_win = %u\n",win->control.sroi.sroi_win))
     if (roi != NULL) win_make_roi_no_display(win,roi->label,1);
     clear_draw_masks(win->data.draw_masks,win->data.num_draw_masks);
     update_dp_delete_roi_menu(win,0);
    }
 PRT1(printf("win_before_enter_next_slice...\n"))

 if (WinFindColorDataRec1(win,PIXEL_TYPE_ID,0,&color_data,NULL) == NO_ERROR)
    if (color_data->_2d.status4 & BUF_NON_EMPTY)
       win->status4 |= RECOMPUTE_PIXEL_TYPES;
     
 if (win->overlay.item != NULL)
    {PRT4(printf("win->overlay.item = %u...RECOMPUTE_OVERLAY\n",win->overlay.item))
     win->status4 |= RECOMPUTE_OVERLAY;}
 PRT1(printf("win->status4 = %x\n",(unsigned long) win->status4));
 return NO_ERROR;
}


/*------------------------------------------------------------------
WinAdvanceSlice - ???
------------------------------------------------------------------*/

static int
WinAdvanceSlice(win,slice_offset,is_locate_scan,is_rel)
 struct image_display_window *win;
 int slice_offset,is_locate_scan,is_rel;
{
 struct img_buf_rec *img;
 char *str;
 int err;

 PRT2(printf("WinAdvanceSlice...\n"))
 win->process_status4 |= WIN_ADVANCE_SLICE_PROCESSED;
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 if (is_rel) str = " ";
   else str = "a";
 if ((err = LoadAllNextSliceEchoes(str,img,slice_offset,
           is_locate_scan,0)) == NO_ERROR)
    {
     WinMultiEchoRunProc(win,win_before_enter_next_slice,0,0);
     WinMultiEchoRunProc(win,win_filter_image_data,0,0);
     WinMultiEchoRunProc(win,win_multiple_set_frame_label,0,0);
    }

 PRT2(printf("WinAdvanceSlice...end\n"))
 return err;
}


/*------------------------------------------------------------------
WinSliceProcessed - ???
------------------------------------------------------------------*/

int 
WinSliceProcessed(win,which_processed4)
 struct image_display_window *win;
 unsigned long which_processed4;
{
 if (win->process_status4 & which_processed4) return NO_ERROR;
 PRT8(printf("WinSliceProcessed wid = %s which_processed4 = %x\n",win->owner->wid,which_processed4);)
 return CANCEL;
}


/*------------------------------------------------------------------
WinClearSliceProcessedStatus - ???
------------------------------------------------------------------*/

static int
WinClearSliceProcessedStatus(win,which_processed4)
 struct image_display_window *win;
 unsigned long which_processed4;
{
 win->process_status4 &= ~which_processed4;
 PRT8(printf("WinClearSliceProcessedStatus wid = %s st = %x which_processed4 = %x\n",win->owner->wid,win->process_status4,which_processed4);)
 return NO_ERROR;
}


/*------------------------------------------------------------------
WinMultiEchoClearSliceProcessedStatus - ???
------------------------------------------------------------------*/

static int
WinMultiEchoClearSliceProcessedStatus(win,which_processed4)
 struct image_display_window *win;
 unsigned long which_processed4;
{
 return WinMultiEchoRunProc(win,WinClearSliceProcessedStatus,
			    which_processed4,0);
}


/*------------------------------------------------------------------
WinClearDrawListProcessedStatus - ???
------------------------------------------------------------------*/

static int 
WinClearDrawListProcessedStatus(win,which_processed4)
 struct image_display_window *win;
 unsigned long which_processed4;
{
 struct strs_rec wids;
 unsigned long lcv;
 struct win_rec *rwin;

 if (win->draw_list.item == NULL) return NO_ERROR;
 init_strs_rec(&wids);
 WinRunProcInList(WIN_DRAW_LIST | WIN_DO_NOT_PROCESS_CUR_WIN,win,
                  WinDrawListWids,&wids,1,0,0,0,0);
 PRT8(printf("WinClearDrawListProcessedStatus...\n");)
 PRT8(pr_strs_rec(&wids);)
 for (lcv = 0; lcv < wids.num_strs; lcv++)
    if ((rwin = get_win_rec_by_wid_str(win->owner,wids.strs[lcv])) != NULL)
       {if (rwin->image_display != NULL)
           rwin->image_display->process_status4 &= ~which_processed4;

        if (rwin->histogram != NULL)
           rwin->histogram->process_status4 &= ~which_processed4;}

 free_strs_rec(&wids);
 return NO_ERROR;
}


/*------------------------------------------------------------------
WinAdvanceDrawListSlice - ???
------------------------------------------------------------------*/

static int 
WinAdvanceDrawListSlice(win,headwin,dummy1,dummy2,dummy3,dummy4,dummy5,win_list)
 struct image_display_window *win,*headwin;
 void *dummy1,*dummy2,*dummy3,*dummy4,*dummy5;
 struct list_rec *win_list;
{
 struct img_buf_rec *img,*img1; 
 long snum,scnum;
 unsigned long num_cuts;
 int err,is_locate_scan,is_changed,is_rel;
 struct file_register_rec *fco;
 struct vector_3d_rec v1;

 PRT8(printf("WinAdvanceDrawListSlice...win = %u headwin = %u\n",win,headwin))
 PRT8(printf("...wid = %s  headwin wid = %s\n",win->owner->wid,headwin->owner->wid))
 PRT8(printf("WinAdvanceDrawListSlice...process_status4 = %x\n",win->process_status4))
 if (!(win->process_status4 & WIN_ADVANCE_SLICE_PROCESSED)) 
    {win->process_status4 |= WIN_ADVANCE_SLICE_PROCESSED;
     if (((img = win_rec_cur_img_buf(win->owner)) == NULL) ||
         ((img1 = win_rec_cur_img_buf(headwin->owner)) == NULL))
        return CANCEL;
     err = NO_ERROR;
     is_locate_scan = 0;
     snum = img1->cur_slice->slice_num - img1->previous_slice->slice_num;
     PRT8(printf("cur snum = %d pre snum = %d snum = %d\n",
           img1->cur_slice->slice_num,img1->previous_slice->slice_num,snum))
     if (snum == 0)
        {scnum = img1->cur_slice->scan_num - img1->previous_slice->scan_num;
         if (scnum != 0) 
            {snum = scnum;
             is_changed = 0;
             is_locate_scan = 1;}
           else is_changed = 1;
        }
       else is_changed = 1;
     PRT8(printf("is_locate_scan = %d is_changed = %d snum = %d\n",
           is_locate_scan,is_changed,snum))
     if ((img->file == img1->file) || is_locate_scan)
        {PRT8(printf("img->file == img1->file) || is_locate_scan\n"))
         is_rel = 1;}
       else 
        if (is_changed &&
            ((fco = find_file_register_rec(img1->file,img->file,NULL,0)) != NULL))
         {PRT8(printf("fco->status = %x\n",(unsigned long) fco->co.status))
          PRT8(printf("reg.xyztoplane & reg.planetoxyz & coreg.planetoxyz\n"))
          PRT8(pr_3d_matrix(&(fco->co.reg.xyztoplane)))
          PRT8(pr_3d_matrix(&(fco->co.reg.planetoxyz)))
          PRT8(pr_3d_matrix(&(fco->co.coreg.planetoxyz)))
          setup_vector_3d_rec(&v1,.0,.0,(float) img1->cur_slice->slice_num);
          PRT8(pr_vector_3d_rec(&v1))
          trans_3d_pt(&(fco->co.reg.xyztoplane),&v1,&v1,0);
          PRT8(printf("img1 cur slice_num = %d\n",img1->cur_slice->slice_num))
          PRT8(pr_vector_3d_rec(&v1))
          trans_3d_pt(&(fco->co.coreg.planetoxyz),&v1,&v1,1);
          PRT8(pr_vector_3d_rec(&v1))
          snum = (long) v1.z;
          num_cuts = NumberOfSliceCuts(&(img->file->hdr.data_info));
          PRT8(printf("slice_num = %d num_cuts = %d  %d\n",snum,num_cuts,
                   snum % num_cuts))
          if (snum < 0)
             {snum = (-snum) % num_cuts;
              snum = num_cuts - snum - 1;}
            else if (snum >= num_cuts) snum %= num_cuts;
          PRT8(printf("slice_num = %d\n",snum))
          is_rel = 0;
         }

     if (is_changed)
        {PRT8(printf("snum = %d is_locate_scan = %d is_rel = %d\n",
                  snum,is_locate_scan,is_rel))
         err = WinAdvanceSlice(win,snum,is_locate_scan,is_rel);}
    }
   else
    {err = NO_ERROR;
     is_changed = 1;}
 if ((err == NO_ERROR) && is_changed && (win->draw_list.item != NULL))
    if (WinRunProcInList(WIN_DRAW_LIST | WIN_DO_NOT_PROCESS_CUR_WIN,win,
                WinSliceProcessed,WIN_ADVANCE_SLICE_PROCESSED,0,0,0,0,0) 
	!= NO_ERROR)
       {PRT8(printf("WinAdvanceDrawListSlice again...\n"))
        WinRunProcInList(WIN_DRAW_LIST | WIN_DO_NOT_PROCESS_CUR_WIN,win,
			 WinAdvanceDrawListSlice,win,0,0,0,0,0);}

 PRT8(printf("WinAdvanceDrawListSlice is_changed = %d err = %d\n",is_changed,err))
 return err;
}


/*------------------------------------------------------------------
WinAdvanceStack - ???
------------------------------------------------------------------*/

static int 
WinAdvanceStack(win,last_slice,list,is_list)
 struct image_display_window *win;
 struct hdr_scan_entry_rec *last_slice;
 struct list_rec *list;
 int is_list;
{
 struct list_rec *next;
 struct img_buf_rec *cur_img,*img;
 struct hdr_scan_entry_rec *cur_slice;
 long snum,scnum,nsnum,nscnum,num_cuts,num_scans,num_echoes;
 struct image_display_window *nextwin;
 int err;

 PRT9(printf("WinAdvanceStack...\n"))
 if ((cur_img = win_rec_cur_img_buf(win->owner)) == NULL) return NO_ERROR;
 cur_slice = cur_img->cur_slice;
 if (is_list && (win->status4 & STACK_WIN_COSECUTIVE))
    {nsnum = (long) cur_slice->slice_num;
     nscnum = (long) cur_slice->scan_num;
     PRT9(printf("nsnum = %d  nscnum = %d\n",nsnum,nscnum))
    }
   else
    {snum = (long) cur_slice->slice_num - (long) last_slice->slice_num;
     scnum = (long) cur_slice->scan_num - (long) last_slice->scan_num;}
 next = list->next;
 num_cuts = NumberOfSliceCuts(&(cur_img->file->hdr.data_info));
 num_scans = NumberOfSliceScans(&(cur_img->file->hdr.data_info));
 num_echoes = cur_img->file->hdr.data_info.num_echoes;
 for (err = NO_ERROR; next != list; next = next->next)
    {
     nextwin = (struct image_display_window *) next->item;
     PRT9(printf("status4 = %x wid = %s\n",nextwin->process_status4,
          nextwin->owner->wid);)
     if (nextwin == win) continue;
     if ((nextwin == NULL) || (nextwin->process_status4 & WIN_ADVANCE_SLICE_PROCESSED))
        continue;
     nextwin->process_status4 |= WIN_ADVANCE_SLICE_PROCESSED;
     if ((nextwin->owner == NULL) ||
         ((img = win_rec_cur_img_buf(nextwin->owner)) == NULL))
        continue;
     if (img->file != cur_img->file) 
        {num_cuts = NumberOfSliceCuts(&(img->file->hdr.data_info));
         num_scans = NumberOfSliceScans(&(img->file->hdr.data_info));}
     if (is_list && (win->status4 & STACK_WIN_COSECUTIVE))
        {
         if (win->status4 & STACK_WIN_SCAN)
            {PRT3(printf("STACK_WIN_SCAN...\n"))
             nscnum = (nscnum + 1) % num_scans;}
           else nsnum = (nsnum + 1) % num_cuts;
         PRT9(printf("STACK_WIN_COSECUTIVE next = %u nextwin = %u win = %u\n",next,nextwin,win);
                  printf("nsnum = %d nscnum = %d num_scans = %d num_cuts = %d\n",
                    nsnum,nscnum,num_scans,num_cuts))
        }
       else
        {nsnum = ((long) img->cur_slice->slice_num + snum) % num_cuts;
         nscnum = ((long) img->cur_slice->scan_num + scnum) % num_scans;
         PRT9(printf("NON STACK_WIN_COSECUTIVE next = %u nextwin = %u win = %u\n",next,nextwin,win);
            printf("nsnum = %d nscnum = %d num_scans = %d num_cuts = %d\n",
                    nsnum,nscnum,num_scans,num_cuts))
        }
     if (((err = LocateAllSliceEchoes(img,nsnum,nscnum,img->cur_slice->echo_num))
             != NO_ERROR) ||
         ((err = RdCurSliceEchoes(img,0,0)) != NO_ERROR))
        break;
     PRT9(printf("wid = %d    s# = %d scan# = %d echo# = %d \n",win->owner->wid,
         img->cur_slice->slice_num,img->cur_slice->scan_num,img->cur_slice->echo_num))
     WinMultiEchoRunProc(nextwin,win_before_enter_next_slice,0,0);
     WinMultiEchoRunProc(nextwin,win_filter_image_data,0,0);
     WinMultiEchoRunProc(nextwin,win_multiple_set_frame_label,0,0);
    }
 return err;
}


/*------------------------------------------------------------------
WinNextSlice - ???
------------------------------------------------------------------*/

static int 
WinNextSlice(win,slice_offset,is_locate_scan,is_rel)
 struct image_display_window *win;
 int slice_offset,is_locate_scan,is_rel;
{
 int err;
 struct img_buf_rec *img;

 PRT3(printf("WinNextSlice...slice_offset = %d is_scan = %d is_rel = %d\n",
          slice_offset,is_locate_scan,is_rel);)
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return NO_ERROR;
 if ((win->link.next != &(win->link)) || (win->list.next != &(win->list)) ||
     (win->draw_list.item != NULL))
    {WinMultiEchoClearSliceProcessedStatus(win,WIN_ALL_PROCESSED_STATUS);
     if (win->link.next != &(win->link))
        WinRunProcInList(WIN_LINK | WIN_DO_NOT_PROCESS_CUR_WIN,win,
			 WinMultiEchoClearSliceProcessedStatus,
			 WIN_ALL_PROCESSED_STATUS,0,0,0,0,0);
     if (win->list.next != &(win->list))
        WinRunProcInList(WIN_LIST | WIN_DO_NOT_PROCESS_CUR_WIN,win,
			 WinMultiEchoClearSliceProcessedStatus,
			 WIN_ALL_PROCESSED_STATUS,0,0,0,0,0);
     if (win->draw_list.item != NULL)
        WinClearDrawListProcessedStatus(win,WIN_ALL_PROCESSED_STATUS);
    }
 if ((err = WinAdvanceSlice(win,slice_offset,is_locate_scan,is_rel)) !=
         NO_ERROR)
    return err;
 if (win->draw_list.item != NULL)
    {PRT3(printf("WinNextSlice...draw_list\n\n");)
     err = WinRunProcInList(WIN_DRAW_LIST | WIN_DO_NOT_PROCESS_CUR_WIN,win,
			    WinAdvanceDrawListSlice,
			    win,0,0,0,0,0);}

 if ((err == NO_ERROR) && (win->list.next != &(win->list)))
    {PRT3(printf("WinNextSlice...list\n\n");)
     err = WinAdvanceStack(win,img->previous_slice,&(win->list),1);}
 if ((err == NO_ERROR) && (win->link.next != &(win->link)))
    {PRT3(printf("WinNextSlice...link\n\n");)
     err = WinAdvanceStack(win,img->previous_slice,&(win->link),0);}

 PRT3(printf("WinNextSlice...end\n\n");)
 return err;
}


/*------------------------------------------------------------------
WinSetStackSliceOrScan - ???
------------------------------------------------------------------*/

static int 
WinSetStackSliceOrScan(win)
 struct image_display_window *win;
{
 if (win->status4 & STACK_WIN_SCAN) win->status4 &= ~STACK_WIN_SCAN;
   else win->status4 |= STACK_WIN_SCAN;
 return NO_ERROR;
}


/*------------------------------------------------------------------
WinStackSliceOrScan - ???
------------------------------------------------------------------*/

int 
WinStackSliceOrScan(win)
 struct image_display_window *win;
{
 struct list_rec *list,*first;
 struct img_buf_rec *img;

 WinRunProcInList(WIN_LIST,win,WinSetStackSliceOrScan,0,0,0,0,0,0);
 PRT2(printf("win_stack_slice_or_scan...\n");)
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return NO_ERROR;
 first = &(win->list);
 list = first->next;
 if (list != first)
    {
     WinAdvanceStack(win,img->cur_slice,&(win->list),1);
     WinRunProcInList(WIN_LIST | WIN_DO_NOT_PROCESS_CUR_WIN,
              win,WinNewMultiEchoSliceColorData,0,0,0,0,0,0);
     WinRunProcInList(WIN_LIST | WIN_DO_NOT_PROCESS_CUR_WIN,
              win,WinMultiEchoMultiScreenCanvas,0,0,0,0,0,0);
    }
 PRT2(printf("win_stack_slice_or_scan...end\n");)
 return NO_ERROR;
}


/*------------------------------------------------------------------
WinNewMultiEchoSliceColorData - ???
------------------------------------------------------------------*/

static int 
WinNewMultiEchoSliceColorData(win,is_processed_draw_list)
 struct image_display_window *win;
 int is_processed_draw_list;
{
 PRT8(printf("WinNewMultiEchoSliceColorData wid = %s\n",win->owner->wid))
 WinMultiEchoRunProc(win,WinNewColorData,0,0);
 WinMultiEchoRunProc(win,update_cur_slice_num,0,0);
 win->process_status4 |= WIN_COLOR_DATA_PROCESSED;
 if (is_processed_draw_list &&
     (win->draw_list.item != NULL) &&
     (WinRunProcInList(WIN_DRAW_LIST | WIN_DO_NOT_PROCESS_CUR_WIN,win,
		       WinSliceProcessed,
		       WIN_PROCESSED_ERROR_STATUS | WIN_COLOR_DATA_PROCESSED,
		       0,0,0,0,0)
      != NO_ERROR))
    {PRT8(printf("WinNewMultiEchoSliceColorData again...\n"))
     WinRunProcInList(WIN_DRAW_LIST | WIN_DO_NOT_PROCESS_CUR_WIN,win,
		      WinNewMultiEchoSliceColorData,
		      is_processed_draw_list,0,0,0,0,0);
    }
 return NO_ERROR;
}


/*------------------------------------------------------------------
WinNewMultiEchoMultiScreenCanvas - ???
------------------------------------------------------------------*/

static int 
WinNewMultiEchoMultiScreenCanvas(win)
 struct image_display_window *win;
{
 PRT8(printf("WinNewMultiEchoMultiScreenCanvas wid = %s\n",win->owner->wid))
 WinMultiEchoMultiScreenCanvas(win);
 win->process_status4 |= WIN_SCREEN_DATA_PROCESSED;
 if ((win->draw_list.item != NULL) &&
     (WinRunProcInList(WIN_DRAW_LIST | WIN_DO_NOT_PROCESS_CUR_WIN,win,
	WinSliceProcessed,
        WIN_PROCESSED_ERROR_STATUS | WIN_SCREEN_DATA_PROCESSED,0,0,0,0,0)
      != NO_ERROR))
    {PRT8(printf("WinNewMultiEchoMultiScreenCanvas again...\n"))
     WinRunProcInList(WIN_DRAW_LIST | WIN_DO_NOT_PROCESS_CUR_WIN,win,
               WinNewMultiEchoMultiScreenCanvas,0,0,0,0,0,0);
    }
 return NO_ERROR;
}


/*------------------------------------------------------------------
WinDisplaySlice - ???
------------------------------------------------------------------*/

static int 
WinDisplaySlice(win)
 struct image_display_window *win;
{
 PRT2(printf("WinDisplaySlice...\n");)

 WinNewMultiEchoSliceColorData(win,1);
 if (win->list.next != &(win->list))
    WinRunProcInList(WIN_LIST | WIN_DO_NOT_PROCESS_CUR_WIN,
         win,WinNewMultiEchoSliceColorData,0,0,0,0,0,0);
 if (win->link.next != &(win->link))
    {PRT4(printf("WinDisplaySlice...WIN_LINK\n");)
     WinRunProcInList(WIN_LINK | WIN_DO_NOT_PROCESS_CUR_WIN,
         win,WinNewMultiEchoSliceColorData,0,0,0,0,0,0);}

 if (win->status4 & RECOMPUTE_OVERLAY)
    {PRT4(printf("win->status4 RECOMPUTE_OVERLAY\n"))
     WinMultiEchoComputeOverlay(win,0,1);}
 if (win->list.next != &(win->list))
    {PRT4(printf("win->list RECOMPUTE_OVERLAY\n"))
     WinRunProcInList(WIN_LIST | WIN_DO_NOT_PROCESS_CUR_WIN,
         win,WinMultiEchoComputeOverlay,0,0,0,0,0,0);}
 if (win->link.next != &(win->link))
    {PRT4(printf("win->link RECOMPUTE_OVERLAY\n"))
     WinRunProcInList(WIN_LINK | WIN_DO_NOT_PROCESS_CUR_WIN,
         win,WinMultiEchoComputeOverlay,0,0,0,0,0,0);}

 WinNewMultiEchoMultiScreenCanvas(win);
 if (win->list.next != &(win->list))
    WinRunProcInList(WIN_LIST | WIN_DO_NOT_PROCESS_CUR_WIN,
         win,WinMultiEchoMultiScreenCanvas,0,0,0,0,0,0);
 if (win->link.next != &(win->link))
    WinRunProcInList(WIN_LINK | WIN_DO_NOT_PROCESS_CUR_WIN,
         win,WinMultiEchoMultiScreenCanvas,0,0,0,0,0,0);

 PRT2(printf("WinDisplaySlice...end\n\n");)
 return NO_ERROR;
}


/*------------------------------------------------------------------
next_group_slice_proc - ???

FIXME:  This function has the same potential nasties discussed for
auto_slice_proc in this file.  -PWH 19980608
------------------------------------------------------------------*/

void
next_group_slice_proc(item,event,arg,window,last_event,val)
 Panel_item item;
 Event *event;      /* Added *, which is Sun default -PWH 19980608 */
 void *arg;
 Xv_Window window;
 Event *last_event;
 int val;
{
 Panel panel;
 struct display_win_rec *win1;
 struct image_display_window *win,*headwin;
 int value,is_locate_scan;
 struct list_rec *head;

 if (item == 0) 
    win1 = (struct display_win_rec *) arg;
 else {
    panel = (Panel) xv_get(item, XV_OWNER);
    if (panel == 0) return /* CANCEL */;
    win1 = (struct display_win_rec *) xv_get(panel, WIN_CLIENT_DATA);  }

 win = win1->owner;
 win->dwin = win1;

 set_win_busy(win);
 if (win1->select_slice.frame != 0)
    xv_set(win1->select_slice.frame,FRAME_BUSY,TRUE,0);
 
 if ((value = num_continuous_win_list_slices(win)) == 1)
    {PRT3(printf("not num_continuous_win_list_slices...value = %d\n",value))
     headwin = win;}
 else
    {PRT3(printf("num_continuous_win_list_slices...value = %d\n",value))
     head = (struct list_rec *) head_list(&(win->list));
     headwin = (struct image_display_window *) head->item;
    }
 if (item != 0) 
    value *= (int) xv_get(item, PANEL_CLIENT_DATA);
 else
    value *= val;
 if (item != 0)
    is_locate_scan = is_scan_order(&(win1->select_slice),item,event);
 else
    is_locate_scan = is_scan_order(&(win1->select_slice),item,last_event);
 PRT3(printf("value = %d val = %d\n",value,val))

 if (value == 0)
     WinNewColorMultiCanvas(headwin);
 else 
    {WinNextSlice(headwin,value,is_locate_scan,1);
     WinDisplaySlice(headwin);}
 set_win_not_busy(win);
 if (win1->select_slice.frame != 0)
    xv_set(win1->select_slice.frame,FRAME_BUSY,FALSE,0);
 return /* NO_ERROR */;
}


/*------------------------------------------------------------------
more_select_slice_proc - ???
------------------------------------------------------------------*/

static void 
more_select_slice_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct display_win_rec *dwin;

 dwin = (struct display_win_rec *) xv_get(item, PANEL_CLIENT_DATA);
 notice_prompt(dwin->frame,(Event *) NULL,
            NOTICE_MESSAGE_STRINGS,
              "Sh (Shift key) - Apply in the reverse direction",
              "G - advance the whole group to the next page",
              "0 - Refresh the image",
              "",
              "When each slice has more than 1 scan...",
              "Alt or Caps-Lock key - Apply in the scan order direction",
              0,
            NOTICE_BUTTON_YES," OK !",
            0);
}


/*------------------------------------------------------------------
select_slice_item_proc - ???

FIXME:  This function doesn't appear to ever be used.  In addition,
it's not clear whether it's type int or void.
------------------------------------------------------------------*/

static void
select_slice_item_proc(item,event)
 Panel_item item;
 Event *event;
{
 Panel panel;
 struct display_win_rec *win1;
 struct image_display_window *win;
 int value,is_locate_scan;

 panel = (Panel) xv_get(item, XV_OWNER);
 if (panel == 0) return /*CANCEL*/;
 win1 = (struct display_win_rec *) xv_get(panel, WIN_CLIENT_DATA);
 win = win1->owner;
 win->dwin = win1;

 set_win_busy(win);
 if (win1->select_slice.frame != 0)
    xv_set(win1->select_slice.frame,FRAME_BUSY,TRUE,0);
 value = (int) xv_get(item, PANEL_CLIENT_DATA);
 is_locate_scan = is_scan_order(&(win1->select_slice),item,event);
 if (value == 0)
    {WinNewColorMultiCanvas(win);
    }
   else
    {WinNextSlice(win,value,is_locate_scan,1);
     WinDisplaySlice(win);}
 set_win_not_busy(win);
 if (win1->select_slice.frame != 0)
    xv_set(win1->select_slice.frame,FRAME_BUSY,FALSE,0);
}


/*------------------------------------------------------------------
is_scan_order - ???
------------------------------------------------------------------*/

static int 
is_scan_order(swin,item,event)
 struct select_slice_win *swin;
 Panel_item item;
 Event *event;
{
 if (item != 0)
    {if (item == swin->scan_num_slider) return 1;
     if (item == swin->slice_num_slider) return 0;}
 if ((event != NULL) && event_alt_is_down(event))
    {PRT(printf("is_scan_order 1....1\n"))
     return 1;}
 else if ((swin->which_order != 0)
             && xv_get(swin->which_order,PANEL_VALUE))
    {PRT(printf("is_scan_order 0....1\n"))
     return 1;}
 else return 0;
}


/*------------------------------------------------------------------
select_slice_slider_proc - ???

FIXME:  This function has the same possible nasties discussed for
auto_slice_proc.  -PWH 19980608
------------------------------------------------------------------*/

void
select_slice_slider_proc(item,value,last_event,arg,window)
 Panel_item item;
 int value;
 Event *last_event;
 void *arg;
 Xv_Window window;
{
 struct display_win_rec *win1;
 struct image_display_window *win;
 int slice_num;
 int is_locate_scan;
 struct img_buf_rec *img;

 TRACE(printf("select_slice_slider_proc...\n"))
 if (item == 0)
    win1 = (struct display_win_rec *) arg;
 else
    win1 = (struct display_win_rec *) xv_get(item, PANEL_CLIENT_DATA);

 win = win1->owner;
 win->dwin = win1;
 if (!(win->type2 & MANY_SLICES_WINDOWS)) return;

 set_win_busy(win);
 if (win1->select_slice.frame != 0)
    xv_set(win1->select_slice.frame,FRAME_BUSY,TRUE,0);
 if (item == 0)
    {
     if (xv_get(win1->select_slice.shift_on_item,PANEL_VALUE))
        if (value > 0) value = -value;
     is_locate_scan = is_scan_order(&(win1->select_slice),item,last_event);
     PRT(printf("is_locate_scan = %d\n",is_locate_scan);)
     if (value == 0) 
        {WinNewColorMultiCanvas(win);}
       else
        {WinNextSlice(win,value,is_locate_scan,1);
         WinDisplaySlice(win);}
    }            
  else
   {
    is_locate_scan = is_scan_order(&(win1->select_slice),item,last_event);
    slice_num = (unsigned int) xv_get(item,PANEL_VALUE);

    if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return /* CANCEL */;
    PRT(printf("is_locate_scan = %d\n",is_locate_scan);)
    WinNextSlice(win,slice_num - 1,is_locate_scan,0);
    WinDisplaySlice(win);
   }
 set_win_not_busy(win);
 if (win1->select_slice.frame != 0)
    xv_set(win1->select_slice.frame,FRAME_BUSY,FALSE,0);
}


/*------------------------------------------------------------------
select_slice_order_proc - ???
------------------------------------------------------------------*/

void 
select_slice_order_proc(menu,item,arg,window,last_event)
 Menu menu;
 Menu_item item;
 void *arg;
 Xv_Window window;
 Event *last_event;
{
 struct display_win_rec *dwin;

 PRT(printf("select_slice_order_proc...\n"))
 dwin = (struct display_win_rec *) arg;
 set_win_busy(dwin->owner);
 if (xv_get(dwin->select_slice.which_order,PANEL_VALUE))
     xv_set(dwin->select_slice.which_order,PANEL_VALUE,0,0);
   else xv_set(dwin->select_slice.which_order,PANEL_VALUE,1,0);
 set_win_not_busy(dwin->owner);
}


/*------------------------------------------------------------------
select_slice_done_proc - ???
------------------------------------------------------------------*/

static void
select_slice_done_proc(frame)
 Frame frame;
{
 int status;

 status = (int) xv_get(frame,XV_KEY_DATA,EXTRA_FRAME_STATUS);
 status &= ~EXTRA_FRAME_SHOW;
 status |= EXTRA_FRAME_HIDE;
 xv_set(frame,XV_KEY_DATA,EXTRA_FRAME_STATUS,status,XV_SHOW,FALSE,0);
}


/*------------------------------------------------------------------
setup_select_slice_win - ???
------------------------------------------------------------------*/

int 
setup_select_slice_win(win,dwin)
 struct select_slice_win *win;
 struct display_win_rec *dwin;
{
 char *str,*wid;
 static char *title = "SLICE SELECTION";
 int row,offset;
 struct img_buf_rec *img;
 int num,err;
 Rect *rr;
 Panel_item item;
 unsigned long num_scans;

 TRACE(printf("setup_select_slice_win...\n"))
 if ((img = win_rec_cur_img_buf(dwin->owner->owner)) == NULL) return CANCEL;
 win->frame = xv_create(dwin->frame, FRAME,
              FRAME_SHOW_LABEL, TRUE,
              FRAME_INHERIT_COLORS,TRUE,
              FRAME_NO_CONFIRM,TRUE,
              FRAME_DONE_PROC,select_slice_done_proc,
              XV_KEY_DATA,EXTRA_FRAME_STATUS,EXTRA_FRAME_HIDE,
              0);
 if (win->frame == 0) return WINDOW_ERROR;
 if ((wid = WinRecGetWid(dwin->owner->owner,dwin->owner->type2)) == NULL)
    xv_set(win->frame,XV_LABEL,title,0);
   else
    {
     if ((err = concat_strs(&str,2,wid,title)) == NO_ERROR)
        xv_set(win->frame,XV_LABEL,str,0);
       else xv_set(win->frame,XV_LABEL,title,0);
     c_free(str);
    }
 win->ancestor = dwin;

 win->panel = xv_create(win->frame,PANEL,
               WIN_CLIENT_DATA,dwin,
               FRAME_INHERIT_COLORS,TRUE,
#ifdef sparc    /* FIXME:  Causes fatal error in Linux */
               XV_FONT,win->ancestor->font,
#endif
               0);
 if (win->panel == 0) return WINDOW_ERROR;

 row = 2;
 offset = xv_rows(win->panel,1) - xv_rows(win->panel,0) - 10;
 num_scans = total_slice_scans(img);
 if (num_scans > 1)
    {win->which_order = xv_create(win->panel,PANEL_CHOICE_STACK,
           PANEL_LABEL_STRING,"APPLY TO-Alt Z",
           PANEL_CHOICE_STRINGS,"SLICE ORDER",
                           "SCAN ORDER",NULL,
           XV_X,5,XV_Y,row - 1,
           0);
     row += offset + 2;}
 else win->which_order = 0;

 win->auto_on = 0;
 str = "<-Sh-1..9";
 item = xv_create(win->panel,PANEL_BUTTON,
           PANEL_LABEL_STRING,str,
	   PANEL_NOTIFY_PROC,select_slice_item_proc,
           PANEL_CLIENT_DATA,-1,
           XV_X,5,XV_Y,row,
           0);
 rr = (Rect *) xv_get(item,XV_RECT);
 str = "1..9->";
 item = xv_create(win->panel,PANEL_BUTTON,
 	   PANEL_LABEL_STRING,str,
	   PANEL_NOTIFY_PROC,select_slice_item_proc,
           XV_X,rr->r_left + rr->r_width + 1,XV_Y,row,
           PANEL_CLIENT_DATA,1,
           0);

 rr = (Rect *) xv_get(item,XV_RECT);
 str = "<-0->";
 xv_create(win->panel,PANEL_BUTTON,
           PANEL_LABEL_STRING,str,
	   PANEL_NOTIFY_PROC,select_slice_item_proc,
           XV_X,rr->r_left + rr->r_width + 1,XV_Y,row,
           PANEL_CLIENT_DATA,0,
           0);

 TRACE(printf("before slice_num_slider...end\n"))
 row += offset;
 str = "Reverse";
 win->shift_on_item = xv_create(win->panel,PANEL_CHECK_BOX,
                       XV_X,5,XV_Y,row,
                       PANEL_LABEL_STRING,str,
                       0);
 rr = (Rect *) xv_get(win->shift_on_item,XV_RECT);
 str = "<-Sh-G";
 item = xv_create(win->panel,PANEL_BUTTON,
           PANEL_LABEL_STRING,str,
	   PANEL_NOTIFY_PROC,next_group_slice_proc,
           XV_X,rr->r_left + rr->r_width - 10,XV_Y,row,
           PANEL_CLIENT_DATA,-1,
           0);
 rr = (Rect *) xv_get(item,XV_RECT);
 str = "G->";
 item = xv_create(win->panel,PANEL_BUTTON,
           PANEL_LABEL_STRING,str,
	   PANEL_NOTIFY_PROC,next_group_slice_proc,
           XV_X,rr->r_left + rr->r_width + 3,XV_Y,row,
           PANEL_CLIENT_DATA,1,
           0);
 rr = (Rect *) xv_get(item,XV_RECT);
 str = "MORE...";
 item = xv_create(win->panel,PANEL_BUTTON,
           PANEL_LABEL_STRING,str,
	   PANEL_NOTIFY_PROC,more_select_slice_proc,
           XV_X,rr->r_left + rr->r_width + 3,XV_Y,row,
           PANEL_CLIENT_DATA,dwin,
           0);
 row += offset;
 if (img->cur_slice == NULL)
    num = 1;
   else num = img->cur_slice->slice_num + 1;

 win->slice_num_slider = xv_create(win->panel,PANEL_SLIDER,
                       XV_X,5,XV_Y,row,
                            PANEL_VALUE,(int) num,
                            PANEL_MIN_VALUE,(int) 1,
                            PANEL_MAX_VALUE,
                               win_total_slice_cuts(dwin->owner->owner),
	                    PANEL_NOTIFY_PROC,select_slice_slider_proc,
                            PANEL_CLIENT_DATA,dwin,
 	                    PANEL_SLIDER_WIDTH,100,
                            0);

 if (num_scans > 1)
    {row += offset;
     xv_set(win->slice_num_slider,PANEL_SLIDER_WIDTH,60,
	               PANEL_LABEL_STRING, "SLICE#",0);
     win->scan_num_slider = xv_create(win->panel,PANEL_SLIDER,
	               PANEL_LABEL_STRING, "Alt-SCAN#",
                       XV_X,5,XV_Y,row,
                            PANEL_VALUE,(int) num,
                            PANEL_MIN_VALUE,(int) 1,
                            PANEL_MAX_VALUE,num_scans,
	                    PANEL_NOTIFY_PROC,select_slice_slider_proc,
                            PANEL_CLIENT_DATA,dwin,
 	                    PANEL_SLIDER_WIDTH,60,
                            0);
     }
    else win->scan_num_slider = 0;

 row += offset;
 str = "By";
 win->auto_offset_item = xv_create(win->panel,PANEL_NUMERIC_TEXT,
	   PANEL_LABEL_STRING, str,
           PANEL_VALUE_DISPLAY_LENGTH,3,
           PANEL_VALUE,1,
           PANEL_MIN_VALUE,1,
           PANEL_MAX_VALUE,200,
           XV_X,5,XV_Y,row,
           0);
 rr = (Rect *) xv_get(win->auto_offset_item,XV_RECT);
 win->auto_item = xv_create(win->panel,PANEL_BUTTON,
           PANEL_LABEL_STRING,AUTO_STR,
	   PANEL_NOTIFY_PROC,auto_slice_proc,
           PANEL_CLIENT_DATA,dwin,
           XV_X,rr->r_left + rr->r_width,XV_Y,row,
           0);
/*
 rr = (Rect *) xv_get(item1,XV_RECT);
 xv_set(win->panel,XV_WIDTH,rr->r_left + rr->r_width + 5,0);
 rr = (Rect *) xv_get(win->auto_offset_item,XV_RECT);
 xv_set(win->panel,XV_HEIGHT,rr->r_top + rr->r_height + 5,0);
*/
 window_fit(win->panel);
 rr = (Rect *) xv_get(win->panel,XV_RECT);
 xv_set(win->panel,XV_WIDTH,rr->r_left + rr->r_width - 15,0,
        XV_HEIGHT,rr->r_top + rr->r_height - 35,0);
 window_fit(win->frame);
 
 TRACE(printf("setup_select_slice_win...end\n"))
 return NO_ERROR;

}


/*------------------------------------------------------------------
free_select_slice - ???
------------------------------------------------------------------*/

static void
free_select_slice_win(win)
 struct select_slice_win *win;
{
 xv_destroy_safe(win->frame);
 win->frame = 0;
}


/*------------------------------------------------------------------
copy_select_slice_win - ???
------------------------------------------------------------------*/

void 
copy_select_slice_win(dst,src,dwin)
 struct select_slice_win *dst,*src;
 struct display_win_rec *dwin;
{
 TRACE(printf("copy_select_slice_win...begin\n"))
 if (dst->frame != 0)
    {if (src->frame == 0)
        {free_select_slice_win(dst);
         return;}
    }
   else if (src->frame == 0) return;
   else setup_select_slice_win(dst,dwin);
 if (dst->frame == 0) return;
 xv_set(dst->slice_num_slider,PANEL_VALUE,
       xv_get(src->slice_num_slider,PANEL_VALUE),0);
 xv_set(dst->auto_offset_item,PANEL_VALUE,
       xv_get(src->auto_offset_item,PANEL_VALUE),0);
 xv_set(dst->shift_on_item,PANEL_VALUE,
       xv_get(src->shift_on_item,PANEL_VALUE),0);
 TRACE(printf("copy_select_slice_win...end\n"))

}


/*------------------------------------------------------------------
select_slice_proc - ???
------------------------------------------------------------------*/

void 
select_slice_proc(menu,item,arg,window,last_event)
 Menu menu;
 Menu_item item;
 void *arg;
 Xv_Window window;
 Event *last_event;
{
 struct display_win_rec *win1;
 struct image_display_window *win;
 int status;

 if (menu == 0)
    {win1 = (struct display_win_rec *) arg;}
   else
    {
     if ((last_event = (Event *) xv_get(menu,MENU_FIRST_EVENT)) != NULL)
        {window = (Xv_Window) event_window(last_event);
         win1 = (struct display_win_rec *) xv_get(window,WIN_CLIENT_DATA);
        }
       else win1 = (struct display_win_rec *) xv_get(menu,MENU_CLIENT_DATA);
    }
 win = win1->owner;
 win->dwin = win1;

 TRACE(printf("select_slice_proc...%u\n",win1->select_slice.frame))

 if (win1->select_slice.frame == 0) return;
 status = (int) xv_get(win1->select_slice.frame,XV_KEY_DATA,EXTRA_FRAME_STATUS);
 status &= ~EXTRA_FRAME_HIDE;
 status |= EXTRA_FRAME_SHOW;
 xv_set(win1->select_slice.frame,XV_KEY_DATA,EXTRA_FRAME_STATUS,status,0);
 if (!xv_get(win1->select_slice.frame,FRAME_CLOSED) &&
      xv_get(win1->select_slice.frame,XV_SHOW))
    {
     win_set_mouse_at_center(win1->select_slice.frame);
     wmgr_top(win1->select_slice.frame);}
   else
    {make_2_wins_adjacent(win1->select_slice.frame,
                          win1->select_slice.ancestor->frame,0);
     win_set_mouse_at_center(win1->select_slice.frame);

     xv_set(win1->select_slice.frame,
           XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
     wmgr_top(win1->select_slice.frame);
    }

}
