/****************************************************************************
dp_ca_options.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: dp_ca_options.c,v 1.9 1999/07/07 17:41:15 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/wmgr.h>
#include <xview/cms.h>

#include <sys/timeb.h>      /* FIXME:  Is this truly necessary? */

#include "mri_format.h"
#include "nria_procedures.h"

/*
#define TRACE(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
*/
#define TRACE(x)
#define PRT(x)



#define CLEAN_UP_GA_PARAMS 0
#define SETUP_GA_PARAMS 1
#define GET_ALL_PTR 2

struct ca_option_rec
   {
    unsigned short status;
    struct image_display_window *win;
    struct display_win_rec *dwin;
    Frame frame;
    Panel panel;
    Panel_item view,percent_on,percent,roi_percent_on,roi_percent,
               neg,echo_method;
   };

struct all_ca_option_recs
   {
    int num_ops;
    struct ca_option_rec **ops;
   };


/* Functions defined in this file */
void init_ca_option_rec(struct ca_option_rec *op);
void free_ca_option_rec(struct ca_option_rec *op);
void init_all_ca_option_recs(struct all_ca_option_recs *all_ops);
void ca_set_initial_values(struct ca_option_rec *op, struct img_buf_rec *img);
int ca_cur_default(struct ca_option_rec *op, Panel_item item);
void ca_cur_default_proc(Panel_item item, Event *event);
void ca_view_proc(Panel_item item, int value, Event *event);
void ca_echo_method_proc(Panel_item item, int value, Event *event);
void ca_neg_equal_zero_proc(Panel_item item, int value, Event *event);
void ca_percent_on_proc(Panel_item item, int value, Event *event);
void ca_def_percent_proc(Panel_item item, Event *event);
void ca_percent_proc(Panel_item item, int value, Event *event);
void CA_frame_done_proc(Frame frame);
int setup_ca_option_rec(struct ca_option_rec *op,
    struct image_display_window *win);
struct ca_option_rec *find_ca_option_rec(struct all_ca_option_recs *all_ops,
    struct image_display_window *win);
int win_update_ca_options(struct image_display_window *win);
int enter_ca_options(struct image_display_window *win, int status);



/*--------------------------------------------------------------------------
init_ca_option_rec - ???
--------------------------------------------------------------------------*/

void
init_ca_option_rec(op)
 struct ca_option_rec *op;
{
 op->status = 0;
 op->win = NULL;
 op->frame = 0;
}


/*--------------------------------------------------------------------------
free_ca_option_rec - ???
--------------------------------------------------------------------------*/

void
free_ca_option_rec(op)
 struct ca_option_rec *op;
{
 struct all_ca_option_recs *all_ops;

 TRACE(printf("free_ca_option_rec...op->win->dwin = %u\n",op->win->dwin))
 TRACE(printf("free_ca_option_rec...1\n"))
 if (op->frame != 0)
    {xv_destroy_safe(op->frame);
     op->frame = 0;
    }
 c_free(op);
 all_ops = (struct all_ca_option_recs *) 
           enter_ca_options(NULL,GET_ALL_PTR);
 DropHandleAryPtr(&(all_ops->ops),&(all_ops->num_ops),op);
 TRACE(printf("free_ca_option_rec...end\n"))
}


/*--------------------------------------------------------------------------
init_all_ca_option_recs - ???
--------------------------------------------------------------------------*/

void
init_all_ca_option_recs(all_ops)
 struct all_ca_option_recs *all_ops;
{
 all_ops->num_ops = 0;
 all_ops->ops = NULL;
}


/*--------------------------------------------------------------------------
ca_set_initial_values - ???
--------------------------------------------------------------------------*/

void
ca_set_initial_values(op,img)
 struct ca_option_rec *op;
 struct img_buf_rec *img;
{
 int val;

 if (op->echo_method != 0)
    {if (img->file->count.out_format_status & OUT_FORMAT_POS_NEG_ACTIVATION_ECHOES)
        val = 1;
       else if (img->file->count.out_format_status & OUT_FORMAT_COUNT_CUR_ECHO_ONLY)
        val = 2;
       else if (img->file->count.out_format_status & OUT_FORMAT_POS_NEG_ECHOES)
        val = 3;
       else val = 0;
     xv_set(op->echo_method,PANEL_VALUE,val,0);}
 if (img->file->count.out_format_status & OUT_FORMAT_COUNT_NEG)
    xv_set(op->neg,PANEL_VALUE,0,0);
   else xv_set(op->neg,PANEL_VALUE,1,0);
 if (img->file->count.out_format_status & OUT_FORMAT_LT_PERCENT_MEAN)
    xv_set(op->percent_on,PANEL_VALUE,1,0);
   else xv_set(op->percent_on,PANEL_VALUE,0,0);

 if (!(img->file->count.out_format_status & OUT_FORMAT_USE_DEFAULT_IMAGE_VIEW))
    {if (img->file->count.out_format_status & OUT_FORMAT_UNIT_2D_VALID)
        xv_set(op->view,PANEL_VALUE,1,0);
       else xv_set(op->view,PANEL_VALUE,0,0);
    }
   else xv_set(op->view,PANEL_VALUE,2,0);
 xv_set(op->percent,PANEL_VALUE,
     (long) rint((double) img->file->count.lt_percent),0);
 xv_set(op->roi_percent,PANEL_VALUE,(long) rint((double) img->file->count.roi_lt_percent),0);
}


/*--------------------------------------------------------------------------
ca_cur_default - ???
--------------------------------------------------------------------------*/

int
ca_cur_default(op,item)
 struct ca_option_rec *op;
 Panel_item item;
{
 struct img_buf_rec *img;
 int which;

 if ((img = win_rec_cur_img_buf(op->win->owner)) == NULL) return CANCEL;
 img->file->count.out_format_status = img->file->count.format.status;
 img->file->count.lt_percent = img->file->count.format.lt_percent;
 img->file->count.gt_percent = img->file->count.format.gt_percent;
 img->file->count.roi_lt_percent = img->file->count.format.roi_lt_percent;
 img->file->count.roi_gt_percent = img->file->count.format.roi_gt_percent;
 img->file->count.img_unit = img->file->count.format.img_unit;
 ca_set_initial_values(op,img);

 if (item != 0)
    if (which = (int) xv_get(item,XV_KEY_DATA,1))
       {if (img->file->count.format.status & OUT_FORMAT_LT_PERCENT_ROI_MEAN)
           xv_set(op->roi_percent,PANEL_VALUE,
            (long) rint((double) img->file->count.format.roi_lt_percent),0);
          else xv_set(op->roi_percent,PANEL_VALUE,0,0);
       }
      else if (img->file->count.format.status & OUT_FORMAT_LT_PERCENT_MEAN)
       xv_set(op->percent,PANEL_VALUE,
           (long) rint((double) img->file->count.format.lt_percent),0);
      else xv_set(op->percent,PANEL_VALUE,0,0);
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
ca_cur_default_proc - ???
--------------------------------------------------------------------------*/

void
ca_cur_default_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct ca_option_rec *op;

 op = (struct ca_option_rec *) xv_get(item,PANEL_CLIENT_DATA);
 ca_cur_default(op,item);
}


/*--------------------------------------------------------------------------
ca_view_proc - ???
--------------------------------------------------------------------------*/

void
ca_view_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct ca_option_rec *op;
 struct img_buf_rec *img;

 op = (struct ca_option_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((img = win_rec_cur_img_buf(op->win->owner)) == NULL)
     return /*CANCEL*/;
 if (value == 2)
    img->file->count.out_format_status |= OUT_FORMAT_USE_DEFAULT_IMAGE_VIEW;
   else 
    {img->file->count.out_format_status &= ~OUT_FORMAT_USE_DEFAULT_IMAGE_VIEW;
     if (value == 1)
        if (!(img->file->count.out_format_status & OUT_FORMAT_UNIT_2D_VALID))
           xv_set(op->view,PANEL_VALUE,0,0);
    }
}


/*--------------------------------------------------------------------------
ca_echo_method_proc - ???
--------------------------------------------------------------------------*/

void
ca_echo_method_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct ca_option_rec *op;
 struct img_buf_rec *img;

 op = (struct ca_option_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((img = win_rec_cur_img_buf(op->win->owner)) == NULL)
     return /*CANCEL*/;
 img->file->count.out_format_status &= ~OUT_FORMAT_COUNT_ECHO_METHOD;
 if (value == 1) 
    img->file->count.out_format_status |= OUT_FORMAT_POS_NEG_ACTIVATION_ECHOES;
   else if (value == 2)
    img->file->count.out_format_status |= OUT_FORMAT_COUNT_CUR_ECHO_ONLY;
   else if (value == 3) img->file->count.out_format_status |= OUT_FORMAT_POS_NEG_ECHOES;
}


/*--------------------------------------------------------------------------
ca_neg_equal_zero_proc - ???
--------------------------------------------------------------------------*/

void
ca_neg_equal_zero_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct ca_option_rec *op;
 struct img_buf_rec *img;

 op = (struct ca_option_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((img = win_rec_cur_img_buf(op->win->owner)) == NULL)
     return /*CANCEL*/;
 if (value == 0) img->file->count.out_format_status |= OUT_FORMAT_COUNT_NEG;
   else img->file->count.out_format_status &= ~OUT_FORMAT_COUNT_NEG;

}


/*--------------------------------------------------------------------------
ca_percent_on_proc - ???
--------------------------------------------------------------------------*/

void
ca_percent_on_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct ca_option_rec *op;
 struct img_buf_rec *img;
 int which;

 op = (struct ca_option_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((img = win_rec_cur_img_buf(op->win->owner)) == NULL)
     return /*CANCEL*/;
 if (which = (int) xv_get(item,XV_KEY_DATA,1))
    {if (value)
        img->file->count.out_format_status |= OUT_FORMAT_LT_PERCENT_ROI_MEAN;
       else img->file->count.out_format_status &= ~OUT_FORMAT_LT_PERCENT_ROI_MEAN;
    }
   else if (value)
    img->file->count.out_format_status |= OUT_FORMAT_LT_PERCENT_MEAN;
   else img->file->count.out_format_status &= ~OUT_FORMAT_LT_PERCENT_MEAN;
}


/*--------------------------------------------------------------------------
ca_def_percent_proc - ???
--------------------------------------------------------------------------*/

void
ca_def_percent_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct ca_option_rec *op;
 struct img_buf_rec *img;
 int which;

 op = (struct ca_option_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((img = win_rec_cur_img_buf(op->win->owner)) == NULL)
     return /*CANCEL*/;
 if (which = (int) xv_get(item,XV_KEY_DATA,1))
    {if (img->file->count.format.status & OUT_FORMAT_LT_PERCENT_ROI_MEAN)
        xv_set(op->roi_percent,PANEL_VALUE,
            (long) rint((double) img->file->count.format.roi_lt_percent),0);
       else xv_set(op->roi_percent,PANEL_VALUE,0,0);
    }
   else if (img->file->count.format.status & OUT_FORMAT_LT_PERCENT_MEAN)
    xv_set(op->percent,PANEL_VALUE,
        (long) rint((double) img->file->count.format.lt_percent),0);
   else xv_set(op->percent,PANEL_VALUE,0,0);
}


/*--------------------------------------------------------------------------
ca_percent_proc - ???
--------------------------------------------------------------------------*/

void
ca_percent_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct ca_option_rec *op;
 struct img_buf_rec *img;
 int which;

 op = (struct ca_option_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((img = win_rec_cur_img_buf(op->win->owner)) == NULL)
     return /*CANCEL*/;
 if (which = (int) xv_get(item,XV_KEY_DATA,1))
    img->file->count.roi_lt_percent = (float) value;
   else img->file->count.lt_percent = (float) value;
 PRT(printf("ca_percent_proc..val = %d lt_percent = %f roi_lt_percent = %f\n",
          value,img->file->count.lt_percent,img->file->count.roi_lt_percent);)
}


/*--------------------------------------------------------------------------
CA_frame_done_proc - ???
--------------------------------------------------------------------------*/

void
CA_frame_done_proc(frame)
 Frame frame;
{
 struct ca_option_rec *op;

 op = (struct ca_option_rec *) xv_get(frame,WIN_CLIENT_DATA);
 free_ca_option_rec(op);
}


/*--------------------------------------------------------------------------
setup_ca_option_rec - ???
--------------------------------------------------------------------------*/

int
setup_ca_option_rec(op,win)
 struct ca_option_rec *op;
 struct image_display_window *win;
{
 char *str,str1[80];
 int row,offset;
 Rect *rr;
 struct img_buf_rec *img;
 Panel_item item;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) 
    {free_ca_option_rec(op);
     return CANCEL;}
 op->win = win;
 op->dwin = win->dwin;
 op->frame = xv_create(win->dwin->frame, FRAME,
              FRAME_INHERIT_COLORS,TRUE,
              0);
 TRACE(printf("setup_ca_option_rec...1\n"))
 if (op->frame == 0)
    {free_ca_option_rec(op);
     return CANCEL;}
 str = win_rec_win_id_title(win->owner,CA_OPTIONS_STR);
 xv_set(op->frame,
              FRAME_SHOW_LABEL, TRUE,
              XV_LABEL,str,
              FRAME_NO_CONFIRM,TRUE,
              FRAME_DONE_PROC,CA_frame_done_proc,
              WIN_CLIENT_DATA,op,
              0);
 c_free(str);
 TRACE(printf("ok111.,..."))
 op->panel = xv_create(op->frame,PANEL,
                         FRAME_INHERIT_COLORS,TRUE,
                         0);
 if (op->panel == 0)
    {free_ca_option_rec(op);
     return CANCEL;}
 row = 2;
 offset = xv_rows(op->panel,1) - xv_rows(op->panel,0) - 13;
 xv_create(op->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING,"CUR DEFAULT CONDITIONS",
                   XV_X,5,XV_Y,row,
                   PANEL_NOTIFY_PROC,ca_cur_default_proc,
                   PANEL_CLIENT_DATA,op,
                   XV_KEY_DATA,1,1,
                   0);
 row += offset;
 xv_create(op->panel,PANEL_MESSAGE,
 	           PANEL_LABEL_STRING,"When counting, the following conditions apply...",
                   XV_X,5,XV_Y,row,
                   0);
 row += offset - 8;
 op->view = xv_create(op->panel,PANEL_CHOICE_STACK,
 	PANEL_CHOICE_STRINGS,
             "USE CURRENT IMAGE SIZE & ORIENTATION",
             "USE EXISTING IMAGE SIZE & ORIENTATION",
             "USE DEFAULT",
             NULL,
	PANEL_VALUE,0,
        PANEL_NOTIFY_PROC,ca_view_proc,
        PANEL_CLIENT_DATA,op,
        XV_X,25,XV_Y,row,
	0);
 row += offset + 2;
 if (img->file->hdr.data_info.num_echoes > 1)
    {sprintf(str1,"TOTAL ECHOES = %d,",img->file->hdr.data_info.num_echoes);
     item = xv_create(op->panel,PANEL_MESSAGE,
 	           PANEL_LABEL_STRING,str1,
                   XV_X,25,XV_Y,row,
                   0);
     rr = (Rect *) xv_get(item,XV_RECT);
     xv_create(op->panel,PANEL_MESSAGE,
                   PANEL_LABEL_STRING," FOR MULTIPLE ECHOES",
                   XV_X,rr->r_left + rr->r_width,XV_Y,row,
                   0);
     row += offset - 7;
     op->echo_method = xv_create(op->panel,PANEL_CHOICE_STACK,
 	PANEL_CHOICE_STRINGS,
             "COUNT = SUM OF ALL ECHOES",
/*            "COUNT EACH ECHO INDIVIDUALLY",*/
             "COUNT 2-ECHO POS/NEG ACTIVATION",
             "COUNT CUR ECHO ONLY",
             "COUNT = ECHO-1 - ECHO-2 - ... - ECHO-N",
             NULL,
	PANEL_VALUE,0,
        PANEL_NOTIFY_PROC,ca_echo_method_proc,
        PANEL_CLIENT_DATA,op,
        XV_X,40,XV_Y,row,
	0);
     row += offset;
    }
   else op->echo_method = 0;
 op->neg = xv_create(op->panel,PANEL_CHECK_BOX,
 	           PANEL_LABEL_STRING,"PIXEL VALUES = PVAL",
                   XV_X,15,XV_Y,row,
 	           PANEL_CHOICE_STRINGS, "(PVAL <= 0) = 0",NULL,
                   PANEL_NOTIFY_PROC,ca_neg_equal_zero_proc,
                   PANEL_CLIENT_DATA,op,
                   0);
 row += offset;
 op->percent_on = xv_create(op->panel,PANEL_CHECK_BOX,
 	           PANEL_LABEL_STRING,"If SLICE_MEAN > 0",
                   PANEL_LAYOUT,PANEL_HORIZONTAL,
                   XV_X,15,XV_Y,row,
 	           PANEL_CHOICE_STRINGS,"(PVAL < SLICE_MEAN * PERCENT) = 0",NULL,
                   PANEL_NOTIFY_PROC,ca_percent_on_proc,
                   XV_KEY_DATA,1,0,
                   PANEL_CLIENT_DATA,op,
                   0);
 row += offset;
 item = xv_create(op->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING,"DEFAULT",
                   XV_X,25,XV_Y,row,
                   PANEL_NOTIFY_PROC,ca_def_percent_proc,
                   XV_KEY_DATA,1,0,
                   PANEL_CLIENT_DATA,op,
                   0);

 rr = (Rect *) xv_get(item,XV_RECT);
 op->percent = xv_create(op->panel,PANEL_SLIDER,
 	PANEL_LABEL_STRING, "PERCENT",
	PANEL_MIN_VALUE,0,
	PANEL_MAX_VALUE,100,
	PANEL_SLIDER_WIDTH,50,
        XV_X,rr->r_left + rr->r_width + 2,XV_Y,row,
        PANEL_NOTIFY_PROC,ca_percent_proc,
        XV_KEY_DATA,1,0,
        PANEL_CLIENT_DATA,op,
	0);

 row += offset + 3;
 op->roi_percent_on = xv_create(op->panel,PANEL_CHECK_BOX,
 	           PANEL_LABEL_STRING,"If ROI_MEAN > 0",
                   PANEL_LAYOUT,PANEL_HORIZONTAL,
                   XV_X,15,XV_Y,row,
 	           PANEL_CHOICE_STRINGS,"(PVAL < ROI_MEAN * PERCENT) = 0",NULL,
                   PANEL_VALUE,0,
                   PANEL_NOTIFY_PROC,ca_percent_on_proc,
                   XV_KEY_DATA,1,1,
                   PANEL_CLIENT_DATA,op,
                   0);
 if (img->file->count.out_format_status & OUT_FORMAT_LT_PERCENT_ROI_MEAN)
    xv_set(op->roi_percent_on,PANEL_VALUE,0,0);
 row += offset + 2;
 item = xv_create(op->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING,"DEFAULT",
                   XV_X,15,XV_Y,row,
                   PANEL_NOTIFY_PROC,ca_def_percent_proc,
                   PANEL_CLIENT_DATA,op,
                   XV_KEY_DATA,1,1,
                   0);

 rr = (Rect *) xv_get(item,XV_RECT);
 op->roi_percent = xv_create(op->panel,PANEL_SLIDER,
 	PANEL_LABEL_STRING, "PERCENT",
	PANEL_MIN_VALUE,0,
	PANEL_MAX_VALUE,100,
	PANEL_SLIDER_WIDTH,50,
        XV_X,rr->r_left + rr->r_width + 2,XV_Y,row,
        PANEL_NOTIFY_PROC,ca_percent_proc,
        XV_KEY_DATA,1,1,
        PANEL_CLIENT_DATA,op,
	0);
 ca_set_initial_values(op,img);

 window_fit(op->panel);
 rr = (Rect *) xv_get(op->panel,XV_RECT);
 xv_set(op->panel,XV_WIDTH,rr->r_width - 10,XV_HEIGHT,rr->r_height - 10,0);
 window_fit(op->frame);
 make_2_wins_adjacent(op->frame,win->dwin->frame,0);
 win_set_mouse_at_center(op->frame);
 xv_set(op->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);

 return NO_ERROR;
}



/*--------------------------------------------------------------------------
find_ca_option_rec - ???
--------------------------------------------------------------------------*/

struct ca_option_rec *
find_ca_option_rec(all_ops,win)
 struct all_ca_option_recs *all_ops;
 struct image_display_window *win;
{
 int lcv;

 for (lcv = 0; lcv < all_ops->num_ops; lcv++)
    if (all_ops->ops[lcv]->win == win)
       return all_ops->ops[lcv];
 return NULL;
}


/*--------------------------------------------------------------------------
win_update_ca_options - ???
--------------------------------------------------------------------------*/

int
win_update_ca_options(win)
 struct image_display_window *win;
{
 struct all_ca_option_recs *all_ops;
 struct image_display_window *win1;
 struct img_buf_rec *img,*img1;
 unsigned long lcv;

 if ((all_ops = (struct all_ca_option_recs *) 
           enter_ca_options(NULL,GET_ALL_PTR)) != NULL)
    {if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
     for (lcv = 0; lcv < all_ops->num_ops; lcv++)
        {win1 = all_ops->ops[lcv]->win;
         if ((win1 == win) ||
             (((img1 = win_rec_cur_img_buf(win1->owner)) != NULL) &&
              (img1->file == img->file)))
            ca_cur_default(all_ops->ops[lcv],NULL);
        }
    }
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
enter_ca_options - ???
--------------------------------------------------------------------------*/

int
enter_ca_options(win,status)
 struct image_display_window *win;
 int status;
{
 static struct all_ca_option_recs all_ops;
 struct ca_option_rec *op;
 static char first_time = 1;
 int err;

 TRACE(printf("ca_option_rec...0\n"))
 err = NO_ERROR;
 if (status == GET_ALL_PTR)
    {if (first_time) return 0;
       else return (unsigned long) &all_ops;}     /* FIXME:  ptr as long */
  else if (status == CLEAN_UP_GA_PARAMS)
   {if (!first_time)
       if ((op = find_ca_option_rec(&all_ops,win)) != NULL)
          free_ca_option_rec(op);
    return NO_ERROR;}
 TRACE(printf("ca_option_rec...1\n"))
 if (first_time)
    {init_all_ca_option_recs(&all_ops);
     first_time = 0;}
   else
    {if ((op = find_ca_option_rec(&all_ops,win)) != NULL)
        {
         xv_set(op->frame,
                XV_SHOW,TRUE,WIN_ALARM,FRAME_CLOSED,FALSE,0);
         win_set_mouse_at_center(op->frame);
         wmgr_top(op->frame);
         return NO_ERROR;}
    }
 TRACE(printf("ca_option_rec...2\n"))
 get_handle_memory(&(all_ops.ops),&(all_ops.num_ops),1,
     (size_t) sizeof(struct ca_option_rec),&err);
 if (err != NO_ERROR) return err;
 op = all_ops.ops[all_ops.num_ops - 1];
 init_ca_option_rec(op);
 TRACE(printf("ca_option_rec...2\n"))
 if ((err = setup_ca_option_rec(op,win)) == NO_ERROR)
    {TRACE(printf("setup_ca_option_rec...\n"))
    }
   else
    ShowMsgStrs(win->dwin->frame,0,err,0,2,
               "An error occured when setting up",
               "   the CA options window !!!",NULL,0);
 return err;

}


