/****************************************************************************
error_handler.c - Do error handling for NRIA

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.


This file depends on XView.

FIXME: signals differ between SunOS 4.x and SunOS 5.x, after 15.
This is not yet accounted for.
****************************************************************************/

static const char rcsid[] = "$Id: error_handler.c,v 1.10 1999/07/07 17:41:16 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <sys/signal.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/notify.h>
#include <xview/xv_error.h>

#include "mri_format.h"
#include "debug.h"
#include "nria_procedures.h"


/* Functions defined in this file */
int abort_program(struct prog_rec *prog, int option);
int exit_handler(int select, struct prog_rec *prog,
    Notify_func exit_called_proc);
int xv_error_handler(Xv_object obj, Attr_avlist vlist);
Notify_value bad_exit_called_proc(struct prog_rec *prog, int sig,
    Notify_signal_mode when);


/*---------------------------------------------------------------------------
abort_program - abort nria or register cleanup program

This function registers a prog_rec which is used to clean up prior
to an abort if option is 0, or aborts NRIA if option is 1.
---------------------------------------------------------------------------*/

int
abort_program( 
      struct prog_rec *prog,
      int option)
{
   static struct prog_rec *the_prog;
  
   /* If option is 0, just record the prog_rec */
   if (option == 0) {
      the_prog = prog;
      return NO_ERROR;
   }
  
   /* Otherwise abort the program */
   printf("FATAL ERROR:  Aborting execution.\n");
   cleanup_prog(the_prog,1);

   /* For debugging, force a core dump that can be analyzed */
   if (abortFlag)   abort(); 

   exit(0);
}


/*--------------------------------------------------------------------------
exit_handler - ???
--------------------------------------------------------------------------*/

#define EXIT_SETUP_PROC 1
#define EXIT_DISABLE 2

int
exit_handler(
     int select,
     struct prog_rec *prog,
     Notify_func exit_called_proc )  
     /* WAS: Notify_func (*exit_called_proc)() ) */
{
  static int bad_exit = 1;

  PRT(printf("exit_handler...\n"));
  if (select == EXIT_DISABLE)
    bad_exit = 0;
  else if (bad_exit && select == EXIT_SETUP_PROC)
  {
    /* DRW 961113: ASYNChronous notification necessary in this program. */

    notify_set_signal_func((Notify_client) prog,
			   (Notify_func) exit_called_proc,
			   SIGILL, /* Illegal Instruction */
			   NOTIFY_ASYNC);
    notify_set_signal_func((Notify_client) prog,
			   (Notify_func) exit_called_proc,
			   SIGTRAP, /* Instruction Trap */
			   NOTIFY_ASYNC);
    notify_set_signal_func((Notify_client) prog,
			   (Notify_func) exit_called_proc,
			   SIGABRT, /* Abort */
			   NOTIFY_ASYNC);
#ifdef SIGEMT   /* PWH 19980323: SIGEMT not defined in Linux */
    notify_set_signal_func((Notify_client) prog,
			   (Notify_func) exit_called_proc,
			   SIGEMT, /* Emulator Trap */
			   NOTIFY_ASYNC);
#endif /* SIGEMT */
    notify_set_signal_func((Notify_client) prog,
			   (Notify_func) exit_called_proc,
			   SIGFPE, /* Floating Point Exception */
			   NOTIFY_ASYNC);
    notify_set_signal_func((Notify_client) prog,
			   (Notify_func) exit_called_proc,
			   SIGBUS, /* Bus Error */
			   NOTIFY_ASYNC);
    notify_set_signal_func((Notify_client) prog,
			   (Notify_func) exit_called_proc,
			   SIGSEGV, /* Segmentation Violation */
			   NOTIFY_ASYNC);
#ifdef SIGSYS   /* PWH 19980323: SIGSYS not defined under Linux */
    notify_set_signal_func((Notify_client) prog,
			   (Notify_func) exit_called_proc,
			   SIGSYS, /* Function Call Error */
			   NOTIFY_ASYNC);
#endif /* SIGSYS */
#if 0
    notify_set_signal_func((Notify_client) prog,
			   (Notify_func) exit_called_proc,
			   SIGTSTP, /* CTRL-Z */
			   NOTIFY_ASYNC);
#endif
  }

  return 0;
}


/*----------------------------------------------------------------------------
error_handler_reason - ???
----------------------------------------------------------------------------*/

static char *
error_handler_reason(int code)
{
  static char str[100];

  /*  DRW 961113  Changed if-else if's to switch() statement  */
  switch (code)
  {
  case SIGILL:
    sprintf(str,"err = %d...illegal instruction.\n",code); break;
  case SIGTRAP:
    sprintf(str,"err = %d...trace trap.\n",code); break;
  case SIGABRT:
    sprintf(str,"err = %d...abort.\n",code); break;
#ifdef SIGEMT   /* PWH 19980323: SIGEMT not defined under Linux */
  case SIGEMT:
    sprintf(str,"err = %d...emulator trap.\n",code); break;
#endif /* SIGEMT */
  case SIGFPE:
    sprintf(str,"err = %d...arithmetic exception.\n",code); break;
  case SIGBUS:
    sprintf(str,"Bus error.\n"); break;
  case SIGSEGV:
    sprintf(str,"Segmentation violation.\n"); break;
#ifdef SIGSYS   /* PWH 19980323: SIGSYS not defined under Linux */
  case SIGSYS:
    sprintf(str,"Bad argument to system call.\n"); break;
#endif /* SIGSYS */
  case SIGKILL:
    sprintf(str,"Process killed.\n"); break;
  case SIGQUIT:
    sprintf(str,"Quit signal (CTRL-| ?) received.\n"); break;
  case SIGINT:
    sprintf(str,"Interrupt signal (CTRL-C ?) received.\n"); break;
  default:
    sprintf(str,"Signal (%d) received: consult signal.h\n",code); break;
  }

  PRT(printf("NRIA error: %s",str));
  return str;
}


/*--------------------------------------------------------------------------
xv_error_handler - ???
--------------------------------------------------------------------------*/

int
xv_error_handler(
   Xv_object obj,
   Attr_avlist vlist)
{
 static unsigned char is_abort = 0;
 Attr_avlist attrs;
 int err;
 /* NOT USED?:  Error_severity severity = ERROR_RECOVERABLE; */
 struct prog_rec *prog;
 Frame *frame;
 char *str;

 PRT2(printf("***xv_error_handler..obj = %u vlist = %u is_abort = %d\n",
         obj,vlist,(unsigned long) is_abort);)
 if (is_abort)
    {PRT2(printf("exit(XV_OK)...1\n"))
     exit(XV_OK);}

 for (  err = 0,attrs = vlist; 
        *attrs != (Attr_attribute)0;
        attrs = attr_next(attrs) )
    {PRT2(printf("xv_error_handler attrs[0] = %x --- \n",attrs[0]);)
     switch ((int) attrs[0])
        {case ERROR_INVALID_OBJECT :
            PRT2(printf("ERROR_INVALID_OBJECT...\n");)
            break;
         case ERROR_STRING :
            PRT2(printf("ERROR_STRING...\n");)
            break;
         case ERROR_BAD_ATTR :
            PRT2(printf("ERROR_BAD_ATTR...\n");)
            break;
         case ERROR_BAD_VALUE :
            PRT2(printf("ERROR_BAD_VALUE...\n");)
            break;
         case ERROR_CANNOT_GET :
            PRT2(printf("ERROR_CANNOT_GET...\n");)
            break;
         case ERROR_CANNOT_SET :
            PRT2(printf("ERROR_CANNOT_SET...\n");)
            break;
         case ERROR_CREATE_ONLY :
            PRT2(printf("ERROR_CREATE_ONLY...\n");)
            break;
         case ERROR_SEVERITY :
            PRT2(printf("ERROR_SEVERITY...\n");)
            if (attrs[1] != ERROR_RECOVERABLE) err = 1;
            break;
         case 0x4c120921 :
         case 0x4c150a01 :
            PRT2(printf("ERROR IGNORED...\n");)
            break;
         default : if (attrs[0] != 0) err = 1;
        }
     if (err != 0) break;}

 PRT2(printf("===xv_error_handler...err = %d\n",err))
 if (err != 0)
    {frame = (Frame *) main_display(2,NULL,0,0);
     if (obj != (Xv_object)0) 
        str = xv_error_format(obj,vlist);
     else str = "";
     if (debugFlag) printf("====err str = %s\n",str);
     is_abort = 1;
     notice_prompt(*frame,(Event *) NULL,
		   NOTICE_MESSAGE_STRINGS,
		   "Sorry, an XView system error occurred.  Attempting",
		   "to save data.  When notifying programmer,",
		   "please mention the following information:",
		   str,
		   0,
		   NOTICE_BUTTON_YES," I understand ",
		   0);
     PRT1(printf("saving before aborting  is_abort= %d\n",is_abort))
     if (*frame != (Frame)0)
        {
         TRACE(printf("saving before aborting\n"))
         prog = (struct prog_rec *) xv_get(*frame,WIN_CLIENT_DATA);
         close_all_files(&(prog->input),1);
         free_prog_rec(prog);
        }

     PRT1(printf("exit(XV_OK)...\n"))
     if (abortFlag)  abort();
     exit(XV_OK);
    }
 PRT2(printf("+++xv_error_handler...end\n"))
 return XV_OK;
}
/*
   else 
    {PRT2(printf("xv_error_handler...obj = %u vlist = %u\n",obj,vlist))
     str = xv_error_format(obj,vlist);
     PRT2(printf("***err str = %s\n",str))
    }
*/


/*----------------------------------------------------------------------------
bad_exit_called_proc - ???
----------------------------------------------------------------------------*/

Notify_value
bad_exit_called_proc( 
     struct prog_rec *prog,
     int sig,
     Notify_signal_mode when)
{
  static unsigned char is_abort = 0;
  char *str1;
  int code;
  Frame *frame;


  TRACE(printf("\nbad_exit_called_proc(): sig = %d, (prog *) = %p\n",
	       sig, (void *) prog));

  if (is_abort)
  {
    PRT1(printf("bad_exit_called_proc exit(XV_OK)...1\n"));
    exit(XV_OK);
  }
  if (prog != NULL)
  {
    code = notify_get_signal_code();

    is_abort = 1;

#if 0
    str1 = (char *) error_handler_reason(code);
#else  /* Let's try this again. */
    str1 = (char *) error_handler_reason( sig );
#endif

    frame = (Frame *) main_display(2,NULL,0,0);
    if (*frame != (Frame)0)
    {
      notice_prompt(*frame,(Event *) NULL,
		    NOTICE_MESSAGE_STRINGS,
		    "Sorry, a program error occurred.  Attempting",
		    "to save data.  When notifying programmer,",
		    "please mention the following information:",
		    str1,
		    0,
		    NOTICE_BUTTON_YES," I understand ",
		    0);
    }
    close_all_files(&(prog->input),1);
    free_prog_rec(prog);
    PRT1(printf("bad_exit_called_proc exit(XV_OK)...2\n"));
  }
  else
    printf("bad_exit_called_proc prog == NULL...unable to save data\n");
  exit(XV_OK);
}
