/****************************************************************************
import_rois.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: import_rois.c,v 1.9 1999/07/07 17:41:17 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/wmgr.h>

#include "geometry.h"
#include "mri_format.h"
#include "nria_procedures.h"

/*
#define PRINT(x) printf(x); fflush(stdout);
#define PRT(x) x; fflush(stdout);
#define PRT2(x) x; fflush(stdout);
#define PRT8(x) x; fflush(stdout);
*/
#define PRINT(x)
#define PRT(x)
#define PRT2(x)
#define PRT8(x)



#define CLEAN_UP_DP 0
#define SETUP_DP 1
#define GET_ALL_DP_PTR 2


struct import_roi_rec
{
  unsigned short status;
  struct image_display_window *win;
  struct display_win_rec *dwin;
  Frame frame;
  Panel panel;
  Panel_item wid,use_option,exist_option,start,srcstart,num_slices,reverse;
};


struct all_import_roi_recs
{
  unsigned long numdps;
  struct import_roi_rec **dps;
};


/* Functions defined in this file */
void init_import_roi_rec(struct import_roi_rec *dp);
void free_import_roi_rec(struct import_roi_rec *dp);
void init_all_import_roi_recs(struct all_import_roi_recs *alldps);
void dp_import_roi_done_proc(Frame frame);
void import_roi_use_option_proc(Panel_item item, int value, Event *event);
void import_roi_fns_proc(Panel_item item, Event *event);
void import_slice_num_proc(Panel_item item, int value, Event *event);
void import_fn_proc(Panel_item item, char *fn, caddr_t client_data,
    Panel_list_op op, Event *event);
int win_import_roi(struct image_display_window *win,
    struct input_file_rec *file, int use_option, int exist_option, int start,
    int srcstart, int num_slices, int reverse);
void import_rois_proc(Panel_item item, Event *event);
int setup_import_roi_rec(struct import_roi_rec *dp,
    struct image_display_window *win);
struct import_roi_rec *find_import_roi_rec(struct all_import_roi_recs *alldps,
    struct image_display_window *win);
struct import_roi_rec *find_import_roi_rec_by_file(
   struct all_import_roi_recs *alldps, struct input_file_rec *file);
struct all_import_roi_recs *enter_import_rois(
   struct image_display_window *win, int status);



/*-----------------------------------------------------------------------
init_import_roi_rec - ???
------------------------------------------------------------------------*/

void
init_import_roi_rec(dp)
 struct import_roi_rec *dp;
{
 dp->status = 0;
 dp->win = NULL;
 dp->frame = 0;
}



/*-----------------------------------------------------------------------
free_import_roi_rec - ???
------------------------------------------------------------------------*/

void
free_import_roi_rec(dp)
 struct import_roi_rec *dp;
{
 struct all_import_roi_recs *alldps;

 PRT(printf("free_import_roi_rec...\n"))
 if (dp->frame != 0) xv_destroy_safe(dp->frame);
 alldps = enter_import_rois(NULL, GET_ALL_DP_PTR);
 c_free(dp);
 DropHandleAryPtr(&(alldps->dps),&(alldps->numdps),dp);
 PRT(printf("free_import_roi_rec...end\n"))
}


/*-----------------------------------------------------------------------
init_all_import_roi_recs - ???
------------------------------------------------------------------------*/

void
init_all_import_roi_recs(alldps)
 struct all_import_roi_recs *alldps;
{
 alldps->numdps = 0;
 alldps->dps = NULL;
}



/*-----------------------------------------------------------------------
dp_import_roi_done_proc - ???
------------------------------------------------------------------------*/

void
dp_import_roi_done_proc(frame)
 Frame frame;
{
 struct import_roi_rec *dp;

 dp = (struct import_roi_rec *) xv_get(frame,WIN_CLIENT_DATA);
 free_import_roi_rec(dp);
}



/*-----------------------------------------------------------------------
import_roi_use_option_proc - ???
------------------------------------------------------------------------*/

void
import_roi_use_option_proc(item,value,event)
     Panel_item item;
     int value;
     Event *event;
{
  struct import_roi_rec *dp;
  struct file_register_rec *fco;
  struct img_buf_rec *img;
  struct input_file_rec *file;
  unsigned long indx;
  char *fn;

  if (value == 0)
      return /*NO_ERROR*/;
  dp = (struct import_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
  PRT8(printf("import_roi_use_option_proc...dp = %u\n",dp));
  if (((img = win_rec_cur_img_buf(dp->win->owner)) == NULL) ||
      (next_selected_item(dp->wid,0,&indx) != NO_ERROR) ||
      ((fn = (char *) xv_get(dp->wid,PANEL_LIST_STRING,indx)) == NULL))
      return /*CANCEL*/;
  PRT8(printf("import_roi_use_option_proc...fn = %s\n",fn));
  if ((file = find_file(img->file,fn,NULL)) == NULL)
    return /*CANCEL*/;
  PRT8(printf("import_roi_use_option_proc...value = %d\n",value));
  if (value)
    if ((fco = find_file_register_rec(img->file,file,NULL,2)) == NULL)
    {
      xv_set(item,PANEL_VALUE,0,0);
    }

}



/*-----------------------------------------------------------------------
import_roi_fns_proc - ???
------------------------------------------------------------------------*/

void
import_roi_fns_proc(item,event)
     Panel_item item;
     Event *event;
{
  struct import_roi_rec *dp;
  struct strs_rec fns;

  dp = (struct import_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
  PRT8(printf("import_roi_fns_proc \n"));
  init_strs_rec(&fns);
  selected_list_items(&fns,dp->wid);
  delete_all_list_items(dp->wid);
  WinSetupFileListItem(dp->win,dp->wid,0);
  select_list_items(&fns,dp->wid);
  free_strs_rec(&fns);
}


/*-----------------------------------------------------------------------
import_slice_num_proc - ???

Purpose:  Range checking.

Called whenever a slice # widget is changed:
   Starting slice # (dp->start)
   Source starting slice # (dp->srcstart)
   # Slices (dp->num_slices)
------------------------------------------------------------------------*/

void
import_slice_num_proc( item, value, event )
     Panel_item item;
     int value;
     Event *event;
{
  struct import_roi_rec *dp;
  int val0,val,val1,val2,val3,val4;

  dp = (struct import_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
  if (value == 0)
  {
    xv_set(item,PANEL_VALUE,1,0);
    val0 = 1;
  }
  else
    val0 = value;

  val = (int) xv_get( dp->start, PANEL_VALUE );
  val1 = (int) xv_get( dp->srcstart, PANEL_VALUE );
  val2 = (int) xv_get( dp->num_slices, PANEL_VALUE );


  if (item == dp->start)
  {
    val3 = (int) xv_get(dp->start,PANEL_MAX_VALUE);
    if ((val4 = (val3 - val0)) < val2) 
      if (val4 != value)
	xv_set(dp->num_slices,PANEL_VALUE,val4,0);
  }

  else if (item == dp->srcstart)
  {
    val3 = (int) xv_get(dp->srcstart,PANEL_MAX_VALUE);
    if ((val4 = (val3 - val0)) < val2) 
      if (val4 != value)
	xv_set(dp->num_slices,PANEL_VALUE,val4,0);
  }

  else if (item == dp->num_slices)
  {
    val3 = (int) xv_get(dp->start,PANEL_MAX_VALUE);
    if ((val4 = (val3 - val)) < val0)
      val0 = val4;
    val3 = (int) xv_get(dp->srcstart,PANEL_MAX_VALUE);
    if ((val4 = (val3 - val1)) < val0)
      val0 = val4;
    if (val4 != value)
      xv_set(dp->num_slices,PANEL_VALUE,value,0);
  }

}



/*-----------------------------------------------------------------------
import_fn_proc - ???
------------------------------------------------------------------------*/

void
import_fn_proc( item, fn, client_data, op, event )
     Panel_item item;
     char *fn;
     caddr_t client_data;
     Panel_list_op op;
     Event *event;
{
  struct import_roi_rec *dp;
  struct img_buf_rec *img;
  unsigned long indx,num_slices,num_slices1;
  struct input_file_rec *file;
  int is_del;
  struct file_register_rec *fco;

  dp = (struct import_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
  if ((img = win_rec_cur_img_buf(dp->win->owner)) == NULL)
      return /*CANCEL*/;

  while (1)
  {
    if ((file = find_file(img->file,fn,NULL)) == NULL)
    {
      is_del = 1;
      PRT8(printf("import_fn_proc...delete_list_item_by_str\n"));
      delete_list_item_by_str(fn,dp->wid);
      if (next_selected_item(dp->wid,0,&indx) != NO_ERROR)
      {
	win_alarm(dp->frame,2);
	return /*CANCEL*/;
      }
      fn = (char *) xv_get(dp->wid,PANEL_LIST_STRING,indx);
      PRT8(printf("import_fn_proc...next fn = %s\n",fn));
    }
    else
      break;
  }

  num_slices = NumberOfSliceCuts(&(img->file->hdr.data_info));
  num_slices1 = NumberOfSliceCuts(&(file->hdr.data_info));
  if (((fco = find_file_register_rec(img->file,file,NULL,0)) == NULL) ||
      ((fco->co.status & CO_REG_PARAM_REG_IN_USE) != CO_REG_PARAM_REG_IN_USE))
    xv_set(dp->use_option,PANEL_VALUE,0,0);
  else
    xv_set(dp->use_option,PANEL_VALUE,1,0);
  
  if (num_slices1 > num_slices)
    indx = num_slices;
  else
    indx = num_slices1;

  xv_set(dp->srcstart,PANEL_INACTIVE,FALSE,PANEL_VALUE,1,
	 PANEL_MIN_VALUE,0,PANEL_MAX_VALUE,(int) num_slices1,0);
  xv_set(dp->num_slices,PANEL_INACTIVE,FALSE,PANEL_VALUE,indx,
	 PANEL_MIN_VALUE,0,PANEL_MAX_VALUE,(int) indx,0);
  xv_set(dp->start,PANEL_VALUE,1,0);
  import_roi_use_option_proc(item,1,event);
}


/*-----------------------------------------------------------------------
win_import_roi - ???

Called from widget routine import_rois_proc() below.

file comes from current image.
exist_option, start, srcstart, num_slices, and reverse come from widgets.
------------------------------------------------------------------------*/

int
win_import_roi( win, file, use_option, exist_option,
		start, srcstart, num_slices, reverse )
     struct image_display_window *win;
     struct input_file_rec *file;
     int use_option,exist_option,start,srcstart,num_slices,reverse;
{
  struct file_register_rec *fco;
  struct img_buf_rec *img;
  unsigned long lcv,indx,cnt,num,num_slices1,num_slices2;
  struct slice_roi_win_rec **sroi_wins1,**sroi_wins2;
  struct slice_roi_rec slice_roi,*sroi1,*sroi2;
  struct roi_rec *roi,*roi1;
  int err,is_in;

 PRT8(printf("win_import_roi use_option = %d exist_option = %d\n",
               use_option,exist_option))
 PRT8(printf("start = %d srcstart = %d num_slices = %d reverse = %d\n",
               start,srcstart,num_slices,reverse))
 if (((img = win_rec_cur_img_buf(win->owner)) == NULL) ||
     ((fco = find_file_register_rec(img->file,file,NULL,use_option)) == NULL))
    return CANCEL;
 num_slices1 = NumberOfSliceCuts(&(img->file->hdr.data_info));
 num_slices2 = NumberOfSliceCuts(&(file->hdr.data_info));
 PRT8(printf("win_import_roi num_slices1 = %d num_slices2 = %d\n",
           num_slices1,num_slices2))
 if (start >= num_slices1) return CANCEL;
 if (srcstart >= num_slices2) return CANCEL;
 if ((start + num_slices - 1) >= num_slices1) num_slices = num_slices1 - start;
 if ((srcstart + num_slices - 1) >= num_slices2) num_slices = num_slices2 - srcstart;
 PRT8(printf("win_import_roi num_slices = %d\n",num_slices))
 init_slice_roi_rec(&slice_roi);
 sroi_wins1 = img->file->slice_info.sroi_wins;
 sroi_wins2 = file->slice_info.sroi_wins;
 err = NO_ERROR;
 if (reverse) cnt = srcstart + num_slices - 1;
   else cnt = srcstart;

 PRT8(printf("fco->co.reg.planetoxyz...\n"))
 PRT8(pr_3d_matrix(&(fco->co.reg.planetoxyz)))
 PRT8(printf("fco->co.reg.xyztoplane...\n"))
 PRT8(pr_3d_matrix(&(fco->co.reg.xyztoplane)))
 PRT8(printf(" .coregxyztoregxyz...\n"))
 PRT8(pr_2d_matrix(&(fco->coregxyztoregxyz)))

 PRT8(printf("fco->co.coreg.planetoxyz...\n"))
 PRT8(pr_3d_matrix(&(fco->co.coreg.planetoxyz)))
 PRT8(printf("fco->co.coreg.xyztoplane...\n"))
 PRT8(pr_3d_matrix(&(fco->co.coreg.xyztoplane)))


 for (is_in = 0,indx = start,lcv = 0; lcv < num_slices; lcv++,indx++)
    {sroi2 = &(sroi_wins2[cnt]->slice_roi);
     PRT8(printf("lcv = %d) indx = %d cnt = %d num_rois = %d\n",lcv,indx,cnt,sroi2->num_rois))
     if (is_slice_roi_non_empty(sroi2))
        {PRT8(printf("copying...\n"))
         if ((err = CpSliceRoiRec(&slice_roi,sroi2,NULL,NULL,
               &(fco->coregxyztoregxyz))) == NO_ERROR)
            {sroi1 = &(sroi_wins1[indx]->slice_roi);
             PRT8(printf("indx = %d...slice_roi.num_rois = %d sroi1 = %u\n",indx,
                        slice_roi.num_rois,sroi1))
             for (num = 0; num < slice_roi.num_rois; num++)
                if (((roi = slice_roi.rois[num]) != NULL) &&
                   (roi->label != NULL) && is_roi_non_empty(roi))
                   {is_in = 1;
                    PRT8(printf("%d) label = %s\n",num,roi->label))
                    if (exist_option == 0)
                       if ((roi1 = get_roi(sroi1,roi->label)) != NULL)
                          delete_roi(sroi1,roi1);
                    if ((err = JoinRoiToSliceRoiRec(sroi1,roi,0,1.0,1.0,
                          NULL,NULL,NULL)) != NO_ERROR)
                       {
                        break;}
                    PRT8(printf("sroi1 num_rois = %d...\n",sroi1->num_rois))
                   }   
             }
          PRT(pr_slice_roi_rec(&slice_roi);)
          free_slice_roi_rec(&slice_roi);
          if (err != NO_ERROR) break;
         }
     if (reverse) cnt--;
       else cnt++;
    }
 PRT8(printf("win_import_roi is_in = %d err = %d\n",is_in,err))
 if (is_in)
    win_run_proc_to_all_file_wins(win,win_redisplay_all_curves,1,0,0,0);
 return err;
}

/*-----------------------------------------------------------------------
import_rois_proc - ???
------------------------------------------------------------------------*/

void
import_rois_proc( item, event )
     Panel_item item;
     Event *event;
{
  struct import_roi_rec *dp;
  int use_option,exist_option,start,srcstart,num_slices,reverse;
  struct img_buf_rec *img,*img1;
  struct image_display_window *win1;
  struct input_file_rec *file;
  struct file_register_rec *fco;
  unsigned long indx;
  char *fn;


  dp = (struct import_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);

  /*
   *  Get file structure associated with current image
   */
  if (((img = win_rec_cur_img_buf(dp->win->owner)) == NULL) ||
      (next_selected_item(dp->wid,0,&indx) != NO_ERROR) ||
      ((fn = (char *) xv_get(dp->wid,PANEL_LIST_STRING,indx)) == NULL))
      return /*CANCEL*/;
  if ((file = find_file(img->file,fn,NULL)) == NULL)
  {
    delete_list_item_by_str(fn,dp->wid);
    return /*CANCEL*/;
  }

  use_option  = (int) xv_get(dp->use_option,PANEL_VALUE) + 1;
  PRT8(printf("import_rois_proc indx = %d fn = %s use_option = %d\n",
	      indx,fn,use_option));
  if ((fco = find_file_register_rec(img->file,file,NULL,use_option)) == NULL)
  {
    if (use_option == 1)
    {
      if ((file != img->file) && 
	  ((win1 = find_file_first_window(file,NULL,1)) != NULL) &&
	  ((img1 = win_rec_cur_img_buf(win1->owner)) != NULL) &&
	  (SetupImgRegParams(img,img1,0x00000007,NULL,NULL) == NO_ERROR))
      {
	fco = find_file_register_rec(img->file,file,NULL,use_option);
	PRT8(printf("import_rois_proc fco = %u\n",fco));
      }
    }

    if (fco == NULL)
    {
      notice_prompt(dp->win->dwin->frame,
		    (Event *) NULL,NOTICE_MESSAGE_STRINGS,
		    "Sorry...this co-registered parameter option",
		    "is not available now !!!",0,
		    NOTICE_BUTTON_YES,FAILED,0);
      return /*CANCEL*/;
    }
  }

  exist_option = (int) xv_get(dp->exist_option,PANEL_VALUE);
  if ((start = (int) xv_get(dp->start,PANEL_VALUE)) > 0)
    start -= 1;
  if ((srcstart = (int) xv_get(dp->srcstart,PANEL_VALUE)) > 0)
    srcstart -= 1;
  num_slices = (int) xv_get(dp->num_slices,PANEL_VALUE);
  reverse = (int) xv_get(dp->reverse,PANEL_VALUE);

  PRT8(printf("import_rois_proc win_import_roi\n"));
  if (win_import_roi( dp->win, file, use_option, exist_option, start,
		      srcstart, num_slices, reverse ) != NO_ERROR)
    notice_prompt( dp->win->dwin->frame,(Event *) NULL,
		   NOTICE_MESSAGE_STRINGS,
		   "An error occurred",
		   "when importing rois !!!" ,0,
		   NOTICE_BUTTON_YES,FAILED,0);
}



/*-----------------------------------------------------------------------
setup_import_roi_rec - ???
------------------------------------------------------------------------*/

int
setup_import_roi_rec( dp, win )
     struct import_roi_rec *dp;
     struct image_display_window *win;
{
  Panel_item item;
  int row,offset;
  unsigned long indx,num_slices;
  struct img_buf_rec *img;
  char *str;
  Rect *rr,*rr1;

  PRT2(printf("setup_import_roi_rec...\n"));
  if ((img = win_rec_cur_img_buf(win->owner)) == NULL)
    return CANCEL;
  
  dp->frame = xv_create(win->dwin->frame, FRAME,
			FRAME_INHERIT_COLORS,TRUE,
			WIN_CLIENT_DATA,dp,
			0);
  if (dp->frame == 0)
  {
    free_import_roi_rec(dp);
    return ~NO_ERROR;
  }
  dp->win = win;
  dp->dwin = win->dwin;
  str = win_rec_win_id_title(win->owner,IMPORT_SLICE_ROIS);
  xv_set(dp->frame,
	 FRAME_SHOW_LABEL, TRUE,
	 XV_LABEL,str,
	 FRAME_NO_CONFIRM,TRUE,
	 FRAME_DONE_PROC,dp_import_roi_done_proc,
	 0);
  c_free(str);
  xv_set(dp->frame,FRAME_CLOSED,FALSE,0);
  PRT2(printf("setup_import_roi_rec...1\n"));
  
  dp->panel = xv_create(dp->frame,PANEL,
			FRAME_INHERIT_COLORS,TRUE,
			0);
  
  if (dp->panel == 0)
  {
    free_import_roi_rec(dp);
    return ~NO_ERROR;
  }
  row = 5;
  offset = xv_rows(dp->panel,1) - xv_rows(dp->panel,0) - 10;
  item = xv_create(dp->panel,PANEL_BUTTON,
		   PANEL_LABEL_STRING,"IMPORT ROIS",
		   PANEL_NOTIFY_PROC,import_rois_proc,
		   XV_X,5,XV_Y,row,
		   PANEL_CLIENT_DATA,dp,
		   0);
  row += offset - 4;
  dp->use_option = xv_create(dp->panel,PANEL_CHOICE_STACK,
			     PANEL_LABEL_STRING,"BY CO-REG PARAMS =",
			     PANEL_LAYOUT,PANEL_HORIZONTAL,
			     PANEL_CHOICE_STRINGS,"DEFAULT",
			     "REGISTERED FROM 2 WIDS",NULL,
			     PANEL_NOTIFY_PROC,import_roi_use_option_proc,
			     PANEL_CLIENT_DATA,dp,
			     XV_X,15,XV_Y,row,
			     PANEL_VALUE,0,
			     0);

  row += offset - 4;
  rr1 = (Rect *) xv_get(item,XV_RECT);
  dp->exist_option = xv_create(dp->panel,PANEL_CHOICE_STACK,
			       PANEL_LABEL_STRING,"IF IMPORTED ROIS EXIST THEN",
			       PANEL_LAYOUT,PANEL_HORIZONTAL,
			       PANEL_CHOICE_STRINGS,"REPLACE","APPEND",NULL,
			       XV_X,15,XV_Y,row,
			       PANEL_VALUE,0,
			       0);
  row += offset - 5;
  num_slices = NumberOfSliceCuts(&(img->file->hdr.data_info));
  dp->start = xv_create(dp->panel,PANEL_SLIDER,
			PANEL_LABEL_STRING,"START SLICE#",
			PANEL_VALUE,1,
			PANEL_MIN_VALUE,0,
			PANEL_MAX_VALUE,(int) num_slices,
			PANEL_SLIDER_WIDTH,100,
			XV_X,5,XV_Y,row,
			PANEL_NOTIFY_PROC,import_slice_num_proc,
			PANEL_CLIENT_DATA,dp,
			0);

  row += offset - 2;
  item = xv_create(dp->panel,PANEL_BUTTON,
		   PANEL_LABEL_STRING,"FILEs",
		   PANEL_NOTIFY_PROC,import_roi_fns_proc,
		   PANEL_CLIENT_DATA,dp,
		   XV_X,5,XV_Y,row,
		   0);

  row += offset;
  dp->wid = xv_create(dp->panel,PANEL_LIST,
		      PANEL_LIST_DISPLAY_ROWS,4,
		      PANEL_LAYOUT,PANEL_VERTICAL,
		      PANEL_NOTIFY_PROC,import_fn_proc,
		      PANEL_CLIENT_DATA,dp,
		      XV_X,5,XV_Y,row,
		      0);
  WinSetupFileListItem(dp->win,dp->wid,0);
  rr = (Rect *) xv_get(dp->wid,XV_RECT);

  PRT2(printf("setup_import_roi_rec...1\n"));
  row = rr->r_top + rr->r_height + 2;
  
  dp->srcstart = xv_create(dp->panel,PANEL_SLIDER,
			   PANEL_LABEL_STRING,"SRC START SLICE#",
			   PANEL_VALUE,1,
			   PANEL_MIN_VALUE,0,
			   PANEL_MAX_VALUE,(int) num_slices,
			   PANEL_SLIDER_WIDTH,100,
			   XV_X,5,XV_Y,row,
			   PANEL_NOTIFY_PROC,import_slice_num_proc,
			   PANEL_CLIENT_DATA,dp,
			   PANEL_INACTIVE,TRUE,
			   0);

  row += offset - 5;
  dp->num_slices = xv_create(dp->panel,PANEL_SLIDER,
			     PANEL_LABEL_STRING,"# of SLICES",
			     PANEL_VALUE,num_slices,
			     PANEL_MIN_VALUE,0,
			     PANEL_MAX_VALUE,(int) num_slices,
			     PANEL_SLIDER_WIDTH,100,
			     XV_X,5,XV_Y,row,
			     PANEL_NOTIFY_PROC,import_slice_num_proc,
			     PANEL_CLIENT_DATA,dp,
			     PANEL_INACTIVE,TRUE,
			     0);
  row += offset - 5;
  dp->reverse = xv_create(dp->panel,PANEL_CHECK_BOX,
			  PANEL_CHOICE_STRINGS,"REVERSE",NULL,
			  PANEL_VALUE,0,
			  XV_X,15,XV_Y,row,
			  0);

  if (next_selected_item(dp->wid,0,&indx) == NO_ERROR)
  {
    str = (char *) xv_get(dp->wid,PANEL_LIST_STRING,indx);
    import_fn_proc(dp->wid,str,NULL,0,NULL);
  }
  
  window_fit(dp->panel);
  rr = (Rect *) xv_get(dp->panel,XV_RECT);
  xv_set(dp->panel,XV_WIDTH,rr->r_width - 20,
	 XV_HEIGHT,rr->r_height - 10,0);

  PRT2(printf("setup_import_roi_rec...11\n"));
  window_fit(dp->frame);
  make_2_wins_adjacent(dp->frame,win->dwin->frame,0);
  win_set_mouse_at_center(dp->frame);

  return NO_ERROR;
}


/*-----------------------------------------------------------------------
find_import_roi_rec - ???
------------------------------------------------------------------------*/

struct import_roi_rec *
find_import_roi_rec( alldps, win )
     struct all_import_roi_recs *alldps;
     struct image_display_window *win;
{
  int lcv;
  struct img_buf_rec *img,*img1;
  
  if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return NULL;
  for (lcv = 0; lcv < alldps->numdps; lcv++)
    if ((img1 = win_rec_cur_img_buf(alldps->dps[lcv]->win->owner)) != NULL)
      if (img1->file == img->file)
	return alldps->dps[lcv];
  return NULL;
}



/*-----------------------------------------------------------------------
find_import_roi_rec_by_file - ???
------------------------------------------------------------------------*/

struct import_roi_rec *
find_import_roi_rec_by_file( alldps, file )
     struct all_import_roi_recs *alldps;
     struct input_file_rec *file;
{
  int lcv;
  struct img_buf_rec *img1;

  for (lcv = 0; lcv < alldps->numdps; lcv++)
    if ((img1 = win_rec_cur_img_buf(alldps->dps[lcv]->win->owner)) != NULL)
      if (img1->file == file)
	return alldps->dps[lcv];
  return NULL;
}


/*-----------------------------------------------------------------------
enter_import_rois - ???

BEWARE: The pre-19990224 version of this function could return either
NULL, a pointer to an all_edge_detector_recs structure, or an integer
error code.  Since the last two of these cannot be reliably
distinguished and since no one ever seems to check for an error code,
I replaced the error code by NULL.  Of course, it is now impossible to
distinguish between NO_ERROR and NULL (=> error), but I don't think
anyone ever looks at the error code returned.  -PWH
------------------------------------------------------------------------*/

struct all_import_roi_recs *
enter_import_rois(
     struct image_display_window *win,
     int status)
{
  static struct all_import_roi_recs alldps;
  static char first_time = 1;
  int err;
  struct import_roi_rec *dp;
  
  err = NO_ERROR;
  if (status == GET_ALL_DP_PTR)
  {
    if (first_time)
      return NULL;
    else
      return &alldps; 
  }
  else if (status == CLEAN_UP_DP)
  {
    if (!first_time)
      if ((dp = find_import_roi_rec(&alldps,win)) != NULL)
	free_import_roi_rec(dp);
    return NO_ERROR;
  }

  if (first_time)
  {
    init_all_import_roi_recs(&alldps);
    first_time = 0;
  }
  else
  { 
    if ((dp = find_import_roi_rec( &alldps, win )) != NULL)
    {
      xv_set(dp->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,WIN_ALARM,0);
      win_set_mouse_at_center(dp->frame);
      wmgr_top(dp->frame);
      return NO_ERROR;
    }
  }

  get_handle_memory(&(alldps.dps),&(alldps.numdps),1,
		    (size_t) sizeof(struct import_roi_rec),&err);
  if (err != NO_ERROR)
      return NULL;   /* Was: return err */
  dp = alldps.dps[alldps.numdps - 1];
  init_import_roi_rec(dp);
  if ((err = setup_import_roi_rec(dp,win)) == NO_ERROR)
    xv_set(dp->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
  return NULL;    /* Was: return err */
}
