/****************************************************************************
menu.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: menu.c,v 1.10 1999/07/07 17:41:17 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include "error_constants.h"
#include "str_def.h"
#include "typedef.h"
#include "menu.h"
#include "nria_lib.h"
#include "nria_procedures.h"


/*
#define TRACE(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
#define PRT1(x) x; fflush(stdout);
#define PRT2(x) x; fflush(stdout);
#define PRT3(x) x; fflush(stdout);
*/
#define TRACE(x)
#define PRT(x)
#define PRT1(x)
#define PRT2(x)
#define PRT3(x)



/* Functions defined in this file */
int SetupUseMenuRec(struct user_menu_rec **user, Menu menu, int is_free_menu,
    char *menu_title, struct strs_rec *strs_rec_ptr, int free_str_status,
    int (*action_proc)(), int (*proc)(), void *arg1, void *arg2, void *arg3,
    char *private_str, int is_return_proc_str_arg);
void free_a_user_menu_rec(struct user_menu_rec **user);
void free_user_menu_rec_tree(struct user_menu_rec *user);
void free_user_menu_rec(struct user_menu_rec **user);
void add_user_menu_rec_children(struct user_menu_rec *user, unsigned int num,
    ...);
int setup_menu_items(Menu *menu, void *proc, ...);
int user_menu_notify_proc(Menu menu, Menu_item item);
void user_menu_done_proc(Menu menu, Xv_opaque result);
int SetupTheMenu(Menu *menu, char *menu_title, int is_release, Xv_Window win,
    struct strs_rec *ptr, int (*action_proc)(), void *menu_client_data);
void menu_show_it(Menu menu, int which);
void SetMenuPtr(Menu menu, int is_setup);
int ShowMenuStrsProc(Xv_Window win, Event *event, char *menu_title,
    struct strs_rec *strs_rec_ptr, int free_str_status, int (*proc)(),
    void *arg1, int is_return_proc_str_arg, void *arg2, void *arg3);
void make_menu_items_inactive(Menu menu);
void make_menu_items_active(Menu menu);
void invert_menu_items(Menu menu, int yes);
void box_menu_items(Menu menu, int yes);
void free_user_on_error(struct user_menu_rec *user);
void free_dir_fns_on_error(struct user_menu_rec *user,
    struct user_menu_rec *user1, struct user_menu_rec *user2,
    struct user_menu_rec *user3, int user4, struct strs_rec *father_ptr,
    struct strs_rec *brother_ptr, struct strs_rec *son_ptr, int dirs,
    Menu menu, Menu menu1, Menu menu2, Menu menu3, Menu menu4);
void show_menu_dir_fns_proc(Menu menu, Menu_item item);
void show_menu_dir_fns_done_proc(Menu menu, Xv_opaque result);
int show_menu_dir_fns(Xv_Window win, Event *event, char *dir_fn,
    struct strs_rec *dirs, int (*action)(), void *arg);



/*--------------------------------------------------------------------------
SetupUseMenuRec - ???
--------------------------------------------------------------------------*/

int
SetupUseMenuRec(user,menu,is_free_menu,menu_title,strs_rec_ptr,free_str_status,
           action_proc,proc,arg1,arg2,arg3,private_str,is_return_proc_str_arg)
 struct user_menu_rec **user;
 Menu menu;
 int is_free_menu;
 char *menu_title;
 struct strs_rec *strs_rec_ptr;
 int free_str_status;
 int (*action_proc)();
 int (*proc)();
 void *arg1,*arg2,*arg3;
 char *private_str;
 int is_return_proc_str_arg;
{
 int err;

 if ((err = get_ptr(user,NULL,(unsigned long) 1,
          (size_t) sizeof(**user))) != NO_ERROR) 
    return err;
 (*user)->menu_title = NULL;
 (*user)->private_str = NULL;
 if (((err = cp_str(&((*user)->menu_title),menu_title)) != NO_ERROR) ||
     ((err = cp_str(&((*user)->private_str),private_str)) != NO_ERROR))
    {c_free((*user)->menu_title);
     c_free((*user)->private_str);
     c_free(*user);
     return err;}

 (*user)->menu = menu;

 (*user)->status = USER_MENU_SETUP;
 if (is_free_menu) (*user)->status |= USER_MENU_FREE_MENU;
 if (is_return_proc_str_arg) (*user)->status |= USER_MENU_RETURN_USER_PROC_STR_ARG;
 (*user)->strs_rec_ptr = strs_rec_ptr;
 if (free_str_status)
    (*user)->status |= free_str_status;
 (*user)->action_proc = action_proc;
 (*user)->proc = proc;
 (*user)->arg1 = arg1;
 (*user)->arg2 = arg2;
 (*user)->arg3 = arg3;
 (*user)->num_children = 0;
 (*user)->children = NULL;
 (*user)->root = NULL;
 return NO_ERROR;

}


/*--------------------------------------------------------------------------
free_a_user_menu_rec - ???
--------------------------------------------------------------------------*/

void
free_a_user_menu_rec(user)
 struct user_menu_rec **user;
{
 int x,y;
 Xv_Window win;
 Event *event;

 TRACE(printf("\nfree_a_user_menu_rec...user = %u\n",*user);)
 if (!((*user)->status & USER_MENU_SETUP)) return;
 if ((*user)->status & USER_MENU_FREE_MENU)
    if ((*user)->menu != 0)
       {if (((event = (Event *) xv_get((*user)->menu,MENU_FIRST_EVENT))
                != NULL) &&
            ((win = (Xv_Window) xv_get((*user)->menu,XV_KEY_DATA,10))
              != 0))
           {x = event_x(event);
            y = event_y(event);
            xv_set(win,WIN_MOUSE_XY,x,y,0);}
        SetMenuPtr((*user)->menu,0);
        PRT1(printf("free_a_user_menu_rec xv_destroy...\n"))
       }
 if ((*user)->strs_rec_ptr != NULL)
    if ((*user)->status & USER_MENU_FREE_STRS)
       {free_strs_rec((*user)->strs_rec_ptr);
        TRACE(printf("free_a_user_menu_rec USER_MENU_FREE_STRS...\n"))
       }
      else if ((*user)->status & USER_MENU_FREE_STR_PTRS)
       {free_strs_rec_ptrs((*user)->strs_rec_ptr);
        TRACE(printf("free_a_user_menu_rec USER_MENU_FREE_STR_PTRS...\n"))
       }
      else if ((*user)->status & USER_MENU_FREE_STRS_AND_PTR)
       {free_strs_rec((*user)->strs_rec_ptr);
        c_free((*user)->strs_rec_ptr);
        TRACE(printf("free_a_user_menu_rec USER_MENU_FREE_STRS_AND_PTR...\n"))
       }
      else if ((*user)->status & USER_MENU_FREE_STR_PTRS_AND_PTR)
       {free_strs_rec_ptrs((*user)->strs_rec_ptr);
        c_free((*user)->strs_rec_ptr);
        TRACE(printf("free_a_user_menu_rec USER_MENU_FREE_STR_PTRS_AND_PTR...\n"))
       }

 c_free((*user)->menu_title);
 c_free((*user)->private_str);
 if ((*user)->num_children) c_free((*user)->children);

 c_free((*user));
 *user = NULL;
}


/*--------------------------------------------------------------------------
free_user_menu_rec_tree - ???
--------------------------------------------------------------------------*/

void
free_user_menu_rec_tree(user)
 struct user_menu_rec *user;
{
 struct user_menu_rec *user1;
 unsigned int cnt;

 TRACE(printf("free_user_menu_rec_tree...\n"))
 if (user->num_children > 0)
    {for (cnt = 0; cnt < user->num_children; cnt++)
        {user1 = user->children[cnt];
         free_user_menu_rec_tree(user1);
         free_a_user_menu_rec(&user1);
        }
    }
 
}


/*--------------------------------------------------------------------------
free_user_menu_rec - ???
--------------------------------------------------------------------------*/

void
free_user_menu_rec(user)
 struct user_menu_rec **user;
{
 struct user_menu_rec *root;

 TRACE(printf("free_user_menu_rec...\n"))
 root = *user;
 for (;root->root != NULL;)
    root = root->root;
 if (root == NULL) return;
 free_user_menu_rec_tree(root);
 free_a_user_menu_rec(&root);
 *user = NULL;

}


/*--------------------------------------------------------------------------
add_user_menu_rec_children - ???

FIXME:  This function is apparently never used.
--------------------------------------------------------------------------*/

void
add_user_menu_rec_children(
    struct user_menu_rec *user,
    unsigned int num,
    ...)
{
 va_list args;
 struct user_menu_rec *next;
 unsigned int cnt;
 int err;

 if (num == 0) return /*NO_ERROR*/;

 get_ptr_memory(&(user->children),&(user->num_children),num,
            (size_t) sizeof(struct user_menu_rec *),&err);
 if (err != NO_ERROR) return /*err*/;

 va_start(args, num);
 for (cnt = 0; cnt < num; cnt++)
    {next = va_arg(args,struct user_menu_rec *);
     user->children[cnt] = next;
     next->root = user;}

 va_end(args);
}


/*--------------------------------------------------------------------------
setup_menu_items - ???

setup_menu_items(result_menu,action_proc[[,select,str,[menu_or_item,]
                  item_notify_proc,item_client_data],...],0)
 Menu *result_menu;
 void (*action_proc)();
 int select;  0 no more item  1 = menu else item
 char *str; MENU_STRING
 if (select == 1) then
     menu
     item_notify_proc
     item_client_data
   else
     item_notify_proc
     item_client_data
--------------------------------------------------------------------------*/

int
setup_menu_items(
    Menu *menu,
    void *proc,
    ...)
{
 va_list args;
 char *str,*str1;
 Menu menu1;
 int select,cnt;
 void *client_data;
 int err;

 if (*menu == 0)
    {*menu = xv_create(0,MENU,NULL);
     if (*menu == 0) return CANCEL;}
 err = NO_ERROR;

 xv_set(*menu,MENU_NOTIFY_PROC,proc,0);

 va_start(args, proc);
 for (cnt = 1; (select = va_arg(args,int)) != 0; cnt++)
    {if (select == 1)
        {
         str = va_arg(args,char *);
         menu1 = va_arg(args,Menu);
         proc = va_arg(args,void *);
         client_data = va_arg(args,void *);
         TRACE(printf("setup_menu_items str = %s menu1 = %u proc = %u client_data = %u\n",
              str,menu1,proc,client_data))
         if (str != NULL)
            if (menu1 != 0)
               if ((err = cp_str(&str1,str)) == NO_ERROR)
                  xv_set(*menu,
                       MENU_ITEM,
                       MENU_CLIENT_DATA,client_data,
                       MENU_STRING,str1,MENU_PULLRIGHT,menu1,
                       MENU_NOTIFY_PROC,proc,0,
                       MENU_RELEASE_IMAGE,
                       0);
                 else break;
        }
       else
        {str = va_arg(args,char *);
         proc = va_arg(args,void *);
         client_data = va_arg(args,void *);
         TRACE(printf("setup_menu_items str = %s proc = %u client_data = %u\n",
              str,proc,client_data))
         if (str != NULL)
            if ((err = cp_str(&str1,str)) == NO_ERROR)
               xv_set(*menu,
                       MENU_ITEM,
                       MENU_CLIENT_DATA,client_data,
                       MENU_STRING,str1,
                       MENU_RELEASE_IMAGE,
                       MENU_NOTIFY_PROC,proc,0,
                       0);
             else break;
        }
    }
 va_end(args);
 return err;
}


/*--------------------------------------------------------------------------
user_menu_notify_proc - ???
--------------------------------------------------------------------------*/

int
user_menu_notify_proc(menu,item)
 Menu menu;
 Menu_item item;
{
 struct user_menu_rec *user;
 int result,err;

 user = (struct user_menu_rec *) xv_get(menu,MENU_CLIENT_DATA);
 PRT(printf("user_menu_notify_proc...user = %u\n",user))
 if (user == NULL) return NO_ERROR;
 PRT(printf("user->proc = %u  \n",user->proc))
 if (user->strs_rec_ptr != NULL)
    {
     result = (int) xv_get(item,MENU_VALUE);
     if ((result > 0) && (result <= user->strs_rec_ptr->num_strs))
        {user->strs_rec_ptr->cur_str = result - 1;
        }
       else user->strs_rec_ptr->cur_str = 0;
    }
 err = NO_ERROR;
 if (user->proc != NULL) 
    if (user->status & USER_MENU_RETURN_USER_PROC_STR_ARG)
       {if ((user->strs_rec_ptr != NULL) && 
            (user->strs_rec_ptr->cur_str < user->strs_rec_ptr->num_strs))
           err = user->proc(user->arg1,
                 user->strs_rec_ptr->strs[user->strs_rec_ptr->cur_str],
                 user->arg2,user->arg3);
       }
      else err = user->proc(user->arg1,user,user->arg2,user->arg3);
 return err;
}


/*--------------------------------------------------------------------------
user_menu_done_proc - ???
--------------------------------------------------------------------------*/

void
user_menu_done_proc(menu,result)
 Menu menu;
 Xv_opaque result;
{
 struct user_menu_rec *user;

 user = (struct user_menu_rec *) xv_get(menu,MENU_CLIENT_DATA);
 PRT(printf("user_menu_done_proc...user = %u\n",user))

 if (user != NULL)
    {
     free_user_menu_rec(&user);}
 PRT(printf("user_menu_done_proc...end\n"))
}


/*--------------------------------------------------------------------------
SetupTheMenu - ???
--------------------------------------------------------------------------*/

int
SetupTheMenu(menu,menu_title,is_release,win,ptr,action_proc,menu_client_data)
 Menu *menu;
 char *menu_title;
 int is_release;
 Xv_Window win;
 struct strs_rec *ptr;
 int (*action_proc)();
 void *menu_client_data;
{
 char *str;
 int err,is_too_long,width,height;
 long longest,indx,cnt,num_strs,ncols,nrows,total_chs;
 Rect *rr;

 *menu = xv_create(0,MENU,NULL);
 if (is_release) xv_set(*menu,MENU_RELEASE,NULL);
 err = NO_ERROR;
 if (ptr->num_strs > 0)
    {num_strs = ptr->num_strs;
     PRT3(printf("WIN_SCREEN_RECT...num_strs = %d\n",ptr->num_strs);)
     rr = (Rect *) xv_get(win,WIN_SCREEN_RECT);
     longest = longest_str_len(ptr,&indx);
     if (menu_title != NULL)
        {cnt = get_strlen(menu_title);
         if (longest < cnt) longest = cnt;}

     largest_ch_dim(win,&width,&height);
     PRT3(printf("w = %d...h = %d long = %d w = %d h = %d\n",
          rr->r_width,rr->r_height,longest,width,height);)

     total_chs = rr->r_width / (width + 1);
     nrows = rr->r_height / (height + 10);
     if (longest > 40) 
        {is_too_long = 1;
         longest = 40;}
       else is_too_long = 0;

     ncols = total_chs / (longest + 2);
     if (ncols * nrows < ptr->num_strs)
        ptr->num_strs = ncols * nrows;
     PRT3(printf("ncols = %d nrows = %d total_chs = %d\n",
                ncols,nrows,total_chs);)
     PRT3(printf("ncols * nrows = %d\n",ncols * nrows);)

     for (cnt = 0; cnt < ptr->num_strs; cnt++)
        {if ((err = cp_str(&str,ptr->strs[cnt])) != NO_ERROR)
            break;
         if (is_too_long && (get_strlen(str) > 40)) str[40] = '\0';

         xv_set(*menu,MENU_ITEM,
                  MENU_STRING,str,MENU_VALUE,cnt+1,
                  MENU_RELEASE_IMAGE,
                  NULL,
             NULL);
        }
     if (ptr->num_strs > nrows)
        {if (ptr->num_strs % nrows) ncols = ptr->num_strs / nrows + 1;
           else ncols = ptr->num_strs / nrows;
         xv_set(*menu,MENU_NROWS,nrows,MENU_NCOLS,ncols,0);
        }

     ptr->num_strs = num_strs;
     if (menu_title != NULL) xv_set(*menu,MENU_TITLE_ITEM,menu_title,0);
     if (ptr->cur_str < ptr->num_strs)
        {PRT3(printf("ptr->cur_str = %d\n",ptr->cur_str))
         if (menu_title != NULL)
            xv_set(*menu,MENU_DEFAULT,ptr->cur_str + 2,0);
           else xv_set(*menu,MENU_DEFAULT,ptr->cur_str + 1,0);
        }

     xv_set(*menu,MENU_NOTIFY_PROC,action_proc,
                  MENU_CLIENT_DATA,menu_client_data,NULL);}
 return err;
}


/*--------------------------------------------------------------------------
menu_show_it - ???
--------------------------------------------------------------------------*/

void
menu_show_it(menu,which)
 Menu menu;
 int which;
{
 Xv_Window win;
 Event *event;

 (void) notify_set_itimer_func((Notify_client) menu,
            NOTIFY_FUNC_NULL,ITIMER_REAL,
             (struct itimerval *) 0,(struct itimerval *) 0);
 win = (Xv_Window) xv_get(menu,XV_KEY_DATA,10);
 event = (Event *) xv_get(menu,XV_KEY_DATA,11);
 PRT3(
      printf("menu = %u nrows = %d  ncols = %d nitems = %d\n",menu,
           (int) xv_get(menu,MENU_NROWS),(int) xv_get(menu,MENU_NCOLS),
          (int) xv_get(menu,MENU_NITEMS));
     )
 menu_show(menu,win,event,NULL);
 PRT3(printf("menu_show_it...end\n"))
}


/*--------------------------------------------------------------------------
SetMenuPtr - ???
--------------------------------------------------------------------------*/

void
SetMenuPtr(menu,is_setup)
 Menu menu;
 int is_setup;
{
 static Menu last_menu = 0;
 struct user_menu_rec *user;

 if (is_setup)
    {if (last_menu != 0)
        {user = (struct user_menu_rec *) xv_get(last_menu,MENU_CLIENT_DATA);
         PRT2(printf("SetMenuPtr user = %u\n",user))
         if (user != NULL) free_user_menu_rec(&user);
           else xv_destroy(last_menu);
        }         
     last_menu = menu;}
   else if (menu != 0)
    if (last_menu == menu)
       {xv_destroy(last_menu);
        last_menu = 0;}
}


/*--------------------------------------------------------------------------
ShowMenuStrsProc - ???
--------------------------------------------------------------------------*/

int
ShowMenuStrsProc(win,event,menu_title,
   strs_rec_ptr,free_str_status,proc,arg1,is_return_proc_str_arg,arg2,arg3)
 Xv_Window win;
 Event *event;
 char *menu_title;
 struct strs_rec *strs_rec_ptr;
 int free_str_status;
 int (*proc)();
 void *arg1;
 int is_return_proc_str_arg;
 void *arg2,*arg3;
{
 Menu menu;
 int err;
 struct user_menu_rec *user;
 Rect *rr;
 static Event last_event;

 PRT3(printf("\n\nShowMenuStrsProc\n");)
 SetupTheMenu(&menu,menu_title,1,win,strs_rec_ptr,user_menu_notify_proc,0);
 if (menu == 0)
    {
     return CANCEL;}
 PRT3(printf("ShowMenuStrsProc...num_strs = %d\n",strs_rec_ptr->num_strs);)
 if ((err = SetupUseMenuRec(&user,menu,1,menu_title,
      strs_rec_ptr,free_str_status,
      user_menu_notify_proc,proc,arg1,arg2,arg3,
        NULL,is_return_proc_str_arg)) != NO_ERROR)
    {xv_destroy(menu);
     return err;}

 SetMenuPtr(menu,1);
 PRT3(printf("ShowMenuStrsProc...1\n");)
 xv_set(menu,MENU_CLIENT_DATA,user,MENU_DONE_PROC,user_menu_done_proc,0);
 if (user->menu_title != NULL) xv_set(menu,MENU_TITLE_ITEM,user->menu_title,0);

 xv_set(menu,XV_KEY_DATA,10,win,0);
 xv_set(menu,XV_KEY_DATA,11,&last_event,0);
 if (event == NULL)
    {event_set_window(&last_event,win);
     if ((rr = (Rect *) xv_get(win,WIN_MOUSE_XY)) != NULL)
        {event_set_x(&last_event,rr->r_left);
         event_set_y(&last_event,rr->r_top);
        }
    }     
   else last_event = *event;

 PRT3(printf("ShowMenuStrsProc...menu = %u\n",menu);)

 (void) notify_set_itimer_func((Notify_client) menu,
             (Notify_func) menu_show_it,ITIMER_REAL,
             &NOTIFY_POLLING_ITIMER,(struct itimerval *) 0);
 PRT3(printf("ShowMenuStrsProc...end\n");)

 return NO_ERROR;
}


/*--------------------------------------------------------------------------
make_menu_items_inactive - ???
--------------------------------------------------------------------------*/

void
make_menu_items_inactive(menu)
 Menu menu;
{
 Menu_item item;
 int lcv;

 for (lcv = 1; (item = xv_get(menu,MENU_NTH_ITEM,lcv)) != 0; lcv++)
     xv_set(item,MENU_INACTIVE,TRUE,0);
}


/*--------------------------------------------------------------------------
make_menu_items_active - ???
--------------------------------------------------------------------------*/

void
make_menu_items_active(menu)
 Menu menu;
{
 Menu_item item;
 int lcv;

 for (lcv = 1; (item = xv_get(menu,MENU_NTH_ITEM,lcv)) != 0; lcv++)
     xv_set(item,MENU_INACTIVE,FALSE,0);
}


/*--------------------------------------------------------------------------
invert_menu_items - ???
--------------------------------------------------------------------------*/

void
invert_menu_items(menu,yes)
 Menu menu;
 int yes;
{
 Menu_item item;
 int lcv;

 for (lcv = 1; (item = xv_get(menu,MENU_NTH_ITEM,lcv)) != 0; lcv++)
    if (yes) xv_set(item,MENU_INVERT,TRUE,0);
      else xv_set(item,MENU_INVERT,FALSE,0);
}


/*--------------------------------------------------------------------------
box_menu_items - ???
--------------------------------------------------------------------------*/

void
box_menu_items(menu,yes)
 Menu menu;
 int yes;
{
 Menu_item item;
 int lcv;


 for (lcv = 1; (item = xv_get(menu,MENU_NTH_ITEM,lcv)) != 0; lcv++)
    if (yes) xv_set(item,MENU_INVERT,TRUE,0);
      else xv_set(item,MENU_INVERT,FALSE,0);
}


/*--------------------------------------------------------------------------
free_user_on_error - ???
--------------------------------------------------------------------------*/

void
free_user_on_error(user)
 struct user_menu_rec *user;
{
 if (user != NULL)
    {user->menu = 0;
     user->strs_rec_ptr = NULL;
     free_a_user_menu_rec(&user);}
}



/*--------------------------------------------------------------------------
free_dir_fns_on_error - ???
--------------------------------------------------------------------------*/

void
free_dir_fns_on_error(user,user1,user2,user3,user4,father_ptr,brother_ptr,son_ptr,
                      dirs,menu,menu1,menu2,menu3,menu4)
 struct user_menu_rec *user,*user1,*user2,*user3;
 struct strs_rec *father_ptr,*brother_ptr,*son_ptr;
 Menu menu,menu1,menu2,menu3,menu4;
{
 if (father_ptr != NULL) free_strs_rec(father_ptr);
 if (brother_ptr != NULL) free_strs_rec(brother_ptr);
 if (son_ptr != NULL) free_strs_rec(son_ptr);
 if (dirs  != 0) free_strs_rec(dirs);
 if (menu  != 0) xv_destroy_safe(menu);
 if (menu1 != 0) xv_destroy_safe(menu1);
 if (menu2 != 0) xv_destroy_safe(menu2);
 if (menu3 != 0) xv_destroy_safe(menu3);
 if (menu4 != 0) xv_destroy_safe(menu4);
 free_user_on_error(user);
 free_user_on_error(user1);
 free_user_on_error(user2);
 free_user_on_error(user3);
 free_user_on_error(user4);
}


/*--------------------------------------------------------------------------
show_menu_dir_fns_proc - ???
--------------------------------------------------------------------------*/

void
show_menu_dir_fns_proc(menu,item)
 Menu menu;
 Menu_item item;
{
 char *fn,*dir;
 Menu root;
 void *arg;
 int (*proc)();

 dir = (char *) xv_get(menu,MENU_CLIENT_DATA);
 fn = (char *) xv_get(item,MENU_STRING);

 TRACE(printf("show_menu_dir_fns_proc...dir = %s  str = %s\n",dir,fn))
 root = (Menu) root_menu(menu);
 proc = (int (*)()) xv_get(root,XV_KEY_DATA,1);
 arg = (void *) xv_get(root,XV_KEY_DATA,2);
 TRACE(printf("show_menu_dir_fns_proc...proc = %u  arg = %u\n",proc,arg))
 if (proc != NULL) proc(arg,dir,fn);
 TRACE(printf("show_menu_dir_fns_proc...end proc = %u  arg = %u\n",proc,arg))
}


/*--------------------------------------------------------------------------
show_menu_dir_fns_done_proc - ???
--------------------------------------------------------------------------*/

void
show_menu_dir_fns_done_proc(menu,result)
 Menu menu;
 Xv_opaque result;
{
 char *str;

 TRACE(printf("show_menu_dir_fns_done_proc...\n"))
 if ((str = (void *) xv_get(menu,XV_KEY_DATA,3)) != NULL)
    c_free(str);
 if ((str = (void *) xv_get(menu,XV_KEY_DATA,4)) != NULL) 
    c_free(str);
}


/*--------------------------------------------------------------------------
show_menu_dir_fns - ???
--------------------------------------------------------------------------*/

int
show_menu_dir_fns(win,event,dir_fn,dirs,action,arg)
 Xv_Window win;
 Event *event;
 char *dir_fn;
 struct strs_rec *dirs;
 int (*action)();
 void *arg;
{
 Menu menu,menu1,menu2,menu3;
 struct strs_rec *brother_ptr,*son_ptr;
 int err;
 char *fatherdir,*temp,*curdir;

 fatherdir = curdir = NULL;
 brother_ptr = son_ptr = NULL;
 TRACE(printf("show_menu_dir_fns....1\n"))
 if (((err = father_dir(dir_fn,&fatherdir)) != NO_ERROR) ||
     ((err = brother_dirs(dir_fn,"*",&brother_ptr)) != NO_ERROR) ||
     ((err = son_dirs(dir_fn,"*",&son_ptr)) != NO_ERROR) ||
     ((err = cp_str(&curdir,dir_fn)) != NO_ERROR))
    {if (fatherdir != NULL) c_free(fatherdir);
     if (brother_ptr != NULL)
        {free_strs_rec(brother_ptr);
         c_free(brother_ptr);}
     if (son_ptr != NULL)
        {free_strs_rec(son_ptr);
         c_free(son_ptr);}
     if (curdir != NULL) c_free(curdir);
     TRACE(printf("show_menu_dir_fns....err = %d\n",err))
     return err;}

 menu = 0;
 setup_menu_items(&menu,NULL,2,fatherdir,
               show_menu_dir_fns_proc,NULL,0);
 TRACE(printf("show_menu_dir_fns fatherdir = %s\n",fatherdir))

 TRACE(printf("brother_ptr...= %d\n");pr_strs_rec(brother_ptr))
 SetupTheMenu(&menu1,NULL,1,win,brother_ptr,show_menu_dir_fns_proc,fatherdir);
 if (menu1 != 0)
    {concat_strs(&temp,3,fatherdir,"/","*");
     setup_menu_items(&menu,NULL,1,temp,menu1,
               NULL,NULL,0);
     c_free(temp);
    }
 TRACE(printf("son_ptr...\n");pr_strs_rec(son_ptr))
 SetupTheMenu(&menu2,NULL,1,win,son_ptr,show_menu_dir_fns_proc,curdir);
 if (menu2 != 0)
    {concat_strs(&temp,3,curdir,"/","*");
     setup_menu_items(&menu,NULL,1,temp,menu2,
               NULL,NULL,0);
     c_free(temp);
    }
 
 if (dirs != NULL)
    {SetupTheMenu(&menu3,NULL,1,win,dirs,show_menu_dir_fns_proc,NULL);
     if (menu3 != 0)
        setup_menu_items(&menu,NULL,
            1,"DIRECTORIES BROWSED",menu3, NULL,NULL,0);
    }
 TRACE(printf("***action = %u  arg = %u\n",action,arg))
 xv_set(menu,XV_KEY_DATA,1,action,XV_KEY_DATA,2,arg,
             XV_KEY_DATA,3,fatherdir,XV_KEY_DATA,4,curdir,
             MENU_DONE_PROC,show_menu_dir_fns_done_proc,0);
 free_strs_rec(brother_ptr);
 c_free(brother_ptr);
 free_strs_rec(son_ptr);
 c_free(son_ptr);
 menu_show(menu,win,event,0);
 TRACE(printf("show_dir_fns...end\n"))
 return NO_ERROR;
}
