/****************************************************************************
mri_file_info.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: mri_file_info.c,v 1.7 1999/07/07 17:41:18 hughett Exp $ Release $Name: nria_1_0_6 $";

#include "mri_format.h"
#include "nria_procedures.h"


/* Functions defined in this file */
int file_info(struct input_file_rec *file, char **fn);


/*--------------------------------------------------------------------------
file_info - ???
--------------------------------------------------------------------------*/

int
file_info(
    struct input_file_rec *file,
    char **fn)
{
 float pixel_thickness,voxel_size;
 FILE *fid;
 char *str,*hdr_fn,*dat_fn;
 struct strs_rec roi_labels;
 unsigned int lcv,histo_dimx,histo_dimy,pixel_depth;
 int err,define;
 size_t amt;

 *fn = NULL;
 if ((err = cp_str(fn,tmpnam(NULL))) != NO_ERROR) return err;
 if ((fid = fopen(*fn,"w")) == NULL)
    {c_free(*fn);
     *fn = NULL;
     return FILE_ACCESS_ERROR;}

 str = hdr_fn = dat_fn = NULL;
 if (((err = build_fn(&str,file->dirfn,file->fn,
                     file->hdr_fn_suffix)) != NO_ERROR) ||
     ((err = concat_strs(&hdr_fn,3,str,"-",file->img_fn_suffix)) != NO_ERROR) ||
     ((err = build_fn(&dat_fn,file->output.dirfn,file->output.fn,
                     file->output.dat_fn_suffix)) != NO_ERROR))
    {c_free(str);
     c_free(hdr_fn);
     c_free(dat_fn);
     return CANCEL;}

 fprintf(fid,"\n******** FILE STATUS ********\n");
 fprintf(fid,".hdr & .img Files = %s\n",hdr_fn);
 fprintf(fid,"  .hdr Version = %s\n",file->hdr.id);
 fprintf(fid,"  Last edit of the .hdr & .img files began at = %s",
              file->hdr.extra_info.last_edit_begin);
 fprintf(fid,"                                     ended at = %s\n",
              file->hdr.extra_info.last_edit_end);

 fprintf(fid,".dat file = %s",dat_fn);

 check_file_save(&(file->output));

 if (!(file->output.status & FILE_READY))
    fprintf(fid," not saved !!!\n");
   else
    {if ((file->output.status & FILE_TO_BE_SAVED) && 
            (file->output.status & FILE_CHANGED))
        fprintf(fid," to be saved !!!\n");
       else fprintf(fid," saved !!!\n");
     fprintf(fid,"  .dat Version = %s\n",file->output.hdr.id);}
 c_free(str);
 c_free(dat_fn);
 c_free(hdr_fn);


 fprintf(fid,"\n\n******** IMAGE DATA INFO. ********\n");
 fprintf(fid,"Date of study = %s    Time = %s\n",
       file->hdr.data_info.date_of_study,file->hdr.data_info.time_of_study);

 pixel_thickness = file->hdr.data_info.pixel_thickness;
 fprintf(fid,"Data pixel size in bits = %d\n",
         (int) file->hdr.data_info.pixel_depth_bits);
 fprintf(fid,"   Dimension size = %d * %d\n",
         (int) file->hdr.data_info.dimx,
         (int) file->hdr.data_info.dimy);

 voxel_size = pixel_thickness * file->hdr.data_info.xpixel_size * 
              file->hdr.data_info.ypixel_size;
 fprintf(fid,"Pixel size (mm) X = %-.4f Y = %-.4f    Voxel size = %-.4f\n",
         file->hdr.data_info.xpixel_size,
         file->hdr.data_info.ypixel_size,voxel_size);

 fprintf(fid,"Pixel thickness (includes the spacing) = %-.4f   Spacing = %-.4f \n",pixel_thickness,file->hdr.data_info.spacing);
 fprintf(fid,"Min pixel = %f  Max pixel = %f\n",
        file->hdr.data_info.min_pixel,file->hdr.data_info.max_pixel);
 fprintf(fid,"Plane type = %s\n",
        get_plane_type(file->hdr.data_info.plane_type));

 fprintf(fid,"# of image echoes = %d\n",(int) file->hdr.data_info.num_echoes);

 fprintf(fid,"Repetition/Recovery time = %f   ",file->hdr.data_info.tr);
 fprintf(fid,"echo delay = %s\n",file->hdr.data_info.te);

 fprintf(fid,"Refered by = %s\n",file->hdr.data_info.refered_by);
 fprintf(fid,"Description = %s\n",file->hdr.data_info.description);
 fprintf(fid,"Hospital name = %s\n",file->hdr.data_info.hosp_name);

 fprintf(fid,"\n\n******** PATIENT INFO. ********\n");
 fprintf(fid,"Name = %s\n",file->hdr.data_info.patient.name);
 fprintf(fid,"SS#  = %s   ",file->hdr.data_info.patient.ssnum);
 fprintf(fid,"Medical record# = %s\n",file->hdr.data_info.patient.patient_id);
 fprintf(fid,"Study# = %s   ",file->hdr.data_info.patient.study_num);
 fprintf(fid,"Patient# = %s\n",file->hdr.data_info.patient.patient_num);
 fprintf(fid,"Sex = %c   ",file->hdr.data_info.patient.sex);
 fprintf(fid,"Age = %d\n",file->hdr.data_info.patient.age);
 fprintf(fid,"Height = %d   ",file->hdr.data_info.patient.height);
 fprintf(fid,"Weight = %d\n",file->hdr.data_info.patient.weight);
 fprintf(fid,"\n");

 fprintf(fid,"# of Slice Cuts = %ld  Scans/slice = %ld Total scans = %d\n",
           NumberOfSliceCuts(&(file->hdr.data_info)),
           NumberOfSliceScans(&(file->hdr.data_info)),
          (int) file->hdr.data_info.max_scans);
 for (lcv = 0; lcv < file->hdr.analysis.num_img_labels; lcv++)
    fprintf(fid,"     Image slices of type %s = %ld\n",
         file->hdr.analysis.img_labels[lcv],
         NumberOfSliceCuts(&(file->hdr.data_info)));

 fprintf(fid,"\n\n******** ANALYSIS ********\n");
 
 if ((file->hdr.status & _1_ECHO_IMAGE) && 
     (file->histo_params.status2 & HISTO_1D))
    {
     get_histo_dim(&(file->histo_params),
                  &histo_dimx,&histo_dimy,&pixel_depth,HISTO_1D);
     fprintf(fid,"1) Histogram 1d is defined.\n");
     fprintf(fid,"   Histogram dimension...x = %d   y = %d\n",
     histo_dimx,histo_dimy);}
   else if ((file->hdr.data_info.num_echoes == 2) &&
            (file->histo_params.status2 & HISTO_MULTI_D))
    {
     get_histo_dim(&(file->histo_params),
                  &histo_dimx,&histo_dimy,&pixel_depth,HISTO_MULTI_D);
     fprintf(fid,"1) Histogram 2d is defined.\n");
     fprintf(fid,"   Histogram dimensions = %d * %d\n",histo_dimx,
     histo_dimy);}
   else fprintf(fid,"1) No histogram is defined yet.\n");

 if ((define = is_all_pixel_type_map_ready(file)) == 2)
    {fprintf(fid,"2) Pixel types are defined.\n");
     if (file->pixel_type.status & HISTO_AND_PIXEL_TYPE_RELATED)
        fprintf(fid,"     They are defined based on the current histogram.\n");
       else fprintf(fid,"     They are not defined based on the current histogram.\n");
     fprintf(fid,"   Pixel type map dimensions = %ld * %ld\n",
        file->pixel_type.map.dimx,file->pixel_type.map.dimy);
    }
   else if (define == 1)
    fprintf(fid,"2) Pixel types are only defined for some slices.\n");
   else fprintf(fid,"2) Pixel types are not defined for all slices.\n");

 if ((file->pixel_type.status & PIXEL_TYPE_ALL_ROIS_COMPUTED) ||
     (file->pixel_type.status & PIXEL_TYPE_COMPUTED))
    {if (file->pixel_type.status & PIXEL_TYPE_ALL_ROIS_COMPUTED)
        fprintf(fid,"3) Pixel type counts are defined for all rois of all slices.\n");
       else
        fprintf(fid,"3) Pixel type counts are defined for all rois of some slices.\n");
     if (file->pixel_type.status & PIXEL_TYPE_AND_ANALYSIS_RELATED)
        fprintf(fid,"     They are defined based on the current pixel types.\n");
       else fprintf(fid,"     They are not defined based on the current pixel types.\n");
     fprintf(fid,"      Pixel Type Names\n");
     for (lcv = 0; lcv < file->pixel_type.order.num_strs; lcv++)
        fprintf(fid,"      %-d) %s\n",lcv,file->pixel_type.order.strs[lcv]);
    }
   else fprintf(fid,"3) Pixel type counts have not yet been defined.\n");


 fprintf(fid,"\n\n******** CURRENTLY DEFINED ROIS ********\n");
 FileGetAllRoiLabelPtrs(file,&roi_labels,0,1);
 for (lcv = 0; lcv < roi_labels.num_strs; lcv++)
    fprintf(fid,"      %-d) %s\n",lcv,roi_labels.strs[lcv]);
 free_strs_rec_ptrs(&roi_labels);

 if (file->hdr.comment_size == 0)
    fprintf(fid,"\n\n******** NO USER-ENTERED COMMENT ********\n");
   else
    {fprintf(fid,"\n\n******** USER-ENTERED COMMENT ********\n");
     fwr(file->hdr.comment,(size_t) file->hdr.comment_size,
           (size_t) 1,fid,&amt);}

 fclose(fid);
 return NO_ERROR;
}
