/****************************************************************************
process_key.c - Convert keystrokes into the right procedure calls

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.

FIXME: Many of the functions, usually ending in _proc, referred to in
the tables here should return void but have been miswritten to return
int instead.  They should be fixed as they are discovered, by checking
all the other uses of the offending routines and then rewriting them
to return void.
****************************************************************************/

static const char rcsid[] = "$Id: process_key.c,v 1.13 1999/07/07 17:41:18 hughett Exp $ Release $Name: nria_1_0_6 $";


/* FIXME:  Not needed?
#include <time.h>
#include <sys/timeb.h>
*/

#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/notify.h>

#include "mri_format.h"
#include "nria_procedures.h"

/*
  #define TRACE(x) x; fflush(stdout);
  #define PRT(x) x; fflush(stdout);
  #define PRT1(x) x; fflush(stdout);
  #define PRT2(x) x; fflush(stdout);
  #define PRT3(x) x; fflush(stdout);
*/
#define PRT(x)
#define TRACE(x)
#define PRT1(x)
#define PRT2(x)
#define PRT3(x)


struct key_proc_rec
{
  int key;
  int (*proc)();
  unsigned short win_type;
};

struct key_value_rec
{
  int key;
  int value;
  unsigned short win_type;
};

struct key_proc_value_rec
{
  int key;
  void (*proc)();
  int value;
  unsigned short win_type;
  unsigned char more;
};

struct key_pressed_rec
{
  int key;
  int value;
  unsigned long cnt;
};


/* Functions defined in this file */
int process_key_top(Xv_Window canvas, Event *event);
int process_right_key(Xv_Window canvas, Event *event);
int process_ascii_key(Xv_Window canvas, Event *event);
int process_control_key(Xv_Window canvas, Event *event);
int process_action_key(Xv_Window canvas, Event *event);
int backspace_pts(Xv_Window canvas);
void key_pressed(Xv_Window canvas_win, struct key_pressed_rec key_pressed[],
    int num_key_pressed);
int process_ascii_event(Xv_Window canvas, Event *event);



/*-------------------------------------------------------------------------
process_key_top - ???
-------------------------------------------------------------------------*/

int
process_key_top(canvas,event)
     Xv_Window canvas;
     Event *event;
{
  int lcv,value;
  static struct key_proc_value_rec ctrl_shift_actions[] = 
  {
    {KEY_TOP(1),delete_roi_proc,DELETE_CUR_CURVE,EDITING_WINDOW,0},
    {KEY_TOP(2),delete_roi_proc,DELETE_CUR_ROI,EDITING_WINDOW,0},
    {KEY_TOP(3),delete_roi_proc,DELETE_EVERY_ROI,EDITING_WINDOW,0},
    {KEY_TOP(4),delete_roi_proc,DELETE_ALL_FILE_ROIS,EDITING_WINDOW,0},
    {KEY_TOP(6),win_exit_cur_win_proc,0,ALL_WINDOWS,0},
    {KEY_TOP(7),win_exit_proc,0,ALL_WINDOWS,0},
    {KEY_TOP(8),win_stack_exit_proc,0,ALL_WINDOWS,0},
    {KEY_TOP(9),color_editor_proc,0,ALL_WINDOWS,0},
    {KEY_TOP(10),about_ria_proc,0,EDITING_WINDOW,0},
    {KEY_TOP(11),beginner_help_proc,0,EDITING_WINDOW,0},
    {KEY_TOP(12),win_rd_breaker_check,0,EDITING_WINDOW,0}
  };
  static int num_ctrl_shift_actions = 11;

  static struct key_proc_value_rec actions[] = 
  {
    {KEY_TOP(1),select_slice_proc,0,MANY_SLICES_WINDOWS,0},
    {KEY_TOP(2),activate_pen_proc,0,EDITING_WINDOW,0},
    {KEY_TOP(3),show_pixel_type_roi_proc,0,ALL_WINDOWS,0},
    {KEY_TOP(4),key_activated_show_roi_proc,0,EDITING_WINDOW,0},
    {KEY_TOP(5),highlight_next_curve_proc,0,EDITING_WINDOW,0},
    {KEY_TOP(6),all_wins_cur_roi_proc,0,EDITING_WINDOW,0},
    {KEY_TOP(7),highlight_next_roi_proc,0,EDITING_WINDOW,0},
    {KEY_TOP(8),edit_next_empty_curve_proc,0,EDITING_WINDOW,0},
    {KEY_TOP(9),show_histogram_menu,0,DISPLAY_WINDOW,1},
    {KEY_TOP(9),_3d_pt_cursor_proc,0,DISPLAY_WINDOW_3D,0},
    {KEY_TOP(10),segment_pixel_type_proc,0,EDIT_NO_3D_WINDOW,1},
    {KEY_TOP(10),_3d_form_plane_by_3_pts_proc,0,DISPLAY_WINDOW_3D,0},
    {KEY_TOP(11),_3d_plane_key_proc,0,DISPLAY_WINDOW_3D,1},
    {KEY_TOP(11),delete_the_grown_roi_proc,0,EDITING_WINDOW,0},
    {KEY_TOP(12),segment_cur_slice_proc,0,EDIT_NO_3D_WINDOW,1},
    {KEY_TOP(12),_3d_cur_wid_roi_proc,0,DISPLAY_WINDOW_3D,0},
  };
  static int num_actions = 16;

  static struct key_proc_value_rec ctrl_actions[] = 
  {
    {KEY_TOP(1),color_pallette_proc,0,ALL_WINDOWS,0},
    {KEY_TOP(2),image_color_scale_proc,0,ALL_WINDOWS,0},
    {KEY_TOP(3),image_color_scale_proc,1,ALL_WINDOWS,0},
    
    {KEY_TOP(4),enable_global_local_contrast_proc,0,ALL_WINDOWS,0},
    {KEY_TOP(5),histogram_roi_proc,VALUE_HISTO_CURRENT_SLICE_1D,
     EDIT_NO_3D_WINDOW,0},
    {KEY_TOP(6),histogram_roi_proc,VALUE_HISTO_ALL_SLICE_1D,
     EDIT_NO_3D_WINDOW,0},
    {KEY_TOP(7),histogram_roi_proc,VALUE_HISTO_CURRENT_SLICE,
     EDIT_NO_3D_WINDOW,0},
    {KEY_TOP(8),histogram_roi_proc,
     VALUE_HISTO_ALL_SLICE,EDIT_NO_3D_WINDOW,0},
    {KEY_TOP(9),histogram_options_proc,0,ALL_WINDOWS,0},
    {KEY_TOP(10),text_output_proc,VALUE_SAVE_THEN_PRINT_TEXT,
     EDIT_NO_3D_WINDOW,0},
    {KEY_TOP(11),_3d_enable_locate_plane_proc,0,DISPLAY_WINDOW_3D,0},
    {KEY_TOP(12),_3d_reproject_proc,0,DISPLAY_WINDOW_3D,0},
  };
  static int num_ctrl_actions = 12;

  static struct key_proc_value_rec shift_actions[] = 
  {
    {KEY_TOP(1),show_roi_menu_proc,0,ALL_WINDOWS,0},
    {KEY_TOP(2),magnify_pixel_proc,0,ALL_WINDOWS,0},
    {KEY_TOP(5),highlight_next_curve_proc,1,EDITING_WINDOW,0},
    {KEY_TOP(7),highlight_next_roi_proc,1,EDITING_WINDOW,0},
    {KEY_TOP(9),slice_rect_proc,0,EDIT_NO_3D_WINDOW,1},
    {KEY_TOP(9),_3d_delete_last_cursor_pt_proc,0,DISPLAY_WINDOW_3D,0},
    {KEY_TOP(10),dp_grow_region_params_proc,0,EDITING_WINDOW,0},
    {KEY_TOP(12),output_text_format_proc,0,EDIT_NO_3D_WINDOW,0},
  };
  static int num_shift_actions = 8;

  static struct key_proc_value_rec meta_actions[] = 
  {
    {KEY_TOP(1),dp_trans_roi_proc,0,EDIT_NO_3D_WINDOW,0},
    {KEY_TOP(2),trans_roi_mode_proc,
     ACTION_ON_ALL_SLICES,EDIT_NO_3D_WINDOW,0},
    {KEY_TOP(3),trans_roi_mode_proc,ACTION_ON_ALL_ROIS,EDIT_NO_3D_WINDOW,0},
    {KEY_TOP(4),trans_roi_mode_proc,ACTION_ON_CUR_ROI,EDIT_NO_3D_WINDOW,0},
    {KEY_TOP(5),trans_roi_mode_proc,0,EDIT_NO_3D_WINDOW,0},
    {KEY_TOP(6),trans_roi_mode_proc,VALUE_SLICE_ROI_PRIOR,EDIT_NO_3D_WINDOW,0},
    {KEY_TOP(7),trans_roi_mode_proc,VALUE_SLICE_ROI_NEXT,EDIT_NO_3D_WINDOW,0},
    {KEY_TOP(8),dp_more_rois_proc,0,EDITING_WINDOW,0},
    {KEY_TOP(9),_3d_enable_dp_xyz_proc,0,DISPLAY_WINDOW_3D,0},
    {KEY_TOP(10),clipboard_proc,0,EDIT_NO_3D_WINDOW,0},
    {KEY_TOP(11),clipboard_it_proc,VALUE_CLIPBOARD_FIRST,
     EDIT_NO_3D_WINDOW,0},
    {KEY_TOP(12),import_image_file_proc,0,ALL_WINDOWS,0},
  };
  static int num_meta_actions = 12;

  static struct key_proc_value_rec meta_ctrl_actions[] = 
  {
    {KEY_TOP(1),change_view_roi_proc,-1,EDITING_WINDOW,0},
    {KEY_TOP(2),change_view_roi_proc,-2,EDITING_WINDOW,0}, 
    {KEY_TOP(3),change_view_roi_proc,-3,EDITING_WINDOW,0},
    {KEY_TOP(4),change_view_roi_proc,-4,EDITING_WINDOW,0},
    {KEY_TOP(5),change_view_roi_proc,-5,EDITING_WINDOW,0},
    {KEY_TOP(6),change_view_roi_proc,-6,EDITING_WINDOW,0},
    {KEY_TOP(7),change_view_roi_proc,-7,EDITING_WINDOW,0},
    {KEY_TOP(8),change_view_roi_proc,-8,EDITING_WINDOW,0},
    {KEY_TOP(9),change_view_roi_proc,-9,EDITING_WINDOW,0},
  };
  static int num_meta_ctrl_actions = 9;

  struct display_win_rec *win1;
  struct image_display_window *win;
  int val;


  win1 = (struct display_win_rec *) xv_get(canvas,WIN_CLIENT_DATA);
  win = win1->owner;
  win->dwin = win1;
  value = event_action(event);
  PRT3(printf("key_top \n"););

  if (event_ctrl_is_down(event) && event_meta_is_down(event))
  {

    /*
     *  CTRL-META actions
     */

    PRT3(printf("event_ctrl_is_down  event_meta_is_down\n"););
    for (lcv = 0; lcv < num_meta_ctrl_actions; lcv++)
    {
      if (meta_ctrl_actions[lcv].key == value)
      {
	if (win->type2 & meta_ctrl_actions[lcv].win_type)
	{
	  if (event_shift_is_down(event))
	    val = -meta_ctrl_actions[lcv].value;
	  else
	    val = meta_ctrl_actions[lcv].value;
	  meta_ctrl_actions[lcv].proc(NULL,NULL,win1,canvas,event,val);
	  return NO_ERROR;
	}
	else if (!meta_ctrl_actions[lcv].more)
	{
	  win_alarm(win1->frame,2);
	  return NO_ERROR;
	}
      }
    }
  }

  else if (event_shift_is_down(event))
  {

    /*
     *  CTRL-SHIFT actions
     */

    if (event_ctrl_is_down(event))
    {
      value = event_id(event);
      for (lcv = 0; lcv < num_ctrl_shift_actions; lcv++)
	if (ctrl_shift_actions[lcv].key == value)
	  if (win->type2 & ctrl_shift_actions[lcv].win_type)
	  {
	    ctrl_shift_actions[lcv].proc(NULL,NULL,win1,canvas,event,
					 ctrl_shift_actions[lcv].value);
	    init_input_pt_state(&(win->control));
	    return NO_ERROR;
	  }
	  else if (!ctrl_shift_actions[lcv].more)
	  {
	    win_alarm(win1->frame,2);
	    return NO_ERROR;
	  }
    }

    /*
     *  SHIFT actions
     */

    else
    {
      for (lcv = 0; lcv < num_shift_actions; lcv++)
      {
	if (shift_actions[lcv].key == value)
	{
	  if (win->type2 & shift_actions[lcv].win_type)
	  {
	    shift_actions[lcv].proc(NULL,NULL,win1,canvas,event,
				    shift_actions[lcv].value);
	    return NO_ERROR;
	  }
	  else if (!shift_actions[lcv].more)
	  {
	    win_alarm(win1->frame,2);
	    return NO_ERROR;
	  }
	}
      }
    }
  }

  else if (event_ctrl_is_down(event))
  {

    /*
     *  CTRL actions
     */

    for (lcv = 0; lcv < num_ctrl_actions; lcv++)
    {
      if (ctrl_actions[lcv].key == value)
      {
	if (win->type2 & ctrl_actions[lcv].win_type)
	{
	  ctrl_actions[lcv].proc(NULL,NULL,win1,canvas,event,
				 ctrl_actions[lcv].value);
	  return NO_ERROR;
	}
	else if (!ctrl_actions[lcv].more)
	{
	  win_alarm(win1->frame,2);
	  return NO_ERROR;
	}
	return NO_ERROR;	/* FIXME: should this be here? */
      }
    }
  }

  else if (event_meta_is_down(event))
  {

    /*
     *  META actions
     */

    for (lcv = 0; lcv < num_meta_actions; lcv++)
    {
      if (meta_actions[lcv].key == value)
      {
	if (win->type2 & meta_actions[lcv].win_type)
	{
	  meta_actions[lcv].proc(NULL,NULL,win1,canvas,event,
				 meta_actions[lcv].value);
	  return NO_ERROR;
	}
	else			/* FIXME: should there be a check such as
				 * if (!meta_actions[lcv].more)  ? */
	{
	  win_alarm(win1->frame,2);
	  return NO_ERROR;	/* FIXME: should this be here? */
	}
	return NO_ERROR;        /* FIXME:  Never reached? */
      }
    }
  }

  else
  {

    /*
     *  Pure actions (no modifier keys pressed)
     */
    
    for (lcv = 0; lcv < num_actions; lcv++)
    {
      if (actions[lcv].key == value)
      {
	if (win->type2 & actions[lcv].win_type)
	{
	  actions[lcv].proc(NULL,NULL,win1,canvas,event,
			    actions[lcv].value);
	  return NO_ERROR;
	}
	else if (!actions[lcv].more)
	{
	  win_alarm(win1->frame,2);
	  return NO_ERROR;
	}
      }
    }
  }

  win_alarm(win1->frame,2);
  return CANCEL;
}



/*-------------------------------------------------------------------------
process_right_key - ???
-------------------------------------------------------------------------*/

int
process_right_key(canvas,event)
 Xv_Window canvas;
 Event *event;
{
 int value;
 struct display_win_rec *win1;
 struct image_display_window *win;
 static char keys[] = {12,13,14,9,10,11,6,7,8,3,4,5,0,1,2};

 win1 = (struct display_win_rec *) xv_get(canvas,WIN_CLIENT_DATA);
 win = win1->owner;
 value = event_id(event);
 TRACE(printf("process_right_key str = %s event_action = %d event_id(event) = %d\n",
          event_string(event),event_action(event),event_id(event)))
 if ((value >= KEY_RIGHT(1)) && (value  <= KEY_RIGHT(15)))
    value -= KEY_RIGHT(1);
   else
    {value = event_action(event);
     if ((value >= KEY_RIGHT(1)) && (value  <= KEY_RIGHT(15)))
        {value -= KEY_RIGHT(1);}
       else return CANCEL;
    }
 if (event_meta_is_down(event))
    if ((win1->owner->control.which_cursor == CURSOR_MOVE_ROI) ||
        (win1->owner->control.which_cursor == CURSOR_RESIZE_ROI) ||
        (win1->owner->control.which_cursor == CURSOR_ROTATE_ROI) ||
        (win1->owner->control.which_cursor == CURSOR_MOVE_SUPERIMPOSE))
       {
        if ((value <= 5) || (value > 14)) return NO_ERROR;
        return win_transform_by_keypad(win1->owner,event,keys[value] + 1);
       }
 if (win->type2 & EDIT_NO_3D_WINDOW)
    {if ((win->type2 & DISPLAY_WINDOW) && event_meta_is_down(event))
        {if ((value >= 0) && (value <= 14))
            change_channel(win1,keys[value]);
           else change_channel(win1,value);}
       else if ((value >= 9) && (value <= 14))
        {select_cursor_proc(0,0,win1,canvas,event,-((long) keys[value] + 1));
        }
     return NO_ERROR;}
   else if (win->type2 & DISPLAY_WINDOW_3D)
    {if ((value >= 0) && (value <= 14))
        if (keys[value] <= 10)
           {_3d_plane_orientation_proc(0,0,win1,canvas,event,
                      (long) keys[value]);
            return NO_ERROR;}
/******
          else if (keys[value] <= 11)
           {if (keys[value] == 9) mode = CURSOR_ROTATE_VERTICAL;
              else if (keys[value] == 10) mode = CURSOR_ROTATE_HORIZONTAL;
              else if (keys[value] == 11) mode = CURSOR_ROTATE;
            _3d_rotate_proc(NULL,NULL,win1,canvas,event,mode);
            return NO_ERROR;}
******/
    }
 win_alarm(win1->frame,2);
 return CANCEL;
}


/*-------------------------------------------------------------------------
process_ascii_key - ???
-------------------------------------------------------------------------*/

int
process_ascii_key(canvas,event)
 Xv_Window canvas;
 Event *event;
{
 int lcv,value;
 static struct key_value_rec num_keys[] =
         {{')',0,MANY_SLICES_WINDOWS},
          {'!',-1,MANY_SLICES_WINDOWS}, {'@',-2,MANY_SLICES_WINDOWS}, 
          {'#',-3,MANY_SLICES_WINDOWS}, {'$',-4,MANY_SLICES_WINDOWS},
          {'%',-5,MANY_SLICES_WINDOWS}, {'^',-6,MANY_SLICES_WINDOWS},
          {'&',-7,MANY_SLICES_WINDOWS}, {'*',-8,MANY_SLICES_WINDOWS},
          {'(',-9,MANY_SLICES_WINDOWS}, 
         };
 static int num_num_keys = 10;

 static struct key_proc_value_rec shift_key_actions[] = 
         {
          {'A',analyze_all_slices_proc,0,EDIT_NO_3D_WINDOW,1},
          {'A',_3d_same_co_plane_proc,0,DISPLAY_WINDOW_3D,0},
          {'B',breaker_it_proc,VALUE_BREAK_IT,EDIT_NO_3D_WINDOW,1},
          {'C',clipboard_it_proc,VALUE_COPY_APPEND,EDITING_WINDOW,0},
          {'E',project_1_echoe_wins_proc,12,EDIT_NO_3D_WINDOW,0},
          {'F',fusion_image_action_proc,FUSION_SWITCHING,ALL_WINDOWS,0},
          {'G',next_group_slice_proc,-1,EDIT_NO_3D_WINDOW,0},
          {'H',cur_roi_to_histo_roi_proc,0,DISPLAY_WINDOW,0},
          {'I',_3d_co_register_proc,
               COREG_UNCO_ALL_GROUP_WIDS_VALUE,DISPLAY_WINDOW_3D,0},
          {'J',co_register_file_proc,UNCO_WID_FILE_VALUE,DISPLAY_WINDOW,1},

          {'J',_3d_co_register_proc,COREG_ADJUST_VIEW_AND_COPLANES_OFF_VALUE,
                   DISPLAY_WINDOW_3D,0},
          {'K',histo_roi_to_slice_roi_proc,0,EDIT_NO_3D_WINDOW,1},
          {'K',_3d_co_register_proc,COREG_UNCO_ALL_WIDS_VALUE,DISPLAY_WINDOW_3D,0},
          {'L',_3d_co_register_proc,COREG_UNLOCK_ALL_COPLANES_VALUE,
                     DISPLAY_WINDOW_3D,0},
          {'M',show_roi_tissue_proc,WHOLE_SLICE,EDIT_NO_3D_WINDOW,0},
          {'N',text_output_proc,VALUE_SHOW_TISSUE_WINS,EDIT_NO_3D_WINDOW,0},
          {'O',ca_options_proc,8,DISPLAY_WINDOW,1},
          {'O',_3d_co_register_proc,
               COREG_UNLOCK_ALL_GROUP_WIDS_VALUE,DISPLAY_WINDOW_3D,0},
          {'P',enter_3d_param_menu_proc,0,DISPLAY_WINDOW_3D,0},
          {'Q',project_1_echoe_wins_proc,9,DISPLAY_WINDOW,0},

          {'R',more_roi_proc,DISPLAY_FILLED_ROI,EDIT_NO_3D_WINDOW,0},
          {'S',fusion_image_action_proc,FUSION_SHOW,EDITING_WINDOW,0},
          {'T',pixel_types_to_histo_rois_proc,
                     VALUE_CUR_SLICE_TYPES_TO_HISTO_ROIS,EDIT_NO_3D_WINDOW,1},
          {'T',_3d_new_plane_proc,3,DISPLAY_WINDOW_3D,0},
          {'U',set_slice_min_max_proc,1,EDITING_WINDOW,0},
          {'W',all_echo_wins_proc,0,MANY_SLICES_WINDOWS,0},
          {'X',clipboard_it_proc,VALUE_CUT_APPEND,EDITING_WINDOW,0},          
          {'Y',pixel_types_to_histo_rois_proc,
                     VALUE_ALL_SLICE_TYPES_TO_HISTO_ROIS,EDIT_NO_3D_WINDOW,0},
          {'Z',more_roi_proc,DISPLAY_HIDE_SHOW_ALL_ROIS,ALL_WINDOWS,0},
          {' ',select_cursor_proc,0,EDIT_NO_3D_WINDOW,1},

          {' ',_3d_select_cursor_proc,0,DISPLAY_WINDOW_3D,0},
          {'<',show_roi_tissue_proc,CUR_CURVE,EDIT_NO_3D_WINDOW,0},
          {'>',show_roi_tissue_proc,CUR_ROI,EDIT_NO_3D_WINDOW,0},
          {'?',show_roi_tissue_proc,ALL_ROIS,EDIT_NO_3D_WINDOW,0},
          {'{',save_open_cur_plane_proc,2,DISPLAY_WINDOW_3D,0},
          {'}',save_open_cur_plane_proc,3,DISPLAY_WINDOW_3D,0},
          {27,auto_slice_proc,0,MANY_SLICES_WINDOWS,0},
          {':',pen_type_proc,DRAW_FREE,EDIT_NO_3D_WINDOW,0},
          {0,_3d_last_plane_proc,0,DISPLAY_WINDOW_3D,0},
          {'~',enter_3d_resize_proc,0,DISPLAY_WINDOW_3D,1},
          {'|',draw_link_with_wid_proc,1,EDITING_WINDOW,0},
          {'+',resize_proc,RESIZE_DEFAULT_VALUE,IMAGE_WINDOWS,0},
          {9,project_1_echoe_wins_proc,4,EDIT_NO_3D_WINDOW,0},
          {'\"',overlay_proc,DELETE_OVERLAY_VALUE,EDITING_WINDOW,0},
        };
 static int num_shift_key_actions = 44;

 static struct key_proc_value_rec shift_action_key_actions[] = 
         {
          {'A',more_roi_proc,DISPLAY_ROW_COL_LABEL,ALL_WINDOWS,0},
          {'B',more_roi_proc,DISPLAY_ENABLED_LABEL,ALL_WINDOWS,0},
          {'I',file_proc,VALUE_FILE_INFO,EDITING_WINDOW,0},
          {'J',co_register_file_proc,REVERSE_CO_FILE_PARAMS_VALUE,
                   DISPLAY_WINDOW,1},
          {'J',_3d_new_plane_proc,2,DISPLAY_WINDOW_3D,0},
          {'L' - 'A' + 1,_3d_co_register_proc,
              COREG_UNLOCK_ALL_GROUP_WIDS_VALUE,DISPLAY_WINDOW_3D,0},
          {'R',redefine_pixel_type_order_proc,0,EDIT_NO_3D_WINDOW,1},
          {'R',_3d_save_image_order_proc,0,DISPLAY_WINDOW_3D,0},
          {'T',_3d_default_image_order_proc,0,DISPLAY_WINDOW_3D,0},
          {'L',file_proc,VALUE_RELOAD_CUR_SLICE_ROIS,EDIT_NO_3D_WINDOW,0},
          {'Q',file_proc,VALUE_RELOAD_PIXMAP,EDITING_WINDOW,0},
          {'\"',overlay_proc,SHOW_HIDE_OVERLAY_VALUE,EDITING_WINDOW,0},
         };
 static int num_shift_action_key_actions = 12;
/******
 static struct key_proc_value_rec shift_action_key_actions[] = 
         {
          {'G',overlay_proc,DELETE_OVERLAY_VALUE,DISPLAY_WINDOW_3D,0},
          {'H',overlay_proc,OVERLAYED_BY_VALUE,EDITING_WINDOW,0},
         };
 static int num_ctrl_action_key_actions = 2;
******/
 static struct key_proc_value_rec action_key_actions[] = 
         {
          {'B',breaker_options_proc,0,EDIT_NO_3D_WINDOW,1},
          {'G',image_view_filter_proc,0,EDITING_WINDOW,0},
          {'H',image_histogram_proc,0,EDITING_WINDOW,0},
          {'J',enter_co_register_file_proc,0,ALL_WINDOWS,0},
          {'K',_3d_co_register_proc,COREG_RECO_VALUE,DISPLAY_WINDOW_3D,0},
          {'N',text_output_proc,VALUE_FILTER_SHOW_TISSUES,
                    EDIT_NO_3D_WINDOW,0},
          {'M',modify_image_proc,0,EDIT_NO_3D_WINDOW,0},
          {'O',win_select_fns_menu_proc,0,ALL_WINDOWS,0},
          {'Q',program_exit_proc,0,ALL_WINDOWS,0},
          {'R',reverse_images_proc,0,EDIT_NO_3D_WINDOW,1},

          {'R',_3d_reverse_image_order_proc,0,DISPLAY_WINDOW_3D,0},
          {'T',pt_to_region_proc,0,EDIT_NO_3D_WINDOW,1},
          {'T',_3d_cur_default_image_order_proc,0,DISPLAY_WINDOW_3D,0},
          {'Y',image_enhancement_proc,0,EDITING_WINDOW,0},
          {'Z',win_stack_slice_or_scan_proc,0,ALL_WINDOWS,0},
          {'=',resize_image_proc,0,IMAGE_WINDOWS,0},
          {'\'',overlay_proc,OVERLAYED_BY_ALL_ECHO_WID_VALUE,EDITING_WINDOW,0},
          {'\\',dp_draw_list_proc,0,EDITING_WINDOW,0},
          {',',enter_3d_rois_and_pixel_types_proc,0,DISPLAY_WINDOW_3D,0},
          {'/',update_3d_rois_proc,0,DISPLAY_WINDOW_3D,0},
          {0,_3d_last_plane_proc,0,DISPLAY_WINDOW_3D,0},
         };
 static int num_action_key_actions = 21;

 static struct key_proc_value_rec alt_key_actions[] = 
         {
          {'Z',select_slice_order_proc,0,MANY_SLICES_WINDOWS,0},
         };
 static int num_alt_key_actions = 1;

 static struct key_proc_value_rec clipboard_key_actions[] =
         {
          {'+',_3d_plane_orientation_proc,15,DISPLAY_WINDOW_3D,0},
          {'-',_3d_plane_orientation_proc,9,DISPLAY_WINDOW_3D,0},
          {' ',select_cursor_proc,0,EDIT_NO_3D_WINDOW,1},
          {' ',_3d_select_cursor_proc,0,DISPLAY_WINDOW_3D,0},
          {'.',text_output_proc,VALUE_SHOW_ALL_ROIS_PIXEL_TYPE,
                         EDIT_NO_3D_WINDOW,0},
          {',',
             text_output_proc,VALUE_SHOW_ROI_PIXEL_TYPE,EDIT_NO_3D_WINDOW,0},
          {'/',text_output_proc,VALUE_SHOW_SLICE_PIXEL_TYPE,
                    EDIT_NO_3D_WINDOW,0},
          {'A',analyze_cur_slice_proc,0,EDIT_NO_3D_WINDOW,1},
          {'A',_3d_same_co_plane_proc,0,DISPLAY_WINDOW_3D,0},
          {'B',bkg_pixels_proc,0,EDIT_NO_3D_WINDOW,1},
          {'B',_3d_reverse_plane_proc,0,DISPLAY_WINDOW_3D,0},
          {'C',clipboard_it_proc,VALUE_COPY,EDITING_WINDOW,0},
          {'D',fusion_image_action_proc,FUSION_SHOW_SRC,ALL_WINDOWS,0},
          {'E',project_all_echoe_wins_proc,12,EDIT_NO_3D_WINDOW,0},
          {'F',pen_type_proc,DRAW_FREE_CLOSED,EDIT_NO_3D_WINDOW,1},
          {'F',fusion_image_action_proc,FUSION_SWITCHING,DISPLAY_WINDOW_3D,0},
          {'G',next_group_slice_proc,1,EDIT_NO_3D_WINDOW,1},
          {'G',_3d_new_plane_proc,1,DISPLAY_WINDOW_3D,0},
          {'H',histogram_roi_proc,VALUE_HISTO_SHOW,EDIT_NO_3D_WINDOW,1},
          {'H',_3d_new_plane_proc,0,DISPLAY_WINDOW_3D,0},

          {'I',co_register_file_proc,LOAD_CO_FILE_VALUE,EDIT_NO_3D_WINDOW,1},
          {'I',enter_3d_more_plane_proc,0,DISPLAY_WINDOW_3D,0},
          {'J',co_register_file_proc,CO_WID_FILE_VALUE,DISPLAY_WINDOW,1},
          {'J',_3d_co_register_proc,COREG_ADJUST_VIEW_AND_COPLANES_ON_VALUE,
                   DISPLAY_WINDOW_3D,0},
          {'K',histogram_roi_proc,VALUE_HISTO_CLEAR,EDIT_NO_3D_WINDOW,1},
          {'K',_3d_co_register_proc,COREG_CO_VALUE,DISPLAY_WINDOW_3D,0},
          {'L',pen_type_proc,DRAW_LINE,EDIT_NO_3D_WINDOW,1},
          {'L',_3d_co_register_proc,COREG_LOCK_ALL_COPLANES_VALUE,
                 DISPLAY_WINDOW_3D,0},
          {'M',enter_3d_co_menu_proc,0,DISPLAY_WINDOW_3D,1},
          {'M',co_register_file_proc,SAVE_CO_FILE_VALUE,EDIT_NO_3D_WINDOW,0},

          {'N',label_roi_proc,0,EDIT_NO_3D_WINDOW,1},
          {'N',enter_3d_file_menu_proc,0,DISPLAY_WINDOW_3D,0},
          {'O',pen_type_proc,DRAW_CIRCLE,EDIT_NO_3D_WINDOW,0},
          {'P',pen_type_proc,DRAW_POLYGON,EDIT_NO_3D_WINDOW,1},
          {'P',enter_3d_plane_proc,0,DISPLAY_WINDOW_3D,0},
          {'Q',project_all_echoe_wins_proc,9,EDIT_NO_3D_WINDOW,0},
          {'R',win_3d_proc,0,IMAGE_WINDOWS,0},
          {'S',file_proc,VALUE_SAVE_DAT_FILE,EDIT_NO_3D_WINDOW,1},
          {'S',enter_3d_save_menu_proc,0,DISPLAY_WINDOW_3D,0},
          {'T',text_output_proc,VALUE_PRINT_TEXT_TO_WINDOW,
                         EDIT_NO_3D_WINDOW,1},
          {'T',_3d_new_plane_proc,2,DISPLAY_WINDOW_3D,0},
          {'U',image_contrast_proc,0,ALL_WINDOWS,0},
          {'V',clipboard_it_proc,VALUE_PASTE,EDITING_WINDOW,0},
          {'W',get_new_win_proc,0,MANY_SLICES_WINDOWS,0},
          {'X',clipboard_it_proc,VALUE_CUT,EDITING_WINDOW,0},
          {'Y',text_output_proc,VALUE_SAVE_TEXT_TO_FILE,EDIT_NO_3D_WINDOW,0},
          {'Z',more_roi_proc,DISPLAY_HIDE_ALL_ROIS_BUT_CUR,ALL_WINDOWS,0},
          {13,edit_scratch_proc,0,EDITING_WINDOW,0},
          {'[',save_open_cur_plane_proc,0,DISPLAY_WINDOW_3D,1},
          {'[',pen_type_proc,DRAW_RECT,EDIT_NO_3D_WINDOW,0},

          {']',save_open_cur_plane_proc,1,DISPLAY_WINDOW_3D,0},
          {27,magnify_pixel_option_proc,MAGNIFY_LAST_DIM,ALL_WINDOWS,0},
          {9,project_all_echoe_wins_proc,4,EDIT_NO_3D_WINDOW,0},
          {';',pen_type_proc,DRAW_ELLIPSE,EDIT_NO_3D_WINDOW,0},
          {'=',resize_proc,RESIZE_TO_WID_SIZE_VALUE,IMAGE_WINDOWS,0},
          {'`',_3d_resize_group_proc,0,DISPLAY_WINDOW_3D,0},
          {'\'',overlay_proc,OVERLAYED_BY_WID_VALUE,EDITING_WINDOW,0},
          {'\\',draw_link_with_wid_proc,0,EDITING_WINDOW,0},
        };
 static int num_clipboard_key_actions = 58;
 struct display_win_rec *win1;
 struct image_display_window *win;
 float ratio;

 win1 = (struct display_win_rec *) xv_get(canvas,WIN_CLIENT_DATA);
 win = win1->owner;
 win->dwin = win1;

 value = event_action(event);
 PRT3(printf("acsii event_shiftmask(event) = %x...value = %d\n",
     (unsigned long) event_shiftmask(event),value);)
 PRT3(if (event_shiftmask(event) & CAPSLOCK) printf("CAPSLOCK is down\n"))
 if ((value >= (int) '0') && (value <= (int) '9'))
    {
     if (event_shift_is_down(event) && event_meta_is_down(event))
        {value -= (int) '0';
         value = num_keys[lcv].key;}
       else
        {value -= (int) '0';
         if (event_meta_is_down(event))
            {if (win->control.which_cursor == CURSOR_MOVE_ROI)
                transform_roi_proc(win,1,(float) value,.0);
               else if (win->control.which_cursor == CURSOR_MOVE_SUPERIMPOSE)
                move_superimpose_proc(win,1,(float) value,.0);
               else if ((win->control.which_cursor == CURSOR_RESIZE_ROI) ||
                        (win->control.which_cursor == CURSOR_ROTATE_ROI))
                transform_roi_proc(win,1,(float) value,(float) value);
               else
                {if (value == 0) value = 10;
                 magnify_pixel_option_proc(0,0,win1,canvas,event,value);}
           } 
          else if (win->type2 & MANY_SLICES_WINDOWS)
           {select_slice_slider_proc(0,value,event,win1,canvas);
           }
          else if (value == 0) 
           if (win->type2 & HISTOGRAM_WINDOW)
              WinHistoMultiCanvas(win);
         return NO_ERROR;}
     }
 if (event_shift_is_down(event))
     {
      for (lcv = 0; lcv < num_num_keys; lcv++)
         if (num_keys[lcv].key == value)
            {if (event_meta_is_down(event))
                {if (win->control.which_cursor == CURSOR_MOVE_ROI)
                    return transform_roi_proc(win,
                              1,(float) num_keys[lcv].value,.0);
                   else if (win->control.which_cursor == CURSOR_MOVE_SUPERIMPOSE)
                    return move_superimpose_proc(win,1,(float) num_keys[lcv].value,.0);
                   else if ((win->control.which_cursor == CURSOR_RESIZE_ROI) ||
                            (win->control.which_cursor == CURSOR_ROTATE_ROI))
                    {ratio = (float) num_keys[lcv].value;
                     if (win->control.which_cursor == CURSOR_RESIZE_ROI)
                        ratio *= .1;
                       else ratio = -ratio;
                     return transform_roi_proc(win,1,ratio,ratio);
                   }
                }

             if (win->type2 & num_keys[lcv].win_type)
                select_slice_slider_proc(0,num_keys[lcv].value,event,
                                        win1,canvas);
               else win_alarm(win1->frame,2);
             return NO_ERROR;}
     if (event_meta_is_down(event))
        {for (lcv = 0; lcv < num_shift_action_key_actions; lcv++)
            if (shift_action_key_actions[lcv].key == value)
               {if (win->type2 & shift_action_key_actions[lcv].win_type)
                   {shift_action_key_actions[lcv].proc(NULL,NULL,win1,canvas,event,
                           shift_action_key_actions[lcv].value);
                    return NO_ERROR;}
                  else if (!shift_action_key_actions[lcv].more)
                   {win_alarm(win1->frame,2);
                    return NO_ERROR;}
               }
         }
        else 
         for (lcv = 0; lcv < num_shift_key_actions; lcv++)
            if (shift_key_actions[lcv].key == value)
               {if (win->type2 & shift_key_actions[lcv].win_type)
                   {shift_key_actions[lcv].proc(NULL,NULL,win1,canvas,event,
                           shift_key_actions[lcv].value);
                    return NO_ERROR;}
                  else if (!shift_key_actions[lcv].more)
                   {win_alarm(win1->frame,2);
                    return NO_ERROR;}
               }
     }
    else if (event_meta_is_down(event))
     {
      value = upper_case_ch(value);
      TRACE(printf("event_meta_is_down..value = %d   = %c\n",value,(char) value))

      for (lcv = 0; lcv < num_action_key_actions; lcv++)
         if (action_key_actions[lcv].key == value)
            {if (win->type2 & action_key_actions[lcv].win_type)
                {action_key_actions[lcv].proc(NULL,NULL,win1,canvas,event,
                           action_key_actions[lcv].value);
                 return NO_ERROR;}
              else if (!action_key_actions[lcv].more)
               {win_alarm(win1->frame,2);
                return NO_ERROR;}
            }
     }
    else if (event_alt_is_down(event))
     {
      value = upper_case_ch(value);
      PRT3(printf("event_alt_is_down..value = %d   = %c\n",value,(char) value))
      for (lcv = 0; lcv < num_alt_key_actions; lcv++)
         if (alt_key_actions[lcv].key == value)
            {if (win->type2 & alt_key_actions[lcv].win_type)
                {alt_key_actions[lcv].proc(NULL,NULL,win1,canvas,event,
                           alt_key_actions[lcv].value);
                 return NO_ERROR;}
              else if (!alt_key_actions[lcv].more)
               {win_alarm(win1->frame,2);
                return NO_ERROR;}
            }
     }
    else
     {
      value = upper_case_ch(value);
      for (lcv = 0; lcv < num_clipboard_key_actions; lcv++)
        if (clipboard_key_actions[lcv].key == value)
           {if (win->type2 & clipboard_key_actions[lcv].win_type)
               {clipboard_key_actions[lcv].proc(NULL,NULL,win1,canvas,event,
                           clipboard_key_actions[lcv].value);
                return NO_ERROR;}
              else if (!clipboard_key_actions[lcv].more)
               {win_alarm(win1->frame,2);
                return NO_ERROR;}
           }
     }
 win_alarm(win1->frame,2);
 return CANCEL;
}


/*-------------------------------------------------------------------------
process_control_key - ???
-------------------------------------------------------------------------*/

int
process_control_key(canvas,event)
 Xv_Window canvas;
 Event *event;
{
 int lcv,value;
 static struct key_proc_value_rec actions[] = 
         {
          {31757 /* A */,enable_move_superimpose_cursor_proc,0,
                       EDITING_WINDOW,0},
          {31752,/*'B' - 'A' + 1,*/breaker_it_proc,VALUE_BREAKER_ENABLE,
                       EDIT_NO_3D_WINDOW,0},
          {'C' - 'A' + 1,do_CA_proc,0,EDIT_NO_3D_WINDOW,0},
          {'E' - 'A' + 1,file_proc,VALUE_ENTER_FILE_COMMENT,EDITING_WINDOW,0},
          {31753/*'F' - 'A' + 1*/,fusion_image_action_proc,FUSION_WHICH_WIN,
                            EDITING_WINDOW,0},
          {'G' - 'A' + 1,do_GA_proc,1,EDIT_NO_3D_WINDOW,0},
          {'I' - 'A' + 1,_3d_co_register_proc,
              COREG_UNCO_CUR_WID_FROM_EVERY_GROUP_VALUE,DISPLAY_WINDOW_3D,0},
          {'J' - 'A' + 1,_3d_co_register_proc,
              COREG_ADJUST_VIEW_ON_VALUE,DISPLAY_WINDOW_3D,0},
          {'K' - 'A' + 1,_3d_co_register_proc,
              COREG_UNCO_VALUE,DISPLAY_WINDOW_3D,0},
          {'L' - 'A' + 1,_3d_co_register_proc,
              COREG_LOCKED_TO_VALUE,DISPLAY_WINDOW_3D,0},
          {'M' - 'A' + 1,text_output_proc,VALUE_PRINT_ROIS_AND_PIXEL_TYPES,
                       EDIT_NO_3D_WINDOW,0},
          {'O' - 'A' + 1,_3d_co_register_proc,
              COREG_UNLOCK_ALL_WIDS_VALUE,DISPLAY_WINDOW_3D,0},
          {31761 /*'P' - 'A' + 1*/,print_or_save_image_proc,
                        VALUE_SAVE_CURRENT_WIN_IMAGES,ALL_WINDOWS,0},
          {'R' - 'A' + 1,win_3d_proc,1,IMAGE_WINDOWS,0},
          {'S' - 'A' + 1,fusion_image_action_proc,FUSION_LAST_LEFT_TOP,
                      ALL_WINDOWS,0},
          {'T' - 'A' + 1,do_TA_proc,0,EDIT_NO_3D_WINDOW,0},
          {31749 /*'U' - 'A' + 1*/,set_slice_min_max_proc,2,EDITING_WINDOW,0},
          {31747 /*'W' - 'A' + 1*/,duplicate_win_proc,0,ALL_WINDOWS,0},
          {'Y' - 'A' + 1,make_window_cur_proc,0,ALL_WINDOWS,0},
          {31764,edit_scratch_proc,0,EDITING_WINDOW,0},
          {13,edit_scratch_proc,0,EDITING_WINDOW,0},
          {32582,clipboard_it_proc,VALUE_CUT_APPEND_OPTION,EDITING_WINDOW,0},
          {32578,clipboard_it_proc,VALUE_COPY_APPEND_OPTION,
                   EDITING_WINDOW,0},
          {32580,clipboard_it_proc,VALUE_PASTE_ALL,EDITING_WINDOW,0},
          {'1',rotate_plane_proc,1,IMAGE_WINDOWS,0},
          {0,rotate_plane_proc,2,IMAGE_WINDOWS,0}, 
          {27,rotate_plane_proc,3,IMAGE_WINDOWS,0},
          {28,rotate_plane_proc,4,IMAGE_WINDOWS,0},
          {29,rotate_plane_proc,5,IMAGE_WINDOWS,0},
          {30,rotate_plane_proc,6,IMAGE_WINDOWS,0}, 
          {31,rotate_plane_proc,7,IMAGE_WINDOWS,0},
          {127,rotate_plane_proc,8,IMAGE_WINDOWS,0},
          {'9',rotate_plane_proc,9,IMAGE_WINDOWS,0},
          {'0',rotate_plane_proc,10,IMAGE_WINDOWS,0},
          {'Q' - 'A' + 1,win_quit_and_save_file_proc,0,ALL_WINDOWS,0},
          {61 /*'+'*/,resize_proc,RESIZE_ZOOM_IN_VALUE,IMAGE_WINDOWS,0},
          {31783,magnify_pixel_option_proc,MAGNIFY_DEFAULT_DIM,ALL_WINDOWS,0},
          {31756,pen_type_proc,DRAW_POINT,EDIT_NO_3D_WINDOW,0},
          {31758 /*'\''*/,overlay_proc,
                 OVERLAYED_BY_WID_STACK_VALUE,EDITING_WINDOW,0},
         };
 static int num_actions = 39;

 static struct key_proc_value_rec shift_actions[] = 
         {
          {33,rotate_plane_proc,-1,IMAGE_WINDOWS,0},
          {0,rotate_plane_proc,-2,IMAGE_WINDOWS,0}, 
          {35,rotate_plane_proc,-3,IMAGE_WINDOWS,0},
          {36,rotate_plane_proc,-4,IMAGE_WINDOWS,0},
          {37,rotate_plane_proc,-5,IMAGE_WINDOWS,0},
          {30,rotate_plane_proc,-6,IMAGE_WINDOWS,0}, 
          {38,rotate_plane_proc,-7,IMAGE_WINDOWS,0},
          {42,rotate_plane_proc,-8,IMAGE_WINDOWS,0},
          {40,rotate_plane_proc,-9,IMAGE_WINDOWS,0},
          {41,rotate_plane_proc,-10,IMAGE_WINDOWS,0},
          {'+',resize_proc,RESIZE_DEFAULT_SIZE_VALUE,IMAGE_WINDOWS,0},
          {31750 /*'U' - 'A' + 1*/,set_slice_min_max_proc,3,EDITING_WINDOW,0},
          {'J' - 'A' + 1,_3d_co_register_proc,
              COREG_ADJUST_VIEW_OFF_VALUE,DISPLAY_WINDOW_3D,0},
          {'L' - 'A' + 1,_3d_co_register_proc,
               COREG_UNLOCK_FROM_VALUE,DISPLAY_WINDOW_3D,0},
          {'O' - 'A' + 1,_3d_co_register_proc,
               COREG_UNCO_AND_UNLOCK_ALL_WIDS_VALUE,DISPLAY_WINDOW_3D,0},
          {'Q' - 'A' + 1,file_proc,VALUE_RELOAD_SLICE_ROIS,EDIT_NO_3D_WINDOW,1},
          {'=' - 'A' + 1,resize_proc,RESIZE_ZOOM_OUT_VALUE,IMAGE_WINDOWS,0},
          {'R' - 'A' + 1,import_slice_rois_proc,0,DISPLAY_WINDOW,0},
          {'\"',overlay_proc,DELETE_OVERLAY_STACK_VALUE,EDITING_WINDOW,0},
         };
 static int num_shift_actions = 19;

 static struct key_proc_value_rec meta_digit_actions[] = 
         {{49,rotate_plane_proc,11,IMAGE_WINDOWS,0},
          {0,rotate_plane_proc,12,IMAGE_WINDOWS,0}, 
          {27,rotate_plane_proc,13,IMAGE_WINDOWS,0},
          {28,rotate_plane_proc,14,IMAGE_WINDOWS,0},
          {29,rotate_plane_proc,15,IMAGE_WINDOWS,0},
          {30,rotate_plane_proc,16,IMAGE_WINDOWS,0},
          {31,rotate_plane_proc,17,IMAGE_WINDOWS,0},
          {127,rotate_plane_proc,18,IMAGE_WINDOWS,0},
          {57,rotate_plane_proc,19,IMAGE_WINDOWS,0},
          {48,rotate_plane_proc,0,IMAGE_WINDOWS,0},
          {61/*'='*/,resize_proc,RESIZE_LAST_VALUE,IMAGE_WINDOWS,0},
          {'R' - 'A' + 1,reverse_slice_rois_proc,0,DISPLAY_WINDOW,0},
          {'\'',overlay_proc,OVERLAYED_BY_ALL_ECHO_WID_STACK_VALUE,EDITING_WINDOW,0},
         };
 static int num_meta_digit_actions = 13;
 static struct key_proc_value_rec meta_shift_digit_actions[] = 
         {{33,rotate_plane_proc,-11,IMAGE_WINDOWS,0},
          {0,rotate_plane_proc,-12,IMAGE_WINDOWS,0}, 
          {35,rotate_plane_proc,-13,IMAGE_WINDOWS,0},
          {36,rotate_plane_proc,-14,IMAGE_WINDOWS,0},
          {37,rotate_plane_proc,-15,IMAGE_WINDOWS,0},
          {30,rotate_plane_proc,-16,IMAGE_WINDOWS,0},
          {38,rotate_plane_proc,-17,IMAGE_WINDOWS,0},
          {42,rotate_plane_proc,-18,IMAGE_WINDOWS,0},
          {40,rotate_plane_proc,-19,IMAGE_WINDOWS,0},
          {41,rotate_plane_proc,0,IMAGE_WINDOWS,0},

          {'\"',overlay_proc,SHOW_HIDE_OVERLAY_STACK_VALUE,EDITING_WINDOW,0},
         };
 static int num_meta_shift_digit_actions = 11;

 struct display_win_rec *win1;
 struct image_display_window *win;

 win1 = (struct display_win_rec *) xv_get(canvas,WIN_CLIENT_DATA);
 win = win1->owner;
 win->dwin = win1;
 value = event_action(event);
 PRT3(printf("process_control_key val = %d\n",value);)
 if (event_meta_is_down(event)) 
    {
     PRT3(printf("process_control_key event_meta_is_down\n");)
     if (event_shift_is_down(event))
        {for (lcv = 0; lcv < num_meta_shift_digit_actions; lcv++)
            if (meta_shift_digit_actions[lcv].key == value)
               {meta_shift_digit_actions[lcv].proc(NULL,NULL,win1,canvas,event,
                           meta_shift_digit_actions[lcv].value);
                return NO_ERROR;}
        }
       else
        for (lcv = 0; lcv < num_meta_digit_actions; lcv++)
           if (meta_digit_actions[lcv].key == value)
               {meta_digit_actions[lcv].proc(NULL,NULL,win1,canvas,event,
                           meta_digit_actions[lcv].value);
               return NO_ERROR;}
   }
 if (event_shift_is_down(event))
    {
     PRT3(printf("event_shift_is_down ...\n");)
     for (lcv = 0; lcv < num_shift_actions; lcv++)
        if (shift_actions[lcv].key == value)
           {if (win->type2 & shift_actions[lcv].win_type)
                   shift_actions[lcv].proc(NULL,NULL,win1,canvas,event,
                      shift_actions[lcv].value);
                  else win_alarm(win1->frame,2);
            return NO_ERROR;}
    }
 PRT3(printf("process_control_key val = %d...1\n",value);)
 for (lcv = 0; lcv < num_actions; lcv++)
    if (actions[lcv].key == value)
       {if (win->type2 & actions[lcv].win_type)
           actions[lcv].proc(NULL,NULL,win1,canvas,event,actions[lcv].value);
          else win_alarm(win1->frame,2);
        return NO_ERROR;}
 return CANCEL;
}



/*-------------------------------------------------------------------------
process_action_key - ???
-------------------------------------------------------------------------*/

int
process_action_key(canvas,event)
 Xv_Window canvas;
 Event *event;
{
 int lcv,value;
 static struct key_proc_value_rec action_keys[] = {
          {31768,_3d_co_register_proc,
              COREG_CO_AND_FUSION_VALUE,DISPLAY_WINDOW_3D,0},
          {ACTION_CUT,clipboard_it_proc,VALUE_CUT_OPTION,EDITING_WINDOW,0},
          {ACTION_COPY,clipboard_it_proc,VALUE_COPY_OPTION,EDITING_WINDOW,0},
          {ACTION_PASTE,clipboard_it_proc,VALUE_PASTE_ALL,EDITING_WINDOW,0},
          {ACTION_INSERT,_3d_last_plane_proc,0,DISPLAY_WINDOW_3D,0},
/* <> P */{ACTION_COPY_THEN_PASTE,pixel_selection_proc,0,MANY_SLICES_WINDOWS,0},
/* <> d */{31795,dp_delete_roi_proc,0,EDITING_WINDOW,0},
/* <> e */{31798,edge_detection_proc,0,EDITING_WINDOW,0},
/* <> i */{ACTION_INCLUDE_FILE,process_slice_scans_proc,8,DISPLAY_WINDOW,0},
/* <> s */{ACTION_STORE,enter_save_dat_proc,
                 VALUE_SAVE_DAT_FILE,EDIT_NO_3D_WINDOW,0},

/* <> l */{ACTION_LOAD,file_proc,VALUE_LOAD_DAT_FILE,EDIT_NO_3D_WINDOW,0},
/* <> f */{31779,fusion_proc,0,ALL_WINDOWS,0},
         };
 static int num_action_keys = 12;
/******
 static struct key_proc_value_rec ctrl_action_keys[] = {
          {ACTION_INSERT,_3d_last_plane_proc,0,DISPLAY_WINDOW_3D,0},
         };
 static int num_ctrl_action_keys = 1;
******/

 Frame frame;
 struct display_win_rec *win1;
 struct image_display_window *win;

 win1 = (struct display_win_rec *) xv_get(canvas,WIN_CLIENT_DATA);
 win = win1->owner;
 win->dwin = win1;
 value = event_action(event);
 PRT2(printf("process_action_key val = %d...1\n",value);)
 if (value == ACTION_HELP)
    {if (event_shift_is_down(event))
        display_menu_help_proc(0,0,win1,canvas,event);
       else analysis_menu_help_proc(0,0,win1,canvas,event);
     return NO_ERROR;}
   else if (value == ACTION_OPEN)
    {frame = (Frame) xv_get(canvas,XV_OWNER);
     if (xv_get(frame,FRAME_CLOSED))
        xv_set(frame,FRAME_CLOSED,FALSE,0);
       else xv_set(frame,FRAME_CLOSED,TRUE,0);
     return NO_ERROR;
    }
 PRT3(printf("process_action_key val = %d\n",value);)
/******
 if (event_ctrl_is_down(event))
    {for (lcv = 0; lcv < num_ctrl_action_keys; lcv++)
        if (ctrl_action_keys[lcv].key == value)
           {if (win->type2 & ctrl_action_keys[lcv].win_type)
               ctrl_action_keys[lcv].proc(NULL,NULL,win1,canvas,
                     event,ctrl_action_keys[lcv].value);
              else win_alarm(win1->frame,2);
            return NO_ERROR;}
    }
   else
******/
    for (lcv = 0; lcv < num_action_keys; lcv++)
       if (action_keys[lcv].key == value)
          {if (win->type2 & action_keys[lcv].win_type)
              action_keys[lcv].proc(NULL,NULL,win1,canvas,
                     event,action_keys[lcv].value);
             else win_alarm(win1->frame,2);
           return NO_ERROR;}
 win_alarm(win1->frame,2);
 return CANCEL;
}


/*-------------------------------------------------------------------------
backspace_pts - ???
-------------------------------------------------------------------------*/

int
backspace_pts(canvas)
 Xv_Window canvas;
{
 struct display_win_rec *win1;
 struct image_display_window *win;
 int value;
 static Event event;
 int x,y,num_pts,temp;

 win1 = (struct display_win_rec *) xv_get(canvas,WIN_CLIENT_DATA);
 win = win1->owner;
 win->dwin = win1;
 num_pts = 0;
 do
    {
     num_pts += 1;
     if (xv_input_readevent(canvas,&event,FALSE,FALSE,NULL) != 0)
        {value = event_action(&event);
         if ((value == BACKSPACE_KEY) || (value == DELETE_KEY) ||
             (value == ACTION_ERASE_CHAR_BACKWARD))
            {}
           else
            {
             notify_post_event((Notify_client) canvas,
                       (Notify_event) &event,NOTIFY_SAFE);
             break;}
        }
       else break;
    }
 while (1);
 if (!win_is_non_empty_cur_curve(win)) return NO_ERROR;
 temp = win_num_cur_curve_pts(win);
 if (temp <= num_pts)
    {win_get_cur_pt_by_indx(win,0,&x,&y);
     num_pts = temp;}
   else if (win->type2 & DISPLAY_WINDOW_3D)
    {if (temp == 2)
        {win_get_cur_pt_by_indx(win,0,&x,&y);
         num_pts = 2;}
    }
   else if (win->control.pen_type & DRAW_MODE_BACKSPACE_ALL)
    {win_get_cur_pt_by_indx(win,0,&x,&y);
     num_pts = temp;
    }

 WinEraseAndDisplayPts(win,0,0,num_pts);
 if (temp == num_pts) win_move_mouse_to(win,x,y);
   else win_move_mouse_to_cur_pt(win);
 return NO_ERROR;
}


/******
rd_until_event_up(canvas_win)
 Xv_Window canvas_win;
{
 static Event event;
 int cnt;

 cnt = 0;
 do
    {
     if (xv_input_readevent(canvas_win,&event,FALSE,FALSE,NULL) != NULL)
        {if (event_is_up(&event))
            {printf("event is up...rd_until_event_up\n");
             break;}
         cnt += 1;
        }
       else break;
    }
 while (1);
 printf("discarding cnt = %d\n",cnt);
}
******/


/*-------------------------------------------------------------------------
key_pressed - ???
-------------------------------------------------------------------------*/

void
key_pressed(canvas_win,key_pressed,num_key_pressed)
 Xv_Window canvas_win;
 struct key_pressed_rec key_pressed[];
 int num_key_pressed;
{
 int value;
 static Event event;
 unsigned int lcv;

 for (lcv = 0; lcv < num_key_pressed; lcv++) key_pressed[lcv].cnt = 0;
 do
    {
     if (xv_input_readevent(canvas_win,&event,FALSE,FALSE,NULL) != 0)
        {value = event_action(&event);
         for (lcv = 0; lcv < num_key_pressed; lcv++)
            if (key_pressed[lcv].key == value)
               {key_pressed[lcv].cnt += 1;
                break;}
         if (lcv == num_key_pressed)
            {
             notify_post_event((Notify_client)canvas_win,
                        (Notify_event) &event,NOTIFY_SAFE);
             break;}
        }
       else break;
    }
 while (1);     /* FIXME:  Can this be right? */
}


/*-------------------------------------------------------------------------
process_ascii_event - ???
-------------------------------------------------------------------------*/

int
process_ascii_event(canvas,event)
 Xv_Window canvas;
 Event *event;
{
 int value;

 value = event_action(event);
 if ((value == BACKSPACE_KEY) || (value == DELETE_KEY) ||
     (value == ACTION_ERASE_CHAR_BACKWARD))
    {if (!event_ctrl_is_down(event))
        return backspace_pts(canvas);
    }
 if (event_is_key_top(event))
    {
     return process_key_top(canvas,event);
    }
   else if (event_is_key_right(event))
     return process_right_key(canvas,event);
   else if (event_ctrl_is_down(event))
    {
     return process_control_key(canvas,event);
    }
   else if (((value >= ASCII_FIRST) && (value <= ASCII_LAST)) ||
            ((value >= META_FIRST) && (value <= META_LAST)))
    {
     return process_ascii_key(canvas,event);
    }
   else if ((value >= ACTION_AGAIN) && (value <= ACTION_SELECT))
    {
     return process_action_key(canvas,event);
    }

 return CANCEL;
}
