/****************************************************************************
analyze_hdr.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: analyze_hdr.c,v 1.5 1999/07/07 18:07:13 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdio.h>
#include <math.h>

#include <sys/types.h>

#include "typedef.h"
#include "analyze_hdr.h"
#include "ia_hdr.h"
#include "error_constants.h"
#include "debug.h"


/* Functions defined in this file */
void initAnalyzeHdr(struct dsr *dsr);
int writeAnalyzeHdr(struct dsr *dsr, char *fn);
int readAnalyzeHdr(struct dsr *dsr, char *fn);


/*--------------------------------------------------------------------------
initAnalyzeHdr - ???
--------------------------------------------------------------------------*/

void
initAnalyzeHdr( dsr )
     struct dsr * dsr;
{
  /*  Database internals (including image data type)  */
  dsr->hk.sizeof_hdr = sizeof( struct dsr );
  strcpy( dsr->hk.data_type, "dsr      \0" );
  strcpy( dsr->hk.db_name, ".hdr\0" );
  dsr->hk.extents = 0;
  dsr->hk.session_error = 0;
  dsr->hk.regular = 'r';
  dsr->hk.hkey_un0 = '0';


  /*  Image dimensions  */
  dsr->dime.dim[0] = 4;	/* num_dims */
  dsr->dime.dim[1] = 0;	/* i, j, k #voxels per side */
  dsr->dime.dim[2] = 0;
  dsr->dime.dim[3] = 0;
  dsr->dime.dim[4] = 1;	/* num_vols */
  dsr->dime.dim[5] = 0;
  dsr->dime.dim[6] = 0;
  dsr->dime.dim[7] = 0;
  strcpy( dsr->dime.vox_units, "mm" );
  strcpy( dsr->dime.cal_units, "\0" );
  dsr->dime.unused1 = 0;
  dsr->dime.datatype = DT_SIGNED_SHORT;
  dsr->dime.bitpix = 16;
  dsr->dime.dim_un0 = 0;
  dsr->dime.pixdim[0] = 0.0;
  dsr->dime.pixdim[1] = 0.8594;
  dsr->dime.pixdim[2] = 0.8594;
  dsr->dime.pixdim[3] = 5.0000;
  dsr->dime.pixdim[4] = 0.0;
  dsr->dime.pixdim[5] = 0.0;
  dsr->dime.pixdim[6] = 0.0;
  dsr->dime.pixdim[7] = 0.0;
  dsr->dime.vox_offset = 0.0;
  dsr->dime.funused1 = 0.0;
  dsr->dime.cal_max = 0.0;
  dsr->dime.cal_min = 0.0;
  dsr->dime.compressed = 0;
  dsr->dime.verified = 0;
  dsr->dime.glmax = 32767;
  dsr->dime.glmin = 0;


  /*  Patient information */
  strcpy( dsr->hist.descrip, "spm compatible\0" );
  strcpy( dsr->hist.aux_file, "none                   \0" );
  dsr->hist.orient = '\0';
  strcpy( dsr->hist.originator, "\0" );
  strcpy( dsr->hist.generated, "\0" );
  strcpy( dsr->hist.scannum, "\0" );
  strcpy( dsr->hist.patient_id, "\0" );
  strcpy( dsr->hist.exp_date, "\0" );
  strcpy( dsr->hist.exp_time, "\0" );
  strcpy( dsr->hist.hist_un0, "\0" );
  dsr->hist.views = 0;
  dsr->hist.vols_added = 0;
  dsr->hist.start_field = 0;
  dsr->hist.field_skip = 0;
  dsr->hist.omax = 0;
  dsr->hist.omin = 0;
  dsr->hist.smax = 0;
  dsr->hist.smin = 0;
}



/*--------------------------------------------------------------------------
writeAnalyzeHdr - ???
--------------------------------------------------------------------------*/

int
writeAnalyzeHdr( dsr, fn )
     struct dsr * dsr;
     char * fn;
{
  FILE * fid;
  
  if ((fid = fopen( fn, "w" )) == NULL)
    return FILE_ACCESS_ERROR;
  
  if (fwrite( dsr, sizeof( struct dsr ), 1, fid ) != 1)
  {
    fclose( fid );
    printf( "Error writing the Analyze format header file \"%s\".", fn );
    return WRITE_ERROR;
  }

  fclose( fid );
  return NO_ERROR;
}



/*--------------------------------------------------------------------------
readAnalyzeHdr - ???
--------------------------------------------------------------------------*/

int
readAnalyzeHdr( dsr, fn )
     struct dsr * dsr;
     char * fn;
{
  FILE * fid;

  if ((fid = fopen( fn, "r" )) == NULL)
    return FILE_ACCESS_ERROR;

  if (fread( dsr, sizeof( struct dsr ), 1, fid ) != 1)
  {
    fclose( fid );
    printf( "Error reading the Analyze format header file \"%s\".", fn );
    return READ_ERROR;
  }

  fclose( fid );
  return NO_ERROR;
}
