/****************************************************************************
dicom_dict.c - ???

Author:  DRW 960922

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.

Dictionary of DICOM element definitions.  Includes lookup routine
DefinitionPtr findElementDefn _ANSI_ARGS_(( ElementGroupId, ElementId ));
****************************************************************************/

static const char rcsid[] = "$Id: dicom_dict.c,v 1.7 1999/07/07 18:07:14 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <sys/types.h>         /* Needed to define off_t for bbl_dicom.h */

#include "bbl_dicom.h"


/*--------------------------------------------------------------------------
dicomDefinitions[] - ???
--------------------------------------------------------------------------*/

static Definition dicomDefinitions[] = {

  /*
   *  Group 0x0008: Study-specific information
   */
  { GstudyInfo, EgroupLength,
     "Group Length",			ELEM_VR_UL, 1 },
  { GstudyInfo, 0x0008,
     "Image Type",			ELEM_VR_CS, ELEM_VM_1_THRU_N },
  { GstudyInfo, EstudyDate,
     "Study Date",			ELEM_VR_DA, 1 },
  { GstudyInfo, 0x0021,
     "Series Date",			ELEM_VR_DA, 1 },
  { GstudyInfo, 0x0022,
     "Acquisition Date",		ELEM_VR_DA, 1 },
  { GstudyInfo, 0x0023,
     "Image Date",			ELEM_VR_DA, 1 },
  { GstudyInfo, EstudyTime,
     "Study Time",			ELEM_VR_TM, 1 },
  { GstudyInfo, 0x0031,
     "Series Time",			ELEM_VR_TM, 1 },
  { GstudyInfo, 0x0032,
     "Acquisition Time",		ELEM_VR_TM, 1 },
  { GstudyInfo, 0x0033,
     "Image Time",			ELEM_VR_TM, 1 },
  { GstudyInfo, 0x0050,
     "Accession Number",		ELEM_VR_SH, 1 },
  { GstudyInfo, Emodality,
     "Modality",			ELEM_VR_CS, 1 },
  { GstudyInfo, 0x0070,
     "Manufacturer",			ELEM_VR_LO, 1 },
  { GstudyInfo, EinstitutionName,
     "Institution Name",		ELEM_VR_LO, 1 },
  { GstudyInfo, EreferringPhysiciansName,
     "Referring Physician's Name",	ELEM_VR_PN, 1 },
  { GstudyInfo, 0x1010,
     "Station Name",			ELEM_VR_SH, 1 },
  { GstudyInfo, 0x1030,
     "Study Description",		ELEM_VR_LO, 1 },
  { GstudyInfo, 0x103E,
     "Series Description",		ELEM_VR_LO, 1 },
  { GstudyInfo, 0x1070,
     "Operators' Name(s)",		ELEM_VR_PN, ELEM_VM_1_THRU_N },
  { GstudyInfo, 0x1090,
     "Manufacturer's Model Name",	ELEM_VR_LO, 1 },

  /*
   *  Group 0x0010: Patient information
   */
  { GpatientInfo, EgroupLength,
     "Group Length",			ELEM_VR_UL, 1 },
  { GpatientInfo, Epatient_name,
     "Patient's Name",			ELEM_VR_PN, 1 },
  { GpatientInfo, Epatient_id,
     "Patient's ID",			ELEM_VR_LO, 1 },
  { GpatientInfo, Epatient_birthDate,
     "Patient's Birth Date",		ELEM_VR_DA, 1 },
  { GpatientInfo, Epatient_sex,
     "Patient's Sex",			ELEM_VR_CS, 1 },
  { GpatientInfo, Epatient_age,
     "Patient's Age",			ELEM_VR_AS, 1 },
  { GpatientInfo, Epatient_weight,
     "Patient's Weight",		ELEM_VR_DS, 1 },

  /*
   *  Group 0x0018: Physical aspects of study
   */
  { GphysicalInfo, EgroupLength,
     "Group Length",			ELEM_VR_UL, 1 },
  { GphysicalInfo, 0x0010,
     "Contrast/Bolus Agent",		ELEM_VR_LO, 1 },
  { GphysicalInfo, 0x0020,
     "Scanning sequence",		ELEM_VR_CS, ELEM_VM_1_THRU_N },
  { GphysicalInfo, 0x0021,
     "Sequence variant",		ELEM_VR_CS, ELEM_VM_1_THRU_N },
  { GphysicalInfo, 0x0022,
     "Scan options",			ELEM_VR_CS, ELEM_VM_1_THRU_N },
  { GphysicalInfo, Emr_acq_type,
     "MR Acquisition Type",		ELEM_VR_CS, 1 },
  { GphysicalInfo, Esequence_name,
     "Sequence Name",			ELEM_VR_SH, 1 },
  { GphysicalInfo, Eslice_thickness,
     "Slice Thickness",			ELEM_VR_DS, 1 },
  { GphysicalInfo, Etr,
     "Repetition Time",			ELEM_VR_DS, 1 },
  { GphysicalInfo, Ete,
     "Echo Time",			ELEM_VR_DS, 1 },
  { GphysicalInfo, Eecho_num,
     "Echo Number(s)",			ELEM_VR_IS, ELEM_VM_1_THRU_N },
  { GphysicalInfo, 0x0087,
     "Magnetic Field Strength",		ELEM_VR_DS, 1 },
  { GphysicalInfo, EsliceSpacing,
     "Spacing Between Slices",		ELEM_VR_DS, 1 },
  { GphysicalInfo, 0x0091,
     "Echo Train Length",		ELEM_VR_IS, 1 },
  { GphysicalInfo, 0x0095,
     "Pixel Bandwidth",			ELEM_VR_DS, 1 },
  { GphysicalInfo, Eserial_num,
     "Device Serial Number",		ELEM_VR_LO, 1 },
  { GphysicalInfo, 0x1020,
     "Software Version(s)",		ELEM_VR_LO, ELEM_VM_1_THRU_N },
  { GphysicalInfo, 0x1050,
     "Spatial Resolution",		ELEM_VR_DS, 1 },
  { GphysicalInfo, 0x1088,
     "Heart Rate",			ELEM_VR_IS, 1 },
  { GphysicalInfo, 0x1149,
     "Field of View Dimension(s)",	ELEM_VR_IS, ELEM_VM_1_THRU_N },
  { GphysicalInfo, 0x1310,
     "Acquisition Matrix",		ELEM_VR_US, 4 },
  { GphysicalInfo, 0x1314,
     "Flip Angle",			ELEM_VR_DS, 1 },
  { GphysicalInfo, 0x1315,
     "Variable Flip Angle Flag",	ELEM_VR_CS, 1 },
  { GphysicalInfo, 0x1316,
     "SAR",				ELEM_VR_DS, 1 },

  /*
   *  Group 0x0020: Storage aspects of study
   */
  { GstorageInfo, EgroupLength,
     "Group Length",			ELEM_VR_UL, 1 },
  { GstorageInfo, Estudy_id,
     "Study ID",			ELEM_VR_SH, 1 },
  { GstorageInfo, EseriesNumber,
     "Series Number",			ELEM_VR_IS, 1 },
  { GstorageInfo, 0x0012,
     "Acquisition Number",		ELEM_VR_IS, 1 },
  { GstorageInfo, Eslice_num,
     "Image Number",			ELEM_VR_IS, 1 },
  { GstorageInfo, Estudy_series,
     "Series in Study",			ELEM_VR_IS, 1 },
  { GstorageInfo, Etotalimages,
     "Images in Acquisition",		ELEM_VR_IS, 1 },
  { GstorageInfo, 0x1040,
     "Position Reference Indicator",	ELEM_VR_LO, 1 },
  { GstorageInfo, EsliceLocation,
     "Slice Location",			ELEM_VR_DS, 1 },
  { GstorageInfo, Ecomment,
     "Image Comments",			ELEM_VR_LT, 1 },

  /*
   *  Group 0x0028: Imaging aspects of study
   */
  { GimagingInfo, EgroupLength,
     "Group Length",			ELEM_VR_UL, 1 },
  { GimagingInfo, Erows,
     "Rows",				ELEM_VR_US, 1 },
  { GimagingInfo, Ecolumns,
     "Columns",				ELEM_VR_US, 1 },
  { GimagingInfo, EpixelSpacing,
     "Pixel Spacing",			ELEM_VR_DS, 2 },
  { GimagingInfo, EbitsAllocated,
     "Bits Allocated",			ELEM_VR_US, 1 },
  { GimagingInfo, EbitsStored,
     "Bits Stored",			ELEM_VR_US, 1 },
  { GimagingInfo, EhighBit,
     "High Bit",			ELEM_VR_US, 1 },

  /*
   *  Group 0x7fe0:  Pixel Data
   */
  { GpixelData, EgroupLength,
     "Group Length",			ELEM_VR_UL, 1 },
  { GpixelData, Edata,
     "Pixel Data",			ELEM_VR_OW, 1 },

};

#define numDicomDefinitions \
	(sizeof( dicomDefinitions ) / sizeof( Definition ))



/*--------------------------------------------------------------------------
findElementDefn - ???

Lookup routine: given group ID and element ID (gggg.eeee, as defined
in NEMA standard DICOM v3.0), return pointer to definition structure.

--------------------------------------------------------------------------*/

DefinitionPtr
findElementDefn( ElementGroupId elementGroupId,
		 ElementId elementId )
{
  int currDefn = 0;

  /*
   *  The following comparisons depend on the fact that the
   *    definitions are sorted in ascending order, by group id,
   *    then by element id, to improve search time.
   */
  while ((dicomDefinitions[currDefn].elementGroupId < elementGroupId) &&
	 currDefn < numDicomDefinitions)
    currDefn++;
  if (currDefn == numDicomDefinitions ||
      dicomDefinitions[currDefn].elementGroupId > elementGroupId)
    return NULL;
  
  while ((dicomDefinitions[currDefn].elementId < elementId) &&
	 currDefn < numDicomDefinitions)
    currDefn++;
  if (currDefn == numDicomDefinitions ||
      dicomDefinitions[currDefn].elementId > elementId)
    return NULL;

  return &dicomDefinitions[currDefn];

} /* findElementDefn() */


