/****************************************************************************
img2ria.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: img2ria.c,v 1.7 1999/07/07 18:07:14 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdio.h>
#include <string.h>
#include <math.h>

#include <sys/types.h>

#include <unistd.h>

#include "ia_hdr.h"
#include "common.h"
#include "error_constants.h"
#include "mark_hdr.h"
#include "nria_lib.h"

#define PRT(x) x; fflush(stdout);
#undef PRT
#define PRT(x)

/* The library debugging code checks the following variable */
int debugFlag;


extern double		Get1dValue();
extern unsigned long	get_file_size_by_fn();
extern unsigned long	GetImageSize();
extern unsigned long	find_num_bytes_needed();


/* img2ria.c */
int ProcessImg(FILE *fid, char *fn, FILE *outfid, char *outfn,
    struct hdr_rec *hdr, unsigned long start_offset);
void input_info(char *fn, struct hdr_rec *hdr, unsigned long start_offset);
int output_hdr(char *fn, struct hdr_rec *hdr);
int OutputImg(char *fn, struct hdr_rec *hdr, unsigned long start_offset);
int SetupHdr(struct hdr_rec *hdr, unsigned long dimx, unsigned long dimy,
    double xpixel_size, double ypixel_size, double zpixel_size,
    unsigned long num_echoes, unsigned long input_depth, unsigned long type);
int prog_usage(int argc, char *argv[], struct hdr_rec *hdr,
    unsigned long *start_offset);
void main(int argc, char *argv[]);



#if 0

/*  FIXME:  We need to use the version in 2dminmaxscal.c, which takes into
    account floating point values.  -DRW ??? */

MinMax2d(_2d,min,max)
     struct buf_2d_rec *_2d;
     double *min,*max;
{
 unsigned long row,col;
 double val;

 *max = *min = _2d->GetVal(_2d,0,0);
 for (row = 0; row < _2d->dimy; row++)
    for (col = 0; col < _2d->dimx; col++)
       {val = _2d->GetVal(_2d,row,col);
        if (val > *max) *max = val;
        if (val < *min) *min = val;}
 return NO_ERROR;
}
#endif


/*-------------------------------------------------------------------------
ProcessImg - ???
-------------------------------------------------------------------------*/

int
ProcessImg(fid,fn,outfid,outfn,hdr,start_offset)
     FILE *fid,*outfid;
     struct hdr_rec *hdr;
     char *fn,*outfn;
     unsigned long start_offset;
{
 struct buf_2d_rec _2d;
 struct data_info_rec *info;
 unsigned long lcv;
 double min,max,min1,max1;
 int err;
 /*  buf_flags is for kludge code below.  DRW 960712  */
 unsigned long buf_flags = 0;



 /*
  *  Initialize data record to hold block of image data.
  */
 init_buf_2d_rec(&_2d);
 info = &(hdr->data_info);


 /*  DRW 960712  added this kludge code since 2d_buf_opr.c uses a
  *    different system for determining data type.  See ia_hdr.h vs.
  *    common.h.  Uses new variable "buf_flags".
  *
  *    It may look like it does not matter in 2d_buf_opr.c, but when
  *    we later call MinMax2d, we are using this flag in 2dminmaxscal.c.
  */
 if (info->var_use_status & INPUT_DATA_SIGNED)
   buf_flags |= BUF_PIXEL_SIGNED;
 else if (info->var_use_status & INPUT_DATA_DOUBLE) {
   buf_flags |= BUF_VALUE_DOUBLE;
   buf_flags |= BUF_PIXEL_SIGNED;
 }
 else if (info->var_use_status & INPUT_DATA_FLOAT) {
   buf_flags |= BUF_VALUE_FLOAT;
   buf_flags |= BUF_PIXEL_SIGNED;
 }


 /*  DRW 960715  Don't use the kludge code above, for now, since I
  *    have discovered some serious problems with signed data handling
  *    in the rest of the code (for the adventurous, try starting with
  *    hdr_opr.c, 2d_buf_opr.c, build_hdr.c, 2dminmaxscal.c, and of
  *    course, a light but necessary perusal of ia_hdr.h).
  *
  *  DRW 960905  Fixed bug.  Added BUF_PIXEL_SIGNED above, fixed
  *    routine bug (GetValProc and SetValProc) in 2d_value.c,
  *    forced use of MinMax2d() in 2dminmaxscal.c by eliminating
  *    same routine from this file (see top of source), and 
  *    cosmetic bug: %f should print double, not float, value.
  */
#if 0
 if ((err = SetupBuf2dRec(&_2d,info->dimx,info->dimy,info->pixel_depth_bits,
       info->var_use_status & INPUT_DATA_TYPE)) != NO_ERROR)
    {printf("Memory error...\n");
     return err;}
#else
 if ((err = SetupBuf2dRec(&_2d,info->dimx,info->dimy,info->pixel_depth_bits,
       buf_flags)) != NO_ERROR)
    {printf("Memory error...\n");
     return err;}
#endif


 fseek(fid,start_offset,0);
 printf("Processing input data...# of slices = %ld\n",
	info->max_scans);
 PRT(pr_buf_2d_rec(&_2d);)

 for (lcv = 0; lcv < info->max_scans; lcv++) {

   if (fread(_2d.buf[0],_2d.total_bytes,1,fid) != 1) {
     printf("Error Reading input data total_bytes = %ld...\n",
	    _2d.total_bytes);
     printf("Input data file = %s...\n",fn);
     free_buf_2d_rec(&_2d);
     return READ_ERROR;
   }

   if (lcv == 0) {
     /*  DRW 960905  New procedure call, see lib/2dminmaxscal.c  */
     MinMax2d( &_2d, 0, 0, 0 );
     min = _2d.minval;
     max = _2d.maxval;
   }
   else {
     /*  Force re-computation of min and max values  */
     _2d.status4 &= ~BUF_MIN_MAX_COMPUTED;

     /*  DRW 960905  New procedure call, see lib/2dminmaxscal.c  */
     MinMax2d( &_2d, 0, 0, 0 );
     min1 = _2d.minval;
     max1 = _2d.maxval;

     if (min1 < min) min = min1;
     if (max1 > max) max = max1;
   }
      
   printf( "Read slice# %ld... Min. = %f, Max. = %f\r",
	   lcv + 1, min, max );
   fflush(stdout);

   if (outfid != NULL) {
     printf("writing slice# %ld/%ld...total_bytes = %ld\n",
	    lcv + 1,info->max_scans,_2d.total_bytes);

     if (fwrite(_2d.buf[0],_2d.total_bytes,1,outfid) != 1)
       {printf("Error writing to the output *.img file = %s...\n",outfn);
	err = WRITE_ERROR;
	break;
      }
   }
 }
 info->min_pixel = min;
 info->max_pixel = max;

 free_buf_2d_rec(&_2d);
 printf("\n\n");
 return err;
}


/*-------------------------------------------------------------------------
input_info - ???

FIXME:  This function name is duplicated in mri2spm.c.
-------------------------------------------------------------------------*/

void
input_info(fn,hdr,start_offset)
 char *fn;
 struct hdr_rec *hdr;
 unsigned long start_offset;
{
 int err;
 unsigned long size,imgsize;
 struct data_info_rec *info;

 info = &(hdr->data_info);
 printf("Input file: %s\n",fn);
 printf("  start_offset = %ld\n  dimx = %ld dimy = %ld\n",
	start_offset,
	info->dimx,
	info->dimy
	);
 printf("  input_pixel_bits = %ld num_echoes = %d\n",
	info->pixel_depth_bits,
	info->num_echoes
	);

 if (info->var_use_status & INPUT_DATA_FLOAT)
   printf("  INPUT_DATA_FLOAT assumed\n");
 else if (info->var_use_status & INPUT_DATA_DOUBLE)
   printf("  INPUT_DATA_DOUBLE assumed\n");
 else if (info->var_use_status & INPUT_DATA_SIGNED)
   printf("  INPUT_DATA_SIGNED assumed\n");
 else
   printf("  INPUT_DATA_UNSIGNED assumed\n");

 size = get_file_size_by_fn(fn,&err);
 if (size <= start_offset)
    {printf("illegal file size = %ld & start_offset = %ld\n",
	    size,start_offset);
    return /*CANCEL*/;}
 imgsize = GetImageSize(info->dimx,info->dimy,info->pixel_depth_bits);
 hdr->data_info.max_scans = (size - start_offset) / imgsize;
 hdr->data_info.max_scans = (info->max_scans / info->num_echoes)
             * info->num_echoes;
 if (info->max_scans == 0)
    {printf("illegal file size = %ld & start_offset = %ld\n",
	    size,start_offset);
     printf("   image size = %ld & num slices = %ld\n",
            imgsize,info->max_scans);
     return /*CANCEL*/;}

 if ((size - start_offset) % imgsize)
    {printf("Warning : image size = %ld & num slices = %ld\n",imgsize,
              info->max_scans);
     printf("        leftover size = %ld\n",(size - start_offset) % imgsize);}

 printf("  # slices: %ld\n",info->max_scans);
 printf("\n");
}


/*-------------------------------------------------------------------------
output_hdr - ???
-------------------------------------------------------------------------*/

int
output_hdr(fn,hdr)
 char *fn;
 struct hdr_rec *hdr;
{
 struct mark_hdr mark;
 char *nhdrfn,*hdrfn,str[250];
 int err;

 strcpy(str,fn);
 drop_suffix(str);
 drop_dirfn(str);
 nhdrfn = hdrfn = NULL;
 if (((err = concat_strs(&nhdrfn,2,str,".nhdr")) != NO_ERROR) ||
     ((err = concat_strs(&hdrfn,2,str,".hdr")) != NO_ERROR))
    {c_free(nhdrfn);
     printf("Memory ERROR ...\n");
     c_free(hdrfn);
     return err;}
 free_hdr_analysis_rec(&(hdr->analysis));
 if (((err = build_extra_file_info_rec(&(hdr->extra_info)))
              != NO_ERROR) ||
     ((err = BuildRiaHdr(hdr,0)) != NO_ERROR))
    {printf("ERROR generating the RIA header file = %s ... err = %d\n",
                 nhdrfn,err);
     c_free(nhdrfn);
     c_free(hdrfn);
     return err;}

 if ((err = wr_the_hdr(nhdrfn,hdr)) != NO_ERROR)
    {printf("ERROR writing the RIA header file = %s ... err = %d\n",nhdrfn,err);
     c_free(nhdrfn);
     c_free(hdrfn);
     return err;}
 printf("...RIA header file %s was generated...\n",nhdrfn);

 init_mark_hdr(&mark);
 if (assign_nhdr_to_hdr(&mark,hdr) != NO_ERROR)
    printf("Warning : ERROR generating the Analyse header file = %s ... \n",
              hdrfn);
   else if (wr_hdr_file(hdrfn,&mark) != NO_ERROR) 
     printf("Warning : error writing the Analyse header file = %s ...\n",
              hdrfn);
   else printf("...Analyse header file %s was generated...\n",hdrfn);
 c_free(nhdrfn);
 c_free(hdrfn);
 return err;
}


/*-------------------------------------------------------------------------
OutputImg - ???
-------------------------------------------------------------------------*/

int
OutputImg(fn,hdr,start_offset)
 char *fn;
 struct hdr_rec *hdr;
 unsigned long start_offset;
{
 FILE *fid,*outfid;
 char *outfn,str[250];
 int err;

 if ((fid = fopen(fn,"r")) == NULL)
    {printf("Error !!! : Unable to open up the input file %s\n",fn);
     return ~NO_ERROR;}
 input_info(fn,hdr,start_offset);
 strcpy(str,fn);
 drop_suffix(str);
 drop_dirfn(str);
 
 if (start_offset != 0)
    {if ((err = concat_strs(&outfn,3,"ria",str,".img")) != NO_ERROR)
        {c_free(outfn);
         fclose(fid);
         return err;}
     PRT(printf("OutputImg outfn = %s\n",outfn))
     if ((outfid = fopen(outfn,"w")) == NULL)
        {printf("Error !!! : Unable to open up the output file %s\n",outfn);
         fclose(fid);
         c_free(outfn);
         return ~NO_ERROR;}
     }
    else
     {outfn = fn;
      outfid = NULL;}

 err = ProcessImg(fid,fn,outfid,outfn,hdr,start_offset);
 if (err != NO_ERROR)
    {if (outfn != fn) unlink(outfn);
     printf("Error saving the output img file = %s...!!!\n",outfn);
    }
 fclose(fid);
 if (outfid != NULL) fclose(outfid);
 if (err == NO_ERROR)
    err = output_hdr(outfn,hdr);
 if (outfn != fn) c_free(outfn);
 return err;
}


/*-------------------------------------------------------------------------
SetupHdr - ???
-------------------------------------------------------------------------*/

int
SetupHdr(hdr,dimx,dimy,xpixel_size,ypixel_size,zpixel_size,num_echoes,input_depth,type)
 struct hdr_rec *hdr;
 unsigned long dimx,dimy,num_echoes,input_depth,type;
 float xpixel_size,ypixel_size,zpixel_size;
{
 struct data_info_rec *info;

 info = &(hdr->data_info);
 info->dimx = dimx;
 info->dimy = dimy;
 info->xpixel_size = xpixel_size;
 info->ypixel_size = ypixel_size;
 info->pixel_thickness = zpixel_size;
 info->num_echoes = num_echoes;
 info->pixel_depth_bits = input_depth;
 info->var_use_status |= type;
 return NO_ERROR;
}
     

/*-------------------------------------------------------------------------
prog_usage - ???

FIXME:  It looks like this function does more than just explain how
to use the program.  -PWH 19990430
-------------------------------------------------------------------------*/

int
prog_usage(argc,argv,hdr,start_offset)
     int argc;
     char *argv[];
     struct hdr_rec *hdr;
     unsigned long *start_offset;
{
  unsigned long dimx,dimy,bits,num_echoes,sign,lcv;
  float xsize,ysize,zsize;
  unsigned long type,input_depth;
  struct mark_hdr mark_hdr;
  int err;
  char *whoami;



  /*  FIXME:  It this info current, and why does it live here rather
   *  than at the top of the file?  -PWH 19990430
   *
   *  3 args:
   *  img2ria image_fn dim pixelsize
   *
   *  6 or 7 args:
   *  img2ria image_fn original_nhdr
   *	     num_echoes input_pixel_bits is_data_signed data_offset
   *
   *  10 or 11 args:
   *  img2ria image_fn dimx dimy xpixelsize ypixelsize zpixelsize
   *          num_echoes input_pixel_bits is_data_signed data_offset
   */


  input_depth = 8;
  type = 0;
  *start_offset = 0;
  num_echoes = 1;


  if (argc == 2)
    return SetupHdr(hdr,128,128,2.0,2.0,2.0,1,input_depth,type);

  if (argc == 4) {
    sscanf(argv[2],"%ld",&dimx);
    sscanf(argv[3],"%f",&xsize);
    return SetupHdr(hdr,dimx,dimx,xsize,xsize,xsize,1,input_depth,type);
  }


  if ((argc == 3) || (argc == 6) || (argc == 7) ||
      (argc == 10) || (argc == 11)) {


    if ((argc == 3) || (argc == 6) || (argc == 7)) {
      lcv = 2;
      if (rd_the_hdr(argv[lcv],hdr) != NO_ERROR) {
	free_hdr_rec(hdr);
	init_mark_hdr(&mark_hdr);
	if ((err = load_hdr_file(argv[lcv],&mark_hdr)) != NO_ERROR) {
	  printf("Error reading %s : neither a *.nhdr file nor a *.hdr file...\n",
		 argv[lcv]);
	  exit(err);
	}
	else if ((err = SetupHdrFromMarkHdr(hdr,&mark_hdr)) != NO_ERROR) {
	  printf("Error setting up the header file %s : ...\n",argv[lcv]);
	  exit(err);
	}
      }
      lcv++;
      dimx = hdr->data_info.dimx;
      dimy = hdr->data_info.dimx;
      xsize = hdr->data_info.xpixel_size;
      ysize = hdr->data_info.ypixel_size;
      zsize = hdr->data_info.pixel_thickness;
    }

    else if ((argc == 10) || (argc == 11)) {
      sscanf(argv[2],"%ld",&dimx);
      sscanf(argv[3],"%ld",&dimy);
      sscanf(argv[4],"%f",&xsize);
      sscanf(argv[5],"%f",&ysize);
      sscanf(argv[6],"%f",&zsize);
      lcv = 7;
    }

    if (dimx == 0) 
      {printf("Illegal dimx = %ld\n",dimx);
       exit(1);}
    if (dimy == 0) 
      {printf("Illegal dimy = %ld\n",dimy);
       exit(1);}
    if (xsize <= .0) 
      {printf("Illegal xsize = %f\n",xsize);
       exit(1);}
    if (ysize <= .0) 
      {printf("Illegal ysize = %f\n",ysize);
       exit(1);}
    if (zsize <= .0) 
      {printf("Illegal zsize = %f\n",zsize);
       exit(1);}
    if (argc == 3)
      return SetupHdr(hdr,dimx,dimy,xsize,ysize,zsize,1,8,0);

    /*  Arg #  */
    sscanf(argv[lcv],"%ld",&num_echoes);
    if ((num_echoes != 1) && (num_echoes != 2)) 
      {printf("Illegal num_echoes = %ld\n",num_echoes);
       exit(1);}
    lcv++;

    /*  Arg #  */
    sscanf(argv[lcv],"%ld",&bits);
    if (bits == 0) 
      {printf("Illegal bits = %ld\n",bits);
       exit(1);}
    PRT(printf("bits = %d\n",bits));

    lcv++;
    if (bits == 33)
      {input_depth = sizeof(float) * 8;
       PRT(printf("INPUT_DATA_FLOAT...\n"));
       type |= INPUT_DATA_FLOAT;}
    else if (bits > 32)
      {input_depth = sizeof(double) * 8;
       PRT(printf("INPUT_DATA_DOUBLE...\n"));
       type |= INPUT_DATA_DOUBLE;}
    else {
      input_depth = bits;
      sscanf(argv[lcv],"%ld",&sign);
      if (sign != 0)
	type |= INPUT_DATA_SIGNED;
      else
	type &= ~(INPUT_DATA_FLOAT ||
		  INPUT_DATA_DOUBLE ||
		  INPUT_DATA_SIGNED);
    }
    lcv++;
    if (argc == 11) sscanf(argv[lcv],"%ld",start_offset);
    return SetupHdr(hdr,dimx,dimy,xsize,ysize,zsize,num_echoes,input_depth,type);
  }

  if (!(whoami = strrchr(argv[0],'/')))
    whoami = argv[0];
  else
    whoami++;

  printf("\nUsage : %s imgfn {[dim pixelsize] |\n",whoami);
  printf("      [orig_nhdr num_echoes input_pixel_bits \n");
  printf("        is_input_data_signed [start_input_offset]]}\n");
  printf("      [dimx dimy xpixelsize ypixelsize zpixelsize\n");
  printf("        num_echoes input_pixel_bits \n");
  printf("        is_input_data_signed [start_input_offset]]}\n");
  printf("\n");
  printf("Example 1 : %s imgfn\n",whoami);
  printf("   Assumed : %s imgfn 128 128 2.0 2.0 2.0 1 16 1 0\n",whoami);
  printf("   Output ==> 1) imgfn.hdr imgfn.nhdr\n\n");
  printf("\n");
  printf("Example 2 : %s imgfn fn.nhdr\n",whoami);
  printf("   Assumed : %s imgfn fn.nhdr 1 8 0 0\n",whoami);
  printf("   Output ==> 1) imgfn.hdr imgfn.nhdr\n\n");
  printf("\n");
  printf("Example 3 : %s imgfn 256 .9\n",whoami);
  printf("   Assumed : %s imgfn 256 256 .9 .9 .9 1 16 1 0\n",whoami);
  printf("\n");
  printf("Example 4 : %s imgfn fn.nhdr 1 16 1\n",whoami);
  printf("   Assumed : %s imgfn fn.nhdr 1 16 1 0\n",whoami);
  printf("\n");
  printf("Example 5 : %s imgfn 256 256 .8 .8 5.0 1 16 1\n",whoami);
  printf("   Assumed : %s imgfn 256 256 .8 .8 5.0 1 16 1 0\n",whoami);
  printf("\n");
  printf("Example 6 : %s imgfn 256 256 .8 .8 5.0 1 16 1 1000\n",whoami);
  printf("   Output ==> 1) riaimgfn riaimgfn.hdr riaimgfn.nhdr\n\n");
  printf("\n");
  printf("num_echoes = [1..2]\n");
  printf("input_pixel_bits = [1..any] where 1..32 = 1..32 bits\n");
  printf("                 33 = float & others = double\n");
  printf("\n");
  printf("is_input_data_signed = [0..any] where 0 = unsigned\n");
  printf("\n");
  printf("\n");
  printf("start_input_offset = [0..any] where 0 = unsigned\n");
  printf("\n");
  printf("Note : if you use '*' in the file specification, please\n");
  printf("       enclose that file specification by double quotes.\n");
  printf("\n\nSummary of usages:\n");
  printf(" 1 arg : %s imgfn\n",whoami);
  printf(" 3 args: %s imgfn pixel_dim pixel_size\n",whoami);
  printf("\n");
  printf(" 2 args: %s imgfn old.nhdrfn\n",whoami);
  printf(" 5 args: %s imgfn old.nhdrfn #echo #bits signed?\n",whoami);
  printf("\n");
  printf("10 args: %s imgfn Xdim Ydim Xsiz Ysiz Zsiz #echo #bits signed?\n",
	 whoami);
  printf("11 args: %s imgfn Xdim Ydim Xsiz Ysiz Zsiz #echo #bits signed? offset\n",
	 whoami);
  printf("\n\n");
  exit(1);
}


/*-------------------------------------------------------------------------
main - ???
-------------------------------------------------------------------------*/

void
main(argc,argv)
 int argc;
 char *argv[];
{
 int err;
 char **fns;
 unsigned int num_fns;
 unsigned long lcv;
 char *dirfn,fn[200];
 struct hdr_rec hdr;
 unsigned long start_offset;

 init_hdr_rec(&hdr);

 prog_usage(argc,argv,&hdr,&start_offset);

 dirfn = (char *) locate_dirfn(argv[1]);
 drop_dirfn(argv[1]);
 if (dirfn == NULL) dirfn = ".";
 if ((err = get_fns(dirfn,NULL,0,0,NULL,&fns,&num_fns,1,argv[1],
         NULL,0,0,NULL)) != NO_ERROR) exit(err);
 if (num_fns == 0)
    {printf("Sorry...No file names...\n");
     sprintf(fn,"%s/%s",dirfn,argv[1]);
     printf("...Fn specification processed = %s\n",fn);
    }
   else
    {
     for (lcv = 0; lcv < num_fns; lcv++)
        {printf("\n");
         sprintf(fn,"%s/%s",dirfn,fns[lcv]);
         printf("%ld) processing %s\n",lcv,fn);

         if ((err = OutputImg(fn,&hdr,start_offset)) != NO_ERROR)
            {printf("%ld) error processing %s\n",lcv,fn);}
        }
    }
 exit(1);
}
