/****************************************************************************
rearrange_img.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: rearrange_img.c,v 1.6 1999/07/07 18:07:14 hughett Exp $ Release $Name: nria_1_0_6 $";


#include "mri_format.h"
#include "nria_lib.h"

#if 1
#define TRACE(x)
#else
#define TRACE(x) x; fflush(stdout);
#endif


struct file_attrs_rec
   {
    char *fn;
    int total_slices;
    FILE *fid;
   };

struct output_order_rec
   {
    int fnum;
    int image_num;
   };

struct process_rec
   {
    int dimx,dimy;

/*  Unused currently - coding incomplete.  DRW 960203  */
#if 0
    int produce_hdr;
    unsigned int pixval_max,pixval_min;
#endif

    char *outfn;
    FILE *outfid;
    int num_inpfiles;
    struct file_attrs_rec *inpfiles;
    int total_output_slices;
    struct output_order_rec *slices;
   };


/* rearrange_img.c */
void init_process_rec(struct process_rec *prog);
void close_files(struct process_rec *prog);
void free_process_rec(struct process_rec *prog, int is_err);
int setup_process_rec(char *inpfn, struct process_rec *prog);
int get_file_sizes(struct process_rec *prog);
int validate_output_order_rec(struct process_rec *prog);
int open_files(struct process_rec *prog);
int process_file_attrs(struct process_rec *prog);
int process_files(struct process_rec *prog);
void prog_usage(int argc, char *argv[]);
void pr_process_rec(struct process_rec *prog);
void main(int argc, char *argv[]);



/*-------------------------------------------------------------------------
init_process_rec - ???
-------------------------------------------------------------------------*/

void
init_process_rec(prog)
 struct process_rec *prog;
{
 prog->outfn = NULL;
 prog->outfid = NULL;
 prog->num_inpfiles = 0;
 prog->inpfiles = NULL;
 prog->total_output_slices = 0;
 prog->slices = NULL;

}


/*-------------------------------------------------------------------------
close_files - ???
-------------------------------------------------------------------------*/

void
close_files(prog)
 struct process_rec *prog;
{
 int lcv;

 if (prog->outfid != NULL) fclose(prog->outfid);
 for (lcv = 0; lcv < prog->num_inpfiles; lcv++)
    if (prog->inpfiles[lcv].fid != NULL) fclose(prog->inpfiles[lcv].fid);

}


/*-------------------------------------------------------------------------
free_process_rec - ???

FIXME:  This function is poorly named, since it clears but does not free
the process_rec.  -PWH 19990430
-------------------------------------------------------------------------*/

void
free_process_rec(prog,is_err)
 struct process_rec *prog;
 int is_err;
{
 close_files(prog);
 if (is_err) delete_file(prog->outfn);

}


/*-------------------------------------------------------------------------
setup_process_rec - ???
-------------------------------------------------------------------------*/

int
setup_process_rec(inpfn,prog)
 char *inpfn;
 struct process_rec *prog;
{
 FILE *fid;
 int num,err;
 char str[255];
 int lcv,indx,cnt;
 unsigned int total;

 TRACE(printf("setup_process_rec...\n"))
 if ((fid = fopen(inpfn,"r")) == NULL)
    {printf("Error occurred when opening up the input data file = %s\n",
            inpfn);
     return FILE_ACCESS_ERROR;
    }
 num = 1;
/*  All rd_text's changed to RdText.  This is to make this program work.
 *  DRW 960203
 */
/* rd_text(fid,&(prog->dimx),&num,&err,RD_INT); */
 err = RdText(fid,&(prog->dimx),&num,RD_INT,0);
 if ((num != 1) || (err != NO_ERROR)) return READ_ERROR;
/* rd_text(fid,&(prog->dimy),&num,&err,RD_INT); */
 err = RdText(fid,&(prog->dimy),&num,RD_INT,0);
 if ((num != 1) || (err != NO_ERROR)) return READ_ERROR;

/*
 *  This section commented out because the coding is not complete! DRW 960203
 */
#if 0
 num = 1;
/* rd_text(fid,&(prog->produce_hdr),&num,&err,RD_INT); */
 err = RdText(fid,&(prog->produce_hdr),&num,RD_INT,0);
 if ((num != 1) || (err != NO_ERROR)) return READ_ERROR;
#endif

 num = sizeof(str);
/* rd_text(fid,str,&num,&err,RD_TEXT); */
 err = RdText(fid,str,&num,RD_TEXT,0);
 if ((num == 0) || (err != NO_ERROR)) return READ_ERROR;
 if ((err = cp_str(&(prog->outfn),str)) != NO_ERROR) return err;
 TRACE(printf("setup_process_rec...1\n"))

 num = 1;
/* rd_text(fid,&total,&num,&err,RD_INT); */
 err = RdText(fid,&total,&num,RD_INT,0);
 if ((num != 1) || (err != NO_ERROR)) return READ_ERROR;

 prog->num_inpfiles = 0;
 get_ptr_memory(&(prog->inpfiles),&(prog->num_inpfiles),total,
     (size_t) sizeof(struct file_attrs_rec),&err);
 if (err != NO_ERROR) return err;

 for (lcv = 0; lcv < prog->num_inpfiles; lcv++)
    {prog->inpfiles[lcv].fn = NULL;
     prog->inpfiles[lcv].fid = NULL;
     prog->inpfiles[lcv].total_slices = 0;
    }

 TRACE(printf("setup_process_rec...2\n"))
 for (lcv = 0; lcv < prog->num_inpfiles; lcv++)
    {num = sizeof(str);
/*     rd_text(fid,str,&num,&err,RD_TEXT); */
     err = RdText(fid,str,&num,RD_TEXT,0);
     if ((num == 0) || (err != NO_ERROR)) return READ_ERROR;
     if ((err = cp_str(&(prog->inpfiles[lcv].fn),str)) != NO_ERROR)
        return err;
    }

 num = 1;
/* rd_text(fid,&total,&num,&err,RD_INT); */
 err = RdText(fid,&total,&num,RD_INT,0);
 if ((num != 1) || (err != NO_ERROR)) return READ_ERROR;
 prog->total_output_slices = 0;
 get_ptr_memory(&(prog->slices),&(prog->total_output_slices),total,
     (size_t) sizeof(struct output_order_rec),&err);
 if (err != NO_ERROR) return err;

 num = 1;
 TRACE(printf("setup_process_rec...3\n"))
 for (lcv = 0; lcv < prog->total_output_slices; lcv = cnt)
    {
/* rd_text(fid,&(prog->slices[lcv].fnum),&num,&err,RD_INT); */
     err = RdText(fid,&(prog->slices[lcv].fnum),&num,RD_INT,0);
     if ((num != 1) || (err != NO_ERROR)) return READ_ERROR;

/*     rd_text(fid,&total,&num,&err,RD_INT); */
     err = RdText(fid,&total,&num,RD_INT,0);
     if ((num != 1) || (err != NO_ERROR)) return READ_ERROR;
     for (cnt = lcv,indx = 0; indx < total; indx++)
        {prog->slices[cnt].fnum = prog->slices[lcv].fnum;
/*         rd_text(fid,&(prog->slices[cnt].image_num),&num,&err,RD_INT); */
         err = RdText(fid,&(prog->slices[cnt].image_num),&num,RD_INT,0);
         if ((num != 1) || (err != NO_ERROR)) return READ_ERROR;
         cnt++;
         if (cnt > prog->total_output_slices) break;
        }
     }
 TRACE(printf("setup_process_rec...4\n"))
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
get_file_sizes - ???
-------------------------------------------------------------------------*/

int
get_file_sizes(prog)
 struct process_rec *prog;
{
 int lcv,err;
 unsigned long size,slice_size;

 /*  DRW960202: add "* 2" to account for 2-byte data.  */
 slice_size = prog->dimx * prog->dimy * 2;
 for (lcv = 0; lcv < prog->num_inpfiles; lcv++)
    {size = (unsigned long) the_file_size("",prog->inpfiles[lcv].fn,&err);
     if (err != NO_ERROR)
        {printf("Error....check input file = %s\n",
              prog->inpfiles[lcv].fn);
         return err;}
     prog->inpfiles[lcv].total_slices = size / slice_size;
    }
 return NO_ERROR;

}


/*-------------------------------------------------------------------------
validate_output_order_rec - ???
-------------------------------------------------------------------------*/

int
validate_output_order_rec(prog)
 struct process_rec *prog;
{
 int lcv;

 for (lcv = 0; lcv < prog->total_output_slices; lcv++)
    {if ((prog->slices[lcv].fnum <= 0) &&
         (prog->slices[lcv].fnum > prog->num_inpfiles))
        {printf("Bad input...\n");
         printf("   Range of file input numbers ==> 1..%d\n",
                    prog->num_inpfiles);
         printf("   Entered invalid file# = %d\n",
                    prog->slices[lcv].fnum);
         return ~NO_ERROR;
        }
     prog->slices[lcv].fnum -= 1;
     if ((prog->slices[lcv].image_num <= 0) &&
         (prog->slices[lcv].image_num >
          prog->inpfiles[prog->slices[lcv].fnum].total_slices))
        {printf("Bad input...\n");
         printf("   Entered file# = %d\n",
                    prog->slices[lcv].fnum + 1);
         printf("   Range of image numbers ==> 1..%d\n",
                    prog->inpfiles[prog->slices[lcv].fnum].total_slices);
         printf("   Entered invalid file# = %d\n",
                    prog->slices[lcv].image_num);
         return ~NO_ERROR;
        }
    }
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
open_files - ???
-------------------------------------------------------------------------*/

int
open_files(prog)
 struct process_rec *prog;
{
 int lcv;

 TRACE(printf("open_files...\n"))
 if ((prog->outfid = fopen(prog->outfn,"w")) == NULL)
    {printf("Unable to open up the output image file = %s\n",
              prog->outfn);
     return FILE_ACCESS_ERROR;
    }
 for (lcv = 0; lcv < prog->num_inpfiles; lcv++)
    {
     if ((prog->inpfiles[lcv].fid =
            fopen(prog->inpfiles[lcv].fn,"r")) == NULL)
        {printf("Unable to open up the input image file = %s\n",
                prog->inpfiles[lcv].fn);
         return FILE_ACCESS_ERROR;
        }
    }
 TRACE(printf("open_files...end\n"))
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
process_file_attrs - ???
-------------------------------------------------------------------------*/

int
process_file_attrs(prog)
 struct process_rec *prog;
{
 int err;

 if ((err = get_file_sizes(prog)) != NO_ERROR) return err;
 if ((err = validate_output_order_rec(prog)) != NO_ERROR) return err;
 return open_files(prog);
}


/*-------------------------------------------------------------------------
process_files - ???
-------------------------------------------------------------------------*/

int
process_files(prog)
 struct process_rec *prog;
{
 int lcv,err;
 unsigned long size,slice_size,offset;
 unsigned char *buf;

 /*  DRW960319: 1-byte data.  */
 slice_size = prog->dimx * prog->dimy;
 size = 0;
 get_ptr_memory(&buf,&size,slice_size,(size_t) sizeof(char),&err);
 if (err != NO_ERROR) return err;

 for (lcv = 0; lcv < prog->total_output_slices; lcv++)
    {offset = (prog->slices[lcv].image_num - 1) * slice_size;
     fseek(prog->inpfiles[prog->slices[lcv].fnum].fid,offset,0);
     if (fread(buf,(size_t) slice_size, (size_t) 1,
            prog->inpfiles[prog->slices[lcv].fnum].fid) != 1)
        {printf("Error occurred when reading from file = %s\n",
              prog->inpfiles[prog->slices[lcv].fnum].fn);
         return READ_ERROR;}

     if (fwrite(buf,(size_t) slice_size, (size_t) 1,prog->outfid) != 1)
        {printf("Error occurred when writing to file = %s\n",
              prog->outfn);
         return WRITE_ERROR;}
    }
 c_free(buf);
 TRACE(printf("...Image output file = %s\n\n",prog->outfn))
 return NO_ERROR;
}

/*  Commented out - coding incomplete.  DRW 960203  */
#if 0
produce_hdrs(prog)
 struct process_rec *prog;
{
 struct hdr_rec hdr;

 init_hdr_rec(&hdr);
 hdr.dimx = prog->dimx;
 hdr.dimy = prog->dimy;
 hdr.histo_dimx = hdr.histo_dimy = 256;
 hdr.max_slices = prog->total_output_slices;
}
#endif



/*-------------------------------------------------------------------------
prog_usage - ???
-------------------------------------------------------------------------*/

void
prog_usage(argc,argv)
 int argc;
 char *argv[];
{

 if (argc != 2)
    {printf("Usage : %s rearrange_img.dat\n",argv[0]);
     printf("Example : %s rearrange_img.dat\n",argv[0]);
     printf("  Where rearrange_img.dat specifies which image slices are\n");
     printf("  to be retrieved and the order how they are to be stored\n");
     printf("  in the new image output file.\n");
     exit(0);}
}


/*-------------------------------------------------------------------------
pr_process_rec - ???
-------------------------------------------------------------------------*/

void
pr_process_rec(prog)
 struct process_rec *prog;
{
 int lcv;

 printf("dimx = %d dimy = %d outfn = %s\n",prog->dimx,prog->dimy,
        prog->outfn);
 printf("num_inpfiles = %d\n",prog->num_inpfiles);
 for (lcv = 0; lcv < prog->num_inpfiles; lcv++)
    {printf("%d) input fn = %s",lcv,prog->inpfiles[lcv].fn);
     printf("    total_slices = %d\n",prog->inpfiles[lcv].total_slices);
    }
 printf("\n");
 printf("total_output_slices = %d\n",prog->total_output_slices);
 for (lcv = 0; lcv < prog->total_output_slices; lcv++)
    {printf("%d) fnum = %d",lcv,prog->slices[lcv].fnum);
     printf("    image_num = %d\n",prog->slices[lcv].image_num);
    }

}


/*-------------------------------------------------------------------------
main - ???
-------------------------------------------------------------------------*/

void
main(argc,argv)
 int argc;
 char *argv[];
{
 int err;
 struct process_rec prog;

 prog_usage(argc,argv);
 init_process_rec(&prog);
 if ((err = setup_process_rec(argv[1],&prog)) != NO_ERROR)
    {printf("Error occurred when reading the input data...err = %d\n",err);
     exit(0);
    }
 if ((err = process_file_attrs(&prog)) == NO_ERROR)
    if ((err = process_files(&prog)) == NO_ERROR)

/*  Commented out - coding incomplete.  DRW 960203  */
#if 0
       if (prog.produce_hdr)
          err = produce_hdrs(&prog);
#else
       ;
#endif

 if (err != NO_ERROR)
    {printf("Error...id = %d\n",err);
    } 
/*
 pr_process_rec(&prog);
*/
 free_process_rec(&prog,err);
 exit(err);
}
